@@ -8,7 +8,7 @@
    Copyright (C) 2005 Boudewijn Rempt <boud@valdyas.org>
    Copyright (C) 2005 Raphael Langerhorst <raphael.langerhorst@kdemail.net>
    Copyright (C) 2005 Thomas Zander <zander@kde.org>
-   Copyright (C) 2005,2007 Jan Hambrecht <jaham@gmx.net>
+   Copyright (C) 2005,2007-2008 Jan Hambrecht <jaham@gmx.net>
    Copyright (C) 2006 Inge Wallin <inge@lysator.liu.se>
    Copyright (C) 2006 Martin Pfeiffer <hubipete@gmx.net>
    Copyright (C) 2006 Gábor Lehel <illissius@gmail.com>
@@ -34,24 +34,39 @@
 
 #include "svgexport.h"
 
-#include <vdocument.h>
-#include <karbon_part.h>
+#include <KarbonDocument.h>
+#include <KarbonPart.h>
 
 #include <KoDocument.h>
 #include <KoFilterChain.h>
 #include <KoShapeLayer.h>
 #include <KoPathShape.h>
 #include <KoLineBorder.h>
+#include <KoColorBackground.h>
+#include <KoGradientBackground.h>
+#include <KoPatternBackground.h>
+#include <plugins/artistictextshape/ArtisticTextShape.h>
+#include <pathshapes/rectangle/KoRectangleShape.h>
+#include <pathshapes/ellipse/KoEllipseShape.h>
+#include <plugins/pictureshape/PictureShape.h>
+#include <KoImageData.h>
 
-#include <kdebug.h>
-#include <kgenericfactory.h>
+#include <KDebug>
+#include <KGenericFactory>
+#include <KMimeType>
+#include <KTemporaryFile>
+#include <KIO/NetAccess>
+#include <KIO/CopyJob>
 
 #include <QtCore/QFile>
 #include <QtCore/QString>
 #include <QtCore/QTextStream>
+#include <QtCore/QBuffer>
+#include <QtCore/QFileInfo>
 #include <QtGui/QLinearGradient>
 #include <QtGui/QRadialGradient>
 
+const double ScaleToUserSpace = 90.0 / 72.0;
 
 static void printIndentation( QTextStream *stream, unsigned int indent )
 {
@@ -66,6 +81,7 @@ K_EXPORT_COMPONENT_FACTORY( libkarbonsvgexport, SvgExportFactory( "kofficefilter
 SvgExport::SvgExport( QObject*parent, const QStringList& )
     : KoFilter(parent), m_indent( 0 ), m_indent2( 0 )
 {
+    m_userSpaceMatrix.scale( ScaleToUserSpace, ScaleToUserSpace );
 }
 
 KoFilter::ConversionStatus SvgExport::convert( const QByteArray& from, const QByteArray& to )
@@ -105,7 +121,7 @@ KoFilter::ConversionStatus SvgExport::convert( const QByteArray& from, const QBy
     return KoFilter::OK;
 }
 
-void SvgExport::saveDocument( VDocument& document )
+void SvgExport::saveDocument( KarbonDocument& document )
 {
     // get the bounding box of the page
     QSizeF pageSize = document.pageSize();
@@ -123,7 +139,7 @@ void SvgExport::saveDocument( VDocument& document )
 
     *m_defs <<
         "<svg xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" width=\"" <<
-        pageSize.width() << "px\" height=\"" << pageSize.height() << "px\">" << endl;
+        pageSize.width() << "pt\" height=\"" << pageSize.height() << "pt\">" << endl;
     printIndentation( m_defs, ++m_indent2 );
     *m_defs << "<defs>" << endl;
 
@@ -145,7 +161,10 @@ void SvgExport::saveLayer( KoShapeLayer * layer )
     printIndentation( m_body, m_indent++ );
     *m_body << "<g" << getID( layer ) << ">" << endl;
 
-    foreach( KoShape * shape, layer->iterator() )
+    QList<KoShape*> sortedShapes = layer->iterator();
+    qSort(sortedShapes.begin(), sortedShapes.end(), KoShape::compareShapeZIndex);
+
+    foreach( KoShape * shape, sortedShapes )
     {
         KoShapeContainer * container = dynamic_cast<KoShapeContainer*>( shape );
         if( container )
@@ -161,9 +180,16 @@ void SvgExport::saveLayer( KoShapeLayer * layer )
 void SvgExport::saveGroup( KoShapeContainer * group )
 {
     printIndentation( m_body, m_indent++ );
-    *m_body << "<g" << getID( group ) << ">" << endl;
+    *m_body << "<g" << getID( group );
+    *m_body << getTransform( group->transformation(), " transform" );
+    if( ! group->isVisible() )
+        *m_body << " display=\"none\"";
+    *m_body << ">" << endl;
 
-    foreach( KoShape * shape, group->iterator() )
+    QList<KoShape*> sortedShapes = group->iterator();
+    qSort(sortedShapes.begin(), sortedShapes.end(), KoShape::compareShapeZIndex);
+
+    foreach( KoShape * shape, sortedShapes )
     {
         KoShapeContainer * container = dynamic_cast<KoShapeContainer*>( shape );
         if( container )
@@ -181,12 +207,29 @@ void SvgExport::saveShape( KoShape * shape )
     KoPathShape * path = dynamic_cast<KoPathShape*>( shape );
     if( path )
     {
-        // TODO differentiate between rects, circles and paths
-        savePath( path );
+        if( path->pathShapeId() == KoRectangleShapeId )
+        {
+            saveRectangle( static_cast<KoRectangleShape*>( path ) );
+        }
+        else if( path->pathShapeId() == KoEllipseShapeId )
+        {
+            saveEllipse( static_cast<KoEllipseShape*>( path ) );
+        }
+        else 
+        {
+            savePath( path );
+        }
     }
     else
     {
-        // TODO export text and images
+        if( shape->shapeId() == ArtisticTextShapeID )
+        {
+            saveText( static_cast<ArtisticTextShape*>( shape ) );
+        }
+        else if( shape->shapeId() == PICTURESHAPEID )
+        {
+            saveImage( static_cast<PictureShape*>( shape ) );
+        }
     }
 }
 
@@ -195,24 +238,62 @@ void SvgExport::savePath( KoPathShape * path )
     printIndentation( m_body, m_indent );
     *m_body << "<path" << getID( path );
 
-    getFill( path->background(), m_body );
-    getStroke( path->border(), m_body );
+    getStyle( path, m_body );
 
-    *m_body << " d=\"" << path->toString( path->absoluteTransformation(0) ) << "\" ";
-
-    if( path->fillRule() == Qt::OddEvenFill )
-        *m_body << " fill-rule=\"evenodd\"";
-    else
-        *m_body << " fill-rule=\"nonzero\"";
+    *m_body << " d=\"" << path->toString( path->transformation() * m_userSpaceMatrix ) << "\" ";
 
     *m_body << " />" << endl;
 }
 
-QString SvgExport::getID( KoShape *obj )
+void SvgExport::saveEllipse( KoEllipseShape * ellipse )
 {
-    if( obj && !obj->name().isEmpty() )
-        return QString( " id=\"%1\"" ).arg( obj->name() );
-    return QString();
+    if( ellipse->type() == KoEllipseShape::Arc && ellipse->startAngle() == ellipse->endAngle() )
+    {
+        printIndentation( m_body, m_indent );
+        QSizeF size = ellipse->size();
+        if( size.width() == size.height() )
+        {
+            *m_body << "<circle" << getID( ellipse );
+            *m_body << " r=\"" << 0.5 * size.width() << "pt\"";
+        }
+        else
+        {
+            *m_body << "<ellipse" << getID( ellipse );
+            *m_body << " rx=\"" << 0.5 * size.width() << "pt\"";
+            *m_body << " ry=\"" << 0.5 * size.height() << "pt\"";
+        }
+        *m_body << " cx=\"" << 0.5 * size.width() << "pt\"";
+        *m_body << " cy=\"" << 0.5 * size.height() << "pt\"";
+        *m_body << getTransform( ellipse->transformation(), " transform" );
+        getStyle( ellipse, m_body );
+        *m_body << "/>" << endl;
+    }
+    else
+    {
+        savePath( ellipse );
+    }
+}
+
+void SvgExport::saveRectangle( KoRectangleShape * rectangle )
+{
+    printIndentation( m_body, m_indent );
+    *m_body << "<rect" << getID( rectangle );
+    *m_body << getTransform( rectangle->transformation(), " transform" );
+
+    getStyle( rectangle, m_body );
+
+    QSizeF size = rectangle->size();
+    *m_body << " width=\"" << size.width() << "pt\""; 
+    *m_body << " height=\"" << size.height() << "pt\""; 
+
+    double rx = rectangle->cornerRadiusX();
+    if( rx > 0.0 )
+        *m_body << " rx=\"" << 0.01 * rx * 0.5 * size.width() << "pt\""; 
+    double ry = rectangle->cornerRadiusY();
+    if( ry > 0.0 )
+        *m_body << " ry=\"" << 0.01 * ry * 0.5 * size.height() << "pt\""; 
+
+    *m_body << "/>" << endl;
 }
 
 static QString createUID()
@@ -222,10 +303,58 @@ static QString createUID()
     return "defitem" + QString().setNum( nr++ );
 }
 
+QString SvgExport::createID( const KoShape * obj )
+{
+    QString id;
+    if( ! m_shapeIds.contains( obj ) )
+    {
+        id = obj->name().isEmpty() ? createUID() : obj->name();
+        m_shapeIds.insert( obj, id );
+    }
+    else
+    {
+        id = m_shapeIds[obj];
+    }
+    return id;
+}
+
+QString SvgExport::getID( const KoShape *obj )
+{
+    return QString( " id=\"%1\"" ).arg( createID( obj ) );
+}
+
+QString SvgExport::getTransform( const QMatrix &matrix, const QString &attributeName )
+{
+    if( matrix.isIdentity() )
+        return "";
+
+    QString transform = attributeName + "=\"";
+    if( isTranslation( matrix ) )
+    {
+        transform += QString("translate(%1, %2)")
+                    .arg( toUserSpace(matrix.dx()) )
+                    .arg( toUserSpace(matrix.dy()) );
+    }
+    else
+    {
+        transform += QString( "matrix(%1 %2 %3 %4 %5 %6)" )
+                    .arg( matrix.m11() ).arg( matrix.m12() )
+                    .arg( matrix.m21() ).arg( matrix.m22() )
+                    .arg( toUserSpace(matrix.dx()) ) .arg( toUserSpace(matrix.dy()) );
+    }
+
+    return transform + "\"";
+}
+
+bool SvgExport::isTranslation( const QMatrix &m )
+{
+    return ( m.m11() == 1.0 && m.m12() == 0.0 && m.m21() == 0.0 && m.m22() == 1.0 );
+}
+
 void SvgExport::getColorStops( const QGradientStops & colorStops )
 {
     m_indent2++;
-    foreach( QGradientStop stop, colorStops )
+    foreach( const QGradientStop &stop, colorStops )
     {
         printIndentation( m_defs, m_indent2 );
         *m_defs << "<stop stop-color=\"";
@@ -236,7 +365,7 @@ void SvgExport::getColorStops( const QGradientStops & colorStops )
     m_indent2--;
 }
 
-void SvgExport::getGradient( const QGradient * grad )
+void SvgExport::getGradient( const QBrush &brush )
 {
     const QString spreadMethod[3] = {
         QString("spreadMethod=\"pad\" "),
@@ -244,6 +373,12 @@ void SvgExport::getGradient( const QGradient * grad )
         QString("spreadMethod=\"repeat\" ")
     };
 
+    const QGradient * grad = brush.gradient();
+    if( ! grad )
+        return;
+
+    QMatrix matrix = brush.matrix();
+
     QString uid = createUID();
     if( grad->type() == QGradient::LinearGradient )
     {
@@ -252,10 +387,11 @@ void SvgExport::getGradient( const QGradient * grad )
         printIndentation( m_defs, m_indent2 );
         *m_defs << "<linearGradient id=\"" << uid << "\" ";
         *m_defs << "gradientUnits=\"userSpaceOnUse\" ";
-        *m_defs << "x1=\"" << g->start().x() << "\" ";
-        *m_defs << "y1=\"" << g->start().y() << "\" ";
-        *m_defs << "x2=\"" << g->finalStop().x() << "\" ";
-        *m_defs << "y2=\"" << g->finalStop().y() << "\" ";
+        *m_defs << getTransform( matrix, "gradientTransform" ) << " ";
+        *m_defs << "x1=\"" << toUserSpace( g->start().x() ) << "\" ";
+        *m_defs << "y1=\"" << toUserSpace( g->start().y() ) << "\" ";
+        *m_defs << "x2=\"" << toUserSpace( g->finalStop().x() ) << "\" ";
+        *m_defs << "y2=\"" << toUserSpace( g->finalStop().y() ) << "\" ";
         *m_defs << spreadMethod[g->spread()];
         *m_defs << ">" << endl;
 
@@ -273,11 +409,12 @@ void SvgExport::getGradient( const QGradient * grad )
         printIndentation( m_defs, m_indent2 );
         *m_defs << "<radialGradient id=\"" << uid << "\" ";
         *m_defs << "gradientUnits=\"userSpaceOnUse\" ";
-        *m_defs << "cx=\"" << g->center().x() << "\" ";
-        *m_defs << "cy=\"" << g->center().y() << "\" ";
-        *m_defs << "fx=\"" << g->focalPoint().x() << "\" ";
-        *m_defs << "fy=\"" << g->focalPoint().y() << "\" ";
-        *m_defs << "r=\"" << QString().setNum( g->radius() ) << "\" ";
+        *m_defs << getTransform( matrix, "gradientTransform" ) << " ";
+        *m_defs << "cx=\"" << toUserSpace( g->center().x() ) << "\" ";
+        *m_defs << "cy=\"" << toUserSpace( g->center().y() ) << "\" ";
+        *m_defs << "fx=\"" << toUserSpace( g->focalPoint().x() ) << "\" ";
+        *m_defs << "fy=\"" << toUserSpace( g->focalPoint().y() ) << "\" ";
+        *m_defs << "r=\"" << QString().setNum( toUserSpace( g->radius() ) ) << "\" ";
         *m_defs << spreadMethod[g->spread()];
         *m_defs << ">" << endl;
 
@@ -333,8 +470,33 @@ void SvgExport::getPattern( const QPixmap & )
     *m_body << "url(#" << uid << ")";
 }
 
-void SvgExport::getFill( const QBrush& fill, QTextStream *stream )
+void SvgExport::getStyle( KoShape * shape, QTextStream * stream )
 {
+    getFill( shape, stream );
+    getStroke( shape, stream );
+    if( ! shape->isVisible() )
+        *stream << " display=\"none\"";
+}
+
+void SvgExport::getFill( KoShape * shape, QTextStream *stream )
+{
+    QBrush fill( Qt::NoBrush );
+    KoColorBackground * cbg = dynamic_cast<KoColorBackground*>( shape->background() );
+    if( cbg )
+        fill = QBrush( cbg->color(), cbg->style() );
+    KoGradientBackground * gbg = dynamic_cast<KoGradientBackground*>( shape->background() );
+    if( gbg )
+    {
+        fill = QBrush( *gbg->gradient() );
+        fill.setMatrix( gbg->matrix() );
+    }
+    KoPatternBackground * pbg = dynamic_cast<KoPatternBackground*>( shape->background() );
+    if( pbg )
+    {
+        fill.setTextureImage( pbg->pattern() );
+        fill.setMatrix( pbg->matrix() );
+    }
+
     *stream << " fill=\"";
 
     switch( fill.style() )
@@ -345,7 +507,7 @@ void SvgExport::getFill( const QBrush& fill, QTextStream *stream )
         case Qt::LinearGradientPattern:
         case Qt::RadialGradientPattern:
         case Qt::ConicalGradientPattern:
-            getGradient( fill.gradient() );
+            getGradient( fill );
             break;
         case Qt::TexturePattern:
             getPattern( fill.texture() );
@@ -360,21 +522,28 @@ void SvgExport::getFill( const QBrush& fill, QTextStream *stream )
     *stream << "\"";
 
     *stream << " fill-opacity=\"" << fill.color().alphaF() << "\"";
+
+    KoPathShape * path = dynamic_cast<KoPathShape*>( shape );
+    if( path )
+    {
+        if( path->fillRule() == Qt::OddEvenFill )
+            *stream << " fill-rule=\"evenodd\"";
+        else
+            *stream << " fill-rule=\"nonzero\"";
+    }
 }
 
-void SvgExport::getStroke( const KoShapeBorderModel * stroke, QTextStream *stream )
+void SvgExport::getStroke( KoShape *shape, QTextStream *stream )
 {
-    const KoLineBorder * line = dynamic_cast<const KoLineBorder*>( stroke );
+    const KoLineBorder * line = dynamic_cast<const KoLineBorder*>( shape->border() );
     if( ! line )
         return;
 
     *stream << " stroke=\"";
     if( line->lineStyle() == Qt::NoPen )
         *stream << "none";
-    /*
-    else if( line->type() == VStroke::grad )
-        getGradient( line->gradient() );
-    */
+    else if( line->lineBrush().gradient() )
+        getGradient( line->lineBrush() );
     else
         getHexColor( stream, line->color() );
     *stream << "\"";
@@ -423,60 +592,167 @@ void SvgExport::getHexColor( QTextStream *stream, const QColor & color )
     *stream << Output;
 }
 
-/*
-void SvgExport::visitVText( VText& text )
+void SvgExport::saveText( ArtisticTextShape * text )
 {
-	VPath path( 0L );
-	path.combinePath( text.basePath() );
+    printIndentation( m_body, m_indent++ );
+    *m_body << "<text" << getID( text );
 
-	QString id = createUID();
-	writePathToStream( path, " id=\""+ id + "\"", m_defs, m_indent2 );
+    getFill( text, m_body );
+    getStroke( text, m_body );
 
-	printIndentation( m_body, m_indent++ );
-	*m_body << "<text" << getID( &text );
-	// *m_body << " transform=\"scale(1, -1) translate(0, -" << text.document()->height() << ")\"";
-	getFill( *( text.fill() ), m_body );
-	getStroke( *( text.stroke() ), m_body );
+    QFont font = text->font();
 
-	*m_body << " font-family=\"" << text.font().family() << "\"";
-	*m_body << " font-size=\"" << text.font().pointSize() << "\"";
-	if( text.font().bold() )
-		*m_body << " font-weight=\"bold\"";
-	if( text.font().italic() )
-		*m_body << " font-style=\"italic\"";
-	if( text.alignment() == VText::Center )
-		*m_body << " text-anchor=\"middle\"";
-	else if( text.alignment() == VText::Right )
-		*m_body << " text-anchor=\"end\"";
+    *m_body << " font-family=\"" << font.family() << "\"";
+    *m_body << " font-size=\"" << font.pointSize() << "pt\"";
 
-	*m_body << ">" << endl;
-	
-	printIndentation( m_body, m_indent );
-	*m_body << "<textPath xlink:href=\"#" << id << "\"";
-	if( text.offset() > 0.0 )
-		*m_body << " startOffset=\"" << text.offset() * 100.0 << "%\"";	
-	*m_body << ">";
-	*m_body << text.text();
-	*m_body << "</textPath>" << endl;
-	printIndentation( m_body, --m_indent );
-	*m_body << "</text>" << endl;
-}
-*/
+    if( font.bold() )
+        *m_body << " font-weight=\"bold\"";
+    if( font.italic() )
+        *m_body << " font-style=\"italic\"";
+
+    qreal anchorOffset = 0.0;
+    if( text->textAnchor() == ArtisticTextShape::AnchorMiddle )
+    {
+        anchorOffset += 0.5 * text->size().width();
+        *m_body << " text-anchor=\"middle\"";
+    }
+    else if( text->textAnchor() == ArtisticTextShape::AnchorEnd )
+    {
+        anchorOffset += text->size().width();
+        *m_body << " text-anchor=\"end\"";
+    }
 
-// horrible but at least something gets exported now
-// will need this for patterns
-void SvgExport::saveImage( QImage& )
-{
     printIndentation( m_body, m_indent );
-    *m_body << "<image ";
 
-    *m_body << "x=\"" << "\" ";
-    *m_body << "y=\"" << "\" ";
-    *m_body << "width=\"" << "\" ";
-    *m_body << "height=\"" << "\" ";
-    *m_body << "xlink:href=\"" << "\"";
+    // check if we are set on a path
+    if( text->layout() == ArtisticTextShape::Straight )
+    {
+        QMatrix m = text->transformation();
+        if( isTranslation( m ) )
+        {
+            QPointF position = text->position();
+            *m_body << " x=\"" << position.x() + anchorOffset << "pt\"";
+            *m_body << " y=\"" << position.y() + text->baselineOffset() << "pt\"";
+        }
+        else
+        {
+            *m_body << " x=\"" << anchorOffset << "pt\"";
+            *m_body << " y=\"" << text->baselineOffset() << "pt\"";
+            *m_body << getTransform( text->transformation(), " transform" );
+        }
+        *m_body << ">" << endl;
+        *m_body << text->text();
+    }
+    else
+    {
+        KoPathShape * baseline = KoPathShape::fromQPainterPath( text->baseline() );
+
+        QString id;
+
+        printIndentation( m_defs, m_indent );
+
+        id = createUID();
+        *m_defs << "<path id=\"" << id << "\"";
+        *m_defs << " d=\"" << baseline->toString( baseline->absoluteTransformation(0) * m_userSpaceMatrix ) << "\" ";
+        *m_defs << " />" << endl;
+
+        *m_body << ">" << endl;
+        *m_body << "<textPath xlink:href=\"#" << id << "\"";
+        if( text->startOffset() > 0.0 )
+            *m_body << " startOffset=\"" << text->startOffset() * 100.0 << "%\"";	
+        *m_body << ">";
+        *m_body << text->text();
+        *m_body << "</textPath>" << endl;
+
+        delete baseline;
+    }
+
+    printIndentation( m_body, --m_indent );
+    *m_body << "</text>" << endl;
+}
+
+void SvgExport::saveImage( PictureShape * picture )
+{
+    KoImageData * imageData = dynamic_cast<KoImageData*>( picture->userData() );
+    if( ! imageData )
+    {
+        qWarning() << "Picture has no image data. Omitting.";
+        return;
+    }
+
+    printIndentation( m_body, m_indent );
+
+    *m_body << "<image" << getID( picture );
+    QMatrix m = picture->transformation();
+    if( isTranslation( m ) )
+    {
+        QPointF position = picture->position();
+        *m_body << " x=\"" << position.x() << "pt\"";
+        *m_body << " y=\"" << position.y() << "pt\"";
+    }
+    else
+    {
+        *m_body << getTransform( picture->transformation(), " transform" );
+    }
+
+    *m_body << " width=\"" << picture->size().width() << "pt\"";
+    *m_body << " height=\"" << picture->size().height() <<"pt\"";
+
+    const bool saveInline = true;
+
+    if( saveInline )
+    {
+        QByteArray ba;
+        QBuffer buffer(&ba);
+        buffer.open(QIODevice::WriteOnly);
+        if( imageData->saveToFile( buffer ) )
+        {
+            const QString mimeType( KMimeType::findByContent( ba )->name() );
+            *m_body << " xlink:href=\"data:" << mimeType << ";base64," << ba.toBase64() <<  "\"";
+        }
+    }
+    else
+    {
+        // write to a temp file first
+        KTemporaryFile imgFile;
+        if( imageData->saveToFile( imgFile ) )
+        {
+            // tz: TODO the new version of KoImageData has the extension save inside maybe that can be used
+            // get the mime type from the temp file content
+            KMimeType::Ptr mimeType = KMimeType::findByFileContent( imgFile.fileName() );
+            // get url of destination directory
+            KUrl url( m_chain->outputFile() );
+            QString dstBaseFilename = QFileInfo( url.fileName() ).baseName();
+            url.setDirectory( url.directory() );
+            // create a filename for the image file at the destination directory
+            QString fname = dstBaseFilename + '_' + createID( picture );
+            // get extension from mimetype
+            QString ext = "";
+            QStringList patterns = mimeType->patterns();
+            if( patterns.count() )
+                ext = patterns.first().mid( 1 );
+            url.setFileName( fname + ext );
+            // check if file exists already
+            int i = 0;
+            // change filename as long as the filename already exists
+            while( KIO::NetAccess::exists( url, KIO::NetAccess::DestinationSide, 0 ) )
+                url.setFileName( fname + QString( "_%1").arg( ++i ) + ext );
+            // move the temp file to the destination directory
+            KIO::Job * job = KIO::move( KUrl( imgFile.fileName() ), url );
+            if( job && KIO::NetAccess::synchronousRun( job, 0 ) )
+                *m_body << " xlink:href=\"" << url.fileName() << "\"";
+            else
+                KIO::NetAccess::removeTempFile( imgFile.fileName() );
+        }
+    }
     *m_body << " />" << endl;
 }
 
+double SvgExport::toUserSpace( double value )
+{
+    return value * ScaleToUserSpace;
+}
+
+
 #include "svgexport.moc"
 
