@@ -1,6 +1,6 @@
 /* This file is part of the KDE project
  * Copyright (c) 2003 thierry lorthiois <lorthioist@wanadoo.fr>
- * Copyright (c) 2007 Jan Hambrecht <jaham@gmx.net>
+ * Copyright (c) 2007-2008 Jan Hambrecht <jaham@gmx.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -19,17 +19,23 @@
 
 #include "wmfimportparser.h"
 
-#include <core/vdocument.h>
+#include <KarbonDocument.h>
 
 #include <KoPathShape.h>
 #include <KoLineBorder.h>
 #include <KoShapeLayer.h>
 #include <KoImageData.h>
+#include <KoImageCollection.h>
+#include <KoColorBackground.h>
+#include <KoGradientBackground.h>
+#include <KoPatternBackground.h>
+#include <KoShapeFactory.h>
+#include <KoShapeRegistry.h>
 
 #include <pathshapes/rectangle/KoRectangleShape.h>
 #include <pathshapes/ellipse/KoEllipseShape.h>
 #include <pictureshape/PictureShape.h>
-
+#include <artistictextshape/ArtisticTextShape.h>
 #include <kdebug.h>
 
 /*
@@ -40,7 +46,7 @@ WMFImportParser::WMFImportParser() : KoWmfRead() {
 }
 
 
-bool WMFImportParser::play( VDocument& doc )
+bool WMFImportParser::play( KarbonDocument& doc )
 {
     mDoc = &doc;
     mScaleX = mScaleY = 1;
@@ -98,6 +104,12 @@ bool WMFImportParser::begin() {
         mScaleX = mDoc->pageSize().width() / (double)bounding.width();
         mScaleY = mDoc->pageSize().height() / (double)bounding.height();
     }
+
+    kDebug() << "bounding rect =" << bounding;
+    kDebug() << "page size =" << mDoc->pageSize();
+    kDebug() << "scale x =" << mScaleX;
+    kDebug() << "scale y =" << mScaleY;
+
     return true;
 }
 
@@ -115,7 +127,8 @@ void WMFImportParser::restore() {
 }
 
 
-void WMFImportParser::setFont( const QFont & ) {
+void WMFImportParser::setFont( const QFont &font ) {
+    mFont = font;
 }
 
 
@@ -160,7 +173,9 @@ void WMFImportParser::setWindowExt( int width, int height ) {
 }
 
 
-void WMFImportParser::setMatrix( const QMatrix &, bool  ) {
+void WMFImportParser::setMatrix( const QMatrix &matrix, bool combine ) {
+    kDebug() << "matrix =" << matrix;
+    kDebug() << "combine =" << combine;
 }
 
 
@@ -180,9 +195,14 @@ void WMFImportParser::moveTo( int left, int top ) {
 
 
 void WMFImportParser::lineTo( int left, int top ) {
-    KoPathShape * line = new KoPathShape();
+    KoPathShape * line = static_cast<KoPathShape*>( createShape( KoPathShapeId ) );
+    if( ! line )
+        return;
+    
     line->moveTo( QPointF( coordX(mCurrentPoint.x()), coordY(mCurrentPoint.y()) ) );
     line->lineTo( QPointF( coordX(left), coordY(top) ) );
+    line->normalize();
+
     appendPen( *line );
 
     mDoc->add( line );
@@ -192,10 +212,14 @@ void WMFImportParser::lineTo( int left, int top ) {
 
 
 void WMFImportParser::drawRect( int left, int top, int width, int height ) {
-    KoRectangleShape * rectangle = new KoRectangleShape();
-    rectangle->setPosition( QPointF( coordX(left), coordY(top) ) );
-    rectangle->setSize( QSizeF( scaleW(width), scaleH(height) ) );
-    rectangle->setScale( width < 0 ? -1.0 : 1.0, height < 0 ? -1.0 : 1.0 );
+    QRectF bound = QRectF( QPointF( coordX(left), coordY(top) ), QSizeF( scaleW(width), scaleH(height) ) ).normalized();
+
+    KoRectangleShape * rectangle = static_cast<KoRectangleShape*>( createShape( KoRectangleShapeId ) );
+    if( ! rectangle )
+        return;
+    
+    rectangle->setPosition( bound.topLeft() );
+    rectangle->setSize( bound.size() );
 
     appendPen( *rectangle );
     appendBrush( *rectangle );
@@ -205,12 +229,16 @@ void WMFImportParser::drawRect( int left, int top, int width, int height ) {
 
 
 void WMFImportParser::drawRoundRect( int left, int top, int width, int height, int roundw, int roundh ) {
-    KoRectangleShape * rectangle = new KoRectangleShape();
-    rectangle->setPosition( QPointF( coordX(left), coordY(top) ) );
-    rectangle->setSize( QSizeF( scaleW(width), scaleH(height) ) );
+    QRectF bound = QRectF( QPointF( coordX(left), coordY(top) ), QSizeF( scaleW(width), scaleH(height) ) ).normalized();
+
+    KoRectangleShape * rectangle = static_cast<KoRectangleShape*>( createShape( KoRectangleShapeId ) );
+    if( ! rectangle )
+        return;
+    
+    rectangle->setPosition( bound.topLeft() );
+    rectangle->setSize( bound.size() );
     rectangle->setCornerRadiusX( 2.0 * qAbs( roundw )  );
     rectangle->setCornerRadiusY( 2.0 * qAbs( roundh ) );
-    rectangle->setScale( width < 0 ? -1.0 : 1.0, height < 0 ? -1.0 : 1.0 );
 
     appendPen( *rectangle );
     appendBrush( *rectangle );
@@ -220,10 +248,14 @@ void WMFImportParser::drawRoundRect( int left, int top, int width, int height, i
 
 
 void WMFImportParser::drawEllipse( int left, int top, int width, int height ) {
-    KoEllipseShape *ellipse = new KoEllipseShape();
-    ellipse->setPosition( QPointF( coordX(left), coordY(top) ) );
-    ellipse->setSize( QSizeF( scaleW(width), scaleH(height) ) );
-    ellipse->setScale( width < 0 ? -1.0 : 1.0, height < 0 ? -1.0 : 1.0 );
+    QRectF bound = QRectF( QPointF( coordX(left), coordY(top) ), QSizeF( scaleW(width), scaleH(height) ) ).normalized();
+
+    KoEllipseShape *ellipse = static_cast<KoEllipseShape*>( createShape( KoEllipseShapeId ) );
+    if( ! ellipse )
+        return;
+    
+    ellipse->setPosition( bound.topLeft() );
+    ellipse->setSize( bound.size() );
 
     appendPen( *ellipse );
     appendBrush( *ellipse );
@@ -237,13 +269,17 @@ void WMFImportParser::drawArc( int x, int y, int w, int h, int aStart, int aLen
     double end = (aLen * 180) / 2880.0;
     end += start;
 
-    KoEllipseShape * arc = new KoEllipseShape();
+    QRectF bound = QRectF( QPointF( coordX(x), coordY(y) ), QSizeF( scaleW(w), scaleH(h) ) ).normalized();
+
+    KoEllipseShape * arc = static_cast<KoEllipseShape*>( createShape( KoEllipseShapeId ) );
+    if( ! arc )
+        return;
+
     arc->setType( KoEllipseShape::Arc );
     arc->setStartAngle( start );
     arc->setEndAngle( end );
-    arc->setPosition( QPointF( coordX(x), coordY(y) ) );
-    arc->setSize( QSizeF( scaleW(w), scaleH(h) ) );
-    arc->setScale( w < 0 ? -1.0 : 1.0, h < 0 ? -1.0 : 1.0 );
+    arc->setPosition( bound.topLeft() );
+    arc->setSize( bound.size() );
 
     appendPen( *arc );
     //appendBrush( *arc );
@@ -257,13 +293,17 @@ void WMFImportParser::drawPie( int x, int y, int w, int h, int aStart, int aLen
     double end = (aLen * 180) / 2880.0;
     end += start;
 
-    KoEllipseShape * pie = new KoEllipseShape();
+    QRectF bound = QRectF( QPointF( coordX(x), coordY(y) ), QSizeF( scaleW(w), scaleH(h) ) ).normalized();
+
+    KoEllipseShape * pie = static_cast<KoEllipseShape*>( createShape( KoEllipseShapeId ) );
+    if( ! pie )
+        return;
+
     pie->setType( KoEllipseShape::Pie );
     pie->setStartAngle( start );
     pie->setEndAngle( end );
-    pie->setPosition( QPointF( coordX(x), coordY(y) ) );
-    pie->setSize( QSizeF( scaleW(w), scaleH(h) ) );
-    pie->setScale( w < 0 ? -1.0 : 1.0, h < 0 ? -1.0 : 1.0 );
+    pie->setPosition( bound.topLeft() );
+    pie->setSize( bound.size() );
 
     appendPen( *pie );
     appendBrush( *pie );
@@ -277,13 +317,17 @@ void WMFImportParser::drawChord( int x, int y, int w, int h, int aStart, int aLe
     double end = (aLen * 180) / 2880.0;
     end += start;
 
-    KoEllipseShape * chord = new KoEllipseShape();
+    QRectF bound = QRectF( QPointF( coordX(x), coordY(y) ), QSizeF( scaleW(w), scaleH(h) ) ).normalized();
+
+    KoEllipseShape * chord = static_cast<KoEllipseShape*>( createShape( KoEllipseShapeId ) );
+    if( ! chord )
+        return;
+
     chord->setType( KoEllipseShape::Chord );
     chord->setStartAngle( start );
     chord->setEndAngle( end );
-    chord->setPosition( QPointF( coordX(x), coordY(y) ) );
-    chord->setSize( QSizeF( scaleW(w), scaleH(h) ) );
-    chord->setScale( w < 0 ? -1.0 : 1.0, h < 0 ? -1.0 : 1.0 );
+    chord->setPosition( bound.topLeft() );
+    chord->setSize( bound.size() );
 
     appendPen( *chord );
     appendBrush( *chord );
@@ -293,7 +337,10 @@ void WMFImportParser::drawChord( int x, int y, int w, int h, int aStart, int aLe
 
 
 void WMFImportParser::drawPolyline( const QPolygon &pa ) {
-    KoPathShape *polyline = new KoPathShape();
+    KoPathShape *polyline = static_cast<KoPathShape*>( createShape( KoPathShapeId ) );
+    if( ! polyline )
+        return;
+    
     appendPen( *polyline );
     appendPoints( *polyline, pa );
 
@@ -302,7 +349,10 @@ void WMFImportParser::drawPolyline( const QPolygon &pa ) {
 
 
 void WMFImportParser::drawPolygon( const QPolygon &pa, bool winding ) {
-    KoPathShape *polygon = new KoPathShape();
+    KoPathShape *polygon = static_cast<KoPathShape*>( createShape( KoPathShapeId ) );
+    if( ! polygon )
+        return;
+    
     appendPen( *polygon );
     appendBrush( *polygon );
     appendPoints( *polygon, pa );
@@ -314,7 +364,9 @@ void WMFImportParser::drawPolygon( const QPolygon &pa, bool winding ) {
 
 
 void WMFImportParser::drawPolyPolygon( QList<QPolygon>& listPa, bool winding ) {
-    KoPathShape *path = new KoPathShape();
+    KoPathShape *path = static_cast<KoPathShape*>( createShape( KoPathShapeId ) );
+    if( ! path )
+        return;
 
     if ( listPa.count() > 0 ) {
         appendPen( *path );
@@ -322,9 +374,12 @@ void WMFImportParser::drawPolyPolygon( QList<QPolygon>& listPa, bool winding ) {
         appendPoints( *path, listPa.first() );
         path->close();
         path->setFillRule( winding ? Qt::WindingFill : Qt::OddEvenFill );
-        foreach( QPolygon pa, listPa )
+        foreach( const QPolygon & pa, listPa )
         {
-            KoPathShape *newPath = new KoPathShape();
+            KoPathShape *newPath = static_cast<KoPathShape*>( createShape( KoPathShapeId ) );
+            if( ! newPath )
+                continue;
+
             appendPoints( *newPath, pa );
             newPath->close();
             path->combine( newPath );
@@ -335,11 +390,15 @@ void WMFImportParser::drawPolyPolygon( QList<QPolygon>& listPa, bool winding ) {
 }
 
 
-void WMFImportParser::drawImage( int x, int y, const QImage &image, int , int , int sw, int sh ) {
-    KoImageData * data = new KoImageData( mDoc->imageCollection() );
-    data->setImage( image );
+void WMFImportParser::drawImage( int x, int y, const QImage &image, int sx, int sy, int sw, int sh ) {
+    KoImageData * data = mDoc->imageCollection()->getImage( image );
+    if( ! data )
+        return;
+
+    PictureShape * pic = static_cast<PictureShape*>( createShape( PICTURESHAPEID ) );
+    if( ! pic )
+        return;
 
-    PictureShape * pic = new PictureShape();
     pic->setUserData( data );
     pic->setPosition( QPointF(x,y) );
     if( sw < 0 )
@@ -349,13 +408,61 @@ void WMFImportParser::drawImage( int x, int y, const QImage &image, int , int ,
     pic->setSize( QSizeF(sw,sh) );
 
     kDebug() << "image data size =" << data->pixmap().size();
+    kDebug() << "source image size =" << image.size();
+    kDebug() << "source position =" << QPointF( sx, sy );
+    kDebug() << "source size =" << QPointF( scaleW(sw), scaleH(sh) );
 
     mDoc->add( pic );
 }
 
 
-void WMFImportParser::drawText( int , int , int , int , int , const QString& , double ) {
-    kDebug() << "importing text is not supported";
+void WMFImportParser::drawText( int x, int y, int , int , int flags, const QString& text, double rotation ) {
+    enum TextFlags { CurrentPosition = 0x01, AlignHCenter = 0x06, AlignBottom = 0x08 };
+
+    if ( flags & CurrentPosition ) {
+        // (left, top) position = current logical position
+        x = mCurrentPoint.x();
+        y = mCurrentPoint.y();
+    }
+
+    // adjust font size
+    QFont font = mFont;
+    font.setPointSizeF( coordY( mFont.pointSize() ) );
+
+    ArtisticTextShape * textShape = static_cast<ArtisticTextShape*>( createShape( ArtisticTextShapeID ) );
+    if( ! textShape )
+        return;
+
+    textShape->setFont( font );
+    textShape->setText( text );
+
+    // determine y-offset from given baseline position
+    qreal yOffset = 0.0;
+    if( flags & AlignBottom )
+        yOffset -= textShape->baselineOffset();
+
+    textShape->setPosition( QPointF( coordX(x), coordY(y)+yOffset ) );
+
+    // set text anchor
+    qreal xOffset = 0.0;
+    if( flags & AlignHCenter ) {
+        textShape->setTextAnchor( ArtisticTextShape::AnchorMiddle );
+        xOffset = -0.5 * textShape->size().width();
+    }
+
+    if ( rotation ) {
+        // we rotate around the anchor point
+        QPointF anchor( -xOffset, -yOffset );
+        QMatrix matrix;
+        matrix.translate( anchor.x(), anchor.y() );
+        matrix.rotate ( rotation );
+        matrix.translate( -anchor.x(), -anchor.y() );
+        textShape->applyTransformation( matrix );
+    }
+
+    textShape->setBackground( new KoColorBackground( mPen.color() ) );
+
+    mDoc->add( textShape );
 }
 
 
@@ -377,7 +484,35 @@ void WMFImportParser::appendPen( KoShape& obj )
 
 void WMFImportParser::appendBrush( KoShape& obj )
 {
-    obj.setBackground( mBrush );
+    switch( mBrush.style() )
+    {
+        case Qt::NoBrush:
+            obj.setBackground( 0 );
+            break;
+        case Qt::TexturePattern:
+        {
+            KoImageCollection * imageCollection = mDoc->imageCollection();
+            if( imageCollection )
+            {
+                KoPatternBackground * bg = new KoPatternBackground( imageCollection );
+                bg->setPattern( mBrush.textureImage() );
+                bg->setMatrix( mBrush.matrix() );
+                obj.setBackground( bg );
+            }
+            break;
+        }
+        case Qt::LinearGradientPattern:
+        case Qt::RadialGradientPattern:
+        case Qt::ConicalGradientPattern:
+        {
+            KoGradientBackground * bg = new KoGradientBackground( *mBrush.gradient() );
+            bg->setMatrix( mBrush.matrix() );
+            obj.setBackground( bg );
+            break;
+        }
+        default:
+            obj.setBackground( new KoColorBackground( mBrush.color(), mBrush.style() ) );
+    }
 }
 
 void  WMFImportParser::setCompositionMode( QPainter::CompositionMode )
@@ -416,3 +551,25 @@ double WMFImportParser::scaleH( int height )
 {
     return (height * mScaleY);
 }
+
+KoShape * WMFImportParser::createShape( const QString &shapeID )
+{
+    KoShapeFactory * factory = KoShapeRegistry::instance()->get( shapeID );
+    if( ! factory )
+    {
+        kWarning(30514) << "Could not find factory for shape id" << shapeID;
+        return 0;
+    }
+
+    KoShape * shape = factory->createDefaultShapeAndInit( mDoc->dataCenterMap() );
+    if( shape && shape->shapeId().isEmpty() )
+        shape->setShapeId( factory->id() );
+
+    KoPathShape * path = dynamic_cast<KoPathShape*>( shape );
+    if( path && shapeID == KoPathShapeId )
+        path->clear();
+    // reset tranformation that might come from the default shape
+    shape->setTransformation( QMatrix() );
+
+    return shape;
+}
