@@ -16,6 +16,7 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  */
+#include "kis_openexr_import.h"
 
 #include <QString>
 #include <QFile>
@@ -27,28 +28,23 @@
 
 #include <half.h>
 #include <ImfRgbaFile.h>
-//#include <ImfStringAttribute.h>
-//#include <ImfMatrixAttribute.h>
-//#include <ImfArray.h>
-//#include <drawImage.h>
 
 #include <KoCompositeOp.h>
+#include <KoColorSpace.h>
 
 #include <iostream>
 
-#include "kis_types.h"
-#include "kis_openexr_import.h"
-#include "kis_doc2.h"
-#include "kis_image.h"
-#include "kis_layer.h"
-#include "kis_group_layer.h"
-#include "kis_paint_layer.h"
-#include "kis_annotation.h"
-#include "KoColorSpaceRegistry.h"
-#include "kis_iterators_pixel.h"
-#include "kis_undo_adapter.h"
-// #include "kis_rgb_f32_colorspace.h"
-// #include "kis_rgb_f16half_colorspace.h"
+#include <kis_paint_device.h>
+#include <kis_types.h>
+#include <kis_doc2.h>
+#include <kis_image.h>
+#include <kis_layer.h>
+#include <kis_group_layer.h>
+#include <kis_paint_layer.h>
+#include <kis_annotation.h>
+#include <KoColorSpaceRegistry.h>
+#include <kis_iterators_pixel.h>
+#include <kis_undo_adapter.h>
 
 using namespace std;
 using namespace Imf;
@@ -71,7 +67,7 @@ KoFilter::ConversionStatus KisOpenEXRImport::convert(const QByteArray& from, con
         return KoFilter::NotImplemented;
     }
 
-    kDebug(41008) <<"\n\n\nKrita importing from OpenEXR";
+    dbgFile << "\n\n\nKrita importing from OpenEXR";
 
     KisDoc2 * doc = dynamic_cast<KisDoc2*>(m_chain -> outputDocument());
     if (!doc) {
@@ -90,8 +86,8 @@ KoFilter::ConversionStatus KisOpenEXRImport::convert(const QByteArray& from, con
     Box2i dataWindow = file.dataWindow();
     Box2i displayWindow = file.displayWindow();
 
-    kDebug(41008) <<"Data window:" << QRect(dataWindow.min.x, dataWindow.min.y, dataWindow.max.x - dataWindow.min.x + 1, dataWindow.max.y - dataWindow.min.y + 1);
-    kDebug(41008) <<"Display window:" << QRect(displayWindow.min.x, displayWindow.min.y, displayWindow.max.x - displayWindow.min.x + 1, displayWindow.max.y - displayWindow.min.y + 1);
+    dbgFile << "Data window:" << QRect(dataWindow.min.x, dataWindow.min.y, dataWindow.max.x - dataWindow.min.x + 1, dataWindow.max.y - dataWindow.min.y + 1);
+    dbgFile << "Display window:" << QRect(displayWindow.min.x, displayWindow.min.y, displayWindow.max.x - displayWindow.min.x + 1, displayWindow.max.y - displayWindow.min.y + 1);
 
     int imageWidth = displayWindow.max.x - displayWindow.min.x + 1;
     int imageHeight = displayWindow.max.y - displayWindow.min.y + 1;
@@ -101,7 +97,7 @@ KoFilter::ConversionStatus KisOpenEXRImport::convert(const QByteArray& from, con
     int dataWidth  = dataWindow.max.x - dataWindow.min.x + 1;
     int dataHeight = dataWindow.max.y - dataWindow.min.y + 1;
 
-    const KoColorSpace *cs = static_cast<const KoColorSpace *>((KoColorSpaceRegistry::instance()->colorSpace(KoID("RGBAF16HALF", ""),"")));
+    const KoColorSpace *cs = static_cast<const KoColorSpace *>((KoColorSpaceRegistry::instance()->colorSpace(KoID("RgbAF16", ""), "")));
 
     if (cs == 0) {
         return KoFilter::InternalError;
@@ -115,8 +111,8 @@ KoFilter::ConversionStatus KisOpenEXRImport::convert(const QByteArray& from, con
         return KoFilter::CreationError;
     }
     image->lock();
-    KisPaintLayerSP layer = new KisPaintLayer( image, image->nextLayerName(), OPACITY_OPAQUE, cs);
-    layer->setCompositeOp( cs->compositeOp( COMPOSITE_OVER ) );
+    KisPaintLayerSP layer = new KisPaintLayer(image, image->nextLayerName(), OPACITY_OPAQUE, cs);
+    layer->setCompositeOp(cs->compositeOp(COMPOSITE_OVER));
 
     if (!layer) {
         return KoFilter::CreationError;
@@ -151,7 +147,7 @@ KoFilter::ConversionStatus KisOpenEXRImport::convert(const QByteArray& from, con
         }
     }
 
-    image->addLayer(layer, image->rootLayer(), 0 );
+    image->addNode(layer.data(), image->rootLayer().data());
     layer->setDirty();
     doc -> setCurrentImage(image);
     doc -> undoAdapter() -> setUndo(true);
