@@ -32,6 +32,7 @@
 #include <q3sortedlist.h>
 #include <QFile>
 #include <QTextStream>
+#include <QPen>
 //Added by qt3to4:
 #include <Q3ValueList>
 #include <QByteArray>
@@ -40,17 +41,18 @@
 #include <KoDocumentInfo.h>
 
 // KSpread
-#include <kspread/ui/Canvas.h>
+#include <kspread/ApplicationSettings.h>
+#include <kspread/part/Canvas.h>
 #include <kspread/CellStorage.h>
 #include <kspread/Currency.h>
-#include <kspread/Doc.h>
+#include <kspread/part/Doc.h>
 #include <kspread/Map.h>
 #include <kspread/NamedAreaManager.h>
 #include <kspread/Sheet.h>
 #include <kspread/SheetPrint.h>
 #include <kspread/RowColumnFormat.h>
 #include <kspread/Validity.h>
-#include <kspread/ui/View.h>
+#include <kspread/part/View.h>
 
 using namespace KSpread;
 
@@ -564,9 +566,16 @@ QDomElement GNUMERICExport::GetCellStyle(QDomDocument gnumeric_doc,const Cell& c
 
     const Style style = cell.style();
     QColor bgColor =  style.backgroundColor();
-	red = bgColor.red()<<8;
-	green = bgColor.green()<<8;
-	blue = bgColor.blue()<<8;
+    if (bgColor.isValid()) {
+        red = bgColor.red()<<8;
+        green = bgColor.green()<<8;
+        blue = bgColor.blue()<<8;
+    } else {
+        // Let's use white default for background
+        red = 0xffff;
+        green = 0xffff;
+        blue = 0xffff;
+    }
 
     switch (style.backgroundBrush().style())
     {
@@ -718,11 +727,14 @@ QDomElement GNUMERICExport::GetCellStyle(QDomDocument gnumeric_doc,const Cell& c
 
 	switch( style.formatType())
 	{
+        case Format::Generic:
+            stringFormat="General";
+            break;
         case Format::Number:
             stringFormat="0.00";
 			break;
         case Format::Text:
-            stringFormat="general";
+            stringFormat="General";
             break;
 		case Format::Money:
 
@@ -896,6 +908,9 @@ QDomElement GNUMERICExport::GetCellStyle(QDomDocument gnumeric_doc,const Cell& c
             stringFormat = style.customFormat();
             break;
         default:
+            // This is a required parameter, so let's write a sane default
+            kWarning(30521) << "Unhandled Format value, setting 'General' as default: " << style.formatType();
+            stringFormat = "General";
             break;
     }
     cell_style.setAttribute("Format",stringFormat);
@@ -1005,10 +1020,10 @@ KoFilter::ConversionStatus GNUMERICExport::convert( const QByteArray& from, cons
     QDomElement attributes = gnumeric_doc.createElement("gmr:Attributes");
     workbook.appendChild(attributes);
 
-    addAttributeItem(gnumeric_doc, attributes, "4", "WorkbookView::show_horizontal_scrollbar", ksdoc->showHorizontalScrollBar());
-    addAttributeItem(gnumeric_doc, attributes, "4", "WorkbookView::show_vertical_scrollbar", ksdoc->showVerticalScrollBar());
-    addAttributeItem(gnumeric_doc, attributes, "4", "WorkbookView::show_notebook_tabs", ksdoc->showTabBar());
-    if (ksdoc->completionMode() == KGlobalSettings::CompletionAuto)
+    addAttributeItem(gnumeric_doc, attributes, "4", "WorkbookView::show_horizontal_scrollbar", ksdoc->map()->settings()->showHorizontalScrollBar());
+    addAttributeItem(gnumeric_doc, attributes, "4", "WorkbookView::show_vertical_scrollbar", ksdoc->map()->settings()->showVerticalScrollBar());
+    addAttributeItem(gnumeric_doc, attributes, "4", "WorkbookView::show_notebook_tabs", ksdoc->map()->settings()->showTabBar());
+    if (ksdoc->map()->settings()->completionMode() == KGlobalSettings::CompletionAuto)
         addAttributeItem(gnumeric_doc, attributes, "4", "WorkbookView::do_auto_completion", "true");
     else
         addAttributeItem(gnumeric_doc, attributes, "4", "WorkbookView::do_auto_completion", "false");
@@ -1058,7 +1073,7 @@ KoFilter::ConversionStatus GNUMERICExport::convert( const QByteArray& from, cons
     </gmr:Name>
   </gmr:Names>
     */
-    const QList<QString> namedAreas = ksdoc->namedAreaManager()->areaNames();
+    const QList<QString> namedAreas = ksdoc->map()->namedAreaManager()->areaNames();
     if (namedAreas.count() > 0)
     {
         Sheet* sheet = 0;
@@ -1066,10 +1081,10 @@ KoFilter::ConversionStatus GNUMERICExport::convert( const QByteArray& from, cons
         QDomElement areaNames = gnumeric_doc.createElement("gmr:Names");
         for (int i = 0; i < namedAreas.count(); ++i)
         {
-            sheet = ksdoc->namedAreaManager()->sheet(namedAreas[i]);
+            sheet = ksdoc->map()->namedAreaManager()->sheet(namedAreas[i]);
             if (!sheet)
                 continue;
-            range = ksdoc->namedAreaManager()->namedArea(namedAreas[i]).firstRange();
+            range = ksdoc->map()->namedAreaManager()->namedArea(namedAreas[i]).firstRange();
             QDomElement areaName = gnumeric_doc.createElement("gmr:Name");
             QDomElement areaNameElement = gnumeric_doc.createElement("gmr:name");
             areaNameElement.appendChild(gnumeric_doc.createTextNode(namedAreas[i]));
@@ -1104,11 +1119,11 @@ KoFilter::ConversionStatus GNUMERICExport::convert( const QByteArray& from, cons
     int indexActiveTable=0;
     foreach(Sheet* table, ksdoc->map()->sheetList())
     {
-        if ( table->print()->paperFormat()== KoPageFormat::CustomSize )
+        if ( table->print()->settings()->pageLayout().format == KoPageFormat::CustomSize )
         {
             customSize = gnumeric_doc.createElement( "gmr:Geometry" );
-            customSize.setAttribute( "Width", POINT_TO_MM ( table->print()->paperWidth() ));
-            customSize.setAttribute( "Height", POINT_TO_MM (table->print()->paperWidth() ));
+            customSize.setAttribute( "Width", POINT_TO_MM ( table->print()->settings()->pageLayout().width ));
+            customSize.setAttribute( "Height", POINT_TO_MM (table->print()->settings()->pageLayout().width ));
             sheets.appendChild(customSize);
             //<gmr:Geometry Width="768" Height="365"/>
         }
@@ -1119,8 +1134,8 @@ KoFilter::ConversionStatus GNUMERICExport::convert( const QByteArray& from, cons
         sheet.setAttribute("DisplayFormulas", table->getShowFormula() ? "true" : "false" );
         sheet.setAttribute("HideZero", table->getHideZero()? "true" : "false");
         sheet.setAttribute("HideGrid", !table->getShowGrid()? "true" : "false");
-        sheet.setAttribute("HideColHeader", ( !ksdoc->showColumnHeader() ? "true" : "false" ));
-        sheet.setAttribute("HideRowHeader", ( !ksdoc->showRowHeader()? "true" : "false" ));
+        sheet.setAttribute("HideColHeader", ( !ksdoc->map()->settings()->showColumnHeader() ? "true" : "false" ));
+        sheet.setAttribute("HideRowHeader", ( !ksdoc->map()->settings()->showRowHeader()? "true" : "false" ));
         /* Not available in KSpread ?
          * sheet.setAttribute("DisplayOutlines", "true");
          * sheet.setAttribute("OutlineSymbolsBelow", "true");
@@ -1150,7 +1165,7 @@ KoFilter::ConversionStatus GNUMERICExport::convert( const QByteArray& from, cons
         // KSpread 200% gives zoom() = 2.5, this in GNumeric = 250%
         tmp = gnumeric_doc.createElement("gmr:Zoom");
         if (view)
-            tmp.appendChild(gnumeric_doc.createTextNode(QString::number(canvas->zoom())));
+            tmp.appendChild(gnumeric_doc.createTextNode(QString::number(canvas->view()->zoom())));
         else
             tmp.appendChild(gnumeric_doc.createTextNode("1.0"));
         sheet.appendChild(tmp);
@@ -1160,22 +1175,22 @@ KoFilter::ConversionStatus GNUMERICExport::convert( const QByteArray& from, cons
         margins = gnumeric_doc.createElement( "gmr:Margins" );
 
         top = gnumeric_doc.createElement( "gmr:top" );
-        top.setAttribute("Points", table->print()->topBorder());
+        top.setAttribute("Points", table->print()->settings()->pageLayout().top);
         top.setAttribute("PrefUnit", "mm");
         margins.appendChild( top );
 
         bottom = gnumeric_doc.createElement( "gmr:bottom" );
-        bottom.setAttribute("Points", table->print()->bottomBorder());
+        bottom.setAttribute("Points", table->print()->settings()->pageLayout().bottom);
         bottom.setAttribute("PrefUnit", "mm");
         margins.appendChild( bottom );
 
         left = gnumeric_doc.createElement( "gmr:left" );
-        left.setAttribute("Points", table->print()->leftBorder());
+        left.setAttribute("Points", table->print()->settings()->pageLayout().left);
         left.setAttribute("PrefUnit", "mm");
         margins.appendChild( left );
 
         right = gnumeric_doc.createElement( "gmr:right" );
-        right.setAttribute("Points", table->print()->rightBorder());
+        right.setAttribute("Points", table->print()->settings()->pageLayout().right);
         right.setAttribute("PrefUnit", "mm");
         margins.appendChild( right );
 
@@ -1183,7 +1198,7 @@ KoFilter::ConversionStatus GNUMERICExport::convert( const QByteArray& from, cons
         sheet.appendChild(tmp);
 
         orientation = gnumeric_doc.createElement( "gmr:orientation" );
-        QString orientString = table->print()->orientation() == KoPageFormat::Landscape ? "landscape" : "portrait";
+        QString orientString = table->print()->settings()->pageLayout().orientation == KoPageFormat::Landscape ? "landscape" : "portrait";
         orientation.appendChild( gnumeric_doc.createTextNode(orientString) );
         tmp.appendChild( orientation );
 
@@ -1531,6 +1546,7 @@ KoFilter::ConversionStatus GNUMERICExport::convert( const QByteArray& from, cons
     QTextStream streamOut(out);
 
     streamOut << str;
+    streamOut.flush();
 
     out->close();
     delete out;
