@@ -16,18 +16,18 @@
 #ifndef __KUGARNOPIMPORT_H__
 #define __KUGARNOPIMPORT_H__
 
-#include <koFilter.h>
-#include <qcstring.h>
-#include <qstringlist.h>
+#include <KoFilter.h>
+#include <q3cstring.h>
+#include <QStringList>
 
 class KugarNOPImport : public KoFilter
 {
     Q_OBJECT
 
 public:
-    KugarNOPImport( KoFilter *parent, const char *name, const QStringList& );
+    KugarNOPImport( QObject* parent, const QStringList& );
 
-    virtual KoFilter::ConversionStatus convert( const QCString& from, const QCString& to );
+    virtual KoFilter::ConversionStatus convert( const QByteArray& from, const QByteArray& to );
 
 };
 
