@@ -2,23 +2,22 @@
    Copyright (C) 2002 Werner Trobin <trobin@kde.org>
    Copyright (C) 2002 David Faure <faure@kde.org>
    Copyright (C) 2008 Benjamin Cail <cricketc@gmail.com>
-   Copyright (C) 2009 Inge Wallin   <inge@lysator.liu.se>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the Library GNU General Public
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public
    version 2 of the License, or (at your option) version 3 or,
    at the discretion of KDE e.V (which shall act as a proxy as in
    section 14 of the GPLv3), any later version..
 
-   This library is distributed in the hope that it will be useful,
+   This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+    General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
+   You should have received a copy of the GNU General Public License
+   along with this program; see the file COPYING.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+ * Boston, MA 02110-1301, USA.
 */
 
 #ifndef DOCUMENT_H
@@ -26,8 +25,8 @@
 
 #include "tablehandler.h"
 
-#include <wv2/src/handlers.h>
-#include <wv2/src/functor.h>
+#include <wv2/handlers.h>
+#include <wv2/functor.h>
 
 #include <QString>
 #include <qdom.h>
@@ -37,7 +36,6 @@
 #include <queue>
 #include <string>
 #include <QBuffer>
-#include <QDomElement>
 
 #include <KoXmlWriter.h>
 #include <KoGenStyles.h>
@@ -45,51 +43,38 @@
 
 class KoStoreDevice;
 
-namespace wvWare
-{
-class Parser;
-namespace Word97 {
-class BRC;
-}
+namespace wvWare {
+    class Parser;
+    namespace Word97 {
+        class BRC;
+    }
 }
 class KoFilterChain;
 class KWordReplacementHandler;
 class KWordTableHandler;
 class KWordPictureHandler;
 class KWordTextHandler;
-class KWordDrawingHandler;
 
 class Document : public QObject, public wvWare::SubDocumentHandler
 {
     Q_OBJECT
 public:
     Document(const std::string& fileName, KoFilterChain* chain, KoXmlWriter* bodyWriter,
-             KoGenStyles* mainStyles, KoXmlWriter* metaWriter, KoStore* store, KoXmlWriter* manifestWriter);
+            KoGenStyles* mainStyles, KoXmlWriter* metaWriter, KoStore* store, KoXmlWriter* manifestWriter);
     virtual ~Document();
 
-    KWordTextHandler *textHandler() const {
-        return m_textHandler;
-    }
-
-    bool hasParser() const {
-        return m_parser != 0L;
-    }
-    bool bodyFound() const {
-        return m_bodyFound;
-    }
+    bool hasParser() const { return m_parser != 0L; }
+    bool bodyFound() const { return m_bodyFound; }
 
     virtual void bodyStart();
     virtual void bodyEnd();
 
-    virtual void headerStart(wvWare::HeaderData::Type type);
+    virtual void headerStart( wvWare::HeaderData::Type type );
     virtual void headerEnd();
 
     virtual void footnoteStart();
     virtual void footnoteEnd();
 
-    virtual void annotationStart();
-    virtual void annotationEnd();
-
     bool parse();
 
     void processSubDocQueue();
@@ -97,9 +82,10 @@ public:
     void finishDocument();
 
     typedef const wvWare::FunctorBase* FunctorPtr;
-    struct SubDocument {
-        SubDocument(FunctorPtr ptr, int d, const QString& n, const QString& extra)
-                : functorPtr(ptr), data(d), name(n), extraName(extra) {}
+    struct SubDocument
+    {
+        SubDocument( FunctorPtr ptr, int d, const QString& n, const QString& extra )
+            : functorPtr(ptr), data(d), name(n), extraName(extra) {}
         ~SubDocument() {}
         FunctorPtr functorPtr;
         int data;
@@ -107,12 +93,8 @@ public:
         QString extraName;
     };
 
-    // Provide access to those attributes for texthandler.cpp and tablehandler.cpp
-    QString masterStyleName(void) const { return m_masterStyleName; }
-    void set_writeMasterStyleName(bool val) { m_writeMasterStyleName = val; }
-    bool writeMasterStyleName(void) const { return m_writeMasterStyleName; }
-    bool writingHeader(void) const { return m_writingHeader; }
-    KoXmlWriter* headerWriter(void) const { return m_headerWriter; }
+    // Called by PictureHandler
+    KoStoreDevice* createPictureFrameSet(QString filename);
 
 public slots:
     // Connected to the KWordTextHandler only when parsing the body
@@ -120,25 +102,22 @@ public slots:
 
     void slotSectionEnd(wvWare::SharedPtr<const wvWare::Word97::SEP>);
 
-    // Add to our parsing queue, for headers, footers, footnotes, annotations, text boxes etc.
+    // Add to our parsing queue, for headers, footers, footnotes, text boxes etc.
     // Note that a header functor will parse ALL the header/footers (of the section)
-    void slotSubDocFound(const wvWare::FunctorBase* functor, int data);
+    void slotSubDocFound( const wvWare::FunctorBase* functor, int data );
 
-    void slotFootnoteFound(const wvWare::FunctorBase* functor, int data);
+    void slotFootnoteFound( const wvWare::FunctorBase* functor, int data );
 
-    void slotAnnotationFound(const wvWare::FunctorBase* functor, int data);
-
-    void slotHeadersFound(const wvWare::FunctorBase* functor, int data);
+    void slotHeadersFound( const wvWare::FunctorBase* functor, int data );
 
     void slotTableFound(KWord::Table* table);
 
     // Write out the frameset and add the key to the PICTURES tag
-    void slotPictureFound(const QString& frameName, const QString& pictureName, KoXmlWriter* writer,
-                          const wvWare::FunctorBase*);
+    void slotPictureFound( const QString& frameName, const QString& pictureName, const wvWare::FunctorBase* );
 
-    void slotDrawingFound(unsigned int globalCP, KoXmlWriter* writer);
-
-    void slotTextBoxFound(uint lid, KoXmlWriter* writer);
+    //track the current list depth when texthandler is parsing lists,
+    //because we may need to close the list here
+    void slotUpdateListDepth( int depth );
 
     // Similar to footnoteStart/footnoteEnd but for cells.
     // This is connected to KWordTableHandler
@@ -148,16 +127,13 @@ public slots:
 private:
     void processStyles();
     void processAssociatedStrings();
-    enum NewFrameBehavior { Reconnect = 0, NoFollowup = 1, Copy = 2 };
-    void generateFrameBorder(QDomElement& frameElementOut, const wvWare::Word97::BRC& brcTop, const wvWare::Word97::BRC& brcBottom, const wvWare::Word97::BRC& brcLeft, const wvWare::Word97::BRC& brcRight, const wvWare::Word97::SHD& shd);
+    enum NewFrameBehavior { Reconnect=0, NoFollowup=1, Copy=2 };
+    void generateFrameBorder( QDomElement& frameElementOut, const wvWare::Word97::BRC& brcTop, const wvWare::Word97::BRC& brcBottom, const wvWare::Word97::BRC& brcLeft, const wvWare::Word97::BRC& brcRight, const wvWare::Word97::SHD& shd );
 
-    // Handlers for different data types in the document.
-    KWordTextHandler*        m_textHandler;
-    KWordTableHandler*       m_tableHandler;
     KWordReplacementHandler* m_replacementHandler;
-    KWordPictureHandler*     m_pictureHandler;
-    KWordDrawingHandler*     m_drawingHandler;
-
+    KWordTableHandler* m_tableHandler;
+    KWordPictureHandler* m_pictureHandler;
+    KWordTextHandler* m_textHandler;
     KoFilterChain* m_chain;
     wvWare::SharedPtr<wvWare::Parser> m_parser;
     std::queue<SubDocument> m_subdocQueue;
@@ -169,26 +145,19 @@ private:
     bool m_oddOpen; //we're processing an odd header or footer
     int m_footNoteNumber; // number of footnote _framesets_ written out
     int m_endNoteNumber; // number of endnote _framesets_ written out
-
-    // Helpers to generate the various parts of an ODF file.
-    KoXmlWriter* m_bodyWriter;      //for writing to the body of content.xml
-    KoGenStyles* m_mainStyles;      //for collecting styles
-    KoXmlWriter* m_metaWriter;      //for writing to meta.xml
-    KoGenStyle*  m_masterStyle;     //for header/footer stuff, at least
-    KoGenStyle*  m_pageLayoutStyle; //page layout style
-    KoXmlWriter* m_headerWriter;    //for header/footer writing in styles.xml
-
+    int m_currentListDepth; //track list depth in case we need to close a list here (-1 if no list)
+    KoXmlWriter* m_bodyWriter; //for writing to the body of content.xml
+    KoGenStyles* m_mainStyles; //for collecting styles
+    KoXmlWriter* m_metaWriter; //for writing to meta.xml
+    KoGenStyle* m_masterStyle; //for header/footer stuff, at least
+    KoGenStyle* m_pageLayoutStyle; //page layout style
+    KoXmlWriter* m_writer; //for header/footer tags
     bool m_hasHeader;
     bool m_hasFooter;
     QBuffer* m_buffer; //for header/footer tags
     QBuffer* m_bufferEven; //for even header/footer tags
     int m_headerCount; //just so we have a unique name for the element we're putting in m_masterStyle
     QString m_masterStyleName; //need to know what the master style name is so we can write it
-    bool m_writeMasterStyleName; //whether to write the master style name or not
-    bool m_writingHeader; //flag for headers/footers, where we write the actual text to styles.xml
-
-    int m_initialFootnoteNumber;
-    int m_initialEndnoteNumber;
 };
 
 #endif // DOCUMENT_H
