@@ -4,7 +4,9 @@
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
-   License version 2 as published by the Free Software Foundation.
+   version 2 of the License, or (at your option) version 3 or,
+   at the discretion of KDE e.V (which shall act as a proxy as in
+   section 14 of the GPLv3), any later version..
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,6 +23,8 @@
 #define MSWORDIMPORT_H
 
 #include <KoFilter.h>
+//Added by qt3to4:
+#include <QByteArray>
 
 class QDomDocument;
 class QDomElement;
@@ -29,10 +33,10 @@ class MSWordImport : public KoFilter
 {
     Q_OBJECT
 public:
-    MSWordImport( KoFilter* parent, const char* name, const QStringList& );
+    MSWordImport( QObject* parent, const QStringList& );
     virtual ~MSWordImport();
 
-    virtual KoFilter::ConversionStatus convert( const QCString& from, const QCString& to );
+    virtual KoFilter::ConversionStatus convert( const QByteArray& from, const QByteArray& to );
 
 private:
     void prepareDocument( QDomDocument& mainDocument, QDomElement& framesetsElem );
