@@ -50,7 +50,6 @@ using std::cerr;
 
 // KOffice
 #include <KoZoomHandler.h>
-#include <KoDom.h>
 #include <KoXmlNS.h>
 #include <KoXmlWriter.h>
 #include <KoXmlReader.h>
@@ -61,12 +60,13 @@ using std::cerr;
 #include <KoStoreDevice.h>
 #include <KoShapeSavingContext.h>
 #include <KoToolManager.h>
-#include <KoOasisLoadingContext.h>
-#include <KoOasisStyles.h>
+#include <KoOdfLoadingContext.h>
+#include <KoOdfStylesReader.h>
 #include <KoOdfReadStore.h>
 #include <KoOdfWriteStore.h>
 #include <KoShapeLoadingContext.h>
 #include <KoShapeRegistry.h>
+#include <KoDataCenter.h>
 
 // KDChart
 #include "KDChartChart"
@@ -98,7 +98,7 @@ KChartPart::KChartPart( QWidget *parentWidget,
     m_chartData( 0 )
 {
     // A KChart part contains a single chart Flake shape. 
-    m_chartShape = ( ChartShape* )( KoShapeRegistry::instance()->value("ChartShape") )->createDefaultShape();
+    m_chartShape = ( ChartShape* )( KoShapeRegistry::instance()->value("ChartShape") )->createDefaultShapeAndInit( dataCenterMap() );
     setComponentData( KChartFactory::global(), false );
     setTemplateType( "kchart_template" );
 
@@ -323,6 +323,13 @@ void KChartPart::removeShape( KoShape* )
 {
 }
 
+QMap<QString, KoDataCenter *> KChartPart::dataCenterMap() const
+{
+    //Q_ASSERT(false);
+    // if the kchart app ever supports more than a single shape this should be implemented
+    // along with several other methods - see any other app or ask boemann
+    return QMap<QString, KoDataCenter *>() ;
+}
 
 // ----------------------------------------------------------------
 
@@ -593,7 +600,7 @@ bool KChartPart::loadOdf( KoOdfReadStore & odfStore )
     KoXmlElement  content = odfStore.contentDoc().documentElement();
 
     // Find office:body
-    KoXmlElement  realBody ( KoDom::namedItemNS( content, 
+    KoXmlElement  realBody ( KoXml::namedItemNS( content, 
                                                  KoXmlNS::office, "body" ) );
     if ( realBody.isNull() ) {
         setErrorMessage( i18n( "Invalid OpenDocument file. No office:body tag found." ));
@@ -601,7 +608,7 @@ bool KChartPart::loadOdf( KoOdfReadStore & odfStore )
     }
 
     // Find office:chart
-    KoXmlElement bodyElement = KoDom::namedItemNS( realBody,
+    KoXmlElement bodyElement = KoXml::namedItemNS( realBody,
                                                    KoXmlNS::office, "chart" );
     if ( bodyElement.isNull() ) {
         kError(32001) << "No office:chart found!" << endl;
@@ -620,7 +627,7 @@ bool KChartPart::loadOdf( KoOdfReadStore & odfStore )
     }
 
     // Find chart:chart
-    KoXmlElement chartElement = KoDom::namedItemNS( bodyElement, 
+    KoXmlElement chartElement = KoXml::namedItemNS( bodyElement, 
                                                     KoXmlNS::chart, "chart" );
     if ( chartElement.isNull() ) {
         setErrorMessage( i18n( "No chart found in the file" ) );
@@ -630,11 +637,11 @@ bool KChartPart::loadOdf( KoOdfReadStore & odfStore )
     // Load styles first
 #if 0
     loadAllStyles( context );
-    styleManager()->loadOasisStyleTemplate( oasisStyles, this );
+    styleManager()->loadOasisStyleTemplate( stylesReader, this );
 #endif
 
-    KoOasisLoadingContext  context( this, odfStore.styles(), odfStore.store() );
-    KoShapeLoadingContext  shapeContext( context );
+    KoOdfLoadingContext  context( odfStore.styles(), odfStore.store() );
+    KoShapeLoadingContext  shapeContext( context, dataCenterMap() );
 
     // ----------------------------------------------------------------
     // FIXME: Load chart here.
@@ -659,7 +666,7 @@ bool KChartPart::loadOdf( KoOdfReadStore & odfStore )
     setChartDefaults();
 
     KoXmlElement  content = doc.documentElement();
-    KoXmlElement  bodyElem ( KoDom::namedItemNS( content,
+    KoXmlElement  bodyElem ( KoXml::namedItemNS( content,
 						KoXmlNS::office, "body" ) );
     if ( bodyElem.isNull() ) {
         kError(32001) << "No office:body found!" << endl;
@@ -668,7 +675,7 @@ bool KChartPart::loadOdf( KoOdfReadStore & odfStore )
     }
 
     // Get the office:chart element.
-    KoXmlElement officeChartElem = KoDom::namedItemNS( bodyElem,
+    KoXmlElement officeChartElem = KoXml::namedItemNS( bodyElem,
 						      KoXmlNS::office, "chart" );
     if ( officeChartElem.isNull() ) {
         kError(32001) << "No office:chart found!" << endl;
@@ -686,7 +693,7 @@ bool KChartPart::loadOdf( KoOdfReadStore & odfStore )
         return false;
     }
 
-    KoXmlElement chartElem = KoDom::namedItemNS( officeChartElem,
+    KoXmlElement chartElem = KoXml::namedItemNS( officeChartElem,
 						KoXmlNS::chart, "chart" );
     if ( chartElem.isNull() ) {
         setErrorMessage( i18n( "Invalid OASIS OpenDocument file. No chart:chart tag found." ) );
@@ -694,7 +701,7 @@ bool KChartPart::loadOdf( KoOdfReadStore & odfStore )
     }
 
     // Get the loading context and stylestack from the styles.
-    KoOasisLoadingContext  loadingContext( this, oasisStyles, store );
+    KoOdfLoadingContext  loadingContext( stylesReader, store );
     //KoStyleStack          &styleStack = loadingContext.styleStack();
 
 #if 0  // Example code!!
@@ -722,7 +729,7 @@ bool KChartPart::loadOdf( KoOdfReadStore & odfStore )
     // TODO Load data direction (see loadAuxiliary)
 
     // Load the data table.
-    KoXmlElement tableElem = KoDom::namedItemNS( chartElem,
+    KoXmlElement tableElem = KoXml::namedItemNS( chartElem,
 						KoXmlNS::table, "table" );
     if ( !tableElem.isNull() ) {
         ok = loadOasisData( tableElem );
@@ -740,7 +747,7 @@ bool KChartPart::loadOasisData( const KoXmlElement& tableElem )
     Q_UNUSED( tableElem );
 #if 0
     int          numberHeaderColumns = 0;
-    KoXmlElement  tableHeaderColumns = KoDom::namedItemNS( tableElem,
+    KoXmlElement  tableHeaderColumns = KoXml::namedItemNS( tableElem,
 							  KoXmlNS::table,
 							  "table-header-columns" );
 
@@ -756,7 +763,7 @@ bool KChartPart::loadOasisData( const KoXmlElement& tableElem )
     Q_ASSERT( numberHeaderColumns == 1 );
 
     int numberDataColumns = 0;
-    KoXmlElement tableColumns = KoDom::namedItemNS( tableElem, KoXmlNS::table, "table-columns" );
+    KoXmlElement tableColumns = KoXml::namedItemNS( tableElem, KoXmlNS::table, "table-columns" );
     forEachElement( elem, tableColumns ) {
         if ( elem.localName() == "table-column" ) {
             int repeated = elem.attributeNS( KoXmlNS::table, "number-columns-repeated", QString() ).toInt();
@@ -766,10 +773,10 @@ bool KChartPart::loadOasisData( const KoXmlElement& tableElem )
 
     // Parse table-header-rows for the column names.
     m_colLabels.clear();
-    KoXmlElement tableHeaderRows = KoDom::namedItemNS( tableElem, KoXmlNS::table, "table-header-rows" );
+    KoXmlElement tableHeaderRows = KoXml::namedItemNS( tableElem, KoXmlNS::table, "table-header-rows" );
     if ( tableHeaderRows.isNull() )
         kWarning(35001) << "No table-header-rows element found!" << endl;
-    KoXmlElement tableHeaderRow = KoDom::namedItemNS( tableHeaderRows, KoXmlNS::table, "table-row" );
+    KoXmlElement tableHeaderRow = KoXml::namedItemNS( tableHeaderRows, KoXmlNS::table, "table-row" );
     if ( tableHeaderRow.isNull() )
         kWarning(35001) << "No table-row inside table-header-rows!" << endl;
 
@@ -778,7 +785,7 @@ bool KChartPart::loadOasisData( const KoXmlElement& tableElem )
         if ( elem.localName() == "table-cell" ) {
             ++cellNum;
             if ( cellNum > numberHeaderColumns ) {
-                KoXmlElement pElem = KoDom::namedItemNS( elem, KoXmlNS::text, "p" );
+                KoXmlElement pElem = KoXml::namedItemNS( elem, KoXmlNS::text, "p" );
                 m_colLabels.append( pElem.text() );
             }
         }
@@ -791,7 +798,7 @@ bool KChartPart::loadOasisData( const KoXmlElement& tableElem )
 
     // Get the number of rows, and read row labels
     int          numberDataRows = 0;
-    KoXmlElement  tableRows = KoDom::namedItemNS( tableElem, KoXmlNS::table, "table-rows" );
+    KoXmlElement  tableRows = KoXml::namedItemNS( tableElem, KoXmlNS::table, "table-rows" );
 
     m_rowLabels.clear();
     forEachElement( elem, tableRows ) {
@@ -800,8 +807,8 @@ bool KChartPart::loadOasisData( const KoXmlElement& tableElem )
             Q_ASSERT( repeated <= 1 ); // we don't handle yet the case where data rows are repeated (can this really happen?)
             numberDataRows += qMax( 1, repeated );
             if ( numberHeaderColumns > 0 ) {
-                KoXmlElement firstCell = KoDom::namedItemNS( elem, KoXmlNS::table, "table-cell" );
-                KoXmlElement pElem = KoDom::namedItemNS( firstCell, KoXmlNS::text, "p" );
+                KoXmlElement firstCell = KoXml::namedItemNS( elem, KoXmlNS::table, "table-cell" );
+                KoXmlElement pElem = KoXml::namedItemNS( firstCell, KoXmlNS::text, "p" );
                 m_rowLabels.append( pElem.text() );
             }
         }
@@ -852,8 +859,10 @@ bool KChartPart::loadOasisData( const KoXmlElement& tableElem )
 }
 
 
-bool KChartPart::saveOasis( KoStore* store, KoXmlWriter* manifestWriter )
+bool KChartPart::saveOdf( SavingContext & documentContext )
 {
+    KoStore * store = documentContext.odfStore.store();
+    KoXmlWriter * manifestWriter = documentContext.odfStore.manifestWriter();
     // Check if we can create the content.xml file inside the store.
     if ( !store->open( "content.xml" ) )
         return false;
@@ -886,8 +895,9 @@ bool KChartPart::saveOasis( KoStore* store, KoXmlWriter* manifestWriter )
     contentTmpWriter.startElement( "office:chart" );
 
     // Write the chart itself.
-    KoGenStyles      mainStyles;
-    KoShapeSavingContext shapeSavingContext( contentTmpWriter, mainStyles );
+    KoGenStyles           mainStyles;
+    KoShapeSavingContext  shapeSavingContext( contentTmpWriter, mainStyles, 
+                                              documentContext.embeddedSaver );
     m_chartShape->saveOdf( shapeSavingContext );
 
     contentTmpWriter.endElement(); // office:chart
@@ -909,7 +919,7 @@ bool KChartPart::saveOasis( KoStore* store, KoXmlWriter* manifestWriter )
     // Phase 3: Add the contents to the file
     contentWriter->addCompleteElement( &contentTmpFile );
 
-    contentWriter->endElement(); // root element
+    contentWriter->endElement(); // office:automatic-styles
     contentWriter->endDocument();
     delete contentWriter;
 
@@ -934,7 +944,7 @@ bool KChartPart::saveOasis( KoStore* store, KoXmlWriter* manifestWriter )
         return false;
 
 
-    if(!store->open("settings.xml"))
+    if( !store->open("settings.xml") )
         return false;
 
     // Write settings
@@ -1087,7 +1097,7 @@ QDomDocument KChartPart::saveXML()
 }
 
 
-bool KChartPart::loadXML( QIODevice*, const KoXmlDocument& doc )
+bool KChartPart::loadXML( const KoXmlDocument& doc, KoStore* )
 {
     Q_UNUSED( doc );
 
