@@ -19,236 +19,68 @@
  * Boston, MA 02110-1301, USA.
 */
 
-
-// Own
 #include "CellRegion.h"
 
-// C
-#include <cmath>
+#include <kdebug.h>
 
-// Qt
 #include <QPoint>
 #include <QRect>
 #include <QVector>
 #include <QDebug>
 #include <QStringList>
 
-// KDE
-#include <kdebug.h>
-
+#include <cmath>
 
 using std::pow;
 using namespace KChart;
 
-static QString columnName( uint column );
-static int rangeCharToInt( char c );
-
-class CellRegion::Private
-{
-public:
-    Private();
-    ~Private();
-
-    QString pointToString( const QPoint &point ) const;
-    
-    // These are actually one-dimensional, but can have different
-    // orientations (hor / vert).
-    QVector<QRect> rects;
-
-    QRect          boundingRect;
-    // NOTE: Don't forget to extend operator=() if you add new members
-
-    // Determines, if not empty, what string this cell region was created from.
-    QString origString;
-    // ODF's definition of table:cell-range-address is much more comprehensive
-    // than what we currently support. Use the original string this cell region
-    // was initialized with (if string constructor used) as fall-back if no
-    // changes were made to this region.
-    bool origStringValid;
-
-    // Name of the sheet this region lies in
-    QString sheetName;
-};
-
-
-CellRegion::Private::Private()
-{
-    origStringValid = false;
-}
-
-CellRegion::Private::~Private()
-{
-}
-
-
-// ================================================================
-//                         Class CellRegion
-
 
 CellRegion::CellRegion()
-    : d( new Private() )
 {
 }
 
-CellRegion::CellRegion( const CellRegion &region )
-    : d( new Private() )
-{
-    // Use operator=(); 
-    *this = region;
-}
-
-CellRegion::CellRegion( const QString& region )
-    : d( new Private() )
-{
-    // A dollar sign before a part of the address means that this part
-    // is absolute. This is irrelevant for us, however, thus we can remove
-    // all occurences of '$', and handle relative and absolute addresses in
-    // the same way.
-    // See ODF specs $8.3.1 "Referencing Table Cells"
-    QString searchStr = QString( region ).remove( "$" );
-    QRegExp regEx;
-
-    const bool isPoint = !region.contains( ':' );
-    if ( isPoint )
-        regEx = QRegExp( "(|.*\\.)([A-Z]+)([0-9]+)" );
-    else // support range-notations like Sheet1.D2:Sheet1.F2 Sheet1.D2:F2 D2:F2
-        regEx = QRegExp ( "(|.*\\.)([A-Z]+)([0-9]+)\\:(|.*\\.)([A-Z]+)([0-9]+)" );
-
-    // Check if region string is valid (e.g. not empty)
-    if ( regEx.indexIn( searchStr ) >= 0 ) {
-        // It is possible for a cell-range-address as defined in ODF to contain
-        // refernces to cells of more than one sheet. This, however, we ignore
-        // here. We do not support more than one table in a cell region.
-        // Also we do not support regions spanned over different sheets. For us
-        // everything is either on no sheet or on the same sheet.
-        d->sheetName = regEx.cap( 1 );
-        if ( d->sheetName.endsWith( "." ) )
-            d->sheetName = d->sheetName.left( d->sheetName.length() - 1 );
-
-        QPoint topLeft( rangeStringToInt( regEx.cap(2) ), regEx.cap(3).toInt() );
-        if ( isPoint ) {
-            d->rects.append( QRect( topLeft, QSize( 1, 1 ) ) );
-        } else {
-            QPoint bottomRight( rangeStringToInt( regEx.cap(5) ), regEx.cap(6).toInt() );
-            d->rects.append( QRect( topLeft, bottomRight ) );
-        }
-    }
-
-    d->origString = region;
-    d->origStringValid = true;
-}
-
 CellRegion::CellRegion( const QPoint &point )
-    : d( new Private() )
 {
     add( point );
 }
 
 CellRegion::CellRegion( const QRect &rect )
-    : d( new Private() )
 {
     add( rect );
 }
 
 CellRegion::CellRegion( const QPoint &point, const QSize &size )
-    : d( new Private() )
 {
     add( QRect( point, size ) );
 }
 
 CellRegion::CellRegion( const QVector<QRect> &rects )
-    : d( new Private() )
 {
     add( rects );
 }
 
 CellRegion::~CellRegion()
 {
-    delete d;
 }
 
-
-CellRegion& CellRegion::operator = ( const CellRegion& region )
-{
-    d->rects        = region.d->rects;
-    d->boundingRect = region.d->boundingRect;
-    d->origString = region.d->origString;
-    d->origStringValid = region.d->origStringValid;
-    d->sheetName = region.d->sheetName;
-
-    return *this;
-}
-
-bool CellRegion::operator == ( const CellRegion &other ) const
-{
-    return d->rects == other.d->rects;
-}
-
-
-
 QVector<QRect> CellRegion::rects() const
 {
-    return d->rects;
+    return m_rects;
 }
 
 int CellRegion::rectCount() const
 {
-    return d->rects.size();
-}
-
-QString CellRegion::sheetName() const
-{
-    return d->sheetName;
+    return m_rects.size();
 }
 
 bool CellRegion::isValid() const
 {
-    return d->rects.size() > 0;
+    return m_rects.size() > 0;
 }
 
-QString CellRegion::Private::pointToString( const QPoint &point ) const
-{
-    QString result;
-
-    if(!sheetName.isEmpty())
-        result.append( '$' + sheetName + '.' );
-    result.append( '$' + columnName( point.x() ) );
-    result.append( '$' + QString::number( point.y() ) );
-
-    return result;
-}
-
-QString CellRegion::toString() const
-{
-    if ( d->origStringValid )
-        return d->origString;
-
-    if ( !isValid() )
-        return QString();
-
-    QString result;
-    for ( int i = 0; i < d->rects.count(); ++i ) {
-        const QRect range = d->rects[i];
-        // Top-left corner
-        result.append( d->pointToString( range.topLeft() ) );
-
-        // If it is not a point, append rect's bottom-right corner
-        if ( range.topLeft() != range.bottomRight() ) {
-            result.append( ':' );
-            result.append( d->pointToString( range.bottomRight() ) );
-        }
-
-        // Separate ranges by a comma, except for the last one
-        if ( i < d->rects.count() - 1 )
-            result.append( ';' );
-    }
-    return result;
-}
-
-
 bool CellRegion::contains( const QPoint &point, bool proper ) const
 {
-    foreach ( const QRect &rect, d->rects ) {
+    foreach ( const QRect &rect, m_rects ) {
         if ( rect.contains( point, proper ) )
             return true;
     }
@@ -258,7 +90,7 @@ bool CellRegion::contains( const QPoint &point, bool proper ) const
 
 bool CellRegion::contains( const QRect &rect, bool proper ) const
 {
-    foreach ( const QRect &r, d->rects ) {
+    foreach ( const QRect &r, m_rects ) {
         if ( r.contains( rect, proper ) )
             return true;
     }
@@ -268,7 +100,7 @@ bool CellRegion::contains( const QRect &rect, bool proper ) const
 
 bool CellRegion::intersects( const QRect &rect ) const
 {
-    foreach ( const QRect &r, d->rects ) {
+    foreach ( const QRect &r, m_rects ) {
         if ( r.intersects( rect ) )
             return true;
     }
@@ -280,7 +112,7 @@ CellRegion CellRegion::intersected( const QRect &rect ) const
 {
     CellRegion intersections;
     
-    foreach ( const QRect &r, d->rects ) {
+    foreach ( const QRect &r, m_rects ) {
         if ( r.intersects( rect ) )
             intersections.add( r.intersected( rect ) );
     }
@@ -290,7 +122,7 @@ CellRegion CellRegion::intersected( const QRect &rect ) const
 
 Qt::Orientation CellRegion::orientation() const
 {
-    foreach ( const QRect &rect, d->rects ) {
+    foreach ( const QRect &rect, m_rects ) {
     	if ( rect.width() > 1 )
     		return Qt::Horizontal;
     	if ( rect.height() > 1 )
@@ -305,11 +137,11 @@ int CellRegion::cellCount() const
 {
     int count = 0;
     if ( orientation() == Qt::Horizontal ) {
-        foreach ( const QRect &rect, d->rects )
+        foreach ( const QRect &rect, m_rects )
             count += rect.width();
     }
     else {
-        foreach( const QRect &rect, d->rects )
+        foreach( const QRect &rect, m_rects )
             count += rect.height();
     }
     
@@ -328,8 +160,6 @@ void CellRegion::add( const QPoint &point )
 
 void CellRegion::add( const QRect &rect )
 {
-    d->origStringValid = false;
-
     if ( !rect.isValid() ) {
         qWarning() << "CellRegion::add() Attempt to add invalid rectangle";
         qWarning() << "CellRegion::add():" << rect;
@@ -342,8 +172,8 @@ void CellRegion::add( const QRect &rect )
         return;
     }
     
-    d->rects.append( rect );
-    d->boundingRect |= rect;
+    m_rects.append( rect );
+    m_boundingRect |= rect;
 }
 
 void CellRegion::add( const QVector<QRect> &rects )
@@ -355,15 +185,15 @@ void CellRegion::add( const QVector<QRect> &rects )
 void CellRegion::subtract( const QPoint &point )
 {
     if ( orientation() == Qt::Horizontal ) {
-        for ( int i = 0; i < d->rects.size(); i++ ) {
-            if ( d->rects[ i ].contains( point ) ) {
+        for ( int i = 0; i < m_rects.size(); i++ ) {
+            if ( m_rects[ i ].contains( point ) ) {
 
-                if ( d->rects[ i ].topLeft().x() == point.x() ) {
-                    d->rects[ i ].translate( 1, 0 );
-                    d->rects[ i ].setWidth( d->rects[ i ].width() - 1 );
+                if ( m_rects[ i ].topLeft().x() == point.x() ) {
+                    m_rects[ i ].translate( 1, 0 );
+                    m_rects[ i ].setWidth( m_rects[ i ].width() - 1 );
                 }
-                else if ( d->rects[ i ].topRight().x() == point.x() ) {
-                    d->rects[ i ].setWidth( d->rects[ i ].width() - 1 );
+                else if ( m_rects[ i ].topRight().x() == point.x() ) {
+                    m_rects[ i ].setWidth( m_rects[ i ].width() - 1 );
                 }
 
                 return;
@@ -371,15 +201,15 @@ void CellRegion::subtract( const QPoint &point )
         }
     }
     else {
-        for ( int i = 0; i < d->rects.size(); i++ ) {
-            if ( d->rects[ i ].contains( point ) ) {
+        for ( int i = 0; i < m_rects.size(); i++ ) {
+            if ( m_rects[ i ].contains( point ) ) {
 
-                if ( d->rects[ i ].topLeft().y() == point.y() ) {
-                    d->rects[ i ].translate( 0, 1 );
-                    d->rects[ i ].setHeight( d->rects[ i ].height() - 1 );
+                if ( m_rects[ i ].topLeft().y() == point.y() ) {
+                    m_rects[ i ].translate( 0, 1 );
+                    m_rects[ i ].setHeight( m_rects[ i ].height() - 1 );
                 }
-                else if ( d->rects[ i ].bottomLeft().y() == point.y() ) {
-                    d->rects[ i ].setHeight( d->rects[ i ].height() - 1 );
+                else if ( m_rects[ i ].bottomLeft().y() == point.y() ) {
+                    m_rects[ i ].setHeight( m_rects[ i ].height() - 1 );
                 }
 
                 return;
@@ -388,19 +218,14 @@ void CellRegion::subtract( const QPoint &point )
     }
 
     // Recalculate bounding rectangle
-    d->boundingRect = QRect();
-    foreach ( const QRect &rect, d->rects )
-        d->boundingRect |= rect;
+    m_boundingRect = QRect();
+    foreach ( const QRect &rect, m_rects )
+        m_boundingRect |= rect;
 }
 
 QRect CellRegion::boundingRect() const
 {
-    return d->boundingRect;
-}
-
-bool CellRegion::hasPointAtIndex( int index ) const
-{
-    return pointAtIndex( index ) != QPoint( -1, -1 );
+    return m_boundingRect;
 }
 
 QPoint CellRegion::pointAtIndex( int index ) const
@@ -408,7 +233,7 @@ QPoint CellRegion::pointAtIndex( int index ) const
     // sum of all previous rectangle indices
     int i = 0;
     
-    foreach ( const QRect &rect, d->rects ) {
+    foreach ( const QRect &rect, m_rects ) {
         // Rectangle is horizontal
         if ( rect.width() > 1 ) {
             // Found it!
@@ -445,7 +270,7 @@ int CellRegion::indexAtPoint( const QPoint &point ) const
     int indicesLeftToPoint = 0;
     bool found = false;
     
-    foreach ( const QRect &rect, d->rects ) {
+    foreach ( const QRect &rect, m_rects ) {
         if ( !rect.contains( point ) ) {
             indicesLeftToPoint += rect.width() > 1 ? rect.width() : rect.height();
             continue;
@@ -461,7 +286,6 @@ int CellRegion::indexAtPoint( const QPoint &point ) const
     return found ? indicesLeftToPoint : -1;
 }
 
-#if 0 // Unused?
 static int rangeCharToInt( char c )
 {
     return (c >= 'A' && c <= 'Z') ? (c - 'A' + 1) : -1;
@@ -472,10 +296,10 @@ static int rangeStringToInt( const QString &string )
     int result = 0;
     const int size = string.size();
     for ( int i = 0; i < size; i++ ) {
-        //kDebug(350001) << "---" << float( rangeCharToInt( string[i].toAscii() ) * pow( 10.0, ( size - i - 1 ) ) );
+        kDebug(350001) << "---" << float( rangeCharToInt( string[i].toAscii() ) * pow( 10.0, ( size - i - 1 ) ) );
         result += rangeCharToInt( string[i].toAscii() ) * pow( 10.0, ( size - i - 1 ) );
     }
-    //kDebug(350001) << "+++++ result=" << result;
+    kDebug(350001) << "+++++ result=" << result;
     return result;
 }
 
@@ -486,14 +310,61 @@ static QString rangeIntToString( int i )
         tmp[j] = 'A' + tmp[j].toAscii() - '1';
     }
 
-    //kDebug(350001) << "tmp=" << tmp;
+    kDebug(350001) << "tmp=" << tmp;
     return tmp;
 }
-#endif
+
 // static
 QVector<QRect> CellRegion::stringToRegion( const QString &string )
 {
-    return CellRegion( string ).rects();
+    if ( string.isEmpty() )
+        return QVector<QRect>();
+    
+    const bool isPoint = !string.contains( ':' );
+    kDebug(350001) << "CellRegion::stringToRegion():" << string;
+    QString s = string;
+    QStringList regionStrings = isPoint ? s.split( '.' ) : s.remove( ':' ).split( '.' );
+    QPoint topLeftPoint;
+    QPoint bottomRightPoint;
+
+    if ( isPoint && regionStrings.size() < 2 || !isPoint && regionStrings.size() < 3 )
+    {
+        qWarning() << "1) CellRegion::stringToRegion(): Invalid region string \"" << string << "\"";
+        return QVector<QRect>();
+    }
+
+    const QString tableName = regionStrings[0];
+
+    const QString topLeft = regionStrings[1];
+    QStringList l = topLeft.split( '$' );
+    if ( l.size() < 3 ) {
+        kDebug(350001) << topLeft;
+        qWarning() << "2) CellRegion::stringToRegion(): Invalid region string \"" << string << "\"";
+        return QVector<QRect>();
+    }
+
+    int column = rangeStringToInt( l[1] );
+    int row = l[2].toInt() - 1;
+    topLeftPoint = QPoint( column, row );
+
+    if ( isPoint ) {
+        kDebug(350001) << "Returning" << QVector<QRect>( 1, QRect( topLeftPoint, QSize( 1, 1 ) ) );
+        return QVector<QRect>( 1, QRect( topLeftPoint, QSize( 1, 1 ) ) );
+    }
+
+    const QString bottomRight = regionStrings[2];
+    l = bottomRight.split( '$' );
+    if ( l.size() < 3 ) {
+        qWarning() << "CellRegion::stringToRegion(): Invalid region string \"" << string << "\"";
+        return QVector<QRect>();
+    }
+    column = rangeStringToInt( l[1] );
+    row = l[2].toInt() - 1;
+    bottomRightPoint = QPoint( column, row );
+
+    kDebug(350001) << "///" << QRect( topLeftPoint, bottomRightPoint );
+
+    return QVector<QRect>( 1, QRect( topLeftPoint, bottomRightPoint ) );
 }
 
 int CellRegion::rangeCharToInt( char c )     
@@ -534,7 +405,7 @@ static QString columnName( uint column )
 
     column--;
 
-    for( unsigned limit = 26; column >= limit + offset; limit *= 26, ++digits )
+    for( unsigned limit = 26; column >= limit + offset; limit *= 26, digits++ )
         offset += limit;
 
     for( unsigned col = column - offset; digits; --digits, col /= 26 )
@@ -546,5 +417,31 @@ static QString columnName( uint column )
 // static
 QString CellRegion::regionToString( const QVector<QRect> &region )
 {
-    return CellRegion( region ).toString();
+    if ( region.isEmpty() )
+        return QString();
+    
+    QString result;
+    for ( int i = 0; i < region.count(); ++i ) {
+        const QRect range = region[i];
+        result.append( '$' );
+        result.append( columnName( range.left() ) );
+            result.append( '$' );
+        result.append( QString::number( range.top() ) );
+        if ( range.topLeft() != range.bottomRight() ) {
+            result.append( ':' );
+            result.append( '$' );
+            result.append( columnName(range.right() ) );
+            result.append( '$' );
+            result.append( QString::number( range.bottom() ) );
+        }
+        if ( i < region.count() - 1 ) {
+            result.append( ';' );
+        }
+    }
+    return result;
+}
+
+bool CellRegion::operator == ( const CellRegion &other ) const
+{
+    return m_rects == other.rects();
 }
