@@ -18,23 +18,21 @@
    Boston, MA 02110-1301, USA.
 */
 
-#ifndef CHART_CHART_TABLE_VIEW
-#define CHRT_CHART_TABLE_VIEW
+#ifndef CHART_TABLE_VIEW
+#define CHART_TABLE_VIEW
 
 #include <QTableView>
 
 namespace KChart {
 
-class ChartProxyModel;
-
 class ChartTableView : public QTableView
 {
 public:
     ChartTableView( QWidget *parent = 0 );
     ~ChartTableView();
 
-   void setModel( ChartProxyModel *model );
-   ChartProxyModel *model();
+   void setModel( QAbstractItemModel *model );
+   QAbstractItemModel *model();
 
    virtual void commitData( QWidget *editor );
 
