@@ -1,5 +1,4 @@
 /* This file is part of the KDE project
- *
  * Copyright (C) 2007      Inge Wallin <inge@lysator.liu.se>
  *
  * This library is free software; you can redistribute it and/or
@@ -18,8 +17,15 @@
  * Boston, MA 02110-1301, USA.
  */
 
-// Own
+// Local
 #include "ChartTool.h"
+#include "ProxyModel.h"
+#include "Surface.h"
+#include "PlotArea.h"
+#include "Axis.h"
+#include "DataSet.h"
+#include "Legend.h"
+#include "KDChartConvertions.h"
 
 // Qt
 #include <QAction>
@@ -29,7 +35,6 @@
 #include <QTabWidget>
 #include <QPen>
 #include <QBrush>
-#include <QPainter>
 
 // KDE
 #include <KLocale>
@@ -42,7 +47,9 @@
 #include <KoShapeManager.h>
 #include <KoPointerEvent.h>
 #include <KoTextShapeData.h>
-#include <KoViewConverter.h>
+
+// ChartShape
+#include "ChartConfigWidget.h"
 
 // KDChart
 #include <KDChartChart>
@@ -52,17 +59,6 @@
 #include <KDChartCartesianCoordinatePlane>
 #include <KDChartPosition>
 
-// KChart
-#include "Surface.h"
-#include "PlotArea.h"
-#include "Axis.h"
-#include "DataSet.h"
-#include "Legend.h"
-#include "ChartProxyModel.h"
-#include "ChartConfigWidget.h"
-#include "KDChartConvertions.h"
-
-
 using namespace KChart;
 
 
@@ -79,7 +75,6 @@ public:
 };
 
 ChartTool::Private::Private()
-    : shape(0)
 {
 }
 
@@ -105,8 +100,6 @@ ChartTool::ChartTool( KoCanvasBase *canvas )
     connect( m_foo, SIGNAL(toggled(bool)), this, SLOT(catchBar(bool)) );
 
 #endif
-    connect( canvas->shapeManager()->selection(), SIGNAL( selectionChanged() ),
-             this, SLOT( shapeSelectionChanged() ) );
 }
 
 ChartTool::~ChartTool()
@@ -114,73 +107,15 @@ ChartTool::~ChartTool()
     delete d;
 }
 
-void ChartTool::shapeSelectionChanged()
-{
-    KoShape *selectedShape = 0;
-    
-    // Get the chart shape that the tool is working on. 
-    // Let d->shape point to it.
-    KoSelection  *selection = m_canvas->shapeManager()->selection();
-    foreach ( KoShape *shape, selection->selectedShapes() ) {
-        // Find out which type of shape that the user clicked on.
-        // We support several here, since the chart shape is comprised
-        // of several subshapes (plotarea, legend)
-        d->shape = dynamic_cast<ChartShape*>( shape );
-        if ( !d->shape ) {
-            PlotArea *plotArea = dynamic_cast<PlotArea*>( shape );
-            if ( plotArea ) {
-                selectedShape = plotArea;
-                d->shape = plotArea->parent();
-            }
-            else {
-                Legend *legend = dynamic_cast<Legend*>( shape );
-                if ( legend ) {
-                    selectedShape = legend;
-                    d->shape = dynamic_cast<ChartShape*>( shape->parent() );
-                }
-            }
-        // The selected shape is the chart
-        } else
-            selectedShape = shape;
-        
-        // Insert the values from the selected shape (note: not only
-        // chart shape, but also plotarea or legend) into the tool
-        // option widget.
-        if ( selectedShape ) {
-            foreach ( QWidget *w, optionWidgets() ) {
-                KoShapeConfigWidgetBase *widget = dynamic_cast<KoShapeConfigWidgetBase*>(w);
-                Q_ASSERT( widget );
-                if ( widget )
-                    widget->open( selectedShape );
-            }
-
-        // We support only one selected chart at the time, so once
-        // we found one, we don't need to search for any more
-        // among the selected shapes.
-        break;
-        }
-    }
-
-    // If we couldn't determine a chart shape, then there is nothing to do.
-    if ( !d->shape ) { // none found
-        emit done();
-        return;
-    }
-}
-
 
 void ChartTool::paint( QPainter &painter, const KoViewConverter &converter)
 {
-    Q_UNUSED( painter );
-    Q_UNUSED( converter );
 }
 
 void ChartTool::mousePressEvent( KoPointerEvent *event )
 {
-#if 1  // disabled
-    Q_UNUSED( event );
     return;
-#else
+    /*
     // Select dataset
     if (    !d->shape || !d->shape->kdChart() || ! d->shape->kdChart()->coordinatePlane()
          || !d->shape->kdChart()->coordinatePlane()->diagram() )
@@ -211,7 +146,7 @@ void ChartTool::mousePressEvent( KoPointerEvent *event )
     ((ChartConfigWidget*)optionWidget())->selectDataset( dataset );
         
     d->shape->update();
-#endif
+    */
 }
 
 void ChartTool::mouseMoveEvent( KoPointerEvent *event )
@@ -227,23 +162,77 @@ void ChartTool::mouseReleaseEvent( KoPointerEvent *event )
 
 void ChartTool::activate( bool )
 {
-    useCursor(Qt::ArrowCursor);
+    KoShape *selectedShape = 0;
+    
+    // Get the chart shape that the tool is working on. 
+    // Let d->shape point to it.
+    KoSelection  *selection = m_canvas->shapeManager()->selection();
+    foreach ( KoShape *shape, selection->selectedShapes() ) {
+
+	// Find out which type of shape that the user clicked on.
+	// We support several here, since the chart shape is comprised
+	// of several subshapes (plotarea, legend)
+        d->shape = dynamic_cast<ChartShape*>( shape );
+        if ( !d->shape ) {
+            PlotArea *plotArea = dynamic_cast<PlotArea*>( shape );
+            if ( plotArea ) {
+                selectedShape = plotArea;
+                d->shape = plotArea->parent();
+            }
+            else {
+                Legend *legend = dynamic_cast<Legend*>( shape );
+                if ( legend ) {
+                    selectedShape = legend;
+                    d->shape = dynamic_cast<ChartShape*>( legend->parent() );
+                }
+            }
+        }
+        
+	// Insert the values from the selected shape (note: not only
+	// chart shape, but also plotarea or legend) into the tool
+	// option widget.
+        if ( selectedShape ) {
+            foreach ( QWidget *w, optionWidgets() ) {
+                KoShapeConfigWidgetBase *widget = dynamic_cast<KoShapeConfigWidgetBase*>(w);
+                Q_ASSERT( widget );
+                if ( widget )
+                    widget->open( selectedShape );
+            }
+
+	    // We support only one selected chart at the time, so once
+	    // we found one, we don't need to search for any more
+	    // among the selected shapes.
+            break;
+        }
+    }
+
+    // If we couldn't determine a chart shape, then there is nothing to do.
+    if ( !d->shape ) { // none found
+        emit done();
+        return;
+    }
+
+    useCursor( Qt::ArrowCursor, true );
+
+    updateActions();
 }
 
 void ChartTool::deactivate()
 {
     d->shape = 0;
+}
 
-    // Tell the config widget to delete all open dialogs.
-    //
-    // The reason why we want to do that explicitly here is because
-    // they are connected to the models, which may disappear when the
-    // chart shape is destructed.
-    foreach (QWidget *w, optionWidgets()) {
-        ChartConfigWidget *configWidget = dynamic_cast<ChartConfigWidget*>(w);
-        if ( configWidget )
-            configWidget->deleteSubDialogs();
+
+void ChartTool::updateActions()
+{
+#if 0 // Taken from DivineProportion
+    switch(d->shape->orientation()) {
+        case ChartTypeShape::BottomRight: m_bottomRightOrientation->setChecked(true); break;
+        case ChartTypeShape::BottomLeft: m_bottomLeftOrientation->setChecked(true); break;
+        case ChartTypeShape::TopRight: m_topRightOrientation->setChecked(true); break;
+        case ChartTypeShape::TopLeft: m_topLeftOrientation->setChecked(true); break;
     }
+#endif
 }
 
 
@@ -251,6 +240,13 @@ QWidget *ChartTool::createOptionWidget()
 {
     ChartConfigWidget  *widget = new ChartConfigWidget();
     widget->open( d->shape );
+
+    connect( widget, SIGNAL( dataDirectionChanged( Qt::Orientation ) ),
+             this,   SLOT( setDataDirection( Qt::Orientation ) ) );
+    connect( widget, SIGNAL( firstRowIsLabelChanged( bool ) ),
+             this,   SLOT( setFirstRowIsLabel( bool ) ) );
+    connect( widget, SIGNAL( firstColumnIsLabelChanged( bool ) ),
+             this,   SLOT( setFirstColumnIsLabel( bool ) ) );
     
     connect( widget, SIGNAL( dataSetXDataRegionChanged( DataSet*, const QString& ) ),
 	     this,   SLOT( setDataSetXDataRegion( DataSet*, const QString& ) ) );
@@ -309,10 +305,10 @@ QWidget *ChartTool::createOptionWidget()
     		 this,   SLOT( setAxisShowGridLines( Axis*, bool ) ) );
     connect( widget, SIGNAL( axisUseLogarithmicScalingChanged( Axis*, bool ) ),
     		 this,   SLOT( setAxisUseLogarithmicScaling( Axis*, bool ) ) );
-    connect( widget, SIGNAL( axisStepWidthChanged( Axis*, qreal ) ),
-    		 this,   SLOT( setAxisStepWidth( Axis*, qreal ) ) );
-    connect( widget, SIGNAL( axisSubStepWidthChanged( Axis*, qreal ) ),
-    		 this,   SLOT( setAxisSubStepWidth( Axis*, qreal ) ) );
+    connect( widget, SIGNAL( axisStepWidthChanged( Axis*, double ) ),
+    		 this,   SLOT( setAxisStepWidth( Axis*, double ) ) );
+    connect( widget, SIGNAL( axisSubStepWidthChanged( Axis*, double ) ),
+    		 this,   SLOT( setAxisSubStepWidth( Axis*, double ) ) );
     connect( widget, SIGNAL( axisUseAutomaticStepWidthChanged( Axis*, bool ) ),
     		 this,   SLOT( setAxisUseAutomaticStepWidth( Axis*, bool ) ) );
     connect( widget, SIGNAL( axisUseAutomaticSubStepWidthChanged( Axis*, bool ) ),
@@ -365,7 +361,6 @@ void ChartTool::setChartSubType( ChartSubtype subtype )
     Q_ASSERT( d->shape );
     if ( !d->shape )
         return;
-
     d->shape->setChartSubType( subtype );
     d->shape->update();
 }
@@ -373,49 +368,43 @@ void ChartTool::setChartSubType( ChartSubtype subtype )
 
 void ChartTool::setDataSetXDataRegion( DataSet *dataSet, const QString &region )
 {
-    if ( !dataSet )
-        return;
-
-    dataSet->setXDataRegionString( region );
+	if ( !dataSet )
+	    return;
+	dataSet->setXDataRegionString( region );
 }
 
 void ChartTool::setDataSetYDataRegion( DataSet *dataSet, const QString &region )
 {
-    if ( !dataSet )
-        return;
-
-    dataSet->setYDataRegionString( region );
+	if ( !dataSet )
+	    return;
+	dataSet->setYDataRegionString( region );
 }
 
 void ChartTool::setDataSetCustomDataRegion( DataSet *dataSet, const QString &region )
 {
-    if ( !dataSet )
-        return;
-
-    dataSet->setCustomDataRegionString( region );
+	if ( !dataSet )
+	    return;
+	dataSet->setCustomDataRegionString( region );
 }
 
 void ChartTool::setDataSetLabelDataRegion( DataSet *dataSet, const QString &region )
 {
-    if ( !dataSet )
-        return;
-
-    dataSet->setLabelDataRegionString( region );
+	if ( !dataSet )
+	    return;
+	dataSet->setLabelDataRegionString( region );
 }
 
 void ChartTool::setDataSetCategoryDataRegion( DataSet *dataSet, const QString &region )
 {
-    if ( !dataSet )
-        return;
-
-    dataSet->setCategoryDataRegionString( region );
+	if ( !dataSet )
+	    return;
+	dataSet->setCategoryDataRegionString( region );
 }
 
 
 void ChartTool::setDataSetChartType( DataSet *dataSet, ChartType type )
 {
     Q_ASSERT( dataSet );
-
     if ( dataSet )
         dataSet->setChartType( type );
     d->shape->update();
@@ -435,7 +424,6 @@ void ChartTool::setDataSetColor( DataSet *dataSet, const QColor& color )
 {
     if ( !dataSet )
         return;
-
     dataSet->setColor( color );
     d->shape->update();
 }
@@ -444,7 +432,6 @@ void ChartTool::setDataSetAxis( DataSet *dataSet, Axis *axis )
 {
     if ( !dataSet || !axis )
         return;
-
     dataSet->attachedAxis()->detachDataSet( dataSet );
     axis->attachDataSet( dataSet );
     d->shape->update();
@@ -454,7 +441,6 @@ void ChartTool::setDataSetShowValues( DataSet *dataSet, bool b )
 {
     if ( !dataSet )
         return;
-
     dataSet->setShowValues( b );
     d->shape->update();
 }
@@ -463,7 +449,6 @@ void ChartTool::setDataSetShowLabels( DataSet *dataSet, bool b )
 {
     if ( !dataSet )
         return;
-
     dataSet->setShowLabels( b );
     d->shape->update();
 }
@@ -471,10 +456,10 @@ void ChartTool::setDataSetShowLabels( DataSet *dataSet, bool b )
 
 void ChartTool::setThreeDMode( bool threeD )
 {
+    qDebug() << "Activating three-d mode... tool: " << this;
     Q_ASSERT( d->shape );
     if ( !d->shape )
         return;
-
     d->shape->setThreeD( threeD );
     d->shape->update();
 }
@@ -484,11 +469,6 @@ void ChartTool::setShowTitle( bool show )
     Q_ASSERT( d->shape );
     if ( !d->shape )
         return;
-
-    // FIXME: Is this the reason why the plotarea doesn't grow again
-    //        if the title checkbox is unchecked?  I think it is.
-    //        Solve this by making showTitle take a bool.
-    //        Similar for the subtitle and footer below.
     if ( show )
         d->shape->showTitle();
     else
@@ -501,7 +481,6 @@ void ChartTool::setShowSubTitle( bool show )
     Q_ASSERT( d->shape );
     if ( !d->shape )
         return;
-
     if ( show )
         d->shape->showSubTitle();
     else
@@ -514,7 +493,6 @@ void ChartTool::setShowFooter( bool show )
     Q_ASSERT( d->shape );
     if ( !d->shape )
         return;
-
     if ( show )
         d->shape->showFooter();
     else
@@ -527,7 +505,6 @@ void ChartTool::setDataDirection( Qt::Orientation direction )
     Q_ASSERT( d->shape );
     if ( !d->shape )
         return;
-
     d->shape->proxyModel()->setDataDirection( direction );
     d->shape->relayout();
 }
@@ -545,7 +522,6 @@ void ChartTool::setFirstColumnIsLabel( bool b )
     Q_ASSERT( d->shape );
     if ( d->shape != 0 )
         d->shape->proxyModel()->setFirstColumnIsLabel( b );
-
     d->shape->relayout();
 }
 
@@ -554,7 +530,6 @@ void ChartTool::setLegendTitle( const QString &title )
 {
     Q_ASSERT( d->shape );
     Q_ASSERT( d->shape->legend() );
-
     d->shape->legend()->setTitle( title );
     d->shape->legend()->update();
 }
@@ -563,17 +538,14 @@ void ChartTool::setLegendFont( const QFont &font )
 {
     Q_ASSERT( d->shape );
     Q_ASSERT( d->shape->legend() );
-
     // There only is a general font, for the legend items and the legend title
     d->shape->legend()->setFont( font );
     d->shape->legend()->update();
 }
-
 void ChartTool::setLegendFontSize( int size )
 {
     Q_ASSERT( d->shape );
     Q_ASSERT( d->shape->legend() );
-
     d->shape->legend()->setFontSize( size );
     d->shape->legend()->update();
 }
@@ -582,7 +554,6 @@ void ChartTool::setLegendOrientation( Qt::Orientation orientation )
 {
     Q_ASSERT( d->shape );
     Q_ASSERT( d->shape->legend() );
-
     d->shape->legend()->setExpansion( QtOrientationToLegendExpansion( orientation ) );
     d->shape->legend()->update();
 }
@@ -591,7 +562,6 @@ void ChartTool::setLegendAlignment( Qt::Alignment alignment )
 {
     Q_ASSERT( d->shape );
     Q_ASSERT( d->shape->legend() );
-
     d->shape->legend()->setAlignment( alignment );
     d->shape->legend()->update();
 }
@@ -600,7 +570,6 @@ void ChartTool::setLegendFixedPosition( LegendPosition position )
 {
     Q_ASSERT( d->shape );
     Q_ASSERT( d->shape->legend() );
-
     d->shape->legend()->setLegendPosition( position );
 
     foreach (QWidget *w, optionWidgets()) {
@@ -614,7 +583,6 @@ void ChartTool::setLegendBackgroundColor( const QColor& color )
 {
     Q_ASSERT( d->shape );
     Q_ASSERT( d->shape->legend() );
-
     d->shape->legend()->setBackgroundColor( color );
     d->shape->legend()->update();
 }
@@ -623,7 +591,6 @@ void ChartTool::setLegendFrameColor( const QColor& color )
 {
     Q_ASSERT( d->shape );
     Q_ASSERT( d->shape->legend() );
-
     d->shape->legend()->setFrameColor( color );
     d->shape->legend()->update();
 }
@@ -632,7 +599,6 @@ void ChartTool::setLegendShowFrame( bool show )
 {
     Q_ASSERT( d->shape );
     Q_ASSERT( d->shape->legend() );
-
     d->shape->legend()->setShowFrame( show );
     d->shape->legend()->update();
 }
@@ -654,21 +620,19 @@ void ChartTool::addAxis( AxisPosition position, const QString& title )
 void ChartTool::removeAxis( Axis *axis )
 {
     Q_ASSERT( d->shape );
-
     d->shape->plotArea()->removeAxis( axis );
     d->shape->update();
 }
 
 void ChartTool::setAxisTitle( Axis *axis, const QString& title )
 {
-    axis->setTitleText( title );
+	axis->setTitleText( title );
     d->shape->update();
 }
 
 void ChartTool::setAxisShowTitle( Axis *axis, bool show )
 {
     Q_ASSERT( d->shape );
-
     axis->title()->setVisible( show );
     d->shape->update();
 }
@@ -686,13 +650,13 @@ void ChartTool::setAxisUseLogarithmicScaling( Axis *axis, bool b )
     d->shape->update();
 }
 
-void ChartTool::setAxisStepWidth( Axis *axis, qreal width )
+void ChartTool::setAxisStepWidth( Axis *axis, double width )
 {
     axis->setMajorInterval( width );
     d->shape->update();
 }
 
-void ChartTool::setAxisSubStepWidth( Axis *axis, qreal width )
+void ChartTool::setAxisSubStepWidth( Axis *axis, double width )
 {
     axis->setMinorInterval( width );
     d->shape->update();
@@ -714,7 +678,6 @@ void ChartTool::setAxisUseAutomaticSubStepWidth( Axis *axis, bool automatic )
 void ChartTool::setGapBetweenBars( int percent )
 {
     Q_ASSERT( d->shape );
-
     d->shape->plotArea()->setGapBetweenBars( percent );
     d->shape->update();
 }
@@ -722,7 +685,6 @@ void ChartTool::setGapBetweenBars( int percent )
 void ChartTool::setGapBetweenSets( int percent )
 {
     Q_ASSERT( d->shape );
-
     d->shape->plotArea()->setGapBetweenSets( percent );
     d->shape->update();
 }
@@ -730,7 +692,6 @@ void ChartTool::setGapBetweenSets( int percent )
 void ChartTool::setPieExplodeFactor( DataSet *dataSet, int percent )
 {
     Q_ASSERT( d->shape );
-
     d->shape->plotArea()->setPieExplodeFactor( dataSet, percent );
     d->shape->update();
 }
@@ -738,7 +699,6 @@ void ChartTool::setPieExplodeFactor( DataSet *dataSet, int percent )
 void ChartTool::setShowLegend( bool b )
 {
     Q_ASSERT( d->shape );
-
     d->shape->legend()->setVisible( b );
     d->shape->legend()->update();
 }
