@@ -1,9 +1,6 @@
 /* This file is part of the KDE project
 
    Copyright 2007-2008 Johannes Simon <johannes.simon@gmail.com>
-   Copyright (C) 2010 Carlos Licea    <carlos@kdab.com>
-   Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
-     Contact: Suresh Chande suresh.chande@nokia.com
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -19,30 +16,20 @@
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
- */
-
+*/
 
 #ifndef KCHART_DATASET_H
 #define KCHART_DATASET_H
 
+// Local
+#include "ChartShape.h"
+#include "CellRegion.h"
 
 // Qt
 #include <QPen>
 
-//KDChart
-#include <KDChartPieAttributes>
-
-// KChart
-#include "ChartShape.h"
-#include "CellRegion.h"
-
-class KoShapeLoadingContext;
-
 namespace KChart {
 
-class KDChartModel;
-
-
 /**
  * @brief The DataSet class stores properties of a single data series.
  * 
@@ -50,36 +37,32 @@ class KDChartModel;
  * on a data series.
  * 
  * To change properties of a single data point inside a data series,
- * use section's like for example \a brush( int section ) where the
- * section refers to the data-point number.
+ * use the DataPointAttributes class. (To be implemented)
  */
 
 class CHARTSHAPELIB_EXPORT DataSet
 {
 public:
-    DataSet( ChartProxyModel *model );
+    DataSet( ProxyModel *model );
     ~DataSet();
 
     // Getter methods
-    QString       title() const;
-    ChartType     chartType() const;
-    ChartSubtype  chartSubType() const;
-    Axis         *attachedAxis() const;
+    QString title() const;
+    ChartType chartType() const;
+    ChartSubtype chartSubType() const;
+    ChartType globalChartType() const;
+    ChartSubtype globalChartSubType() const;
+    Axis *attachedAxis() const;
 
-    ChartProxyModel   *model() const;
+    ProxyModel *model() const;
 
     bool showValues() const;
     bool showLabels() const;
 
-    // Graphics properties for the visualization of this dataset.
-    QPen   pen() const;
+    QPen pen() const;
     QBrush brush() const;
-    KDChart::PieAttributes pieAttributes() const;
-    QPen   pen( int section ) const;
-    QBrush brush( int section ) const;
-    KDChart::PieAttributes pieAttributes( int section ) const;
     QColor color() const;
-    int    number() const;
+    int number() const;
 
     bool showMeanValue() const;
     QPen meanValuePen() const;
@@ -88,14 +71,16 @@ public:
     bool showUpperErrorIndicator() const;
     QPen errorIndicatorPen() const;
     ErrorCategory errorCategory() const;
-    qreal errorPercentage() const;
-    qreal errorMargin() const;
-    qreal lowerErrorLimit() const;
-    qreal upperErrorLimit() const;
+    double errorPercentage() const;
+    double errorMargin() const;
+    double lowerErrorLimit() const;
+    double upperErrorLimit() const;
 
     // Setter methods
     void setChartType( ChartType type );
     void setChartSubType( ChartSubtype type );
+    void setGlobalChartType( ChartType type );
+    void setGlobalChartSubType( ChartSubtype type );
     void setAttachedAxis( Axis *axis );
 
     void setShowValues( bool showValues );
@@ -103,14 +88,9 @@ public:
 
     void setPen( const QPen &pen );
     void setBrush( const QBrush &brush );
-    void setPen( int section, const QPen &pen );
-    void setBrush( int section, const QBrush &brush );
     void setColor( const QColor &color );
     void setNumber( int num );
 
-    void setPieExplodeFactor( int factor );
-    void setPieExplodeFactor( int section, int factor );
-
     void setShowMeanValue( bool b );
     void setMeanValuePen( const QPen &pen );
 
@@ -119,10 +99,10 @@ public:
     void setShowErrorIndicators( bool lower, bool upper );
     void setErrorIndicatorPen( const QPen &pen );
     void setErrorCategory( ErrorCategory category );
-    void setErrorPercentage( qreal percentage );
-    void setErrorMargin( qreal margin );
-    void setLowerErrorLimit( qreal limit );
-    void setUpperErrorLimit( qreal limit );
+    void setErrorPercentage( double percentage );
+    void setErrorMargin( double margin );
+    void setLowerErrorLimit( double limit );
+    void setUpperErrorLimit( double limit );
 
     QVariant xData( int index ) const;
     QVariant yData( int index ) const;
@@ -182,9 +162,6 @@ public:
     
     void blockSignals( bool block );
 
-    bool loadOdf( const KoXmlElement &n,
-                  KoShapeLoadingContext &context );
-
 private:
     class Private;
     Private *const d;
