@@ -1,7 +1,6 @@
 /* This file is part of the KDE project
 
    Copyright 2007 Johannes Simon <johannes.simon@gmail.com>
-   Copyright 2010 Inge Wallin    <inge@lysator.liu.se>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -19,8 +18,10 @@
    Boston, MA 02110-1301, USA.
 */
 
-// Own
+// Local
 #include "Legend.h"
+#include "PlotArea.h"
+#include "KDChartConvertions.h"
 
 // Qt
 #include <QString>
@@ -31,6 +32,14 @@
 #include <QFont>
 #include <QImage>
 
+// KDChart
+#include <KDChartChart>
+#include <KDChartBarDiagram>
+#include <KDChartAbstractDiagram>
+#include <KDChartFrameAttributes>
+#include <KDChartBackgroundAttributes>
+#include <KDChartLegend>
+
 // KOffice
 #include <KoXmlReader.h>
 #include <KoXmlWriter.h>
@@ -41,20 +50,7 @@
 #include <KoXmlNS.h>
 #include <KoGenStyles.h>
 #include <KoUnit.h>
-#include <KoColorBackground.h>
-#include <KoLineBorder.h>
-
-// KDChart
-#include <KDChartChart>
-#include <KDChartBarDiagram>
-#include <KDChartAbstractDiagram>
-#include <KDChartFrameAttributes>
-#include <KDChartBackgroundAttributes>
-#include <KDChartLegend>
-#include "KDChartConvertions.h"
-
-// Kchart
-#include "PlotArea.h"
+#include <KoShapeBackground.h>
 
 using namespace KChart;
 
@@ -64,8 +60,6 @@ public:
     ~Private();
     
     ChartShape *shape;
-
-    // Properties of the Legend
     QString title;
     bool showFrame;
     QPen framePen;
@@ -76,22 +70,18 @@ public:
     QFont titleFont;
     QColor fontColor;
     Qt::Alignment alignment;
-    KoLineBorder *lineBorder;
-
-    // The connection to KDChart
+    
     KDChart::Legend *kdLegend;
     
     QImage image;
     
-    mutable bool pixmapRepaintRequested;
+    bool pixmapRepaintRequested;
     QSizeF lastSize;
     QPointF lastZoomLevel;
 };
 
-
 Legend::Private::Private()
 {
-    lineBorder = new KoLineBorder( 0.5, Qt::black );
     showFrame = true;
     framePen = QPen();
     backgroundBrush = QBrush();
@@ -103,40 +93,31 @@ Legend::Private::Private()
 
 Legend::Private::~Private()
 {
-    delete lineBorder;
 }
 
 
 Legend::Legend( ChartShape *parent )
     : d( new Private() )
-    , QObject( parent )
 {
     Q_ASSERT( parent );
     
     setShapeId( ChartShapeId );
     
     d->shape = parent;
-
+    
     d->kdLegend = new KDChart::Legend();
     
     setTitleFontSize( 10 );
     setTitle( QString() );
     setFontSize( 8 );
-
+    
     update();
     
     parent->addChild( this );
-
-    connect ( d->kdLegend, SIGNAL( propertiesChanged() ),
-              this,        SLOT( slotKdLegendChanged() ) );
-    connect ( parent, SIGNAL( chartTypeChanged( ChartType ) ),
-              this,   SLOT( slotChartTypeChanged( ChartType ) ) );
 }
 
 Legend::~Legend()
 {
-    delete d->kdLegend;
-    delete d;
 }
 
 
@@ -160,7 +141,12 @@ bool Legend::showFrame() const
 void Legend::setShowFrame( bool show )
 {
     d->showFrame = show;
-    setBorder( show ? d->lineBorder : 0 );
+    
+    // KDChart
+    KDChart::FrameAttributes attributes = d->kdLegend->frameAttributes();
+    attributes.setVisible( show );
+    d->kdLegend->setFrameAttributes( attributes );
+    d->pixmapRepaintRequested = true;
 }
 
 QPen Legend::framePen() const
@@ -176,7 +162,6 @@ void Legend::setFramePen( const QPen &pen )
     KDChart::FrameAttributes attributes = d->kdLegend->frameAttributes();
     attributes.setPen( pen  );
     d->kdLegend->setFrameAttributes( attributes );
-
     d->pixmapRepaintRequested = true;
 }
 
@@ -196,7 +181,6 @@ void Legend::setFrameColor( const QColor &color )
     pen.setColor( color );
     attributes.setPen( pen );
     d->kdLegend->setFrameAttributes( attributes );
-
     d->pixmapRepaintRequested = true;
 }
 
@@ -214,7 +198,6 @@ void Legend::setBackgroundBrush( const QBrush &brush )
     attributes.setVisible( true );
     attributes.setBrush( brush );
     d->kdLegend->setBackgroundAttributes( attributes );
-
     d->pixmapRepaintRequested = true;
 }
 
@@ -234,7 +217,6 @@ void Legend::setBackgroundColor( const QColor &color )
     brush.setColor( color );
     attributes.setBrush( brush );
     d->kdLegend->setBackgroundAttributes( attributes );
-
     d->pixmapRepaintRequested = true;
 }
 
@@ -251,16 +233,15 @@ void Legend::setFont( const QFont &font )
     KDChart::TextAttributes attributes = d->kdLegend->textAttributes();
     attributes.setFont( font );
     d->kdLegend->setTextAttributes( attributes );
-
     d->pixmapRepaintRequested = true;
 }
 
-qreal Legend::fontSize() const
+double Legend::fontSize() const
 {
     return d->font.pointSizeF();
 }
 
-void Legend::setFontSize( qreal size )
+void Legend::setFontSize( double size )
 {
     d->font.setPointSizeF( size );
 
@@ -270,7 +251,6 @@ void Legend::setFontSize( qreal size )
     m.setValue( size );
     attributes.setFontSize( m );
     d->kdLegend->setTextAttributes( attributes );
-
     d->pixmapRepaintRequested = true;
 }
 
@@ -287,16 +267,15 @@ void Legend::setTitleFont( const QFont &font )
     KDChart::TextAttributes attributes = d->kdLegend->titleTextAttributes();
     attributes.setFont( font );
     d->kdLegend->setTitleTextAttributes( attributes );
-
     d->pixmapRepaintRequested = true;
 }
 
-qreal Legend::titleFontSize() const
+double Legend::titleFontSize() const
 {
     return d->titleFont.pointSizeF();
 }
 
-void Legend::setTitleFontSize( qreal size )
+void Legend::setTitleFontSize( double size )
 {
     d->titleFont.setPointSizeF( size );
 
@@ -304,7 +283,6 @@ void Legend::setTitleFontSize( qreal size )
     KDChart::TextAttributes attributes = d->kdLegend->titleTextAttributes();
     attributes.setFontSize( KDChart::Measure( size, KDChartEnums::MeasureCalculationModeAbsolute ) );
     d->kdLegend->setTitleTextAttributes( attributes );
-
     d->pixmapRepaintRequested = true;
 }
 
@@ -360,7 +338,7 @@ void Legend::paintPixmap( QPainter &painter, const KoViewConverter &converter )
     pixmapPainter.setRenderHints( painter.renderHints() );
     pixmapPainter.setRenderHint( QPainter::Antialiasing, false );
 
-    // Scale the painter's coordinate system to fit the current zoom level.
+    // scale the painter's coordinate system to fit the current zoom level
     applyConversion( pixmapPainter, converter );
     d->kdLegend->paint( &pixmapPainter );
 }
@@ -400,7 +378,8 @@ void Legend::paint( QPainter &painter, const KoViewConverter &converter )
     }*/
 
     // Paint the background
-    if ( background() ) {
+    if( background() )
+    {
         QPainterPath p;
         p.addRect( paintRect );
         background()->paint( painter, p );
@@ -414,17 +393,19 @@ void Legend::paint( QPainter &painter, const KoViewConverter &converter )
 }
 
 
-// Only reimplemented because pure virtual in KoShape, but not needed
-bool Legend::loadOdf( const KoXmlElement &legendElement, 
-                      KoShapeLoadingContext &context )
+// Only reimplemneted because pure virtual in KoShape, but not needed
+bool Legend::loadOdf( const KoXmlElement &legendElement, KoShapeLoadingContext &context )
 {
     KoStyleStack &styleStack = context.odfLoadingContext().styleStack();
     styleStack.save();
 
-    if ( legendElement.hasAttributeNS( KoXmlNS::chart, "style-name" ) ) {
+    styleStack.clear();
+    if( legendElement.hasAttributeNS( KoXmlNS::chart, "style-name" ) )
+    {
         context.odfLoadingContext().fillStyleStack( legendElement, KoXmlNS::chart, "style-name", "chart" );
         styleStack.setTypeProperties( "graphic" );
     }
+    loadOdfAttributes( legendElement, context, OdfAllAttributes );
 
     // TODO: Read optional attributes
     // 1. Legend expansion
@@ -434,14 +415,9 @@ bool Legend::loadOdf( const KoXmlElement &legendElement,
 
     if ( !legendElement.isNull() ) {
         QString lp;
-        int attributesToLoad = OdfAllAttributes;
         if ( legendElement.hasAttributeNS( KoXmlNS::chart, "legend-position" ) ) {
-            attributesToLoad ^= OdfPosition;
             lp = legendElement.attributeNS( KoXmlNS::chart, "legend-position", QString() );
         }
-
-        loadOdfAttributes( legendElement, context, attributesToLoad );
-
         QString lalign;
         if ( legendElement.hasAttributeNS( KoXmlNS::chart, "legend-align" ) ) {
             lalign = legendElement.attributeNS( KoXmlNS::chart, "legend-align", QString() );
@@ -503,7 +479,8 @@ bool Legend::loadOdf( const KoXmlElement &legendElement,
             
             styleStack.setTypeProperties( "text" );
             
-            if ( styleStack.hasProperty( KoXmlNS::fo, "font-size" ) ) {
+            if ( styleStack.hasProperty( KoXmlNS::fo, "font-size" ) )
+            {
                 setFontSize( KoUnit::parseValue( styleStack.property( KoXmlNS::fo, "font-size" ) ) );
             }
         }
@@ -553,7 +530,8 @@ void Legend::saveOdf( KoShapeSavingContext &context ) const
         saveStyle( *style, context );
     
     QString  lexpansion;
-    switch ( expansion() ) {
+    switch ( expansion() )
+    {
     case WideLegendExpansion:
         lexpansion = "wide";
         break;
@@ -578,34 +556,11 @@ KDChart::Legend *Legend::kdLegend() const
     return d->kdLegend;
 }
 
-void Legend::update() const
+void Legend::update()
 {
     d->pixmapRepaintRequested = true;
+    QSize size = d->kdLegend->sizeHint();
+    // Scale size from px to pt
+    setSize( QSizeF( size ) );
     KoShape::update();
 }
-
-void Legend::slotKdLegendChanged()
-{
-    // FIXME: Update legend properly by implementing all *DataChanged() slots
-    // in KDChartModel. Right now, only yDataChanged() is implemented.
-    //d->kdLegend->forceRebuild();
-    QSize size = d->kdLegend->sizeHint();
-    // FIXME: Scale size from px to pt?
-    d->shape->setLegendSize( size );
-    update();
-}
-
-void Legend::slotChartTypeChanged( ChartType chartType )
-{
-    // TODO: Once we support markers, this switch will have to be
-    // more clever.
-    switch ( chartType ) {
-    case LineChartType:
-    case ScatterChartType:
-        d->kdLegend->setLegendStyle( KDChart::Legend::LinesOnly );
-        break;
-    default:
-        d->kdLegend->setLegendStyle( KDChart::Legend::MarkersOnly );
-        break;
-    }
-}
