@@ -1,7 +1,6 @@
 /* This file is part of the KDE project
 
    Copyright 2007-2008 Johannes Simon <johannes.simon@gmail.com>
-   Copyright 2009-2010 Inge Wallin <inge@lysator.liu.se>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -19,16 +18,13 @@
    Boston, MA 02110-1301, USA.
 */
 
-// Own
+// Local
 #include "PlotArea.h"
-
-// Qt
-#include <QPointF>
-#include <QSizeF>
-#include <QList>
-#include <QImage>
-#include <QPainter>
-#include <kdebug.h>
+#include "Axis.h"
+#include "DataSet.h"
+#include "Surface.h"
+#include "ThreeDScene.h"
+#include "ProxyModel.h"
 
 // KOffice
 #include <KoXmlReader.h>
@@ -45,11 +41,17 @@
 #include <KoShapeBackground.h>
 #include <KoOdfGraphicStyles.h>
 
+// Qt
+#include <QPointF>
+#include <QSizeF>
+#include <QList>
+#include <QImage>
+#include <QPainter>
+
 // KDChart
 #include <KDChartChart>
 #include <KDChartCartesianAxis>
 #include <KDChartAbstractDiagram>
-
 #include <KDChartAbstractCartesianDiagram>
 #include <KDChartAbstractCoordinatePlane>
 #include <KDChartBarAttributes>
@@ -67,80 +69,42 @@
 #include <KDChartRingDiagram>
 #include <KDChartPolarDiagram>
 
-// KChart
-#include "Legend.h"
-#include "Surface.h"
-#include "Axis.h"
-#include "DataSet.h"
-#include "ThreeDScene.h"
-#include "ChartProxyModel.h"
-#include "ScreenConversions.h"
-#include "Layout.h"
-
 using namespace KChart;
 
 const int MAX_PIXMAP_SIZE = 1000;
 
-Q_DECLARE_METATYPE( QPointer<QAbstractItemModel> )
-
 class PlotArea::Private
 {
 public:
-    Private( PlotArea *q, ChartShape *parent );
+    Private();
     ~Private();
 
-    void initAxes();
-
-    PlotArea *q;
     // The parent chart shape
     ChartShape *shape;
 
-    // ----------------------------------------------------------------
-    // Parts and properties of the chart
+    // The list of axes
+    QList<Axis*> axes;
+    
+    QList<KDChart::AbstractDiagram*> kdDiagrams;
+    
+    KDChart::Chart *kdChart;
+    KDChart::AbstractCoordinatePlane *kdPlane;
+    
+    Surface *wall;
+    Surface *floor;
+    
+    ChartType chartType;
+    ChartSubtype chartSubtype;
+    bool threeD;
+    
+    ThreeDScene *threeDScene;
+    
+    int gapBetweenBars;
+    int gapBetweenSets;
+    
+    QList<KoShape*> automaticallyHiddenAxisTitles;
 
-    ChartType     chartType;
-    ChartSubtype  chartSubtype;
-    
-    Surface       *wall;
-    Surface       *floor;       // Only used in 3D charts
-    
-    // The axes
-    QList<Axis*>     axes;
-    QList<KoShape*>  automaticallyHiddenAxisTitles;
-    
-    // 3D properties
-    bool          threeD;
-    ThreeDScene  *threeDScene;
-    
-    // ----------------------------------------------------------------
-    // Data specific to each types
-
-    // table:cell-range-address, ODF v1.2, $18.595
-    CellRegion cellRangeAddress;
-
-    // 1. Bar charts
-    // FIXME: OpenOffice stores these attributes in the axes' elements.
-    // The specs don't say anything at all about what elements can have
-    // these style attributes.
-    // chart:vertical attribute: see ODF v1.2, $19.63
-    bool  vertical;
-    int   gapBetweenBars;
-    int   gapBetweenSets;
-    
-    // 2. Pie charts
-    int   pieExplodeFactor;     // in percents
-    // TODO: Load+Save
-    qreal pieAngleOffset;       // in degrees
-
-    // ----------------------------------------------------------------
-    // The embedded KD Chart
-
-    // The KD Chart parts
-    KDChart::Chart                    *kdChart;
-    KDChart::AbstractCoordinatePlane  *kdPlane;
-    QList<KDChart::AbstractDiagram*>   kdDiagrams;
-    
-    // Caching: We can rerender faster if we cache KDChart's output
+    // We can rerender faster if we cache KDChart's output
     QImage   image;
     bool     paintPixmap;
     QPointF  lastZoomLevel;
@@ -148,129 +112,89 @@ public:
     mutable bool pixmapRepaintRequested;
 };
 
-PlotArea::Private::Private( PlotArea *q, ChartShape *parent )
-    : q(q)
-    , shape(parent)
+PlotArea::Private::Private()
 {
-    // Default type: normal bar chart
-    chartType    = BarChartType;
-    chartSubtype = NormalChartSubtype;
-
-    wall  = 0;
-    floor = 0;
-
-    // By default, x and y axes are not swapped.
-    vertical = false;
-
-    threeD      = false;
-    threeDScene = 0;
-
-    // Data specific for bar charts
-    vertical       = false;
-    gapBetweenBars = 0;
-    gapBetweenSets = 100;
-
-    // OpenOffice.org's default. It means the first pie slice starts at the
-    // very top (and then going counter-clockwise).
-    pieAngleOffset = 90.0;
-
-    // KD Chart stuff
+    shape = 0;
     kdChart = new KDChart::Chart();
     kdPlane = new KDChart::CartesianCoordinatePlane( kdChart );
-
-    // Cache
+    wall = 0;
+    floor = 0;
+    chartType = BarChartType;
+    chartSubtype = NormalChartSubtype;
+    threeD = false;
+    threeDScene = 0;
+    gapBetweenBars = 0;
+    gapBetweenSets = 100;
     pixmapRepaintRequested = true;
-    paintPixmap            = true;
+    paintPixmap = true;
 }
 
 PlotArea::Private::~Private()
 {
-    delete kdPlane;
-    delete kdChart;
 }
 
-void PlotArea::Private::initAxes()
-{
-    // The category data region is anchored to an axis and will be set on addAxis if the
-    // axis defines the Axis::categoryDataRegionString(). So, clear it now.
-    q->proxyModel()->setCategoryDataRegion(QString());
-    // Remove all old axes
-    while( !axes.isEmpty() ) {
-        Axis *axis = axes.takeLast();
-        Q_ASSERT( axis );
-        if ( axis->title() )
-            automaticallyHiddenAxisTitles.removeAll( axis->title() );
-        delete axis;
-    }
-    // There need to be at least these two axes. Do not delete, but
-    // hide them instead.
-    Axis *xAxis = new Axis( q );
-    Axis *yAxis = new Axis( q );
-    xAxis->setPosition( BottomAxisPosition );
-    yAxis->setPosition( LeftAxisPosition );
-    yAxis->setShowMajorGrid( true );
-    q->addAxis( xAxis );
-    q->addAxis( yAxis );
-}
 
 PlotArea::PlotArea( ChartShape *parent )
-    : QObject()
-    , KoShape()
-    , d( new Private( this, parent ) )
+    : KoShape()
+    , d( new Private )
 {
+    Q_ASSERT( parent );
+    
     setShapeId( ChartShapeId );
+    d->shape = parent;
     
-    Q_ASSERT( d->shape );
     Q_ASSERT( d->shape->proxyModel() );
-
+    
     connect( d->shape->proxyModel(), SIGNAL( modelReset() ),
-             this,                   SLOT( dataSetCountChanged() ) );
+             this, SLOT( dataSetCountChanged() ) );
     connect( d->shape->proxyModel(), SIGNAL( rowsInserted( const QModelIndex, int, int ) ),
-             this,                   SLOT( dataSetCountChanged() ) );
+             this, SLOT( dataSetCountChanged() ) );
     connect( d->shape->proxyModel(), SIGNAL( rowsRemoved( const QModelIndex, int, int ) ),
-             this,                   SLOT( dataSetCountChanged() ) );
-    connect( d->shape->proxyModel(), SIGNAL( columnsInserted( const QModelIndex, int, int ) ),
-             this,                   SLOT( plotAreaUpdate() ) );
-    connect( d->shape->proxyModel(), SIGNAL( columnsRemoved( const QModelIndex, int, int ) ),
-             this,                   SLOT( plotAreaUpdate() ) );
+             this, SLOT( dataSetCountChanged() ) );
     connect( d->shape->proxyModel(), SIGNAL( dataChanged() ),
-             this,                   SLOT( plotAreaUpdate() ) );
+             this, SLOT( update() ) );
 }
 
 PlotArea::~PlotArea()
 {
-    delete d;
 }
 
 
-void PlotArea::plotAreaInit()
+void PlotArea::init()
 {
     d->kdChart->resize( size().toSize() );
     d->kdChart->replaceCoordinatePlane( d->kdPlane );
-
     KDChart::FrameAttributes attr = d->kdChart->frameAttributes();
     attr.setVisible( false );
     d->kdChart->setFrameAttributes( attr );
     
     d->wall = new Surface( this );
-    //d->floor = new Surface( this );
+    d->floor = new Surface( this );
     
-    d->initAxes();
+    // There need to be at least these two axes. Do not delete, but hide them instead.
+    Axis *xAxis = new Axis( this );
+    xAxis->setPosition( BottomAxisPosition );
+    Axis *yAxis = new Axis( this );
+    yAxis->setPosition( LeftAxisPosition );
+    yAxis->setShowMajorGrid( true );
+    d->axes.append( xAxis );
+    d->axes.append( yAxis );
 }
 
 void PlotArea::dataSetCountChanged()
 {
     if ( !yAxis() )
         return;
-
-    foreach( DataSet *dataSet, proxyModel()->dataSets() ) {
-        if ( !dataSet->attachedAxis() ) {
+    foreach( DataSet *dataSet, proxyModel()->dataSets() )
+    {
+        if ( !dataSet->attachedAxis() )
+        {
             yAxis()->attachDataSet( dataSet );
         }
     }
 }
 
-ChartProxyModel *PlotArea::proxyModel() const
+ProxyModel *PlotArea::proxyModel() const
 {
     return d->shape->proxyModel();
 }
@@ -291,31 +215,41 @@ int PlotArea::dataSetCount() const
     return proxyModel()->dataSets().size();
 }
 
+Surface *PlotArea::wall() const
+{
+    return d->wall;
+}
+
+Surface *PlotArea::floor() const
+{
+    return d->floor;
+}
+
 Axis *PlotArea::xAxis() const
 {
-    foreach( Axis *axis, d->axes ) {
+    foreach( Axis *axis, d->axes )
+    {
         if ( axis->orientation() == Qt::Horizontal )
             return axis;
     }
-
     return 0;
 }
 
 Axis *PlotArea::yAxis() const
 {
-    foreach( Axis *axis, d->axes ) {
+    foreach( Axis *axis, d->axes )
+    {
         if ( axis->orientation() == Qt::Vertical )
             return axis;
     }
-
     return 0;
 }
 
 Axis *PlotArea::secondaryXAxis() const
 {
     bool firstXAxisFound = false;
-
-    foreach( Axis *axis, d->axes ) {
+    foreach( Axis *axis, d->axes )
+    {
         if ( axis->orientation() == Qt::Horizontal ) {
             if ( firstXAxisFound )
                 return axis;
@@ -323,15 +257,14 @@ Axis *PlotArea::secondaryXAxis() const
                 firstXAxisFound = true;
         }
     }
-
     return 0;
 }
 
 Axis *PlotArea::secondaryYAxis() const
 {
     bool firstYAxisFound = false;
-
-    foreach( Axis *axis, d->axes ) {
+    foreach( Axis *axis, d->axes )
+    {
         if ( axis->orientation() == Qt::Vertical ) {
             if ( firstYAxisFound )
                 return axis;
@@ -339,7 +272,6 @@ Axis *PlotArea::secondaryYAxis() const
                 firstYAxisFound = true;
         }
     }
-
     return 0;
 }
 
@@ -358,16 +290,6 @@ bool PlotArea::isThreeD() const
     return d->threeD;
 }
 
-CellRegion PlotArea::cellRangeAddress() const
-{
-    return d->cellRangeAddress;
-}
-
-bool PlotArea::isVertical() const
-{
-    return d->vertical;
-}
-
 ThreeDScene *PlotArea::threeDScene() const
 {
     return d->threeDScene;
@@ -383,35 +305,24 @@ int PlotArea::gapBetweenSets() const
     return d->gapBetweenSets;
 }
 
-qreal PlotArea::pieAngleOffset() const
-{
-    return d->pieAngleOffset;
-}
-
 bool PlotArea::addAxis( Axis *axis )
 {
-    if ( d->axes.contains( axis ) ) {
-    	qWarning() << "PlotArea::addAxis(): Trying to add already added axis.";
+    if ( d->axes.contains( axis ) )
+    {
+    	qWarning() << "PlotArea::addAxis(): This axis has already been added.";
     	return false;
     }
-
-    if ( !axis ) {
+    if ( !axis )
+    {
     	qWarning() << "PlotArea::addAxis(): Pointer to axis is NULL!";
     	return false;
     }
     d->axes.append( axis );
     
-    if ( axis->dimension() == XAxisDimension ) {
-        // set the categoryDataRegion of the proxyModel. This will then be used on
-        // ChartProxyModel::createDataSetsFromRegion to create the dataSets.
-        if ( proxyModel()->categoryDataRegion().isEmpty() && ! axis->categoryDataRegionString().isEmpty() )
-            proxyModel()->setCategoryDataRegion(axis->categoryDataRegionString());
-
-        // let each axis know about the other axis
-        foreach ( Axis *_axis, d->axes ) {
-            if ( _axis->isVisible() )
-                _axis->registerKdAxis( axis->kdAxis() );
-        }
+    if ( axis->dimension() == XAxisDimension )
+    {
+        foreach ( Axis *_axis, d->axes )
+            _axis->registerKdXAxis( axis->kdAxis() );
     }
     
     requestRepaint();
@@ -421,40 +332,24 @@ bool PlotArea::addAxis( Axis *axis )
 
 bool PlotArea::removeAxis( Axis *axis )
 {
-    if ( !d->axes.contains( axis ) ) {
-    	qWarning() << "PlotArea::removeAxis(): Trying to remove non-added axis.";
+    if ( !d->axes.contains( axis ) )
+    {
+    	qWarning() << "PlotArea::removeAxis(): This axis has not been added previously.";
     	return false;
     }
-
-    if ( !axis ) {
+    if ( !axis )
+    {
     	qWarning() << "PlotArea::removeAxis(): Pointer to axis is NULL!";
     	return false;
     }
-    
-    if ( axis->title() )
-        d->automaticallyHiddenAxisTitles.removeAll( axis->title() );
-
     d->axes.removeAll( axis );
     
-    if ( axis->dimension() == XAxisDimension ) {
-        // If the axis is removed we probably need to update the used categoryDataRegion too.
-        if ( ! proxyModel()->categoryDataRegion().isEmpty() && proxyModel()->categoryDataRegion() == axis->categoryDataRegionString() ) {
-            proxyModel()->setCategoryDataRegion(QString());
-            foreach ( Axis *_axis, d->axes ) {
-                 if ( _axis->dimension() == XAxisDimension && ! _axis->categoryDataRegionString().isEmpty()) {
-                     proxyModel()->setCategoryDataRegion( _axis->categoryDataRegionString() );
-                     break;
-                 }
-            }
-        }
-        
+    if ( axis->dimension() == XAxisDimension )
+    {
         foreach ( Axis *_axis, d->axes )
-            _axis->deregisterKdAxis( axis->kdAxis() );
+            _axis->deregisterKdXAxis( axis->kdAxis() );
     }
     
-    // This also removes the axis' title, which is a shape as well
-    delete axis;
-    
     requestRepaint();
     
     return true;
@@ -464,51 +359,38 @@ bool PlotArea::removeAxis( Axis *axis )
 
 void PlotArea::setChartType( ChartType type )
 {
-    // Lots of things to do if the old and new types of coordinate
-    // systems don't match.
-    if ( !isPolar( d->chartType ) && isPolar( type ) ) {
-        foreach ( Axis *axis, d->axes ) {
+    if ( !isPolar( d->chartType ) && isPolar( type ) )
+    {
+        foreach ( Axis *axis, d->axes )
+        {
             if ( !axis->title()->isVisible() )
                 continue;
-
             axis->title()->setVisible( false );
             d->automaticallyHiddenAxisTitles.append( axis->title() );
         }
     }
-    else if ( isPolar( d->chartType ) && !isPolar( type ) ) {
-        foreach ( KoShape *title, d->automaticallyHiddenAxisTitles ) {
+    else if ( isPolar( d->chartType ) && !isPolar( type ) )
+    {
+        foreach ( KoShape *title, d->automaticallyHiddenAxisTitles )
+        {
             title->setVisible( true );
         }
         d->automaticallyHiddenAxisTitles.clear();
     }
-
-    // Set the dimensionality of the data points.
-    int dimensions = 1;
-    switch ( type ) {
-        case BarChartType:
-        case LineChartType:
-        case AreaChartType:
-        case CircleChartType:
-        case RingChartType:
-        case RadarChartType:
-        case StockChartType:
-        case GanttChartType:
-            dimensions = 1;
-            break;
-        case ScatterChartType:
-        case SurfaceChartType:
-            dimensions = 2;
-            break;
-        case BubbleChartType:
-            dimensions = 3;
-            break;
+    
+    if ( d->chartType != ScatterChartType && type == ScatterChartType )
+    {
+        d->shape->proxyModel()->setDataDimensions( 2 );
+    }
+    else if ( d->chartType == ScatterChartType && type != ScatterChartType )
+    {
+        d->shape->proxyModel()->setDataDimensions( 1 );
     }
-
-    d->shape->proxyModel()->setDataDimensions( dimensions );
     
     d->chartType = type;
     
-    foreach ( Axis *axis, d->axes ) {
+    foreach ( Axis *axis, d->axes )
+    {
         axis->plotAreaChartTypeChanged( type );
     }
     
@@ -519,7 +401,8 @@ void PlotArea::setChartSubType( ChartSubtype subType )
 {
     d->chartSubtype = subType;
     
-    foreach ( Axis *axis, d->axes ) {
+    foreach ( Axis *axis, d->axes )
+    {
         axis->plotAreaChartSubTypeChanged( subType );
     }
     
@@ -536,157 +419,37 @@ void PlotArea::setThreeD( bool threeD )
     requestRepaint();
 }
 
-void PlotArea::setCellRangeAddress( const CellRegion &region )
-{
-    d->cellRangeAddress = region;
-}
-
-void PlotArea::setVertical( bool vertical )
-{
-    d->vertical = vertical;
-    // TODO: Propagate
-}
-
-// ----------------------------------------------------------------
-//                         loading and saving
-
-
-bool PlotArea::loadOdf( const KoXmlElement &plotAreaElement,
-                        KoShapeLoadingContext &context )
+bool PlotArea::loadOdf( const KoXmlElement &plotAreaElement, KoShapeLoadingContext &context )
 {
     KoStyleStack &styleStack = context.odfLoadingContext().styleStack();
     styleStack.save();
 
     styleStack.clear();
-
-    // First step is to load the axis. Datasets are attached to an
-    // axis and we need the axis to check for categories.
-
-    //remove the default axes first
-    while( !d->axes.isEmpty() ) {
-        Axis *axis = d->axes.takeLast();
-        Q_ASSERT( axis );
-        if ( axis->title() )
-            d->automaticallyHiddenAxisTitles.removeAll( axis->title() );
-        delete axis;
-    }
-
-    KoXmlElement n;
-    forEachElement ( n, plotAreaElement ) {
-        if ( n.namespaceURI() != KoXmlNS::chart )
-            continue;
-
-        if ( n.localName() == "axis" ) {
-            Axis *axis = new Axis( this );
-            axis->loadOdf( n, context );
-            addAxis( axis );
-        }
-    }
-
-    //2 axes are mandatory, check that we have them
-    if( !xAxis() ) {
-        Axis *xAxis = new Axis( this );
-        xAxis->setPosition( BottomAxisPosition );
-        xAxis->setVisible( false );
-        addAxis( xAxis );
-    }
-    if( !yAxis() ) {
-        Axis *yAxis = new Axis( this );
-        yAxis->setPosition( LeftAxisPosition );
-        yAxis->setVisible( false );
-        addAxis( yAxis );
-    }
-
-    CellRegion cellRangeAddress;
-    if ( plotAreaElement.hasAttributeNS( KoXmlNS::table, "cell-range-address" ) )
+    if( plotAreaElement.hasAttributeNS( KoXmlNS::chart, "style-name" ) )
     {
-        cellRangeAddress = CellRegion( plotAreaElement.attributeNS( KoXmlNS::table, "cell-range-address" ) );
-    }
-
-    // Find out about things that are in the plotarea style.
-    // 
-    // These things include chart subtype, special things for some
-    // chart types like line charts, stock charts, etc.
-    QString seriesSource;
-    if ( plotAreaElement.hasAttributeNS( KoXmlNS::chart, "style-name" ) ) {
         context.odfLoadingContext().fillStyleStack( plotAreaElement, KoXmlNS::chart, "style-name", "chart" );
 
         styleStack.setTypeProperties( "graphic" );
+
         styleStack.setTypeProperties( "chart" );
 
-        if ( styleStack.hasProperty( KoXmlNS::chart, "angle-offset" ) ) {
-            bool ok;
-            const int angleOffset = styleStack.property( KoXmlNS::chart, "angle-offset" ).toInt(&ok);
-            if (ok)
-                setPieAngleOffset( angleOffset );
-        }
-
-        if ( styleStack.hasProperty( KoXmlNS::chart, "three-dimensional" ) )
-            setThreeD( styleStack.property( KoXmlNS::chart, "three-dimensional" ) == "true" );
-
-        // Set subtypes stacked or percent.
-        // These are valid for Bar, Line, Area and Radar types.
-        if ( styleStack.hasProperty( KoXmlNS::chart, "percentage" )
-             && styleStack.property( KoXmlNS::chart, "percentage" ) == "true" )
-        {
+        if ( styleStack.hasProperty( KoXmlNS::chart, "percentage" ) && styleStack.property( KoXmlNS::chart, "percentage" ) == "true" )
             setChartSubType( PercentChartSubtype );
-        }
-        else if ( styleStack.hasProperty( KoXmlNS::chart, "stacked" )
-                  && styleStack.property( KoXmlNS::chart, "stacked" ) == "true" )
-        {
+        else if ( styleStack.hasProperty( KoXmlNS::chart, "stacked" ) && styleStack.property( KoXmlNS::chart, "stacked" ) == "true" )
             setChartSubType( StackedChartSubtype );
-        }
-
-        // Data specific to bar charts
-        if ( styleStack.hasProperty( KoXmlNS::chart, "vertical" ) )
-            setVertical( styleStack.property( KoXmlNS::chart, "vertical" ) == "true" );
-
-        // Data direction: It's in the plotarea style.
-        if ( styleStack.hasProperty( KoXmlNS::chart, "series-source" ) ) {
-            seriesSource = styleStack.property( KoXmlNS::chart, "series-source" );
-            kDebug(35001) << "series-source=" << seriesSource;
-        }
-
-        // Special properties for various chart types
-#if 0
-        switch () {
-        case BarChartType:
-            if ( styleStack )
-                ;
-        }
-#endif
     }
-
-    if ( seriesSource == "rows" )
-        proxyModel()->setDataDirection( Qt::Horizontal );
-    else if ( seriesSource == "columns" )
-        proxyModel()->setDataDirection( Qt::Vertical );
-    else
-        // By default, OOo creates consecutive data series column-wise;
-        // by adopting that behaviour we avoid extra special-handling of
-        // documents created by OOo.
-        proxyModel()->setDataDirection( Qt::Vertical );
-
-    // The exact position defined in ODF overwrites the default layout position
-    if ( plotAreaElement.hasAttributeNS( KoXmlNS::svg, "x" ) ||
-         plotAreaElement.hasAttributeNS( KoXmlNS::svg, "y" ) ||
-         plotAreaElement.hasAttributeNS( KoXmlNS::svg, "width" ) ||
-         plotAreaElement.hasAttributeNS( KoXmlNS::svg, "height" ) )
-        parent()->layout()->setPosition( this, FloatingPosition );
-
     loadOdfAttributes( plotAreaElement, context, OdfAllAttributes );
     
-    // Find out if the data table contains labels as first row and/or column.
-    // This is in the plot-area element itself.
+    KoOdfStylesReader &stylesReader = context.odfLoadingContext().stylesReader();
+    
+    KoXmlElement dataHasLabelsElem = KoXml::namedItemNS( plotAreaElement, 
+                                                         KoXmlNS::chart, "data-source-has-labels" );
+    if ( plotAreaElement.hasAttributeNS( KoXmlNS::chart,
+                                         "data-source-has-labels" ) ) {
 
-    // Do not ignore the data-source-has-labels in any case, even if a category data region is specified for an axis, as the first column still
-    // has to be exluded from the actual data region if e.g. data-source-has-labels is set to "column"
-    // If an axis contains the chart:categories element, the category data region will automatically be set on every data set attached to that
-    // axis. See Axis::attachDataSet().
-    if ( plotAreaElement.hasAttributeNS( KoXmlNS::chart, "data-source-has-labels" ) ) {
         const QString  dataSourceHasLabels
             = plotAreaElement.attributeNS( KoXmlNS::chart,
-                                        "data-source-has-labels" );
+                                           "data-source-has-labels" );
         if ( dataSourceHasLabels == "both" ) {
             proxyModel()->setFirstRowIsLabel( true );
             proxyModel()->setFirstColumnIsLabel( true );
@@ -708,57 +471,42 @@ bool PlotArea::loadOdf( const KoXmlElement &plotAreaElement,
         proxyModel()->setFirstColumnIsLabel( false );
     }
 
-    QAbstractItemModel *sheetAccessModel = 0;
-    if (d->shape->resourceManager()->hasResource(75751149)) { // duplicated from kspread
-        QVariant var = d->shape->resourceManager()->resource(75751149);
-        sheetAccessModel = static_cast<QAbstractItemModel*>(var.value<void*>());
-    }
-
-    setCellRangeAddress( cellRangeAddress );
-
-    if ( sheetAccessModel )
+    // Remove all axes before loading new ones
+    KoXmlElement e;
+    while( !d->axes.isEmpty() )
     {
-        const QString sheetName = cellRangeAddress.sheetName();
-        int sheetIndex = 0;
-        // Find sheet that this cell range address is associated with
-        if ( !sheetName.isEmpty() ) {
-            while ( sheetIndex + 1 < sheetAccessModel->columnCount() &&
-                    sheetAccessModel->headerData( sheetIndex, Qt::Horizontal ) != sheetName )
-                sheetIndex++;
-        }
-        QPointer<QAbstractItemModel> sheet = sheetAccessModel->data( sheetAccessModel->index( 0, sheetIndex ) ).value< QPointer<QAbstractItemModel> >();
-
-        // If sheet can't be found, we'll stay with the back-up model loaded from the
-        // chart document.
-        if ( sheet )
-            d->shape->setModel( sheet.data() );
+        Axis *axis = d->axes.takeLast();
+        Q_ASSERT( axis );
+        delete axis;
     }
     
-    // Now, after the axes, load the datasets.
-    // Note that this only contains properties of the datasets, the
-    // actual data is not stored here.
-    //
-    // FIXME: Isn't the proxy model a strange place to store this data?
-    proxyModel()->loadOdf( plotAreaElement, context );
-
-    // Now load the surfaces (wall and possibly floor)
-    // FIXME: Use named tags instead of looping?
-    forEachElement ( n, plotAreaElement ) {
+    // A data set is always attached to an axis, so load them first
+    KoXmlElement n;
+    forEachElement ( n, plotAreaElement )
+    {
         if ( n.namespaceURI() != KoXmlNS::chart )
             continue;
+        if ( n.localName() == "axis" ) {
+            Axis *axis = new Axis( this );
+            axis->loadOdf( n, context );
+            addAxis( axis );
+        }
+    }
+    // Load data sets
+    d->shape->proxyModel()->loadOdf( plotAreaElement, context );
 
-        if ( n.localName() == "wall" ) {
+    forEachElement ( n, plotAreaElement )
+    {
+        if ( n.namespaceURI() != KoXmlNS::chart )
+            continue;
+        if ( n.localName() == "wall" )
+        {
             d->wall->loadOdf( n, context );
         }
-        else if ( n.localName() == "floor" ) {
-            // The floor is not always present, so allocate it if needed.
-            // FIXME: Load floor, even if we don't really support it yet
-            // and save it back to ODF.
-            //if ( !d->floor )
-            //    d->floor = new Surface( this );
-            //d->floor->loadOdf( n, context );
-        }
-        else if ( n.localName() != "axis" && n.localName() != "series" ) {
+        //else if ( n.localName() == "floor" )
+        //    d->floor->loadOdf( n, context );
+        else if ( n.localName() != "axis" && n.localName() != "series" )
+        {
             qWarning() << "PlotArea::loadOdf(): Unknown tag name " << n.localName();
         }
     }
@@ -773,12 +521,12 @@ bool PlotArea::loadOdf( const KoXmlElement &plotAreaElement,
 void PlotArea::saveOdf( KoShapeSavingContext &context ) const
 {
     KoXmlWriter &bodyWriter = context.xmlWriter();
-    //KoGenStyles &mainStyles = context.mainStyles();
+    KoGenStyles &mainStyles = context.mainStyles();
+    
     bodyWriter.startElement( "chart:plot-area" );
     
-    // FIXME: Somehow this style gets the name gr2 instead of ch2.
-    //        Fix that as well.
-    KoGenStyle plotAreaStyle( KoGenStyle::ChartAutoStyle, "chart" );
+    KoGenStyle plotAreaStyle;
+    plotAreaStyle = KoGenStyle( KoGenStyle::StyleGraphicAuto, "chart" );
     
     // Data direction
     const Qt::Orientation direction = proxyModel()->dataDirection();
@@ -788,18 +536,15 @@ void PlotArea::saveOdf( KoShapeSavingContext &context ) const
     // Save chart subtype
     saveOdfSubType( bodyWriter, plotAreaStyle );
     
-    bodyWriter.addAttribute( "chart:style-name",
-                             saveStyle( plotAreaStyle, context ) );
+    bodyWriter.addAttribute( "chart:style-name", saveStyle( plotAreaStyle, context ) );
     
     const QSizeF s( size() );
     const QPointF p( position() );
-    bodyWriter.addAttributePt( "svg:width",  s.width() );
+    bodyWriter.addAttributePt( "svg:width", s.width() );
     bodyWriter.addAttributePt( "svg:height", s.height() );
     bodyWriter.addAttributePt( "svg:x", p.x() );
     bodyWriter.addAttributePt( "svg:y", p.y() );
 
-    bodyWriter.addAttribute( "table:cell-range-address", d->cellRangeAddress.toString() );
-
     // About the data:
     //   Save if the first row / column contain headers.
     QString  dataSourceHasLabels;
@@ -815,9 +560,8 @@ void PlotArea::saveOdf( KoShapeSavingContext &context ) const
             dataSourceHasLabels = "none";
     // Note: this is saved in the plotarea attributes and not the style.
     bodyWriter.addAttribute( "chart:data-source-has-labels",
-                             dataSourceHasLabels );
+                            dataSourceHasLabels );
 
-    // Save the axes.
     if ( isCartesian( d->chartType ) ) {
         foreach( Axis *axis, d->axes ) {
             axis->saveOdf( context );
@@ -827,19 +571,17 @@ void PlotArea::saveOdf( KoShapeSavingContext &context ) const
     // Save data series
     d->shape->proxyModel()->saveOdf( context );
 
-    // Save the floor and wall of the plotarea.
-    d->wall->saveOdf( context, "chart:wall" );
-    //if ( d->floor )
-    //    d->floor->saveOdf( context, "chart:floor" );
+    d->wall->saveOdf( context );
 
+    // TODO chart:series
+    // TODO chart:grid
+    // TODO chart:floor
+    
     bodyWriter.endElement(); // chart:plot-area
 }
 
-void PlotArea::saveOdfSubType( KoXmlWriter& xmlWriter,
-                               KoGenStyle& plotAreaStyle ) const
+void PlotArea::saveOdfSubType( KoXmlWriter& xmlWriter, KoGenStyle& plotAreaStyle ) const
 {
-    Q_UNUSED( xmlWriter );
-
     switch ( d->chartType ) {
     case BarChartType:
         switch( d->chartSubtype ) {
@@ -853,15 +595,12 @@ void PlotArea::saveOdfSubType( KoXmlWriter& xmlWriter,
             plotAreaStyle.addProperty( "chart:percentage", "true" );
             break;
         }
-        if ( d->threeD ) {
-            plotAreaStyle.addProperty( "chart:three-dimensional", "true" );
-            // FIXME: Save all 3D attributes too.
-        }
-        // Data specific to bar charts
-        if ( d->vertical )
-            plotAreaStyle.addProperty( "chart:vertical", "true" );
-        // Don't save this if zero, because that's the default.
-        //plotAreaStyle.addProperty( "chart:lines-used", 0 ); // FIXME: for now
+    if ( d->threeD ) {
+        plotAreaStyle.addProperty( "chart:three-dimensional", "true" );
+        // FIXME: Save all 3D attributes too.
+    }
+        plotAreaStyle.addProperty( "chart:vertical", "false" ); // FIXME
+        plotAreaStyle.addProperty( "chart:lines-used", 0 ); // FIXME: for now
         break;
 
     case LineChartType:
@@ -876,10 +615,10 @@ void PlotArea::saveOdfSubType( KoXmlWriter& xmlWriter,
             plotAreaStyle.addProperty( "chart:percentage", "true" );
             break;
         }
-        if ( d->threeD ) {
-            plotAreaStyle.addProperty( "chart:three-dimensional", "true" );
-            // FIXME: Save all 3D attributes too.
-        }
+    if ( d->threeD ) {
+        plotAreaStyle.addProperty( "chart:three-dimensional", "true" );
+        // FIXME: Save all 3D attributes too.
+    }
         // FIXME: What does this mean?
         plotAreaStyle.addProperty( "chart:symbol-type", "automatic" );
         break;
@@ -888,47 +627,28 @@ void PlotArea::saveOdfSubType( KoXmlWriter& xmlWriter,
         switch( d->chartSubtype ) {
         case NoChartSubtype:
         case NormalChartSubtype:
-            break;
-        case StackedChartSubtype:
+        break;
+    case StackedChartSubtype:
             plotAreaStyle.addProperty( "chart:stacked", "true" );
             break;
         case PercentChartSubtype:
             plotAreaStyle.addProperty( "chart:percentage", "true" );
             break;
         }
-
-        if ( d->threeD ) {
-            plotAreaStyle.addProperty( "chart:three-dimensional", "true" );
-            // FIXME: Save all 3D attributes too.
-        }
-        break;
+        //plotAreaStyle.addProperty( "chart:lines-used", 0 ); // #### for now
+    if ( d->threeD ) {
+        plotAreaStyle.addProperty( "chart:three-dimensional", "true" );
+        // FIXME: Save all 3D attributes too.
+    }
 
     case CircleChartType:
         // FIXME
         break;
+    break;
 
     case RingChartType:
-        // FIXME
-        break;
-
     case ScatterChartType:
-        // FIXME
-        break;
     case RadarChartType:
-        // Save subtype of the Radar chart.
-        switch( d->chartSubtype ) {
-        case NoChartSubtype:
-        case NormalChartSubtype:
-            break;
-        case StackedChartSubtype:
-            plotAreaStyle.addProperty( "chart:stacked", "true" );
-            break;
-        case PercentChartSubtype:
-            plotAreaStyle.addProperty( "chart:percentage", "true" );
-            break;
-        }
-        break;
-
     case StockChartType:
     case BubbleChartType:
     case SurfaceChartType:
@@ -947,32 +667,19 @@ void PlotArea::saveOdfSubType( KoXmlWriter& xmlWriter,
 
 void PlotArea::setGapBetweenBars( int percent )
 {
-    d->gapBetweenBars = percent;
-
     emit gapBetweenBarsChanged( percent );
 }
 
 void PlotArea::setGapBetweenSets( int percent )
 {
-    d->gapBetweenSets = percent;
-
     emit gapBetweenSetsChanged( percent );
 }
 
 void PlotArea::setPieExplodeFactor( DataSet *dataSet, int percent )
 {
-    d->pieExplodeFactor = percent;
-
     emit pieExplodeFactorChanged( dataSet, percent );
 }
 
-void PlotArea::setPieAngleOffset( qreal angle )
-{
-    d->pieAngleOffset = angle;
-
-    emit pieAngleOffsetChanged( angle );
-}
-
 ChartShape *PlotArea::parent() const
 {
     // There has to be a valid parent
@@ -1003,18 +710,15 @@ bool PlotArea::deregisterKdDiagram( KDChart::AbstractDiagram *diagram )
 {
     if ( !d->kdDiagrams.contains( diagram ) )
         return false;
-
     d->kdDiagrams.removeAll( diagram );
     return true;
 }
 
-void PlotArea::plotAreaUpdate() const
+void PlotArea::update() const
 {
-    parent()->legend()->update();
     requestRepaint();
     foreach( Axis* axis, d->axes )
         axis->update();
-
     KoShape::update();
 }
 
@@ -1066,7 +770,8 @@ void PlotArea::paint( QPainter& painter, const KoViewConverter& converter )
     painter.setClipRect( paintRect );
     
     // Paint the background
-    if ( background() ) {
+    if( background() )
+    {
         QPainterPath p;
         p.addRect( paintRect );
         background()->paint( painter, p );
@@ -1076,35 +781,25 @@ void PlotArea::paint( QPainter& painter, const KoViewConverter& converter )
     QPointF zoomLevel;
     converter.zoom( &zoomLevel.rx(), &zoomLevel.ry() );
 
-    // Only repaint the pixmap if it is scheduled, the zoom level
-    // changed or the shape was resized.
+    // Only repaint the pixmap if it is scheduled, the zoom level changed or the shape was resized
     /*if (    d->pixmapRepaintRequested
          || d->lastZoomLevel != zoomLevel
          || d->lastSize      != size()
          || !d->paintPixmap ) {
-        // TODO (js): What if two zoom levels are constantly being
-        //            requested?  At the moment, this *is* the case,
-        //            due to the fact that the shape is also rendered
-        //            in the page overview in KPresenter Every time
-        //            the window is hidden and shown again, a repaint
-        //            is requested --> laggy performance, especially
-        //            when quickly switching through windows.
-        //
-        // ANSWER (iw): what about having a small mapping between size
-        //              in pixels and pixmaps?  The size could be 2 or
-        //              at most 3.  We could manage the replacing
-        //              using LRU.
+        // TODO: What if two zoom levels are constantly being requested?
+        // At the moment, this *is* the case, due to the fact
+        // that the shape is also rendered in the page overview
+        // in KPresenter
+        // Every time the window is hidden and shown again, a repaint is
+        // requested --> laggy performance, especially when quickly
+        // switching through windows
         paintPixmap( painter, converter );
         d->pixmapRepaintRequested = false;
         d->lastZoomLevel = zoomLevel;
         d->lastSize      = size();
     }*/
     painter.setRenderHint( QPainter::Antialiasing, false );
-
-    // KDChart thinks in pixels, KOffice in pt
-    ScreenConversions::scaleFromPtToPx( painter );
-
-    d->kdChart->paint( &painter, ScreenConversions::scaleFromPtToPx( paintRect ) );
+    d->kdChart->paint( &painter, paintRect.toRect() );
     //painter.restore();
 
     // Paint the cached pixmap if we got a GO from paintPixmap()
