@@ -38,8 +38,6 @@
 #include <QApplication>
 #include <qpainter.h>
 #include <qfileinfo.h>
-#include <QTimer>
-#include <QUndoCommand>
 
 #include <kdebug.h>
 #include <kconfig.h>
@@ -59,14 +57,24 @@ namespace KPlato
 Part::Part( QWidget *parentWidget, QObject *parent, bool singleViewMode )
         : KoDocument( parentWidget, parent, singleViewMode ),
         m_project( 0 ), m_parentWidget( parentWidget ),
-        m_context( 0 ), m_xmlLoader()
+        m_context( 0 ), m_xmlLoader(),
+        m_loadingTemplate( false )
 {
     setComponentData( Factory::global() );
     setTemplateType( "kplato_template" );
+    // Add library translation files
+    KLocale *locale = KGlobal::locale();
+    if ( locale ) {
+        locale->insertCatalog( "kplatokernel" );
+        locale->insertCatalog( "kplatomodels" );
+        locale->insertCatalog( "kplatoui" );
+        locale->insertCatalog( "kdgantt" );
+    }
     m_config.setReadWrite( isReadWrite() || !isEmbedded() );
     m_config.load();
 
     setProject( new Project() ); // after config is loaded
+    m_project->setId( m_project->uniqueNodeId() );
 }
 
 
@@ -76,6 +84,11 @@ Part::~Part()
     delete m_project;
 }
 
+void Part::configChanged()
+{
+    m_project->setTaskDefaults( m_config.taskDefaults() );
+}
+
 void Part::setProject( Project *project )
 {
     if ( m_project ) {
@@ -85,6 +98,7 @@ void Part::setProject( Project *project )
     m_project = project;
     if ( m_project ) {
         connect( m_project, SIGNAL( changed() ), this, SIGNAL( changed() ) );
+        m_project->setTaskDefaults( m_config.taskDefaults() );
     }
     emit changed();
 }
@@ -100,10 +114,10 @@ bool Part::loadOdf( KoOdfReadStore & odfStore )
 {
     Q_UNUSED( odfStore );
     kWarning()<< "OpenDocument not supported, let's try native xml format";
-    return loadXML( 0, odfStore.contentDoc() ); // We have only one format, so try to load that!
+    return loadXML( odfStore.contentDoc(), 0 ); // We have only one format, so try to load that!
 }
 
-bool Part::loadXML( QIODevice *, const KoXmlDocument &document )
+bool Part::loadXML( const KoXmlDocument &document, KoStore* )
 {
     kDebug();
     QTime dt;
@@ -160,6 +174,9 @@ bool Part::loadXML( QIODevice *, const KoXmlDocument &document )
             Project * newProject = new Project();
             m_xmlLoader.setProject( newProject );
             if ( newProject->load( e, m_xmlLoader ) ) {
+                if ( newProject->id().isEmpty() ) {
+                    newProject->setId( newProject->uniqueNodeId() );
+                }
                 // The load went fine. Throw out the old project
                 setProject( newProject );
             } else {
@@ -272,7 +289,7 @@ bool Part::saveWorkPackageFormat( const QString &file, const Node *node, long id
         store->setPassword( d->m_password );
     }*/
     if ( store->bad() ) {
-        setErrorMessage( i18n( "Could not create the workpackage file for saving" ) ); // more details needed?
+        setErrorMessage( i18n( "Could not create the workpackage file for saving: %1", file ) ); // more details needed?
         delete store;
         return false;
     }
@@ -286,13 +303,13 @@ bool Part::saveWorkPackageFormat( const QString &file, const Node *node, long id
     }
     KoStoreDevice dev( store );
     if ( !saveWorkPackageToStream( &dev, node, id ) || !store->close() ) {
-        kDebug(30003) <<"saveToStream failed";
+        kDebug() <<"saveToStream failed";
         delete store;
         return false;
     }
     node->documents().saveToStore( store );
     
-    kDebug(30003) <<"Saving done of url:" << file;
+    kDebug() <<"Saving done of url:" << file;
     if ( !store->finalize() ) {
         delete store;
         return false;
@@ -305,27 +322,135 @@ bool Part::saveWorkPackageFormat( const QString &file, const Node *node, long id
 
 bool Part::saveWorkPackageUrl( const KUrl & _url, const Node *node, long id  )
 {
-    kDebug()<<_url;
-
+    //kDebug()<<_url;
     QApplication::setOverrideCursor( Qt::WaitCursor );
-
     emit statusBarMessage( i18n("Saving...") );
     bool ret = false;
-    bool suppressErrorDialog = false;
-
-    ret = saveWorkPackageFormat( _url.url(), node, id );
-
-
+    ret = saveWorkPackageFormat( _url.path(), node, id ); // kzip don't handle file://
     QApplication::restoreOverrideCursor();
-    if ( ! ret ) {
-        if ( !suppressErrorDialog ) {
-            showSavingErrorDialog();
-        }
-    }
     emit clearStatusBarMessage();
     return ret;
 }
 
+bool Part::loadWorkPackage( Project &project, const KUrl &url )
+{
+    kDebug()<<url;
+    if ( ! url.isLocalFile() ) {
+        kDebug()<<"TODO: download if url not local";
+        return false;
+    }
+    KoStore * store = KoStore::createStore( url.path(), KoStore::Read, "", KoStore::Auto );
+    if ( store->bad() ) {
+//        d->lastErrorMessage = i18n( "Not a valid KOffice file: %1", file );
+        kDebug()<<"bad store"<<url.prettyUrl();
+        delete store;
+//        QApplication::restoreOverrideCursor();
+        return false;
+    }
+    if ( ! store->open( "root" ) ) { // "old" file format (maindoc.xml)
+        // i18n( "File does not have a maindoc.xml: %1", file );
+        kDebug()<<"No root"<<url.prettyUrl();
+        delete store;
+//        QApplication::restoreOverrideCursor();
+        return false;
+    }
+    KoXmlDocument doc;
+    QString errorMsg; // Error variables for QDomDocument::setContent
+    int errorLine, errorColumn;
+    bool ok = doc.setContent( store->device(), &errorMsg, &errorLine, &errorColumn );
+    if ( ! ok ) {
+        kError(30003) << "Parsing error in " << url.url() << "! Aborting!" << endl
+                << " In line: " << errorLine << ", column: " << errorColumn << endl
+                << " Error message: " << errorMsg << endl;
+        //d->lastErrorMessage = i18n( "Parsing error in %1 at line %2, column %3\nError message: %4",filename  ,errorLine, errorColumn , QCoreApplication::translate("QXml", errorMsg.toUtf8(), 0, QCoreApplication::UnicodeUTF8));
+    } else {
+        ok = loadWorkPackageXML( project, store->device(), doc );
+    }
+    store->close();
+    delete store;
+    if ( ! ok ) {
+//        QApplication::restoreOverrideCursor();
+        return false;
+    }
+    return true;
+}
+
+bool Part::loadWorkPackageXML( Project &project, QIODevice *, const KoXmlDocument &document )
+{
+    kDebug();
+    QTime dt;
+    dt.start();
+    emit sigProgress( 0 );
+
+    QString value;
+    KoXmlElement plan = document.documentElement();
+
+    // Check if this is the right app
+    value = plan.attribute( "mime", QString() );
+    if ( value.isEmpty() ) {
+        kError() << "No mime type specified!" << endl;
+        setErrorMessage( i18n( "Invalid document. No mimetype specified." ) );
+        return false;
+    } else if ( value != "application/x-vnd.kde.kplato.work" ) {
+        kError() << "Unknown mime type " << value << endl;
+        setErrorMessage( i18n( "Invalid document. Expected mimetype application/x-vnd.kde.kplato.work, got %1", value ) );
+        return false;
+    }
+    QString m_syntaxVersion = plan.attribute( "version", CURRENT_SYNTAX_VERSION );
+    m_xmlLoader.setVersion( m_syntaxVersion );
+    if ( m_syntaxVersion > CURRENT_SYNTAX_VERSION ) {
+        int ret = KMessageBox::warningContinueCancel(
+                0, i18n( "This document was created with a newer version of KPlatoWork (syntax version: %1)\n"
+                "Opening it in this version of KPlatoWork will lose some information.", m_syntaxVersion ),
+                i18n( "File-Format Mismatch" ), KGuiItem( i18n( "Continue" ) ) );
+        if ( ret == KMessageBox::Cancel ) {
+            setErrorMessage( "USER_CANCELED" );
+            return false;
+        }
+    }
+    emit sigProgress( 5 );
+
+#ifdef KOXML_USE_QDOM
+    int numNodes = plan.childNodes().count();
+#else
+    int numNodes = plan.childNodesCount();
+#endif
+    if ( numNodes > 2 ) {
+        //TODO: Make a proper bitching about this
+        kDebug() <<"*** Error ***";
+        kDebug() <<"  Children count should be maximum 2, but is" << numNodes;
+        return false;
+    }
+    emit sigProgress( 100 ); // the rest is only processing, not loading
+
+    kDebug() <<"Loading took" << ( float ) ( dt.elapsed() ) / 1000 <<" seconds";
+
+    bool ok = true;
+    m_xmlLoader.startLoad();
+    KoXmlNode n = plan.firstChild();
+    for ( ; ! n.isNull(); n = n.nextSibling() ) {
+        if ( ! n.isElement() ) {
+            continue;
+        }
+        KoXmlElement e = n.toElement();
+        if ( e.tagName() == "project" ) {
+            m_xmlLoader.setProject( &project );
+            ok = project.load( e, m_xmlLoader );
+            if ( ! ok ) {
+                m_xmlLoader.addMsg( XMLLoaderObject::Errors, "Loading of work package failed" );
+                //TODO add some ui here
+            }
+        }
+    }
+    m_xmlLoader.stopLoad();
+    emit sigProgress( 100 ); // the rest is only processing, not loading
+
+    kDebug() <<"Loading took" << ( float ) ( dt.elapsed() ) / 1000 <<" seconds";
+
+    emit sigProgress( -1 );
+    return ok;
+}
+
 
 void Part::paintContent( QPainter &, const QRect &)
 {
@@ -336,11 +461,6 @@ void Part::slotViewDestroyed()
 {
 }
 
-void Part::generateWBS()
-{
-    m_project->generateWBS( 1, m_wbsDefinition );
-}
-
 void Part::activate( QWidget *w )
 {
     if ( manager() )
@@ -385,8 +505,22 @@ bool Part::loadChildren( KoStore* store )
     return true;
 }
 
+void Part::openTemplate( const KUrl& url )
+{
+    //kDebug()<<url;
+    // HACK because we can't really reimplemt openTemplate() (private methods)
+    m_loadingTemplate = true;
+    KoDocument::openTemplate( url );
+    m_loadingTemplate = false;
+}
+
 bool Part::completeLoading( KoStore *store )
 {
+    if ( m_loadingTemplate ) {
+        // If we get here the new project is loaded and set
+        //kDebug()<<"Loading template, generate unique ids";
+        m_project->generateUniqueIds();
+    }
     if ( store == 0 ) {
         // can happen if loading a template
         kDebug()<<"No store"<<endl;
