@@ -21,12 +21,12 @@
 
 #include <QString>
 #include <QStringList>
-#include <QHeaderView>
 #include <QTreeWidget>
 #include <QTreeWidgetItem>
 #include <QItemDelegate>
 #include <QStyle>
-#include <QVariant>
+#include <QHeaderView>
+#include <QBrush>
 
 #include <kmenu.h>
 
@@ -67,7 +67,6 @@ void ViewCategoryDelegate::paint( QPainter * painter, const QStyleOptionViewItem
         // this is a top-level item.
         QStyleOptionButton buttonOption;
         buttonOption.state = option.state;
-        buttonOption.state &= ~QStyle::State_HasFocus;
 
         buttonOption.rect = option.rect;
         buttonOption.palette = option.palette;
@@ -100,20 +99,20 @@ void ViewCategoryDelegate::paint( QPainter * painter, const QStyleOptionViewItem
 }
 
 ViewListItem::ViewListItem( const QString &tag, const QStringList &strings, int type )
-: QTreeWidgetItem( strings, type ),
-m_tag( tag )
+    : QTreeWidgetItem( strings, type ),
+    m_tag( tag )
 {
 }
 
 ViewListItem::ViewListItem( QTreeWidget *parent, const QString &tag, const QStringList &strings, int type )
-: QTreeWidgetItem( parent, strings, type ),
-m_tag( tag )
+    : QTreeWidgetItem( parent, strings, type ),
+    m_tag( tag )
 {
 }
 
 ViewListItem::ViewListItem( QTreeWidgetItem *parent, const QString &tag, const QStringList &strings, int type )
-: QTreeWidgetItem( parent, strings, type ),
-m_tag( tag )
+    : QTreeWidgetItem( parent, strings, type ),
+    m_tag( tag )
 {
 }
 
@@ -170,12 +169,17 @@ void ViewListItem::save( QDomElement &element ) const
     element.setAttribute( "name", text( 0 ) );
     element.setAttribute( "tooltip", toolTip( 0 ) );
     if ( type() == ItemType_SubView ) {
-        element.setAttribute( "viewtype", view()->metaObject()->className() );
+        QString name = view()->metaObject()->className();
+        if ( name.contains( ':' ) ) {
+            name = name.remove( 0, name.lastIndexOf( ':' ) + 1 );
+        }
+        kDebug()<<view()->metaObject()->className()<<" -> "<<name;
+        element.setAttribute( "viewtype", name );
     }
 }
 
 ViewListTreeWidget::ViewListTreeWidget( QWidget *parent )
-: QTreeWidget( parent )
+    : QTreeWidget( parent )
 {
     header() ->hide();
     setRootIsDecorated( false );
@@ -281,15 +285,16 @@ ViewListItem *ViewListTreeWidget::category( const KoView *view ) const
 
 //-----------------------
 ViewListWidget::ViewListWidget( Part *part, QWidget *parent )//QString name, KXmlGuiWindow *parent )
-: QWidget( parent ),
-m_part( part )
+    : QWidget( parent ),
+    m_part( part ),
+    m_prev( 0 )
 {
     setObjectName("ViewListWidget");
     m_viewlist = new ViewListTreeWidget( this );
     QVBoxLayout *l = new QVBoxLayout( this );
     l->setMargin( 0 );
     l->addWidget( m_viewlist );
-    m_viewlist->setEditTriggers( QAbstractItemView::DoubleClicked );
+    m_viewlist->setEditTriggers( QAbstractItemView::EditKeyPressed | QAbstractItemView::DoubleClicked );
 
     connect( m_viewlist, SIGNAL( currentItemChanged( QTreeWidgetItem*, QTreeWidgetItem* ) ), SLOT( slotActivated( QTreeWidgetItem*, QTreeWidgetItem* ) ) );
 
@@ -329,7 +334,14 @@ void ViewListWidget::slotActivated( QTreeWidgetItem *item, QTreeWidgetItem *prev
     if ( item == 0 || item->type() == ViewListItem::ItemType_Category ) {
         return ;
     }
+    QVariant v;
+    if ( m_prev ) {
+        m_prev->setData( 0, Qt::BackgroundRole, v );
+    }
     emit activated( static_cast<ViewListItem*>( item ), static_cast<ViewListItem*>( prev ) );
+    v = QBrush( QColor( Qt::yellow ) );
+    item->setData( 0, Qt::BackgroundRole, v );
+    m_prev = item;
 }
 
 ViewListItem *ViewListWidget::addCategory( const QString &tag, const QString& name )
@@ -377,16 +389,17 @@ QString ViewListWidget::uniqueTag( const QString &seed ) const
     return tag;
 }
 
-ViewListItem *ViewListWidget::addView( QTreeWidgetItem *category, const QString &tag, const QString& name, KoView *view, KoDocument *doc, const QString& icon )
+ViewListItem *ViewListWidget::addView( QTreeWidgetItem *category, const QString &tag, const QString& name, KoView *view, KoDocument *doc, const QString& icon, int index )
 {
-    ViewListItem * item = new ViewListItem( category, uniqueTag( tag ), QStringList( name ), ViewListItem::ItemType_SubView );
+    ViewListItem * item = new ViewListItem( uniqueTag( tag ), QStringList( name ), ViewListItem::ItemType_SubView );
     item->setView( view );
     item->setDocument( doc );
     if ( !icon.isEmpty() ) {
         item->setData( 0, Qt::DecorationRole, KIcon( icon ) );
     }
     item->setFlags( item->flags() | Qt::ItemIsEditable );
-    //kDebug() << "added: " << item->tag() << endl;
+    category->insertChild( ( index == -1 ? category->childCount() : index ), item );
+    //kDebug() << "added: " << item->tag();
     return item;
 }
 
@@ -414,6 +427,23 @@ void ViewListWidget::setSelected( QTreeWidgetItem *item )
     //kDebug()<<item<<","<<m_viewlist->currentItem();
 }
 
+ViewListItem *ViewListWidget::currentItem() const
+{
+    return static_cast<ViewListItem*>( m_viewlist->currentItem() );
+}
+
+ViewListItem *ViewListWidget::currentCategory() const
+{
+    ViewListItem *item = static_cast<ViewListItem*>( m_viewlist->currentItem() );
+    if ( item == 0 ) {
+        return 0;
+    }
+    if ( item->type() == ViewListItem::ItemType_Category ) {
+        return item;
+    }
+    return static_cast<ViewListItem*>( item->parent() );
+}
+
 KoView *ViewListWidget::findView( const QString &tag ) const
 {
     ViewListItem *i = findItem( tag );
@@ -600,11 +630,12 @@ void ViewListWidget::setupContextMenus()
     QAction *action;
     // document insert actions
     // Query for document types
+#if 0
     m_lstEntries = KoDocumentEntry::query(KoDocumentEntry::OnlyEmbeddableDocuments);
     QList<KoDocumentEntry>::const_iterator it = m_lstEntries.begin();
     for( ; it != m_lstEntries.end(); ++it ) {
         KService::Ptr serv = (*it).service();
-        // TODO: Make this test safer (possibly add desktopfile to define embedable parts)
+        // TODO: Make this test safer (possibly add desktopfile to define embeddable parts)
         if ( serv->genericName().isEmpty() ||
             serv->serviceTypes().contains( "application/vnd.oasis.opendocument.formula" ) ||
             serv->serviceTypes().contains( "application/x-vnd.kde.kplato" ) ) {
@@ -615,39 +646,40 @@ void ViewListWidget::setupContextMenus()
             connect(action, SIGNAL( triggered( bool ) ), this, SLOT( slotCreatePart() ) );
             m_adddocument.append( action );
     }
+#endif
     // no item actions
     //action = new QAction( KIcon( "document-new" ), i18n( "New Category..." ), this );
     //m_noitem.append( action );
     
     // view insert actions
-    action = new QAction( KIcon( "edit-add" ), i18n( "View..." ), this );
+    action = new QAction( KIcon( "list-add" ), i18n( "View..." ), this );
     connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotAddView() ) );
     m_addview.append( action );
 
     // Category edit actions
-    action = new QAction( KIcon( "editinput" ), i18n( "Rename" ), this );
+    action = new QAction( KIcon( "edit-rename" ), i18n( "Rename" ), this );
     connect( action, SIGNAL( triggered( bool ) ), SLOT( renameCategory() ) );
     m_editcategory.append( action );
     action = new QAction( KIcon( "list-remove" ), i18n( "Remove" ), this );
     connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotRemoveCategory() ) );
     m_editcategory.append( action );
-    action = new QAction( KIcon( "configure" ), i18n( "Entry..." ), this );
+    action = new QAction( KIcon( "configure" ), i18n( "Configure..." ), this );
     connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotConfigureItem() ) );
     m_editcategory.append( action );
 
     // view edit actions
-    action = new QAction( KIcon( "editinput" ), i18n( "Rename" ), this );
+    action = new QAction( KIcon( "edit-rename" ), i18n( "Rename" ), this );
     connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotEditViewTitle() ) );
     m_editview.append( action );
-    action = new QAction( KIcon( "edit-delete" ), i18n( "Remove" ), this );
+    action = new QAction( KIcon( "list-remove" ), i18n( "Remove" ), this );
     connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotRemoveView() ) );
     m_editview.append( action );
-    action = new QAction( KIcon( "configure" ), i18n( "Entry..." ), this );
+    action = new QAction( KIcon( "configure" ), i18n( "Configure..." ), this );
     connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotConfigureItem() ) );
     m_editview.append( action );
 
     // document edit actions
-    action = new QAction( KIcon( "editinput" ), i18n( "Rename" ), this );
+    action = new QAction( KIcon( "edit-rename" ), i18n( "Rename" ), this );
     connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotEditDocumentTitle() ) );
     m_editdocument.append( action );
     action = new QAction( KIcon( "edit-delete" ), i18n( "Remove" ), this );
@@ -675,7 +707,7 @@ void ViewListWidget::contextMenuEvent ( QContextMenuEvent *event )
             lst += m_editview;
             ViewBase *v = dynamic_cast<ViewBase*>( m_contextitem->view() );
             if ( v ) {
-                lst += v->contextActionList();
+                lst += v->viewlistActionList();
             }
         } else if ( m_contextitem->type() == ViewListItem::ItemType_ChildDocument ) {
             lst += m_editdocument;
