@@ -1,10 +1,10 @@
 /* This file is part of the KDE project
-   Copyright (C) 2005 Dag Andersen <danders@get2net.dk>
+   Copyright (C) 2005 - 2007 Dag Andersen <danders@get2net.dk>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation;
-   version 2 of the License.
+   License as published by the Free Software Foundation; either
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,6 +30,7 @@
 #include <kdebug.h>
 
 #include <qdatetime.h>
+#include <QMutableMapIterator>
 
 namespace KPlato
 {
@@ -46,7 +47,7 @@ AppointmentInterval::AppointmentInterval(const AppointmentInterval &interval) {
     m_end = interval.endTime();
     m_load = interval.load();
 }
-AppointmentInterval::AppointmentInterval(const DateTime &start, const DateTime end, double load) {
+AppointmentInterval::AppointmentInterval(const DateTime &start, const DateTime &end, double load) {
     //kDebug()<<this;
     m_start = start;
     m_end = end;
@@ -66,13 +67,16 @@ Duration AppointmentInterval::effort(const DateTime &start, const DateTime end)
 }
 
 Duration AppointmentInterval::effort(const QDate &time, bool upto) const {
-    DateTime t( DateTime( QDateTime( time ), m_start.timeSpec() ) );
+    DateTime t( time, QTime(), m_start.timeSpec() );
+    //kDebug()<<time<<upto<<t<<m_start<<m_end;
     if (upto) {
         if (t <= m_start) {
             return Duration::zeroDuration;
         }
         DateTime e = (t < m_end ? t : m_end);
-        return (e - m_start) * m_load / 100;
+        Duration d = (e - m_start) * m_load / 100;
+        //kDebug()<<d.toString();
+        return d;
     }
     // from time till end
     if (t >= m_end) {
@@ -158,12 +162,151 @@ AppointmentInterval AppointmentInterval::firstInterval(const AppointmentInterval
     return a;
 }
 
-void AppointmentIntervalList::inSort(AppointmentInterval *a)
+//-----------------------
+AppointmentIntervalList &AppointmentIntervalList::operator+=( const AppointmentIntervalList &lst )
 {
-    Q_ASSERT( a->startTime() < a->endTime() );
-    insert( a->startTime().toString( KDateTime::ISODate ) + a->endTime().toString( KDateTime::ISODate ), a );
+    if ( lst.isEmpty() ) {
+        return *this;
+    }
+    foreach ( const AppointmentInterval &ai, lst ) {
+        add( ai );
+    }
+    return *this;
 }
 
+void AppointmentIntervalList::add( const AppointmentInterval &ai )
+{
+    add( ai.startTime(), ai.endTime(), ai.load() );
+}
+
+void AppointmentIntervalList::add( const DateTime &st, const DateTime &et, double load )
+{
+    //kDebug()<<st<<et<<load;
+    Q_ASSERT( st < et );
+    DateTime s = st;
+    DateTime e = et;
+    if ( isEmpty() || et < values().first().startTime() || st > values().last().endTime() ) {
+        inSort( AppointmentInterval( s, e, load ) );
+    } else {
+        // see if we have overlapping intervals
+        AppointmentIntervalList lst;
+        QMutableMapIterator<QString, AppointmentInterval> it( *this );
+        while ( it.hasNext() ) {
+            AppointmentInterval &ai = it.next().value();
+            DateTime ist = ai.startTime();
+            DateTime iet = ai.endTime();
+            //kDebug()<<"Check:"<<s<<ist<<e<<iet;
+            if ( s == ist && e == iet ) {
+                //kDebug()<<"Exact match";
+                ai.setLoad( ai.load() + load );
+                return;
+            }
+            if ( e <= ist ) {
+                //kDebug()<<"Before first or in between";
+                inSort( AppointmentInterval( s, e, load ) );
+                break;
+            }
+            if ( s >= iet ) {
+                continue;
+            }
+            if ( s > ist ) {
+                //kDebug()<<"Start in the middle";
+                double l = ai.load();
+                lst.inSort( AppointmentInterval( ist, s, l ) );
+                if ( e < iet ) {
+                    //kDebug()<<"End in the midle";
+                    lst.inSort( AppointmentInterval( s, e, l + load ) );
+                    lst.inSort( AppointmentInterval( e, iet, l ) );
+                    it.remove();
+                    break;
+                }
+                if ( e == iet ) {
+                    //kDebug()<<"End at end";
+                    lst.inSort( AppointmentInterval( s, e, l + load ) );
+                    it.remove();
+                    break;
+                }
+                lst.inSort( AppointmentInterval( s, iet, l + load ) );
+                if ( ! it.hasNext() ) {
+                    //kDebug()<<"e > iet -> at last interval";
+                    lst.inSort( AppointmentInterval( iet, e, l + load ) );
+                    it.remove();
+                    break;
+                }
+                //kDebug()<<"e > iet -> may overlap next interval";
+                s = iet;
+                it.remove();
+                continue;
+            }
+            if ( s < ist ) {
+                // Add the part that's before this interval
+                //kDebug()<<"start before";
+                lst.inSort( AppointmentInterval( s, ist, load ) );
+                s = ist;
+            }
+            if ( s == ist ) {
+                //kDebug()<<"Start now at start";
+                double l = ai.load();
+                if ( e < iet ) {
+                    //kDebug()<<"End in the midle";
+                    lst.inSort( AppointmentInterval( s, e, l + load ) );
+                    lst.inSort( AppointmentInterval( e, iet, l ) );
+                    it.remove();
+                    break;
+                }
+                if ( e == iet ) {
+                    //kDebug()<<"End at end";
+                    ai.setLoad( l + load );
+                    break;
+                }
+                ai.setLoad( l + load );
+                if ( ! it.hasNext() ) {
+                    //kDebug()<<"e > iet -> at last interval";
+                    lst.inSort( AppointmentInterval( iet, e, l ) );
+                    break;
+                }
+                //kDebug()<<"e > iet -> may overlap next interval";
+                s = iet;
+                continue;
+            }
+        }
+        foreach ( const AppointmentInterval &a, lst ) {
+            //kDebug()<<"Add:"<<a;
+            inSort( a );
+        }
+    }
+}
+
+void AppointmentIntervalList::inSort(const AppointmentInterval &a)
+{
+    Q_ASSERT( a.startTime() < a.endTime() );
+    insert( a.startTime().toString( KDateTime::ISODate ) + a.endTime().toString( KDateTime::ISODate ), a );
+}
+
+void AppointmentIntervalList::saveXML( QDomElement &element ) const
+{
+    foreach (const AppointmentInterval &i, *this ) {
+        i.saveXML(element);
+    }
+}
+
+bool AppointmentIntervalList::loadXML( KoXmlElement &element, XMLLoaderObject &status )
+{
+    KoXmlElement e;
+    forEachElement(e, element) {
+        if (e.tagName() == "interval") {
+            AppointmentInterval a;
+            if (a.loadXML(e, status)) {
+                inSort(a);
+            } else {
+                kError()<<"Could not load interval";
+            }
+        }
+    }
+    return true;
+}
+
+
 ////
 Appointment::Appointment()
     : m_extraRepeats(), m_skipRepeats() {
@@ -211,18 +354,46 @@ Appointment::Appointment( const Appointment &app)
 Appointment::~Appointment() {
     //kDebug()<<"("<<this<<")";
     detach();
-    qDeleteAll( m_intervals );
 }
 
-void Appointment::addInterval(AppointmentInterval *a) {
-    Q_ASSERT( a->startTime() < a->endTime() );
+AppointmentIntervalList Appointment::intervals( const DateTime &start, const DateTime &end ) const
+{
+    //kDebug()<<start<<end;
+    AppointmentIntervalList lst;
+    foreach ( const AppointmentInterval &i, m_intervals ) {
+        if ( i.startTime() >= end || i.endTime() <= start ) {
+            continue;
+        }
+        AppointmentInterval ai( i );
+        if ( ai.startTime() < start ) {
+            ai.setStartTime( start );
+        }
+        if ( ai.endTime() > end ) {
+            ai.setEndTime( end );
+        }
+        lst.inSort( ai );
+        //kDebug()<<ai.startTime().toString()<<ai.endTime().toString();
+    }
+    return lst;
+}
+
+void Appointment::setIntervals(const AppointmentIntervalList &lst) {
+    m_intervals.clear();
+    foreach( const AppointmentInterval &i, lst ) {
+	m_intervals.inSort( i );
+    }
+}
+
+void Appointment::addInterval(const AppointmentInterval &a) {
+    Q_ASSERT( a.startTime() < a.endTime() );
     m_intervals.inSort(a);
-    //if ( m_resource && m_resource->resource() && m_node && m_node->node() ) kDebug()<<"Mode="<<m_calculationMode<<":"<<m_resource->resource()->name()<<" to"<<m_node->node()->name()<<""<<a->startTime()<<a->endTime();
+    //if ( m_resource && m_resource->resource() && m_node && m_node->node() ) kDebug()<<"Mode="<<m_calculationMode<<":"<<m_resource->resource()->name()<<" to"<<m_node->node()->name()<<""<<a.startTime()<<a.endTime();
 }
 void Appointment::addInterval(const DateTime &start, const DateTime &end, double load) {
     Q_ASSERT( start < end );
-    addInterval(new AppointmentInterval(start, end, load));
+    addInterval(AppointmentInterval(start, end, load));
 }
+
 void Appointment::addInterval(const DateTime &start, const Duration &duration, double load) {
     DateTime e = start+duration;
     addInterval(start, e, load);
@@ -230,9 +401,9 @@ void Appointment::addInterval(const DateTime &start, const Duration &duration, d
 
 double Appointment::maxLoad() const {
     double v = 0.0;
-    foreach (AppointmentInterval *i, m_intervals.values() ) {
-        if (v < i->load())
-            v = i->load();
+    foreach (const AppointmentInterval &i, m_intervals ) {
+        if (v < i.load())
+            v = i.load();
     }
     return v;
 }
@@ -242,7 +413,7 @@ DateTime Appointment::startTime() const {
         //kDebug()<<"empty list";
         return DateTime();
     }
-    return m_intervals.values().first()->startTime();
+    return m_intervals.values().first().startTime();
 }
 
 DateTime Appointment::endTime() const {
@@ -250,7 +421,7 @@ DateTime Appointment::endTime() const {
         //kDebug()<<"empty list";
         return DateTime();
     }
-    return m_intervals.values().last()->endTime();
+    return m_intervals.values().last().endTime();
 }
 
 void Appointment::deleteAppointmentFromRepeatList(DateTime) {
@@ -267,36 +438,25 @@ bool Appointment::loadXML(KoXmlElement &element, XMLLoaderObject &status, Schedu
     //kDebug()<<project.name();
     Node *node = status.project().findNode(element.attribute("task-id"));
     if (node == 0) {
-        kError()<<"The referenced task does not exists: "<<element.attribute("task-id")<<endl;
+        kError()<<"The referenced task does not exists: "<<element.attribute("task-id");
         return false;
     }
     Resource *res = status.project().resource(element.attribute("resource-id"));
     if (res == 0) {
-        kError()<<"The referenced resource does not exists: resource id="<<element.attribute("resource-id")<<endl;
+        kError()<<"The referenced resource does not exists: resource id="<<element.attribute("resource-id");
         return false;
     }
     if (!res->addAppointment(this, sch)) {
-        kError()<<"Failed to add appointment to resource: "<<res->name()<<endl;
+        kError()<<"Failed to add appointment to resource: "<<res->name();
         return false;
     }
     if (!node->addAppointment(this, sch)) {
-        kError()<<"Failed to add appointment to node: "<<node->name()<<endl;
+        kError()<<"Failed to add appointment to node: "<<node->name();
         m_resource->takeAppointment(this);
         return false;
     }
     //kDebug()<<"res="<<m_resource<<" node="<<m_node;
-    KoXmlElement e;
-    forEachElement(e, element) {
-            if (e.tagName() == "interval") {
-            AppointmentInterval *a = new AppointmentInterval();
-                if (a->loadXML(e, status)) {
-                    addInterval(a);
-                } else {
-                    kError()<<"Could not load interval"<<endl;
-                    delete a;
-                }
-            }
-    }
+    m_intervals.loadXML( element, status );
     if (isEmpty()) {
         return false;
     }
@@ -305,14 +465,14 @@ bool Appointment::loadXML(KoXmlElement &element, XMLLoaderObject &status, Schedu
 
 void Appointment::saveXML(QDomElement &element) const {
     if (isEmpty()) {
-        kError()<<"Incomplete appointment data: No intervals"<<endl;
+        kError()<<"Incomplete appointment data: No intervals";
     }
     if (m_resource == 0 || m_resource->resource() == 0) {
-        kError()<<"Incomplete appointment data: No resource"<<endl;
+        kError()<<"Incomplete appointment data: No resource";
         return;
     }
     if (m_node == 0 || m_node->node() == 0) {
-        kError()<<"Incomplete appointment data: No node"<<endl;
+        kError()<<"Incomplete appointment data: No node";
         return; // shouldn't happen
     }
     //kDebug();
@@ -321,16 +481,14 @@ void Appointment::saveXML(QDomElement &element) const {
 
     me.setAttribute("resource-id", m_resource->resource()->id());
     me.setAttribute("task-id", m_node->node()->id());
-    foreach (AppointmentInterval *i, m_intervals.values() ) {
-        i->saveXML(me);
-    }
+    m_intervals.saveXML( me );
 }
 
 // Returns the total planned effort for this appointment
 Duration Appointment::plannedEffort() const {
     Duration d;
-    foreach (AppointmentInterval *i, m_intervals.values() ) {
-        d += i->effort();
+    foreach (const AppointmentInterval &i, m_intervals ) {
+        d += i.effort();
     }
     return d;
 }
@@ -340,8 +498,8 @@ Duration Appointment::plannedEffort(const QDate &date) const {
     Duration d;
     QDateTime s(date);
     QDateTime e(date.addDays(1));
-    foreach (AppointmentInterval *i, m_intervals.values() ) {
-        d += i->effort(DateTime(s, i->startTime().timeSpec()), DateTime(e, i->startTime().timeSpec()));
+    foreach (const AppointmentInterval &i, m_intervals ) {
+        d += i.effort(DateTime(s, i.startTime().timeSpec()), DateTime(e, i.startTime().timeSpec()));
     }
     return d;
 }
@@ -350,24 +508,28 @@ Duration Appointment::plannedEffort(const QDate &date) const {
 Duration Appointment::plannedEffortTo(const QDate& date) const {
     Duration d;
     QDate e(date.addDays(1));
-    foreach (AppointmentInterval *i, m_intervals.values() ) {
-        d += i->effort(e, true);
+    foreach (const AppointmentInterval &i, m_intervals ) {
+        d += i.effort(e, true); // upto e, not including
     }
+    //kDebug()<<date<<d.toString();
     return d;
 }
 
 // Returns a list of efforts pr day for interval start, end inclusive
 // The list only includes days with any planned effort
-EffortCostMap Appointment::plannedPrDay(const QDate& start, const QDate& end) const {
+EffortCostMap Appointment::plannedPrDay(const QDate& pstart, const QDate& pend) const {
     //kDebug()<<m_node->id()<<","<<m_resource->id();
     EffortCostMap ec;
     Duration eff;
+    QDate start = pstart.isValid() ? pstart : startTime().date();
+    QDate end = pend.isValid() ? pend : endTime().date();
     QDate dt(start);
     QDate ndt(dt.addDays(1));
-    double rate = m_resource->normalRatePrHour();
-    foreach (AppointmentInterval *i, m_intervals.values() ) {
-        DateTime st = i->startTime();
-        DateTime e = i->endTime();
+    double rate = m_resource ? m_resource->normalRatePrHour() : 0.0;
+    //kDebug()<<rate<<m_intervals.count();
+    foreach (const AppointmentInterval &i, m_intervals ) {
+        DateTime st = i.startTime();
+        DateTime e = i.endTime();
         if (end < st.date()) {
             break;
         }
@@ -375,10 +537,11 @@ EffortCostMap Appointment::plannedPrDay(const QDate& start, const QDate& end) co
             dt = st.date();
         }
         ndt = dt.addDays(1);
-        while (dt <= e.date()) {
-            eff = i->effort(DateTime(dt, QTime( 0, 0, 0 ), st.timeSpec()), DateTime(ndt, QTime( 0, 0, 0 ), st.timeSpec()));
+        while (dt <= e.date() && dt <= end ) {
+            //kDebug()<<start<<end<<dt;
+            eff = i.effort(DateTime(dt, QTime( 0, 0, 0 ), st.timeSpec()), DateTime(ndt, QTime( 0, 0, 0 ), st.timeSpec()));
             ec.add(dt, eff, eff.toDouble(Duration::Unit_h) * rate);
-            if (dt < e.date()) {
+            if (dt < e.date() && dt < end) {
                 // loop trough the interval (it spans dates)
                 dt = ndt;
                 ndt = ndt.addDays(1);
@@ -391,11 +554,13 @@ EffortCostMap Appointment::plannedPrDay(const QDate& start, const QDate& end) co
 }
 
 
-double Appointment::plannedCost() {
+EffortCost Appointment::plannedCost() const {
+    EffortCost ec;
+    ec.setEffort( plannedEffort() );
     if (m_resource && m_resource->resource()) {
-        return plannedEffort().toDouble(Duration::Unit_h) * m_resource->resource()->normalRate(); //FIXME overtime
+        ec.setCost( ec.hours() * m_resource->resource()->normalRate() ); //FIXME overtime
     }
-    return 0.0;
+    return ec;
 }
 
 //Calculates the planned cost on date
@@ -422,7 +587,7 @@ bool Appointment::attach() {
         return true;
     }
     kWarning()<<"Failed: "<<(m_resource ? "" : "resource=0 ")
-                                       <<(m_node ? "" : "node=0")<<endl;
+                                       <<(m_node ? "" : "node=0");
     return false;
 }
 
@@ -439,24 +604,24 @@ void Appointment::detach() {
 // Returns the effort from start to end
 Duration Appointment::effort(const DateTime &start, const DateTime &end) const {
     Duration d;
-    foreach (AppointmentInterval *i, m_intervals.values() ) {
-        d += i->effort(start, end);
+    foreach (const AppointmentInterval &i, m_intervals ) {
+        d += i.effort(start, end);
     }
     return d;
 }
 // Returns the effort from start for the duration
 Duration Appointment::effort(const DateTime &start, const Duration &duration) const {
     Duration d;
-    foreach (AppointmentInterval *i, m_intervals.values() ) {
-        d += i->effort(start, start+duration);
+    foreach (const AppointmentInterval &i, m_intervals ) {
+        d += i.effort(start, start+duration);
     }
     return d;
 }
 // Returns the effort upto time / from time
 Duration Appointment::effortFrom(const QDate &time) const {
     Duration d;
-    foreach (AppointmentInterval *i, m_intervals.values() ) {
-        d += i->effort(time, false);
+    foreach (const AppointmentInterval &i, m_intervals ) {
+        d += i.effort(time, false);
     }
     return d;
 }
@@ -486,62 +651,68 @@ void Appointment::copy(const Appointment &app) {
     //m_repeatInterval = app.repeatInterval();
     //m_repeatCount = app.repeatCount();
 
-    qDeleteAll( m_intervals );
     m_intervals.clear();
-    foreach (AppointmentInterval *i, app.intervals().values() ) {
-        addInterval(new AppointmentInterval(*i));
+    foreach (const AppointmentInterval &i, app.intervals() ) {
+        addInterval(i);
     }
 }
 
 void Appointment::merge(const Appointment &app) {
-    QList<AppointmentInterval*> result;
-    QList<AppointmentInterval*> lst1 = m_intervals.values();
-    AppointmentInterval *i1;
-    QList<AppointmentInterval*> lst2 = app.intervals().values();
+    //kDebug()<<this<<(m_node ? m_node->node()->name() : "no node")<<(app.node() ? app.node()->node()->name() : "no node");
+    if ( app.isEmpty() ) {
+	return;
+    }
+    if ( isEmpty() ) {
+	setIntervals( app.intervals() );
+	return;
+    }
+    QList<AppointmentInterval> result;
+    QList<AppointmentInterval> lst1 = m_intervals.values();
+    AppointmentInterval i1;
+    QList<AppointmentInterval> lst2 = app.intervals().values();
     //kDebug()<<"add"<<lst1.count()<<" intervals to"<<lst2.count()<<" intervals";
-    AppointmentInterval *i2;
+    AppointmentInterval i2;
     int index1 = 0, index2 = 0;
     DateTime from;
     while (index1 < lst1.size() || index2 < lst2.size()) {
         if (index1 >= lst1.size()) {
             i2 = lst2[index2];
-            if (!from.isValid() || from < i2->startTime())
-                from = i2->startTime();
-            result.append(new AppointmentInterval(from, i2->endTime(), i2->load()));
-            //kDebug()<<"Interval+ (i2):"<<from<<" -"<<i2->endTime();
-            from = i2->endTime();
+            if (!from.isValid() || from < i2.startTime())
+                from = i2.startTime();
+            result.append(AppointmentInterval(from, i2.endTime(), i2.load()));
+            //kDebug()<<"Interval+ (i2):"<<from<<" -"<<i2.endTime();
+            from = i2.endTime();
             ++index2;
             continue;
         }
         if (index2 >= lst2.size()) {
             i1 = lst1[index1];
-            if (!from.isValid() || from < i1->startTime())
-                from = i1->startTime();
-            result.append(new AppointmentInterval(from, i1->endTime(), i1->load()));
-            //kDebug()<<"Interval+ (i1):"<<from<<" -"<<i1->endTime();
-            from = i1->endTime();
+            if (!from.isValid() || from < i1.startTime())
+                from = i1.startTime();
+            result.append(AppointmentInterval(from, i1.endTime(), i1.load()));
+            //kDebug()<<"Interval+ (i1):"<<from<<" -"<<i1.endTime();
+            from = i1.endTime();
             ++index1;
             continue;
         }
         i1 = lst1[index1];
         i2 = lst2[index2];
-        AppointmentInterval i =  i1->firstInterval(*i2, from);
+        AppointmentInterval i =  i1.firstInterval(i2, from);
         if (!i.isValid()) {
             break;
         }
-        result.append(new AppointmentInterval(i)); 
+        result.append(AppointmentInterval(i)); 
         from = i.endTime();
         //kDebug()<<"Interval+ (i):"<<i.startTime()<<" -"<<i.endTime()<<" load="<<i.load();
-        if (i.endTime() >= i1->endTime()) {
+        if (i.endTime() >= i1.endTime()) {
             ++index1;
         }
-        if (i.endTime() >= i2->endTime()) {
+        if (i.endTime() >= i2.endTime()) {
             ++index2;
         }
     }
-    qDeleteAll( m_intervals );
     m_intervals.clear();
-    foreach ( AppointmentInterval *i, result ) {
+    foreach ( const AppointmentInterval &i, result ) {
         m_intervals.inSort( i );
     }
     //kDebug()<<this<<":"<<m_intervals.count();
@@ -574,8 +745,8 @@ void Appointment::printDebug(const QString& _indent)
         kDebug()<<indent<<"  +"<<m_intervals.count()<<" appointment intervals:";
     }
     indent += "  + ";
-    foreach (AppointmentInterval *i, m_intervals.values() ) {
-        kDebug()<<indent<<"----"<<i->startTime().toString()<<" -"<<i->endTime().toString()<<" load="<<i->load();
+    foreach (const AppointmentInterval &i, m_intervals ) {
+        kDebug()<<indent<<"----"<<i.startTime().toString()<<" -"<<i.endTime().toString()<<" load="<<i.load();
     }
 }
 #endif
