@@ -41,6 +41,7 @@
 #include <kdebug.h>
 #include <ksystemtimezone.h>
 #include <ktimezone.h>
+#include <krandom.h>
 
 namespace KPlato
 {
@@ -49,7 +50,8 @@ namespace KPlato
 Project::Project( Node *parent )
         : Node( parent ),
         m_accounts( *this ),
-        m_defaultCalendar( 0 )
+        m_defaultCalendar( 0 ),
+        m_taskDefaults( new Task() )
 {
     //kDebug()<<"("<<this<<")";
     m_constraint = Node::MustStartOn;
@@ -67,14 +69,14 @@ void Project::init()
     if ( m_parent == 0 ) {
         // set sensible defaults for a project wo parent
         m_constraintStartTime = DateTime( QDate::currentDate(), QTime(), m_spec );
-        m_constraintEndTime = m_constraintStartTime.addDays( 1 );
+        m_constraintEndTime = m_constraintStartTime.addYears( 2 );
     }
 }
 
 
 Project::~Project()
 {
-    disconnect(); // NOTE: may be a problem if sombody uses the destroyd() signal
+    disconnect(); // NOTE: may be a problem if somebody uses the destroyd() signal
     delete m_standardWorktime;
     while ( !m_resourceGroups.isEmpty() )
         delete m_resourceGroups.takeFirst();
@@ -82,10 +84,36 @@ Project::~Project()
         delete m_calendars.takeFirst();
     while ( !m_managers.isEmpty() )
         delete m_managers.takeFirst();
+    
+    delete m_taskDefaults;
 }
 
 int Project::type() const { return Node::Type_Project; }
 
+void Project::generateUniqueIds()
+{
+    foreach ( Node *n, nodeIdDict ) {
+        nodeIdDict.remove( n->id() );
+        n->setId( uniqueNodeId() );
+        nodeIdDict[ n->id() ] = n;
+    }
+    foreach ( ResourceGroup *g, resourceGroupIdDict ) {
+        resourceGroupIdDict.remove( g->id() );
+        g->setId( uniqueResourceGroupId() );
+        resourceGroupIdDict[ g->id() ] = g;
+    }
+    foreach ( Resource *r, resourceIdDict ) {
+        resourceIdDict.remove( r->id() );
+        r->setId( uniqueResourceId() );
+        resourceIdDict[ r->id() ] = r;
+    }
+    foreach ( Calendar *c, calendarIdDict ) {
+        calendarIdDict.remove( c->id() );
+        c->setId( uniqueCalendarId() );
+        calendarIdDict[ c->id() ] = c;
+    }
+}
+
 void Project::calculate( Schedule *schedule, const DateTime &dt )
 {
     if ( schedule == 0 ) {
@@ -102,22 +130,39 @@ void Project::calculate( const DateTime &dt )
         kError() << "No current schedule to calculate" << endl;
         return ;
     }
+    KLocale *locale = KGlobal::locale();
+    DateTime time = dt.isValid() ? dt : DateTime( KDateTime::currentLocalDateTime() );
     MainSchedule *cs = static_cast<MainSchedule*>( m_currentSchedule );
     Estimate::Use estType = ( Estimate::Use ) cs->type();
     if ( type() == Type_Project ) {
+        cs->setPhaseName( 0, i18n( "Init" ) );
+        cs->logInfo( i18n( "Schedule project from: %1", locale->formatDateTime( dt ) ), 0 );
         initiateCalculation( *cs );
         initiateCalculationLists( *cs ); // must be after initiateCalculation() !!
-        //kDebug()<<"Node="<<m_name<<" Start="<<m_constraintStartTime.toString();
-        cs->startTime = dt;
-        cs->earlyStart = dt;
-        // Calculate from start time
-        propagateEarliestStart( cs->earlyStart );
+        propagateEarliestStart( time );
+        // Calculate lateFinish from time. If a task has started, remaingEffort is used.
+        cs->setPhaseName( 1, i18n( "Forward" ) );
+        cs->logInfo( i18n( "Calculate finish" ), 1 );
         cs->lateFinish = calculateForward( estType );
+        cs->lateFinish = checkEndConstraints( cs->lateFinish );
         propagateLatestFinish( cs->lateFinish );
+        // Calculate earlyFinish. If a task has started, remaingEffort is used.
+        cs->setPhaseName( 2, i18n( "Backward" ) );
+        cs->logInfo( i18n( "Calculate start" ), 2 );
         cs->calculateBackward( estType );
+        // Schedule. If a task has started, remaingEffort is used and appointments are copied from parent
+        cs->setPhaseName( 3, i18n( "Schedule" ) );
+        cs->logInfo( i18n( "Schedule tasks forward" ), 3 );
         cs->endTime = scheduleForward( cs->startTime, estType );
+        cs->logInfo( i18n( "Scheduled finish: %1", locale->formatDateTime( cs->endTime ) ), 3 );
+        if ( cs->endTime > m_constraintEndTime ) {
+            cs->logError( i18n( "Could not finish project in time: %1", locale->formatDateTime( m_constraintEndTime ) ), 3 );
+        } else if ( cs->endTime == m_constraintEndTime ) {
+            cs->logWarning( i18n( "Finished project exactly on time: %1", locale->formatDateTime( m_constraintEndTime ) ), 3 );
+        } else {
+            cs->logInfo( i18n( "Finished project before time: %1", locale->formatDateTime( m_constraintEndTime ) ), 3 );
+        }
         calcCriticalPath( false );
-        //makeAppointments();
         calcResourceOverbooked();
         cs->notScheduled = false;
         calcFreeFloat();
@@ -132,13 +177,35 @@ void Project::calculate( const DateTime &dt )
 
 void Project::calculate( ScheduleManager &sm )
 {
-    emit sigProgress( 0 );
+    m_progress = 0;
+    int nodes = 0;
+    foreach ( Node *n, nodeIdDict ) {
+        if ( n->type() == Node::Type_Task || n->type() == Node::Type_Milestone ) {
+            nodes++;
+        }
+    }
+    int maxprogress = nodes * 3;
     //kDebug();
     if ( sm.recalculate() ) {
+        emit maxProgress( maxprogress );
+        incProgress();
         sm.setCalculateAll( false );
         sm.createSchedules();
-        calculate( sm.expected(), sm.fromDateTime() );
+        sm.setRecalculateFrom( KDateTime::currentLocalDateTime() );
+        if ( sm.parentManager() ) {
+            sm.expected()->startTime = sm.parentManager()->expected()->startTime;
+            sm.expected()->earlyStart = sm.parentManager()->expected()->earlyStart;
+        }
+        incProgress();
+        calculate( sm.expected(), KDateTime::currentLocalDateTime() );
     } else {
+        if ( sm.optimistic() ) {
+            maxprogress += nodes * 3;
+        }
+        if ( sm.pessimistic() ) {
+            maxprogress += nodes * 3;
+        }
+        emit maxProgress( maxprogress );
         sm.createSchedules();
         calculate( sm.expected() );
         emit scheduleChanged( sm.expected() );
@@ -150,9 +217,9 @@ void Project::calculate( ScheduleManager &sm )
         }
         setCurrentSchedule( sm.expected()->id() );
     }
-    emit sigProgress( 100 );
-    emit sigProgress( -1 );
+    emit sigProgress( maxprogress );
     emit projectCalculated( &sm );
+    emit scheduleManagerChanged( &sm );
     emit changed();
 }
 
@@ -169,37 +236,87 @@ void Project::calculate( Schedule *schedule )
 void Project::calculate()
 {
     if ( m_currentSchedule == 0 ) {
-        kError() << "No current schedule to calculate" << endl;
+        kError() << "No current schedule to calculate";
         return ;
     }
     MainSchedule *cs = static_cast<MainSchedule*>( m_currentSchedule );
+    bool backwards = false;
+    if ( cs->manager() ) {
+        backwards = cs->manager()->schedulingDirection();
+    }
+    KLocale *locale = KGlobal::locale();
     Estimate::Use estType = ( Estimate::Use ) cs->type();
     if ( type() == Type_Project ) {
         initiateCalculation( *cs );
         initiateCalculationLists( *cs ); // must be after initiateCalculation() !!
-        if ( m_constraint == Node::MustStartOn ) {
-            //kDebug()<<"Node="<<m_name<<" Start="<<m_constraintStartTime.toString();
+        if ( ! backwards ) {
+            cs->setPhaseName( 0, i18n( "Init" ) );
+            cs->logInfo( i18n( "Schedule project forward from: %1", locale->formatDateTime( m_constraintStartTime ) ), 0 );
             cs->startTime = m_constraintStartTime;
             cs->earlyStart = m_constraintStartTime;
             // Calculate from start time
             propagateEarliestStart( cs->earlyStart );
+            cs->setPhaseName( 1, i18n( "Forward" ) );
+            cs->logInfo( i18n( "Calculate late finish" ), 1 );
             cs->lateFinish = calculateForward( estType );
+            cs->lateFinish = checkEndConstraints( cs->lateFinish );
             propagateLatestFinish( cs->lateFinish );
+            cs->setPhaseName( 2, i18n( "Backward" ) );
+            cs->logInfo( i18n( "Calculate early start" ), 2 );
             cs->calculateBackward( estType );
+            cs->setPhaseName( 3, i18n( "Schedule" ) );
+            cs->logInfo( i18n( "Schedule tasks forward" ), 3 );
             cs->endTime = scheduleForward( cs->startTime, estType );
             cs->duration = cs->endTime - cs->startTime;
+            cs->logInfo( i18n( "Scheduled finish: %1", locale->formatDateTime( cs->endTime ) ), 3 );
+            if ( cs->endTime > m_constraintEndTime ) {
+                cs->schedulingError = true;
+                cs->logError( i18n( "Could not finish project in time: %1", locale->formatDateTime( m_constraintEndTime ) ), 3 );
+            } else if ( cs->endTime == m_constraintEndTime ) {
+                cs->logWarning( i18n( "Finished project exactly on time: %1", locale->formatDateTime( m_constraintEndTime ) ), 3 );
+            } else {
+                cs->logInfo( i18n( "Finished project before time: %1", locale->formatDateTime( m_constraintEndTime ) ), 3 );
+            }
             calcCriticalPath( false );
         } else {
-            //kDebug()<<"Node="<<m_name<<" End="<<m_constraintEndTime.toString();
-            cs->endTime = m_constraintEndTime;
-            cs->lateFinish = m_constraintEndTime;
+            cs->setPhaseName( 0, i18n( "Init" ) );
+            cs->logInfo( i18n( "Schedule project backward from: %1", locale->formatDateTime( m_constraintEndTime ) ), 0 );
             // Calculate from end time
-            propagateLatestFinish( cs->lateFinish );
+            propagateLatestFinish( m_constraintEndTime );
+            cs->setPhaseName( 1, i18n( "Backward" ) );
+            cs->logInfo( i18n( "Calculate early start" ), 1 );
             cs->earlyStart = calculateBackward( estType );
+            cs->earlyStart = checkStartConstraints( cs->earlyStart );
             propagateEarliestStart( cs->earlyStart );
-            cs->calculateForward( estType );
-            cs->startTime = scheduleBackward( cs->endTime, estType );
+            cs->setPhaseName( 2, i18n( "Forward" ) );
+            cs->logInfo( i18n( "Calculate late finish" ), 2 );
+            cs->lateFinish = cs->calculateForward( estType );
+            cs->setPhaseName( 3, i18n( "Schedule" ) );
+            cs->logInfo( i18n( "Schedule tasks backward" ), 3 );
+            cs->startTime = scheduleBackward( cs->lateFinish, estType );
+            cs->endTime = cs->startTime;
+            foreach ( Node *n, allNodes() ) {
+                if ( n->type() == Type_Task || n->type() == Type_Milestone ) {
+                    DateTime e = n->endTime( cs->id() );
+                    if ( cs->endTime <  e ) {
+                        cs->endTime = e;
+                    }
+                }
+            }
+            if ( cs->endTime > m_constraintEndTime ) {
+                cs->schedulingError = true;
+                cs->logError( i18n( "Failed to finish project within target time" ), 3 );
+            }
             cs->duration = cs->endTime - cs->startTime;
+            cs->logInfo( i18n( "Scheduled start: %1, target time: %2", locale->formatDateTime( cs->startTime ), locale->formatDateTime( m_constraintStartTime) ), 3 );
+            if ( cs->startTime < m_constraintStartTime ) {
+                cs->schedulingError = true;
+                cs->logError( i18n( "Must start project early in order to finish in time: %1", locale->formatDateTime( m_constraintStartTime) ), 3 );
+            } else if ( cs->startTime == m_constraintStartTime ) {
+                cs->logWarning( i18n( "Start project exactly on time: %1", locale->formatDateTime( m_constraintStartTime ) ), 3 );
+            } else {
+                cs->logInfo( i18n( "Can start project later than time: %1", locale->formatDateTime( m_constraintStartTime ) ), 3 );
+            }
             calcCriticalPath( true );
         }
         //makeAppointments();
@@ -215,6 +332,12 @@ void Project::calculate()
     }
 }
 
+void Project::incProgress()
+{
+    m_progress += 1;
+    emit sigProgress( m_progress );
+}
+
 bool Project::calcCriticalPath( bool fromEnd )
 {
     //kDebug();
@@ -239,7 +362,7 @@ bool Project::calcCriticalPath( bool fromEnd )
 
 void Project::calcCriticalPathList( MainSchedule *cs )
 {
-    kDebug()<<m_name<<", "<<cs->name()<<endl;
+    //kDebug()<<m_name<<", "<<cs->name()<<endl;
     cs->clearCriticalPathList();
     foreach ( Node *n, allNodes() ) {
         if ( n->numDependParentNodes() == 0 && n->inCriticalPath( cs->id() ) ) {
@@ -249,19 +372,19 @@ void Project::calcCriticalPathList( MainSchedule *cs )
         }
     }
     cs->criticalPathListCached = true;
-    kDebug()<<*(criticalPathList( cs->id() ))<<endl;
+    //kDebug()<<*(criticalPathList( cs->id() ))<<endl;
 }
 
 void Project::calcCriticalPathList( MainSchedule *cs, Node *node )
 {
-    kDebug()<<node->name()<<", "<<cs->id()<<endl;
+    //kDebug()<<node->name()<<", "<<cs->id()<<endl;
     bool newPath = false;
     QList<Node*> lst = *( cs->currentCriticalPath() );
     foreach ( Relation *r, node->dependChildNodes() ) {
         if ( r->child()->inCriticalPath( cs->id() ) ) {
             if ( newPath ) {
                 cs->addCriticalPath( &lst );
-                kDebug()<<node->name()<<" new path"<<endl;
+                //kDebug()<<node->name()<<" new path"<<endl;
             }
             cs->addCriticalPathNode( r->child() );
             calcCriticalPathList( cs, r->child() );
@@ -274,7 +397,7 @@ const QList< QList<Node*> > *Project::criticalPathList( long id )
 {
     Schedule *s = schedule( id );
     if ( s == 0 ) {
-        kDebug()<<"No schedule with id="<<id<<endl;
+        //kDebug()<<"No schedule with id="<<id<<endl;
         return 0;
     }
     MainSchedule *ms = static_cast<MainSchedule*>( s );
@@ -289,7 +412,7 @@ QList<Node*> Project::criticalPath( long id, int index )
 {
     Schedule *s = schedule( id );
     if ( s == 0 ) {
-        kDebug()<<"No schedule with id="<<id<<endl;
+        //kDebug()<<"No schedule with id="<<id<<endl;
         return QList<Node*>();
     }
     MainSchedule *ms = static_cast<MainSchedule*>( s );
@@ -312,11 +435,53 @@ DateTime Project::endTime(  long id ) const
     return s ? s->endTime : m_constraintEndTime;
 }
 
+Duration Project::duration( long id ) const
+{
+    Schedule *s = schedule( id );
+    return s ? s->duration : Duration::zeroDuration;
+}
+
 Duration *Project::getRandomDuration()
 {
     return 0L;
 }
 
+DateTime Project::checkStartConstraints( const DateTime &dt ) const
+{
+    DateTime t = dt;
+    foreach ( Node *n, nodeIdDict.values() ) {
+        if ( n->type() == Node::Type_Task || n->type() == Node::Type_Milestone ) {
+            switch ( n->constraint() ) {
+                case Node::FixedInterval:
+                case Node::StartNotEarlier:
+                case Node::MustStartOn:
+                        t = qMin( t, qMax( n->constraintStartTime(), m_constraintStartTime ) );
+                        break;
+                default: break;
+            }
+        }
+    }
+    return t;
+}
+
+DateTime Project::checkEndConstraints( const DateTime &dt ) const
+{
+    DateTime t = dt;
+    foreach ( Node *n, nodeIdDict.values() ) {
+        if ( n->type() == Node::Type_Task || n->type() == Node::Type_Milestone ) {
+            switch ( n->constraint() ) {
+                case Node::FixedInterval:
+                case Node::FinishNotLater:
+                case Node::MustFinishOn:
+                        t = qMax( t, qMin( n->constraintEndTime(), m_constraintEndTime ) );
+                        break;
+                default: break;
+            }
+        }
+    }
+    return t;
+}
+
 DateTime Project::calculateForward( int use )
 {
     //kDebug()<<m_name;
@@ -666,16 +831,12 @@ bool Project::load( KoXmlElement &element, XMLLoaderObject &status )
                 }
             }
             //kDebug()<<"Node schedules<---";
+        } else if ( e.tagName() == "wbs-definition" ) {
+            m_wbsDefinition.loadXML( e, status );
+        } else {
+            kWarning()<<"Unhandled tag:"<<e.tagName();
         }
     }
-    //kDebug()<<"Project schedules--->";
-    foreach ( Schedule * s, m_schedules ) {
-        if ( m_constraint == Node::MustFinishOn )
-            m_constraintEndTime = s->endTime;
-        else
-            m_constraintStartTime = s->startTime;
-    }
-    //kDebug()<<"Project schedules<---";
     //kDebug()<<"<---";
     return true;
 }
@@ -695,10 +856,12 @@ void Project::save( QDomElement &element ) const
     me.setAttribute( "start-time", m_constraintStartTime.toString( KDateTime::ISODate ) );
     me.setAttribute( "end-time", m_constraintEndTime.toString( KDateTime::ISODate ) );
 
+    m_wbsDefinition.saveXML( me );
+    
     m_accounts.save( me );
 
     // save calendars
-    foreach ( Calendar *c, calendarIdDict.values() ) {
+    foreach ( Calendar *c, calendarIdDict ) {
         c->save( me );
     }
     // save standard worktime
@@ -843,6 +1006,7 @@ Resource *Project::takeResource( ResourceGroup *group, Resource *resource )
 {
     emit resourceToBeRemoved( resource );
     Q_ASSERT( removeResourceId( resource->id() ) == true );
+    resource->removeRequests(); // not valid anymore
     Resource *r = group->takeResource( resource );
     Q_ASSERT( resource == r );
     emit resourceRemoved( resource );
@@ -850,6 +1014,16 @@ Resource *Project::takeResource( ResourceGroup *group, Resource *resource )
     return r;
 }
 
+void Project::moveResource( ResourceGroup *group, Resource *resource )
+{
+    if ( group == resource->parentGroup() ) {
+        return;
+    }
+    takeResource( resource->parentGroup(), resource );
+    addResource( group, resource );
+    return;
+}
+
 
 bool Project::addTask( Node* task, Node* position )
 {
@@ -886,7 +1060,7 @@ bool Project::addSubTask( Node* task, Node* parent )
     return addSubTask( task, -1, parent );
 }
 
-bool Project::addSubTask( Node* task, int index, Node* parent )
+bool Project::addSubTask( Node* task, int index, Node* parent, bool emitSignal )
 {
     // we want to add a subtask to the node "parent" at the given index.
     // If parent is 0, add to this
@@ -899,25 +1073,39 @@ bool Project::addSubTask( Node* task, int index, Node* parent )
         return false;
     }
     int i = index == -1 ? p->numChildren() : index;
-    emit nodeToBeAdded( p, i );
+    if ( emitSignal ) emit nodeToBeAdded( p, i );
     p->insertChildNode( i, task );
-    emit nodeAdded( task );
-    emit changed();
+    connect( this, SIGNAL( standardWorktimeChanged( StandardWorktime* ) ), task, SLOT( slotStandardWorktimeChanged( StandardWorktime* ) ) );
+    if ( emitSignal ) {
+        emit nodeAdded( task );
+        emit changed();
+    }
+    if ( p != this && p->numChildren() == 1 ) {
+        emit nodeChanged( p );
+    }
     return true;
 }
 
-void Project::takeTask( Node *node )
+void Project::takeTask( Node *node, bool emitSignal )
 {
+    //kDebug()<<node->name();
     Node * parent = node->parentNode();
     if ( parent == 0 ) {
         kDebug() <<"Node must have a parent!";
         return;
     }
     removeId( node->id() );
-    emit nodeToBeRemoved( node );
+    if ( emitSignal ) emit nodeToBeRemoved( node );
+    disconnect( this, SIGNAL( standardWorktimeChanged( StandardWorktime* ) ), node, SLOT( slotStandardWorktimeChanged( StandardWorktime* ) ) );
     parent->takeChildNode( node );
-    emit nodeRemoved( node );
-    emit changed();
+    if ( emitSignal ) {
+        emit nodeRemoved( node );
+        emit changed();
+    }
+    //kDebug()<<node->name()<<"removed";
+    if ( parent != this && parent->type() != Node::Type_Summarytask ) {
+        emit nodeChanged( parent );
+    }
 }
 
 bool Project::canMoveTask( Node* node, Node *newParent )
@@ -939,10 +1127,19 @@ bool Project::moveTask( Node* node, Node *newParent, int newPos )
     if ( ! canMoveTask( node, newParent ) ) {
         return false;
     }
+    Node *oldParent = node->parentNode();
     const Node *before = newParent->childNode( newPos );
-    takeTask( node );
+    emit nodeToBeMoved( node );
+    takeTask( node, false );
     int i = before == 0 ? newParent->numChildren() : newPos;
-    addSubTask( node, i, newParent );
+    addSubTask( node, i, newParent, false );
+    emit nodeMoved( node );
+    if ( oldParent != this && oldParent->numChildren() == 0 ) {
+        emit nodeChanged( oldParent );
+    }
+    if ( newParent != this && newParent->numChildren() == 1 ) {
+        emit nodeChanged( newParent );
+    }
     return true;
 }
 
@@ -1100,7 +1297,7 @@ Task *Project::createTask( Node* parent )
     return node;
 }
 
-Task *Project::createTask( Task &def, Node* parent )
+Task *Project::createTask( const Task &def, Node* parent )
 {
     Task * node = new Task( def, parent );
     node->setId( uniqueNodeId() );
@@ -1109,11 +1306,12 @@ Task *Project::createTask( Task &def, Node* parent )
 
 QString Project::uniqueNodeId( int seed )
 {
+    QString ident = KRandom::randomString( 10 );
     int i = seed;
-    while ( findNode( QString( "%1" ).arg( i ) ) ) {
-        ++i;
+    while ( findNode( ident ) ) {
+        ident = KRandom::randomString( 10 );
     }
-    return QString( "%1" ).arg( i );
+    return ident;
 }
 
 bool Project::removeId( const QString &id )
@@ -1182,14 +1380,11 @@ bool Project::setResourceGroupId( ResourceGroup *group )
 }
 
 QString Project::uniqueResourceGroupId() const {
-    QString id;
-    for (int i=0; i<32000 ; ++i) {
-        id = id.setNum(i);
-        if ( ! resourceGroupIdDict.contains( id ) ) {
-            return id;
-        }
+    QString id = KRandom::randomString( 10 );
+    while ( resourceGroupIdDict.contains( id ) ) {
+        id = KRandom::randomString( 10 );
     }
-    return QString();
+    return id;
 }
 
 ResourceGroup *Project::group( const QString& id )
@@ -1243,14 +1438,11 @@ bool Project::setResourceId( Resource *resource )
 }
 
 QString Project::uniqueResourceId() const {
-    QString id;
-    for (int i=0; i<32000 ; ++i) {
-        id = id.setNum(i);
-        if ( ! resourceIdDict.contains( id ) ) {
-            return id;
-        }
+    QString id = KRandom::randomString( 10 );
+    while ( resourceIdDict.contains( id ) ) {
+        id = KRandom::randomString( 10 );
     }
-    return QString();
+    return id;
 }
 
 Resource *Project::resource( const QString& id )
@@ -1273,19 +1465,32 @@ Resource *Project::resourceByName( const QString& name ) const
 QStringList Project::resourceNameList() const
 {
     QStringList lst;
-    foreach ( Resource *r, resourceIdDict.values() ) {
+    foreach ( Resource *r, resourceIdDict ) {
         lst << r->name();
     }
     return lst;
 }
 
-// TODO
-EffortCostMap Project::plannedEffortCostPrDay( const QDate & /*start*/, const QDate & /*end*/, long id ) const
+EffortCostMap Project::plannedEffortCostPrDay( const QDate & start, const QDate &end, long id ) const
 {
-    //kDebug();
+    //kDebug()<<start<<end<<id;
     EffortCostMap ec;
+    QListIterator<Node*> it( childNodeIterator() );
+    while ( it.hasNext() ) {
+        ec += it.next() ->plannedEffortCostPrDay( start, end, id );
+    }
     return ec;
+}
 
+EffortCostMap Project::actualEffortCostPrDay( const QDate & start, const QDate &end, long id ) const
+{
+    //kDebug()<<start<<end<<id;
+    EffortCostMap ec;
+    QListIterator<Node*> it( childNodeIterator() );
+    while ( it.hasNext() ) {
+        ec += it.next() ->actualEffortCostPrDay( start, end, id );
+    }
+    return ec;
 }
 
 // Returns the total planned effort for this project (or subproject)
@@ -1325,46 +1530,46 @@ Duration Project::plannedEffortTo( const QDate &date, long id ) const
 }
 
 // Returns the total actual effort for this project (or subproject)
-Duration Project::actualEffort( long id ) const
+Duration Project::actualEffort() const
 {
     //kDebug();
     Duration eff;
     QListIterator<Node*> it( childNodeIterator() );
     while ( it.hasNext() ) {
-        eff += it.next() ->actualEffort(id);
+        eff += it.next() ->actualEffort();
     }
     return eff;
 }
 
 // Returns the total actual effort for this project (or subproject) on date
-Duration Project::actualEffort( const QDate &date, long id ) const
+Duration Project::actualEffort( const QDate &date ) const
 {
     //kDebug();
     Duration eff;
     QListIterator<Node*> it( childNodeIterator() );
     while ( it.hasNext() ) {
-        eff += it.next() ->actualEffort( date, id );
+        eff += it.next() ->actualEffort( date );
     }
     return eff;
 }
 
 // Returns the total actual effort for this project (or subproject) upto and including date
-Duration Project::actualEffortTo( const QDate &date, long id ) const
+Duration Project::actualEffortTo( const QDate &date ) const
 {
     //kDebug();
     Duration eff;
     QListIterator
     <Node*> it( childNodeIterator() );
     while ( it.hasNext() ) {
-        eff += it.next() ->actualEffortTo( date, id );
+        eff += it.next() ->actualEffortTo( date );
     }
     return eff;
 }
 
-double Project::plannedCost( long id ) const
+EffortCost Project::plannedCost( long id ) const
 {
     //kDebug();
-    double c = 0;
+    EffortCost c;
     QListIterator
     <Node*> it( childNodeIterator() );
     while ( it.hasNext() ) {
@@ -1399,71 +1604,144 @@ double Project::plannedCostTo( const QDate &date, long id ) const
     return c;
 }
 
-double Project::actualCost( long id ) const
+double Project::actualCost() const
 {
     //kDebug();
     double c = 0;
     QListIterator
     <Node*> it( childNodeIterator() );
     while ( it.hasNext() ) {
-        c += it.next() ->actualCost(id);
+        c += it.next() ->actualCost();
     }
     return c;
 }
 
 // Returns the total planned effort for this project (or subproject) on date
-double Project::actualCost( const QDate &date, long id ) const
+double Project::actualCost( const QDate &date ) const
 {
     //kDebug();
     double c = 0;
     QListIterator
     <Node*> it( childNodeIterator() );
     while ( it.hasNext() ) {
-        c += it.next() ->actualCost( date, id );
+        c += it.next() ->actualCost( date );
     }
     return c;
 }
 
 // Returns the total planned effort for this project (or subproject) upto and including date
-double Project::actualCostTo( const QDate &date, long id ) const
+EffortCost Project::actualCostTo( const QDate &date ) const
 {
     //kDebug();
-    double c = 0;
-    QListIterator
-    <Node*> it( childNodeIterator() );
+    EffortCost c;
+    QListIterator<Node*> it( childNodeIterator() );
     while ( it.hasNext() ) {
-        c += it.next() ->actualCostTo( date, id );
+        c += it.next() ->actualCostTo( date );
     }
     return c;
 }
 
+Duration Project::budgetedWorkPerformed( const QDate &date, long id ) const
+{
+    //kDebug();
+    Duration e;
+    foreach (Node *n, childNodeIterator()) {
+        e += n->budgetedWorkPerformed( date, id );
+    }
+    return e;
+}
+
+double Project::budgetedCostPerformed( const QDate &date, long id ) const
+{
+    //kDebug();
+    double c = 0.0;
+    foreach (Node *n, childNodeIterator()) {
+        c += n->budgetedCostPerformed( date, id );
+    }
+    return c;
+}
+
+double Project::effortPerformanceIndex( const QDate &date, long id ) const
+{
+    //kDebug();
+    kDebug()<<date<<id;
+    Duration b = budgetedWorkPerformed( date, id );
+    if ( b == Duration::zeroDuration ) {
+        return 1.0;
+    }
+    Duration a = actualEffortTo( date );
+    if ( b == Duration::zeroDuration ) {
+        return 1.0;
+    }
+    return b.toDouble() / a.toDouble();
+}
+
+double Project::schedulePerformanceIndex( const QDate &date, long id ) const
+{
+    //kDebug();
+    double r = 1.0;
+    double s = bcws( date, id );
+    double p = bcwp( date, id );
+    if ( s > 0.0 ) {
+        r = p / s;
+    }
+    kDebug()<<s<<p<<r;
+    return r;
+}
+
+double Project::bcws( const QDate &date, long id ) const
+{
+    //kDebug();
+    double c = plannedCostTo( date, id );
+    kDebug()<<c;
+    return c;
+}
+
 double Project::bcwp( long id ) const
 {
-    //kDebug();
-    double c = 0;
-    foreach (Node *n, childNodeIterator()) {
-        c += n->bcwp(id);
-    }
-    return c;
+    QDate date = QDate::currentDate();
+    return bcwp( date, id );
 }
 
 double Project::bcwp( const QDate &date, long id ) const
 {
-    //kDebug();
-    double c = 0;
-    foreach (Node *n, childNodeIterator()) {
-        c += n->bcwp( date, id );
+    kDebug()<<date<<id;
+    QDate start = startTime( id ).date();
+    QDate end = endTime( id ).date();
+    EffortCostMap plan = plannedEffortCostPrDay( start, end, id );
+    EffortCostMap actual = actualEffortCostPrDay( start, (end > date ? end : date), id );
+
+    double budgetAtCompletion;
+    double plannedCompleted;
+    double actualCompleted;
+    double budgetedCompleted;
+    bool useEffort = false; //FIXME
+    if ( useEffort ) {
+        budgetAtCompletion = plan.totalEffort().toDouble( Duration::Unit_h );
+        plannedCompleted = plan.effortTo( date ).toDouble( Duration::Unit_h );
+        //actualCompleted = actual.effortTo( date ).toDouble( Duration::Unit_h );
+        actualCompleted = actualEffortTo( date ).toDouble( Duration::Unit_h );
+        budgetedCompleted = budgetedWorkPerformed( date, id ).toDouble( Duration::Unit_h );
+    } else {
+        budgetAtCompletion = plan.totalCost();
+        plannedCompleted = plan.costTo( date );
+        actualCompleted = actual.costTo( date );
+        budgetedCompleted = budgetedCostPerformed( date, id );
     }
+    double percentageCompletion = budgetedCompleted / budgetAtCompletion;
+    
+    double c = budgetAtCompletion * percentageCompletion; //??
+    kDebug()<<percentageCompletion<<budgetAtCompletion<<budgetedCompleted<<plannedCompleted;
     return c;
 }
 
-
 void Project::addCalendar( Calendar *calendar, Calendar *parent )
 {
     Q_ASSERT( calendar != 0 );
     //kDebug()<<calendar->name()<<","<<(parent?parent->name():"No parent");
     int row = parent == 0 ? m_calendars.count() : parent->calendars().count();
     emit calendarToBeAdded( parent, row );
+    calendar->setProject( this );
     if ( parent == 0 ) {
         calendar->setParentCal( 0 ); // in case
         m_calendars.append( calendar );
@@ -1494,6 +1772,7 @@ void Project::takeCalendar( Calendar *calendar )
         calendar->setParentCal( 0 );
     }
     emit calendarRemoved( calendar );
+    calendar->setProject( 0 );
     emit changed();
 }
 
@@ -1509,7 +1788,7 @@ Calendar *Project::calendar( const QString& id ) const
 
 Calendar *Project::calendarByName( const QString& name ) const
 {
-    foreach( Calendar *c, calendarIdDict.values() ) {
+    foreach( Calendar *c, calendarIdDict ) {
         if ( c->name() == name ) {
             return c;
         }
@@ -1530,7 +1809,7 @@ QList<Calendar*> Project::allCalendars() const
 QStringList Project::calendarNames() const
 {
     QStringList lst;
-    foreach( Calendar *c, calendarIdDict.values() ) {
+    foreach( Calendar *c, calendarIdDict ) {
         lst << c->name();
     }
     return lst;
@@ -1560,14 +1839,11 @@ bool Project::setCalendarId( Calendar *calendar )
 }
 
 QString Project::uniqueCalendarId() const {
-    QString id;
-    for (int i=1; i<32000 ; ++i) {
-        id = id.setNum(i);
-        if ( ! calendarIdDict.contains( id ) ) {
-            return id;
-        }
+    QString id = KRandom::randomString( 10 );
+    while ( calendarIdDict.contains( id ) ) {
+        id = KRandom::randomString( 10 );
     }
-    return QString();
+    return id;
 }
 
 void Project::setDefaultCalendar( Calendar *cal )
@@ -1588,19 +1864,37 @@ void Project::setStandardWorktime( StandardWorktime * worktime )
     if ( m_standardWorktime != worktime ) {
         delete m_standardWorktime;
         m_standardWorktime = worktime;
+        m_standardWorktime->setProject( this );
+        emit standardWorktimeChanged( worktime );
     }
 }
 
-bool Project::legalToLink( Node *par, Node *child )
+bool Project::linkExists( const Node *par, const Node *child ) const
+{
+    if ( par == 0 || child == 0 || par == child || par->isDependChildOf( child ) ) {
+        return false;
+    }
+    foreach ( Relation *r, par->dependChildNodes() ) {
+        if ( r->child() == child ) {
+            return true;
+        }
+    }
+    return false;
+}
+
+bool Project::legalToLink( const Node *par, const Node *child ) const
 {
     //kDebug()<<par.name()<<" ("<<par.numDependParentNodes()<<" parents)"<<child.name()<<" ("<<child.numDependChildNodes()<<" children)";
 
     if ( par == 0 || child == 0 || par == child || par->isDependChildOf( child ) ) {
         return false;
     }
+    if ( linkExists( par, child ) ) {
+        return false;
+    }
     bool legal = true;
     // see if par/child is related
-    if ( par->isParentOf( child ) || child->isParentOf( par ) ) {
+    if ( legal && ( par->isParentOf( child ) || child->isParentOf( par ) ) ) {
         legal = false;
     }
     if ( legal )
@@ -1608,10 +1902,17 @@ bool Project::legalToLink( Node *par, Node *child )
     if ( legal )
         legal = legalParents( par, child );
 
+    if ( legal ) {
+        foreach ( Node *p, par->childNodeIterator() ) {
+            if ( ! legalToLink( p, child ) ) {
+                return false;
+            }
+        }
+    }
     return legal;
 }
 
-bool Project::legalParents( Node *par, Node *child )
+bool Project::legalParents( const Node *par, const Node *child ) const
 {
     bool legal = true;
     //kDebug()<<par->name()<<" ("<<par->numDependParentNodes()<<" parents)"<<child->name()<<" ("<<child->numDependChildNodes()<<" children)";
@@ -1629,7 +1930,7 @@ bool Project::legalParents( Node *par, Node *child )
     return legal;
 }
 
-bool Project::legalChildren( Node *par, Node *child )
+bool Project::legalChildren( const Node *par, const Node *child ) const
 {
     bool legal = true;
     //kDebug()<<par->name()<<" ("<<par->numDependParentNodes()<<" parents)"<<child->name()<<" ("<<child->numDependChildNodes()<<" children)";
@@ -1645,17 +1946,36 @@ bool Project::legalChildren( Node *par, Node *child )
     return legal;
 }
 
-void Project::generateWBS( int count, WBSDefinition &def, const QString& wbs )
+WBSDefinition &Project::wbsDefinition()
 {
-    if ( type() == Type_Subproject || def.level0Enabled() ) {
-        Node::generateWBS( count, def, wbs );
-    } else {
-        QListIterator<Node*> it = m_nodes;
-        int i = 0;
-        while ( it.hasNext() ) {
-            it.next() ->generateWBS( ++i, def, m_wbs );
-        }
+    return m_wbsDefinition;
+}
+
+void Project::setWbsDefinition( const WBSDefinition &def )
+{
+    //kDebug();
+    m_wbsDefinition = def;
+    emit wbsDefinitionChanged();
+    emit changed();
+}
+
+QString Project::generateWBSCode( QList<int> &indexes ) const
+{
+    QString code = m_wbsDefinition.projectCode();
+    if ( ! code.isEmpty() && ! indexes.isEmpty() ) {
+        code += m_wbsDefinition.projectSeparator();
     }
+    int level = 1;
+    foreach ( int index, indexes ) {
+        code += m_wbsDefinition.code( index + 1, level  );
+        if ( level < indexes.count() ) {
+            // not last level, add separator also
+            code += m_wbsDefinition.separator( level );
+        }
+        ++level;
+    }
+    //kDebug()<<code;
+    return code;
 }
 
 void Project::setCurrentSchedule( long id )
@@ -1738,6 +2058,9 @@ void Project::addScheduleManager( ScheduleManager *sm, ScheduleManager *parent )
 
 int Project::takeScheduleManager( ScheduleManager *sm )
 {
+    foreach ( ScheduleManager *s, sm->children() ) {
+        takeScheduleManager( s );
+    }
     int index = -1;
     if ( sm->parentManager() ) {
         int index = sm->parentManager()->indexOf( sm );
@@ -1786,6 +2109,20 @@ ScheduleManager *Project::createScheduleManager()
     return createScheduleManager( uniqueScheduleName() );
 }
 
+bool Project::isBaselined( long id ) const
+{
+    if ( id == ANYSCHEDULED ) {
+        foreach ( ScheduleManager *sm, allScheduleManagers() ) {
+            if ( sm->isBaselined() ) {
+                return true;
+            }
+        }
+        return false;
+    }
+    Schedule *s = schedule( id );
+    return s == 0 ? false : s->isBaselined();
+}
+
 MainSchedule *Project::createSchedule( const QString& name, Schedule::Type type )
 {
     //kDebug()<<"No of schedules:"<<m_schedules.count();
@@ -1802,7 +2139,7 @@ void Project::addMainSchedule( MainSchedule *sch )
         return;
     }
     //kDebug()<<"No of schedules:"<<m_schedules.count();
-    long i = 1;
+    long i = 1; // keep this positive (negative values are special...)
     while ( m_schedules.contains( i ) ) {
         ++i;
     }
@@ -1867,6 +2204,7 @@ void Project::sendScheduleAdded( const MainSchedule *sch )
 
 void Project::sendScheduleToBeRemoved( const MainSchedule *sch )
 {
+    //kDebug()<<sch->id();
     emit scheduleToBeRemoved( sch );
 }
 
@@ -1889,6 +2227,12 @@ void Project::changed( Calendar *cal )
     emit changed();
 }
 
+void Project::changed( StandardWorktime *w )
+{
+    emit standardWorktimeChanged( w );
+    emit changed();
+}
+
 bool Project::addRelation( Relation *rel, bool check )
 {
     if ( rel->parent() == 0 || rel->child() == 0 ) {
@@ -1928,6 +2272,26 @@ void Project::setRelationLag( Relation *rel, const Duration &lag )
     emit changed();
 }
 
+QList<Node*> Project::flatNodeList( Node *parent )
+{
+    QList<Node*> lst;
+    Node *p = parent == 0 ? this : parent;
+    //kDebug()<<p->name()<<lst.count();
+    foreach ( Node *n, p->childNodeIterator() ) {
+        lst.append( n );
+        if ( n->numChildren() > 0 ) {
+            lst += flatNodeList( n );
+        }
+    }
+    return lst;
+}
+
+void Project::setTaskDefaults( const Task &task )
+{
+    delete m_taskDefaults;
+    m_taskDefaults = new Task( task );
+}
+
 #ifndef NDEBUG
 void Project::printDebug( bool children, const QByteArray& _indent )
 {
@@ -1944,7 +2308,7 @@ void Project::printCalendarDebug( const QByteArray& _indent )
 {
     QByteArray indent = _indent;
     kDebug() << indent <<"-------- Calendars debug printout --------";
-    foreach ( Calendar *c, calendarIdDict.values() ) {
+    foreach ( Calendar *c, calendarIdDict ) {
         c->printDebug( indent + "--" );
         kDebug();
     }
@@ -1952,17 +2316,6 @@ void Project::printCalendarDebug( const QByteArray& _indent )
         m_standardWorktime->printDebug();
 }
 #endif
-//use in pert to have the project slack
-void Project::setProjectSlack(const int& theValue)
-{
-    m_projectSlack = theValue;
-}
-
-int Project::projectSlack() const
-{
-    return m_projectSlack;
-}
-
 
 
 }  //KPlato namespace
