@@ -27,11 +27,13 @@
 
 #include "kptnode.h"
 
+#include "kptglobal.h"
 #include "kptaccount.h"
 #include "kptcalendar.h"
 #include "kptdatetime.h"
 #include "kptduration.h"
 #include "kptresource.h"
+#include "kptwbsdefinition.h"
 
 #include <QMap>
 #include <QList>
@@ -48,6 +50,7 @@ class Schedule;
 class StandardWorktime;
 class ScheduleManager;
 class XMLLoaderObject;
+class Task;
 
 /**
  * Project is the main node in a project, it contains child nodes and
@@ -94,6 +97,9 @@ public:
     virtual DateTime startTime( long id = -1 ) const;
     virtual DateTime endTime( long id = -1 ) const;
 
+    /// Returns the calculated duration for schedule @p id
+    Duration duration( long id = -1 ) const;
+    using Node::duration;
     /**
      * Instead of using the expected duration, generate a random value using
      * the Distribution of each Task. This can be used for Monte-Carlo
@@ -104,9 +110,10 @@ public:
     virtual bool load( KoXmlElement &element, XMLLoaderObject &status );
     virtual void save( QDomElement &element ) const;
     
-    /// Save a workpackage document containing @node with schedule identity @id
+    using Node::saveWorkPackageXML;
+    /// Save a workpackage document containing @node with schedule identity @p id
     void saveWorkPackageXML( QDomElement &element, const Node *node, long id ) const;
-
+    
     /**
      * Add the node @p task to the project, after node @p position
      * If @p postition is zero, it will be added to this project.
@@ -120,12 +127,12 @@ public:
      * Add the node @p task to @p parent, in position @p index
      * If @p parent is zero, it will be added to this project.
      */
-    bool addSubTask( Node* task, int index, Node* parent );
+    bool addSubTask( Node* task, int index, Node* parent, bool emitSignal = true );
     /**
      * Remove the @p node.
      * The node is not deleted.
      */
-    void takeTask( Node *node );
+    void takeTask( Node *node, bool emitSignal = true );
     bool canMoveTask( Node* node, Node *newParent );
     bool moveTask( Node* node, Node *newParent, int newPos );
     bool canIndentTask( Node* node );
@@ -145,8 +152,9 @@ public:
      * Create a copy of @p def with a unique id with its parent set to @p parent.
      * The task is not added to the parent. Do this with addSubTask().
      */
-    Task *createTask( Task &def, Node* parent );
+    Task *createTask( const Task &def, Node* parent );
 
+    int resourceGroupCount() const { return m_resourceGroups.count(); }
     QList<ResourceGroup*> &resourceGroups();
     /// Adds the resource group to the project.
     virtual void addResourceGroup( ResourceGroup *resource, int index = -1 );
@@ -175,6 +183,8 @@ public:
      * Always use this to remove resources.
      */ 
     Resource *takeResource( ResourceGroup *group, Resource *resource );
+    /// Move @p resource to the new @p group. Requests are removed.
+    void moveResource( ResourceGroup *group, Resource *resource );
     /// Returns the resource with identity id.
     Resource *resource( const QString& id );
     /// Returns the resource with matching name, 0 if no match is found.
@@ -193,16 +203,16 @@ public:
     virtual Duration plannedEffortTo( const QDate &date, long id = -1  ) const;
 
     /// Returns the actual effort
-    virtual Duration actualEffort( long id = -1 ) const;
-    /// Returns the actual effort on date
-    virtual Duration actualEffort( const QDate &date, long id = -1  ) const;
-    /// Returns the actual effort up to and including date
-    virtual Duration actualEffortTo( const QDate &date, long id = -1  ) const;
+    virtual Duration actualEffort() const;
+    /// Returns the actual effort on @p date
+    virtual Duration actualEffort( const QDate &date) const;
+    /// Returns the actual effort up to and including @p date
+    virtual Duration actualEffortTo( const QDate &date ) const;
     /**
      * Returns the total planned cost for this project
      * @param id Identity of the schedule to be used
      */
-    virtual double plannedCost( long id = -1 ) const;
+    virtual EffortCost plannedCost( long id = CURRENTSCHEDULE ) const;
     /**
      * Planned cost on date
      * @param date The cost is calulated for this date (only)
@@ -218,27 +228,37 @@ public:
 
     /**
      * Returns the actually reported cost for this project
+     */
+    virtual double actualCost() const;
+    /**
+     *  Actual cost on @p date
      * @param date The cost is calulated for this date (only)
      * @param id Identity of the schedule to be used
      */
-    virtual double actualCost( long id = -1 ) const;
+    virtual double actualCost( const QDate &date ) const;
     /**
-     *  Actual cost on date
-     * @param date The cost is calulated for this date (only)
-     * @param id Identity of the schedule to be used
-     */
-    virtual double actualCost( const QDate &date, long id = -1  ) const;
-    /**
-     * Actual cost up to and including date
+     * Actual cost up to and including @p date
      * @param date The cost is calculated from the start of the project upto including date.
-     * @param id Identity of the schedule to be used.
      */
-    virtual double actualCostTo( const QDate &date, long id = -1  ) const;
+    virtual EffortCost actualCostTo( const QDate &date ) const;
     
+    virtual EffortCostMap actualEffortCostPrDay( const QDate &start, const QDate &end, long id = -1 ) const;
+    
+    double effortPerformanceIndex( const QDate &date, long id ) const;
+    
+    double schedulePerformanceIndex( const QDate &date, long id ) const;
+    
+    /// Returns the effort planned to be used to reach the actual percent finished
+    virtual Duration budgetedWorkPerformed( const QDate &date, long id = CURRENTSCHEDULE ) const;
+    /// Returns the cost planned to be used to reach the actual percent finished
+    virtual double budgetedCostPerformed( const QDate &date, long id = CURRENTSCHEDULE ) const;
+
+    /// Budgeted Cost of Work Scheduled ( up to @p date )
+    virtual double bcws( const QDate &date, long id = BASELINESCHEDULE ) const;
     /// Budgeted Cost of Work Performed
-    virtual double bcwp( long id = -1 ) const;
+    virtual double bcwp( long id = BASELINESCHEDULE ) const;
     /// Budgeted Cost of Work Performed ( up to @p date )
-    virtual double bcwp( const QDate &date, long id = -1 ) const;
+    virtual double bcwp( const QDate &date, long id = BASELINESCHEDULE ) const;
 
     Calendar *defaultCalendar() const { return m_defaultCalendar; }
     void setDefaultCalendar( Calendar *cal );
@@ -254,7 +274,10 @@ public:
     Calendar *calendarByName( const QString &name ) const;
     void changed( Calendar *cal );
     QList<Calendar*> allCalendars() const;
-
+    /// Return number of calendars
+    int calendarCount() const { return m_calendars.count(); }
+    /// Return the calendar at @p index, 0 if index out of bounds
+    Calendar *calendarAt( int index ) const { return m_calendars.value( index ); }
     /**
      * Defines the length of days, weeks, months and years
      * and the standard working week.
@@ -263,14 +286,21 @@ public:
      */
     StandardWorktime *standardWorktime() { return m_standardWorktime; }
     void setStandardWorktime( StandardWorktime * worktime );
+    void changed( StandardWorktime* );
 
-    /// Check if node par can be linked to node child.
-    bool legalToLink( Node *par, Node *child );
+    /// Check if a link exists between node @p par and @p child.
+    bool linkExists( const Node *par, const Node *child ) const;
+    /// Check if node @p par can be linked to node @p child.
+    bool legalToLink( const Node *par, const Node *child ) const;
+    using Node::legalToLink;
 
     virtual const QHash<QString, Node*> &nodeDict() { return nodeIdDict; }
-    /// Return a list of all nodes in the project
+    /// Return a list of all nodes in the project (exluding myself)
     QList<Node*> allNodes();
+    /// Return the number of all nodes in the project (exluding myself)
+    int nodeCount() const { return nodeIdDict.count() - 1; }
 
+    using Node::findNode;
     /// Find the node with identity id
     virtual Node *findNode( const QString &id ) const
     {
@@ -281,8 +311,11 @@ public:
         }
         return m_parent->findNode( id );
     }
+    using Node::removeId;
     /// Remove the node with identity id from the register
     virtual bool removeId( const QString &id );
+    
+    using Node::insertId;
     /// Insert the node with identity id
     virtual void insertId( const QString &id, Node *node );
     /// Register node. The nodes id must be unique and non-empty.
@@ -344,9 +377,15 @@ public:
     /// returns a unique calendar id
     QString uniqueCalendarId() const;
 
-    void generateWBS( int count, WBSDefinition &def, const QString& wbs = QString() );
-
+    /// Return reference to WBS Definition
+    WBSDefinition &wbsDefinition();
+    /// Set WBS Definition to @p def
+    void setWbsDefinition( const WBSDefinition &def );
+    /// Generate WBS Code
+    virtual QString generateWBSCode( QList<int> &indexes ) const;
+    
     Accounts &accounts() { return m_accounts; }
+    const Accounts &accounts() const { return m_accounts; }
 
     /**
      * Set current schedule to schedule with identity id, for me and my children
@@ -376,6 +415,8 @@ public:
     ScheduleManager *findScheduleManager( const QString &name ) const;
     /// Returns a list of all schedule managers
     QList<ScheduleManager*> allScheduleManagers() const;
+    /// Return true if schedule with identity @p id is baselined
+    bool isBaselined( long id = ANYSCHEDULED ) const;
     
     void changed( ResourceGroup *group );
     void changed( Resource *resource );
@@ -387,9 +428,6 @@ public:
     void sendScheduleRemoved( const MainSchedule *sch );
     void sendScheduleToBeRemoved( const MainSchedule *sch );
 
-    void setProjectSlack(const int& theValue);
-    int projectSlack() const;
-
     /// Return the time spec used in this project
     const KDateTime::Spec &timeSpec() const { return m_spec; }
     /// Return the time zone used in this project
@@ -401,13 +439,13 @@ public:
      * Add a relation between the nodes specified in the relation rel.
      * Emits signals relationToBeAdded() before the relation is added,
      * and relationAdded() after it has been added.
-     * @parem rel The relation to be added.
-     * @parem check If true, the relation is checked for validity
-     * @Return true if successful.
+     * @param rel The relation to be added.
+     * @param check If true, the relation is checked for validity
+     * @return true if successful.
      */
     bool addRelation( Relation *rel, bool check=true );
     /**
-     * Removes the relation without deleting it.
+     * Removes the relation @p rel without deleting it.
      * Emits signals relationToBeRemoved() before the relation is removed,
      * and relationRemoved() after it has been removed.
      */
@@ -427,19 +465,31 @@ public:
     /**
      * Returns the list of critical paths for schedule @p id
      */
-    const QList< QList<Node*> > *criticalPathList( long id = -1 );
-    QList<Node*> criticalPath( long id = -1, int index = 0 );
+    const QList< QList<Node*> > *criticalPathList( long id = CURRENTSCHEDULE );
+    QList<Node*> criticalPath( long id = CURRENTSCHEDULE, int index = 0 );
+
+    /// Returns a flat list af all nodes
+    QList<Node*> flatNodeList( Node *parent = 0 );
+    
+    void generateUniqueIds();
+    
+    void setTaskDefaults( const Task &task );
+    const Task &taskDefaults() const { return *m_taskDefaults; }
+    
+    void incProgress();
 
 signals:
     /// Emitted when anything in the project is changed (use with care)
     void changed();
+    /// Emitted when the WBS code definition has changed. This may change all nodes.
+    void wbsDefinitionChanged();
     /// Emitted when a schedule has been calculated
     void projectCalculated( ScheduleManager *sm );
     /// Emitted when the pointer to the current schedule has been changed
     void currentScheduleChanged();
     /// Use to show progress during calculation
     void sigProgress( int );
-    
+    void maxProgress( int );
     /// This signal is emitted when one of the nodes members is changed.
     void nodeChanged( Node* );
     /// This signal is emitted when the node is to be added to the project.
@@ -492,6 +542,10 @@ signals:
      * @parem cal The new default calendar. May be 0.
      */
     void defaultCalendarChanged( Calendar *cal );
+    /**
+     * Emitted when the the standard worktime has been changed.
+     */
+    void standardWorktimeChanged( StandardWorktime* );
     
     /// Emitted when the relation @p rel is about to be added.
     void relationToBeAdded( Relation *rel, int parentIndex, int childIndex );
@@ -534,12 +588,14 @@ protected:
     DateTime scheduleForward( const DateTime &earliest, int use );
     DateTime scheduleBackward( const DateTime &latest, int use );
     void adjustSummarytask();
+    DateTime checkStartConstraints( const DateTime &dt ) const;
+    DateTime checkEndConstraints( const DateTime &dt ) const;
 
     void initiateCalculation( MainSchedule &sch );
     void initiateCalculationLists( MainSchedule &sch );
 
-    bool legalParents( Node *par, Node *child );
-    bool legalChildren( Node *par, Node *child );
+    bool legalParents( const Node *par, const Node *child ) const;
+    bool legalChildren( const Node *par, const Node *child ) const;
 
 private:
     void init();
@@ -552,10 +608,14 @@ private:
     QList<ScheduleManager*> m_managers;
     KDateTime::Spec m_spec;
     
+    WBSDefinition m_wbsDefinition;
+    
     //use in pert to store the project slack
     int m_projectSlack;
 
-    
+    Task *m_taskDefaults;
+    int m_progress;
+
 #ifndef NDEBUG
 public:
     void printDebug( bool children, const QByteArray& indent );
