@@ -50,6 +50,13 @@ ResourceGroup::ResourceGroup()
     //kDebug()<<"("<<this<<")";
 }
 
+ResourceGroup::ResourceGroup( const ResourceGroup *group )
+    : QObject( 0 ) 
+{
+    m_project = 0;
+    copy( group );
+}
+
 ResourceGroup::~ResourceGroup() {
     //kDebug()<<"("<<this<<")";
     if (findId() == this) {
@@ -64,6 +71,14 @@ ResourceGroup::~ResourceGroup() {
     //kDebug()<<"("<<this<<")";
 }
 
+void ResourceGroup::copy( const ResourceGroup *group )
+{
+    //m_project = group->m_project; //Don't copy
+    m_id = group->m_id;
+    m_type = group->m_type;
+    m_name = group->m_name;
+}
+
 void ResourceGroup::changed() {
     if ( m_project ) {
         m_project->changed( this );
@@ -253,13 +268,13 @@ Resource::Resource()
     : QObject( 0 ), // atm QObject is only for casting
     m_project(0),
     m_parent( 0 ),
-    m_schedules(), 
-    m_workingHours() 
+    m_schedules(),
+    m_currentSchedule( 0 )
 {
     m_type = Type_Work;
     m_units = 100; // %
 
-    m_availableFrom = DateTime(KDateTime::currentLocalDateTime());
+    m_availableFrom = DateTime( QDate::currentDate(), QTime( 0, 0, 0 ) );
     m_availableUntil = m_availableFrom.addYears(2);
 
     cost.normalRate = 100;
@@ -273,7 +288,8 @@ Resource::Resource()
 Resource::Resource(Resource *resource)
     : QObject( 0 ), // atm QObject is only for casting
     m_project( 0 ),
-    m_parent( 0 )
+    m_parent( 0 ),
+    m_currentSchedule( 0 )
 {
     //kDebug()<<"("<<this<<") from ("<<resource<<")";
     copy(resource); 
@@ -284,15 +300,18 @@ Resource::~Resource() {
     if (findId() == this) {
         removeId(); // only remove myself (I may be just a working copy)
     }
-    foreach (ResourceRequest *r, m_requests) {
-        r->setResource(0); // avoid the request to mess with my list
-    }
-    foreach (ResourceRequest *r, m_requests) {
-        r->parent()->deleteResourceRequest(r);
-    }
+    removeRequests();
     foreach (long key, m_schedules.keys()) {
         delete m_schedules.take(key);
     }
+    clearExternalAppointments();
+}
+
+void Resource::removeRequests() {
+    foreach (ResourceRequest *r, m_requests) {
+        r->setResource(0); // avoid the request to mess with my list
+        r->parent()->deleteResourceRequest(r);
+    }
 }
 
 void Resource::setId(const QString& id) {
@@ -309,9 +328,7 @@ void Resource::copy(Resource *resource) {
     m_email = resource->email();
     m_availableFrom = resource->availableFrom();
     m_availableUntil = resource->availableUntil();
-    m_workingHours.clear();
-    m_workingHours = resource->workingHours();
-
+    
     m_units = resource->units(); // available units in percent
 
     m_type = resource->type();
@@ -320,6 +337,10 @@ void Resource::copy(Resource *resource) {
     cost.overtimeRate = resource->overtimeRate();
     
     m_calendar = resource->m_calendar;
+
+    // hmmmm
+    //m_externalAppointments = resource->m_externalAppointments;
+    //m_externalNames = resource->m_externalNames;
 }
 
 void Resource::changed()
@@ -374,12 +395,6 @@ void Resource::setEmail( const QString email )
     changed();
 }
 
-void Resource::addWorkingHour(QTime from, QTime until) {
-    //kDebug();
-    m_workingHours.append(new QTime(from));
-    m_workingHours.append(new QTime(until));
-}
-
 Calendar *Resource::calendar(bool local) const {
     if (!local && project() != 0 && m_calendar == 0 ) {
         return project()->defaultCalendar();
@@ -418,6 +433,25 @@ bool Resource::load(KoXmlElement &element, XMLLoaderObject &status) {
         
     cost.normalRate = KGlobal::locale()->readMoney(element.attribute("normal-rate"));
     cost.overtimeRate = KGlobal::locale()->readMoney(element.attribute("overtime-rate"));
+    
+    KoXmlElement parent = element.namedItem( "external-appointments" ).toElement();
+    KoXmlElement e;
+    forEachElement( e, parent ) {
+        if ( e.nodeName() == "project" ) {
+            QString id = e.attribute( "id" );
+            if ( id.isEmpty() ) {
+		kError()<<"Missing project id";
+                continue;
+            }
+            clearExternalAppointments( id ); // in case...
+            AppointmentIntervalList lst;
+            lst.loadXML( e, status );
+            Appointment *a = new Appointment();
+            a->setIntervals( lst );
+            a->setAuxcilliaryInfo( e.attribute( "name", "Unknown" ) );
+            m_externalAppointments[ id ] = a;
+        }
+    }
     return true;
 }
 
@@ -438,9 +472,21 @@ void Resource::save(QDomElement &element) const {
     me.setAttribute("available-until", m_availableUntil.toString( KDateTime::ISODate ));
     me.setAttribute("normal-rate", KGlobal::locale()->formatMoney(cost.normalRate));
     me.setAttribute("overtime-rate", KGlobal::locale()->formatMoney(cost.overtimeRate));
+    
+    if ( ! m_externalAppointments.isEmpty() ) {
+        QDomElement e = me.ownerDocument().createElement("external-appointments");
+        me.appendChild(e);
+        foreach ( QString id, m_externalAppointments.uniqueKeys() ) {
+            QDomElement el = e.ownerDocument().createElement("project");
+            e.appendChild( el );
+            el.setAttribute( "id", id );
+            el.setAttribute( "name", m_externalAppointments[ id ]->auxcilliaryInfo() );
+            m_externalAppointments[ id ]->intervals().saveXML( el );
+        }
+    }
 }
 
-bool Resource::isAvailable(Task */*task*/) {
+bool Resource::isAvailable(Task * /*task*/) {
     bool busy = false;
 /*
     foreach (Appointment *a, m_appointments) {
@@ -460,7 +506,7 @@ QList<Appointment*> Resource::appointments( long id ) const {
     return s->appointments();
 }
 
-Appointment *Resource::findAppointment(Node */*node*/) {
+Appointment *Resource::findAppointment(Node * /*node*/) {
 /*
     foreach (Appointment *a, m_appointments) {
         if (a->node() == node)
@@ -568,31 +614,33 @@ KDateTime::Spec Resource::timeSpec() const
 }
 
 void Resource::makeAppointment(Schedule *node, const DateTime &from, const DateTime &end) {
+    KLocale *locale = KGlobal::locale();
     //kDebug()<<"node id="<<node->id()<<" mode="<<node->calculationMode()<<""<<from<<" -"<<end;
     if (!from.isValid() || !end.isValid()) {
-        kWarning()<<"Invalid time"<<endl;
+        m_currentSchedule->logWarning( i18n( "Make appointments: Invalid time" ) );
         return;
     }
     Calendar *cal = calendar();
     if (cal == 0) {
+        m_currentSchedule->logWarning( i18n( "Resource %1 has no calendar defined", m_name ) );
         return;
     }
     DateTime time = from;
     while (time < end) {
         //kDebug()<<time<<" to"<<end;
         if (!time.isValid() || !end.isValid()) {
-            kWarning()<<"Invalid time"<<endl;
+            m_currentSchedule->logWarning( i18n( "Make appointments: Invalid time" ) );
             return;
         }
         if (!cal->hasInterval(time, end, m_currentSchedule)) {
             //kDebug()<<time<<" to"<<end<<": No (more) interval(s)";
-            kWarning()<<m_name<<": Resource only partially available"<<endl;
+            m_currentSchedule->logWarning( i18n ( "Resource %1 only partially available", m_name ) );
             //node->resourceNotAvailable = true;
             return; // nothing more to do
         }
         DateTimeInterval i = cal->firstInterval(time, end, m_currentSchedule);
         if (!i.second.isValid()) {
-            kWarning()<<"Invalid interval: "<<time.toString()<<", "<<end.toString()<<endl;
+            m_currentSchedule->logWarning( i18n( "Make appointments: Invalid interval %1 to %2", locale->formatDateTime( time ), locale->formatDateTime( end ) ) );
             return;
         }
         if (time == i.second)
@@ -609,12 +657,13 @@ void Resource::makeAppointment(Schedule *node, const DateTime &from, const DateT
 }
 void Resource::makeAppointment(Schedule *node) {
     //kDebug()<<m_name<<": id="<<m_currentSchedule->id()<<" mode="<<m_currentSchedule->calculationMode()<<node->node()->name()<<": id="<<node->id()<<" mode="<<node->calculationMode()<<""<<node->startTime;
+    KLocale *locale = KGlobal::locale();
     if (!node->startTime.isValid()) {
-        kWarning()<<m_name<<": startTime invalid"<<endl;
+        m_currentSchedule->logWarning( i18n( "Make appointments: Node start time is not valid" ) );
         return;
     }
     if (!node->endTime.isValid()) {
-        kWarning()<<m_name<<": endTime invalid"<<endl;
+        m_currentSchedule->logWarning( i18n( "Make appointments: Node end time is not valid" ) );
         return;
     }
     node->resourceNotAvailable = false;
@@ -635,7 +684,7 @@ void Resource::makeAppointment(Schedule *node) {
         makeAppointment(node, from, end);
     }
     if (!cal) {
-        kWarning()<<m_name<<": No calendar defined"<<endl;
+        m_currentSchedule->logWarning( i18n( "Resource %1 has no calendar defined", m_name ) );
         return; 
     }
     //TODO: units and standard non-working days
@@ -643,13 +692,13 @@ void Resource::makeAppointment(Schedule *node) {
     DateTime end = node->endTime;
     time = availableAfter(time, end);
     if (!time.isValid()) {
-        kWarning()<<m_name<<": Resource not available (after="<<node->startTime.toString()<<", "<<end.toString()<<")"<<endl;
+        m_currentSchedule->logWarning( i18n( "Resource %1 not available in interval: %2 to %3", m_name, locale->formatDateTime( node->startTime ), locale->formatDateTime( end ) ) );
         node->resourceNotAvailable = true;
         return;
     }
     end = availableBefore(end, time);
     if (!end.isValid()) {
-        kWarning()<<m_name<<": Resource not available (before="<<node->endTime.toString()<<", "<<time.toString()<<")"<<endl;
+        m_currentSchedule->logWarning( i18n( "Resource %1 not available in interval: %2 to %3", m_name, locale->formatDateTime( time ), locale->formatDateTime( node->endTime ) ) );
         node->resourceNotAvailable = true;
         return;
     }
@@ -657,38 +706,53 @@ void Resource::makeAppointment(Schedule *node) {
     makeAppointment(node, time, end);
 }
 
+Duration Resource::effort( const DateTime &start, const Duration &duration, bool backward, bool *ok) const
+{
+    return effort( m_currentSchedule, start, duration, backward, ok );
+}
+
 // the amount of effort we can do within the duration
-Duration Resource::effort(const DateTime &start, const Duration &duration, bool backward, bool *ok) const {
-    //kDebug()<<m_name<<":"<<start.date().toString()<<" for duration"<<duration.toString(Duration::Format_Day);
+Duration Resource::effort(Schedule *sch, const DateTime &start, const Duration &duration, bool backward, bool *ok) const {
+    //kDebug()<<m_name<<":"<<start<<" for duration"<<duration.toString(Duration::Format_Day);
     bool sts=false;
     Duration e;
     if (duration == 0) {
-        kWarning()<<"zero duration"<<endl;
+        kWarning()<<"zero duration";
         return e;
     }
     Calendar *cal = calendar();
     if (cal == 0) {
-        kWarning()<<m_name<<": No calendar defined"<<endl;
+        sch->logWarning( i18n( "Resource %1 has no calendar defined", m_name ) );
         return e;
     }
     if (backward) {
         DateTime limit = start - duration;
+        if ( limit < m_availableFrom ) {
+            limit = m_availableFrom;
+        }
         DateTime t = availableBefore(start, limit);
         if (t.isValid()) {
             sts = true;
-            e = (cal->effort(limit, t, m_currentSchedule) * m_units)/100;
+            if ( t < limit ) sch->logDebug( " t < limit: t=" + t.toString() + " limit=" + limit.toString() );
+            e = (cal->effort(limit, t, sch) * m_units)/100;
         } else {
-            //kDebug()<<m_name<<": Not available (start="<<start<<","<<limit<<")";
+            //sch->logDebug( "Resource not available in interval:" + start.toString() + "," + limit.toString() );
         }
     } else {
         DateTime limit = start + duration;
+        if ( limit > m_availableUntil ) {
+            limit = m_availableUntil;
+        }
         DateTime t = availableAfter(start, limit);
         if (t.isValid()) {
             sts = true;
-            e = (cal->effort(t, limit, m_currentSchedule) * m_units)/100;
+            if ( t > limit ) sch->logDebug( "t > limit: t=" + t.toString() + " limit=" + limit.toString() );
+            e = (cal->effort(t, limit, sch) * m_units)/100;
+        } else {
+            //sch->logDebug( "Resource not available in interval:" + start.toString() + "," + limit.toString() );
         }
     }
-    //kDebug()<<start.toString()<<" e="<<e.toString(Duration::Format_Day)<<" ("<<m_units<<")";
+    //kDebug()<<start<<" e="<<e.toString(Duration::Format_Day)<<" ("<<m_units<<")";
     if (ok) *ok = sts;
     return e;
 }
@@ -720,9 +784,14 @@ DateTime Resource::availableAfter(const DateTime &time, const DateTime limit, Sc
     }
     Calendar *cal = calendar();
     if (cal == 0) {
+        if ( sch ) sch->logWarning( i18n( "Resource %1 has no calendar defined", m_name ) );
         return t;
     }
     t = m_availableFrom > time ? m_availableFrom : time;
+    if ( t >= lmt ) {
+        //kDebug()<<t<<lmt;
+        return DateTime();
+    }
     t = cal->firstAvailableAfter(t, lmt, sch);
     //kDebug()<<m_currentSchedule<<""<<m_name<<" id="<<m_currentSchedule->id()<<" mode="<<m_currentSchedule->calculationMode()<<" returns:"<<time.toString()<<"="<<t.toString()<<""<<lmt.toString();
     return t;
@@ -750,14 +819,14 @@ DateTime Resource::availableBefore(const DateTime &time, const DateTime limit, S
         return t;
     }
     if (!m_availableUntil.isValid()) {
-        kWarning()<<m_name<<": availabelUntil is invalid"<<endl;
+        sch->logDebug( "availabelUntil is invalid" );
         t = time;
     } else {
         t = m_availableUntil < time ? m_availableUntil : time;
     }
-    //kDebug()<<t<<","<<lmt;
+    if ( t < lmt ) sch->logDebug( "t < lmt: " + t.toString() + " < " + lmt.toString() );
     t = cal->firstAvailableBefore(t, lmt, sch );
-    //kDebug()<<m_name<<" id="<<m_currentSchedule->id()<<" mode="<<m_currentSchedule->calculationMode()<<" returns:"<<time<<"="<<t<<""<<lmt;
+    if ( t.isValid() && t < lmt ) sch->logDebug( " t < lmt: t=" + t.toString() + " lmt=" + lmt.toString() );
     return t;
 }
 
@@ -780,18 +849,30 @@ Calendar *Resource::findCalendar(const QString &id) const {
 }
 
 bool Resource::isOverbooked() const {
-    return isOverbooked(QDateTime(), QDateTime());
+    return isOverbooked(KDateTime(), KDateTime());
 }
 
 bool Resource::isOverbooked(const QDate &date) const {
-    return isOverbooked(QDateTime(date), QDateTime(date.addDays(1)));
+    return isOverbooked(KDateTime(date), KDateTime(date.addDays(1)));
 }
 
-bool Resource::isOverbooked(const QDateTime &start, const QDateTime &end) const {
+bool Resource::isOverbooked(const KDateTime &start, const KDateTime &end) const {
     //kDebug()<<m_name<<":"<<start.toString()<<" -"<<end.toString()<<" cs=("<<m_currentSchedule<<")";
     return m_currentSchedule ? m_currentSchedule->isOverbooked(start, end) : false;
 }
 
+Appointment Resource::appointmentIntervals( long id ) const {
+    Appointment a;
+    Schedule *s = findSchedule( id );
+    if ( s == 0 ) {
+        return a;
+    }
+    foreach (Appointment *app, static_cast<ResourceSchedule*>( s )->appointments()) {
+        a += *app;
+    }
+    return a;
+}
+
 Appointment Resource::appointmentIntervals() const {
     Appointment a;
     if (m_currentSchedule == 0)
@@ -816,6 +897,64 @@ void Resource::setProject( Project *project )
     m_project = project;
 }
 
+void Resource::addExternalAppointment( const QString &id, const QString &name, const DateTime &from, const DateTime &end, double load )
+{
+    Appointment *a = m_externalAppointments.value( id );
+    if ( a == 0 ) {
+        a = new Appointment();
+        a->setAuxcilliaryInfo( name );
+        a->addInterval( from, end, load );
+        //kDebug()<<m_name<<name<<"new appointment:"<<a<<from<<end<<load;
+        m_externalAppointments[ id ] = a;
+        int row = m_externalAppointments.keys().indexOf( id );
+        m_externalAppointments.remove( id );
+        emit externalAppointmentToBeAdded( this, row );
+        m_externalAppointments[ id ] = a;
+        emit externalAppointmentAdded( this, a );
+    } else {
+        //kDebug()<<m_name<<name<<"new interval:"<<a<<from<<end<<load;
+        a->addInterval( from, end, load );
+        emit externalAppointmentChanged( this, a );
+    }
+}
+
+void Resource::clearExternalAppointments()
+{
+    foreach ( QString id, m_externalAppointments.keys() ) {
+        clearExternalAppointments( id );
+    }
+}
+
+void Resource::clearExternalAppointments( const QString projectId )
+{
+    while ( m_externalAppointments.contains( projectId ) ) {
+        int row = m_externalAppointments.keys().indexOf( projectId );
+        emit externalAppointmentToBeRemoved( this, row );
+        Appointment *a  = m_externalAppointments.take( projectId );
+        emit externalAppointmentRemoved();
+        delete a;
+    }
+}
+
+AppointmentIntervalList Resource::externalAppointments( const QString &id )
+{
+    if ( ! m_externalAppointments.contains( id ) ) {
+        return AppointmentIntervalList();
+    }
+    return m_externalAppointments[ id ]->intervals();
+}
+
+AppointmentIntervalList Resource::externalAppointments() const
+{
+    //kDebug()<<m_externalAppointments;
+    Appointment app;
+    foreach ( Appointment *a, m_externalAppointments ) {
+        app += *a;
+    }
+    return app.intervals();
+}
+
+
 /////////   Risk   /////////
 Risk::Risk(Node *n, Resource *r, RiskType rt) {
     m_node=n;
@@ -883,6 +1022,7 @@ Schedule *ResourceRequest::resourceSchedule( Schedule *ns )
         s = r->createSchedule(ns->parent());
     }
     s->setCalculationMode( ns->calculationMode() );
+    static_cast<ResourceSchedule*>( s )->setNodeSchedule( ns );
     //kDebug()<<s->name()<<": id="<<s->id()<<" mode="<<s->calculationMode();
     return s;
 }
@@ -1084,10 +1224,12 @@ int ResourceGroupRequest::numDays(const DateTime &time, bool backward) const {
 
 Duration ResourceGroupRequest::duration(const DateTime &time, const Duration &_effort, Schedule *ns, bool backward) {
     //kDebug()<<"--->"<<(backward?"(B)":"(F)")<<m_group->name()<<""<<time.toString()<<": effort:"<<_effort.toString(Duration::Format_Day)<<" ("<<_effort.milliseconds()<<")";
+    KLocale *locale = KGlobal::locale();
     Duration e;
     if (_effort == Duration::zeroDuration) {
         return e;
     }
+    DateTime logtime = time;
     bool sts=true;
     bool match = false;
     DateTime start = time;
@@ -1112,78 +1254,101 @@ Duration ResourceGroupRequest::duration(const DateTime &time, const Duration &_e
             break;
         }
     }
-    //kDebug()<<"duration"<<(backward?"backward":"forward:")<<start.toString()<<" -"<<end.toString()<<" e="<<e.toString()<<" ("<<e.milliseconds()<<")  match="<<match<<" sts="<<sts;
-    d = Duration(0, 1, 0); // 1 hour
-    for (int i=0; !match && i < 24; ++i) {
-        // hours
-        end = end.addSecs(inc*60*60);
-        e1 = effort(start, d, ns, backward, &sts);
-        if (e + e1 < _effort) {
-            e += e1;
-            start = end;
-        } else if (e + e1 == _effort) {
-            e += e1;
-            match = true;
-        } else {
-            end = start;
-            break;
+    if ( ! match ) {
+        ns->logDebug( "Days: duration " + logtime.toString() + " - " + end.toString() + " e=" + e.toString() + " (" + (_effort - e).toString() + ')' );
+        
+        logtime = start;
+        d = Duration(0, 1, 0); // 1 hour
+        for (int i=0; !match && i < 24; ++i) {
+            // hours
+            end = end.addSecs(inc*60*60);
+            e1 = effort(start, d, ns, backward, &sts);
+            if (e + e1 < _effort) {
+                e += e1;
+                start = end;
+            } else if (e + e1 == _effort) {
+                e += e1;
+                match = true;
+            } else {
+                end = start;
+                break;
+            }
+            //kDebug()<<"duration(h)["<<i<<"]"<<(backward?"backward":"forward:")<<" time="<<start.time()<<" e="<<e.toString()<<" ("<<e.milliseconds()<<")";
         }
-        //kDebug()<<"duration(h)["<<i<<"]"<<(backward?"backward":"forward:")<<" time="<<start.time().toString()<<" e="<<e.toString()<<" ("<<e.milliseconds()<<")";
+        //kDebug()<<"duration"<<(backward?"backward":"forward:")<<start.toString()<<" e="<<e.toString()<<" ("<<e.milliseconds()<<")  match="<<match<<" sts="<<sts;
     }
-    //kDebug()<<"duration"<<(backward?"backward":"forward:")<<start.toString()<<" e="<<e.toString()<<" ("<<e.milliseconds()<<")  match="<<match<<" sts="<<sts;
-    d = Duration(0, 0, 1); // 1 minute
-    for (int i=0; !match && i < 60; ++i) {
-        //minutes
-        end = end.addSecs(inc*60);
-        e1 = effort(start, d, ns, backward, &sts);
-        if (e + e1 < _effort) {
-            e += e1;
-            start = end;
-        } else if (e + e1 == _effort) {
-            e += e1;
-            match = true;
-        } else if (e + e1 > _effort) {
-            end = start;
-            break;
+    if ( ! match ) {
+        ns->logDebug( "Hours: duration " + logtime.toString() + " - " + end.toString() + " e=" + e.toString() + " (" + (_effort - e).toString() + ')' );
+        
+        logtime = start;
+        d = Duration(0, 0, 1); // 1 minute
+        for (int i=0; !match && i < 60; ++i) {
+            //minutes
+            end = end.addSecs(inc*60);
+            e1 = effort(start, d, ns, backward, &sts);
+            if (e + e1 < _effort) {
+                e += e1;
+                start = end;
+            } else if (e + e1 == _effort) {
+                e += e1;
+                match = true;
+            } else if (e + e1 > _effort) {
+                end = start;
+                break;
+            }
+            //kDebug()<<"duration(m)"<<(backward?"backward":"forward:")<<"  time="<<start.time().toString()<<" e="<<e.toString()<<" ("<<e.milliseconds()<<")";
         }
-        //kDebug()<<"duration(m)"<<(backward?"backward":"forward:")<<"  time="<<start.time().toString()<<" e="<<e.toString()<<" ("<<e.milliseconds()<<")";
+        //kDebug()<<"duration"<<(backward?"backward":"forward:")<<"  start="<<start.toString()<<" e="<<e.toString()<<" match="<<match<<" sts="<<sts;
     }
-    //kDebug()<<"duration"<<(backward?"backward":"forward:")<<"  start="<<start.toString()<<" e="<<e.toString()<<" match="<<match<<" sts="<<sts;
-    d = Duration(0, 0, 0, 1); // 1 second
-    for (int i=0; !match && i < 60 && sts; ++i) {
-        //seconds
-        end = end.addSecs(inc);
-        e1 = effort(start, d, ns, backward, &sts);
-        if (e + e1 < _effort) {
-            e += e1;
-            start = end;
-        } else if (e + e1 == _effort) {
-            e += e1;
-            match = true;
-        } else if (e + e1 > _effort) {
-            end = start;
-            break;
+    if ( ! match ) {
+        ns->logDebug( "Minutes: duration " + logtime.toString() + " - " + end.toString() + " e=" + e.toString() + " (" + (_effort - e).toString() + ')' );
+        
+        logtime = start;
+        d = Duration(0, 0, 0, 1); // 1 second
+        for (int i=0; !match && i < 60 && sts; ++i) {
+            //seconds
+            end = end.addSecs(inc);
+            e1 = effort(start, d, ns, backward, &sts);
+            if (e + e1 < _effort) {
+                e += e1;
+                start = end;
+            } else if (e + e1 == _effort) {
+                e += e1;
+                match = true;
+            } else if (e + e1 > _effort) {
+                end = start;
+                break;
+            }
+            //kDebug()<<"duration(s)["<<i<<"]"<<(backward?"backward":"forward:")<<" time="<<start.time().toString()<<" e="<<e.toString()<<" ("<<e.milliseconds()<<")";
         }
-        //kDebug()<<"duration(s)["<<i<<"]"<<(backward?"backward":"forward:")<<" time="<<start.time().toString()<<" e="<<e.toString()<<" ("<<e.milliseconds()<<")";
     }
-    d = Duration(0, 0, 0, 0, 1); // 1 millisecond
-    for (int i=0; !match && i < 1000; ++i) {
-        //milliseconds
-        end.setTime(end.time().addMSecs(inc));
-        e1 = effort(start, d, ns, backward, &sts);
-        if (e + e1 < _effort) {
-            e += e1;
-            start = end;
-        } else if (e + e1 == _effort) {
-            e += e1;
-            match = true;
-        } else if (e + e1 > _effort) {
-            break;
+    if ( ! match ) {
+        ns->logDebug( "Seconds: duration " + logtime.toString() + " - " + end.toString() + " e=" + e.toString() + " (" + (_effort - e).toString() + ')' );
+        
+        d = Duration(0, 0, 0, 0, 1); // 1 millisecond
+        for (int i=0; !match && i < 1000; ++i) {
+            //milliseconds
+            end.setTime(end.time().addMSecs(inc));
+            e1 = effort(start, d, ns, backward, &sts);
+            if (e + e1 < _effort) {
+                e += e1;
+                start = end;
+            } else if (e + e1 == _effort) {
+                e += e1;
+                match = true;
+            } else if (e + e1 > _effort) {
+                break;
+            }
+            //kDebug()<<"duration(ms)["<<i<<"]"<<(backward?"backward":"forward:")<<" time="<<start.time().toString()<<" e="<<e.toString()<<" ("<<e.milliseconds()<<")";
         }
-        //kDebug()<<"duration(ms)["<<i<<"]"<<(backward?"backward":"forward:")<<" time="<<start.time().toString()<<" e="<<e.toString()<<" ("<<e.milliseconds()<<")";
     }
     if (!match) {
-        kError()<<(task()?task()->name():"No task")<<" "<<time.toString()<<": Could not match effort."<<" Want: "<<_effort.toString(Duration::Format_Day)<<" got: "<<e.toString(Duration::Format_Day)<<" sts="<<sts<<endl;
+        ns->logError( i18n( "Could not match effort. Want: %1 got: %2", _effort.toString( Duration::Format_Hour ), e.toString( Duration::Format_Hour ) ) );
+        foreach (ResourceRequest *r, m_resourceRequests) {
+            Resource *res = r->resource();
+            ns->logInfo( i18n( "Resource %1 available from %2 to %3", res->name(), locale->formatDateTime( res->availableFrom() ), locale->formatDateTime( res->availableUntil() ) ) );
+        }
+
     }
     DateTime t;
     if (e != Duration::zeroDuration) {
@@ -1514,9 +1679,17 @@ void Resource::printDebug(const QString& _indent)
     QString indent = _indent;
     kDebug()<<indent<<"  + Resource:"<<m_name<<" id="<<m_id/*<<" Overbooked="<<isOverbooked()*/;
     indent += "      ";
+    foreach (ResourceRequest *r, m_requests) {
+        r->printDebug(indent);
+    }
     foreach (Schedule *s, m_schedules) {
         s->printDebug(indent);
     }
+    kDebug()<<indent<<"  + External appointments:"<<m_externalAppointments.count();
+    foreach ( Appointment *a, m_externalAppointments ) {
+        kDebug()<<indent<<"  ! Appointment to:"<<a->auxcilliaryInfo();
+        a->printDebug(indent);
+    }
     indent += "  !";
 }
 
@@ -1543,6 +1716,7 @@ void ResourceRequestCollection::printDebug(const QString& indent)
         r->printDebug(indent+"  ");
     }
 }
+
 #endif
 
 }  //KPlato namespace
