@@ -22,11 +22,13 @@
 
 #include "kplatokernel_export.h"
 
+#include "kptglobal.h"
 #include "kptcalendar.h"
 #include "kpteffortcostmap.h"
 #include "kptresource.h"
 
 #include <QList>
+#include <QMap>
 #include <QString>
 
 //class KoXmlElement;
@@ -52,7 +54,7 @@ class XMLLoaderObject;
  * appointments.
  * There is one schedule per node and one per resource.
  * Schedules can be of type Expected, Optimistic or Pessimistic
- * refering to which estimate is used for the calculation.
+ * referring to which estimate is used for the calculation.
  * Schedule is subclassed into:
  * MainSchedule     Used by the main project.
  * NodeSchedule     Used by all other nodes (tasks).
@@ -85,21 +87,27 @@ public:
     virtual bool isDeleted() const;
     virtual void setDeleted( bool on );
     virtual bool recalculate() const { return m_parent == 0 ? false : m_parent->recalculate(); }
+    virtual DateTime recalculateFrom() const { return m_parent == 0 ? DateTime() : m_parent->recalculateFrom(); }
+
+    virtual long parentScheduleId() const { return m_parent == 0 ? NOTSCHEDULED : m_parent->parentScheduleId(); }
 
     virtual Resource *resource() const { return 0; }
     virtual Node *node() const { return 0; }
     
+    virtual bool isBaselined() const;
     virtual bool usePert() const;
     virtual bool allowOverbooking() const;
+    virtual bool checkExternalAppointments() const;
 
     bool isCritical() const { return positiveFloat == Duration::zeroDuration; }
 
-    virtual bool loadXML( const KoXmlElement &element );
+    virtual bool loadXML( const KoXmlElement &element, XMLLoaderObject &status );
     virtual void saveXML( QDomElement &element ) const;
     void saveCommonXML( QDomElement &element ) const;
     void saveAppointments( QDomElement &element ) const;
 
-    virtual TimeInterval available( const DateTime &date, const TimeInterval &interval ) const;
+    /// Return the effort available in the @p interval
+    virtual Duration effort( const DateTimeInterval &interval ) const;
     virtual DateTimeInterval available( const DateTimeInterval &interval ) const;
 
     enum CalculationMode { Scheduling, CalculateForward, CalculateBackward };
@@ -125,7 +133,7 @@ public:
     virtual Appointment appointmentIntervals( int which = Scheduling ) const;
 
     virtual bool isOverbooked() const { return false; }
-    virtual bool isOverbooked( const QDateTime & /*start*/, const QDateTime & /*end*/ ) const { return false; }
+    virtual bool isOverbooked( const KDateTime & /*start*/, const KDateTime & /*end*/ ) const { return false; }
     virtual QStringList overbookedResources() const;
 
     /// Return the resources that has appointments to this schedule
@@ -133,6 +141,7 @@ public:
     /// Return the resource names that has appointments to this schedule
     virtual QStringList resourceNameList() const;
     
+    virtual EffortCostMap bcwsPrDay() const;
     virtual EffortCostMap plannedEffortCostPrDay( const QDate &start, const QDate &end ) const;
 
     /// Returns the total planned effort for this schedule
@@ -146,7 +155,7 @@ public:
      * Planned cost is the sum total of all resources and other costs
      * planned for this node.
      */
-    virtual double plannedCost() const;
+    virtual EffortCost plannedCost() const;
 
     /// Planned cost on date
     virtual double plannedCost( const QDate &date ) const;
@@ -182,6 +191,35 @@ public:
     
     virtual ScheduleManager *manager() const { return 0; }
     
+    class Log {
+        public:
+            enum Type { Type_Debug = 0, Type_Info, Type_Warning, Type_Error };
+            Log() 
+                : node( 0 ), resource( 0 ), severity( 0 ), phase( -1 )
+            {}
+            Log( const Node *n, int sev, const QString &msg, int ph = -1 )
+                : node( n ), resource( 0 ), message( msg ), severity( sev ), phase( ph )
+            {}
+            Log( const Node *n, const Resource *r, int sev, const QString &msg, int ph = -1 )
+                : node( n ), resource( r ), message( msg ), severity( sev ), phase( ph )
+            {}
+            const Node *node;
+            const Resource *resource;
+            QString message;
+            int severity;
+            int phase;
+
+            QString formatMsg() const;
+    };
+    virtual void addLog( Log &log );
+    virtual void clearLogs() {};
+    virtual void logError( const QString &, int = -1 ) {}
+    virtual void logWarning( const QString &, int = -1 ) {}
+    virtual void logInfo( const QString &, int = -1 ) {}
+    virtual void logDebug( const QString &, int = -1 ) {}
+    
+    virtual void incProgress() { if ( m_parent ) m_parent->incProgress(); }
+
 protected:
     virtual void changed( Schedule * /*sch*/ ) {}
     
@@ -227,13 +265,13 @@ protected:
     DateTime lateFinish;
     /**  startTime is the scheduled start time.
       *  It depends on constraints (i.e. ASAP/ALAP) and resource availability.
-      *  It will always be later or equal to @ref earliestStart
+      *  It will always be later or equal to earliestStart
       */
     DateTime startTime;
     /**
       *  m_endTime is the scheduled finish time.
       *  It depends on constraints (i.e. ASAP/ALAP) and resource availability.
-      *  It will always be earlier or equal to @ref latestFinish
+      *  It will always be earlier or equal to latestFinish
       */
     DateTime endTime;
     /**
@@ -298,6 +336,11 @@ public:
     /// Return the resource names that has appointments to this schedule
     virtual QStringList resourceNameList() const;
 
+    virtual void logError( const QString &msg, int phase = -1 );
+    virtual void logWarning( const QString &msg, int phase = -1 );
+    virtual void logInfo( const QString &msg, int phase = -1 );
+    virtual void logDebug( const QString &, int = -1 );
+
 protected:
     void init();
 
@@ -328,18 +371,27 @@ public:
     virtual void takeAppointment( Appointment *appointment, int type = Scheduling );
 
     virtual bool isOverbooked() const;
-    virtual bool isOverbooked( const DateTime &start, const DateTime &end ) const;
+    virtual bool isOverbooked( const KDateTime &start, const KDateTime &end ) const;
 
     virtual Resource *resource() const { return m_resource; }
     virtual double normalRatePrHour() const;
 
-    virtual TimeInterval available( const DateTime &date, const TimeInterval &interval ) const;
+    /// Return the effort available in the @p interval
+    virtual Duration effort( const DateTimeInterval &interval ) const;
     virtual DateTimeInterval available( const DateTimeInterval &interval ) const;
     
+    virtual void logError( const QString &msg, int phase = -1 );
+    virtual void logWarning( const QString &msg, int phase = -1 );
+    virtual void logInfo( const QString &msg, int phase = -1 );
+    virtual void logDebug( const QString &, int = -1 );
+
+    void setNodeSchedule( const Schedule *sch ) { m_nodeSchedule = sch; }
+    
 private:
     Resource *m_resource;
     Schedule *m_parent;
-
+    const Schedule *m_nodeSchedule; // used during scheduling
+    
 #ifndef NDEBUG
 public:
     virtual void printDebug( const QString& ident );
@@ -358,7 +410,9 @@ public:
     ~MainSchedule();
     virtual bool isDeleted() const { return m_deleted; }
     
+    virtual bool isBaselined() const;
     virtual bool allowOverbooking() const;
+    virtual bool checkExternalAppointments() const;
     virtual bool usePert() const;
 
     virtual bool loadXML( const KoXmlElement &element, XMLLoaderObject &status );
@@ -367,6 +421,8 @@ public:
     void setManager( ScheduleManager *sm ) { m_manager = sm; }
     ScheduleManager *manager() const { return m_manager; }
     virtual bool recalculate() const;
+    virtual DateTime recalculateFrom() const;
+    virtual long parentScheduleId() const;
     
     DateTime calculateForward( int use );
     DateTime calculateBackward( int use );
@@ -405,6 +461,19 @@ public:
     }
     void addCriticalPathNode( Node *node );
     
+    QList<Schedule::Log> logs();
+    const QList<Schedule::Log> &logs() const;
+    virtual void addLog( Schedule::Log &log );
+    virtual void clearLogs() { m_log.clear(); m_logPhase.clear(); }
+    
+    void setPhaseName( int phase, const QString &name ) { m_logPhase[ phase ] = name; }
+    QString logPhase( int phase ) const { return m_logPhase.value( phase ); }
+    static QString logSeverity( int severity );
+    
+    virtual void incProgress();
+
+    QStringList logMessages() const;
+
 protected:
     virtual void changed( Schedule *sch );
 
@@ -424,6 +493,10 @@ private:
     QList<Node*> *m_currentCriticalPath;
     bool criticalPathListCached;
     
+    
+    QList<Schedule::Log> m_log;
+    QMap<int, QString> m_logPhase;
+    
 #ifndef NDEBUG
 public:
     virtual void printDebug( const QString& ident );
@@ -450,11 +523,13 @@ public:
     void setParentManager( ScheduleManager *sm );
     ScheduleManager *parentManager() const { return m_parent; }
     
-    long id() const { return m_expected == 0 ? -1 : m_expected->id(); }
+    long id() const { return m_expected == 0 ? NOTSCHEDULED : m_expected->id(); }
     
     int removeChild( const ScheduleManager *sm );
     void insertChild( ScheduleManager *sm, int index = -1 );
     const QList<ScheduleManager*> &children() const { return m_children; }
+    int childCount() const { return m_children.count(); }
+    ScheduleManager *childAt( int index ) const { return m_children.value( index ); }
     /// Return list of all child managers (also childrens children)
     QList<ScheduleManager*> allChildren() const;
     int indexOf( const ScheduleManager* child ) const;
@@ -463,8 +538,9 @@ public:
     
     bool recalculate() const { return m_recalculate; }
     void setRecalculate( bool on ) { m_recalculate = on; }
-    DateTime fromDateTime() { return KDateTime::currentLocalDateTime(); }
-    
+    DateTime recalculateFrom() const { return m_recalculateFrom; }
+    void setRecalculateFrom( const DateTime &dt ) { m_recalculateFrom = dt; }
+    long parentScheduleId() const { return m_parent == 0 ? NOTSCHEDULED : m_parent->id(); }
     void createSchedules();
     
     void setDeleted( bool on );
@@ -482,15 +558,24 @@ public:
 
     QStringList state() const;
 
+    void setBaselined( bool on );
+    bool isBaselined() const { return m_baselined; }
+    bool isChildBaselined() const;
     void setAllowOverbooking( bool on );
     bool allowOverbooking() const;
     
+    void setCheckExternalAppointments( bool on );
+    bool checkExternalAppointments() const;
+
     void setUsePert( bool on );
     bool usePert() const { return m_usePert; }
 
     void setCalculateAll( bool on );
     bool calculateAll() const { return m_calculateAll; }
 
+    void setSchedulingDirection( bool on );
+    bool schedulingDirection() const { return m_schedulingDirection; }
+
     QList<MainSchedule*> schedules() const;
     int numSchedules() const;
     int indexOf( const MainSchedule *sch ) const;
@@ -503,6 +588,8 @@ public:
             
     void scheduleChanged( MainSchedule *sch );
     
+    void incProgress();
+
 protected:
     MainSchedule *loadMainSchedule( KoXmlElement &element, XMLLoaderObject &status );
     
@@ -510,10 +597,14 @@ protected:
     Project &m_project;
     ScheduleManager *m_parent;
     QString m_name;
+    bool m_baselined;
     bool m_allowOverbooking;
+    bool m_checkExternalAppointments;
     bool m_calculateAll;
     bool m_usePert;
     bool m_recalculate;
+    DateTime m_recalculateFrom;
+    bool m_schedulingDirection;
     MainSchedule *m_expected;
     MainSchedule *m_optimistic;
     MainSchedule *m_pessimistic;
