@@ -3,7 +3,7 @@
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
-  License as published by the Free Software Foundation;
+  License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
@@ -27,22 +27,11 @@
 
 #include <KoDocument.h>
 
-#include <QAbstractItemModel>
-#include <QApplication>
-#include <QComboBox>
-#include <QDragEnterEvent>
-#include <QDragMoveEvent>
-#include <QDropEvent>
-#include <QMimeData>
-#include <QHeaderView>
-#include <QItemDelegate>
 #include <QItemSelectionModel>
-#include <QLineEdit>
-#include <QMap>
 #include <QModelIndex>
-#include <QStyleOptionViewItem>
 #include <QVBoxLayout>
 #include <QWidget>
+#include <QMenu>
 
 #include <kicon.h>
 #include <kaction.h>
@@ -59,42 +48,59 @@ namespace KPlato
 {
 
 //--------------------
-NodeTreeView::NodeTreeView( KoDocument *part, QWidget *parent )
+NodeTreeView::NodeTreeView( QWidget *parent )
     : DoubleTreeViewBase( parent )
 {
-    setModel( new NodeItemModel( part ) );
+    NodeItemModel *m = new NodeItemModel( this );
+    setModel( m );
     //setSelectionBehavior( QAbstractItemView::SelectItems );
     setSelectionMode( QAbstractItemView::ExtendedSelection );
     setSelectionBehavior( QAbstractItemView::SelectRows );
     
-    for ( int c = 0; c < model()->columnCount(); ++c ) {
-        QItemDelegate *delegate = model()->createDelegate( c, this );
-        if ( delegate ) {
-            setItemDelegateForColumn( c, delegate );
-        }
-    }
+    createItemDelegates( m );
+
+    connect( this, SIGNAL( dropAllowed( const QModelIndex&, int, QDragMoveEvent* ) ), SLOT(slotDropAllowed( const QModelIndex&, int, QDragMoveEvent* ) ) );
 }
 
+NodeItemModel *NodeTreeView::baseModel() const
+{
+    NodeSortFilterProxyModel *pr = proxyModel();
+    if ( pr ) {
+        return static_cast<NodeItemModel*>( pr->sourceModel() );
+    }
+    return static_cast<NodeItemModel*>( model() );
+}
+    
 void NodeTreeView::slotActivated( const QModelIndex index )
 {
     kDebug()<<index.column();
 }
 
+void NodeTreeView::slotDropAllowed( const QModelIndex &index, int dropIndicatorPosition, QDragMoveEvent *event )
+{
+    QModelIndex idx = index;
+    NodeSortFilterProxyModel *pr = proxyModel();
+    if ( pr ) {
+        idx = pr->mapToSource( index );
+    }
+    if ( baseModel()->dropAllowed( idx, dropIndicatorPosition, event->mimeData() ) ) {
+        event->accept();
+    }
+}
+
 //-----------------------------------
 TaskEditor::TaskEditor( KoDocument *part, QWidget *parent )
     : ViewBase( part, parent )
 {
-    setupGui();
-
+    kDebug()<<"----------------- Create TaskEditor ----------------------";
     QVBoxLayout * l = new QVBoxLayout( this );
     l->setMargin( 0 );
-    m_view = new NodeTreeView( part, this );
+    m_view = new NodeTreeView( this );
     l->addWidget( m_view );
-    m_view->setEditTriggers( m_view->editTriggers() | QAbstractItemView::EditKeyPressed );
-    QList<int> lst1; lst1 << 1 << -1;
-    QList<int> lst2; lst2 << 0 << 18 << -1;
-    m_view->hideColumns( lst1, lst2 );
+    kDebug()<<m_view->actionSplitView();
+    setupGui();
 
+    m_view->setEditTriggers( m_view->editTriggers() | QAbstractItemView::EditKeyPressed );
 
     m_view->setDragDropMode( QAbstractItemView::InternalMove );
     m_view->setDropIndicatorShown( true );
@@ -102,6 +108,37 @@ TaskEditor::TaskEditor( KoDocument *part, QWidget *parent )
     m_view->setAcceptDrops( true );
     m_view->setAcceptDropsOnView( true );
     
+    QList<int> lst1; lst1 << 1 << -1; // only display column 0 (NodeName) in left view
+    QList<int> show;
+    show << NodeModel::NodeType
+            << NodeModel::NodeResponsible
+            << NodeModel::NodeAllocation
+            << NodeModel::NodeEstimateType
+            << NodeModel::NodeEstimateCalendar
+            << NodeModel::NodeEstimate
+            << NodeModel::NodeOptimisticRatio
+            << NodeModel::NodePessimisticRatio
+            << NodeModel::NodeRisk
+            << NodeModel::NodeConstraint 
+            << NodeModel::NodeConstraintStart 
+            << NodeModel::NodeConstraintEnd 
+            << NodeModel::NodeRunningAccount 
+            << NodeModel::NodeStartupAccount 
+            << NodeModel::NodeStartupCost 
+            << NodeModel::NodeShutdownAccount 
+            << NodeModel::NodeShutdownCost 
+            << NodeModel::NodeDescription;
+
+    QList<int> lst2; 
+    for ( int i = 0; i < model()->columnCount(); ++i ) {
+        if ( ! show.contains( i ) ) {
+            lst2 << i;
+        }
+    }
+    m_view->hideColumns( lst1, lst2 );
+    m_view->masterView()->setDefaultColumns( QList<int>() << 0 );
+    m_view->slaveView()->setDefaultColumns( show );
+    
     connect( model(), SIGNAL( executeCommand( QUndoCommand* ) ), part, SLOT( addCommand( QUndoCommand* ) ) );
     
     connect( m_view, SIGNAL( currentChanged( const QModelIndex &, const QModelIndex & ) ), this, SLOT ( slotCurrentChanged( const QModelIndex &, const QModelIndex & ) ) );
@@ -111,6 +148,7 @@ TaskEditor::TaskEditor( KoDocument *part, QWidget *parent )
     connect( m_view, SIGNAL( contextMenuRequested( const QModelIndex&, const QPoint& ) ), SLOT( slotContextMenuRequested( const QModelIndex&, const QPoint& ) ) );
 
     connect( m_view, SIGNAL( headerContextMenuRequested( const QPoint& ) ), SLOT( slotHeaderContextMenuRequested( const QPoint& ) ) );
+    
 }
 
 void TaskEditor::updateReadWrite( bool rw )
@@ -162,7 +200,7 @@ QList<Node*> TaskEditor::selectedNodes() const {
         return lst;
     }
     foreach ( QModelIndex i, sm->selectedRows() ) {
-        Node * n = m_view->model()->node( i );
+        Node * n = m_view->baseModel()->node( i );
         if ( n != 0 && n->type() != Node::Type_Project ) {
             lst.append( n );
         }
@@ -180,7 +218,7 @@ Node *TaskEditor::selectedNode() const
 }
 
 Node *TaskEditor::currentNode() const {
-    Node * n = m_view->model()->node( m_view->selectionModel()->currentIndex() );
+    Node * n = m_view->baseModel()->node( m_view->selectionModel()->currentIndex() );
     if ( n == 0 || n->type() == Node::Type_Project ) {
         return 0;
     }
@@ -189,7 +227,7 @@ Node *TaskEditor::currentNode() const {
 
 void TaskEditor::slotContextMenuRequested( const QModelIndex& index, const QPoint& pos )
 {
-    Node *node = m_view->model()->node( index );
+    Node *node = m_view->baseModel()->node( index );
     if ( node == 0 ) {
         return;
     }
@@ -197,32 +235,32 @@ void TaskEditor::slotContextMenuRequested( const QModelIndex& index, const QPoin
     QString name;
     switch ( node->type() ) {
         case Node::Type_Task:
+            name = node->isScheduled( baseModel()->id() ) ? "task_popup" : "task_edit_popup";
+            break;
         case Node::Type_Milestone:
-            name = "task_popup";
+            name = node->isScheduled( baseModel()->id() ) ? "taskeditor_milestone_popup" : "task_edit_popup";
             break;
         case Node::Type_Summarytask:
             name = "summarytask_popup";
             break;
+        case Node::Type_Project:
+            break;
         default:
             name = "node_popup";
+            break;
+    }
+    if ( name.isEmpty() ) {
+        slotHeaderContextMenuRequested( pos );
+        return;
     }
     kDebug()<<name;
     emit requestPopupMenu( name, pos );
 }
 
-void TaskEditor::slotCurrentScheduleManagerChanged( ScheduleManager *sm )
+void TaskEditor::setScheduleManager( ScheduleManager *sm )
 {
     //kDebug()<<endl;
-    static_cast<NodeItemModel*>( m_view->model() )->setManager( sm );
-}
-
-void TaskEditor::slotHeaderContextMenuRequested( const QPoint &pos )
-{
-    kDebug();
-    QList<QAction*> lst = contextActionList();
-    if ( ! lst.isEmpty() ) {
-        QMenu::exec( lst, pos,  lst.first() );
-    }
+    m_view->baseModel()->setManager( sm );
 }
 
 void TaskEditor::slotEnableActions()
@@ -252,25 +290,27 @@ void TaskEditor::updateActionsEnabled( bool on )
 
 void TaskEditor::setupGui()
 {
-    KActionCollection *coll = actionCollection();
-    
     QString name = "taskeditor_add_list";
     actionAddTask  = new KAction(KIcon( "add_task" ), i18n("Add Task..."), this);
+    actionAddTask->setShortcut( KShortcut( Qt::CTRL + Qt::Key_I ) );
     actionCollection()->addAction("add_task", actionAddTask );
     connect( actionAddTask, SIGNAL( triggered( bool ) ), SLOT( slotAddTask() ) );
     addAction( name, actionAddTask );
     
     actionAddSubtask  = new KAction(KIcon( "add_sub_task" ), i18n("Add Sub-Task..."), this);
+    actionAddSubtask->setShortcut( KShortcut( Qt::CTRL + Qt::SHIFT + Qt::Key_I ) );
     actionCollection()->addAction("add_sub_task", actionAddSubtask );
     connect( actionAddSubtask, SIGNAL( triggered( bool ) ), SLOT( slotAddSubtask() ) );
     addAction( name, actionAddSubtask );
     
     actionAddMilestone  = new KAction(KIcon( "add_milestone" ), i18n("Add Milestone..."), this);
+    actionAddMilestone->setShortcut( KShortcut( Qt::CTRL + Qt::ALT + Qt::Key_I ) );
     actionCollection()->addAction("add_milestone", actionAddMilestone );
     connect( actionAddMilestone, SIGNAL( triggered( bool ) ), SLOT( slotAddMilestone() ) );
     addAction( name, actionAddMilestone );
     
     actionDeleteTask  = new KAction(KIcon( "edit-delete" ), i18n("Delete Task"), this);
+    actionDeleteTask->setShortcut( KShortcut( Qt::Key_Delete ) );
     actionCollection()->addAction("delete_task", actionDeleteTask );
     connect( actionDeleteTask, SIGNAL( triggered( bool ) ), SLOT( slotDeleteTask() ) );
     addAction( name, actionDeleteTask );
@@ -298,15 +338,23 @@ void TaskEditor::setupGui()
     addAction( name, actionMoveTaskDown );
 
     // Add the context menu actions for the view options
-    actionOptions = new KAction(KIcon("configure"), i18n("Configure..."), this);
-    connect(actionOptions, SIGNAL(triggered(bool) ), SLOT(slotOptions()));
-    addContextAction( actionOptions );
+    connect(m_view->actionSplitView(), SIGNAL(triggered(bool) ), SLOT(slotSplitView()));
+    addContextAction( m_view->actionSplitView() );
+
+    createOptionAction();
 }
 
+void TaskEditor::slotSplitView()
+{
+    kDebug();
+    m_view->setViewSplitMode( ! m_view->isViewSplit() );
+}
+
+
 void TaskEditor::slotOptions()
 {
     kDebug();
-    ItemViewSettupDialog dlg( m_view->slaveView() );
+    SplitItemViewSettupDialog dlg( m_view );
     dlg.exec();
 }
 
@@ -315,17 +363,20 @@ void TaskEditor::slotAddTask()
     kDebug();
     if ( selectedNodeCount() == 0 ) {
         // insert under main project
-        Task *t = m_view->project()->createTask( /*TODO part()->config().taskDefaults(),*/ m_view->project() );
-        edit( m_view->model()->insertSubtask( t, t->parentNode() ) );
-	kDebug()<<"test"<<t->type();
+        Task *t = m_view->project()->createTask( m_view->project()->taskDefaults(),  m_view->project() );
+        QModelIndex idx = m_view->baseModel()->insertSubtask( t, t->parentNode() );
+        Q_ASSERT( idx.isValid() );
+        edit( idx );
         return;
     }
     Node *sib = selectedNode();
     if ( sib == 0 ) {
         return;
     }
-    Task *t = m_view->project()->createTask( /*TODO part()->config().taskDefaults(),*/ sib->parentNode() );
-    edit( m_view->model()->insertTask( t, sib ) );
+    Task *t = m_view->project()->createTask( m_view->project()->taskDefaults(), sib->parentNode() );
+    QModelIndex idx = m_view->baseModel()->insertTask( t, sib );
+    Q_ASSERT( idx.isValid() );
+    edit( idx );
 }
 
 void TaskEditor::slotAddMilestone()
@@ -333,18 +384,22 @@ void TaskEditor::slotAddMilestone()
     kDebug();
     if ( selectedNodeCount() == 0 ) {
         // insert under main project
-        Task *t = m_view->project()->createTask( /*TODO part()->config().taskDefaults(),*/ m_view->project() );
-        t->estimate()->set( Duration::zeroDuration );
-        edit( m_view->model()->insertSubtask( t, t->parentNode() ) );
+        Task *t = m_view->project()->createTask( m_view->project() );
+        t->estimate()->clear();
+        QModelIndex idx = m_view->baseModel()->insertSubtask( t, t->parentNode() );
+        Q_ASSERT( idx.isValid() );
+        edit( idx );
         return;
     }
     Node *sib = selectedNode(); // sibling
     if ( sib == 0 ) {
         return;
     }
-    Task *t = m_view->project()->createTask( /*TODO part()->config().taskDefaults(),*/ sib->parentNode() );
-    t->estimate()->set( Duration::zeroDuration );
-    edit( m_view->model()->insertTask( t, sib ) );
+    Task *t = m_view->project()->createTask( sib->parentNode() );
+    t->estimate()->clear();
+    QModelIndex idx = m_view->baseModel()->insertTask( t, sib );
+    Q_ASSERT( idx.isValid() );
+    edit( idx );
 }
 
 void TaskEditor::slotAddSubtask()
@@ -354,8 +409,10 @@ void TaskEditor::slotAddSubtask()
     if ( parent == 0 ) {
         return;
     }
-    Task *t = m_view->project()->createTask( /*TODO part()->config().taskDefaults(),*/ parent );
-    edit( m_view->model()->insertSubtask( t, parent ) );
+    Task *t = m_view->project()->createTask( m_view->project()->taskDefaults(), parent );
+    QModelIndex idx = m_view->baseModel()->insertSubtask( t, parent );
+    Q_ASSERT( idx.isValid() );
+    edit( idx );
 }
 
 void TaskEditor::edit( QModelIndex i )
@@ -421,43 +478,90 @@ void TaskEditor::slotMoveTaskDown()
 
 bool TaskEditor::loadContext( const KoXmlElement &context )
 {
-    kDebug()<<endl;
-    return m_view->loadContext( context );
+    kDebug();
+    return m_view->loadContext( baseModel()->columnMap(), context );
 }
 
 void TaskEditor::saveContext( QDomElement &context ) const
 {
-   m_view->saveContext( context );
+    m_view->saveContext( baseModel()->columnMap(), context );
 }
 
+KoPrintJob *TaskEditor::createPrintJob()
+{
+    return m_view->createPrintJob( this );
+}
+
+
 //-----------------------------------
 TaskView::TaskView( KoDocument *part, QWidget *parent )
     : ViewBase( part, parent )
 {
-    setupGui();
-
     QVBoxLayout * l = new QVBoxLayout( this );
     l->setMargin( 0 );
-    m_view = new NodeTreeView( part, this );
+    m_view = new NodeTreeView( this );
+    NodeSortFilterProxyModel *p = new NodeSortFilterProxyModel( m_view->baseModel(), m_view );
+    m_view->setModel( p );
     l->addWidget( m_view );
-    updateReadWrite( false );
-    //m_view->setEditTriggers( m_view->editTriggers() | QAbstractItemView::EditKeyPressed );
-    QList<int> lst1; lst1 << 1 << -1;
-    QList<int> lst2; lst2 << 0 << 1 << 3 << 4 << 5 << 6 << 7 << 8 << 9 << 10 << 11 << 12 << 13 << 14 << 15 << 16 << 18 << 19 << 20 << 21 << 22 << 23 << 24 << 25 << 26 << 27 << 28 << 29 << 30 << 31 << 32 << 34 << 35 << 36 << 37 << 40 << -1;
-    m_view->hideColumns( lst1, lst2 );
-    
-    m_view->slaveView()->mapToSection( 38, 1 );
-    m_view->slaveView()->mapToSection( 39, 2 );
-    m_view->slaveView()->mapToSection( 2, 3 );
-    m_view->slaveView()->mapToSection( 33, 4 );
-    m_view->slaveView()->mapToSection( 17, 5 );
+    setupGui();
 
+    //m_view->setEditTriggers( m_view->editTriggers() | QAbstractItemView::EditKeyPressed );
     m_view->setDragDropMode( QAbstractItemView::InternalMove );
     m_view->setDropIndicatorShown( false );
     m_view->setDragEnabled ( true );
     m_view->setAcceptDrops( false );
     m_view->setAcceptDropsOnView( false );
+
+    QList<int> readonly; 
+    readonly << NodeModel::NodeName
+            << NodeModel::NodeResponsible
+            << NodeModel::NodeAllocation
+            << NodeModel::NodeEstimateType
+            << NodeModel::NodeEstimateCalendar
+            << NodeModel::NodeEstimate
+            << NodeModel::NodeOptimisticRatio
+            << NodeModel::NodePessimisticRatio
+            << NodeModel::NodeRisk
+            << NodeModel::NodeConstraint 
+            << NodeModel::NodeConstraintStart 
+            << NodeModel::NodeConstraintEnd 
+            << NodeModel::NodeRunningAccount 
+            << NodeModel::NodeStartupAccount 
+            << NodeModel::NodeStartupCost 
+            << NodeModel::NodeShutdownAccount 
+            << NodeModel::NodeShutdownCost 
+            << NodeModel::NodeDescription;
+    foreach ( int c, readonly ) {
+        m_view->baseModel()->setReadOnly( c, true );
+    }
+
+    QList<int> lst1; lst1 << 1 << -1;
+    QList<int> show; 
+    show << NodeModel::NodeStatus
+            << NodeModel::NodeCompleted
+            << NodeModel::NodeResponsible
+            << NodeModel::NodeAssigments
+            << NodeModel::NodePerformanceIndex
+            << NodeModel::NodeBCWS
+            << NodeModel::NodeBCWP
+            << NodeModel::NodeACWP
+            << NodeModel::NodeDescription;
     
+    for ( int s = 0; s < show.count(); ++s ) {
+        m_view->slaveView()->mapToSection( show[s], s );
+    }
+    QList<int> lst2; 
+    for ( int i = 0; i < m_view->model()->columnCount(); ++i ) {
+        if ( ! show.contains( i ) ) {
+            lst2 << i;
+        }
+    }
+    m_view->hideColumns( lst1, lst2 );
+    m_view->masterView()->setDefaultColumns( QList<int>() << 0 );
+    m_view->slaveView()->setDefaultColumns( show );
+    
+    connect( m_view->model(), SIGNAL( executeCommand( QUndoCommand* ) ), part, SLOT( addCommand( QUndoCommand* ) ) );
+
     connect( m_view, SIGNAL( currentChanged( const QModelIndex &, const QModelIndex & ) ), this, SLOT ( slotCurrentChanged( const QModelIndex &, const QModelIndex & ) ) );
 
     connect( m_view, SIGNAL( selectionChanged( const QModelIndexList ) ), this, SLOT ( slotSelectionChanged( const QModelIndexList ) ) );
@@ -467,10 +571,9 @@ TaskView::TaskView( KoDocument *part, QWidget *parent )
     connect( m_view, SIGNAL( headerContextMenuRequested( const QPoint& ) ), SLOT( slotHeaderContextMenuRequested( const QPoint& ) ) );
 }
 
-void TaskView::updateReadWrite( bool /*rw*/ )
+void TaskView::updateReadWrite( bool rw )
 {
-    // This is view only, don't allow editing
-    m_view->setReadWrite( false );
+    m_view->setReadWrite( rw );
 }
 
 void TaskView::draw( Project &project )
@@ -517,7 +620,7 @@ QList<Node*> TaskView::selectedNodes() const {
         return lst;
     }
     foreach ( QModelIndex i, sm->selectedRows() ) {
-        Node * n = m_view->model()->node( i );
+        Node * n = m_view->baseModel()->node( proxyModel()->mapToSource( i ) );
         if ( n != 0 && n->type() != Node::Type_Project ) {
             lst.append( n );
         }
@@ -535,7 +638,7 @@ Node *TaskView::selectedNode() const
 }
 
 Node *TaskView::currentNode() const {
-    Node * n = m_view->model()->node( m_view->selectionModel()->currentIndex() );
+    Node * n = m_view->baseModel()->node( proxyModel()->mapToSource( m_view->selectionModel()->currentIndex() ) );
     if ( n == 0 || n->type() == Node::Type_Project ) {
         return 0;
     }
@@ -545,13 +648,14 @@ Node *TaskView::currentNode() const {
 void TaskView::slotContextMenuRequested( const QModelIndex& index, const QPoint& pos )
 {
     QString name;
-    Node *node = m_view->model()->node( index );
+    Node *node = m_view->baseModel()->node( proxyModel()->mapToSource( index ) );
     if ( node ) {
         switch ( node->type() ) {
             case Node::Type_Task:
                 name = "taskview_popup";
                 break;
             case Node::Type_Milestone:
+                name = "taskview_milestone_popup";
                 break;
             case Node::Type_Summarytask:
                 break;
@@ -561,24 +665,16 @@ void TaskView::slotContextMenuRequested( const QModelIndex& index, const QPoint&
     } else kDebug()<<"No node: "<<index;
     if ( name.isEmpty() ) {
         kDebug()<<"No menu";
+        slotHeaderContextMenuRequested( pos );
         return;
     }
     emit requestPopupMenu( name, pos );
 }
 
-void TaskView::slotCurrentScheduleManagerChanged( ScheduleManager *sm )
+void TaskView::setScheduleManager( ScheduleManager *sm )
 {
     //kDebug()<<endl;
-    static_cast<NodeItemModel*>( m_view->model() )->setManager( sm );
-}
-
-void TaskView::slotHeaderContextMenuRequested( const QPoint &pos )
-{
-    kDebug();
-    QList<QAction*> lst = contextActionList();
-    if ( ! lst.isEmpty() ) {
-        QMenu::exec( lst, pos,  lst.first() );
-    }
+    m_view->baseModel()->setManager( sm );
 }
 
 void TaskView::slotEnableActions()
@@ -596,29 +692,40 @@ void TaskView::setupGui()
     KActionCollection *coll = actionCollection();
     
     // Add the context menu actions for the view options
-    actionOptions = new KAction(KIcon("configure"), i18n("Configure..."), this);
-    connect(actionOptions, SIGNAL(triggered(bool) ), SLOT(slotOptions()));
-    addContextAction( actionOptions );
+    connect(m_view->actionSplitView(), SIGNAL(triggered(bool) ), SLOT(slotSplitView()));
+    addContextAction( m_view->actionSplitView() );
+    
+    createOptionAction();
+}
+
+void TaskView::slotSplitView()
+{
+    kDebug();
+    m_view->setViewSplitMode( ! m_view->isViewSplit() );
 }
 
 void TaskView::slotOptions()
 {
     kDebug();
-    ItemViewSettupDialog dlg( m_view->slaveView() );
+    SplitItemViewSettupDialog dlg( m_view );
     dlg.exec();
 }
 
 bool TaskView::loadContext( const KoXmlElement &context )
 {
-    kDebug()<<endl;
-    return m_view->loadContext( context );
+    kDebug();
+    return m_view->loadContext( m_view->baseModel()->columnMap(), context );
 }
 
 void TaskView::saveContext( QDomElement &context ) const
 {
-    m_view->saveContext( context );
+    m_view->saveContext( m_view->baseModel()->columnMap(), context );
 }
 
+KoPrintJob *TaskView::createPrintJob()
+{
+    return m_view->createPrintJob( this );
+}
 
 } // namespace KPlato
 
