@@ -22,14 +22,21 @@
 
 #include <KoPAMasterPage.h>
 
-#include "KPrAnimationController.h"
+#include "KPrPageData.h"
 #include "KPrShapeAnimations.h"
 
-class KPrMasterPage : public KoPAMasterPage, public KPrAnimationController
+class KPrMasterPage : public KoPAMasterPage, public KPrPageData
 {
 public:
     explicit KPrMasterPage();
     virtual ~KPrMasterPage();
+
+    /**
+     * Get the page type used in the document
+     *
+     * @return KoPageApp::Slide
+     */
+    virtual KoPageApp::PageType pageType() const;
 };
 
 #endif /* KPRMASTERPAGE_H */
