@@ -1,6 +1,6 @@
 /* This file is part of the KDE project
  * Copyright (C) 2008 Fredy Yanardi <fyanardi@gmail.com>
- * Copyright (C) 2008-2009 Thorsten Zachmann <zachmann@kde.org>
+ * Copyright (C) 2008 Thorsten Zachmann <zachmann@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -25,14 +25,13 @@
 #include <KoImageCollection.h>
 #include <KoImageData.h>
 #include <KoShape.h>
-#include <KoShapeFactoryBase.h>
+#include <KoShapeFactory.h>
 #include <KoShapeLayer.h>
 #include <KoShapeRegistry.h>
 #include <KoShapeSavingContext.h>
 #include <KoUnit.h>
 #include <KoXmlNS.h>
 #include <KoXmlWriter.h>
-#include <KoPASavingContext.h>
 
 #include "KPrDocument.h"
 #include "KPrPage.h"
@@ -57,20 +56,19 @@ public:
 };
 
 KPrNotes::KPrNotes( KPrPage *page, KPrDocument * document )
-: KoPAPageBase()
-, m_page( page )
-, m_doc( document )
-, m_imageCollection( new KoImageCollection() )
+    : KoShapeContainer()
+    , m_page( page )
+    , m_imageCollection( new KoImageCollection() )
 {
     // add default layer
     KoShapeLayer* layer = new KoShapeLayer;
     addChild( layer );
 
     // All sizes and positions are hardcoded for now
-    KoShapeFactoryBase *factory = KoShapeRegistry::instance()->value("TextShapeID");
+    KoShapeFactory *factory = KoShapeRegistry::instance()->value("TextShapeID");
     Q_ASSERT(factory);
-    m_textShape = factory->createDefaultShape(m_doc->resourceManager());
-    m_textShape->setGeometryProtected(true);
+    m_textShape = factory->createDefaultShapeAndInit( document->dataCenterMap() );
+    m_textShape->setLocked(true);
     m_textShape->setAdditionalAttribute( "presentation:class", "notes" );
     m_pageLayout = KoPageLayout::standardLayout();
     m_textShape->setPosition(QPointF(62.22, 374.46));
@@ -78,8 +76,8 @@ KPrNotes::KPrNotes( KPrPage *page, KPrDocument * document )
 
     factory = KoShapeRegistry::instance()->value("PictureShape");
     Q_ASSERT(factory);
-    m_thumbnailShape = factory->createDefaultShape(m_doc->resourceManager());
-    m_thumbnailShape->setGeometryProtected(true);
+    m_thumbnailShape = factory->createDefaultShapeAndInit( document->dataCenterMap() );
+    m_thumbnailShape->setLocked(true);
     m_thumbnailShape->setAdditionalAttribute( "presentation:class", "page" );
     m_thumbnailShape->setPosition(QPointF(108.00, 60.18));
     m_thumbnailShape->setSize(QSizeF(396.28, 296.96));
@@ -111,8 +109,8 @@ void KPrNotes::saveOdf(KoShapeSavingContext &context) const
     writer.addAttribute("draw:page-number", static_cast<KoPASavingContext &>(context).page());
     writer.endElement(); // draw:page-thumbnail
 
-    KoShapeLayer* layer = dynamic_cast<KoShapeLayer*>( childShapes().last() );
-    foreach ( KoShape *shape, layer->childShapes() ) {
+    KoShapeLayer* layer = dynamic_cast<KoShapeLayer*>( iterator().last() );
+    foreach ( KoShape *shape, layer->iterator() ) {
         if ( shape != m_textShape && shape != m_thumbnailShape ) {
             shape->saveOdf( context );
         }
@@ -124,7 +122,7 @@ void KPrNotes::saveOdf(KoShapeSavingContext &context) const
 bool KPrNotes::loadOdf(const KoXmlElement &element, KoShapeLoadingContext &context)
 {
     KoXmlElement child;
-    KoShapeLayer* layer = dynamic_cast<KoShapeLayer*>( childShapes().last() );
+    KoShapeLayer* layer = dynamic_cast<KoShapeLayer*>( iterator().last() );
 
     forEachElement( child, element ) {
         if ( child.namespaceURI() != KoXmlNS::draw )
@@ -166,50 +164,9 @@ void KPrNotes::paintComponent(QPainter& painter, const KoViewConverter& converte
     Q_UNUSED(converter);
 }
 
-KoPageLayout & KPrNotes::pageLayout()
-{
-    return m_pageLayout;
-}
-
-const KoPageLayout & KPrNotes::pageLayout() const
-{
-    return m_pageLayout;
-}
-
-bool KPrNotes::displayMasterShapes()
-{
-    return false;
-}
-
-void KPrNotes::setDisplayMasterShapes( bool )
-{
-}
-
-bool KPrNotes::displayMasterBackground()
-{
-    return false;
-}
-
-void KPrNotes::setDisplayMasterBackground( bool )
-{
-}
-
-QPixmap KPrNotes::generateThumbnail( const QSize& )
-{
-    Q_ASSERT( 0 );
-    return QPixmap();
-}
-
 void KPrNotes::updatePageThumbnail()
 {
-    // set image at least to 150 dpi we might need more when printing
-    KoImageData *imageData = m_imageCollection->createImageData(m_doc->pageThumbnail(m_page, (m_thumbnailShape->size() * 150 / 72.).toSize()).toImage());
+    KoImageData *imageData = m_imageCollection->getImage(m_page->thumbnail( m_thumbnailShape->size().toSize() ).toImage());
     m_thumbnailShape->setUserData( imageData );
 }
 
-void KPrNotes::paintPage( QPainter & painter, KoZoomHandler & /*zoomHandler*/ )
-{
-    Q_UNUSED(painter);
-    // TODO implement when printing page with notes
-    Q_ASSERT( 0 );
-}
