@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
- * Copyright (C) 2008-2009 Thorsten Zachmann <zachmann@kde.org>
+ * Copyright (C) 2008 Thorsten Zachmann <zachmann@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -25,40 +25,34 @@
 #include <QPainter>
 
 #include <KoOdfLoadingContext.h>
-#include <KoProperties.h>
 #include <KoOdfStylesReader.h>
-#include <KoGenStyles.h>
-#include <KoXmlWriter.h>
 #include <KoParagraphStyle.h>
 #include <KoShape.h>
 #include <KoShapeLoadingContext.h>
-#include <KoShapeFactoryBase.h>
+#include <KoShapeFactory.h>
 #include <KoShapeRegistry.h>
-#include <KoShapeSavingContext.h>
 #include <KoTextShapeData.h>
 #include <KoTextDocument.h>
-#include <KoTextEditor.h>
-#include <KoTextDocumentLayout.h>
 #include <KoStyleManager.h>
 #include <KoXmlReader.h>
 #include <KoXmlNS.h>
 
+#include <kdebug.h>
+
 KPrPlaceholderTextStrategy::KPrPlaceholderTextStrategy( const QString & presentationClass )
 : KPrPlaceholderStrategy( presentationClass )
 , m_textShape( 0 )
-, m_paragraphStyle( 0 )
 {
 }
 
 KPrPlaceholderTextStrategy::~KPrPlaceholderTextStrategy()
 {
     delete m_textShape;
-    delete m_paragraphStyle;
 }
 
-KoShape *KPrPlaceholderTextStrategy::createShape(KoResourceManager *documentResources)
+KoShape * KPrPlaceholderTextStrategy::createShape( const QMap<QString, KoDataCenter *> & dataCenterMap )
 {
-    KoShape * shape = KPrPlaceholderStrategy::createShape(documentResources);
+    KoShape * shape = KPrPlaceholderStrategy::createShape( dataCenterMap );
     if ( m_textShape ) {
         KoTextShapeData * data = qobject_cast<KoTextShapeData*>( m_textShape->userData() );
         KoTextShapeData * newData = qobject_cast<KoTextShapeData*>( shape->userData() );
@@ -85,13 +79,6 @@ void KPrPlaceholderTextStrategy::paint( QPainter & painter, const KoViewConverte
     if ( m_textShape ) {
         painter.save();
         m_textShape->setSize( rect.size() );
-        // this code is needed to make sure the text of the textshape is layouted before it is painted
-        KoTextShapeData * shapeData = qobject_cast<KoTextShapeData*>( m_textShape->userData() );
-        QTextDocument * document = shapeData->document();
-        KoTextDocumentLayout * lay = qobject_cast<KoTextDocumentLayout*>( document->documentLayout() );
-        if ( lay ) {
-            lay->layout();
-        }
         m_textShape->paint( painter, converter );
 
         KoShape::applyConversion( painter, converter );
@@ -108,14 +95,6 @@ void KPrPlaceholderTextStrategy::paint( QPainter & painter, const KoViewConverte
 
 void KPrPlaceholderTextStrategy::saveOdf( KoShapeSavingContext & context )
 {
-    if ( m_paragraphStyle ) {
-        KoGenStyle style( KoGenStyle::ParagraphStyle, "presentation" );
-        m_paragraphStyle->saveOdf( style, context.mainStyles() );
-        QString displayName = m_paragraphStyle->name();
-        QString internalName = QString( QUrl::toPercentEncoding( displayName, "", " " ) ).replace( '%', '_' );
-        QString styleName = context.mainStyles().insert( style, internalName, KoGenStyles::DontAddNumberToName );
-        context.xmlWriter().addAttribute( "presentation:style-name", styleName );
-    }
     KPrPlaceholderStrategy::saveOdf( context );
 }
 
@@ -124,52 +103,25 @@ bool KPrPlaceholderTextStrategy::loadOdf( const KoXmlElement & element, KoShapeL
     KoStyleStack &styleStack = context.odfLoadingContext().styleStack();
     styleStack.save();
 
-    const KoXmlElement * style = 0;
+    context.odfLoadingContext().fillStyleStack( element, KoXmlNS::presentation, "style-name", "presentation" );
     if ( element.hasAttributeNS( KoXmlNS::draw, "text-style-name" ) ) {
-        context.odfLoadingContext().fillStyleStack( element, KoXmlNS::presentation, "style-name", "presentation" );
         const QString styleName = element.attributeNS( KoXmlNS::draw, "text-style-name", QString() );
-        style = context.odfLoadingContext().stylesReader().findStyle( styleName, "paragraph", context.odfLoadingContext().useStylesAutoStyles() );
-    }
-    else {
-        const QString styleName = element.attributeNS( KoXmlNS::presentation, "style-name", QString() );
-        style = context.odfLoadingContext().stylesReader().findStyle( styleName, "presentation", context.odfLoadingContext().useStylesAutoStyles() );
-    }
-
-    if ( style ) {
+        const KoXmlElement * style = context.odfLoadingContext().stylesReader().findStyle( styleName, "paragraph", context.odfLoadingContext().useStylesAutoStyles() );
+        Q_ASSERT( style );
         KoParagraphStyle paragraphStyle;
         paragraphStyle.loadOdf( style, context.odfLoadingContext() );
 
-        KoShapeFactoryBase *factory = KoShapeRegistry::instance()->value( "TextShapeID" );
+        KoShapeFactory *factory = KoShapeRegistry::instance()->value( "TextShapeID" );
         Q_ASSERT( factory );
-        m_textShape = factory->createDefaultShape(context.documentResourceManager());
+        m_textShape = factory->createDefaultShapeAndInit( context.dataCenterMap() );
 
-        KoTextShapeData * shapeData = qobject_cast<KoTextShapeData*>(  m_textShape->userData() );
-        shapeData->document()->setUndoRedoEnabled(false);
-
-        QTextDocument * document = shapeData->document();
+        QTextDocument * document = qobject_cast<KoTextShapeData*>( m_textShape->userData() )->document();
         QTextCursor cursor( document );
         QTextBlock block = cursor.block();
         paragraphStyle.applyStyle( block, false );
         cursor.insertText( text() );
-        shapeData->foul();
-        m_paragraphStyle = paragraphStyle.clone();
-        shapeData->document()->setUndoRedoEnabled(true);
     }
 
     styleStack.restore();
     return true;
 }
-
-void KPrPlaceholderTextStrategy::init(KoResourceManager *documentResources)
-{
-    KoShapeFactoryBase *factory = KoShapeRegistry::instance()->value( "TextShapeID" );
-    Q_ASSERT( factory );
-    KoProperties props;
-    props.setProperty("text", text());
-    m_textShape = factory->createShape(&props, documentResources);
-}
-
-KoShapeUserData * KPrPlaceholderTextStrategy::userData() const
-{
-    return m_textShape ? m_textShape->userData() : 0;
-}
