@@ -21,8 +21,11 @@
 
 #include <QKeyEvent>
 
+#include <KoShape.h>
+#include <KoShapeManager.h>
+#include <KoPointerEvent.h>
+#include <KoEventAction.h>
 #include <KoPACanvas.h>
-
 #include "KPrViewModePresentation.h"
 
 KPrPresentationTool::KPrPresentationTool( KPrViewModePresentation & viewMode )
@@ -35,9 +38,9 @@ KPrPresentationTool::~KPrPresentationTool()
 {
 }
 
-bool KPrPresentationTool::wantsAutoScroll() 
-{ 
-    return false; 
+bool KPrPresentationTool::wantsAutoScroll()
+{
+    return false;
 }
 void KPrPresentationTool::paint( QPainter &painter, const KoViewConverter &converter )
 {
@@ -45,6 +48,27 @@ void KPrPresentationTool::paint( QPainter &painter, const KoViewConverter &conve
 
 void KPrPresentationTool::mousePressEvent( KoPointerEvent *event )
 {
+    if ( event->button() & Qt::LeftButton ) {
+        event->accept();
+        finishEventActions();
+        KoShape * shapeClicked = m_canvas->shapeManager()->shapeAt( event->point );
+        if (shapeClicked) {
+            m_eventActions = shapeClicked->eventActions();
+            if ( m_eventActions.size() ) {
+                foreach ( KoEventAction * eventAction, m_eventActions ) {
+                    eventAction->execute( this );
+                }
+                // don't do next step if a action was executed
+                return;
+            }
+        }
+        m_viewMode.navigate( KPrAnimationDirector::NextStep );
+    }
+    else if ( event->button() & Qt::RightButton ) {
+        event->accept();
+        finishEventActions();
+        m_viewMode.navigate( KPrAnimationDirector::PreviousStep );
+    }
 }
 
 void KPrPresentationTool::mouseDoubleClickEvent( KoPointerEvent *event )
@@ -61,6 +85,7 @@ void KPrPresentationTool::mouseReleaseEvent( KoPointerEvent *event )
 
 void KPrPresentationTool::keyPressEvent( QKeyEvent *event )
 {
+    finishEventActions();
     event->accept();
 
     switch ( event->key() )
@@ -83,14 +108,14 @@ void KPrPresentationTool::keyPressEvent( QKeyEvent *event )
         case Qt::Key_Space:
             m_viewMode.navigate( KPrAnimationDirector::NextStep );
             break;
-        case Qt::Key_Down:            
+        case Qt::Key_Down:
         case Qt::Key_PageDown:
             m_viewMode.navigate( KPrAnimationDirector::NextPage );
             break;
         case Qt::Key_End:
             m_viewMode.navigate( KPrAnimationDirector::LastPage );
             break;
-        default:    
+        default:
             event->ignore();
             break;
     }
@@ -110,6 +135,14 @@ void KPrPresentationTool::activate( bool temporary )
 
 void KPrPresentationTool::deactivate()
 {
+    finishEventActions();
+}
+
+void KPrPresentationTool::finishEventActions()
+{
+    foreach ( KoEventAction * eventAction, m_eventActions ) {
+        eventAction->finish( this );
+    }
 }
 
 #include "KPrPresentationTool.moc"
