@@ -24,7 +24,7 @@
 
 #include "KPrPageEffectFactory.h"
 
-#include "KoXmlReaderForward.h"
+class KoXmlElement;
 class KPrPageEffect;
 
 class KPRESENTER_EXPORT KPrPageEffectRegistry : public KoGenericRegistry<KPrPageEffectFactory *>
