@@ -36,11 +36,10 @@
 #include <KoShapeController.h>
 #include <KoShapeManager.h>
 #include <KoEventAction.h>
-#include <KoEventActionFactoryBase.h>
+#include <KoEventActionFactory.h>
 #include <KoEventActionRegistry.h>
-#include <KPrEventActionWidget.h>
+#include <KoEventActionWidget.h>
 #include "KPrSoundData.h"
-#include <KPresenter.h>
 #include "KPrSoundCollection.h"
 #include "KPrView.h"
 #include "KPrPage.h"
@@ -53,15 +52,14 @@ KPrClickActionDocker::KPrClickActionDocker( QWidget* parent, Qt::WindowFlags fla
 , m_view( 0 )
 , m_soundCollection( 0 )
 {
-    setObjectName( "KPrClickActionDocker" );
     // setup widget layout
     QVBoxLayout* layout = new QVBoxLayout;
     m_cbPlaySound = new QComboBox();
     //layout->addWidget(m_cbPlaySound);
 
-    QList<KoEventActionFactoryBase *> factories = KoEventActionRegistry::instance()->presentationEventActions();
-    foreach ( KoEventActionFactoryBase * factory, factories ) {
-        QWidget * optionWidget = factory->createOptionWidget();
+    QList<KoEventActionFactory *> factories = KoEventActionRegistry::instance()->presentationEventActions();
+    foreach ( KoEventActionFactory * factory, factories ) {
+        KoEventActionWidget * optionWidget = factory->createOptionWidget();
         layout->addWidget( optionWidget );
         m_eventActionWidgets.insert( factory->id(), optionWidget );
         connect( optionWidget, SIGNAL( addCommand( QUndoCommand * ) ),
@@ -79,30 +77,24 @@ void KPrClickActionDocker::selectionChanged()
     KoShape *shape = selection->firstSelectedShape();
 
     if ( shape ) {
-        QSet<KoEventAction *> eventActions = shape->eventActions();
+        QList<KoEventAction *> eventActions = shape->eventActions();
         QMap<QString, KoEventAction*> eventActionMap;
         foreach ( KoEventAction * eventAction, eventActions ) {
             eventActionMap.insert( eventAction->id(), eventAction );
         }
 
-        QMap<QString, QWidget *>::const_iterator it(m_eventActionWidgets.constBegin());
+        QMap<QString, KoEventActionWidget *>::const_iterator it( m_eventActionWidgets.begin() );
 
-        for (; it != m_eventActionWidgets.constEnd(); ++it)  {
-            KPrEventActionWidget *actionWidget = dynamic_cast<KPrEventActionWidget*>(it.value());
-            if (actionWidget) {
-                // if it is not in the map a default value 0 pointer will be returned
-                KPrEventActionData data(shape, eventActionMap.value(it.key()), m_soundCollection);
-                actionWidget->setData(&data);
-            }
+        for ( ; it != m_eventActionWidgets.end(); ++it )  {
+            // if it is not in the map a default value 0 pointer will be returned
+            KPrEventActionData data( shape, eventActionMap.value( it.key() ), m_soundCollection );
+            it.value()->setData( &data );
         }
     }
     else {
-        foreach (QWidget *widget, m_eventActionWidgets) {
-            KPrEventActionWidget *actionWidget = dynamic_cast<KPrEventActionWidget*>(widget);
-            if (actionWidget) {
-                KPrEventActionData data(0, 0, m_soundCollection);
-                actionWidget->setData( &data );
-            }
+        foreach ( KoEventActionWidget * widget, m_eventActionWidgets ) {
+            KPrEventActionData data( 0, 0, m_soundCollection );
+            widget->setData( &data );
         }
     }
 }
@@ -122,10 +114,7 @@ void KPrClickActionDocker::setCanvas( KoCanvasBase *canvas )
 void KPrClickActionDocker::setView(KoPAView  *view )
 {
     m_view = view;
-    if (m_view->kopaDocument()->resourceManager()->hasResource(KPresenter::SoundCollection)) {
-        QVariant variant = m_view->kopaDocument()->resourceManager()->resource(KPresenter::SoundCollection);
-        m_soundCollection = variant.value<KPrSoundCollection*>();
-    }
+    m_soundCollection = dynamic_cast<KPrSoundCollection *>( m_view->kopaDocument()->dataCenterMap()["SoundCollection"] );
 
     setCanvas(view->kopaCanvas());
 }
