@@ -36,7 +36,7 @@ KoEventAction * KPrSoundEventActionFactory::createEventAction()
     return new KPrSoundEventAction();
 }
 
-QWidget *KPrSoundEventActionFactory::createOptionWidget()
+KoEventActionWidget * KPrSoundEventActionFactory::createOptionWidget()
 {
     return new KPrSoundEventActionWidget();
 }
