@@ -17,12 +17,13 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
+#include "kis_fill_painter.h"
+
 #include <stdlib.h>
 #include <string.h>
 #include <cfloat>
 #include <stack>
 
-#include "qbrush.h"
 #include "qfontinfo.h"
 #include "qfontmetrics.h"
 #include "qpen.h"
@@ -35,11 +36,15 @@
 #include <QRect>
 #include <QString>
 
-#include <kdebug.h>
 #include <klocale.h>
 
-#include "kis_brush.h"
-#include "kis_debug_areas.h"
+#include <KoProgressUpdater.h>
+
+#include "generator/kis_generator.h"
+#include "filter/kis_filter_configuration.h"
+#include "generator/kis_generator_registry.h"
+#include "kis_processing_information.h"
+#include "kis_debug.h"
 #include "kis_image.h"
 #include "kis_layer.h"
 #include "kis_paint_device.h"
@@ -50,29 +55,30 @@
 #include "kis_types.h"
 
 #include "kis_pixel_selection.h"
-#include "kis_fill_painter.h"
+
 #include "kis_iterators_pixel.h"
 #include "kis_iterator.h"
 #include "KoColor.h"
 #include "kis_selection.h"
 
-namespace {
+namespace
+{
 }
 
 KisFillPainter::KisFillPainter()
-    : KisPainter()
+        : KisPainter()
 {
     initFillPainter();
 }
 
 KisFillPainter::KisFillPainter(KisPaintDeviceSP device)
-    : KisPainter(device)
+        : KisPainter(device)
 {
     initFillPainter();
 }
 
 KisFillPainter::KisFillPainter(KisPaintDeviceSP device, KisSelectionSP selection)
-    : KisPainter( device, selection )
+        : KisPainter(device, selection)
 {
     initFillPainter();
 }
@@ -94,24 +100,25 @@ void KisFillPainter::fillRect(qint32 x1, qint32 y1, qint32 w, qint32 h, const Ko
         // Make sure we're in the right colorspace
 
         KoColor kc2(kc); // get rid of const
-        kc2.convertTo(m_device->colorSpace());
+        kc2.convertTo(device()->colorSpace());
         quint8 * data = kc2.data();
-        m_device->colorSpace()->setAlpha(data, opacity, 1);
+        device()->colorSpace()->setAlpha(data, opacity, 1);
 
-        m_device->fill(x1, y1, w, h, data);
+        device()->fill(x1, y1, w, h, data);
 
         addDirtyRect(QRect(x1, y1, w, h));
     }
 }
 
-void KisFillPainter::fillRect(qint32 x1, qint32 y1, qint32 w, qint32 h, KisPattern * pattern) {
+void KisFillPainter::fillRect(qint32 x1, qint32 y1, qint32 w, qint32 h, KisPattern * pattern)
+{
     if (!pattern) return;
     if (!pattern->valid()) return;
-    if (!m_device) return;
-    if ( w < 1 ) return;
-    if ( h < 1 ) return;
+    if (!device()) return;
+    if (w < 1) return;
+    if (h < 1) return;
 
-    KisPaintDeviceSP patternLayer = pattern->image(m_device->colorSpace());
+    KisPaintDeviceSP patternLayer = pattern->image(device()->colorSpace());
 
     int sx, sy, sw, sh;
 
@@ -137,47 +144,68 @@ void KisFillPainter::fillRect(qint32 x1, qint32 y1, qint32 w, qint32 h, KisPatte
         while (x < x1 + w) {
             sw = qMin((x1 + w) - x, pattern->width() - sx);
 
-            bitBlt(x, y, m_compositeOp, patternLayer, m_opacity, sx, sy, sw, sh);
+            bitBlt(x, y, compositeOp(), patternLayer, opacity(), sx, sy, sw, sh);
             x += sw; sx = 0;
         }
 
-        y+=sh; sy = 0;
+        y += sh; sy = 0;
     }
 
     addDirtyRect(QRect(x1, y1, w, h));
 }
 
+void KisFillPainter::fillRect(qint32 x1, qint32 y1, qint32 w, qint32 h, KisFilterConfiguration * generator)
+{
+    if (!generator) return;
+    KisGeneratorSP g = KisGeneratorRegistry::instance()->value(generator->name());
+    if (!generator) return;
+    if (!device()) return;
+    if (w < 1) return;
+    if (h < 1) return;
+
+    QRect tmpRc(x1, y1, w, h);
+
+    KisProcessingInformation dstCfg(device(), tmpRc.topLeft(), 0);
+
+    g->generate(dstCfg, tmpRc.size(), generator);
+
+    addDirtyRect(tmpRc);
+}
+
 // flood filling
 
-void KisFillPainter::fillColor(int startX, int startY, KisPaintDeviceSP projection) {
+void KisFillPainter::fillColor(int startX, int startY, KisPaintDeviceSP projection)
+{
     genericFillStart(startX, startY, projection);
 
     // Now create a layer and fill it
-    KisPaintDeviceSP filled = KisPaintDeviceSP(new KisPaintDevice(m_device->colorSpace(), "filled_fillcolor"));
+    KisPaintDeviceSP filled = KisPaintDeviceSP(new KisPaintDevice(device()->colorSpace(), "filled_fillcolor"));
     Q_CHECK_PTR(filled);
     KisFillPainter painter(filled);
-    painter.fillRect(0, 0, m_width, m_height, m_paintColor);
+    painter.fillRect(0, 0, m_width, m_height, paintColor());
     painter.end();
 
     genericFillEnd(filled);
 }
 
-void KisFillPainter::fillPattern(int startX, int startY, KisPaintDeviceSP projection) {
+void KisFillPainter::fillPattern(int startX, int startY, KisPaintDeviceSP projection)
+{
     genericFillStart(startX, startY, projection);
 
     // Now create a layer and fill it
-    KisPaintDeviceSP filled = KisPaintDeviceSP(new KisPaintDevice(m_device->colorSpace(), "filled_fillpattern"));
+    KisPaintDeviceSP filled = KisPaintDeviceSP(new KisPaintDevice(device()->colorSpace(), "filled_fillpattern"));
     Q_CHECK_PTR(filled);
     KisFillPainter painter(filled);
-    painter.fillRect(0, 0, m_width, m_height, m_pattern);
+    painter.fillRect(0, 0, m_width, m_height, pattern());
     painter.end();
 
     genericFillEnd(filled);
 }
 
-void KisFillPainter::genericFillStart(int startX, int startY, KisPaintDeviceSP projection) {
-    Q_ASSERT( m_width > 0 );
-    Q_ASSERT( m_height > 0 );
+void KisFillPainter::genericFillStart(int startX, int startY, KisPaintDeviceSP projection)
+{
+    Q_ASSERT(m_width > 0);
+    Q_ASSERT(m_height > 0);
 
     m_size = m_width * m_height;
 
@@ -185,19 +213,25 @@ void KisFillPainter::genericFillStart(int startX, int startY, KisPaintDeviceSP p
     m_fillSelection = createFloodSelection(startX, startY, projection);
 }
 
-void KisFillPainter::genericFillEnd(KisPaintDeviceSP filled) {
-    if ( m_progressUpdater && m_progressUpdater->interrupted() ) {
+void KisFillPainter::genericFillEnd(KisPaintDeviceSP filled)
+{
+    if (progressUpdater() && progressUpdater()->interrupted()) {
         m_width = m_height = -1;
         return;
     }
 
-    QRect rc = m_fillSelection->selectedExactRect();
+//  TODO: filling using the correct bound of the selection would be better, *but*
+//  the selection is limited to the exact bound of a layer, while in reality, we don't
+//  want that, since we want a transparent layer to be completely filled
+//     QRect rc = m_fillSelection->selectedExactRect();
 
     // Sets dirty!
-    bltMask(rc.x(), rc.y(), m_compositeOp, filled, m_fillSelection, m_opacity,
-                 rc.x(), rc.y(), rc.width(), rc.height());
+//     bltSelection(rc.x(), rc.y(), compositeOp(), filled, m_fillSelection, opacity(),
+//                  rc.x(), rc.y(), rc.width(), rc.height());
 
-    if ( m_progressUpdater ) m_progressUpdater->setProgress(100);
+    bltSelection(0, 0, compositeOp(), filled, m_fillSelection, opacity(), 0, 0, m_width, m_height);
+    
+    if (progressUpdater()) progressUpdater()->setProgress(100);
 
     m_width = m_height = -1;
 }
@@ -211,42 +245,45 @@ struct FillSegment {
 
 typedef enum { None = 0, Added = 1, Checked = 2 } Status;
 
-KisPixelSelectionSP KisFillPainter::createFloodSelection(int startX, int startY, KisPaintDeviceSP projection) {
-
+KisSelectionSP KisFillPainter::createFloodSelection(int startX, int startY, KisPaintDeviceSP projection)
+{
     if (m_width < 0 || m_height < 0) {
-        if (m_selection && m_careForSelection) {
-            QRect rc = m_selection->selectedExactRect();
+        if (selection() && m_careForSelection) {
+            QRect rc = selection()->selectedExactRect();
             m_width = rc.width() - (startX - rc.x());
             m_height = rc.height() - (startY - rc.y());
         }
     }
+    dbgImage << "Width: " << m_width << " Height: " << m_height;
     // Otherwise the width and height should have been set
-    Q_ASSERT( m_width > 0 && m_height > 0 );
+    Q_ASSERT(m_width > 0 && m_height > 0);
 
     // Don't try to fill if we start outside the borders, just return an empty 'fill'
     if (startX < 0 || startY < 0 || startX >= m_width || startY >= m_height)
-        return KisPixelSelectionSP(new KisPixelSelection(m_device));
+        return new KisSelection(device());
 
     KisPaintDeviceSP sourceDevice = KisPaintDeviceSP(0);
 
     // sample merged?
     if (m_sampleMerged) {
         if (!projection) {
-            return KisPixelSelectionSP(new KisPixelSelection(m_device));
+            return new KisSelection(device());
         }
         sourceDevice = projection;
     } else {
-        sourceDevice = m_device;
+        sourceDevice = device();
     }
 
     m_size = m_width * m_height;
 
-    KisPixelSelectionSP selection = KisPixelSelectionSP(new KisPixelSelection(m_device));
-    const KoColorSpace * colorSpace = selection->colorSpace();
+    KisSelectionSP selection = new KisSelection(device());
+    KisPixelSelectionSP pSel = selection->getOrCreatePixelSelection();
+
+    const KoColorSpace * colorSpace = pSel->colorSpace();
     const KoColorSpace * devColorSpace = sourceDevice->colorSpace();
 
     quint8* source = new quint8[sourceDevice->pixelSize()];
-    KisHLineConstIteratorPixel pixelIt = sourceDevice->createHLineConstIterator(startX, startY, startX+1);
+    KisHLineConstIteratorPixel pixelIt = sourceDevice->createHLineConstIterator(startX, startY, startX + 1);
 
     memcpy(source, pixelIt.rawData(), sourceDevice->pixelSize());
 
@@ -259,17 +296,17 @@ KisPixelSelectionSP KisFillPainter::createFloodSelection(int startX, int startY,
     memset(map, None, m_size * sizeof(Status));
 
     int progressPercent = 0; int pixelsDone = 0; int currentPercent = 0;
-    if (m_progressUpdater) m_progressUpdater->setProgress(0);
+    if (progressUpdater()) progressUpdater()->setProgress(0);
 
-    bool hasSelection = m_careForSelection && m_selection;
+    bool hasSelection = m_careForSelection && this->selection();
     KisSelectionSP srcSel = KisSelectionSP(0);
     if (hasSelection) {
-        srcSel = m_selection;
-        if(!srcSel->hasPixelSelection())
-            srcSel->setPixelSelection(KisPixelSelectionSP(new KisPixelSelection(sourceDevice)));
+        srcSel = this->selection();
+        if (!srcSel->hasPixelSelection())
+            srcSel->setPixelSelection(new KisPixelSelection(sourceDevice));
     }
 
-    while(!stack.empty()) {
+    while (!stack.empty()) {
         FillSegment* segment = stack.top();
         stack.pop();
         if (map[m_width * segment->y + segment->x] == Checked) {
@@ -288,26 +325,27 @@ KisPixelSelectionSP KisFillPainter::createFloodSelection(int startX, int startY,
         quint8 diff = devColorSpace->difference(source, pixelIt.rawData());
 
         if (diff >= m_threshold
-            || (hasSelection && srcSel->selected(pixelIt.x(), pixelIt.y()) == MIN_SELECTED)) {
+                || (hasSelection && srcSel->selected(pixelIt.x(), pixelIt.y()) == MIN_SELECTED)) {
             delete segment;
             continue;
         }
 
         // Here as well: start the iterator at (0,y)
-        KisHLineIteratorPixel selIt = selection->createHLineIterator(0, y, m_width);
+        KisHLineIteratorPixel selIt = pSel->createHLineIterator(0, y, m_width);
         selIt += x;
-        if (m_fuzzy)
-            colorSpace->fromQColor(Qt::white, MAX_SELECTED - diff, selIt.rawData());
-        else
-            colorSpace->fromQColor(Qt::white, MAX_SELECTED, selIt.rawData());
+        if (m_fuzzy) {
+            colorSpace->fromQColor(Qt::white, selIt.rawData());
+            colorSpace->setAlpha(selIt.rawData(), MAX_SELECTED - diff, 1);
+        } else
+            colorSpace->fromQColor(Qt::white, selIt.rawData());
 
         if (y > 0 && (map[m_width * (y - 1) + x] == None)) {
             map[m_width * (y - 1) + x] = Added;
-            stack.push(new FillSegment(x, y-1));
+            stack.push(new FillSegment(x, y - 1));
         }
         if (y < (m_height - 1) && (map[m_width * (y + 1) + x] == None)) {
             map[m_width * (y + 1) + x] = Added;
-            stack.push(new FillSegment(x, y+1));
+            stack.push(new FillSegment(x, y + 1));
         }
 
         ++pixelsDone;
@@ -319,27 +357,28 @@ KisPixelSelectionSP KisFillPainter::createFloodSelection(int startX, int startY,
         --x;
 
         // go to the left
-        while(!stop && x >= 0 && (map[m_width * y + x] != Checked) ) { // FIXME optimizeable?
+        while (!stop && x >= 0 && (map[m_width * y + x] != Checked)) { // FIXME optimizeable?
             map[m_width * y + x] = Checked;
             diff = devColorSpace->difference(source, pixelIt.rawData());
             if (diff >= m_threshold
-                || (hasSelection && srcSel->selected(pixelIt.x(), pixelIt.y()) == MIN_SELECTED)) {
+                    || (hasSelection && srcSel->selected(pixelIt.x(), pixelIt.y()) == MIN_SELECTED)) {
                 stop = true;
                 continue;
             }
 
-            if (m_fuzzy)
-                colorSpace->fromQColor(Qt::white, MAX_SELECTED - diff, selIt.rawData());
-            else
-                colorSpace->fromQColor(Qt::white, MAX_SELECTED, selIt.rawData());
+            if (m_fuzzy) {
+                colorSpace->fromQColor(Qt::white, selIt.rawData());
+                colorSpace->setAlpha(selIt.rawData(), MAX_SELECTED - diff, 1);
+            } else
+                colorSpace->fromQColor(Qt::white, selIt.rawData());
 
             if (y > 0 && (map[m_width * (y - 1) + x] == None)) {
                 map[m_width * (y - 1) + x] = Added;
-                stack.push(new FillSegment(x, y-1));
+                stack.push(new FillSegment(x, y - 1));
             }
             if (y < (m_height - 1) && (map[m_width * (y + 1) + x] == None)) {
                 map[m_width * (y + 1) + x] = Added;
-                stack.push(new FillSegment(x, y+1));
+                stack.push(new FillSegment(x, y + 1));
             }
             ++pixelsDone;
             --pixelIt;
@@ -355,31 +394,32 @@ KisPixelSelectionSP KisFillPainter::createFloodSelection(int startX, int startY,
 
         // and go to the right
         pixelIt = sourceDevice->createHLineIterator(x, y, m_width);
-        selIt = selection->createHLineIterator(x, y, m_width);
+        selIt = pSel->createHLineIterator(x, y, m_width);
 
         stop = false;
-        while(!stop && x < m_width && (map[m_width * y + x] != Checked) ) {
+        while (!stop && x < m_width && (map[m_width * y + x] != Checked)) {
             diff = devColorSpace->difference(source, pixelIt.rawData());
             map[m_width * y + x] = Checked;
 
             if (diff >= m_threshold
-                || (hasSelection && srcSel->selected(pixelIt.x(), pixelIt.y()) == MIN_SELECTED) ) {
+                    || (hasSelection && srcSel->selected(pixelIt.x(), pixelIt.y()) == MIN_SELECTED)) {
                 stop = true;
                 continue;
             }
 
-            if (m_fuzzy)
-                colorSpace->fromQColor(Qt::white, MAX_SELECTED - diff, selIt.rawData());
-            else
-                colorSpace->fromQColor(Qt::white, MAX_SELECTED, selIt.rawData());
+            if (m_fuzzy) {
+                colorSpace->fromQColor(Qt::white, selIt.rawData());
+                colorSpace->setAlpha(selIt.rawData(), MAX_SELECTED - diff, 1);
+            } else
+                colorSpace->fromQColor(Qt::white, selIt.rawData());
 
             if (y > 0 && (map[m_width * (y - 1) + x] == None)) {
                 map[m_width * (y - 1) + x] = Added;
-                stack.push(new FillSegment(x, y-1));
+                stack.push(new FillSegment(x, y - 1));
             }
             if (y < (m_height - 1) && (map[m_width * (y + 1) + x] == None)) {
                 map[m_width * (y + 1) + x] = Added;
-                stack.push(new FillSegment(x, y+1));
+                stack.push(new FillSegment(x, y + 1));
             }
             ++pixelsDone;
             ++pixelIt;
@@ -390,7 +430,7 @@ KisPixelSelectionSP KisFillPainter::createFloodSelection(int startX, int startY,
         if (m_size > 0) {
             progressPercent = (pixelsDone * 100) / m_size;
             if (progressPercent > currentPercent) {
-                if ( m_progressUpdater ) m_progressUpdater->setProgress(progressPercent);
+                if (progressUpdater()) progressUpdater()->setProgress(progressPercent);
                 currentPercent = progressPercent;
             }
         }
@@ -400,5 +440,6 @@ KisPixelSelectionSP KisFillPainter::createFloodSelection(int startX, int startY,
     delete[] map;
     delete[] source;
 
+    selection->updateProjection();
     return selection;
 }
