@@ -16,7 +16,9 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
+
 #include "kis_image.h"
+
 #include <config-endian.h> // WORDS_BIGENDIAN
 
 #include <stdlib.h>
@@ -35,46 +37,45 @@
 
 #include <klocale.h>
 
+#include "KoUnit.h"
 #include "KoColorSpaceRegistry.h"
 #include "KoColor.h"
 #include "KoColorConversionTransformation.h"
 #include "colorprofiles/KoIccColorProfile.h"
 
-#include "kis_action_recorder.h"
+#include "recorder/kis_action_recorder.h"
 #include "kis_adjustment_layer.h"
 #include "kis_annotation.h"
 #include "kis_change_profile_visitor.h"
 #include "kis_colorspace_convert_visitor.h"
 #include "kis_count_visitor.h"
-#include "kis_extent_visitor.h"
 #include "kis_filter_strategy.h"
 #include "kis_group_layer.h"
-#include "kis_image_commands.h"
+#include "commands/kis_image_commands.h"
 #include "kis_iterators_pixel.h"
 #include "kis_layer.h"
-#include "kis_merge_visitor.h"
 
-#include "kis_nameserver.h"
+#include "kis_name_server.h"
 #include "kis_paint_device.h"
 #include "kis_paint_device_action.h"
 #include "kis_paint_layer.h"
-#include "kis_paint_layer.h"
 #include "kis_painter.h"
 #include "kis_perspective_grid.h"
-#include "kis_projection.h"
 #include "kis_selection.h"
 #include "kis_shear_visitor.h"
 #include "kis_transaction.h"
 #include "kis_transform_visitor.h"
 #include "kis_types.h"
 #include "kis_crop_visitor.h"
+#include "kis_projection_update_strategy.h"
+#include "kis_meta_data_merge_strategy.h"
 
-class KisImage::KisImagePrivate {
+class KisImage::KisImagePrivate
+{
 public:
     KoColor backgroundColor;
     quint32 lockCount;
     bool sizeChangedWhileLocked;
-    bool selectionChangedWhileLocked;
     KisPerspectiveGrid* perspectiveGrid;
 
     KUrl uri;
@@ -92,40 +93,35 @@ public:
     KisGroupLayerSP rootLayer; // The layers are contained in here
     QList<KisLayer*> dirtyLayers; // for thumbnails
 
-    KisProjectionSP projection;
-
     KisNameServer *nserver;
     KisUndoAdapter *adapter;
     KisActionRecorder *recorder;
 
     vKisAnnotationSP annotations;
 
-    KisSelectionSP globalSelection; // XXX_SELECTION: Use the selection from the root layer for this
+    KisSelectionSP globalSelection;
+    KisSelectionSP deselectedGlobalSelection;
 
 };
-
-
-
-
 KisImage::KisImage(KisUndoAdapter *adapter, qint32 width, qint32 height, const KoColorSpace * colorSpace, const QString& name)
-    : QObject(0)
-    , KisShared()
-    , m_d( new KisImagePrivate() )
+        : QObject(0)
+        , KisShared()
+        , m_d(new KisImagePrivate())
 {
     setObjectName(name);
     init(adapter, width, height, colorSpace);
 }
 
 KisImage::KisImage(const KisImage& rhs)
-    : QObject()
-    , KisNodeFacade( rhs )
-    , KisNodeGraphListener( rhs )
-    , KisShared( rhs )
-    , m_d( new KisImagePrivate() )
+        : QObject()
+        , KisNodeFacade(rhs)
+        , KisNodeGraphListener(rhs)
+        , KisShared(rhs)
+        , m_d(new KisImagePrivate())
 {
     if (this != &rhs) {
 
-        if ( rhs.m_d->perspectiveGrid )
+        if (rhs.m_d->perspectiveGrid)
             m_d->perspectiveGrid = new KisPerspectiveGrid(*rhs.m_d->perspectiveGrid);
         else
             m_d->perspectiveGrid = 0;
@@ -139,20 +135,14 @@ KisImage::KisImage(const KisImage& rhs)
         m_d->colorSpace = rhs.m_d->colorSpace;
         m_d->adapter = rhs.m_d->adapter;
         m_d->globalSelection = 0;
-        m_d->projection = new KisProjection( this );
-        setRootLayer( static_cast<KisGroupLayer*>(rhs.m_d->rootLayer->clone().data()) );
+        m_d->deselectedGlobalSelection = 0;
+        setRootLayer(static_cast<KisGroupLayer*>(rhs.m_d->rootLayer->clone().data()));
         m_d->annotations = rhs.m_d->annotations; // XXX the annotations would probably need to be deep-copied
-
-
-
-        m_d->nserver = new KisNameServer(rhs.m_d->nserver->currentSeed() + 1);
+        m_d->nserver = new KisNameServer( *rhs.m_d->nserver );
         Q_CHECK_PTR(m_d->nserver);
 
     }
 }
-
-
-
 KisImage::~KisImage()
 {
     delete m_d->perspectiveGrid;
@@ -161,19 +151,19 @@ KisImage::~KisImage()
 }
 
 
-void KisImage::aboutToAddANode( KisNode *parent, int index )
+void KisImage::aboutToAddANode(KisNode *parent, int index)
 {
-    emit sigAboutToAddANode( parent, index );
+    emit sigAboutToAddANode(parent, index);
 }
 
-void KisImage::nodeHasBeenAdded( KisNode *parent, int index )
+void KisImage::nodeHasBeenAdded(KisNode *parent, int index)
 {
     // XXX: Temporarily for compatibility
 
-    KisLayer * layer = dynamic_cast<KisLayer*>( parent->at( index ).data() );
-    if ( layer ) {
+    KisLayer * layer = dynamic_cast<KisLayer*>(parent->at(index).data());
+    if (layer) {
 
-        KisPaintLayerSP player = KisPaintLayerSP( dynamic_cast<KisPaintLayer*>( layer ) );
+        KisPaintLayerSP player = KisPaintLayerSP(dynamic_cast<KisPaintLayer*>(layer));
         if (!player.isNull()) {
             // XXX: This should also be done whenever a layer grows!
             QList<KisPaintDeviceAction *> actions =
@@ -186,59 +176,59 @@ void KisImage::nodeHasBeenAdded( KisNode *parent, int index )
 
         // The addition of temporary layers is not interesting
         KoProperties props;
-        props.setProperty( "temporary", false );
+        props.setProperty("temporary", false);
 
-        if ( !layer->check( props ) ) {
-            emit sigLayerMoved( layer );
+        if (!layer->check(props)) {
+            emit sigLayerMoved(layer);
         }
     }
-    emit sigNodeHasBeenAdded( parent, index );
+    emit sigNodeHasBeenAdded(parent, index);
 }
 
-void KisImage::aboutToRemoveANode( KisNode *parent, int index )
+void KisImage::aboutToRemoveANode(KisNode *parent, int index)
 {
-    emit sigAboutToRemoveANode( parent, index );
+    emit sigAboutToRemoveANode(parent, index);
 }
 
-void KisImage::nodeHasBeenRemoved( KisNode *parent, int index )
+void KisImage::nodeHasBeenRemoved(KisNode *parent, int index)
 {
     // XXX: Temporarily for compatibility
-    emit sigNodeHasBeenRemoved( parent, index );
+    emit sigNodeHasBeenRemoved(parent, index);
 
-    KisLayer * l = dynamic_cast<KisLayer*>( parent->at( index ).data() );
-    if ( l ) {
+    KisLayer * l = dynamic_cast<KisLayer*>(parent->at(index).data());
+    if (l) {
         // The removal of temporary layers is not interesting
         KoProperties props;
-        props.setProperty( "temporary", false );
+        props.setProperty("temporary", false);
 
-        if ( !l->check( props ) ) {
-            emit sigLayerRemoved( l );
+        if (!l->check(props)) {
+            emit sigLayerRemoved(l);
         }
     }
 }
 
-void KisImage::aboutToMoveNode( KisNode * parent, int oldIndex, int newIndex )
+void KisImage::aboutToMoveNode(KisNode * parent, int oldIndex, int newIndex)
 {
-    Q_UNUSED( parent );
-    Q_UNUSED( oldIndex );
-    Q_UNUSED( newIndex );
+    Q_UNUSED(parent);
+    Q_UNUSED(oldIndex);
+    Q_UNUSED(newIndex);
 }
 
-void KisImage::nodeHasBeenMoved( KisNode * parent, int oldIndex, int newIndex )
+void KisImage::nodeHasBeenMoved(KisNode * parent, int oldIndex, int newIndex)
 {
     Q_UNUSED(oldIndex);
 
-    KisLayer * l = dynamic_cast<KisLayer*>( parent->at( newIndex ).data() );
-    if ( l ) {
+    KisLayer * l = dynamic_cast<KisLayer*>(parent->at(newIndex).data());
+    if (l) {
 
         // The moving of temporary layers is not interesting
 
         KoProperties props;
-        props.setProperty( "temporary", false );
+        props.setProperty("temporary", false);
 
-        if ( !l->check( props ) ) {
+        if (!l->check(props)) {
 
-            emit sigLayerMoved( l );
+            emit sigLayerMoved(l);
         }
     }
 }
@@ -249,18 +239,30 @@ KisSelectionSP KisImage::globalSelection() const
     return m_d->globalSelection;
 }
 
-void KisImage::setGlobalSelection( KisSelectionSP globalSelection )
+void KisImage::setGlobalSelection(KisSelectionSP globalSelection)
 {
-    if ( globalSelection == 0 )
-        m_d->globalSelection = new KisSelection( m_d->rootLayer->projection() );
+    if (globalSelection == 0)
+        m_d->globalSelection = new KisSelection(m_d->rootLayer->projection());
     else
         m_d->globalSelection = globalSelection;
 }
 
-void KisImage::removeGlobalSelection() {
+void KisImage::removeGlobalSelection()
+{
     m_d->globalSelection = 0;
 }
 
+KisSelectionSP KisImage::deleselectedGlobalSelection()
+{
+    return m_d->deselectedGlobalSelection;
+}
+
+void KisImage::setDeleselectedGlobalSelection(KisSelectionSP selection)
+{
+    m_d->deselectedGlobalSelection = selection;
+}
+
+
 KoColor KisImage::backgroundColor() const
 {
     return m_d->backgroundColor;
@@ -296,7 +298,6 @@ void KisImage::init(KisUndoAdapter *adapter, qint32 width, qint32 height, const
     m_d->backgroundColor = KoColor(Qt::white, colorSpace);
     m_d->lockCount = 0;
     m_d->sizeChangedWhileLocked = false;
-    m_d->selectionChangedWhileLocked = false;
     m_d->perspectiveGrid = 0;
 
     m_d->adapter = adapter;
@@ -304,8 +305,8 @@ void KisImage::init(KisUndoAdapter *adapter, qint32 width, qint32 height, const
     m_d->nserver = new KisNameServer(1);
 
     m_d->colorSpace = colorSpace;
-    m_d->projection = new KisProjection( this );
-    setRootLayer( new KisGroupLayer(this, "root", OPACITY_OPAQUE) );
+
+    setRootLayer(new KisGroupLayer(this, "root", OPACITY_OPAQUE));
 
     m_d->xres = 1.0;
     m_d->yres = 1.0;
@@ -324,11 +325,10 @@ bool KisImage::locked() const
 void KisImage::lock()
 {
     if (!locked()) {
-        if (m_d->projection)
-            m_d->projection->lock();
+        if (m_d->rootLayer)
+            m_d->rootLayer->updateStrategy()->lock();
         m_d->sizeChangedWhileLocked = false;
-        m_d->selectionChangedWhileLocked = false;
-        blockSignals( true );
+//         blockSignals(true);
     }
     m_d->lockCount++;
 }
@@ -341,18 +341,14 @@ void KisImage::unlock()
         m_d->lockCount--;
 
         if (m_d->lockCount == 0) {
-            blockSignals( false );
+//             blockSignals(false);
 
             if (m_d->sizeChangedWhileLocked) {
                 emit sigSizeChanged(m_d->width, m_d->height);
             }
 
-            if (m_d->selectionChangedWhileLocked) {
-                emit sigActiveSelectionChanged(KisImageSP(this));
-            }
-
-            if (m_d->projection)
-                m_d->projection->unlock();
+            if (m_d->rootLayer)
+                m_d->rootLayer->updateStrategy()->unlock();
 
         }
     }
@@ -363,11 +359,10 @@ void KisImage::notifyLayerUpdated(KisLayerSP layer)
     // Add the layer to the list of layers that need to be
     // rescanned for the thumbnails in the layerbox
     KisLayer *l = layer.data();
-    while( l )
-    {
-        if( !m_d->dirtyLayers.contains( l ) )
-            m_d->dirtyLayers.append( l );
-        l = dynamic_cast<KisLayer*>( l->parent().data() );
+    while (l) {
+        if (!m_d->dirtyLayers.contains(l))
+            m_d->dirtyLayers.append(l);
+        l = dynamic_cast<KisLayer*>(l->parent().data());
     }
 }
 
@@ -429,14 +424,14 @@ void KisImage::scale(double sx, double sy, KoUpdater *progress, KisFilterStrateg
     list << "KisLayer";
 
     KisCountVisitor visitor(list, KoProperties());
-    m_d->rootLayer->accept( visitor );
+    m_d->rootLayer->accept(visitor);
 
     if (visitor.count() == 0) return; // Nothing to scale
 
     // New image size. XXX: Pass along to discourage rounding errors?
     qint32 w, h;
-    w = (qint32)(( width() * sx) + 0.5);
-    h = (qint32)(( height() * sy) + 0.5);
+    w = (qint32)((width() * sx) + 0.5);
+    h = (qint32)((height() * sy) + 0.5);
 
     if (w != width() || h != height()) {
 
@@ -448,7 +443,7 @@ void KisImage::scale(double sx, double sy, KoUpdater *progress, KisFilterStrateg
         }
 
         {
-            KisTransformVisitor visitor (KisImageSP(this), sx, sy, 0.0, 0.0, 0.0, 0, 0, progress, filterStrategy);
+            KisTransformVisitor visitor(KisImageSP(this), sx, sy, 0.0, 0.0, 0.0, 0, 0, progress, filterStrategy);
             m_d->rootLayer->accept(visitor);
         }
 
@@ -469,19 +464,16 @@ void KisImage::scale(double sx, double sy, KoUpdater *progress, KisFilterStrateg
         }
     }
 }
-
-
-
 void KisImage::rotate(double radians, KoUpdater *progress)
 {
     lock();
 
     qint32 w = width();
     qint32 h = height();
-    qint32 tx = qint32((w*cos(radians) - h*sin(radians) - w) / 2 + 0.5);
-    qint32 ty = qint32((h*cos(radians) + w*sin(radians) - h) / 2 + 0.5);
-    w = (qint32)(width()*QABS(cos(radians)) + height()*QABS(sin(radians)) + 0.5);
-    h = (qint32)(height()*QABS(cos(radians)) + width()*QABS(sin(radians)) + 0.5);
+    qint32 tx = qint32((w * cos(radians) - h * sin(radians) - w) / 2 + 0.5);
+    qint32 ty = qint32((h * cos(radians) + w * sin(radians) - h) / 2 + 0.5);
+    w = (qint32)(width() * QABS(cos(radians)) + height() * QABS(sin(radians)) + 0.5);
+    h = (qint32)(height() * QABS(cos(radians)) + width() * QABS(sin(radians)) + 0.5);
 
     tx -= (w - width()) / 2;
     ty -= (h - height()) / 2;
@@ -492,7 +484,7 @@ void KisImage::rotate(double radians, KoUpdater *progress)
     }
 
     KisFilterStrategy *filter = KisFilterStrategyRegistry::instance()->value("Triangle");
-    KisTransformVisitor visitor (KisImageSP(this), 1.0, 1.0, 0, 0, radians, -tx, -ty, progress, filter);
+    KisTransformVisitor visitor(KisImageSP(this), 1.0, 1.0, 0, 0, radians, -tx, -ty, progress, filter);
     m_d->rootLayer->accept(visitor);
 
     if (undo()) m_d->adapter->addCommand(new KisImageResizeCommand(KisImageSP(this), w, h, width(), height()));
@@ -512,26 +504,26 @@ void KisImage::rotate(double radians, KoUpdater *progress)
 
 void KisImage::shear(double angleX, double angleY, KoUpdater *progress)
 {
-    const double pi=3.1415926535897932385;
+    const double pi = 3.1415926535897932385;
 
     //new image size
-    qint32 w=width();
-    qint32 h=height();
+    qint32 w = width();
+    qint32 h = height();
 
 
-    if(angleX != 0 || angleY != 0){
-        double deltaY=height()*QABS(tan(angleX*pi/180)*tan(angleY*pi/180));
-        w = (qint32) ( width() + QABS(height()*tan(angleX*pi/180)) );
+    if (angleX != 0 || angleY != 0) {
+        double deltaY = height() * QABS(tan(angleX * pi / 180) * tan(angleY * pi / 180));
+        w = (qint32)(width() + QABS(height() * tan(angleX * pi / 180)));
         //ugly fix for the problem of having two extra pixels if only a shear along one
         //axis is done. This has to be fixed in the cropping code in KisRotateVisitor!
         if (angleX == 0 || angleY == 0)
-            h = (qint32) ( height() + QABS(w*tan(angleY*pi/180)) );
+            h = (qint32)(height() + QABS(w * tan(angleY * pi / 180)));
         else if (angleX > 0 && angleY > 0)
-            h = (qint32) ( height() + QABS(w*tan(angleY*pi/180))- 2 * deltaY + 2 );
+            h = (qint32)(height() + QABS(w * tan(angleY * pi / 180)) - 2 * deltaY + 2);
         else if (angleX < 0 && angleY < 0)
-            h = (qint32) ( height() + QABS(w*tan(angleY*pi/180))- 2 * deltaY + 2 );
+            h = (qint32)(height() + QABS(w * tan(angleY * pi / 180)) - 2 * deltaY + 2);
         else
-            h = (qint32) ( height() + QABS(w*tan(angleY*pi/180)) );
+            h = (qint32)(height() + QABS(w * tan(angleY * pi / 180)));
     }
 
     if (w != width() || h != height()) {
@@ -564,8 +556,7 @@ void KisImage::shear(double angleX, double angleY, KoUpdater *progress)
 
 void KisImage::convertTo(const KoColorSpace * dstColorSpace, KoColorConversionTransformation::Intent renderingIntent)
 {
-    if ( m_d->colorSpace == dstColorSpace )
-    {
+    if (*m_d->colorSpace == *dstColorSpace) {
         return;
     }
 
@@ -580,7 +571,7 @@ void KisImage::convertTo(const KoColorSpace * dstColorSpace, KoColorConversionTr
 
     setColorSpace(dstColorSpace);
 
-    KoColorSpaceConvertVisitor visitor(dstColorSpace, renderingIntent);
+    KisColorSpaceConvertVisitor visitor(dstColorSpace, renderingIntent);
     m_d->rootLayer->accept(visitor);
 
     unlock();
@@ -602,8 +593,8 @@ void KisImage::setProfile(const KoColorProfile * profile)
 {
     if (profile == 0) return;
 
-    const KoColorSpace * dstCs= KoColorSpaceRegistry::instance()->colorSpace( colorSpace()->id(),
-                                                                                         profile);
+    const KoColorSpace * dstCs = KoColorSpaceRegistry::instance()->colorSpace(colorSpace()->id(),
+                                 profile);
     if (dstCs) {
 
         lock();
@@ -678,66 +669,67 @@ qint32 KisImage::height() const
 
 KisGroupLayerSP KisImage::rootLayer() const
 {
-    Q_ASSERT( m_d->rootLayer );
+    Q_ASSERT(m_d->rootLayer);
     return m_d->rootLayer;
 }
 
 KisPaintDeviceSP KisImage::projection()
 {
-    m_d->projection->sync();
-    
-    Q_ASSERT( m_d->rootLayer );
+    // XXX: Projection moved to the updateStrategy -- is synching really needed?
+    //m_d->projection->sync();
+
+    Q_ASSERT(m_d->rootLayer);
     KisPaintDeviceSP projection = m_d->rootLayer->projection();
-    Q_ASSERT( projection );
+    Q_ASSERT(projection);
     return projection;
 }
 
 bool KisImage::addLayer(KisLayerSP layer, KisGroupLayerSP group)
 {
     // XXX: Temporarily for compatibility
-    if ( group )
-        return addNode( layer.data(), group.data() );
+    if (group)
+        return addNode(layer.data(), group.data());
     else {
         KisNode * root = rootLayer().data();
-        return addNode( layer, root );
+        return addNode(layer, root);
     }
 }
 
 bool KisImage::addLayer(KisLayerSP layer, KisGroupLayerSP parent, KisLayerSP aboveThis)
 {
-    if ( parent && aboveThis )
-        return addNode( layer.data(), parent.data(), aboveThis.data() );
-    else if ( parent )
-        return addNode( layer.data(), parent.data() );
+    if (parent && aboveThis)
+        return addNode(layer.data(), parent.data(), aboveThis.data());
+    else if (parent)
+        return addNode(layer.data(), parent.data());
     else
-        return addNode( layer.data(), rootLayer().data() );
+        return addNode(layer.data(), rootLayer().data());
 }
 
-bool KisImage::addLayer( KisLayerSP layer,  KisGroupLayerSP parent, int index )
+bool KisImage::addLayer(KisLayerSP layer,  KisGroupLayerSP parent, int index)
 {
-    if ( parent )
-        return addNode( layer.data(), parent.data(), index );
+    if (parent)
+        return addNode(layer.data(), parent.data(), index);
     else
-        return addNode( layer.data(), rootLayer().data(), index );
+        return addNode(layer.data(), rootLayer().data(), index);
 }
 
 
 bool KisImage::removeLayer(KisLayerSP layer)
 {
-    if (KisGroupLayerSP parent = dynamic_cast<KisGroupLayer*>( layer->parent().data() ) ) {
+    if (KisGroupLayerSP parent = dynamic_cast<KisGroupLayer*>(layer->parent().data())) {
 
         KisNodeSP wasAbove = layer->nextSibling();
         KisNodeSP wasBelow = layer->prevSibling();
 
-        const bool success = removeNode( layer.data() );
+        const bool success = removeNode(layer.data());
 
         if (success) {
             layer->setImage(0);
             if (!layer->temporary() && undo()) {
                 m_d->adapter->addCommand(new KisImageLayerRemoveCommand(KisImageSP(this),
-                                                                        layer,
-                                                                        parent,
-                                                                        wasAbove));
+                                         layer,
+                                         parent,
+                                         wasAbove));
             }
         }
         return success;
@@ -748,41 +740,29 @@ bool KisImage::removeLayer(KisLayerSP layer)
 
 bool KisImage::raiseLayer(KisLayerSP layer)
 {
-    return raiseNode( layer.data() );
+    return raiseNode(layer.data());
 }
 
 bool KisImage::lowerLayer(KisLayerSP layer)
 {
-    return lowerNode( layer.data() );
-}
-
-bool KisImage::toTop(KisLayerSP layer)
-{
-    KisNode* node = dynamic_cast<KisNode*>( layer.data() );
-    return KisNodeFacade::toTop( node );
-}
-
-bool KisImage::toBottom(KisLayerSP layer)
-{
-    KisNode* node = dynamic_cast<KisNode*>( layer.data() );
-    return KisNodeFacade::toBottom( node );
+    return lowerNode(layer.data());
 }
 
 bool KisImage::moveLayer(KisLayerSP layer, KisGroupLayerSP parent, KisLayerSP aboveThis)
 {
     // XXX: Temporary until everything is moved to handling just nodes
     lock();
-    KisGroupLayerSP wasParent = dynamic_cast<KisGroupLayer*>( layer->parent().data() );
-    KisLayerSP wasAbove = dynamic_cast<KisLayer*>( layer->nextSibling().data() );
-    if ( wasParent.data() == parent.data() && wasAbove.data() == aboveThis.data() )
+    KisGroupLayerSP wasParent = dynamic_cast<KisGroupLayer*>(layer->parent().data());
+    KisLayerSP wasAbove = dynamic_cast<KisLayer*>(layer->nextSibling().data());
+    if (wasParent.data() == parent.data() && wasAbove.data() == aboveThis.data())
         return false;
-    bool success = moveNode( layer.data(), parent.data(), aboveThis.data() );
-    if ( success ) {
+    bool success = moveNode(layer.data(), parent.data(), aboveThis.data());
+    if (success) {
         if (undo())
             m_d->adapter->addCommand(new KisImageLayerMoveCommand(KisImageSP(this),
-                                                                  layer,
-                                                                  wasParent,
-                                                                  wasAbove));
+                                     layer,
+                                     wasParent,
+                                     wasAbove));
     }
 
     unlock();
@@ -796,7 +776,7 @@ qint32 KisImage::nlayers() const
     list << "KisLayer";
 
     KisCountVisitor visitor(list, KoProperties());
-    m_d->rootLayer->accept( visitor );
+    m_d->rootLayer->accept(visitor);
     return visitor.count();
 }
 
@@ -805,9 +785,9 @@ qint32 KisImage::nHiddenLayers() const
     QStringList list;
     list << "KisLayer";
     KoProperties properties;
-    properties.setProperty( "visible", false );
+    properties.setProperty("visible", false);
     KisCountVisitor visitor(list, properties);
-    m_d->rootLayer->accept( visitor );
+    m_d->rootLayer->accept(visitor);
 
     return visitor.count();
 }
@@ -824,7 +804,7 @@ void KisImage::flatten()
     KisPainter gc(dst->paintDevice());
     gc.bitBlt(rc.x(), rc.y(), COMPOSITE_COPY, mergedImage(), OPACITY_OPAQUE, rc.left(), rc.top(), rc.width(), rc.height());
 
-    setRootLayer( new KisGroupLayer(this, "", OPACITY_OPAQUE) );
+    setRootLayer(new KisGroupLayer(this, "", OPACITY_OPAQUE));
 
     if (undo()) {
         m_d->adapter->beginMacro(i18n("Flatten Image"));
@@ -847,25 +827,51 @@ void KisImage::flatten()
 }
 
 
-void KisImage::mergeLayer(KisLayerSP layer)
+KisLayerSP KisImage::mergeLayer(KisLayerSP layer, const KisMetaData::MergeStrategy* strategy)
 {
-    KisPaintLayer *newLayer = new KisPaintLayer(this, layer->name(), OPACITY_OPAQUE, colorSpace());
+    if (!layer->prevSibling()) return 0;
+    // XXX: this breaks if we allow free mixing of masks and layers
+    KisLayerSP layer2 = dynamic_cast<KisLayer*>(layer->prevSibling().data());
+    if (!layer2) return 0;
+
+    KisPaintLayerSP newLayer = new KisPaintLayer(this, layer->name(), OPACITY_OPAQUE, colorSpace());
     Q_CHECK_PTR(newLayer);
 
-    QRect rc = layer->extent() | layer->nextSibling()->extent();
+
+    QRect layerExtent = layer->extent();
+    QRect layerPrevSiblingExtent = layer->prevSibling()->extent();
+
+    QRect rc = layerExtent | layerPrevSiblingExtent;
 
     undoAdapter()->beginMacro(i18n("Merge with Layer Below"));
 
-    //Abuse the merge visitor to only merge two layers (if either are groups they'll recursively merge)
-    KisMergeVisitor visitor(newLayer->paintDevice(), rc);
-    layer->nextSibling()->accept(visitor);
-    layer->accept(visitor);
+    KisPainter gc(newLayer->paintDevice());
+    gc.setCompositeOp(newLayer->colorSpace()->compositeOp(COMPOSITE_COPY));
+    gc.bitBlt(rc.topLeft(), layer2->projection(), rc);
 
-    removeNode(layer->nextSibling());
-    addNode(newLayer, layer->parent(), layer.data());
-    removeNode(layer.data());
+    gc.setCompositeOp(layer->compositeOp());
+    gc.setOpacity(layer->opacity());
+    gc.bitBlt(rc.topLeft(), layer->projection(), rc);
+
+    // Merge meta data
+    QList<const KisMetaData::Store*> srcs;
+    srcs.append(static_cast<KisLayer*>(layer->prevSibling().data())->metaData());
+    srcs.append(layer->metaData());
+    QList<double> scores;
+    int layerPrevSiblingArea = layerPrevSiblingExtent.width() * layerPrevSiblingExtent.height();
+    int layerArea = layerExtent.width() * layerExtent.height();
+    double norm = qMax(layerPrevSiblingArea, layerArea);
+    scores.append(layerPrevSiblingArea / norm);
+    scores.append(layerArea / norm);
+    strategy->merge(newLayer->metaData(), srcs, scores);
+
+    removeNode(layer->prevSibling());
+    removeNode(layer);
+    addNode(newLayer, layer->parent());
 
     undoAdapter()->endMacro();
+    
+    return newLayer;
 }
 
 
@@ -881,10 +887,9 @@ void KisImage::renderToPainter(qint32 srcX,
                                qint32 width,
                                qint32 height,
                                QPainter &painter,
-                               const KoColorProfile *  monitorProfile,
-                               float exposure)
+                               const KoColorProfile *  monitorProfile)
 {
-    QImage img = convertToQImage(srcX, srcY, width, height, monitorProfile, exposure);
+    QImage img = convertToQImage(srcX, srcY, width, height, monitorProfile);
     painter.drawImage(dstX, dstY, img, 0, 0, width, height);
 }
 
@@ -892,11 +897,11 @@ QImage KisImage::convertToQImage(qint32 x,
                                  qint32 y,
                                  qint32 w,
                                  qint32 h,
-                                 const KoColorProfile * profile,
-                                 float exposure)
+                                 const KoColorProfile * profile)
 {
     KisPaintDeviceSP dev = m_d->rootLayer->projection();
-    QImage img = dev->convertToQImage(const_cast<KoColorProfile*>( profile ), x, y, w, h, exposure);
+    if (!dev) return QImage();
+    QImage img = dev->convertToQImage(const_cast<KoColorProfile*>(profile), x, y, w, h);
 
     if (!img.isNull()) {
 
@@ -923,12 +928,12 @@ QImage KisImage::convertToQImage(qint32 x,
 }
 
 
-QImage KisImage::convertToQImage(const QRect& r, const double xScale, const double yScale, const KoColorProfile *profile, float exposure, KisSelectionSP mask)
+QImage KisImage::convertToQImage(const QRect& r, const double xScale, const double yScale, const KoColorProfile *profile, KisSelectionSP mask)
 {
-    Q_UNUSED( mask )
+    Q_UNUSED(mask)
 
 #ifdef __GNUC__
-    #warning "KisImage::convertToQImage: Implement direct rendering of current mask onto scaled image pixels"
+#warning "KisImage::convertToQImage: Implement direct rendering of current mask onto scaled image pixels"
 #endif
 
     if (r.isEmpty()) {
@@ -945,16 +950,12 @@ QImage KisImage::convertToQImage(const QRect& r, const double xScale, const doub
     srcRect.setBottom(static_cast<int>(ceil((r.bottom() + 1) * yScale)) - 1);
 
     KisPaintDeviceSP mergedImage = m_d->rootLayer->projection();
-    QTime t;
-    t.start();
 
     quint8 *scaledImageData = new quint8[r.width() * r.height() * pixelSize];
 
     quint8 *imageRow = new quint8[srcRect.width() * pixelSize];
     const qint32 imageRowX = srcRect.x();
 
-    t.restart();
-
     for (qint32 y = 0; y < r.height(); ++y) {
 
         qint32 dstY = r.y() + y;
@@ -978,25 +979,24 @@ QImage KisImage::convertToQImage(const QRect& r, const double xScale, const doub
         }
     }
 
-    t.restart();
     delete [] imageRow;
 
-    QImage image = colorSpace()->convertToQImage(scaledImageData, r.width(), r.height(), const_cast<KoColorProfile*>( profile ), KoColorConversionTransformation::IntentPerceptual, exposure);
+    QImage image = colorSpace()->convertToQImage(scaledImageData, r.width(), r.height(), const_cast<KoColorProfile*>(profile), KoColorConversionTransformation::IntentPerceptual);
     delete [] scaledImageData;
 
 #ifdef WORDS_BIGENDIAN
     uchar * data = image.bits();
     for (int i = 0; i < image.width() * image.height(); ++i) {
-      uchar r, g, b, a;
-      a = data[0];
-      b = data[1];
-      g = data[2];
-      r = data[3];
-      data[0] = r;
-      data[1] = g;
-      data[2] = b;
-      data[3] = a;
-      data += 4;
+        uchar r, g, b, a;
+        a = data[0];
+        b = data[1];
+        g = data[2];
+        r = data[3];
+        data[0] = r;
+        data[1] = g;
+        data[2] = b;
+        data[3] = a;
+        data += 4;
     }
 #endif
     return image;
@@ -1007,11 +1007,6 @@ KisPaintDeviceSP KisImage::mergedImage()
     return m_d->rootLayer->projection();
 }
 
-KoColor KisImage::mergedPixel(qint32 x, qint32 y)
-{
-    return m_d->rootLayer->projection()->colorAt(x, y);
-}
-
 void KisImage::notifyLayersChanged()
 {
     emit sigLayersChanged(rootLayer());
@@ -1049,23 +1044,6 @@ bool KisImage::undo() const
     return (m_d->adapter && m_d->adapter->undo());
 }
 
-
-void KisImage::slotSelectionChanged()
-{
-    slotSelectionChanged(bounds());
-}
-
-void KisImage::slotSelectionChanged(const QRect& r)
-{
-    QRect r2(r.x() - 1, r.y() - 1, r.width() + 2, r.height() + 2);
-
-    if (!locked()) {
-        emit sigActiveSelectionChanged(KisImageSP(this));
-    } else {
-        m_d->selectionChangedWhileLocked = true;
-    }
-}
-
 const KoColorSpace * KisImage::colorSpace() const
 {
     return m_d->colorSpace;
@@ -1082,10 +1060,9 @@ void KisImage::setRootLayer(KisGroupLayerSP rootLayer)
 {
     m_d->rootLayer = rootLayer;
     m_d->rootLayer->disconnect();
-    m_d->rootLayer->setGraphListener( this );
-//     m_d->rootLayer->setProjectionManager( m_d->projection );
-    m_d->projection->setRootLayer( rootLayer );
-    setRoot( m_d->rootLayer.data() );
+    m_d->rootLayer->setGraphListener(this);
+    m_d->rootLayer->updateStrategy()->setImage(this);
+    setRoot(m_d->rootLayer.data());
 }
 
 void KisImage::addAnnotation(KisAnnotationSP annotation)
@@ -1131,22 +1108,19 @@ vKisAnnotationSP_it KisImage::beginAnnotations()
     const KoColorProfile * profile = colorSpace()->profile();
     KisAnnotationSP annotation;
 
-    if (profile)
-    {
+    if (profile) {
         // XXX we hardcode icc, this is correct for lcms?
         // XXX productName(), or just "ICC Profile"?
-        if( profile->valid())
-        {
+        if (profile->valid()) {
             const KoIccColorProfile* iccprofile = dynamic_cast<const KoIccColorProfile*>(profile);
-            if (!iccprofile->rawData().isEmpty())
-            {
+            if (!iccprofile->rawData().isEmpty()) {
                 annotation = new  KisAnnotation("icc", iccprofile->name(), iccprofile->rawData());
             }
         }
     }
 
     if (annotation)
-         addAnnotation(annotation);
+        addAnnotation(annotation);
     else
         removeAnnotation("icc");
 
@@ -1160,14 +1134,14 @@ vKisAnnotationSP_it KisImage::endAnnotations()
 
 KisPerspectiveGrid* KisImage::perspectiveGrid()
 {
-    if ( m_d->perspectiveGrid == 0 )
+    if (m_d->perspectiveGrid == 0)
         m_d->perspectiveGrid = new KisPerspectiveGrid();
     return m_d->perspectiveGrid;
 }
 
-void KisImage::slotProjectionUpdated( const QRect & rc )
+void KisImage::slotProjectionUpdated(const QRect & rc)
 {
-    emit sigImageUpdated( rc );
+    emit sigImageUpdated(rc);
 }
 
 #include "kis_image.moc"
