@@ -1,5 +1,6 @@
 /*
  *  Copyright (c) 2002 Patrick Julien <freak@codepimps.org>
+ *  Copyright (c) 2007 Boudewijn Rempt <boud@valdyas.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,11 +26,9 @@
 #include <QRegion>
 #include <QBitArray>
 
-#include <kdebug.h>
 #include <kurl.h>
 
 #include <KoColorConversionTransformation.h>
-#include "KoUnit.h"
 
 #include "kis_global.h"
 #include "kis_types.h"
@@ -49,11 +48,17 @@ class KoColorProfile;
 class KoUpdater;
 class KisPerspectiveGrid;
 
+namespace KisMetaData
+{
+class MergeStrategy;
+}
+
 /**
  * This is the image class, it contains a tree of KisLayer stack and meta information about the image. And it also provides some functions
  * to manipulate the whole image.
  */
-class KRITAIMAGE_EXPORT KisImage : public QObject, public KisNodeFacade, public KisNodeGraphListener, public KisShared {
+class KRITAIMAGE_EXPORT KisImage : public QObject, public KisNodeFacade, public KisNodeGraphListener, public KisShared
+{
 
     Q_OBJECT
 
@@ -64,19 +69,18 @@ public:
 
 public: // KisNodeGraphListener implementation
 
-    void aboutToAddANode( KisNode *parent, int index );
-    void nodeHasBeenAdded( KisNode *parent, int index );
-    void aboutToRemoveANode( KisNode *parent, int index );
-    void nodeHasBeenRemoved( KisNode *parent, int index );
-    void aboutToMoveNode( KisNode * parent, int oldIndex, int newIndex );
-    void nodeHasBeenMoved( KisNode * parent, int oldIndex, int newIndex );
+    void aboutToAddANode(KisNode *parent, int index);
+    void nodeHasBeenAdded(KisNode *parent, int index);
+    void aboutToRemoveANode(KisNode *parent, int index);
+    void nodeHasBeenRemoved(KisNode *parent, int index);
+    void aboutToMoveNode(KisNode * parent, int oldIndex, int newIndex);
+    void nodeHasBeenMoved(KisNode * parent, int oldIndex, int newIndex);
 
 public:
 
     /**
      * Paint the specified rect onto the painter, adjusting the colors
-     * using the given profile. The exposure setting is used if the image
-     * has a high dynamic range.
+     * using the given profile.
      */
     void renderToPainter(qint32 srcX,
                          qint32 srcY,
@@ -85,8 +89,7 @@ public:
                          qint32 width,
                          qint32 height,
                          QPainter &painter,
-                         const KoColorProfile *profile,
-                         float exposure = 0.0f);
+                         const KoColorProfile *profile);
 
     /**
      * Render the projection onto a QImage.
@@ -95,32 +98,29 @@ public:
                            qint32 y1,
                            qint32 width,
                            qint32 height,
-                           const KoColorProfile * profile,
-                           float exposure = 0.0f);
+                           const KoColorProfile * profile);
 
-     /**
-      * Render the projection scaled onto a QImage. Use this when
-      * zoom < 100% to avoid color-adjusting pixels that will be
-      * filtered away anyway. It uses nearest-neighbour sampling, so
-      * the result is inaccurate and ugly. Set the option "fast_zoom"
-      * to true to make Krita use this.
-      *
-      * XXX: Implement the mask option to draw the mask onto the
-      * scaled image.
-      *
-      * @param r the source rectangle in pixels that needs to be drawn
-      * @param xScale the X axis scale (1.0 == 100%)
-      * @param yScale the Y axis scale (1.0 == 100%)
-      * @param projection the display profile
-      * @param exposure the exposure (for hdr images)
-      * @param mask the mask that will be rendered on top of the image
-      * @return a qimage containing the sampled image pixels
-      */
-     QImage convertToQImage(const QRect& r,
-                            const double xScale, const double yScale,
-                            const KoColorProfile *profile,
-                            float exposure = 0.0f,
-                            KisSelectionSP mask = 0);
+    /**
+     * Render the projection scaled onto a QImage. Use this when
+     * zoom < 100% to avoid color-adjusting pixels that will be
+     * filtered away anyway. It uses nearest-neighbour sampling, so
+     * the result is inaccurate and ugly. Set the option "fast_zoom"
+     * to true to make Krita use this.
+     *
+     * XXX: Implement the mask option to draw the mask onto the
+     * scaled image.
+     *
+     * @param r the source rectangle in pixels that needs to be drawn
+     * @param xScale the X axis scale (1.0 == 100%)
+     * @param yScale the Y axis scale (1.0 == 100%)
+     * @param projection the display profile
+     * @param mask the mask that will be rendered on top of the image
+     * @return a qimage containing the sampled image pixels
+     */
+    QImage convertToQImage(const QRect& r,
+                           const double xScale, const double yScale,
+                           const KoColorProfile *profile,
+                           KisSelectionSP mask = 0);
 
     /**
      * Lock the image to make sure no recompositing-causing signals get emitted
@@ -153,7 +153,7 @@ public:
      * globalSelection is empty, a new selection object will be
      * created that is by default completely deselected.
      */
-    void setGlobalSelection( KisSelectionSP globalSelection = 0 );
+    void setGlobalSelection(KisSelectionSP globalSelection = 0);
 
     /**
      * Removes the global selection.
@@ -161,7 +161,17 @@ public:
     void removeGlobalSelection();
 
     /**
-     * Retrieve the next automatic layername.
+     * @return the deselected global selection or 0 if no global selection was deselected
+     */
+    KisSelectionSP deleselectedGlobalSelection();
+
+    /**
+     * Set deselected global selection
+     */
+    void setDeleselectedGlobalSelection(KisSelectionSP selection);
+
+    /**
+     * Retrieve the next automatic layername (XXX: fix to add option to return Mask X)
      */
     QString nextLayerName() const;
 
@@ -245,6 +255,7 @@ public:
      * @return the action recorder associated with this image
      */
     KisActionRecorder* actionRecorder() const;
+
     /**
      * Returns true if this image wants undo information, false otherwise
      */
@@ -333,7 +344,9 @@ public:
     /**
      * Return the size of the image
      */
-    QSize size() const { return QSize( width(), height() ); }
+    QSize size() const {
+        return QSize(width(), height());
+    }
 
     /**
      *  returns a paintdevice that contains the merged layers of this image, within
@@ -341,11 +354,9 @@ public:
      */
     KisPaintDeviceSP mergedImage();
 
-    /*
-     * Returns the color of the merged image at pixel (x, y).
+    /**
+     * @return the root node of the image node graph
      */
-    KoColor mergedPixel(qint32 x, qint32 y);
-
     KisGroupLayerSP rootLayer() const;
 
     /// Return the projection; that is, the complete, composited representation
@@ -382,7 +393,7 @@ public:
      * returns false if the layer is already in this group or any
      * other (remove it first.)
      */
-    bool KDE_DEPRECATED addLayer( KisLayerSP layer,  KisGroupLayerSP parent, int index );
+    bool KDE_DEPRECATED addLayer(KisLayerSP layer,  KisGroupLayerSP parent, int index);
 
     /// Remove layer
     bool KDE_DEPRECATED removeLayer(KisLayerSP layer);
@@ -393,12 +404,6 @@ public:
     /// Move layer down one slot
     bool KDE_DEPRECATED lowerLayer(KisLayerSP layer);
 
-    /// Move layer to top slot
-    bool KDE_DEPRECATED toTop(KisLayerSP layer);
-
-    /// Move layer to bottom slot
-    bool KDE_DEPRECATED toBottom(KisLayerSP layer);
-
     /**
      * Return the number of layers (not other nodes) that are in this
      * image.
@@ -420,7 +425,7 @@ public:
      * Merge the specified layer with the layer
      * below this layer, remove the specified layer.
      */
-    void mergeLayer(KisLayerSP l);
+    KisLayerSP mergeLayer(KisLayerSP l, const KisMetaData::MergeStrategy* strategy);
 
     QRect bounds() const;
 
@@ -440,6 +445,7 @@ public:
     void notifyLayerUpdated(KisLayerSP layer);
 
     void setColorSpace(const KoColorSpace * colorSpace);
+
     void setRootLayer(KisGroupLayerSP rootLayer);
 
     /**
@@ -465,8 +471,6 @@ public:
 
 signals:
 
-    void sigActiveSelectionChanged(KisImageSP image);
-
     /// Emitted after a layer's properties (visible, locked, opacity, composite op, name, ...) change
     void sigLayerPropertiesChanged(KisLayerSP layer);
 
@@ -477,8 +481,8 @@ signals:
      */
     void sigLayersChanged(KisGroupLayerSP rootLayer);
 
-    void sigLayerMoved( KisLayerSP layer );
-    void sigLayerRemoved( KisLayerSP layer );
+    void sigLayerMoved(KisLayerSP layer);
+    void sigLayerRemoved(KisLayerSP layer);
 
     /**
      *  Emitted whenever an action has caused the image to be
@@ -486,7 +490,7 @@ signals:
      *
      * @param rc The recty that has been recomposited.
      */
-    void sigImageUpdated( const QRect & );
+    void sigImageUpdated(const QRect &);
 
     /**
        Emitted whenever the image has been modified, so that it
@@ -501,35 +505,32 @@ signals:
     /**
      * Inform the model that we're going to add a layer.
      */
-    void sigAboutToAddANode( KisNode *parent, int index );
+    void sigAboutToAddANode(KisNode *parent, int index);
 
     /**
      * Inform the model we're done adding a layer.
      */
-    void sigNodeHasBeenAdded( KisNode *parent, int index );
+    void sigNodeHasBeenAdded(KisNode *parent, int index);
 
     /**
      * Inform the model we're going to remove a layer.
      */
-    void sigAboutToRemoveANode( KisNode *parent, int index );
+    void sigAboutToRemoveANode(KisNode *parent, int index);
 
     /**
      * Inform the model we're done removing a layer.
      */
-    void sigNodeHasBeenRemoved( KisNode *parent, int index );
+    void sigNodeHasBeenRemoved(KisNode *parent, int index);
 
 
 public slots:
-    void slotSelectionChanged();
-    void slotSelectionChanged(const QRect& r);
-
-   void slotProjectionUpdated( const QRect & rc );
+    void slotProjectionUpdated(const QRect & rc);
 
 private:
     KisImage& operator=(const KisImage& rhs);
     void init(KisUndoAdapter * adapter, qint32 width, qint32 height, const KoColorSpace * colorSpace);
     void emitSizeChanged();
-    void preparePaintLayerAfterAdding( KisLayerSP layer );
+    void preparePaintLayerAfterAdding(KisLayerSP layer);
 
 private:
     class KisImagePrivate;
