@@ -27,21 +27,21 @@
 
 #include "krita_export.h"
 
-#include "KoCompositeOp.h"
 #include "KoDocumentSectionModel.h"
 
 #include "kis_types.h"
 #include "kis_node.h"
-#include "kis_paint_device.h"
 
 class QIcon;
 class QBitArray;
-class KisGroupLayer;
 class KoColorSpace;
+class KoCompositeOp;
+class KisGroupLayer;
 class KisNodeVisitor;
 
-namespace KisMetaData {
-    class Store;
+namespace KisMetaData
+{
+class Store;
 }
 
 /**
@@ -67,7 +67,7 @@ public:
     KisLayer(const KisLayer& rhs);
     virtual ~KisLayer();
 
-    virtual const KoColorSpace * colorSpace();
+    virtual const KoColorSpace * colorSpace() const;
 
     /**
      * Ask the layer to assemble its data & apply all the effect masks
@@ -87,16 +87,6 @@ public:
      */
     virtual KisPaintDeviceSP original() const;
 
-    /**
-     * Return the paintdevice you can use to change pixels on. For a
-     * paint layer these will be paint pixels, for an adjustment layer
-     * the selection paint device.
-     *
-     * @return the paint device to paint on. Can be 0 if the actual
-     * layer type does not support painting.
-     */
-    virtual KisPaintDeviceSP paintDevice() const = 0;
-
     /**
      * @return the selection associated with this layer, if there is
      * one. Otherwise, return 0;
@@ -110,7 +100,7 @@ public:
     virtual KisSelectionSP selection() const;
 
     virtual KoDocumentSectionModel::PropertyList sectionModelProperties() const;
-    virtual void setSectionModelProperties( const KoDocumentSectionModel::PropertyList &properties  );
+    virtual void setSectionModelProperties(const KoDocumentSectionModel::PropertyList &properties);
 
     /**
      * set the channelflags for this layer to the specified bit array.
@@ -118,14 +108,14 @@ public:
      * the colorspace this layer is in, or be empty, in which case all
      * channels are active.
      */
-    void setChannelFlags( const QBitArray & channelFlags );
+    void setChannelFlags(const QBitArray & channelFlags);
 
     /**
      * Return a bit array where each bit indicates whether a
      * particular channel is active or not. If the channelflags bit
      * array is empty, all channels are active.
      */
-    QBitArray & channelFlags();
+    QBitArray & channelFlags() const;
 
 
     /**
@@ -193,6 +183,10 @@ public:
 
 public:
 
+    using KisNode::setDirty;
+    virtual void setDirty(const QRect & rect);
+
+    virtual void setDirty(const QRegion & region);
     /**
      * Returns true if there are any effect masks present
      */
@@ -206,7 +200,7 @@ public:
     /**
      * Set a temporary effect mask on this layer for filter previews.
      */
-    void setPreviewMask( KisEffectMaskSP mask );
+    void setPreviewMask(KisEffectMaskSP mask);
 
     /**
      * Retrieve the current preview effect mask
@@ -230,7 +224,7 @@ protected:
      * Apply the effect masks to the given projection, producing
      * finally the dst paint device.
      */
-    void applyEffectMasks( KisPaintDeviceSP projection, const QRect & rc );
+    void applyEffectMasks(const KisPaintDeviceSP original, KisPaintDeviceSP projection, const QRect & rc);
 
 private:
     class Private;
@@ -244,7 +238,8 @@ private:
  * XXX: Name doesn't suggest an object -- is KisIndirectPaintingLayer
  * a better name? (BSAR)
  */
-class KRITAIMAGE_EXPORT KisIndirectPaintingSupport {
+class KRITAIMAGE_EXPORT KisIndirectPaintingSupport
+{
 
 public:
 
@@ -256,8 +251,10 @@ public:
     void setTemporaryCompositeOp(const KoCompositeOp* c);
     void setTemporaryOpacity(quint8 o);
     KisPaintDeviceSP temporaryTarget();
+    const KisPaintDeviceSP temporaryTarget() const;
     const KoCompositeOp* temporaryCompositeOp() const;
     quint8 temporaryOpacity() const;
+    bool hasTemporaryTarget() const;
 
     // Or I could make KisLayer a virtual base of KisIndirectPaintingSupport and so, but
     // I'm sure virtual diamond inheritance isn't as appreciated as this
@@ -265,14 +262,11 @@ public:
     virtual KisLayer* layer() = 0;
 
 private:
-
-    // To simulate the indirect painting
-    KisPaintDeviceSP m_temporaryTarget;
-    const KoCompositeOp* m_compositeOp;
-    quint8 m_compositeOpacity;
+    struct Private;
+    Private* const d;
 
 };
 
-Q_DECLARE_METATYPE( KisLayerSP )
+Q_DECLARE_METATYPE(KisLayerSP)
 
 #endif // KIS_LAYER_H_
