@@ -21,10 +21,14 @@
 #include "kis_types.h"
 #include "kis_base_node.h"
 #include "krita_export.h"
-#include <KoProperties.h>
+
+class QStringList;
+class KoProperties;
 
 class KisNodeVisitor;
 class KisNodeGraphListener;
+class KisProjectionUpdateStrategy;
+class KisNodeProgressProxy;
 
 /**
  * A KisNode is a KisBaseNode that knows about its direct peers, parent
@@ -33,7 +37,8 @@ class KisNodeGraphListener;
  * NOTE: your subclasses must have the Q_OBJECT declaration, even if
  * you do not define new signals or slots.
  */
-class KRITAIMAGE_EXPORT KisNode : public KisBaseNode {
+class KRITAIMAGE_EXPORT KisNode : public KisBaseNode
+{
 
     Q_OBJECT
 
@@ -51,14 +56,16 @@ public:
      * Create a copy of this node. The copy will not have a parent
      * node.
      */
-    KisNode( const KisNode & rhs );
-
+    KisNode(const KisNode & rhs);
+private:
+    void init();
+public:
     /**
      * Delete this node
      */
     virtual ~KisNode();
 
-    virtual KisNodeSP clone() = 0;
+    virtual KisNodeSP clone() const = 0;
 
     virtual bool accept(KisNodeVisitor &v);
 
@@ -71,9 +78,15 @@ protected:
      * @return false if the given node is not allowed as a subclass to
      * this node
      */
-    virtual bool allowAsChild( KisNodeSP ) = 0;
+    virtual bool allowAsChild(KisNodeSP) const = 0;
+
+public:
+
+    /**
+     * @return the update strategy
+     */
+    KisProjectionUpdateStrategy * updateStrategy() const;
 
-public: // dirty region methods. XXX: Make these slots?
 
     /**
      * Set the entire node extent dirty; this percolates up to parent
@@ -94,35 +107,7 @@ public: // dirty region methods. XXX: Make these slots?
      * this percolates up to parent nodes all the way to the root
      * node, if propagate is true;
      */
-    virtual void setDirty( const QRegion & region);
-
-public:
-
-    /**
-     * @return true if any part of this layer has been marked dirty
-     */
-    bool isDirty();
-
-    /**
-     *  @return true if the given rect overlaps with the dirty region
-     *  of this node
-     */
-    bool isDirty( const QRect & rect );
-
-    /**
-     * Mark the specified area as clean
-     */
-    void setClean( QRect rc );
-
-    /**
-     * Mark the whole layer as clean
-     */
-    void setClean();
-
-    /**
-     * @return the region from the given rect that is dirty.
-     */
-    QRegion dirtyRegion( const QRect & rc );
+    virtual void setDirty(const QRegion & region);
 
 public: // Graph methods
 
@@ -136,7 +121,7 @@ public: // Graph methods
      * Set the graph listener for this node. The graphlistener will be
      * informed before and after the list of child nodes has changed.
      */
-    void setGraphListener( KisNodeGraphListener * graphListener );
+    void setGraphListener(KisNodeGraphListener * graphListener);
 
     /**
      * Returns the parent node of this node. This is 0 only for a root
@@ -183,7 +168,7 @@ public: // Graph methods
      *
      * @return 0 if there is no node at this index.
      */
-    KisNodeSP at( quint32 index ) const;
+    KisNodeSP at(quint32 index) const;
 
     /**
      * Retrieve the index of the specified child node.
@@ -191,7 +176,7 @@ public: // Graph methods
      * @return -1 if the specified node is not a child node of this
      * node.
      */
-    int index( const KisNodeSP node ) const;
+    int index(const KisNodeSP node) const;
 
 
     /**
@@ -204,8 +189,21 @@ public: // Graph methods
      * @param properties. if not empty, only nodes for which
      * KisNodeBase::check(properties) returns true will be returned.
      */
-    QList<KisNodeSP> childNodes( QStringList nodeTypes, const KoProperties & properties ) const;
-
+    QList<KisNodeSP> childNodes(const QStringList & nodeTypes, const KoProperties & properties) const;
+public:
+    /**
+     * @return the node progress proxy used by this node, if this node has no progress
+     *         proxy, it will return the proxy of its parent, if the parent has no progress proxy
+     *         it will return 0
+     */
+    KisNodeProgressProxy* nodeProgressProxy() const;
+private:
+    /**
+     * Create a node progress proxy for this node. You need to create a progress proxy only
+     * if the node is going to appear in the layerbox, and it needs to be created before
+     * the layer box is made aware of the proxy.
+     */
+    void createNodeProgressProxy( );
 
 protected:
 
@@ -236,27 +234,27 @@ private:
     /**
      * Set the parent of this node.
      */
-    void setParent( KisNodeSP parent );
+    void setParent(KisNodeSP parent);
 
     /**
      * Add the specified node above the specified node. If aboveThis
      * is 0, the node is added at the bottom.
      */
-    bool add( KisNodeSP newNode, KisNodeSP aboveThis );
+    bool add(KisNodeSP newNode, KisNodeSP aboveThis);
 
     /**
      * Removes the node at the specified index from the child nodes.
      *
      * @return false if there is no node at this index
      */
-    bool remove( quint32 index );
+    bool remove(quint32 index);
 
     /**
      * Removes the node from the child nodes.
      *
      * @return false if there's no such node in this node.
      */
-    bool remove( KisNodeSP node );
+    bool remove(KisNodeSP node);
 
 
 private:
@@ -266,4 +264,6 @@ private:
 
 };
 
+Q_DECLARE_METATYPE(KisNodeSP)
+
 #endif
