@@ -16,13 +16,14 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
+
+#include "kis_paint_device.h"
+
 #include <QRect>
 #include <QMatrix>
 #include <QImage>
 #include <QApplication>
 #include <QList>
-#include <QTime>
-#include <QTimer>
 #include <QUndoCommand>
 #include <QHash>
 
@@ -31,60 +32,50 @@
 #include <klocale.h>
 #include <kconfiggroup.h>
 
-#include "KoColorProfile.h"
-#include "KoColor.h"
-#include "KoColorSpace.h"
-#include "KoColorSpaceRegistry.h"
-#include "KoIntegerMaths.h"
+#include <KoColorProfile.h>
+#include <KoColor.h>
+#include <KoColorSpace.h>
+#include <KoColorSpaceRegistry.h>
+#include <KoIntegerMaths.h>
 #include <KoStore.h>
 
 #include "kis_global.h"
-#include "kis_paint_engine.h"
 #include "kis_types.h"
-#include "kis_painter.h"
-#include "kis_fill_painter.h"
-#include "kis_filter.h"
 #include "kis_iterator.h"
 #include "kis_iterators_pixel.h"
-#include "kis_iteratorpixeltrait.h"
+#include "kis_iterator_pixel_trait.h"
 #include "kis_random_accessor.h"
 #include "kis_random_sub_accessor.h"
 #include "kis_selection.h"
 #include "kis_node.h"
 #include "kis_painterly_overlay.h"
-#include "kis_paint_device.h"
+
 #include "kis_datamanager.h"
 
 #include "kis_undo_adapter.h"
 #include "kis_selection_component.h"
 #include "kis_pixel_selection.h"
 #include "kis_paint_device_jobs.h"
+#include "kis_repeat_iterators_pixel.h"
 
-//#define CACHE_EXACT_BOUNDS
-
-class KisPaintDevice::Private {
+class KisPaintDevice::Private
+{
 
 public:
 
     KisNodeWSP parent;
     qint32 x;
     qint32 y;
-    const KoColorSpace * colorSpace;
+    KoColorSpace * colorSpace;
     qint32 pixelSize;
     qint32 nChannels;
-    QPaintEngine * paintEngine;
-
-#ifdef CACHE_EXACT_BOUNDS
-    QRect exactBounds;
-#endif
-
     KisPainterlyOverlaySP painterlyOverlay;
 
 };
 
 KisPaintDevice::KisPaintDevice(const KoColorSpace * colorSpace, const QString& name)
-    : QObject(0)
-    , m_d( new Private() )
+        : QObject(0)
+        , m_d(new Private())
 {
     setObjectName(name);
 
@@ -97,8 +88,7 @@ KisPaintDevice::KisPaintDevice(const KoColorSpace * colorSpace, const QString& n
     m_d->nChannels = colorSpace->channelCount();
 
     quint8 *defaultPixel = new quint8 [ m_d->pixelSize ];
-    memset( defaultPixel, 0, m_d->pixelSize );
-
+    memset(defaultPixel, 0, m_d->pixelSize);
     m_datamanager = new KisDataManager(m_d->pixelSize, defaultPixel);
     delete [] defaultPixel;
 
@@ -106,151 +96,85 @@ KisPaintDevice::KisPaintDevice(const KoColorSpace * colorSpace, const QString& n
 
     m_d->parent = 0;
 
-    m_d->colorSpace = colorSpace;
-
-    m_d->paintEngine = new KisPaintEngine();
+    m_d->colorSpace = colorSpace->clone();
 
 }
 
 KisPaintDevice::KisPaintDevice(KisNodeWSP parent, const KoColorSpace * colorSpace, const QString& name)
-    : QObject(0)
-    , m_d( new Private() )
+        : QObject(0)
+        , m_d(new Private())
 {
     setObjectName(name);
-    Q_ASSERT( colorSpace );
+    Q_ASSERT(colorSpace);
 
     m_d->x = 0;
     m_d->y = 0;
 
     m_d->parent = parent;
 
-    m_d->colorSpace = colorSpace;
+    m_d->colorSpace = colorSpace->clone();
+
+    Q_ASSERT(m_d->colorSpace);
 
     m_d->pixelSize = m_d->colorSpace->pixelSize();
     m_d->nChannels = m_d->colorSpace->channelCount();
 
     quint8 *defaultPixel = new quint8[ m_d->pixelSize ];
-    colorSpace->fromQColor(Qt::black, OPACITY_TRANSPARENT, defaultPixel);
+    m_d->colorSpace->fromQColor(Qt::black, defaultPixel);
+    m_d->colorSpace->setAlpha(defaultPixel, OPACITY_TRANSPARENT, 1);
 
     m_datamanager = new KisDataManager(m_d->pixelSize, defaultPixel);
     delete [] defaultPixel;
     Q_CHECK_PTR(m_datamanager);
-    m_d->paintEngine = new KisPaintEngine();
 
 }
 
 
 KisPaintDevice::KisPaintDevice(const KisPaintDevice& rhs)
-    : QObject()
-    , KisShared(rhs)
-    , QPaintDevice()
-    , m_d( new Private() )
+        : QObject()
+        , KisShared(rhs)
+        , m_d(new Private())
 {
     if (this != &rhs) {
         m_d->parent = 0;
         if (rhs.m_datamanager) {
             m_datamanager = new KisDataManager(*rhs.m_datamanager);
             Q_CHECK_PTR(m_datamanager);
-        }
-        else {
-            kWarning() << "rhs " << rhs.objectName() << " has no datamanager\n";
+        } else {
+            warnKrita << "rhs " << rhs.objectName() << " has no datamanager\n";
         }
         m_d->x = rhs.m_d->x;
         m_d->y = rhs.m_d->y;
-        m_d->colorSpace = rhs.m_d->colorSpace;
+        m_d->colorSpace = rhs.m_d->colorSpace->clone();
 
         m_d->pixelSize = rhs.m_d->pixelSize;
 
         m_d->nChannels = rhs.m_d->nChannels;
-        m_d->paintEngine = rhs.m_d->paintEngine;
     }
 }
 
 KisPaintDevice::~KisPaintDevice()
 {
-    delete m_d->paintEngine;
+    delete m_d->colorSpace;
     delete m_d;
 }
 
-QPaintEngine * KisPaintDevice::paintEngine () const
-{
-    return m_d->paintEngine;
-}
-
-int KisPaintDevice::metric( PaintDeviceMetric metric ) const
-{
-    QRect rc = exactBounds();
-    int depth = colorSpace()->pixelSize() - 1;
-
-    switch (metric) {
-    case PdmWidth:
-        return rc.width();
-        break;
-
-    case PdmHeight:
-        return rc.height();
-        break;
-
-    case PdmWidthMM:
-        return qRound( rc.width() * ( 72 / 25.4 ) );
-        break;
-
-    case PdmHeightMM:
-        return qRound( rc.height() * ( 72 / 25.4 ) );
-        break;
-
-    case PdmNumColors:
-        return depth * depth;
-        break;
-
-    case PdmDepth:
-        return qRound( colorSpace()->pixelSize() * 8 ); // in bits
-        break;
-
-    case PdmDpiX:
-        return 72;
-        break;
-
-    case PdmDpiY:
-        return 72;
-        break;
-
-    case PdmPhysicalDpiX:
-        return 72;
-        break;
-
-    case PdmPhysicalDpiY:
-        return 72;
-        break;
-    default:
-        qWarning("QImage::metric(): Unhandled metric type %d", metric);
-        break;
-    }
-    return 0;
-}
-
 void KisPaintDevice::setDirty(const QRect & rc)
 {
 
-#ifdef CACHE_EXACT_BOUNDS
-    m_d->exactBounds |= rc;
-#endif
-    if ( m_d->parent.isValid() )
-        m_d->parent->setDirty( rc );
+    if (m_d->parent.isValid())
+        m_d->parent->setDirty(rc);
 }
 
-void KisPaintDevice::setDirty( const QRegion & region )
+void KisPaintDevice::setDirty(const QRegion & region)
 {
-#ifdef CACHE_EXACT_BOUNDS
-    m_d->exactBounds |= region.boundingRect();
-#endif
-    if ( m_d->parent.isValid() )
-        m_d->parent->setDirty( region );
+    if (m_d->parent.isValid())
+        m_d->parent->setDirty(region);
 }
 
 void KisPaintDevice::setDirty()
 {
-    if ( m_d->parent.isValid() )
+    if (m_d->parent.isValid())
         m_d->parent->setDirty();
 }
 
@@ -272,6 +196,15 @@ void KisPaintDevice::move(const QPoint& pt)
     move(pt.x(), pt.y());
 }
 
+void KisPaintDevice::extent(qint32 &x, qint32 &y, qint32 &w, qint32 &h) const
+{
+    QRect rc = extent();
+    x = rc.x();
+    y = rc.y();
+    w = rc.width();
+    h = rc.height();
+}
+
 QRect KisPaintDevice::extent() const
 {
     qint32 x, y, w, h;
@@ -282,11 +215,17 @@ QRect KisPaintDevice::extent() const
     return QRect(x, y, w, h);
 }
 
+void KisPaintDevice::exactBounds(qint32 &x, qint32 &y, qint32 &w, qint32 &h) const {
+    QRect rc = exactBounds();
+    x = rc.x();
+    y = rc.y();
+    w = rc.width();
+    h = rc.height();
+}
+
+
 QRect KisPaintDevice::exactBounds() const
 {
-#ifdef CACHE_EXACT_BOUNDS
-    return m_d->exactBounds;
-#else
     // Solution n°2
     qint32  x, y, w, h, boundX2, boundY2, boundW2, boundH2;
     QRect rc = extent();
@@ -367,134 +306,46 @@ QRect KisPaintDevice::exactBounds() const
         }
     }
     return QRect(boundX2, boundY2, boundW2, boundH2);
-#endif
 }
 
 
 void KisPaintDevice::crop(qint32 x, qint32 y, qint32 w, qint32 h)
 {
-#ifdef CACHE_EXACT_BOUNDS
-    m_d->exactBounds = QRect( x, y, w, h );
-#endif
     m_datamanager->setExtent(x - m_d->x, y - m_d->y, w, h);
 }
 
 
 void KisPaintDevice::crop(const QRect & r)
 {
-    QRect rc( r );
-#ifdef CACHE_EXACT_BOUNDS
-    m_d->exactBounds = rc;
-#endif
+    QRect rc(r);
     rc.translate(-m_d->x, -m_d->y);
     m_datamanager->setExtent(rc);
 }
 
+void KisPaintDevice::setDefaultPixel(const quint8 *defPixel)
+{
+    m_datamanager->setDefaultPixel(defPixel);
+}
+
+const quint8 *KisPaintDevice::defaultPixel() const
+{
+    return m_datamanager->defaultPixel();
+}
+
 void KisPaintDevice::clear()
 {
-#ifdef CACHE_EXACT_BOUNDS
-    m_d->exactBounds = QRect( 0, 0, 0, 0 );
-#endif
     m_datamanager->clear();
 }
 
 void KisPaintDevice::fill(qint32 x, qint32 y, qint32 w, qint32 h, const quint8 *fillPixel)
 {
-#ifdef CACHE_EXACT_BOUNDS
-    m_d->exactBounds &= QRect( x, y, w, h );
-#endif
     m_datamanager->clear(x, y, w, h, fillPixel);
 }
 
 
-void KisPaintDevice::clear( const QRect & rc )
+void KisPaintDevice::clear(const QRect & rc)
 {
-#ifdef CACHE_EXACT_BOUNDS
-    if ( rc.contains( m_d->exactBounds) )
-         m_d->exactBounds = QRect();
-#endif
-    m_datamanager->clear( rc.x(), rc.y(), rc.width(), rc.height(), m_datamanager->defaultPixel() );
-}
-
-void KisPaintDevice::mirrorX( KisSelection * selection )
-{
-    KisPaintDeviceSP dst = this;
-    if ( !m_datamanager->hasCurrentMemento() )
-        dst = new KisPaintDevice( colorSpace() );
-
-    QRect r;
-    if ( selection ) {
-        r = selection->selectedExactRect();
-    }
-    else {
-        r = exactBounds();
-    }
-    {
-        KisHLineConstIteratorPixel srcIt = createHLineConstIterator(r.x(), r.top(), r.width(), selection);
-        KisHLineIteratorPixel dstIt = dst->createHLineIterator(r.x(), r.top(), r.width());
-
-        for (qint32 y = r.top(); y <= r.bottom(); ++y) {
-
-            dstIt += r.width() - 1;
-
-            while (!srcIt.isDone()) {
-                if (srcIt.isSelected()) {
-                    if ( m_datamanager->hasCurrentMemento() )
-                        memcpy( dstIt.rawData(), srcIt.oldRawData(), m_d->pixelSize );
-                    else
-                        memcpy( dstIt.rawData(), srcIt.rawData(), m_d->pixelSize );
-                }
-                ++srcIt;
-                --dstIt;
-
-            }
-            srcIt.nextRow();
-            dstIt.nextRow();
-        }
-    }
-    if ( !m_datamanager->hasCurrentMemento() )
-        m_datamanager = dst->dataManager();
-
-    setDirty(r);
-}
-
-void KisPaintDevice::mirrorY( KisSelection * selection )
-{
-    KisPaintDeviceSP dst = this;
-    if ( !m_datamanager->hasCurrentMemento() )
-        dst = new KisPaintDevice( colorSpace() );
-
-    /* Read a line from bottom to top and and from top to bottom and write their values to each other */
-    QRect r;
-    if ( selection ) {
-        r = selection->selectedExactRect();
-    }
-    else {
-        r = exactBounds();
-    }
-
-    qint32 y1, y2;
-    for (y1 = r.top(), y2 = r.bottom(); y1 <= r.bottom(); ++y1, --y2) {
-
-        KisHLineIteratorPixel itTop = dst->createHLineIterator(r.x(), y1, r.width(), selection);
-        KisHLineConstIteratorPixel itBottom = createHLineConstIterator(r.x(), y2, r.width());
-
-        while (!itTop.isDone() && !itBottom.isDone()) {
-            if (itBottom.isSelected()) {
-                if ( m_datamanager->hasCurrentMemento() )
-                    memcpy(itTop.rawData(), itBottom.oldRawData(), m_d->pixelSize);
-                else
-                    memcpy(itTop.rawData(), itBottom.rawData(), m_d->pixelSize);
-            }
-            ++itBottom;
-            ++itTop;
-        }
-    }
-
-    if ( !m_datamanager->hasCurrentMemento() )
-        m_datamanager = dst->dataManager();
-
-    setDirty(r);
+    m_datamanager->clear(rc.x(), rc.y(), rc.width(), rc.height(), m_datamanager->defaultPixel());
 }
 
 bool KisPaintDevice::write(KoStore *store)
@@ -502,7 +353,7 @@ bool KisPaintDevice::write(KoStore *store)
     bool retval = m_datamanager->write(store);
     emit ioProgress(100);
 
-        return retval;
+    return retval;
 }
 
 bool KisPaintDevice::read(KoStore *store)
@@ -510,14 +361,12 @@ bool KisPaintDevice::read(KoStore *store)
     bool retval = m_datamanager->read(store);
     emit ioProgress(100);
 
-        return retval;
+    return retval;
 }
 
 void KisPaintDevice::convertTo(const KoColorSpace * dstColorSpace, KoColorConversionTransformation::Intent renderingIntent)
 {
-    if (   colorSpace()->id() == dstColorSpace->id()
-        && colorSpace()->profile() == dstColorSpace->profile() )
-    {
+    if (*colorSpace() == *dstColorSpace) {
         return;
     }
 
@@ -538,12 +387,12 @@ void KisPaintDevice::convertTo(const KoColorSpace * dstColorSpace, KoColorConver
     // of multithreading yet.
 
     KConfigGroup cfg = KGlobal::config()->group("");
-    bool threadColorSpaceConversion = cfg.readEntry( "thread_colorspace_conversion", false );
+    bool threadColorSpaceConversion = cfg.readEntry("thread_colorspace_conversion", false);
     ThreadWeaver::Weaver * weaver = 0;
 
-    if ( threadColorSpaceConversion ) {
+    if (threadColorSpaceConversion) {
         weaver = new Weaver();
-        weaver->setMaximumNumberOfThreads( cfg.readEntry("threadsafe",  QThread::idealThreadCount() ) );
+        weaver->setMaximumNumberOfThreads(cfg.readEntry("threadsafe",  QThread::idealThreadCount()));
     }
 
     // Experiment with these versions to see which is faster,  but it
@@ -553,25 +402,24 @@ void KisPaintDevice::convertTo(const KoColorSpace * dstColorSpace, KoColorConver
     // Because this is anew paint device, its tiles should always be
     // aligned with ours.
     {
-        KisRectConstIteratorPixel srcIter = createRectConstIterator( x, y, w, h );
-        KisRectIteratorPixel dstIter = dst.createRectIterator( x, y, w, h );
-        while ( !srcIter.isDone() && !dstIter.isDone() ) {
+        KisRectConstIteratorPixel srcIter = createRectConstIterator(x, y, w, h);
+        KisRectIteratorPixel dstIter = dst.createRectIterator(x, y, w, h);
+        while (!srcIter.isDone() && !dstIter.isDone()) {
 
             qint32 numContiguousSrcPixels = srcIter.nConseqPixels();
             qint32 numContiguousDstPixels = dstIter.nConseqPixels();
 
             // Because the tiles should be aligned
-            Q_ASSERT( numContiguousDstPixels == numContiguousSrcPixels );
+            Q_ASSERT(numContiguousDstPixels == numContiguousSrcPixels);
 
-            if ( threadColorSpaceConversion ) {
+            if (threadColorSpaceConversion) {
                 ConversionJob * job = new ConversionJob(srcIter.rawData(), dstIter.rawData(), colorSpace(), dstColorSpace, numContiguousSrcPixels, renderingIntent, 0);
-                weaver->enqueue( job );
-            }
-            else
+                weaver->enqueue(job);
+            } else
                 m_d->colorSpace->convertPixelsTo(srcIter.rawData(), dstIter.rawData(), dstColorSpace, numContiguousSrcPixels, renderingIntent);
 
-            srcIter+=numContiguousSrcPixels;
-            dstIter+=numContiguousDstPixels;
+            srcIter += numContiguousSrcPixels;
+            dstIter += numContiguousDstPixels;
         }
     }
 #else
@@ -594,11 +442,10 @@ void KisPaintDevice::convertTo(const KoColorSpace * dstColorSpace, KoColorConver
             const quint8 *srcData = srcIt.rawData();
             quint8 *dstData = dstIt.rawData();
 
-            if ( threadColorSpaceConversion ) {
+            if (threadColorSpaceConversion) {
                 ConversionJob * job = new ConversionJob(srcData, dstData, colorSpace(), dstColorSpace, columns, renderingIntent, 0);
-                weaver->enqueue( job );
-            }
-            else
+                weaver->enqueue(job);
+            } else
                 m_d->colorSpace->convertPixelsTo(srcData, dstData, dstColorSpace, columns, renderingIntent);
 
             column += columns;
@@ -607,7 +454,7 @@ void KisPaintDevice::convertTo(const KoColorSpace * dstColorSpace, KoColorConver
     }
 #endif
 
-    if ( threadColorSpaceConversion ) {
+    if (threadColorSpaceConversion) {
         weaver->finish();
     }
 
@@ -621,8 +468,9 @@ void KisPaintDevice::convertTo(const KoColorSpace * dstColorSpace, KoColorConver
 //         undoAdapter()->addCommand(new KisConvertLayerTypeCmd(KisPaintDeviceSP(this), oldData, oldColorSpace, m_datamanager, m_d->colorSpace));
 //     }
 
+    emit colorSpaceChanged(dstColorSpace);
+    delete oldColorSpace;
     delete weaver;
-    // XXX: emit colorSpaceChanged(dstColorSpace);
 }
 
 void KisPaintDevice::setProfile(const KoColorProfile * profile)
@@ -630,67 +478,44 @@ void KisPaintDevice::setProfile(const KoColorProfile * profile)
     if (profile == 0) return;
 
     const KoColorSpace * dstSpace =
-            KoColorSpaceRegistry::instance()->colorSpace( colorSpace()->id(),
-                                                                      const_cast<KoColorProfile*>(profile)); // FIXME don't const_cast and anyway should clone when cloning is available
+        KoColorSpaceRegistry::instance()->colorSpace(colorSpace()->id(), profile);
     if (dstSpace)
-        m_d->colorSpace = dstSpace;
+        m_d->colorSpace = dstSpace->clone();
     emit profileChanged(profile);
 }
 
 void KisPaintDevice::setDataManager(KisDataManagerSP data, const KoColorSpace * colorSpace)
 {
     m_datamanager = data;
-    m_d->colorSpace = colorSpace;
+    m_d->colorSpace = colorSpace->clone();
     m_d->pixelSize = m_d->colorSpace->pixelSize();
     m_d->nChannels = m_d->colorSpace->channelCount();
 
-    setDirty(extent());
-#ifdef CACHE_EXACT_BOUNDS
-    m_d->exactBounds = extent();
-#endif
-
 }
 
 void KisPaintDevice::convertFromQImage(const QImage& image, const QString &srcProfileName,
                                        qint32 offsetX, qint32 offsetY)
 {
-    Q_UNUSED( srcProfileName );
-
     QImage img = image;
 
     if (img.format() != QImage::Format_ARGB32) {
         img = img.convertToFormat(QImage::Format_ARGB32);
     }
-#ifdef __GNUC__
-#warning "KisPaintDevice::convertFromQImage. re-enable use of srcProfileName"
-#endif
-#if 0
-
-    // XXX: Apply import profile
-    if (colorSpace() == KoColorSpaceRegistry::instance() ->colorSpace("RGBA",0)) {
+    // Don't convert if not no profile is given and both paint dev and qimage are rgba.
+    if (srcProfileName.isEmpty() && colorSpace()->id() == "RGBA") {
         writeBytes(img.bits(), 0, 0, img.width(), img.height());
-    }
-    else {
-#endif
-#if 0
+    } else {
         quint8 * dstData = new quint8[img.width() * img.height() * pixelSize()];
         KoColorSpaceRegistry::instance()
-                ->colorSpace("RGBA", srcProfileName)->
-                        convertPixelsTo(img.bits(), dstData, colorSpace(), img.width() * img.height());
+            ->colorSpace("RGBA", srcProfileName)
+            ->convertPixelsTo(img.bits(), dstData, colorSpace(), img.width() * img.height());
+            
         writeBytes(dstData, offsetX, offsetY, img.width(), img.height());
-#endif
-        KisPainter p( this );
-        p.bitBlt(offsetX, offsetY, colorSpace()->compositeOp( COMPOSITE_OVER ),
-                 &image, OPACITY_OPAQUE,
-                 0, 0, image.width(), image.height());
-        p.end();
-#ifdef CACHE_EXACT_BOUNDS
-        m_d->exactBounds = image.rect();
-#endif
-//    }
+        delete[] dstData;
+    }
 }
 
-QImage KisPaintDevice::convertToQImage(const KoColorProfile *  dstProfile, float exposure)
+QImage KisPaintDevice::convertToQImage(const KoColorProfile *  dstProfile)
 {
     qint32 x1;
     qint32 y1;
@@ -706,11 +531,12 @@ QImage KisPaintDevice::convertToQImage(const KoColorProfile *  dstProfile, float
     w = rc.width();
     h = rc.height();
 
-    return convertToQImage(dstProfile, x1, y1, w, h, exposure);
+    return convertToQImage(dstProfile, x1, y1, w, h);
 }
 
-QImage KisPaintDevice::convertToQImage(const KoColorProfile *  dstProfile, qint32 x1, qint32 y1, qint32 w, qint32 h, float exposure)
+QImage KisPaintDevice::convertToQImage(const KoColorProfile *  dstProfile, qint32 x1, qint32 y1, qint32 w, qint32 h)
 {
+
     if (w < 0)
         return QImage();
 
@@ -720,9 +546,8 @@ QImage KisPaintDevice::convertToQImage(const KoColorProfile *  dstProfile, qint3
     quint8 * data;
     try {
         data = new quint8 [w * h * m_d->pixelSize];
-    } catch(std::bad_alloc)
-    {
-        kWarning() << "KisPaintDevice::convertToQImage std::bad_alloc for " << w << " * " << h << " * " << m_d->pixelSize;
+    } catch (std::bad_alloc) {
+        warnKrita << "KisPaintDevice::convertToQImage std::bad_alloc for " << w << " * " << h << " * " << m_d->pixelSize;
         //delete[] data; // data is not allocated, so don't free it
         return QImage();
     }
@@ -731,7 +556,8 @@ QImage KisPaintDevice::convertToQImage(const KoColorProfile *  dstProfile, qint3
     // XXX: Is this really faster than converting line by line and building the QImage directly?
     //      This copies potentially a lot of data.
     readBytes(data, x1, y1, w, h);
-    QImage image = colorSpace()->convertToQImage(data, w, h, dstProfile, KoColorConversionTransformation::IntentPerceptual, exposure);
+    QImage image = colorSpace()->convertToQImage(data, w, h, dstProfile,
+                   KoColorConversionTransformation::IntentPerceptual);
     delete[] data;
 
     return image;
@@ -747,13 +573,11 @@ KisPaintDeviceSP KisPaintDevice::createThumbnailDevice(qint32 w, qint32 h) const
     srcw = e.width();
     srch = e.height();
 
-    if (w > srcw)
-    {
+    if (w > srcw) {
         w = srcw;
         h = qint32(double(srcw) / w * h);
     }
-    if (h > srch)
-    {
+    if (h > srch) {
         h = srch;
         w = qint32(double(srch) / h * w);
     }
@@ -763,11 +587,11 @@ KisPaintDeviceSP KisPaintDevice::createThumbnailDevice(qint32 w, qint32 h) const
     else if (srch > srcw)
         w = qint32(double(srcw) / srch * h);
 
-    KisRandomConstAccessorPixel iter = createRandomConstAccessor(0,0);
+    KisRandomConstAccessorPixel iter = createRandomConstAccessor(0, 0);
     for (qint32 y = 0; y < h; ++y) {
-        qint32 iY = (y * srch ) / h;
+        qint32 iY = (y * srch) / h;
         for (qint32 x = 0; x < w; ++x) {
-            qint32 iX = (x * srcw ) / w;
+            qint32 iX = (x * srcw) / w;
             iter.moveTo(iX, iY);
             thumbnail->setPixel(x, y, KoColor(iter.rawData(), m_d->colorSpace));
         }
@@ -784,9 +608,9 @@ KisPainterlyOverlaySP KisPaintDevice::painterlyOverlay()
 
 void KisPaintDevice::createPainterlyOverlay()
 {
-    if ( !m_d->painterlyOverlay ) {
+    if (!m_d->painterlyOverlay) {
         m_d->painterlyOverlay = new KisPainterlyOverlay();
-        emit ( painterlyOverlayCreated() );
+        emit(painterlyOverlayCreated());
     }
 
 }
@@ -799,18 +623,15 @@ void KisPaintDevice::removePainterlyOverlay()
 
 QImage KisPaintDevice::createThumbnail(qint32 w, qint32 h)
 {
-    KisPaintDeviceSP dev = createThumbnailDevice( w, h );
-    return dev->convertToQImage( KoColorSpaceRegistry::instance()->rgb8()->profile() );
+    KisPaintDeviceSP dev = createThumbnailDevice(w, h);
+    return dev->convertToQImage(KoColorSpaceRegistry::instance()->rgb8()->profile());
 }
 
 KisRectIteratorPixel KisPaintDevice::createRectIterator(qint32 left, qint32 top, qint32 w, qint32 h, const KisSelection * selection)
 {
-#ifdef CACHE_EXACT_BOUNDS
-    m_d->exactBounds &= QRect( left, top, w, h );
-#endif
     KisDataManager* selectionDm = 0;
 
-    if ( selection )
+    if (selection)
         selectionDm = selection->dataManager().data();
 
     return KisRectIteratorPixel(m_datamanager.data(), selectionDm, left, top, w, h, m_d->x, m_d->y);
@@ -821,7 +642,7 @@ KisRectConstIteratorPixel KisPaintDevice::createRectConstIterator(qint32 left, q
     KisDataManager* dm = const_cast< KisDataManager*>(m_datamanager.data()); // TODO: don't do this
     KisDataManager* selectionDm = 0;
 
-    if(selection)
+    if (selection)
         selectionDm = const_cast< KisDataManager*>(selection->dataManager().data());
 
     return KisRectConstIteratorPixel(dm, selectionDm, left, top, w, h, m_d->x, m_d->y);
@@ -829,13 +650,9 @@ KisRectConstIteratorPixel KisPaintDevice::createRectConstIterator(qint32 left, q
 
 KisHLineIteratorPixel  KisPaintDevice::createHLineIterator(qint32 x, qint32 y, qint32 w, const KisSelection * selection)
 {
-#ifdef CACHE_EXACT_BOUNDS
-    m_d->exactBounds &= QRect( x, y, w, 1 );
-#endif
-
     KisDataManager* selectionDm = 0;
 
-    if(selection)
+    if (selection)
         selectionDm = selection->dataManager().data();
 
     return KisHLineIteratorPixel(m_datamanager.data(), selectionDm, x, y, w, m_d->x, m_d->y);
@@ -846,21 +663,39 @@ KisHLineConstIteratorPixel  KisPaintDevice::createHLineConstIterator(qint32 x, q
     KisDataManager* dm = const_cast< KisDataManager*>(m_datamanager.data()); // TODO: don't do this
     KisDataManager* selectionDm = 0;
 
-    if(selection)
+    if (selection)
         selectionDm = const_cast< KisDataManager*>(selection->dataManager().data());
 
     return KisHLineConstIteratorPixel(dm, selectionDm, x, y, w, m_d->x, m_d->y);
 }
 
-KisVLineIteratorPixel  KisPaintDevice::createVLineIterator(qint32 x, qint32 y, qint32 h, const KisSelection * selection)
+KisRepeatHLineConstIteratorPixel KisPaintDevice::createRepeatHLineConstIterator(qint32 x, qint32 y, qint32 w, const QRect& _dataWidth, const KisSelection * selection ) const
 {
-#ifdef CACHE_EXACT_BOUNDS
-    m_d->exactBounds &= QRect( x, y, 1, h );
-#endif
-
+    KisDataManager* dm = const_cast< KisDataManager*>(m_datamanager.data()); // TODO: don't do this
     KisDataManager* selectionDm = 0;
 
-    if(selection)
+    if (selection)
+        selectionDm = const_cast< KisDataManager*>(selection->dataManager().data());
+
+    return KisRepeatHLineConstIteratorPixel(dm, selectionDm, x, y, w, m_d->x, m_d->y, _dataWidth);
+}
+
+KisRepeatVLineConstIteratorPixel KisPaintDevice::createRepeatVLineConstIterator(qint32 x, qint32 y, qint32 h, const QRect& _dataWidth, const KisSelection * selection ) const
+{
+    KisDataManager* dm = const_cast< KisDataManager*>(m_datamanager.data()); // TODO: don't do this
+    KisDataManager* selectionDm = 0;
+
+    if (selection)
+        selectionDm = const_cast< KisDataManager*>(selection->dataManager().data());
+
+    return KisRepeatVLineConstIteratorPixel(dm, selectionDm, x, y, h, m_d->x, m_d->y, _dataWidth);
+}
+
+KisVLineIteratorPixel  KisPaintDevice::createVLineIterator(qint32 x, qint32 y, qint32 h, const KisSelection * selection)
+{
+    KisDataManager* selectionDm = 0;
+
+    if (selection)
         selectionDm = selection->dataManager().data();
 
     return KisVLineIteratorPixel(m_datamanager.data(), selectionDm, x, y, h, m_d->x, m_d->y);
@@ -871,7 +706,7 @@ KisVLineConstIteratorPixel  KisPaintDevice::createVLineConstIterator(qint32 x, q
     KisDataManager* dm = const_cast< KisDataManager*>(m_datamanager.data()); // TODO: don't do this
     KisDataManager* selectionDm = 0;
 
-    if(selection)
+    if (selection)
         selectionDm = const_cast< KisDataManager*>(selection->dataManager().data());
 
     return KisVLineConstIteratorPixel(dm, selectionDm, x, y, h, m_d->x, m_d->y);
@@ -881,7 +716,7 @@ KisRandomAccessorPixel KisPaintDevice::createRandomAccessor(qint32 x, qint32 y,
 {
     KisDataManager* selectionDm = 0;
 
-    if(selection)
+    if (selection)
         selectionDm = selection->dataManager().data();
 
     return KisRandomAccessorPixel(m_datamanager.data(), selectionDm, x, y, m_d->x, m_d->y);
@@ -892,19 +727,21 @@ KisRandomConstAccessorPixel KisPaintDevice::createRandomConstAccessor(qint32 x,
     KisDataManager* dm = const_cast< KisDataManager*>(m_datamanager.data()); // TODO: don't do this
     KisDataManager* selectionDm = 0;
 
-    if(selection)
+    if (selection)
         selectionDm = const_cast< KisDataManager*>(selection->dataManager().data());
 
     return KisRandomConstAccessorPixel(dm, selectionDm, x, y, m_d->x, m_d->y);
 }
 
-KisRandomSubAccessorPixel KisPaintDevice::createRandomSubAccessor( const KisSelection * selection ) const
+KisRandomSubAccessorPixel KisPaintDevice::createRandomSubAccessor(const KisSelection * selection) const
 {
+    // XXX: shouldn't we use the selection here?
+    Q_UNUSED(selection);
     KisPaintDevice* pd = const_cast<KisPaintDevice*>(this);
     return KisRandomSubAccessorPixel(pd);
 }
 
-void KisPaintDevice::clearSelection( KisSelectionSP selection )
+void KisPaintDevice::clearSelection(KisSelectionSP selection)
 {
     QRect r = selection->selectedExactRect();
 
@@ -918,7 +755,7 @@ void KisPaintDevice::clearSelection( KisSelectionSP selection )
             while (!devIt.isDone()) {
                 // XXX: Optimize by using stretches
 
-                m_d->colorSpace->applyInverseAlphaU8Mask( devIt.rawData(), selectionIt.rawData(), 1);
+                m_d->colorSpace->applyInverseAlphaU8Mask(devIt.rawData(), selectionIt.rawData(), 1);
 
                 ++devIt;
                 ++selectionIt;
@@ -944,7 +781,7 @@ void KisPaintDevice::applySelectionMask(KisSelectionSP mask)
         while (!pixelIt.isDone()) {
             // XXX: Optimize by using stretches
 
-            m_d->colorSpace->applyAlphaU8Mask( pixelIt.rawData(), maskIt.rawData(), 1);
+            m_d->colorSpace->applyAlphaU8Mask(pixelIt.rawData(), maskIt.rawData(), 1);
 
             ++pixelIt;
             ++maskIt;
@@ -954,7 +791,7 @@ void KisPaintDevice::applySelectionMask(KisSelectionSP mask)
     }
 }
 
-bool KisPaintDevice::pixel(qint32 x, qint32 y, QColor *c, quint8 *opacity)
+bool KisPaintDevice::pixel(qint32 x, qint32 y, QColor *c)
 {
     KisHLineConstIteratorPixel iter = createHLineIterator(x, y, 1);
 
@@ -962,7 +799,7 @@ bool KisPaintDevice::pixel(qint32 x, qint32 y, QColor *c, quint8 *opacity)
 
     if (!pix) return false;
 
-    colorSpace()->toQColor(pix, c, opacity);
+    colorSpace()->toQColor(pix, c);
 
     return true;
 }
@@ -981,34 +818,27 @@ bool KisPaintDevice::pixel(qint32 x, qint32 y, KoColor * kc)
     return true;
 }
 
-KoColor KisPaintDevice::colorAt(qint32 x, qint32 y) const
-{
-    KisRandomConstAccessorPixel iter = createRandomConstAccessor(x, y);
-    return KoColor(iter.rawData(), m_d->colorSpace);
-}
-
-bool KisPaintDevice::setPixel(qint32 x, qint32 y, const QColor& c, quint8  opacity)
+bool KisPaintDevice::setPixel(qint32 x, qint32 y, const QColor& c)
 {
     KisHLineIteratorPixel iter = createHLineIterator(x, y, 1);
 
-    colorSpace()->fromQColor(c, opacity, iter.rawData());
+    colorSpace()->fromQColor(c, iter.rawData());
 
     return true;
 }
 
 bool KisPaintDevice::setPixel(qint32 x, qint32 y, const KoColor& kc)
 {
-    quint8 * pix;
-    if (kc.colorSpace() != m_d->colorSpace) {
-        KoColor kc2 (kc, m_d->colorSpace);
-        pix = kc2.data();
-    }
-    else {
-        pix = kc.data();
-    }
-
+    const quint8 * pix;
     KisHLineIteratorPixel iter = createHLineIterator(x, y, 1);
-    memcpy(iter.rawData(), pix, m_d->colorSpace->pixelSize());
+    if (kc.colorSpace() != m_d->colorSpace) {
+        KoColor kc2(kc, m_d->colorSpace);
+        pix = kc2.data();
+        memcpy(iter.rawData(), pix, m_d->colorSpace->pixelSize());
+    } else {
+        pix = kc.data();
+        memcpy(iter.rawData(), pix, m_d->colorSpace->pixelSize());
+    }
 
     return true;
 }
@@ -1052,10 +882,7 @@ void KisPaintDevice::readBytes(quint8 * data, const QRect &rect)
 
 void KisPaintDevice::writeBytes(const quint8 * data, qint32 x, qint32 y, qint32 w, qint32 h)
 {
-#ifdef CACHE_EXACT_BOUNDS
-    m_d->exactBounds &= QRect( x, y, w, h );
-#endif
-    m_datamanager->writeBytes( data, x - m_d->x, y - m_d->y, w, h);
+    m_datamanager->writeBytes(data, x - m_d->x, y - m_d->y, w, h);
 }
 
 void KisPaintDevice::writeBytes(const quint8 * data, const QRect &rect)
@@ -1063,14 +890,14 @@ void KisPaintDevice::writeBytes(const quint8 * data, const QRect &rect)
     writeBytes(data, rect.x(), rect.y(), rect.width(), rect.height());
 }
 
-QVector<quint8*> KisPaintDevice::readPlanarBytes( qint32 x, qint32 y, qint32 w, qint32 h)
+QVector<quint8*> KisPaintDevice::readPlanarBytes(qint32 x, qint32 y, qint32 w, qint32 h)
 {
-    return m_datamanager->readPlanarBytes( channelSizes(), x, y, w, h );
+    return m_datamanager->readPlanarBytes(channelSizes(), x, y, w, h);
 }
 
-void KisPaintDevice::writePlanarBytes( QVector<quint8*> planes, qint32 x, qint32 y, qint32 w, qint32 h)
+void KisPaintDevice::writePlanarBytes(QVector<quint8*> planes, qint32 x, qint32 y, qint32 w, qint32 h)
 {
-    m_datamanager->writePlanarBytes( planes, channelSizes(), x, y, w, h );
+    m_datamanager->writePlanarBytes(planes, channelSizes(), x, y, w, h);
 }
 
 
@@ -1090,7 +917,13 @@ quint32 KisPaintDevice::channelCount() const
 {
     Q_ASSERT(m_d->nChannels > 0);
     return m_d->nChannels;
-;
+    ;
+}
+
+KoColorSpace * KisPaintDevice::colorSpace()
+{
+    Q_ASSERT(m_d->colorSpace != 0);
+    return m_d->colorSpace;
 }
 
 const KoColorSpace * KisPaintDevice::colorSpace() const
@@ -1114,11 +947,10 @@ QVector<qint32> KisPaintDevice::channelSizes()
 {
     QVector<qint32> sizes;
     QList<KoChannelInfo*> channels = colorSpace()->channels();
-    qSort( channels );
+    qSort(channels);
 
-    foreach( KoChannelInfo * channelInfo, channels )
-    {
-        sizes.append( channelInfo->size() );
+    foreach(KoChannelInfo * channelInfo, channels) {
+        sizes.append(channelInfo->size());
     }
     return sizes;
 }
