@@ -19,24 +19,15 @@
 #ifndef KIS_PAINT_DEVICE_IMPL_H_
 #define KIS_PAINT_DEVICE_IMPL_H_
 
-#include <QColor>
-#include <QHash>
 #include <QObject>
-#include <QPixmap>
-#include <QList>
-#include <QString>
-#include <QPaintDevice>
-#include <QPaintEngine>
 
-#include "kdebug.h"
+#include "kis_debug.h"
 
-#include "KoColorSpace.h"
-#include "KoColor.h"
+#include <KoColorConversionTransformation.h>
 
 #include "kis_types.h"
 #include "kis_global.h"
-#include "kis_image.h"
-#include <kis_shared.h>
+#include "kis_shared.h"
 #include "kis_iterators_pixel.h"
 
 #include <krita_export.h>
@@ -44,17 +35,21 @@
 class QRect;
 class QImage;
 class QPoint;
-class QPaintEngine;
+class QString;
+class QColor;
 
 class KoStore;
+class KoColorSpace;
+class KoColor;
+class KoColorProfile;
+class KoColor;
 
-class KisImage;
 class KisRandomSubAccessorPixel;
 class KisDataManager;
 class KisSelectionComponent;
 class KisPainterlyOverlay;
-typedef KisSharedPtr<KisDataManager> KisDataManagerSP;
 
+typedef KisSharedPtr<KisDataManager> KisDataManagerSP;
 
 /**
  * A paint device contains the actual pixel data and offers methods
@@ -64,12 +59,11 @@ typedef KisSharedPtr<KisDataManager> KisDataManagerSP;
  * when pixels are accessed by an iterator.
  */
 class KRITAIMAGE_EXPORT KisPaintDevice
-    : public QObject
-    , public KisShared
-    , public QPaintDevice
+            : public QObject
+            , public KisShared
 {
 
-        Q_OBJECT
+    Q_OBJECT
 
 public:
 
@@ -96,21 +90,6 @@ public:
 
 public:
 
-    /**
-     * Returns an instance of KisPaintEngine for QPainters to work with.
-     */
-    QPaintEngine * paintEngine () const;
-
-protected:
-
-    /**
-     * Returns the metric information for the given paint device
-     * metric. Part of the QPaintDevice api necessary for
-     * KisPaintEngine.
-     */
-    int metric( PaintDeviceMetric metric ) const;
-
-public:
     /**
      * Write the pixels of this paint device into the specified file store.
      */
@@ -144,12 +123,12 @@ public:
     qint32 y() const;
 
     /**
-     * Return the X offset of the paint device
+     * set the X offset of the paint device
      */
     void setX(qint32 x);
 
     /**
-     * Return the Y offset of the paint device
+     * set the Y offset of the paint device
      */
     void setY(qint32 y);
 
@@ -159,32 +138,18 @@ public:
      * For instance, the tiled datamanager keeps the extent to the nearest
      * multiple of 64.
      */
-    void extent(qint32 &x, qint32 &y, qint32 &w, qint32 &h) const
-        {
-            QRect rc = extent();
-            x = rc.x();
-            y = rc.y();
-            w = rc.width();
-            h = rc.height();
-        }
+    void extent(qint32 &x, qint32 &y, qint32 &w, qint32 &h) const;
 
+    /// Convience method for the above
     virtual QRect extent() const;
 
-
     /**
      * Get the exact bounds of this paint device. This may be very slow,
      * especially on larger paint devices because it does a linear scanline search.
      */
-    void exactBounds(qint32 &x, qint32 &y, qint32 &w, qint32 &h) const
-        {
-            QRect rc = exactBounds();
-            x = rc.x();
-            y = rc.y();
-            w = rc.width();
-            h = rc.height();
-        }
-
-
+    void exactBounds(qint32 &x, qint32 &y, qint32 &w, qint32 &h) const;
+        
+    /// Convience method for the above
     virtual QRect exactBounds() const;
 
     /**
@@ -202,11 +167,22 @@ public:
      */
     virtual void clear();
 
+    /**
+     * Sets the default pixel. New data will be initialised with this pixel. The pixel is copied: the
+     * caller still owns the pointer and needs to delete it to avoid memory leaks.
+     */
+    void setDefaultPixel(const quint8 *defPixel);
+
+    /**
+     * Get a pointer to the default pixel.
+     */
+    const quint8 *defaultPixel() const;
+
     /**
      * Clear the given rectangle to transparent black. This does not take the selection
      * into account.
      */
-    void clear( const QRect & rc );
+    void clear(const QRect & rc);
 
     /**
      * Fill the given rectangle with the given pixel. This does not take the
@@ -269,7 +245,7 @@ public:
      * paint device. If the specified area is larger than the paint
      * device's extent, the default pixel will be read.
      */
-    QVector<quint8*> readPlanarBytes( qint32 x, qint32 y, qint32 w, qint32 h);
+    QVector<quint8*> readPlanarBytes(qint32 x, qint32 y, qint32 w, qint32 h);
 
     /**
      * Write the data in the separate arrays to the channes. If there
@@ -284,8 +260,7 @@ public:
      *
      * XXX: what about undo?
      */
-    void writePlanarBytes( QVector<quint8*> planes, qint32 x, qint32 y, qint32 w, qint32 h);
-
+    void writePlanarBytes(QVector<quint8*> planes, qint32 x, qint32 y, qint32 w, qint32 h);
 
     /**
      *   Converts the paint device to a different colorspace
@@ -314,9 +289,8 @@ public:
      * @param dstProfile RGB profile to use in conversion. May be 0, in which
      * case it's up to the color strategy to choose a profile (most
      * like sRGB).
-     * @param exposure The exposure setting used to render a preview of a high dynamic range image.
      */
-    virtual QImage convertToQImage(const KoColorProfile *  dstProfile, qint32 x, qint32 y, qint32 w, qint32 h, float exposure = 0.0f);
+    virtual QImage convertToQImage(const KoColorProfile *  dstProfile, qint32 x, qint32 y, qint32 w, qint32 h);
 
     /**
      * Create an RGBA QImage from a rectangle in the paint device. The
@@ -325,9 +299,8 @@ public:
      * @param dstProfile RGB profile to use in conversion. May be 0, in which
      * case it's up to the color strategy to choose a profile (most
      * like sRGB).
-     * @param exposure The exposure setting used to render a preview of a high dynamic range image.
      */
-    virtual QImage convertToQImage(const KoColorProfile *  dstProfile, float exposure = 0.0f);
+    virtual QImage convertToQImage(const KoColorProfile *  dstProfile);
 
     /**
      * Creates a paint device thumbnail of the paint device, retaining
@@ -344,17 +317,16 @@ public:
      */
     virtual QImage createThumbnail(qint32 maxw, qint32 maxh);
 
-
     /**
      * Fill c and opacity with the values found at x and y.
      *
      * The color values will be transformed from the profile of
      * this paint device to the display profile.
-     *
+     
+     * @deprecated use iterators instead
      * @return true if the operation was successful.
      */
-    bool pixel(qint32 x, qint32 y, QColor *c, quint8 *opacity);
-
+    bool KDE_DEPRECATED pixel(qint32 x, qint32 y, QColor *c);
 
     /**
      * Fill kc with the values found at x and y. This method differs
@@ -363,14 +335,10 @@ public:
      * The color values will be transformed from the profile of
      * this paint device to the display profile.
      *
+     * @deprecated use iterators instead
      * @return true if the operation was successful.
      */
-    bool pixel(qint32 x, qint32 y, KoColor * kc);
-
-    /**
-     * Return the KoColor of the pixel at x,y.
-     */
-    KoColor colorAt(qint32 x, qint32 y) const;
+    bool KDE_DEPRECATED pixel(qint32 x, qint32 y, KoColor * kc);
 
     /**
      * Set the specified pixel to the specified color. Note that this
@@ -384,12 +352,19 @@ public:
      * Note that this will use 8-bit values and may cause a significant
      * degradation when used on 16-bit or hdr quality images.
      *
+     * @deprecated use iterators instead
      * @return true if the operation was successful
      *
      */
-    bool setPixel(qint32 x, qint32 y, const QColor& c, quint8 opacity);
+    bool KDE_DEPRECATED setPixel(qint32 x, qint32 y, const QColor& c);
 
-    bool setPixel(qint32 x, qint32 y, const KoColor& kc);
+    /// Convience method for the above
+    bool KDE_DEPRECATED setPixel(qint32 x, qint32 y, const KoColor& kc);
+
+    /**
+     * @return the colorspace of the pixels in this paint device
+     */
+    KoColorSpace * colorSpace();
 
     /**
      * @return the colorspace of the pixels in this paint device
@@ -416,7 +391,6 @@ public:
      */
     virtual quint32 channelCount() const;
 
-
     /**
      * Return the painterly overlay -- a special paint device that
      * uses the KisPainterlyOverlayColorSpace that defines things
@@ -439,34 +413,29 @@ public:
     void removePainterlyOverlay();
 
 public:
-
+    
     /**
-     *  Add the specified rect to the parent layer's set of dirty rects
-     *  (if there is a parent layer)
+     * Add the specified rect to the parent layer's set of dirty rects
+     * (if there is a parent layer)
+     * @deprecated: call setDirty on the node
      */
-    virtual void setDirty(const QRect & rc);
+    virtual void KDE_DEPRECATED setDirty(const QRect & rc);
 
     /**
      *  Add the specified region to the parent layer's dirty region
      *  (if there is a parent layer)
+     * @deprecated: call setDirty on the node
      */
-    virtual void setDirty( const QRegion & region );
+    virtual void KDE_DEPRECATED setDirty(const QRegion & region);
 
     /**
      *  Set the parent layer completely dirty, if this paint device has
      *  as parent layer.
+     * @deprecated: call setDirty on the node
      */
-    virtual void setDirty();
-
-
-    /**
-     * Mirror the device along the X axis
-     */
-    void mirrorX( KisSelection * selection = 0);
-    /**
-     * Mirror the device along the Y axis
-     */
-    void mirrorY( KisSelection * selection = 0);
+    virtual void KDE_DEPRECATED setDirty();
+    
+public:
 
     /**
      * Create an iterator over a rectangle section of a paint device, the path followed by
@@ -497,6 +466,21 @@ public:
      */
     KisHLineConstIteratorPixel createHLineConstIterator(qint32 x, qint32 y, qint32 w, const KisSelection * selection = 0) const;
 
+    /**
+     * Create an iterator that will "artificially" extend the paint device with the
+     * value of the border when trying to access values outside the range of data.
+     * 
+     * @param rc indicates the rectangle that truly contains data
+     */
+    KisRepeatHLineConstIteratorPixel createRepeatHLineConstIterator(qint32 x, qint32 y, qint32 w, const QRect& _dataWidth, const KisSelection * selection = 0) const;
+    /**
+     * Create an iterator that will "artificially" extend the paint device with the
+     * value of the border when trying to access values outside the range of data.
+     * 
+     * @param rc indicates the rectangle that trully contains data
+     */
+    KisRepeatVLineConstIteratorPixel createRepeatVLineConstIterator(qint32 x, qint32 y, qint32 h, const QRect& _dataWidth, const KisSelection * selection = 0) const;
+    
     /**
     * @param selection an up-to-date selection that has the same origin as the paint device
      * @return an iterator which points to the first pixel of a horizontal line
@@ -537,11 +521,11 @@ public:
      * This function create a random accessor which can easily access to sub pixel values.
      * @param selection an up-to-date selection that has the same origin as the paint device
      */
-    KisRandomSubAccessorPixel createRandomSubAccessor( const KisSelection * selection = 0) const;
+    KisRandomSubAccessorPixel createRandomSubAccessor(const KisSelection * selection = 0) const;
 
 
     /** Clear the selected pixels from the paint device */
-    void clearSelection( KisSelectionSP selection);
+    void clearSelection(KisSelectionSP selection);
 
     /**
      * Apply a mask to the image data, i.e. multiply each pixel's opacity by its
@@ -561,6 +545,7 @@ signals:
      * manage the undo system.
      */
     void painterlyOverlayCreated();
+    
 private:
 
     KisPaintDevice& operator=(const KisPaintDevice&);
