@@ -18,7 +18,9 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-#include <kdebug.h>
+#include "kis_paint_layer.h"
+
+#include <kis_debug.h>
 #include <QIcon>
 #include <QImage>
 #include <QUndoCommand>
@@ -26,21 +28,19 @@
 
 #include <klocale.h>
 
+#include "KoColorSpaceRegistry.h"
 #include <KoColorSpace.h>
 #include <KoColorProfile.h>
+#include <KoCompositeOp.h>
+#include <KoProperties.h>
 
-#include "kis_debug_areas.h"
 #include "kis_image.h"
-#include "kis_paint_layer.h"
 #include "kis_selection.h"
 #include "kis_painter.h"
 #include "kis_undo_adapter.h"
 #include "kis_iterators_pixel.h"
 #include "kis_paint_device.h"
-
-#include "KoColorSpaceRegistry.h"
 #include "kis_datamanager.h"
-#include "kis_undo_adapter.h"
 #include "kis_effect_mask.h"
 #include "kis_transparency_mask.h"
 #include "kis_mask.h"
@@ -56,7 +56,7 @@ public:
 
 KisPaintLayer::KisPaintLayer(KisImageSP img, const QString& name, quint8 opacity, KisPaintDeviceSP dev)
     : KisLayer(img, name, opacity)
-    , m_d( new Private() )
+    , m_d(new Private())
 {
     Q_ASSERT(img);
     Q_ASSERT(dev);
@@ -68,7 +68,7 @@ KisPaintLayer::KisPaintLayer(KisImageSP img, const QString& name, quint8 opacity
 
 KisPaintLayer::KisPaintLayer(KisImageSP img, const QString& name, quint8 opacity)
     : KisLayer(img, name, opacity)
-    , m_d( new Private() )
+    , m_d(new Private())
 {
     Q_ASSERT(img);
     m_d->paintDevice = new KisPaintDevice(this, img->colorSpace(), name);
@@ -77,12 +77,12 @@ KisPaintLayer::KisPaintLayer(KisImageSP img, const QString& name, quint8 opacity
 
 KisPaintLayer::KisPaintLayer(KisImageSP img, const QString& name, quint8 opacity, const KoColorSpace * colorSpace)
     : KisLayer(img, name, opacity)
-    , m_d( new Private() )
+    , m_d(new Private())
 {
-     Q_ASSERT(img);
-    if ( colorSpace == 0 )
+//     Q_ASSERT(img);
+    if (colorSpace == 0)
         colorSpace = img->colorSpace();
-    Q_ASSERT( colorSpace );
+    Q_ASSERT(colorSpace);
     m_d->paintDevice = new KisPaintDevice(this, colorSpace, name);
     init();
 }
@@ -90,9 +90,9 @@ KisPaintLayer::KisPaintLayer(KisImageSP img, const QString& name, quint8 opacity
 KisPaintLayer::KisPaintLayer(const KisPaintLayer& rhs)
     : KisLayer(rhs)
     , KisIndirectPaintingSupport(rhs)
-    , m_d (new Private)
+    , m_d(new Private)
 {
-    m_d->paintDevice = new KisPaintDevice( *rhs.m_d->paintDevice.data() );
+    m_d->paintDevice = new KisPaintDevice(*rhs.m_d->paintDevice.data());
     init();
 }
 
@@ -101,54 +101,86 @@ KisPaintLayer::~KisPaintLayer()
     delete m_d;
 }
 
-bool KisPaintLayer::allowAsChild( KisNodeSP node)
+bool KisPaintLayer::allowAsChild(KisNodeSP node) const
 {
-    if ( node->inherits( "KisMask" ) )
-       return true;
+    if (node->inherits("KisMask"))
+        return true;
     else
         return false;
 }
 
 void KisPaintLayer::init()
 {
-    connect( m_d->paintDevice.data(), SIGNAL( colorSpaceChanged( const KoColorSpace* ) ), this, SLOT( slotColorSpaceChanged() ) );
-    connect( m_d->paintDevice.data(), SIGNAL( profileChanged( KoColorProfile* ) ), this, SLOT( slotColorSpaceChanged() ) );
+    connect(m_d->paintDevice.data(), SIGNAL(colorSpaceChanged(const KoColorSpace*)), this, SLOT(slotColorSpaceChanged()));
+    connect(m_d->paintDevice.data(), SIGNAL(profileChanged(const KoColorProfile*)), this, SLOT(slotColorSpaceChanged()));
 }
 
 
 KisPaintDeviceSP KisPaintLayer::projection() const
 {
-    if ( !hasEffectMasks() )
+    if (!hasEffectMasks() && !hasTemporaryTarget()) {
         return m_d->paintDevice;
-    else {
+    } else {
         return m_d->projection;
     }
 }
 
 void KisPaintLayer::updateProjection(const QRect & rc)
 {
-    if ( !rc.isValid() ) return ;
-    if ( !hasEffectMasks() ) return;
-    if ( !m_d->paintDevice ) return;
-    if ( !isDirty( rc ) ) return;
+    if (!rc.isValid()) return ;
+    if (!hasEffectMasks() && !hasTemporaryTarget()) return;
+    if (!m_d->paintDevice) return;
+    if (!visible()) return;
 
-    QRegion dirty = dirtyRegion( rc );
+    dbgImage << name() << ": updateProjection " << rc;
 
-    if ( !m_d->projection ) {
-        m_d->projection = new KisPaintDevice( *m_d->paintDevice );
+    if (!m_d->projection) {
+        m_d->projection = new KisPaintDevice(*m_d->paintDevice);
+    } else {
+        m_d->projection->setX(m_d->paintDevice->x());
+        m_d->projection->setY(m_d->paintDevice->y());
+
+        KisPainter gc(m_d->projection);
+        gc.setCompositeOp(colorSpace()->compositeOp(COMPOSITE_COPY));
+        gc.bitBlt(rc.topLeft(), m_d->paintDevice, rc);
     }
-    else {
-        KisPainter gc( m_d->projection );
-        gc.setCompositeOp( colorSpace()->compositeOp( COMPOSITE_COPY ) );
-        foreach (QRect rect, dirty.rects() ) {
-            kDebug() << "Resetting projection rc " << rect;
-            gc.bitBlt( rect.topLeft(), m_d->paintDevice, rect );
+
+    KisPaintDeviceSP source = m_d->paintDevice;
+
+    KoProperties props;
+    props.setProperty("visible", true);
+    QList<KisNodeSP> masks = childNodes(QStringList("KisEffectMask"), props);
+
+    if( temporaryTarget() ) {
+        if ( masks.size() == 0 ) {
+            KisPainter gc(m_d->projection);
+            gc.setCompositeOp(temporaryCompositeOp());
+            gc.setOpacity(temporaryOpacity());
+            gc.bitBlt( rc.topLeft(), temporaryTarget(), rc);
         }
-    }
+        else {
+            source = new KisPaintDevice( m_d->paintDevice->colorSpace() );
+            QRect currentNeededRc = rc;
+            for( int i = masks.size() - 1; i >= 0 ; --i )
+            {
+                const KisEffectMask * effectMask = dynamic_cast<const KisEffectMask*>(masks.at(i).data());
+                if (effectMask) {
+                    currentNeededRc |= effectMask->neededRect( currentNeededRc );
+                }
+            }
 
-    foreach ( QRect rect, dirty.rects() ) {
-        kDebug() << "Applying effect masks on rc " << rect;
-        applyEffectMasks( m_d->projection, rect );
+            KisPainter gc(source);
+            gc.bitBlt( currentNeededRc.left(), currentNeededRc.top(), COMPOSITE_COPY,
+                       m_d->paintDevice, temporaryOpacity(), currentNeededRc.left(),
+                       currentNeededRc.top(), currentNeededRc.width(), currentNeededRc.height());
+            gc.bitBlt( currentNeededRc.left(), currentNeededRc.top(), temporaryCompositeOp(),
+                       temporaryTarget(), temporaryOpacity(), currentNeededRc.left(),
+                       currentNeededRc.top(), currentNeededRc.width(), currentNeededRc.height());
+        }
+        
+    }
+    if (masks.size() > 0) {
+        applyEffectMasks(source, m_d->projection, rc);
     }
 }
 
@@ -161,19 +193,13 @@ QIcon KisPaintLayer::icon() const
 KoDocumentSectionModel::PropertyList KisPaintLayer::sectionModelProperties() const
 {
     KoDocumentSectionModel::PropertyList l = KisLayer::sectionModelProperties();
-    l << KoDocumentSectionModel::Property(i18n("Colorspace"), m_d->paintDevice->colorSpace()->name());
-    if( const KoColorProfile *profile = m_d->paintDevice->colorSpace()->profile() )
+    l << KoDocumentSectionModel::Property(i18n("ColorSpace"), m_d->paintDevice->colorSpace()->name());
+    if (const KoColorProfile *profile = m_d->paintDevice->colorSpace()->profile())
         l << KoDocumentSectionModel::Property(i18n("Profile"), profile->name());
     return l;
 }
 
-KisLayerSP KisPaintLayer::clone() const
-{
-    return KisLayerSP(new KisPaintLayer(*this));
-}
-
-
-const KoColorSpace * KisPaintLayer::colorSpace()
+const KoColorSpace * KisPaintLayer::colorSpace() const
 {
     return m_d->paintDevice->colorSpace();
 }
@@ -191,9 +217,6 @@ bool KisPaintLayer::accept(KisNodeVisitor &v)
     return v.visit(this);
 }
 
-
-
-/// Returns the paintDevice that accompanies this layer
 KisPaintDeviceSP KisPaintLayer::paintDevice() const
 {
     return m_d->paintDevice;
@@ -229,27 +252,45 @@ void KisPaintLayer::setX(qint32 x)
         m_d->paintDevice->setX(x);
 }
 
-qint32 KisPaintLayer::y() const {
+qint32 KisPaintLayer::y() const
+{
     if (m_d->paintDevice)
         return m_d->paintDevice->y();
     else
         return 0;
 }
-void KisPaintLayer::setY(qint32 y) {
+void KisPaintLayer::setY(qint32 y)
+{
     if (m_d->paintDevice)
         m_d->paintDevice->setY(y);
 }
 
-QRect KisPaintLayer::extent() const {
+QRect KisPaintLayer::extent() const
+{
     if (m_d->paintDevice)
-        return m_d->paintDevice->extent();
+    {
+        QRect r = m_d->paintDevice->extent();
+        if( temporaryTarget() )
+        {
+            r |= temporaryTarget()->extent();
+        }
+        return r;
+    }
     else
         return QRect();
 }
 
-QRect KisPaintLayer::exactBounds() const {
+QRect KisPaintLayer::exactBounds() const
+{
     if (m_d->paintDevice)
-        return m_d->paintDevice->exactBounds();
+    {
+        QRect r = m_d->paintDevice->exactBounds();
+        if( temporaryTarget() )
+        {
+            r |= temporaryTarget()->exactBounds();
+        }
+        return r;
+    }
     else
         return QRect();
 }
