@@ -21,7 +21,6 @@
 
 #include "kis_types.h"
 #include "kis_layer.h"
-#include "kis_paint_device.h"
 
 class KoColorSpace;
 
@@ -34,7 +33,8 @@ class KoColorSpace;
  * The transparency mask has two rendering forms: as a selection mask
  * and by changing the transparency of the paint layer's pixels.
  */
-class KRITAIMAGE_EXPORT KisPaintLayer : public KisLayer, public KisIndirectPaintingSupport {
+class KRITAIMAGE_EXPORT KisPaintLayer : public KisLayer, public KisIndirectPaintingSupport
+{
 
     Q_OBJECT
 
@@ -45,14 +45,13 @@ public:
     KisPaintLayer(const KisPaintLayer& rhs);
     virtual ~KisPaintLayer();
 
-    KisNodeSP clone()
-        {
-            return KisNodeSP(new KisPaintLayer(*this));
-        }
+    KisNodeSP clone() const {
+        return KisNodeSP(new KisPaintLayer(*this));
+    }
 
-    bool allowAsChild( KisNodeSP );
+    bool allowAsChild(KisNodeSP) const;
 
-    const KoColorSpace * colorSpace();
+    const KoColorSpace * colorSpace() const;
 
     /**
      * Update the projection for the specified rect r, whether that rect is dirty
@@ -68,7 +67,6 @@ public:
 
     QIcon icon() const;
     KoDocumentSectionModel::PropertyList sectionModelProperties() const;
-    KisLayerSP clone() const;
 
 public:
 
@@ -84,6 +82,7 @@ public:
     QImage createThumbnail(qint32 w, qint32 h);
 
     bool accept(KisNodeVisitor &v);
+
     /**
      * Returns the paintDevice that accompanies this layer
      */
@@ -111,7 +110,9 @@ public:
 public slots:
 
     // KisIndirectPaintingSupport
-    KisLayer* layer() { return this; }
+    KisLayer* layer() {
+        return this;
+    }
 
 private slots:
     void slotColorSpaceChanged();
