@@ -20,29 +20,34 @@
 #ifndef KIS_PAINTER_H_
 #define KIS_PAINTER_H_
 
-#include <QBitArray>
+#include <math.h>
+
+#include <KoColorSpaceConstants.h>
 
-#include "KoColor.h"
 #include "kis_global.h"
 #include "kis_types.h"
 #include "kis_paint_device.h"
 
-#include <KoColorSpace.h>
-#include <KoProgressUpdater.h>
-#include "kis_selection.h"
-
 #include <krita_export.h>
 
+class QRegion;
 class QUndoCommand;
 class QRect;
+class QRectF;
 class QStringList;
-class KisTransaction;
-class KisBrush;
-class KisComplexColor;
-class KisPattern;
-class KisPaintOp;
-class KisPaintInformation;
+class QBitArray;
+class QPainterPath;
 
+class KoAbstractGradient;
+class KoUpdater;
+class KoColor;
+class KoCompositeOp;
+
+class KisTransaction;
+class KisPattern;
+class KisFilterConfiguration;
+class KisPaintInformation;
+class KisPaintOp;
 /**
  * KisPainter contains the graphics primitives necessary to draw on a
  * KisPaintDevice. This is the same kind of abstraction as used in Qt
@@ -57,7 +62,8 @@ class KisPaintInformation;
  * For more complex operations, you might want to have a look at the subclasses
  * of KisPainter: KisConvolutionPainter, KisFillPainter and KisGradientPainter
  */
-class KRITAIMAGE_EXPORT KisPainter {
+class KRITAIMAGE_EXPORT KisPainter
+{
 
 
 public:
@@ -65,7 +71,9 @@ public:
     KisPainter();
 
     /// Construct a painter, and begin painting on the device
-    KisPainter( KisPaintDeviceSP device );
+    KisPainter(KisPaintDeviceSP device);
+
+    /// Construct a painter, and begin painting on the device. All actions will be masked by the given selection.
     KisPainter(KisPaintDeviceSP device, KisSelectionSP selection);
     virtual ~KisPainter();
 
@@ -73,8 +81,12 @@ public:
     /**
      * Start painting on the specified device. Not undoable.
      */
-    void begin( KisPaintDeviceSP device );
-    void begin( KisPaintDeviceSP device, KisSelectionSP selection );
+    void begin(KisPaintDeviceSP device);
+
+    /**
+     * Start painting on the specified paint device. All actions will be masked by the given selection.
+     */
+    void begin(KisPaintDeviceSP device, KisSelectionSP selection);
 
     /**
      * Finish painting on the current device
@@ -84,56 +96,24 @@ public:
     /**
      * If set, the painter action is cancelable, if the action supports that.
      */
-    void setProgress(KoUpdater * progressUpdater)
-        {
-            m_progressUpdater = progressUpdater;
-        }
+    void setProgress(KoUpdater * progressUpdater);
 
     /// Begin an undoable paint operation
-    void beginTransaction(const QString& customName = QString::null);
+    void beginTransaction(const QString& customName = "");
 
     /// Finish the undoable paint operation
     QUndoCommand *endTransaction();
 
     /// begin a transaction with the given command
-    void beginTransaction( KisTransaction* command);
+    void beginTransaction(KisTransaction* command);
 
     /// Return the current transcation
-    KisTransaction  * transaction() { return m_transaction; }
+    KisTransaction  * transaction();
 
 
     /// Returns the current paint device.
-    KisPaintDeviceSP device() const { return m_device; }
-
-    /**
-     * Paint the given QImage onto the current paint device.
-     * If necessary, the QImage is first converted to the right
-     * colorspace. If the QImage is not of the QImage::Format_ARGB32
-     * or QImage::Format_RGB32 type, the result is undefined.
-     *
-     * @param dx the destination x-coordinate
-     * @param dy the destination y-coordinate
-     * @param op a pointer to the composite op used to blast the pixels from src on dst
-     * @param src the source device
-     * @param opacity the opacity of the source pixel
-     * @param sx the source x-coordinate
-     * @param sy the source y-coordinate
-     * @param sw the width of the region
-     * @param sh the height of the region
-     */
-    void bitBlt(qint32 dx, qint32 dy,
-                const KoCompositeOp* op,
-                const QImage * src,
-                quint8 opacity,
-                qint32 sx, qint32 sy,
-                qint32 sw, qint32 sh);
-
-
-    /**
-     * Convenience method that uses the opacity and composite op set
-     * in the painter. If nothing is set, opaque and OVER are assumed.
-     */
-    void bitBlt(QPoint pos, const QImage * src, QRect srcRect );
+    const KisPaintDeviceSP device() const;
+    KisPaintDeviceSP device();
 
     /**
      * Blast the specified region from src onto the current paint device.
@@ -158,8 +138,7 @@ public:
      * Convenience method that uses the opacity and composite op set
      * in the painter. If nothing is set, opaque and OVER are assumed.
      */
-    void bitBlt(QPoint pos, const KisPaintDeviceSP src, QRect srcRect );
-
+    void bitBlt(const QPoint & pos, const KisPaintDeviceSP src, const QRect & srcRect);
 
     /**
      * Overloaded function of the previous which differs that you can pass the composite op using
@@ -174,16 +153,12 @@ public:
      * @param sw the width of the region
      * @param sh the height of the region
      */
-    inline void bitBlt(qint32 dx, qint32 dy,
+    void bitBlt(qint32 dx, qint32 dy,
                 const QString & op,
                 const KisPaintDeviceSP src,
                 quint8 opacity,
                 qint32 sx, qint32 sy,
-                qint32 sw, qint32 sh)
-    {
-        bitBlt(dx, dy, m_colorSpace->compositeOp(op), src, opacity, sx, sy, sw, sh);
-    }
-
+                qint32 sw, qint32 sh);
     /**
      * Overloaded version of the previous, differs in that the opacity is forced to OPACITY_OPAQUE
      * @param dx the destination x-coordinate
@@ -195,14 +170,11 @@ public:
      * @param sw the width of the region
      * @param sh the height of the region
      */
-    inline void bitBlt(qint32 dx, qint32 dy,
+    void bitBlt(qint32 dx, qint32 dy,
                 const KoCompositeOp* op,
                 const KisPaintDeviceSP src,
                 qint32 sx, qint32 sy,
-                qint32 sw, qint32 sh)
-    {
-        bitBlt(dx, dy, op, src, OPACITY_OPAQUE, sx, sy, sw, sh);
-    }
+                qint32 sw, qint32 sh);
 
     /**
      * Overloaded function of the previous which differs that you can pass the composite op using
@@ -217,46 +189,13 @@ public:
      * @param sh the height of the region
      */
     inline void bitBlt(qint32 dx, qint32 dy,
-                const QString & op,
-                const KisPaintDeviceSP src,
-                qint32 sx, qint32 sy,
-                qint32 sw, qint32 sh)
-    {
+                       const QString & op,
+                       const KisPaintDeviceSP src,
+                       qint32 sx, qint32 sy,
+                       qint32 sw, qint32 sh) {
         bitBlt(dx, dy, op, src, OPACITY_OPAQUE, sx, sy, sw, sh);
     }
 
-
-    /**
-     * A version of bitBlt that renders using an external mask, ignoring
-     * the src device's own selection, if it has one.
-     *
-     * @param dx the destination x-coordinate
-     * @param dy the destination y-coordinate
-     * @param op  a pointer to the composite op use to blast the pixels from src on dst
-     * @param src the source device
-     * @param selMask the mask
-     * @param opacity the opacity of the source pixel
-     * @param sx the source x-coordinate
-     * @param sy the source y-coordinate
-     * @param sw the width of the region
-     * @param sh the height of the region
-     */
-    void bltMask(qint32 dx, qint32 dy,
-                 const KoCompositeOp *op,
-                 const KisPaintDeviceSP src,
-                 const KisPaintDeviceSP selMask,
-                 quint8 opacity,
-                 qint32 sx, qint32 sy,
-                 qint32 sw, qint32 sh);
-
-
-    /**
-     * Convenience method that uses the opacity and composite op set
-     * in the painter. If noting is set, opaque and OVER are assumed.
-     */
-    void bltMask(QPoint pos, const KisPaintDeviceSP src, KisPaintDeviceSP selMask, QRect srcRect );
-
-
     /**
      * Overloaded function of the previous that take a KisSelection instead of a KisPaintDevice.
      * @param dx the destination x-coordinate
@@ -273,17 +212,16 @@ public:
     void bltSelection(qint32 dx, qint32 dy,
                       const KoCompositeOp  *op,
                       const KisPaintDeviceSP src,
-                      const KisSelectionSP selMask,
+                      const KisSelectionSP seldev,
                       quint8 opacity,
                       qint32 sx, qint32 sy,
                       qint32 sw, qint32 sh);
 
-
     /**
      * Convenience method that uses the opacity and composite op set
      * in the painter. If noting is set, opaque and OVER are assumed.
      */
-    void bltSelection(QPoint pos, const KisPaintDeviceSP src, const KisSelectionSP selDev, QRect srcRect );
+    void bltSelection(const QPoint & pos, const KisPaintDeviceSP src, const KisSelectionSP selDev, const QRect & srcRect);
 
     /**
      * Overloaded function of the previous that takes a KisSelection
@@ -307,10 +245,7 @@ public:
                       const KisSelectionSP selMask,
                       quint8 opacity,
                       qint32 sx, qint32 sy,
-                      qint32 sw, qint32 sh)
-    {
-        bltSelection(dx, dy, m_colorSpace->compositeOp(op), src, selMask, opacity, sx, sy, sw, sh);
-    }
+                      qint32 sw, qint32 sh);
 
     /**
      * A version of bitBlt that renders using the src device's selection mask, if it has one.
@@ -326,7 +261,7 @@ public:
      */
     void bltSelection(qint32 dx, qint32 dy,
                       const KoCompositeOp *op,
-                      const KisPaintDeviceSP src,
+                      const KisPaintDeviceSP srcdev,
                       quint8 opacity,
                       qint32 sx, qint32 sy,
                       qint32 sw, qint32 sh);
@@ -348,10 +283,7 @@ public:
                       const KisPaintDeviceSP src,
                       quint8 opacity,
                       qint32 sx, qint32 sy,
-                      qint32 sw, qint32 sh)
-    {
-       bltSelection(dx, dy, m_colorSpace->compositeOp(op), src, opacity, sx, sy, sw, sh);
-    }
+                      qint32 sw, qint32 sh);
 
     /**
      * The methods below are 'higher' level than the above methods. They need brushes, colors
@@ -388,67 +320,58 @@ public:
      * because the currenlty set brush has a spacing greater than that distance.
      */
     double paintBezierCurve(const KisPaintInformation &pi1,
-                const QPointF &control1,
-                const QPointF &control2,
-                const KisPaintInformation &pi2,
-                const double savedDist = -1);
+                            const QPointF &control1,
+                            const QPointF &control2,
+                            const KisPaintInformation &pi2,
+                            const double savedDist = -1);
     /**
      * Fill the given vector points with the points needed to draw the Bezier curve between
      * pos1 and pos2 using control points 1 and 2, excluding the final pos2.
      */
     void getBezierCurvePoints(const QPointF &pos1,
-                const QPointF &control1,
-                const QPointF &control2,
-                const QPointF &pos2,
-                vQPointF& points) const;
+                              const QPointF &control1,
+                              const QPointF &control2,
+                              const QPointF &pos2,
+                              vQPointF& points) const;
 
     /**
      * Paint a rectangle.
      * @param rect the rectangle to paint.
      */
-    void paintRect(const QRectF &rect,
-                   const double pressure,
-                   const double xTilt,
-                   const double yTilt);
+    void paintRect(const QRectF &rect);
 
     /**
-    * Paint a rectangle.
-    * @param x x coordinate of the top-left corner
-    * @param y y coordinate of the top-left corner
-    * @param w the rectangle width
-    * @param h the rectangle height
-    */
+     * Paint a rectangle.
+     *
+     * @param x x coordinate of the top-left corner
+     * @param y y coordinate of the top-left corner
+     * @param w the rectangle width
+     * @param h the rectangle height
+     */
     void paintRect(const double x,
                    const double y,
                    const double w,
-                   const double h,
-                   const double pressure,
-                   const double xTilt,
-                   const double yTilt);
+                   const double h);
 
     /**
      * Paint the ellipse that fills the given rectangle.
+     *
      * @param rect the rectangle containing the ellipse to paint.
      */
-    void paintEllipse(const QRectF &rect,
-                      const double pressure,
-                      const double xTilt,
-                      const double yTilt);
+    void paintEllipse(const QRectF &rect);
 
     /**
-    * Paint the ellipse that fills the given rectangle.
-    * @param x x coordinate of the top-left corner
-    * @param y y coordinate of the top-left corner
-    * @param w the rectangle width
-    * @param h the rectangle height
-    */
+     * Paint the ellipse that fills the given rectangle.
+     *
+     * @param x x coordinate of the top-left corner
+     * @param y y coordinate of the top-left corner
+     * @param w the rectangle width
+     * @param h the rectangle height
+     */
     void paintEllipse(const double x,
                       const double y,
                       const double w,
-                      const double h,
-                      const double pressure,
-                      const double xTilt,
-                      const double yTilt);
+                      const double h);
 
     /**
      * Paint the polygon with the points given in points. It automatically closes the polygon
@@ -459,9 +382,55 @@ public:
     /** Draw a spot at pos using the currently set paint op, brush and color */
     void paintAt(const KisPaintInformation &pos);
 
+    /**
+     * Stroke the given QPainterPath.
+     */
+    void paintPainterPath(const QPainterPath& path);
 
-    // ------------------------------------------------------------------------
-    // Set the parameters for the higher level graphics primitives.
+    /**
+     * Fills the area enclosed by the given QPainterPath
+     */
+    void fillPainterPath(const QPainterPath& path);
+
+    /**
+     * paint an unstroked one-pixel wide line from specified start position to the
+     * specified end position.
+     *
+     * XXX: this method really should work with subpixel precision for start and end position
+     */
+    void drawLine(const QPointF & start, const QPointF & end);
+
+    /**
+     * paints an unstroked one-pixel line using the DDA algorithm from specified start position to the
+     * specified end position.
+     *
+     * XXX: this method really should work with subpixel precision for start and end position
+     */
+    void drawDDALine(const QPointF & start, const QPointF & end);
+
+    /**
+     * Paint an unstroked, wobbly one-pixel wide line from the specified start to the specified
+     * end position.
+     *
+     * XXX: this method really should work with subpixel precision for start and end position
+     */
+    void drawWobblyLine(const QPointF & start, const QPointF & end);
+
+    /**
+     * Paint an unstroked, one-pixel wide line from the specified start to the specified
+     * end position using the Wu algorithm
+     */
+    void drawWuLine(const QPointF & start, const QPointF & end);
+
+    /**
+     * Paint an unstroked wide line from the specified start to the specified
+     * end position with width varying from @param w1 at the start to @param w2 at
+     * the end.
+     *
+     * XXX: the width should be set in doubles, not integers.
+     * XXX: this method really should work with subpixel precision for start and end position
+     */
+    void drawThickLine(const QPointF & start, const QPointF & end, int startWidth, int endWidth);
 
     /**
      * Set the channelflags: a bit array where true means that the
@@ -475,45 +444,64 @@ public:
      * the channels where the corresponding bit is true will will be
      * composited onto the destination device.
      */
-    void setChannelFlags( QBitArray channelFlags )
-        {
-            m_channelFlags = channelFlags;
-        }
+    void setChannelFlags(QBitArray channelFlags);
 
 
-    QBitArray channelFlags()
-        {
-            return m_channelFlags;
-        }
+    QBitArray channelFlags();
 
-    // Set the current brush
-    void setBrush(KisBrush* brush) { m_brush = brush; }
-    // Returns the currently set brush
-    KisBrush * brush() const { return m_brush; }
+    /**
+     * Set the paintop preset to use. If @param image is given,
+     * the paintop will be created using this image as parameter.
+     * Some paintops really want to know about the image they work
+     * for, e.g. the duplicate paintop.
+     */
+    void setPaintOpPreset(KisPaintOpPresetSP preset, KisImageSP image);
 
-    // Set the current pattern
-    void setPattern(KisPattern * pattern) { m_pattern = pattern; }
-    // Returns the currently set pattern
-    KisPattern * pattern() const { return m_pattern; }
+    /// Return the paintop preset
+    KisPaintOpPresetSP preset() const;
+
+    /**
+     * Return the active paintop (which is created based on the specified preset and
+     * will be deleted as soon as the KisPainter instance dies).
+     */
+    KisPaintOp* paintOp() const;
+
+    /// Set the current pattern
+    void setPattern(KisPattern * pattern);
+
+    /// Returns the currently set pattern
+    KisPattern * pattern() const;
+
+    /**
+     * Set the color that will be used to paint with, and convert it
+     * to the color space of the current paint device.
+     */
+    void setPaintColor(const KoColor& color);
 
-    // Set the color that will be used to paint with
-    void setPaintColor(const KoColor& color) { m_paintColor = color;}
     /// Returns the color that will be used to paint with
-    KoColor paintColor() const { return m_paintColor; }
+    KoColor paintColor() const;
 
-    // Set the current background color
-    void setBackgroundColor(const KoColor& color) {m_backgroundColor = color; }
-    // Returns the current background color
-    KoColor backgroundColor() const { return m_backgroundColor; }
+    /**
+     * Set the current background color, and convert it
+     * to the color space of the current paint device.
+     */
+    void setBackgroundColor(const KoColor& color);
 
-    // Set the current fill color
-    void setFillColor(const KoColor& color) { m_fillColor = color; }
-    // Returns the current fill color
-    KoColor fillColor() const { return m_fillColor; }
+    /// Returns the current background color
+    KoColor backgroundColor() const;
+
+    /// Set the current fill color
+    void setFillColor(const KoColor& color);
+
+    /// Returns the current fill color
+    KoColor fillColor() const;
+
+    /// Set the current generator (a generator can be used to fill an area
+    void setGenerator(KisFilterConfiguration * generator);
+
+    /// @return the current generator configuration
+    KisFilterConfiguration * generator() const;
 
-    // XXX
-    void setComplexColor(KisComplexColor *color) { m_complexColor = color; }
-    KisComplexColor *complexColor() { return m_complexColor; }
 
     /// This enum contains the styles with which we can fill things like polygons and ellipses
     enum FillStyle {
@@ -522,20 +510,21 @@ public:
         FillStyleBackgroundColor,
         FillStylePattern,
         FillStyleGradient,
-        FillStyleStrokes
+        FillStyleStrokes,
+        FillStyleGenerator
     };
 
     /// Set the current style with which to fill
-    void setFillStyle(FillStyle fillStyle) { m_fillStyle = fillStyle; }
+    void setFillStyle(FillStyle fillStyle);
 
     /// Returns the current fill style
-    FillStyle fillStyle() const { return m_fillStyle; }
+    FillStyle fillStyle() const;
 
     /// Set whether a polygon's filled area should be anti-aliased or not. The default is true.
-    void setAntiAliasPolygonFill(bool antiAliasPolygonFill) { m_antiAliasPolygonFill = antiAliasPolygonFill; }
+    void setAntiAliasPolygonFill(bool antiAliasPolygonFill);
 
     /// Return whether a polygon's filled area should be anti-aliased or not
-    bool antiAliasPolygonFill() { return m_antiAliasPolygonFill; }
+    bool antiAliasPolygonFill();
 
     /// The style of the brush stroke around polygons and so
     enum StrokeStyle {
@@ -544,57 +533,44 @@ public:
     };
 
     /// Set the current brush stroke style
-    void setStrokeStyle(StrokeStyle strokeStyle) { m_strokeStyle = strokeStyle; }
+    void setStrokeStyle(StrokeStyle strokeStyle);
+
     /// Returns the current brush stroke style
-    StrokeStyle strokeStyle() const { return m_strokeStyle; }
+    StrokeStyle strokeStyle() const;
 
     /// Set the opacity which is used in painting (like filling polygons)
-    void setOpacity(quint8 opacity) { m_opacity = opacity; }
+    void setOpacity(quint8 opacity);
     /// Returns the opacity that is used in painting
-    quint8 opacity() const { return m_opacity; }
-
-    /// Sets the current pressure for things that like to use this
-    void setPressure(double pressure) { m_pressure = pressure; }
-    /// Returns the current pressure
-    double pressure() { return m_pressure; }
+    quint8 opacity() const;
 
     /// Sets the bounds of the painter area; if not set, the painter
     /// will happily paint where you ask it, making the paint device
     /// larger as it goes
-    void setBounds( const QRect & bounds ) { m_bounds = bounds;  }
-    QRect bounds() { return m_bounds;  }
-
-    /**
-     * Set the current paint operation. This is used for all drawing functions.
-     * The painter will DELETE the paint op itself!!
-     * That means no that you should not delete it yourself (or put it on the stack)
-     */
-    void setPaintOp(KisPaintOp * paintOp);
-    /// Returns the current paint operation
-    KisPaintOp * paintOp() const { return m_paintOp; }
-
-    /// Set a current 'dab'. This usually is a paint device containing a rendered brush
-    void setDab(KisPaintDeviceSP dab) { m_dab = dab; }
-    /// Get the currently set dab
-    KisPaintDeviceSP dab() const { return m_dab; }
+    void setBounds(const QRect & bounds);
+    QRect bounds();
 
     /// Set the composite op for this painter
-    void setCompositeOp(const KoCompositeOp * op) { m_compositeOp = op; }
-    const KoCompositeOp * compositeOp() { return m_compositeOp; }
-
-    /// Calculate the distance that point p is from the line made by connecting l0 and l1
-    static double pointToLineDistance(const QPointF& p, const QPointF& l0, const QPointF& l1);
+    void setCompositeOp(const KoCompositeOp * op);
+    const KoCompositeOp * compositeOp();
 
     /**
      * Add the r to the current dirty rect, and return the dirtyRegion after adding r to it.
      */
-    QRegion addDirtyRect(QRect r);
+    QRegion addDirtyRect(const QRect & r);
 
-    //TODO expand to paint PainterPath
     /**
-     * Paints a painterpath. The points are in pixel coordinates.
+     * Reset the selection to the given selection. All painter actions will be
+     * masked by the specified selection.
      */
-    void fillPainterPath(const QPainterPath& path);
+    void setSelection(KisSelectionSP selection);
+
+    /**
+     * @return the selection set on this painter.
+     */
+    KisSelectionSP selection();
+
+    void setGradient(KoAbstractGradient* gradient);
+    KoAbstractGradient* gradient();
 
 
 protected:
@@ -605,41 +581,26 @@ protected:
     void fillPolygon(const vQPointF& points, FillStyle fillStyle);
 
 private:
+
     KisPainter(const KisPainter&);
     KisPainter& operator=(const KisPainter&);
 
+    float frac(float value) {
+        float tmp = 0;
+        return modff(value , &tmp);
+    }
+
+    float invertFrac(float value) {
+        float tmp = 0;
+        return 1.0f - modff(value , &tmp);
+    }
+
 protected:
-    KisPaintDeviceSP m_device;
-    KisSelectionSP m_selection;
-    KisTransaction  *m_transaction;
-    KoUpdater * m_progressUpdater;
-
-    QRegion m_dirtyRegion;
-    QRect m_dirtyRect;
-
-    QRect m_bounds;
-    KoColor m_paintColor;
-    KoColor m_backgroundColor;
-    KoColor m_fillColor;
-    KisPaintLayer *m_sourceLayer;
-    KisComplexColor *m_complexColor;
-    FillStyle m_fillStyle;
-    StrokeStyle m_strokeStyle;
-    bool m_antiAliasPolygonFill;
-    KisBrush *m_brush;
-    KisPattern *m_pattern;
-    QPointF m_duplicateOffset;
-    quint8 m_opacity;
-    KisPaintOp * m_paintOp;
-    double m_pressure;
-    qint32 m_pixelSize;
-    const KoColorSpace * m_colorSpace;
-    KoColorProfile *  m_profile;
-    KisPaintDeviceSP m_dab;
-    const KoCompositeOp * m_compositeOp;
-    QBitArray m_channelFlags;
-    bool m_useBoundingDirtyRect;
+    KoUpdater * progressUpdater();
+private:
 
+    struct Private;
+    Private* const d;
 };
 
 
