@@ -16,15 +16,17 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
+
 #include "kis_painterly_overlay.h"
 
-class KisPainterlyOverlay::Private {
+class KisPainterlyOverlay::Private
+{
 public:
 };
 
 KisPainterlyOverlay::KisPainterlyOverlay()
-    : KisPaintDevice( KisPainterlyOverlayColorSpace::instance() )
-    , m_d( new Private() )
+        : KisPaintDevice(KisPainterlyOverlayColorSpace::instance())
+        , m_d(new Private())
 {
 }
 
