@@ -2,7 +2,7 @@
  *  kis_pattern.cc - part of Krayon
  *
  *  Copyright (c) 2000 Matthias Elter <elter@kde.org>
- *                2001 John Califf
+ *                2001 John Califf <jwcaliff@compuzone.net>
  *                2004 Boudewijn Rempt <boud@valdyas.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -19,8 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
+
 #include "kis_pattern.h"
 
+
 #include <sys/types.h>
 #include <netinet/in.h>
 
@@ -34,20 +36,21 @@
 #include <QFile>
 #include <QTextStream>
 
-#include <kdebug.h>
+#include "KoColor.h"
+#include "KoColorSpace.h"
 #include <klocale.h>
 
-#include "KoColor.h"
+#include "kis_debug.h"
 #include "kis_layer.h"
 #include "kis_paint_device.h"
 
 KisPattern::KisPattern(const QString& file)
-    : KoPattern(file)
+        : KoPattern(file)
 {
 }
 
 KisPattern::KisPattern(KisPaintDevice* image, int x, int y, int w, int h)
-    : KoPattern("")
+        : KoPattern("")
 {
     // Forcefully convert to RGBA8
     // XXX profile and exposure?
@@ -59,7 +62,8 @@ KisPattern::~KisPattern()
 {
 }
 
-KisPaintDeviceSP KisPattern::image(const KoColorSpace * colorSpace) {
+KisPaintDeviceSP KisPattern::image(const KoColorSpace * colorSpace)
+{
     // Check if there's already a pattern prepared for this colorspace
     QMap<QString, KisPaintDeviceSP>::const_iterator it = m_colorspaces.find(colorSpace->id());
     if (it != m_colorspaces.end())
@@ -70,7 +74,7 @@ KisPaintDeviceSP KisPattern::image(const KoColorSpace * colorSpace) {
 
     Q_CHECK_PTR(layer);
 
-    layer->convertFromQImage(img(),"");
+    layer->convertFromQImage(img(), "");
 
     m_colorspaces[colorSpace->id()] = layer;
     return layer;
@@ -78,7 +82,7 @@ KisPaintDeviceSP KisPattern::image(const KoColorSpace * colorSpace) {
 
 KisPattern* KisPattern::clone() const
 {
-    KisPattern* pattern = new KisPattern("");
+    KisPattern* pattern = new KisPattern(filename());
     pattern->setImage(img());
     pattern->setName(name());
     return pattern;
