@@ -23,7 +23,7 @@
 
 #include <kio/job.h>
 
-#include "kis_debug_areas.h"
+#include "kis_debug.h"
 #include <KoPattern.h>
 #include <krita_export.h>
 #include "kis_types.h"
@@ -32,7 +32,8 @@ class QImage;
 class KoColorSpace;
 class KisPaintDevice;
 
-class KRITAIMAGE_EXPORT KisPattern : public KoPattern {
+class KRITAIMAGE_EXPORT KisPattern : public KoPattern
+{
 
     Q_OBJECT
 
