@@ -21,7 +21,7 @@
 #include <QString>
 #include <QMap>
 #include <QVariant>
-#include <kdebug.h>
+#include <kis_debug.h>
 
 class QDomElement;
 class QDomDocument;
@@ -31,14 +31,11 @@ class QDomDocument;
 
 
 /**
- * A KisSerializableConfiguration is the serializable representation of
- * parameters. This can subclass this class to implement
- * direct accessors to properties, but properties not in the map will
- * not be serialized.
- *
- * XXX: Use KoProperties here!
+ * KisPropertiesConfiguration is a map-based properties class that can be serialized
+ * and deserialized.
  */
-class KRITAIMAGE_EXPORT KisPropertiesConfiguration : public KisSerializableConfiguration {
+class KRITAIMAGE_EXPORT KisPropertiesConfiguration : public KisSerializableConfiguration
+{
 
 public:
 
@@ -56,7 +53,7 @@ public:
 
 public:
 
-    
+
     /**
      * Fill the filter configuration object from the XML encoded representation in s.
      * This function use the "Legacy" style XML in the 1.x kra file.
@@ -67,19 +64,22 @@ public:
      * This function use the "Legacy" style XML in the 1.x kra file.
      */
     virtual void fromXML(const QDomElement&);
-    
+
     /**
      * Create a serialized version of this filter config
      * This function use the "Legacy" style XML in the 1.x kra file.
      */
     virtual void toXML(QDomDocument&, QDomElement&) const;
+
     /**
      * Create a serialized version of this filter config
      * This function use the "Legacy" style XML in the 1.x kra file.
      */
     virtual QString toXML() const;
-    
-    
+
+
+    bool hasProperty( const QString& name );
+
     /**
      * Set the property with name to value.
      */
@@ -93,34 +93,41 @@ public:
 
     virtual QVariant getProperty(const QString & name) const;
 
-    
-    
     int getInt(const QString & name, int def = 0) const;
+
     double getDouble(const QString & name, double def = 0.0) const;
+
+    float getFloat( const QString& name, float def = 0.0 ) const;
+
     bool getBool(const QString & name, bool def = false) const;
-    QString getString(const QString & name, const QString & def = QString::null) const;
+
+    QString getString(const QString & name, const QString & def = "") const;
 
     QMap<QString, QVariant> getProperties() const;
 
-protected:
     /// Clear the map of properties
     void clearProperties();
+
 private:
-        void dump();
+
+    void dump();
+
+private:
+
+    struct Private;
+    Private* const d;
+};
+
+class KRITAIMAGE_EXPORT KisPropertiesConfigurationFactory : public KisSerializableConfigurationFactory
+{
+public:
+    KisPropertiesConfigurationFactory();
+    virtual ~KisPropertiesConfigurationFactory();
+    virtual KisSerializableConfiguration* createDefault();
+    virtual KisSerializableConfiguration* create(const QDomElement& e);
 private:
     struct Private;
     Private* const d;
 };
 
-class KRITAIMAGE_EXPORT KisPropertiesConfigurationFactory : public KisSerializableConfigurationFactory {
-    public:
-        KisPropertiesConfigurationFactory();
-        virtual ~KisPropertiesConfigurationFactory();
-        virtual KisSerializableConfiguration* createDefault();
-        virtual KisSerializableConfiguration* create(const QDomElement& e);
-    private:
-        struct Private;
-        Private* const d;
-};
-
 #endif // _KIS_FILTER_CONFIGURATION_H_
