@@ -50,16 +50,22 @@ const QString KIS_SELECTION_ID = "KisSelection";
 class KisSelectionComponent;
 
 /**
- * KisSelection contains a byte-map representation of a layer, where
- * the value of a byte signifies whether a corresponding pixel is selected, or not.
+ * KisSelection is a paint device that is constructed out of several components.
+ * such as a pixel selection, a vector selection or something else we haven't thought
+ * up yet.
  *
- * NOTE: You need to manually call emitSelectionChanged on the owner
- * paint device of a selection. KisSelection does not emit any signals
- * by itself because often you want to combine several actions in to
- * perfom one operation and you do not want recomposition to happen
- * all the time.
+ * Every pixel in the paint device can indicate a degree of selectedness, varying
+ * between MIN_SELECTED and MAX_SELECTED.
+ *
+ * The paint device itself is only a projection:
+ * you can read from it, but not write to it. You need to keep track of the need
+ * for updating the projection yourself: there is no automatic updating after changing
+ * the contents of one or more of the selection components.
+ *
+ * XXX: optimize: if there is only a PixelSelection, use that as projection.
  */
-class KRITAIMAGE_EXPORT KisSelection : public KisPaintDevice {
+class KRITAIMAGE_EXPORT KisSelection : public KisPaintDevice
+{
 
 public:
     /**
@@ -74,89 +80,87 @@ public:
      * Create a new KisSelection from the given mask. The selection
      * will share its pixel data with the mask
      */
-    KisSelection( KisPaintDeviceSP parent, KisMaskSP mask );
+    KisSelection(KisPaintDeviceSP parent, KisMaskSP mask);
 
     /**
-     * Create a new KisSelection. This selection will not have a parent paint device.
+     * Create a new KisSelection. This selection will not have a
+     * parent paint device.
      */
     KisSelection();
 
     /**
-     * Copy the selection
+     * Copy the selection. The selection components are copied, too.
      */
     KisSelection(const KisSelection& rhs);
 
+    /**
+     * Delete the selection. The shape selection component is deleted, the
+     * pixel selection component is contained in a shared pointer, so that
+     * may still be valid.
+     *
+     * XXX: Make this sane!
+     */
     virtual ~KisSelection();
 
-    QIcon icon() const
-        {
-            return KIcon("frame-edit"); //XXX: Get good icon!
-        }
-
-
     /**
      * Returns selectedness of the specified pixel, or 0 if invalid
-     * coordinates
+     * coordinates. The projection is not updated before determinging selectedness.
      */
     quint8 selected(qint32 x, qint32 y) const;
 
     /**
-     * Invert the selection.
-     *
-     * XXX: The extent that is inverted is the total
-     * extent of the selection projection, not that of the selection
-     * components, the parent paint device or the image. Shouldn't we
-     * fix this?
-     */
-    void invert();
-
-    /**
-     * Clear the selection.
-     *
-     * XXX: shouldn't we also clear the selection components?
+     * Clear the selection. This invalidates the projection. It does not clear
+     * the selection components.
+     * Afterwards, you can call updateProjection to recompute the selection.
      */
     void clear();
+
+    /**
+     * Clear the specified rect in the selection projection. This invalidates
+     * the projection. It does not clear the selection components.
+     * Afterwards, you can call updateProjection to recompute
+     * the selection.
+     */
     void clear(const QRect& r);
 
     /**
-     * Tests if the the rect is totally outside the selection
+     * Tests if the the rect is totally outside the selection.
      */
-    bool isTotallyUnselected(QRect r) const;
+    bool isTotallyUnselected(const QRect & r) const;
 
     /**
      * Tests if the the rect is totally outside the selection, but
      * uses selectedRect instead of selectedRect, and this is faster
      * (but might deliver false positives!)
      */
-    bool isProbablyTotallyUnselected(QRect r) const;
+    bool isProbablyTotallyUnselected(const QRect & r) const;
 
     /**
      * Rough, but fastish way of determining the area
-     * of the tiles used by the selection.
+     * of the tiles used by the selection projection.
      */
     QRect selectedRect() const;
 
     /**
      * Slow, but exact way of determining the rectangle
-     * that encloses the selection
+     * that encloses the selection projection.
      */
     QRect selectedExactRect() const;
 
-    void paint(QImage* img, const QRect & r);
-
     /**
      * If the parent paint device is interested in keeping up to date
      * with the dirtyness of this selection, set to true
      */
-    void setInterestedInDirtyness(bool b) { m_interestedInDirtyness = b; }
+    void setInterestedInDirtyness(bool b);
 
     /**
      * returns true if the parent paint device is interested in
      * keeping up with the dirtyness of the selection.
      */
-    bool interestedInDirtyness() const { return m_interestedInDirtyness; }
+    bool interestedInDirtyness() const;
 
     virtual void setDirty(const QRect & rc);
+    virtual void setDirty(const QRegion & region);
     virtual void setDirty();
 
     bool hasPixelSelection() const;
@@ -166,14 +170,13 @@ public:
      * return the pixel selection component of this selection or zero
      * if hasPixelSelection() returns false.
      */
-    KisPixelSelectionSP pixelSelection();
+    KisPixelSelectionSP pixelSelection() const;
 
     /**
      * return the vector selection component of this selection or zero
      * if hasShapeSelection() returns false.
      */
-    KisSelectionComponent* shapeSelection();
-
+    KisSelectionComponent* shapeSelection() const;
 
     /**
      * Return the pixel selection associated with this selection or
@@ -188,31 +191,27 @@ public:
     void updateProjection();
     void updateProjection(const QRect& r);
 
+    void setDeselected(bool deselected);
+    bool isDeselected();
+
 private:
 
     // We don't want these methods to be used on selections:
 
-    QRect extent() const
-        {
-            return KisPaintDevice::extent();
-        }
+    QRect extent() const {
+        return KisPaintDevice::extent();
+    }
 
 
-    QRect exactBounds() const
-        {
-            return KisPaintDevice::exactBounds();
-        }
+    QRect exactBounds() const {
+        return KisPaintDevice::exactBounds();
+    }
 
 
 private:
 
-    // XXX: Move to Private class!
-    KisPaintDeviceWSP m_parentPaintDevice;
-    bool m_interestedInDirtyness;
-    bool m_hasPixelSelection;
-    bool m_hasShapeSelection;
-    KisPixelSelectionSP m_pixelSelection;
-    KisSelectionComponent* m_shapeSelection;
+    struct Private;
+    Private * const m_d;
 };
 
 #endif // KIS_SELECTION_H_
