@@ -1,7 +1,6 @@
 /*
  *  Copyright (c) 2006 Boudewijn Rempt <boud@valdyas.org>
  *
- *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -28,10 +27,11 @@
 /**
  * An selection mask is a single channel mask that applies a
  * particular selection to the layer the mask belongs to. A selection
- * can contain both
+ * can contain both vector and pixel selection components.
 */
 class KRITAIMAGE_EXPORT KisSelectionMask : public KisMask
 {
+    Q_OBJECT
 public:
 
     /**
@@ -41,55 +41,32 @@ public:
     KisSelectionMask(KisImageWSP image);
 
     virtual ~KisSelectionMask();
-    KisSelectionMask( const KisSelectionMask& rhs );
+    KisSelectionMask(const KisSelectionMask& rhs);
 
-    bool allowAsChild( KisNodeSP );
+    bool allowAsChild(KisNodeSP) const;
 
-    QIcon icon() const
-        {
-            return KIcon(""); // XXX: Find nice icon for the subclasses.
-        }
-
-    KisNodeSP clone()
-    {
-        return KisNodeSP(new KisSelectionMask(*this));
+    QIcon icon() const {
+        return KIcon(""); // XXX: Find nice icon for the subclasses.
     }
 
-
-    KisSelectionSP selection() const;
+    KisNodeSP clone() const {
+        return KisNodeSP(new KisSelectionMask(*this));
+    }
 
     /// Set the selection of this adjustment layer to a copy of selection.
     void setSelection(KisSelectionSP selection);
 
-    /**
-     * overriden from KisBaseNode
-     */
-    qint32 x() const;
+    bool accept(KisNodeVisitor &v);
 
     /**
-     * overriden from KisBaseNode
+     * @return the deselected selection or 0 if no selection was deselected
      */
-    void setX(qint32 x);
+    KisSelectionSP deleselectedSelection();
 
     /**
-     * overriden from KisBaseNode
+     * Set deselected selection
      */
-    qint32 y() const;
-
-    /**
-     * overriden from KisBaseNode
-     */
-    void setY(qint32 y);
-
-    /**
-     * overriden from KisBaseNode
-     */
-    QRect extent() const;
-
-    /**
-     * overriden from KisBaseNode
-     */
-    QRect exactBounds() const;
+    void setDeleselectedSelection(KisSelectionSP selection);
 
 private:
 
