@@ -15,11 +15,14 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
+
 #include "kis_threaded_applicator.h"
 
 #include <math.h>
 
+#include <QThread>
 #include <QRect>
+#include <QCoreApplication>
 
 #include <threadweaver/ThreadWeaver.h>
 
@@ -33,7 +36,8 @@
 
 using namespace ThreadWeaver;
 
-class KisThreadedApplicator::Private {
+class KisThreadedApplicator::Private
+{
 
 public:
 
@@ -48,8 +52,12 @@ public:
 };
 
 
-KisThreadedApplicator::KisThreadedApplicator( KisPaintDeviceSP dev, const QRect & rc, KisJobFactory * jobFactory, int margin, KoProgressUpdater * progressUpdater )
-    : m_d( new Private() )
+KisThreadedApplicator::KisThreadedApplicator(KisPaintDeviceSP dev,
+        const QRect & rc,
+        KisJobFactory * jobFactory,
+        KoProgressUpdater * progressUpdater,
+        int margin)
+        : m_d(new Private())
 {
     m_d->dev = dev;
     m_d->rc = rc;
@@ -58,12 +66,12 @@ KisThreadedApplicator::KisThreadedApplicator( KisPaintDeviceSP dev, const QRect
     m_d->progressUpdater = progressUpdater;
 
     KConfigGroup cfg = KGlobal::config()->group("");
-    m_d->maxThreads = cfg.readEntry("maxthreads",  10);
-    m_d->tileSize = cfg.readEntry( "threadingtilesize", 512 );
+    m_d->maxThreads = cfg.readEntry("maxthreads",  QThread::idealThreadCount());
+    m_d->tileSize = cfg.readEntry("threadingtilesize", 512);
 
     m_d->weaver = new Weaver();
-    m_d->weaver->setMaximumNumberOfThreads( m_d->maxThreads );
-    connect( m_d->weaver, SIGNAL( jobDone(ThreadWeaver::Job*) ), this, SLOT( jobDone( ThreadWeaver::Job* ) ) );
+    m_d->weaver->setMaximumNumberOfThreads(m_d->maxThreads);
+    connect(m_d->weaver, SIGNAL(jobDone(ThreadWeaver::Job*)), this, SLOT(jobDone(ThreadWeaver::Job*)));
 }
 
 
@@ -76,9 +84,6 @@ KisThreadedApplicator::~KisThreadedApplicator()
 
 void KisThreadedApplicator::execute()
 {
-    // XXX_PROGRESS: compute the total number of jobs we're going to
-    // start first, so we can set the KoProgressUpdater up in the
-    // right way.
     double h = m_d->rc.height();
     double w = m_d->rc.width();
     int x = m_d->rc.x();
@@ -87,25 +92,25 @@ void KisThreadedApplicator::execute()
     // Note: we're doing columns first, so when we have small strip left
     // at the bottom, we have as few and as long runs of pixels left
     // as possible.
-    if ( w <= m_d->tileSize && h <= m_d->tileSize ) {
+    if (w <= m_d->tileSize && h <= m_d->tileSize) {
         KoUpdater updater = m_d->progressUpdater->startSubtask();
-        Job * job = m_d->jobFactory->createJob( this, m_d->dev, m_d->rc, m_d->margin, &updater );
-        m_d->weaver->enqueue( job );
-    }
-    else {
-        int numTasks = static_cast<int>( ceil( w / m_d->tileSize * h / m_d->tileSize) );
-        m_d->progressUpdater->start( numTasks );    
+        updater.setRange(0, 100);
+        Job * job = m_d->jobFactory->createJob(this, m_d->dev, m_d->rc, m_d->margin, updater);
+        m_d->weaver->enqueue(job);
+    } else {
+        int numTasks = static_cast<int>(ceil(w / m_d->tileSize * h / m_d->tileSize));
+        m_d->progressUpdater->start(100);
 
         int wleft = w;
         int col = 0;
-        while ( wleft > 0 ) {
+        while (wleft > 0) {
             int hleft = h;
             int row = 0;
-            while ( hleft > 0 ) {
-                QRect subrect( col + x, row + y, qMin( wleft, m_d->tileSize ), qMin( hleft, m_d->tileSize ) );
-                KoUpdater updater = m_d->progressUpdater->startSubtask(); // Will be deleted by the progress updater    
-                Job * job = m_d->jobFactory->createJob( this, m_d->dev, subrect, m_d->margin, &updater );
-                m_d->weaver->enqueue( job );
+            while (hleft > 0) {
+                QRect subrect(col + x, row + y, qMin(wleft, m_d->tileSize), qMin(hleft, m_d->tileSize));
+                KoUpdater updater = m_d->progressUpdater->startSubtask();
+                Job * job = m_d->jobFactory->createJob(this, m_d->dev, subrect, m_d->margin, updater);
+                m_d->weaver->enqueue(job);
                 hleft -= m_d->tileSize;
                 row += m_d->tileSize;
             }
@@ -113,12 +118,17 @@ void KisThreadedApplicator::execute()
             col += m_d->tileSize;
         }
     }
+    while (!m_d->weaver->isIdle()) {
+        QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
+    }
     m_d->weaver->finish();
 }
 
-void KisThreadedApplicator::jobDone( Job* job)
+void KisThreadedApplicator::jobDone(Job* job)
 {
+    QRect rc = static_cast<KisJob*>(job)->area();
     delete job;
+    emit areaDone(rc);
 }
 
 #include "kis_threaded_applicator.moc"
