@@ -26,24 +26,22 @@
 #include "kis_types.h"
 #include <krita_export.h>
 
-class KoUpdater;
-class KoProgressUpdater;
+#include <KoProgressUpdater.h>
 
 /**
-   A threadweaver job that knows about paint devices and rects. Note
-   that it is the task of the job implementation to handle the margin!
+ * A threadweaver job that knows about paint devices and rects. Note
+ * that it is the task of the job implementation to handle the margin!
  */
 class KRITAIMAGE_EXPORT KisJob : public ThreadWeaver::Job
 {
 public:
 
-    KisJob( QObject * parent, KisPaintDeviceSP dev, const QRect & rc, int margin )
-        : ThreadWeaver::Job( parent )
-        , m_dev( dev )
-        , m_rc( rc )
-        , m_margin( margin )
-        {
-        }
+    KisJob(QObject * parent, KisPaintDeviceSP dev, const QRect & rc, int margin)
+            : ThreadWeaver::Job(parent)
+            , m_dev(dev)
+            , m_rc(rc)
+            , m_margin(margin) {
+    }
 
 
     virtual ~KisJob() {}
@@ -53,17 +51,22 @@ public:
      * job is done.
      */
     virtual void jobDone() {}
+
+    QRect area() {
+        return m_rc;
+    }
 protected:
 
     KisPaintDeviceSP m_dev;
-    QRect m_rc;
-    int m_margin;
+    QRect m_rc; // the area we execute on without the margin
+    int m_margin; // we will execute on m_rc enlarged by the margin
 };
 
 /**
    Implement this interface to create the specific jobs you need.
  */
-class KRITAIMAGE_EXPORT KisJobFactory {
+class KRITAIMAGE_EXPORT KisJobFactory
+{
 
 public:
 
@@ -71,7 +74,7 @@ public:
     virtual ~KisJobFactory() {}
 
 
-    virtual ThreadWeaver::Job * createJob(QObject * parent, KisPaintDeviceSP dev,  const QRect & rc, int margin, KoUpdater * updater) = 0;
+    virtual ThreadWeaver::Job * createJob(QObject * parent, KisPaintDeviceSP dev,  const QRect & rc, int margin, KoUpdater updater) = 0;
 };
 
 /**
@@ -79,39 +82,42 @@ public:
    paint device and creates threadweaver jobs for as many subrects as
    are needed to cover the whole paint device.
 */
-class KRITAIMAGE_EXPORT KisThreadedApplicator : public QObject {
+class KRITAIMAGE_EXPORT KisThreadedApplicator : public QObject
+{
 
     Q_OBJECT
 
 public:
 
     /**
-       @param dev The paintdevice that is the subject of the jobs
-       @param rc The part of the paintdevice that needs to be acted on
-       @param jobFactory The factory class that creates the
-              specialized jobs
-       @param margin. If present, the rects parcelled out to the jobs
-                      will have the specified margin. When the results
-                      are put together again, the margin is cut off.
-                      Use this for convolutions, for instance.
-       @param updater The master KoProgressUpdater that will track updates for
-              all threads.
-    */
-    KisThreadedApplicator( KisPaintDeviceSP dev, const QRect & rc, KisJobFactory * jobFactory, int margin = 0, KoProgressUpdater * updater = 0 );
+     * @param dev The paintdevice that is the subject of the jobs
+     *  @param rc The part of the paintdevice that needs to be acted on
+     *  @param jobFactory The factory class that creates the
+     *         specialized jobs
+     *  @param updater The master KoProgressUpdater that will track updates for
+     *         all threads.
+     *  @param margin. If present, the rects parcelled out to the jobs
+     *                 will have the specified margin. When the results
+     *                 are put together again, the margin is cut off.
+     *                 Use this for convolutions, for instance.
+     *
+     */
+    KisThreadedApplicator(KisPaintDeviceSP dev, const QRect & rc, KisJobFactory * jobFactory, KoProgressUpdater * updater, int margin = 0);
     ~KisThreadedApplicator();
 
     /**
-      Start all threads. Returns when all subthreads are done.
-
-      XXX: make it possible to cancel the jobs!
-      XXX: integrate with the progress updater thingy
-
+     * Execute the given job and return when the whole rect has
+     * been done.
      */
     void execute();
 
+signals:
+
+    void areaDone(const QRect & rc);
+
 private slots:
 
-    void jobDone( ThreadWeaver::Job* );
+    void jobDone(ThreadWeaver::Job*);
 
 private:
 
