@@ -25,7 +25,6 @@
 
 class QRect;
 
-
 /**
  *  A transparency mask is a single channel mask that applies a particular
  *  transparency to the layer the mask belongs to. It differs from an
@@ -45,17 +44,18 @@ public:
 
     virtual ~KisTransparencyMask();
 
-    KisNodeSP clone()
-    {
+    KisNodeSP clone() const {
         return KisNodeSP(new KisTransparencyMask(*this));
     }
 
-    KisTransparencyMask( const KisTransparencyMask& rhs );
+    KisTransparencyMask(const KisTransparencyMask& rhs);
 
 
-    bool allowAsChild( KisNodeSP );
+    bool allowAsChild(KisNodeSP) const;
 
-    virtual void apply( KisPaintDeviceSP projection, const QRect & rc ) const;
+    virtual void apply(KisPaintDeviceSP projection, const QRect & rc) const;
+
+    bool accept(KisNodeVisitor &v);
 };
 
 #endif //_KIS_TRANSPARENCY_MASK_
