@@ -22,14 +22,17 @@
 #include <QMap>
 #include <QPoint>
 
-#include <kis_shared_ptr.h>
+template<class T>
+class KisWeakSharedPtr;
+template<class T>
+class KisSharedPtr;
 
 #include "kis_shared_ptr_vector.h"
 
 /**
  * Define lots of shared pointer versions of Krita classes.
  * Shared pointer classes have the advantage of near automatic
- * memory management (but take care of circular references)
+ * memory management (but beware of circular references)
  * and the disadvantage that inheritiance relations are no longer
  * recognizable
  */
@@ -40,12 +43,6 @@ class KisImage;
 typedef KisSharedPtr<KisImage> KisImageSP;
 typedef KisWeakSharedPtr<KisImage> KisImageWSP;
 
-class KisAnnotation;
-typedef KisSharedPtr<KisAnnotation> KisAnnotationSP;
-typedef KisSharedPtrVector<KisAnnotation> vKisAnnotationSP;
-typedef vKisAnnotationSP::iterator vKisAnnotationSP_it;
-typedef vKisAnnotationSP::const_iterator vKisAnnotationSP_cit;
-
 class KisPaintDevice;
 typedef KisSharedPtr<KisPaintDevice> KisPaintDeviceSP;
 typedef KisWeakSharedPtr<KisPaintDevice> KisPaintDeviceWSP;
@@ -120,6 +117,12 @@ typedef KisSharedPtr<KisPaintLayer> KisPaintLayerSP;
 class KisAdjustmentLayer;
 typedef KisSharedPtr<KisAdjustmentLayer> KisAdjustmentLayerSP;
 
+class KisGeneratorLayer;
+typedef KisSharedPtr<KisGeneratorLayer> KisGeneratorLayerSP;
+
+class KisCloneLayer;
+typedef KisSharedPtr<KisCloneLayer> KisCloneLayerSP;
+
 class KisGroupLayer;
 typedef KisSharedPtr<KisGroupLayer> KisGroupLayerSP;
 typedef KisWeakSharedPtr<KisGroupLayer> KisGroupLayerWSP;
@@ -136,20 +139,17 @@ typedef KisSharedPtr<KisPixelSelection> KisPixelSelectionSP;
 class KisHistogram;
 typedef KisSharedPtr<KisHistogram> KisHistogramSP;
 
-class KisPaintOpFactory;
-typedef KisSharedPtr<KisPaintOpFactory> KisPaintOpFactorySP;
-
 typedef QVector<QPoint> vKisSegments;
 
-//class KisGuide;
-//typedef KisSharedPtr<KisGuide> KisGuideSP;
-
-class KisQImagemask;
-typedef KisSharedPtr<KisQImagemask> KisQImagemaskSP;
-
 class KisFilter;
 typedef KisSharedPtr<KisFilter> KisFilterSP;
 
+class KisGenerator;
+typedef KisSharedPtr<KisGenerator> KisGeneratorSP;
+
+class KisConvolutionKernel;
+typedef KisSharedPtr<KisConvolutionKernel> KisConvolutionKernelSP;
+
 class KisAnnotation;
 typedef KisSharedPtr<KisAnnotation> KisAnnotationSP;
 typedef KisSharedPtrVector<KisAnnotation> vKisAnnotationSP;
@@ -172,6 +172,13 @@ typedef KisRectIteratorPixelBase<KisRectConstIterator, const quint8*> KisRectCon
 class KisRectIterator;
 typedef KisRectIteratorPixelBase<KisRectIterator, quint8*> KisRectIteratorPixel;
 
+// Repeat iterators
+template<class T> class KisRepeatHLineIteratorPixelBase;
+typedef KisRepeatHLineIteratorPixelBase< KisHLineConstIteratorPixel > KisRepeatHLineConstIteratorPixel;
+template<class T> class KisRepeatVLineIteratorPixelBase;
+typedef KisRepeatVLineIteratorPixelBase< KisVLineConstIteratorPixel > KisRepeatVLineConstIteratorPixel;
+
+// Random accessor
 template<class T, typename TSelect> class KisRandomAccessorPixelBase;
 class KisRandomConstAccessor;
 typedef KisRandomAccessorPixelBase<KisRandomConstAccessor, const quint8*> KisRandomConstAccessorPixel;
@@ -180,4 +187,12 @@ typedef KisRandomAccessorPixelBase<KisRandomAccessor, quint8*> KisRandomAccessor
 
 typedef QVector<QPointF> vQPointF;
 
+class KisPaintOpPreset;
+typedef KisSharedPtr<KisPaintOpPreset> KisPaintOpPresetSP;
+
+class KisPaintOpSettings;
+typedef KisSharedPtr<KisPaintOpSettings> KisPaintOpSettingsSP;
+
 #endif // KISTYPES_H_
+
+#include <kis_shared_ptr.h>
