@@ -17,18 +17,13 @@
  */
 
 #include "kis_meta_data_entry.h"
-
 #include <QString>
 
-#include <kdebug.h>
+#include <kis_debug.h>
 
 #include "kis_meta_data_value.h"
 #include "kis_meta_data_schema.h"
 
-#ifdef Q_CC_MSVC
-#include <iso646.h>
-#endif
-
 using namespace KisMetaData;
 
 struct Entry::Private {
@@ -41,13 +36,15 @@ struct Entry::Private {
 Entry::Entry() :
         d(new Private)
 {
+    d->name = "";
     d->schema = 0;
     d->valid = false;
 }
 
-Entry::Entry(QString name, const Schema* schema, const Value& value) :
+Entry::Entry(const Schema* schema, QString name, const Value& value) :
         d(new Private)
 {
+    Q_ASSERT(!name.isEmpty());
     d->name = name;
     d->schema = schema;
     d->value = value;
@@ -72,6 +69,7 @@ QString Entry::name() const
 
 const Schema* Entry::schema() const
 {
+    Q_ASSERT(d->schema);
     return d->schema;
 }
 
@@ -82,7 +80,7 @@ void Entry::setSchema(const KisMetaData::Schema* schema)
 
 QString Entry::qualifiedName() const
 {
-    return d->schema->generateQualifiedName( d->name);
+    return d->schema->generateQualifiedName(d->name);
 }
 
 const Value& Entry::value() const
@@ -95,7 +93,8 @@ Value& Entry::value()
     return d->value;
 }
 
-bool Entry::isValid() const {
+bool Entry::isValid() const
+{
     return d->valid;
 }
 
@@ -105,11 +104,10 @@ bool Entry::operator==(const Entry& e)
     return qualifiedName() == e.qualifiedName();
 }
 
-Entry& Entry::operator=(const Entry& e)
+Entry& Entry::operator=(const Entry & e)
 {
-    if(e.isValid())
-    {
-        Q_ASSERT( not isValid() or *this == e);
+    if (e.isValid()) {
+        Q_ASSERT(!isValid() || *this == e);
         d->name = e.d->name;
         d->schema = e.d->schema;
         d->value = e.d->value;
@@ -118,8 +116,8 @@ Entry& Entry::operator=(const Entry& e)
     return *this;
 }
 
-QDebug operator<<(QDebug dbg, const Entry &c)
+QDebug operator<<(QDebug debug, const Entry &c)
 {
-    dbg.nospace() << "Name: " << c.name() << " Qualified name: " << c.qualifiedName() << " Value: " << c.value();
-    return dbg.space();
+    debug.nospace() << "Name: " << c.name() << " Qualified name: " << c.qualifiedName() << " Value: " << c.value();
+    return debug.space();
 }
