@@ -17,20 +17,35 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  */
+
 #include <kcmdlineargs.h>
 #include <KoApplication.h>
 #include <krita_export.h>
 #include <stdlib.h>
 #include <stdio.h>
-#include <kdebug.h>
+#include <kis_debug.h>
 #include <QString>
 
 #include "ui/kis_aboutdata.h"
 
+#ifndef NDEBUG
+static void qtMessageHandler(QtMsgType type, const char *msg)
+{
+    fprintf(stderr, "%s\n", msg);
+    if (type == QtFatalMsg)
+        abort();
+}
+#endif
+
 extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
 {
+    int state;
+    KAboutData * aboutData=newKritaAboutData();
 
-    KCmdLineArgs::init(argc, argv, newKritaAboutData());
+#ifndef NDEBUG
+    qInstallMsgHandler(qtMessageHandler);
+#endif
+    KCmdLineArgs::init(argc, argv, aboutData);
 
     KCmdLineOptions options;
     options.add("+[file(s)]", ki18n("File(s) or URL(s) to open"));
@@ -41,6 +56,10 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
     if (!app.start())
         return 1;
 
-    return app.exec();
+    state=app.exec();
+
+    delete (aboutData);
+
+    return state;
 }
 
