@@ -19,14 +19,13 @@
  */
 
 #include "blur.h"
-
 #include <kgenericfactory.h>
 
 #include "kis_blur_filter.h"
-#include "kis_filter_registry.h"
+#include "filter/kis_filter_registry.h"
 
 typedef KGenericFactory<BlurFilterPlugin> BlurFilterPluginFactory;
-K_EXPORT_COMPONENT_FACTORY( kritablurfilter, BlurFilterPluginFactory( "krita" ) )
+K_EXPORT_COMPONENT_FACTORY(kritablurfilter, BlurFilterPluginFactory("krita"))
 
 BlurFilterPlugin::BlurFilterPlugin(QObject *parent, const QStringList &)
         : KParts::Plugin(parent)
