@@ -19,14 +19,15 @@
  */
 
 #include "kis_minmax_filters.h"
-
 #include <KoProgressUpdater.h>
 
 #include <kis_iterators_pixel.h>
-#include <kis_paint_device.h>
+#include <filter/kis_filter_configuration.h>
 #include <kis_selection.h>
+#include <kis_paint_device.h>
+#include <kis_processing_information.h>
 
-typedef void (*funcMaxMin)(const quint8* , quint8* , uint );
+typedef void (*funcMaxMin)(const quint8* , quint8* , uint);
 
 template<typename _TYPE>
 void maximize(const quint8* s, quint8* d, uint nbpixels)
@@ -34,17 +35,13 @@ void maximize(const quint8* s, quint8* d, uint nbpixels)
     const _TYPE* sT = (_TYPE*)(s);
     _TYPE* dT = (_TYPE*)(d);
     _TYPE vmax = *sT;
-    for(uint i = 1; i < nbpixels; i ++)
-    {
-        if(sT[i] > vmax)
-        {
+    for (uint i = 1; i < nbpixels; i ++) {
+        if (sT[i] > vmax) {
             vmax = sT[i];
         }
     }
-    for(uint i = 0; i < nbpixels; i ++)
-    {
-        if(dT[i] != vmax)
-        {
+    for (uint i = 0; i < nbpixels; i ++) {
+        if (dT[i] != vmax) {
             dT[i] = 0;
         }
     }
@@ -56,17 +53,13 @@ void minimize(const quint8* s, quint8* d, uint nbpixels)
     const _TYPE* sT = (_TYPE*)(s);
     _TYPE* dT = (_TYPE*)(d);
     _TYPE vmin = *sT;
-    for(uint i = 1; i < nbpixels; i ++)
-    {
-        if(sT[i] < vmin)
-        {
+    for (uint i = 1; i < nbpixels; i ++) {
+        if (sT[i] < vmin) {
             vmin = sT[i];
         }
     }
-    for(uint i = 0; i < nbpixels; i ++)
-    {
-        if(dT[i] != vmin)
-        {
+    for (uint i = 0; i < nbpixels; i ++) {
+        if (dT[i] != vmin) {
             dT[i] = 0;
         }
     }
@@ -80,71 +73,14 @@ KisFilterMax::KisFilterMax() : KisFilter(id(), CategoryColors, i18n("M&aximize C
     setColorSpaceIndependence(FULLY_INDEPENDENT);
 }
 
-void KisFilterMax::process(KisFilterConstantProcessingInformation srcInfo,
-                 KisFilterProcessingInformation dstInfo,
-                 const QSize& size,
-                 const KisFilterConfiguration* config,
-                 KoUpdater* progressUpdater
-        ) const
-{
-    const KisPaintDeviceSP src = srcInfo.paintDevice();
-    KisPaintDeviceSP dst = dstInfo.paintDevice();
-    QPoint dstTopLeft = dstInfo.topLeft();
-    QPoint srcTopLeft = srcInfo.topLeft();
-    Q_ASSERT(src != 0);
-    Q_ASSERT(dst != 0);
-
-    KisRectIteratorPixel dstIt = dst->createRectIterator(dstTopLeft.x(), dstTopLeft.y(), size.width(), size.height(), dstInfo.selection() );
-    KisRectConstIteratorPixel srcIt = src->createRectConstIterator(srcTopLeft.x(), srcTopLeft.y(), size.width(), size.height(), srcInfo.selection());
-
-    int pixelsProcessed = 0;
-    int totalCost = size.width() * size.height() / 100;
-
-    const KoColorSpace * cs = src->colorSpace();
-    qint32 nC = cs->colorChannelCount();
-
-    funcMaxMin F;
-    KoChannelInfo::enumChannelValueType cT = cs->channels()[0]->channelValueType();
-    if( cT == KoChannelInfo::UINT8 || cT == KoChannelInfo::INT8 )
-    {
-        F = & maximize<quint8>;
-    } else if( cT == KoChannelInfo::UINT16 || cT == KoChannelInfo::INT16 )
-    {
-        F = & maximize<quint16>;
-    } else if( cT == KoChannelInfo::FLOAT32 )
-    {
-        F = & maximize<float>;
-    } else {
-        return;
-    }
-
-    while( ! srcIt.isDone() )
-    {
-        if(srcIt.isSelected())
-        {
-            F( srcIt.oldRawData(), dstIt.rawData(), nC);
-        }
-        if(progressUpdater) progressUpdater->setProgress((++pixelsProcessed) / totalCost);
-        ++srcIt;
-        ++dstIt;
-    }
-}
-
-KisFilterMin::KisFilterMin() : KisFilter(id(), CategoryColors, i18n("M&inimize Channel"))
-{
-    setSupportsPainting(true);
-    setSupportsPreview(true);
-    setSupportsIncrementalPainting(false);
-    setColorSpaceIndependence(FULLY_INDEPENDENT);
-}
-
-void KisFilterMin::process(KisFilterConstantProcessingInformation srcInfo,
-                 KisFilterProcessingInformation dstInfo,
-                 const QSize& size,
-                 const KisFilterConfiguration* config,
-                 KoUpdater* progressUpdater
-        ) const
+void KisFilterMax::process(KisConstProcessingInformation srcInfo,
+                           KisProcessingInformation dstInfo,
+                           const QSize& size,
+                           const KisFilterConfiguration* config,
+                           KoUpdater* progressUpdater
+                          ) const
 {
+    Q_UNUSED(config);
     const KisPaintDeviceSP src = srcInfo.paintDevice();
     KisPaintDeviceSP dst = dstInfo.paintDevice();
     QPoint dstTopLeft = dstInfo.topLeft();
@@ -163,26 +99,76 @@ void KisFilterMin::process(KisFilterConstantProcessingInformation srcInfo,
 
     funcMaxMin F;
     KoChannelInfo::enumChannelValueType cT = cs->channels()[0]->channelValueType();
-    if( cT == KoChannelInfo::UINT8 || cT == KoChannelInfo::INT8 )
-    {
-        F = & minimize<quint8>;
-    } else if( cT == KoChannelInfo::UINT16 || cT == KoChannelInfo::INT16 )
-    {
-        F = & minimize<quint16>;
-    } else if( cT == KoChannelInfo::FLOAT32 )
-    {
-        F = & minimize<float>;
+    if (cT == KoChannelInfo::UINT8 || cT == KoChannelInfo::INT8) {
+        F = & maximize<quint8>;
+    } else if (cT == KoChannelInfo::UINT16 || cT == KoChannelInfo::INT16) {
+        F = & maximize<quint16>;
+    } else if (cT == KoChannelInfo::FLOAT32) {
+        F = & maximize<float>;
     } else {
         return;
     }
 
-    while( ! srcIt.isDone() )
-    {
-        if(srcIt.isSelected())
-        {
-            F( srcIt.oldRawData(), dstIt.rawData(), nC);
+    while (! srcIt.isDone()) {
+        if (srcIt.isSelected()) {
+            F(srcIt.oldRawData(), dstIt.rawData(), nC);
         }
-        if(progressUpdater) progressUpdater->setProgress((++pixelsProcessed) / totalCost);
+        if (progressUpdater) progressUpdater->setProgress((++pixelsProcessed) / totalCost);
+        ++srcIt;
+        ++dstIt;
+    }
+}
+
+KisFilterMin::KisFilterMin() : KisFilter(id(), CategoryColors, i18n("M&inimize Channel"))
+{
+    setSupportsPainting(true);
+    setSupportsPreview(true);
+    setSupportsIncrementalPainting(false);
+    setColorSpaceIndependence(FULLY_INDEPENDENT);
+}
+
+void KisFilterMin::process(KisConstProcessingInformation srcInfo,
+                           KisProcessingInformation dstInfo,
+                           const QSize& size,
+                           const KisFilterConfiguration* config,
+                           KoUpdater* progressUpdater
+                          ) const
+{
+    Q_UNUSED(config);
+    const KisPaintDeviceSP src = srcInfo.paintDevice();
+    KisPaintDeviceSP dst = dstInfo.paintDevice();
+    QPoint dstTopLeft = dstInfo.topLeft();
+    QPoint srcTopLeft = srcInfo.topLeft();
+    Q_ASSERT(src != 0);
+    Q_ASSERT(dst != 0);
+
+    KisRectIteratorPixel dstIt = dst->createRectIterator(dstTopLeft.x(), dstTopLeft.y(), size.width(), size.height(), dstInfo.selection());
+    KisRectConstIteratorPixel srcIt = src->createRectConstIterator(srcTopLeft.x(), srcTopLeft.y(), size.width(), size.height(), srcInfo.selection());
+
+    int pixelsProcessed = 0;
+    int totalCost = size.width() * size.height() / 100;
+    if (totalCost == 0) totalCost = 1;
+
+    const KoColorSpace * cs = src->colorSpace();
+    qint32 nC = cs->colorChannelCount();
+
+    funcMaxMin F;
+    KoChannelInfo::enumChannelValueType cT = cs->channels()[0]->channelValueType();
+    if (cT == KoChannelInfo::UINT8 || cT == KoChannelInfo::INT8) {
+        F = & minimize<quint8>;
+    } else if (cT == KoChannelInfo::UINT16 || cT == KoChannelInfo::INT16) {
+        F = & minimize<quint16>;
+    } else if (cT == KoChannelInfo::FLOAT32) {
+        F = & minimize<float>;
+    } else {
+        return;
+    }
+
+    while (! srcIt.isDone()) {
+        if (srcIt.isSelected()) {
+            F(srcIt.oldRawData(), dstIt.rawData(), nC);
+        }
+        if (progressUpdater) progressUpdater->setProgress((++pixelsProcessed) / totalCost);
         ++srcIt;
         ++dstIt;
     }
