@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. */
 
-#ifndef BRIGHTNESSCONTRAST_H
-#define BRIGHTNESSCONTRAST_H
+#ifndef COLORSFILTERS_H
+#define COLORSFILTERS_H
 
 #include <kparts/plugin.h>
 #include "kis_perchannel_filter.h"
@@ -33,45 +33,43 @@ public:
     virtual ~ColorsFilters();
 };
 
-class KisAutoContrast : public KisFilter {
+class KisAutoContrast : public KisFilter
+{
 public:
     KisAutoContrast();
 public:
-    void process(KisFilterConstantProcessingInformation src,
-                 KisFilterProcessingInformation dst,
+    using KisFilter::process;
+    void process(KisConstProcessingInformation src,
+                 KisProcessingInformation dst,
                  const QSize& size,
                  const KisFilterConfiguration* config,
-                 KoUpdater* progressUpdater = 0
-        ) const;
-    static inline KoID id() { return KoID("autocontrast", i18n("Auto Contrast")); }
-    virtual bool supportsPreview() const { return true; }
-    virtual bool supportsPainting() const { return false; }
-    virtual bool supportsThreading() const { return false; }
-    
-    virtual ColorSpaceIndependence colorSpaceIndependence() const { return TO_LAB16; }
-    virtual bool workWith(KoColorSpace* cs);
+                 KoUpdater* progressUpdater
+                ) const;
+    static inline KoID id() {
+        return KoID("autocontrast", i18n("Auto Contrast"));
+    }
+    virtual bool workWith(const KoColorSpace* cs) const;
 
 };
 
 
-class KisDesaturateFilter : public KisFilter {
+class KisDesaturateFilter : public KisFilter
+{
 public:
     KisDesaturateFilter();
     ~KisDesaturateFilter();
 public:
-    void process(KisFilterConstantProcessingInformation src,
-                 KisFilterProcessingInformation dst,
+    using KisFilter::process;
+    void process(KisConstProcessingInformation src,
+                 KisProcessingInformation dst,
                  const QSize& size,
                  const KisFilterConfiguration* config,
-                 KoUpdater* progressUpdater = 0
-        ) const;
-    static inline KoID id() { return KoID("desaturate", i18n("Desaturate")); }
-    virtual bool supportsPainting() const { return true; }
-    virtual bool supportsPreview() const { return true; }
-    virtual bool supportsIncrementalPainting() const { return false; }
-    
-    virtual ColorSpaceIndependence colorSpaceIndependence() const { return TO_LAB16; }
-    virtual bool workWith(KoColorSpace* cs);
+                 KoUpdater* progressUpdater
+                ) const;
+    static inline KoID id() {
+        return KoID("desaturate", i18n("Desaturate"));
+    }
+    virtual bool workWith(const KoColorSpace* cs) const;
 
 };
 
