@@ -23,10 +23,14 @@
 
 #include <QList>
 
-#include "kis_filter.h"
-#include "kis_filter_config_widget.h"
+#include "filter/kis_filter.h"
+#include "kis_config_widget.h"
 #include "ui_wdg_brightness_contrast.h"
-
+#include <filter/kis_filter_configuration.h>
+#include <kis_selection.h>
+#include <kis_paint_device.h>
+#include <kis_processing_information.h>
+#include <KoProgressUpdater.h>
 class QWidget;
 class KoColorTransformation;
 
@@ -34,17 +38,21 @@ class WdgBrightnessContrast : public QWidget, public Ui::WdgBrightnessContrast
 {
     Q_OBJECT
 
-    public:
-        WdgBrightnessContrast(QWidget *parent) : QWidget(parent) { setupUi(this); }
+public:
+    WdgBrightnessContrast(QWidget *parent) : QWidget(parent) {
+        setupUi(this);
+    }
 };
 
-class KisBrightnessContrastFilterConfiguration : public KisFilterConfiguration {
+class KisBrightnessContrastFilterConfiguration : public KisFilterConfiguration
+{
 
 public:
+    using KisFilterConfiguration::fromXML;
 
     KisBrightnessContrastFilterConfiguration();
     virtual ~KisBrightnessContrastFilterConfiguration();
-    virtual void fromXML( const QString&  );
+    virtual void fromXML(const QString&);
     virtual QString toString();
 
 public:
@@ -64,30 +72,37 @@ public:
     KisBrightnessContrastFilter();
 
 public:
+    using KisFilter::process;
 
-    virtual KisFilterConfigWidget * createConfigurationWidget(QWidget* parent, KisPaintDeviceSP dev);
-    virtual KisFilterConfiguration * configuration() { return new KisBrightnessContrastFilterConfiguration(); }
-    void process(KisFilterConstantProcessingInformation src,
-                 KisFilterProcessingInformation dst,
+    void process(KisConstProcessingInformation src,
+                 KisProcessingInformation dst,
                  const QSize& size,
                  const KisFilterConfiguration* config,
-                 KoUpdater* progressUpdater = 0
-        ) const;
-    static inline KoID id() { return KoID("brightnesscontrast", i18n("Brightness / Contrast")); }
-    virtual KisFilterConfiguration* factoryConfiguration(const KisPaintDeviceSP);
-    
-    virtual bool workWith(KoColorSpace* cs);
+                 KoUpdater* progressUpdater
+                ) const;
+    static inline KoID id() {
+        return KoID("brightnesscontrast", i18n("Brightness / Contrast"));
+    }
+    virtual KisFilterConfiguration* factoryConfiguration(const KisPaintDeviceSP) const;
+
+    virtual KisConfigWidget * createConfigurationWidget(QWidget* parent, const KisPaintDeviceSP dev, const KisImageSP image = 0) const;
+    virtual KisPropertiesConfiguration * configuration() {
+        return new KisBrightnessContrastFilterConfiguration();
+    }
+
+    virtual bool workWith(const KoColorSpace* cs) const;
 };
 
 
-class KisBrightnessContrastConfigWidget : public KisFilterConfigWidget {
+class KisBrightnessContrastConfigWidget : public KisConfigWidget
+{
 
 public:
-    KisBrightnessContrastConfigWidget(QWidget * parent, KisPaintDeviceSP dev, Qt::WFlags f = 0 );
+    KisBrightnessContrastConfigWidget(QWidget * parent, KisPaintDeviceSP dev, Qt::WFlags f = 0);
     virtual ~KisBrightnessContrastConfigWidget() {}
 
     virtual KisBrightnessContrastFilterConfiguration * configuration() const;
-    virtual void setConfiguration( KisFilterConfiguration * config );
+    virtual void setConfiguration( const KisPropertiesConfiguration* config);
     WdgBrightnessContrast * m_page;
 };
 
