@@ -17,6 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
+
+#include "kis_perchannel_filter.h"
 #include <QLayout>
 #include <QPixmap>
 #include <QPainter>
@@ -30,38 +32,40 @@
 #include "KoColorTransformation.h"
 #include "KoCompositeOp.h"
 #include "KoID.h"
+#include <KoProgressUpdater.h>
 
 #include "kis_bookmarked_configuration_manager.h"
-#include "kis_filter_configuration.h"
-#include "kis_filter_config_widget.h"
-#include "kis_perchannel_filter.h"
-#include "kis_paint_device.h"
+#include "kis_config_widget.h"
+#include <filter/kis_filter_configuration.h>
+#include <kis_selection.h>
+#include <kis_paint_device.h>
+#include <kis_processing_information.h>
+
 #include "kis_iterators_pixel.h"
-#include "kcurve.h"
+#include "widgets/kcurve.h"
 #include "kis_histogram.h"
 #include "kis_painter.h"
 
-class KisPerChannelFilterConfigurationFactory : public KisFilterConfigurationFactory {
-    public:
-        KisPerChannelFilterConfigurationFactory() :KisFilterConfigurationFactory("perchannel", 1) {}
-        virtual ~KisPerChannelFilterConfigurationFactory() { }
-        virtual KisSerializableConfiguration* createDefault()
-        {
-            return new KisPerChannelFilterConfiguration(0);
-        }
-        virtual KisSerializableConfiguration* create(const QDomElement& e)
-        {
-            KisFilterConfiguration* fc = new KisPerChannelFilterConfiguration(0);
-            fc->fromXML( e );
-            return fc;
-        }
+class KisPerChannelFilterConfigurationFactory : public KisFilterConfigurationFactory
+{
+public:
+    KisPerChannelFilterConfigurationFactory() : KisFilterConfigurationFactory("perchannel", 1) {}
+    virtual ~KisPerChannelFilterConfigurationFactory() { }
+    virtual KisSerializableConfiguration* createDefault() {
+        return new KisPerChannelFilterConfiguration(0);
+    }
+    virtual KisSerializableConfiguration* create(const QDomElement& e) {
+        KisFilterConfiguration* fc = new KisPerChannelFilterConfiguration(0);
+        fc->fromXML(e);
+        return fc;
+    }
 };
 
 KisPerChannelFilterConfiguration::KisPerChannelFilterConfiguration(int n)
-    : KisFilterConfiguration( "perchannel", 1 )
+        : KisFilterConfiguration("perchannel", 1)
 {
     transfers = new quint16*[n];
-    for(int i = 0; i < n; i++) {
+    for (int i = 0; i < n; i++) {
         curves.append(KisCurve());
 
         transfers[i] = new quint16[256];
@@ -78,7 +82,7 @@ KisPerChannelFilterConfiguration::KisPerChannelFilterConfiguration(int n)
 
 KisPerChannelFilterConfiguration::~KisPerChannelFilterConfiguration()
 {
-    for(int i=0;i<nTransfers;i++)
+    for (int i = 0;i < nTransfers;i++)
         delete [] transfers[i];
     delete[] transfers;
     delete adjustment;
@@ -86,13 +90,14 @@ KisPerChannelFilterConfiguration::~KisPerChannelFilterConfiguration()
 
 bool KisPerChannelFilterConfiguration::isCompatible(const KisPaintDeviceSP dev) const
 {
-    return dev->colorSpace() == oldCs;
+    if (!oldCs) return false;
+    return *dev->colorSpace() == *oldCs;
 }
 
-void KisPerChannelFilterConfiguration::fromXML( const QString& s )
+void KisPerChannelFilterConfiguration::fromXML(const QString& s)
 {
     QDomDocument doc;
-    doc.setContent( s );
+    doc.setContent(s);
     QDomElement e = doc.documentElement();
     QDomNode n = e.firstChild();
 
@@ -119,13 +124,13 @@ void KisPerChannelFilterConfiguration::fromXML( const QString& s )
                     QDomElement curvesElement = curvesNode.toElement();
 
                     if (!curvesElement.isNull() && !curvesElement.text().isEmpty()) {
-                        QStringList data = curvesElement.text().split( ";" );
+                        QStringList data = curvesElement.text().split(';');
 
-                        foreach (QString pair, data) {
-                            if (pair.indexOf(",") > -1) {
+                        foreach(const QString & pair, data) {
+                            if (pair.indexOf(',') > -1) {
                                 QPointF p;
-                                p.rx() = pair.section(",", 0, 0).toDouble();
-                                p.ry() = pair.section(",", 1, 1).toDouble();
+                                p.rx() = pair.section(',', 0, 0).toDouble();
+                                p.ry() = pair.section(',', 1, 1).toDouble();
                                 curves[count].append(p);
                             }
                         }
@@ -138,17 +143,15 @@ void KisPerChannelFilterConfiguration::fromXML( const QString& s )
         n = n.nextSibling();
     }
 
-    for(int ch = 0; ch < nTransfers; ++ch)
-    {
+    for (int ch = 0; ch < nTransfers; ++ch) {
         transfers[ch] = new quint16[256];
-        for(int i = 0; i < 256; ++i)
-        {
+        for (int i = 0; i < 256; ++i) {
             qint32 val;
             val = int(0xFFFF * KCurve::getCurveValue(curves[ch], i /
-255.0));
-            if(val > 0xFFFF)
+                      255.0));
+            if (val > 0xFFFF)
                 val = 0xFFFF;
-            if(val < 0)
+            if (val < 0)
                 val = 0;
 
             transfers[ch][i] = val;
@@ -160,9 +163,9 @@ void KisPerChannelFilterConfiguration::fromXML( const QString& s )
 QString KisPerChannelFilterConfiguration::toString()
 {
     QDomDocument doc = QDomDocument("filterconfig");
-    QDomElement root = doc.createElement( "filterconfig" );
-    root.setAttribute( "name", name() );
-    root.setAttribute( "version", version() );
+    QDomElement root = doc.createElement("filterconfig");
+    root.setAttribute("name", name());
+    root.setAttribute("version", version());
 
     QDomElement c = doc.createElement("curves");
     c.setAttribute("number", nTransfers);
@@ -171,8 +174,8 @@ QString KisPerChannelFilterConfiguration::toString()
         QDomElement t = doc.createElement("curve");
         KisCurve curve = curves[i];
         QString sCurve;
-        QPointF pair;
-        foreach (pair, curve) {
+
+        foreach(const QPointF & pair, curve) {
             sCurve += QString::number(pair.x());
             sCurve += ',';
             sCurve += QString::number(pair.y());
@@ -183,30 +186,32 @@ QString KisPerChannelFilterConfiguration::toString()
         c.appendChild(t);
     }
     root.appendChild(c);
-    doc.appendChild( root );
+    doc.appendChild(root);
     return doc.toString();
 }
 
-KisPerChannelFilter::KisPerChannelFilter() : KisFilter( id(), CategoryAdjust, i18n("&Color Adjustment curves..."))
+KisPerChannelFilter::KisPerChannelFilter() : KisFilter(id(), CategoryAdjust, i18n("&Color Adjustment curves..."))
 {
     setSupportsPainting(true);
     setSupportsPreview(true);
     setSupportsIncrementalPainting(false);
     setColorSpaceIndependence(TO_LAB16);
-    setBookmarkManager(new KisBookmarkedConfigurationManager(configEntryGroup(), new KisPerChannelFilterConfigurationFactory()));
+    setBookmarkManager(new KisBookmarkedConfigurationManager(configEntryGroup(),
+                       new KisPerChannelFilterConfigurationFactory()));
 }
 
-KisFilterConfigWidget * KisPerChannelFilter::createConfigurationWidget(QWidget *parent, KisPaintDeviceSP dev)
+KisConfigWidget * KisPerChannelFilter::createConfigurationWidget(QWidget *parent, KisPaintDeviceSP dev, const KisImageSP image) const
 {
+    Q_UNUSED(image);
     return new KisPerChannelConfigWidget(parent, dev);
 }
 
-void KisPerChannelFilter::process(KisFilterConstantProcessingInformation srcInfo,
-                 KisFilterProcessingInformation dstInfo,
-                 const QSize& size,
-                 const KisFilterConfiguration* config,
-                 KoUpdater* progressUpdater
-        ) const
+void KisPerChannelFilter::process(KisConstProcessingInformation srcInfo,
+                                  KisProcessingInformation dstInfo,
+                                  const QSize& size,
+                                  const KisFilterConfiguration* config,
+                                  KoUpdater* progressUpdater
+                                 ) const
 {
     const KisPaintDeviceSP src = srcInfo.paintDevice();
     KisPaintDeviceSP dst = dstInfo.paintDevice();
@@ -215,12 +220,14 @@ void KisPerChannelFilter::process(KisFilterConstantProcessingInformation srcInfo
     Q_ASSERT(src != 0);
     Q_ASSERT(dst != 0);
     if (!config) {
-        kWarning() <<"No configuration object for per-channel filter";
+        warnKrita << "No configuration object for per-channel filter";
         return;
     }
 
     KisPerChannelFilterConfiguration* configBC =
-            const_cast<KisPerChannelFilterConfiguration*>(static_cast<const KisPerChannelFilterConfiguration*>(config)); // Somehow, this shouldn't happen
+        const_cast<KisPerChannelFilterConfiguration*>(dynamic_cast<const KisPerChannelFilterConfiguration*>(config)); // Somehow, this shouldn't happen
+    if (not configBC)
+        return;
     if (configBC->nTransfers != src->colorSpace()->colorChannelCount()) {
         // We got an illegal number of colorchannels.KisFilter
         return;
@@ -229,8 +236,8 @@ void KisPerChannelFilter::process(KisFilterConstantProcessingInformation srcInfo
     if (configBC->dirty || (src->colorSpace() != configBC->oldCs)) {
         delete configBC->adjustment;
         configBC->adjustment =
-                src->colorSpace()->createPerChannelAdjustment(configBC->transfers);
-//         kDebug() << configBC->adjustment;
+            src->colorSpace()->createPerChannelAdjustment(configBC->transfers);
+//         dbgKrita << configBC->adjustment;
         configBC->oldCs = src->colorSpace();
         configBC->dirty = false;
     }
@@ -238,64 +245,59 @@ void KisPerChannelFilter::process(KisFilterConstantProcessingInformation srcInfo
     KoColorTransformation *adj = configBC->adjustment;
 
 
-    if (src!=dst) {
+    if (src != dst) {
         KisPainter gc(dst, dstInfo.selection());
         gc.bitBlt(dstTopLeft.x(), dstTopLeft.y(), COMPOSITE_COPY, src, srcTopLeft.x(), srcTopLeft.y(), size.width(), size.height());
         gc.end();
     }
 
-    KisRectIteratorPixel iter = dst->createRectIterator(dstTopLeft.x(), dstTopLeft.y(), size.width(), size.height(), dstInfo.selection() );
+    KisRectIteratorPixel iter = dst->createRectIterator(dstTopLeft.x(), dstTopLeft.y(), size.width(), size.height(), dstInfo.selection());
     KoMixColorsOp * mixOp = src->colorSpace()->mixColorsOp();
     qint32 totalCost = (size.width() * size.height()) / 100;
+    if (totalCost == 0) totalCost = 1;
     qint32 pixelsProcessed = 0;
 
-    while( not iter.isDone()  and (progressUpdater and not progressUpdater->interrupted()))
-    {
-        quint32 npix=0, maxpix = iter.nConseqPixels();
+    while (!iter.isDone()  && (progressUpdater && !progressUpdater->interrupted())) {
+        quint32 npix = 0, maxpix = iter.nConseqPixels();
         quint8 selectedness = iter.selectedness();
         // The idea here is to handle stretches of completely selected and completely unselected pixels.
         // Partially selected pixels are handled one pixel at a time.
-        switch(selectedness)
-        {
-            case MIN_SELECTED:
-                while(iter.selectedness()==MIN_SELECTED && maxpix)
-                {
-                    --maxpix;
-                    if (maxpix != 0)
-                        ++iter;
-
-                    ++npix;
-                }
-                pixelsProcessed += npix;
+        switch (selectedness) {
+        case MIN_SELECTED:
+            while (iter.selectedness() == MIN_SELECTED && maxpix) {
+                --maxpix;
                 ++iter;
-                break;
-
-            case MAX_SELECTED:
-            {
-                quint8 *firstPixel = iter.rawData();
-                while(iter.selectedness()==MAX_SELECTED && maxpix)
-                {
-                    --maxpix;
-                    ++iter;
-                    ++npix;
-                }
-                // adjust
-                adj->transform(firstPixel, firstPixel, npix);
-                pixelsProcessed += npix;
-                break;
+                ++npix;
             }
+            pixelsProcessed += npix;
+            break;
 
-            default:
-                // adjust, but since it's partially selected we also only partially adjust
-                adj->transform(iter.oldRawData(), iter.rawData(), 1);
-                const quint8 *pixels[2] = {iter.oldRawData(), iter.rawData()};
-                quint8 weights[2] = {MAX_SELECTED - selectedness, selectedness};
-                mixOp->mixColors(pixels, weights, 2, iter.rawData());
-                ++iter;
-                pixelsProcessed++;
-                break;
+        case MAX_SELECTED: {
+            quint8 *firstPixel = iter.rawData();
+            while (iter.selectedness() == MAX_SELECTED && maxpix) {
+                --maxpix;
+                if (maxpix != 0)
+                    ++iter;
+                ++npix;
+            }
+            // adjust
+            adj->transform(firstPixel, firstPixel, npix);
+            pixelsProcessed += npix;
+            ++iter;
+            break;
         }
-        if(progressUpdater) progressUpdater->setProgress(pixelsProcessed / totalCost);
+
+        default:
+            // adjust, but since it's partially selected we also only partially adjust
+            adj->transform(iter.oldRawData(), iter.rawData(), 1);
+            const quint8 *pixels[2] = {iter.oldRawData(), iter.rawData()};
+            qint16 weights[2] = {MAX_SELECTED - selectedness, selectedness};
+            mixOp->mixColors(pixels, weights, 2, iter.rawData());
+            ++iter;
+            pixelsProcessed++;
+            break;
+        }
+        if (progressUpdater) progressUpdater->setProgress(pixelsProcessed / totalCost);
     }
 
 }
@@ -307,7 +309,7 @@ void KisPerChannelConfigWidget::setActiveChannel(int ch)
     QPixmap pix(256, height);
     pix.fill();
     QPainter p(&pix);
-    p.setPen(QPen::QPen(Qt::gray,1, Qt::SolidLine));
+    p.setPen(QPen::QPen(Qt::gray, 1, Qt::SolidLine));
 
     m_histogram->setChannel(ch);
 
@@ -316,12 +318,12 @@ void KisPerChannelConfigWidget::setActiveChannel(int ch)
 
     if (m_histogram->getHistogramType() == LINEAR) {
         double factor = (double)height / highest;
-        for( i=0; i<bins; ++i ) {
+        for (i = 0; i < bins; ++i) {
             p.drawLine(i, height, i, height - int(m_histogram->getValue(i) * factor));
         }
     } else {
         double factor = (double)height / (double)log(highest);
-        for( i = 0; i < bins; ++i ) {
+        for (i = 0; i < bins; ++i) {
             p.drawLine(i, height, i, height - int(log((double)m_histogram->getValue(i)) * factor));
         }
     }
@@ -334,8 +336,9 @@ void KisPerChannelConfigWidget::setActiveChannel(int ch)
 }
 
 KisPerChannelConfigWidget::KisPerChannelConfigWidget(QWidget * parent, KisPaintDeviceSP dev, Qt::WFlags f)
-    : KisFilterConfigWidget(parent, f)
+        : KisConfigWidget(parent, f)
 {
+    Q_ASSERT(dev);
     int i;
     int height;
     m_page = new WdgPerChannel(this);
@@ -345,8 +348,7 @@ KisPerChannelConfigWidget::KisPerChannelConfigWidget(QWidget * parent, KisPaintD
     m_dev = dev;
     m_curves.clear();
     m_activeCh = 0;
-    for(unsigned int ch = 0; ch < m_dev->colorSpace()->colorChannelCount(); ch++)
-    {
+    for (unsigned int ch = 0; ch < m_dev->colorSpace()->colorChannelCount(); ch++) {
         m_curves.append(KisCurve());
         m_curves[ch].append(QPointF(0, 0));
         m_curves[ch].append(QPointF(1, 1));
@@ -354,21 +356,20 @@ KisPerChannelConfigWidget::KisPerChannelConfigWidget(QWidget * parent, KisPaintD
 
     l->addWidget(m_page);
     height = 256;
-    connect(m_page->kCurve, SIGNAL(modified()), SIGNAL(sigPleaseUpdatePreview()));
+    connect(m_page->kCurve, SIGNAL(modified()), SIGNAL(sigConfigChanged()));
 
     // Fill in the channel chooser
     QList<KoChannelInfo *> channels = dev->colorSpace()->channels();
-    for(unsigned int val=0; val < dev->colorSpace()->colorChannelCount(); val++)
+    for (unsigned int val = 0; val < dev->colorSpace()->colorChannelCount(); val++)
         m_page->cmbChannel->addItem(channels.at(val)->name());
-    connect( m_page->cmbChannel, SIGNAL(activated(int)), this, SLOT(setActiveChannel(int)));
+    connect(m_page->cmbChannel, SIGNAL(activated(int)), this, SLOT(setActiveChannel(int)));
 
     // Create the horizontal gradient label
     QPixmap hgradientpix(256, 1);
     QPainter hgp(&hgradientpix);
-    hgp.setPen(QPen::QPen(QColor(0,0,0),1, Qt::SolidLine));
-    for( i=0; i<256; ++i )
-    {
-        hgp.setPen(QColor(i,i,i));
+    hgp.setPen(QPen::QPen(QColor(0, 0, 0), 1, Qt::SolidLine));
+    for (i = 0; i < 256; ++i) {
+        hgp.setPen(QColor(i, i, i));
         hgp.drawPoint(i, 0);
     }
     m_page->hgradient->setPixmap(hgradientpix);
@@ -376,11 +377,10 @@ KisPerChannelConfigWidget::KisPerChannelConfigWidget(QWidget * parent, KisPaintD
     // Create the vertical gradient label
     QPixmap vgradientpix(1, 256);
     QPainter vgp(&vgradientpix);
-    vgp.setPen(QPen::QPen(QColor(0,0,0),1, Qt::SolidLine));
-    for( i=0; i<256; ++i )
-    {
-        vgp.setPen(QColor(i,i,i));
-        vgp.drawPoint(0, 255-i);
+    vgp.setPen(QPen::QPen(QColor(0, 0, 0), 1, Qt::SolidLine));
+    for (i = 0; i < 256; ++i) {
+        vgp.setPen(QColor(i, i, i));
+        vgp.drawPoint(0, 255 - i);
     }
     m_page->vgradient->setPixmap(vgradientpix);
 
@@ -393,24 +393,22 @@ KisPerChannelConfigWidget::KisPerChannelConfigWidget(QWidget * parent, KisPaintD
     setActiveChannel(0);
 }
 
-KisFilterConfiguration * KisPerChannelConfigWidget::configuration() const
+KisPropertiesConfiguration * KisPerChannelConfigWidget::configuration() const
 {
     int nCh = m_dev->colorSpace()->colorChannelCount();
     KisPerChannelFilterConfiguration * cfg = new KisPerChannelFilterConfiguration(nCh);
 
     m_curves[m_activeCh] = m_page->kCurve->getCurve();
 
-    for(int ch = 0; ch < nCh; ch++)
-    {
+    for (int ch = 0; ch < nCh; ch++) {
         cfg->curves[ch] = m_curves[ch];
 
-        for(int i = 0; i < 256; i++)
-        {
+        for (int i = 0; i < 256; i++) {
             qint32 val;
             val = int(0xFFFF * m_page->kCurve->getCurveValue(m_curves[ch],  i / 255.0));
-            if ( val > 0xFFFF )
+            if (val > 0xFFFF)
                 val = 0xFFFF;
-            if ( val < 0 )
+            if (val < 0)
                 val = 0;
 
             cfg->transfers[ch][i] = val;
@@ -422,18 +420,17 @@ KisFilterConfiguration * KisPerChannelConfigWidget::configuration() const
     return cfg;
 }
 
-void KisPerChannelConfigWidget::setConfiguration(KisFilterConfiguration * config)
+void KisPerChannelConfigWidget::setConfiguration(const KisPropertiesConfiguration * config)
 {
-    KisPerChannelFilterConfiguration * cfg = dynamic_cast<KisPerChannelFilterConfiguration *>(config);
+    const KisPerChannelFilterConfiguration * cfg = dynamic_cast<const KisPerChannelFilterConfiguration *>(config);
     if (!cfg)
         return;
 
-    for(unsigned int ch = 0; ch < cfg->nTransfers; ch++)
-    {
+    for (unsigned int ch = 0; ch < cfg->nTransfers; ch++) {
         m_curves[ch] = cfg->curves[ch];
     }
     m_page->kCurve->setCurve(m_curves[m_activeCh]);
-    setActiveChannel( 0 );
+    setActiveChannel(0);
 }
 
 #include "kis_perchannel_filter.moc"
