@@ -19,31 +19,37 @@
  */
 
 #include "kis_convolution_filter.h"
-
 #include <klocale.h>
-#include <kdebug.h>
+#include <kis_debug.h>
 
 #include <KoCompositeOp.h>
 
 #include "kis_painter.h"
 #include "kis_convolution_painter.h"
+#include "kis_convolution_kernel.h"
 #include "KoProgressUpdater.h"
+#include <filter/kis_filter_configuration.h>
+#include <kis_selection.h>
+#include <kis_paint_device.h>
+#include <kis_processing_information.h>
 
 KisConvolutionFilter::KisConvolutionFilter(const KoID& id, const KoID & category, const QString & entry)
-    : KisFilter( id, category, entry )
+        : KisFilter(id, category, entry)
 {
-    setSupportsIncrementalPainting( false );
-    setColorSpaceIndependence( FULLY_INDEPENDENT);
+    setSupportsIncrementalPainting(false);
+    setColorSpaceIndependence(FULLY_INDEPENDENT);
 }
 
 
-void KisConvolutionFilter::process(KisFilterConstantProcessingInformation srcInfo,
-                 KisFilterProcessingInformation dstInfo,
-                 const QSize& size,
-                 const KisFilterConfiguration* config,
-                 KoUpdater* progressUpdater
-        ) const
+void KisConvolutionFilter::process(KisConstProcessingInformation srcInfo,
+                                   KisProcessingInformation dstInfo,
+                                   const QSize& size,
+                                   const KisFilterConfiguration* config,
+                                   KoUpdater* progressUpdater
+                                  ) const
 {
+    Q_UNUSED(config);
+
     const KisPaintDeviceSP src = srcInfo.paintDevice();
     KisPaintDeviceSP dst = dstInfo.paintDevice();
     QPoint dstTopLeft = dstInfo.topLeft();
@@ -51,20 +57,20 @@ void KisConvolutionFilter::process(KisFilterConstantProcessingInformation srcInf
     Q_ASSERT(src != 0);
     Q_ASSERT(dst != 0);
 
-    if (dst != src) { // TODO: fix the convolution painter to avoid that stupid copy
-//         kDebug() <<"src != dst";
-        KisPainter gc(dst);
-        gc.bitBlt(dstTopLeft.x(), dstTopLeft.y(), COMPOSITE_COPY, src, srcTopLeft.x(), srcTopLeft.y(), size.width(), size.height());
-        gc.end();
+    KisConvolutionPainter painter(dst, dstInfo.selection());
+
+    QBitArray channelFlags;
+    if (config) channelFlags = config->channelFlags();
+    if (channelFlags.isEmpty()) {
+        channelFlags = dst->colorSpace()->channelFlags();
     }
 
-    KisConvolutionPainter painter( dst, dstInfo.selection() );
-
-    painter.setProgress( progressUpdater );
-    painter.applyMatrix(m_matrix, dstTopLeft.x(), dstTopLeft.y(), size.width(), size.height(), BORDER_REPEAT);
+    painter.setProgress(progressUpdater);
+    painter.applyMatrix(m_matrix, src, dstTopLeft.x(), dstTopLeft.y(), size.width(), size.height(), BORDER_REPEAT);
 
 }
 
-int KisConvolutionFilter::overlapMarginNeeded(KisFilterConfiguration* /*c*/) const {
-    return qMax(m_matrix->width / 2, m_matrix->height / 2);
+int KisConvolutionFilter::overlapMarginNeeded(const KisFilterConfiguration* /*c*/) const
+{
+    return qMax(m_matrix->width() / 2, m_matrix->height() / 2);
 }
