@@ -18,20 +18,21 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-#include <kgenericfactory.h>
 #include "kis_cubism_filter_plugin.h"
+
+#include <kgenericfactory.h>
+
 #include "kis_cubism_filter.h"
-#include "kis_filter_registry.h"
+#include "filter/kis_filter_registry.h"
 
 typedef KGenericFactory<KisCubismFilterPlugin> KisCubismFilterPluginFactory;
-K_EXPORT_COMPONENT_FACTORY( kritacubismfilter, KisCubismFilterPluginFactory( "krita" ) )
+K_EXPORT_COMPONENT_FACTORY(kritacubismfilter, KisCubismFilterPluginFactory("krita"))
 
 KisCubismFilterPlugin::KisCubismFilterPlugin(QObject *parent, const QStringList &) : KParts::Plugin(parent)
 {
     setComponentData(KisCubismFilterPluginFactory::componentData());
 
-    if ( parent->inherits("KisFilterRegistry") )
-    {
+    if (parent->inherits("KisFilterRegistry")) {
         KisFilterRegistry * r = dynamic_cast<KisFilterRegistry*>(parent);
         r->add(new KisCubismFilter());
     }
