@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2004 Michael Thaler <michael.thaler@physik.tu-muenchen.de>
  *
- * ported from digikam, Copyright 2004 by Gilles Caulier,
+ * ported from digikam, Copyrighted 2004 Gilles Caulier,
  * Original Emboss algorithm copyrighted 2004 by
  * Pieter Z. Voloshyn <pieter_voloshyn at ame.com.br>.
  *
@@ -22,6 +22,9 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
+#include "kis_emboss_filter.h"
+
+
 #include <stdlib.h>
 #include <vector>
 
@@ -32,7 +35,7 @@
 #include <kiconloader.h>
 #include <kmessagebox.h>
 #include <kstandarddirs.h>
-#include <kdebug.h>
+#include <kis_debug.h>
 #include <kgenericfactory.h>
 #include <knuminput.h>
 
@@ -41,26 +44,27 @@
 
 #include <kis_iterators_pixel.h>
 #include <kis_random_accessor.h>
-#include <kis_filter_registry.h>
+#include <filter/kis_filter_registry.h>
 #include <kis_global.h>
-#include <kis_paint_device.h>
 #include <kis_selection.h>
 #include <kis_types.h>
+#include <filter/kis_filter_configuration.h>
+#include <kis_paint_device.h>
+#include <kis_processing_information.h>
 
-#include "kis_multi_integer_filter_widget.h"
-#include "kis_emboss_filter.h"
+#include "widgets/kis_multi_integer_filter_widget.h"
 
 KisEmbossFilter::KisEmbossFilter() : KisFilter(id(), CategoryEmboss, i18n("&Emboss with Variable Depth..."))
 {
-    setSupportsPainting( false );
-    setSupportsPreview( true );
+    setSupportsPainting(false);
+    setSupportsPreview(true);
     setColorSpaceIndependence(TO_RGBA8);
 }
 
 KisFilterConfiguration* KisEmbossFilter::factoryConfiguration(const KisPaintDeviceSP) const
 {
     KisFilterConfiguration* config = new KisFilterConfiguration(id().id(), 0);
-    config->setProperty("depth",30);
+    config->setProperty("depth", 30);
     return config;
 }
 
@@ -77,12 +81,12 @@ KisFilterConfiguration* KisEmbossFilter::factoryConfiguration(const KisPaintDevi
  *                     understand. You get the diference between the colors and
  *                     increase it. After this, get the gray tone
  */
-void KisEmbossFilter::process(KisFilterConstantProcessingInformation srcInfo,
-                 KisFilterProcessingInformation dstInfo,
-                 const QSize& size,
-                 const KisFilterConfiguration* config,
-                 KoUpdater* progressUpdater
-        ) const
+void KisEmbossFilter::process(KisConstProcessingInformation srcInfo,
+                              KisProcessingInformation dstInfo,
+                              const QSize& size,
+                              const KisFilterConfiguration* config,
+                              KoUpdater* progressUpdater
+                             ) const
 {
     const KisPaintDeviceSP src = srcInfo.paintDevice();
     KisPaintDeviceSP dst = dstInfo.paintDevice();
@@ -90,9 +94,9 @@ void KisEmbossFilter::process(KisFilterConstantProcessingInformation srcInfo,
     QPoint srcTopLeft = srcInfo.topLeft();
     Q_ASSERT(src != 0);
     Q_ASSERT(dst != 0);
-    
+
     //read the filter configuration values from the KisFilterConfiguration object
-    quint32 embossdepth = config->getInt("depth",30);
+    quint32 embossdepth = config ?  config->getInt("depth", 30) : 30;
 
     //the actual filter function from digikam. It needs a pointer to a quint8 array
     //with the actual pixel data.
@@ -103,28 +107,25 @@ void KisEmbossFilter::process(KisFilterConstantProcessingInformation srcInfo,
     int Width = size.width();
     int Height = size.height();
 
-    int totalCost = Height / 100;
+    if (progressUpdater) {
+        progressUpdater->setRange(0, Height);
+    }
 
     KisHLineConstIteratorPixel it = src->createHLineConstIterator(srcTopLeft.x(), srcTopLeft.y(), size.width(), srcInfo.selection());
     KisHLineIteratorPixel dstIt = dst->createHLineIterator(dstTopLeft.x(), dstTopLeft.y(), size.width(), dstInfo.selection());
     QColor color1;
     QColor color2;
-    quint8 opacity = 0;
-
-    for (int y = 0 ; ! progressUpdater->interrupted() && (y < Height) ; ++y)
-    {
+    for (int y = 0; !(progressUpdater && progressUpdater->interrupted()) && (y < Height); ++y) {
         KisRandomConstAccessorPixel acc = src->createRandomConstAccessor(srcTopLeft.x(), srcTopLeft.y());
 
-        for (int x = 0 ; progressUpdater->interrupted() && (x < Width) ; ++x, ++it, ++dstIt)
-        {
+        for (int x = 0; !(progressUpdater && progressUpdater->interrupted()) && (x < Width); ++x, ++it, ++dstIt) {
             if (dstIt.isSelected()) {
 
-// FIXME: COLORSPACE_INDEPENDENCE or at least work IN RGB16A
+// XXX: COLORSPACE_INDEPENDENCE or at least work IN RGB16A
 
-                opacity = 0;
 
                 src->colorSpace()->toQColor(it.oldRawData(), &color1);
-                acc.moveTo(srcTopLeft.x() + x + Lim_Max(x, 1, Width), srcTopLeft.y() + y + Lim_Max(y, 1, Height) );
+                acc.moveTo(srcTopLeft.x() + x + Lim_Max(x, 1, Width), srcTopLeft.y() + y + Lim_Max(y, 1, Height));
 
                 src->colorSpace()->toQColor(acc.oldRawData(), &color2);
 
@@ -134,14 +135,13 @@ void KisEmbossFilter::process(KisFilterConstantProcessingInformation srcInfo,
 
                 Gray = CLAMP((R + G + B) / 3, 0, quint8_MAX);
 
-                dst->colorSpace()->fromQColor(QColor(Gray, Gray, Gray), opacity, dstIt.rawData());
+                dst->colorSpace()->fromQColor(QColor(Gray, Gray, Gray, color1.alpha()), dstIt.rawData());
             }
         }
         it.nextRow();
         dstIt.nextRow();
-        progressUpdater->setProgress(y / totalCost);
+        if (progressUpdater) progressUpdater->setValue(y);
     }
-
 }
 
 // This method have been ported from Pieter Z. Voloshyn algorithm code.
@@ -161,7 +161,7 @@ void KisEmbossFilter::process(KisFilterConstantProcessingInformation srcInfo,
  *                      "for step", when necessary, until reach the last possible value
  */
 
-int KisEmbossFilter::Lim_Max (int Now, int Up, int Max) const
+int KisEmbossFilter::Lim_Max(int Now, int Up, int Max) const
 {
     --Max;
     while (Now > Max - Up)
@@ -169,11 +169,14 @@ int KisEmbossFilter::Lim_Max (int Now, int Up, int Max) const
     return (Up);
 }
 
-KisFilterConfigWidget * KisEmbossFilter::createConfigurationWidget(QWidget* parent, const KisPaintDeviceSP) const
+KisConfigWidget * KisEmbossFilter::createConfigurationWidget(QWidget* parent, const KisPaintDeviceSP dev, const KisImageSP image) const
 {
+    Q_UNUSED(dev);
+    Q_UNUSED(image);
+
     vKisIntegerWidgetParam param;
-    param.push_back( KisIntegerWidgetParam( 10, 300, 30, i18n("Depth"), "depth" ) );
-    KisFilterConfigWidget * w = new KisMultiIntegerFilterWidget( id().id(), parent, id().id(), param );
+    param.push_back(KisIntegerWidgetParam(10, 300, 30, i18n("Depth"), "depth"));
+    KisConfigWidget * w = new KisMultiIntegerFilterWidget(id().id(), parent, id().id(), param);
     Q_CHECK_PTR(w);
     return w;
 }
