@@ -16,6 +16,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
+
+#include "example.h"
 #include <stdlib.h>
 #include <vector>
 
@@ -27,36 +29,33 @@
 #include <kcomponentdata.h>
 #include <kmessagebox.h>
 #include <kstandarddirs.h>
-#include <kdebug.h>
+#include <kis_debug.h>
 #include <kgenericfactory.h>
 
 #include <KoColorTransformation.h>
 #include <KoProgressUpdater.h>
 
+#include <kis_processing_information.h>
 #include <kis_types.h>
 #include <kis_selection.h>
 #include <kis_image.h>
 #include <kis_iterators_pixel.h>
 #include <kis_layer.h>
-#include <kis_filter_registry.h>
+#include <filter/kis_filter_registry.h>
 #include <kis_global.h>
 
-// #include <kmessagebox.h>
-
-#include "example.h"
-
 typedef KGenericFactory<KritaExample> KritaExampleFactory;
-K_EXPORT_COMPONENT_FACTORY( kritaexample, KritaExampleFactory( "krita" ) )
+K_EXPORT_COMPONENT_FACTORY(kritaexample, KritaExampleFactory("krita"))
 
 KritaExample::KritaExample(QObject *parent, const QStringList &)
         : KParts::Plugin(parent)
 {
     setComponentData(KritaExampleFactory::componentData());
 
-
     if (parent->inherits("KisFilterRegistry")) {
         KisFilterRegistry * manager = dynamic_cast<KisFilterRegistry *>(parent);
-        manager->add(KisFilterSP(new KisFilterInvert()));
+        if (manager)
+            manager->add(KisFilterSP(new KisFilterInvert()));
     }
 }
 
@@ -72,22 +71,24 @@ KisFilterInvert::KisFilterInvert() : KisFilter(id(), CategoryAdjust, i18n("&Inve
     setSupportsIncrementalPainting(false);
 }
 
-void KisFilterInvert::process(KisFilterConstantProcessingInformation srcInfo,
-                 KisFilterProcessingInformation dstInfo,
-                 const QSize& size,
-                 const KisFilterConfiguration* config,
-                 KoUpdater* progressUpdater
-        ) const
+void KisFilterInvert::process(KisConstProcessingInformation srcInfo,
+                              KisProcessingInformation dstInfo,
+                              const QSize& size,
+                              const KisFilterConfiguration* config,
+                              KoUpdater* progressUpdater
+                             ) const
 {
     const KisPaintDeviceSP src = srcInfo.paintDevice();
     KisPaintDeviceSP dst = dstInfo.paintDevice();
     QPoint dstTopLeft = dstInfo.topLeft();
     QPoint srcTopLeft = srcInfo.topLeft();
-    Q_UNUSED( config );
+    Q_UNUSED(config);
     Q_ASSERT(!src.isNull());
     Q_ASSERT(!dst.isNull());
 
-    int cost = (size.width() * size.height()) / 100;
+    if (progressUpdater) {
+        progressUpdater->setRange(0, size.width() * size.height());
+    }
     int count = 0;
 
     const KoColorSpace * cs = src->colorSpace();
@@ -105,25 +106,24 @@ void KisFilterInvert::process(KisFilterConstantProcessingInformation srcInfo,
     KisHLineConstIteratorPixel srcIt = src->createHLineConstIterator(srcTopLeft.x(), srcTopLeft.y(), size.width(), srcInfo.selection());
     KisHLineIteratorPixel dstIt = dst->createHLineIterator(dstTopLeft.x(), dstTopLeft.y(), size.width(), dstInfo.selection());
 
-    for ( int row = 0; row < size.height(); ++row ) {
-        while ( !srcIt.isDone() ) {
-            if ( srcIt.isSelected() ) {
-                inverter->transform( srcIt.oldRawData(), dstIt.rawData(), 1);
+    for (int row = 0; row < size.height() && !(progressUpdater && progressUpdater->interrupted()); ++row) {
+        while (!srcIt.isDone() && !(progressUpdater && progressUpdater->interrupted())) {
+            if (srcIt.isSelected()) {
+                inverter->transform(srcIt.oldRawData(), dstIt.rawData(), 1);
             }
             ++srcIt;
             ++dstIt;
-            if(progressUpdater) progressUpdater->setProgress( (++count) / cost);
-
+            if (progressUpdater) progressUpdater->setValue(++count);
         }
         srcIt.nextRow();
         dstIt.nextRow();
     }
-    kDebug(41006) <<"Per-pixel isSelected():" << t.elapsed() <<" ms";
+    dbgPlugins << "Per-pixel isSelected():" << t.elapsed() << " ms";
 
 #if 0
     t.restart();
 
-    bool hasSelection = src->hasSelection();
+    bool hasSelection = srcInfo.selection();
 
 // Method two: check the number of consecutive pixels the iterators
 // points to. Take as large stretches of unselected pixels as possible
@@ -135,31 +135,29 @@ void KisFilterInvert::process(KisFilterConstantProcessingInformation srcInfo,
     dstIt = dst->createHLineIterator(dstTopLeft.x(), dstTopLeft.y(), size.width());
 
     for (int row = 0; row < size.height(); ++row) {
-        while( ! srcIt.isDone() )
-        {
+        while (! srcIt.isDone()) {
             int srcItConseq = srcIt.nConseqHPixels();
             int dstItConseq = srcIt.nConseqHPixels();
-            int conseqPixels = qMin( srcItConseq, dstItConseq );
+            int conseqPixels = qMin(srcItConseq, dstItConseq);
 
             int pixels = 0;
 
-            if ( hasSelection ) {
+            if (hasSelection) {
                 // Get largest horizontal row of selected pixels
 
-                while ( srcIt.isSelected() && pixels < conseqPixels ) {
+                while (srcIt.isSelected() && pixels < conseqPixels) {
                     ++pixels;
                 }
-                inverter->transform( srcIt.oldRawData(), dstIt.rawData(), pixels);
+                inverter->transform(srcIt.oldRawData(), dstIt.rawData(), pixels);
 
                 // We apparently found a non-selected pixels, or the row
                 // was done; get the stretch of non-selected pixels
-                while ( !srcIt.isSelected() && pixels < conseqPixels ) {
+                while (!srcIt.isSelected() && pixels < conseqPixels) {
                     ++ pixels;
                 }
-            }
-            else {
+            } else {
                 pixels = conseqPixels;
-                inverter->transform( srcIt.oldRawData(), dstIt.rawData(), pixels );
+                inverter->transform(srcIt.oldRawData(), dstIt.rawData(), pixels);
             }
 
             // Update progress
@@ -169,11 +167,11 @@ void KisFilterInvert::process(KisFilterConstantProcessingInformation srcInfo,
         srcIt.nextRow();
         dstIt.nextRow();
     }
-    kDebug(41006) <<"Consecutive pixels:" << t.elapsed() <<" ms";
+    dbgPlugins << "Consecutive pixels:" << t.elapsed() << " ms";
 
 #endif
     delete inverter;
-
+    //if(progressUpdater) progressUpdater->setProgress( 100 );
 
     // Two inversions make no inversion? No -- because we're reading
     // from the oldData in both loops without saving the transaction
