@@ -18,17 +18,22 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-#include <kgenericfactory.h>
-#include "kis_paint_device.h"
 #include "kis_pixelize_filter_plugin.h"
+
+
+#include <kgenericfactory.h>
+
+#include <kis_paint_device.h>
+#include <kis_global.h>
+#include <filter/kis_filter_registry.h>
+
 #include "kis_pixelize_filter.h"
-#include "kis_global.h"
-#include "kis_filter_registry.h"
+
 typedef KGenericFactory<KisPixelizeFilterPlugin> KisPixelizeFilterPluginFactory;
-K_EXPORT_COMPONENT_FACTORY( kritapixelizefilter, KisPixelizeFilterPluginFactory( "krita" ) )
+K_EXPORT_COMPONENT_FACTORY(kritapixelizefilter, KisPixelizeFilterPluginFactory("krita"))
 
 KisPixelizeFilterPlugin::KisPixelizeFilterPlugin(QObject *parent, const QStringList &)
-    : KParts::Plugin(parent)
+        : KParts::Plugin(parent)
 {
     setComponentData(KisPixelizeFilterPluginFactory::componentData());
 
