@@ -18,16 +18,18 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
+#include "kis_raindrops_filter_plugin.h"
+
 #include <kgenericfactory.h>
 
-#include <kis_filter_registry.h>
+#include <filter/kis_filter_registry.h>
+
 
-#include "kis_raindrops_filter_plugin.h"
 #include "kis_raindrops_filter.h"
 
 
 typedef KGenericFactory<KisRainDropsFilterPlugin> KisRainDropsFilterPluginFactory;
-K_EXPORT_COMPONENT_FACTORY( kritaraindropsfilter, KisRainDropsFilterPluginFactory( "krita" ) )
+K_EXPORT_COMPONENT_FACTORY(kritaraindropsfilter, KisRainDropsFilterPluginFactory("krita"))
 
 KisRainDropsFilterPlugin::KisRainDropsFilterPlugin(QObject *parent, const QStringList &) : KParts::Plugin(parent)
 {
