@@ -21,6 +21,8 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
+#include "kis_sobel_filter.h"
+
 #include <stdlib.h>
 #include <vector>
 
@@ -31,7 +33,7 @@
 #include <kiconloader.h>
 #include <kmessagebox.h>
 #include <kstandarddirs.h>
-#include <kdebug.h>
+#include <kis_debug.h>
 #include <kgenericfactory.h>
 #include <knuminput.h>
 
@@ -39,14 +41,17 @@
 #include <kis_image.h>
 #include <kis_iterators_pixel.h>
 #include <kis_layer.h>
-#include <kis_filter_registry.h>
+#include <filter/kis_filter_registry.h>
 #include <kis_global.h>
+#include <kis_selection.h>
 #include <kis_types.h>
 #include <kis_paint_device.h>
 #include <KoProgressUpdater.h>
+#include <filter/kis_filter_configuration.h>
+#include <kis_processing_information.h>
+
+#include "widgets/kis_multi_bool_filter_widget.h"
 
-#include "kis_multi_bool_filter_widget.h"
-#include "kis_sobel_filter.h"
 
 #define MIN(a,b) (((a)<(b))?(a):(b))
 
@@ -71,15 +76,18 @@
 
 KisSobelFilter::KisSobelFilter() : KisFilter(id(), CategoryEdgeDetection, i18n("&Sobel..."))
 {
+    setSupportsPainting(false);
+    setSupportsPreview(true);
+    setSupportsThreading(false); // TODO Sobel doesn't support threading on image with height > 512
 }
 
 
-void KisSobelFilter::prepareRow (KisPaintDeviceSP src, quint8* data, quint32 x, quint32 y, quint32 w, quint32 h)
+void KisSobelFilter::prepareRow(KisPaintDeviceSP src, quint8* data, quint32 x, quint32 y, quint32 w, quint32 h) const
 {
-    if (y > h -1) y = h -1;
+    if (y > h - 1) y = h - 1;
     quint32 pixelSize = src->pixelSize();
 
-    src->readBytes( data, x, y, w, 1 );
+    src->readBytes(data, x, y, w, 1);
 
     for (quint32 b = 0; b < pixelSize; b++) {
         int offset = pixelSize - b;
@@ -91,14 +99,21 @@ void KisSobelFilter::prepareRow (KisPaintDeviceSP src, quint8* data, quint32 x,
 #define RMS(a, b) (sqrt ((a) * (a) + (b) * (b)))
 #define ROUND(x) ((int) ((x) + 0.5))
 
-void KisSobelFilter::process(KisFilterConstantProcessingInformation src,
-                 KisFilterProcessingInformation dst,
-                 const QSize& size,
-                 const KisFilterConfiguration* config,
-                 KoUpdater* progressUpdater
-        ) const
+void KisSobelFilter::process(KisConstProcessingInformation srcInfo,
+                             KisProcessingInformation dstInfo,
+                             const QSize& size,
+                             const KisFilterConfiguration* configuration,
+                             KoUpdater* progressUpdater
+                            ) const
 {
-#if 0
+    const KisPaintDeviceSP src = srcInfo.paintDevice();
+    KisPaintDeviceSP dst = dstInfo.paintDevice();
+    QPoint dstTopLeft = dstInfo.topLeft();
+    QPoint srcTopLeft = srcInfo.topLeft();
+    Q_ASSERT(!src.isNull());
+    Q_ASSERT(!dst.isNull());
+
+
     //read the filter configuration values from the KisFilterConfiguration object
     bool doHorizontal = configuration->getBool("doHorizontally", true);
     bool doVertical = configuration->getBool("doVertically", true);
@@ -109,15 +124,14 @@ void KisSobelFilter::process(KisFilterConstantProcessingInformation src,
     quint32 height = size.height();
     quint32 pixelSize = src->pixelSize();
 
-    setProgressTotalSteps( height );
-    setProgressStage(i18n("Applying sobel filter..."),0);
+    int cost = size.height();
 
     /*  allocate row buffers  */
-    quint8* prevRow = new quint8[ (width + 2) * pixelSize];
+    quint8* prevRow = new quint8[(width + 2) * pixelSize];
     Q_CHECK_PTR(prevRow);
-    quint8* curRow = new quint8[ (width + 2) * pixelSize];
+    quint8* curRow = new quint8[(width + 2) * pixelSize];
     Q_CHECK_PTR(curRow);
-    quint8* nextRow = new quint8[ (width + 2) * pixelSize];
+    quint8* nextRow = new quint8[(width + 2) * pixelSize];
     Q_CHECK_PTR(nextRow);
     quint8* dest = new quint8[ width  * pixelSize];
     Q_CHECK_PTR(dest);
@@ -126,26 +140,24 @@ void KisSobelFilter::process(KisFilterConstantProcessingInformation src,
     quint8* cr = curRow + pixelSize;
     quint8* nr = nextRow + pixelSize;
 
-    prepareRow (src, pr, srcTopLeft.x(), srcTopLeft.y() - 1, width, height);
-    prepareRow (src, cr, srcTopLeft.x(), srcTopLeft.y(), width, height);
+    prepareRow(src, pr, srcTopLeft.x(), srcTopLeft.y() - 1, width, height);
+    prepareRow(src, cr, srcTopLeft.x(), srcTopLeft.y(), width, height);
 
-    quint32 counter =0;
+    quint32 counter = 0;
     quint8* d;
     quint8* tmp;
     qint32 gradient, horGradient, verGradient;
     // loop through the rows, applying the sobel convolution
 
-    KisHLineIteratorPixel dstIt = dst->createHLineIterator(dstTopLeft.x(), dstTopLeft.y(), width);
+    KisHLineIteratorPixel dstIt = dst->createHLineIterator(dstTopLeft.x(), dstTopLeft.y(), width, dstInfo.selection());
 
-    for (quint32 row = 0; row < height; row++)
-    {
+    for (quint32 row = 0; row < height; row++) {
 
         // prepare the next row
-        prepareRow (src, nr, srcTopLeft.x(), srcTopLeft.y() + row + 1, width, height);
+        prepareRow(src, nr, srcTopLeft.x(), srcTopLeft.y() + row + 1, width, height);
         d = dest;
 
-        for (quint32 col = 0; col < width * pixelSize; col++)
-        {
+        for (quint32 col = 0; col < width * pixelSize; col++) {
             int positive = col + pixelSize;
             int negative = col - pixelSize;
             horGradient = (doHorizontal ?
@@ -158,9 +170,9 @@ void KisSobelFilter::process(KisFilterConstantProcessingInformation src,
                             (pr[positive] + 2 * cr[positive] + nr[positive]))
                            : 0);
             gradient = (qint32)((doVertical && doHorizontal) ?
-                                (ROUND (RMS (horGradient, verGradient)) / 5.66) // always >0
-                                : (keepSign ? (127 + (ROUND ((horGradient + verGradient) / 8.0)))
-                                   : (ROUND (QABS (horGradient + verGradient) / 4.0))));
+                                (ROUND(RMS(horGradient, verGradient)) / 5.66)   // always >0
+                                : (keepSign ? (127 + (ROUND((horGradient + verGradient) / 8.0)))
+                                   : (ROUND(QABS(horGradient + verGradient) / 4.0))));
 
             *d++ = gradient;
             if (gradient > 10) counter ++;
@@ -175,33 +187,29 @@ void KisSobelFilter::process(KisFilterConstantProcessingInformation src,
         //store the dest
         dst->writeBytes(dest, dstTopLeft.x(), row, width, 1);
 
-        if ( makeOpaque )
-        {
-            while( ! dstIt.isDone() )
-            {
-                dst->colorSpace()->setAlpha(dstIt.rawData(), 255,1);
+        if (makeOpaque) {
+            while (! dstIt.isDone()) {
+                dst->colorSpace()->setAlpha(dstIt.rawData(), 255, 1);
                 ++dstIt;
             }
             dstIt.nextRow();
         }
-        setProgress(row);
+        if (progressUpdater) progressUpdater->setProgress(row / cost);
     }
-    setProgressDone();
 
     delete[] prevRow;
     delete[] curRow;
     delete[] nextRow;
     delete[] dest;
-#endif
 }
 
 
-KisFilterConfigWidget * KisSobelFilter::createConfigurationWidget(QWidget* parent, KisPaintDeviceSP)
+KisConfigWidget * KisSobelFilter::createConfigurationWidget(QWidget* parent, const KisPaintDeviceSP, const KisImageSP) const
 {
     vKisBoolWidgetParam param;
-    param.push_back( KisBoolWidgetParam( true, i18n("Sobel horizontally"), "doHorizontally" ) );
-    param.push_back( KisBoolWidgetParam( true, i18n("Sobel vertically"), "doVertically" ) );
-    param.push_back( KisBoolWidgetParam( true, i18n("Keep sign of result"), "keepSign" ) );
-    param.push_back( KisBoolWidgetParam( true, i18n("Make image opaque"), "makeOpaque" ) );
-    return new KisMultiBoolFilterWidget(id().id(), parent, id().id(), param );
+    param.push_back(KisBoolWidgetParam(true, i18n("Sobel horizontally"), "doHorizontally"));
+    param.push_back(KisBoolWidgetParam(true, i18n("Sobel vertically"), "doVertically"));
+    param.push_back(KisBoolWidgetParam(true, i18n("Keep sign of result"), "keepSign"));
+    param.push_back(KisBoolWidgetParam(true, i18n("Make image opaque"), "makeOpaque"));
+    return new KisMultiBoolFilterWidget(id().id(), parent, id().id(), param);
 }
