@@ -3,8 +3,7 @@
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  the Free Software Foundation; version 2 of the License.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,14 +29,16 @@ class KisBookmarkedConfigurationManager;
 class DynamicBrush : public KParts::Plugin
 {
     Q_OBJECT
-    public:
-        DynamicBrush(QObject *parent, const QStringList &);
-        virtual ~DynamicBrush();
-    private slots:
-        void slotEditDynamicPrograms();
-    private:
-        KisView2 * m_view;
-        KisBookmarkedConfigurationManager* m_bookmarksManager;
+public:
+    DynamicBrush(QObject *parent, const QStringList &);
+    virtual ~DynamicBrush();
+private slots:
+    void slotEditDynamicShapePrograms();
+    void slotEditDynamicColoringPrograms();
+private:
+    KisView2 * m_view;
+    KisBookmarkedConfigurationManager* m_shapeBookmarksManager;
+    KisBookmarkedConfigurationManager* m_coloringBookmarksManager;
 };
 
 #endif // DYNAMIC_BRUSH_H
