@@ -23,16 +23,18 @@
 
 #include "KoToolFactory.h"
 
-#include <kis_layer_shape.h>
+#include <flake/kis_layer_shape.h>
 
 class QTimer;
 class QCheckBox;
+class QComboBox;
 class QGridLayout;
 class QSlider;
 
 class KoCanvasBase;
 
-class KisToolBrush : public KisToolFreehand {
+class KisToolBrush : public KisToolFreehand
+{
     Q_OBJECT
 
 public:
@@ -52,37 +54,38 @@ protected:
 private slots:
 
     void timeoutPaint();
-    void slotSetPaintingMode( int mode );
-    void slotSetSmoothness( int smoothness );
+    void slotSetSmoothness(int smoothness);
+    void slotSetMagnetism(int magnetism);
 
 private:
 
     qint32 m_rate;
     QTimer * m_timer;
     QGridLayout* m_optionLayout;
-    QCheckBox * m_chkDirect;
     QCheckBox * m_chkSmooth;
+    QCheckBox * m_chkAssistant;
+    QSlider * m_sliderMagnetism;
     QSlider * m_sliderSmoothness;
 };
 
 
-class KisToolBrushFactory : public KoToolFactory {
+class KisToolBrushFactory : public KoToolFactory
+{
 
 public:
     KisToolBrushFactory(QObject *parent, const QStringList&)
-        : KoToolFactory(parent, "KritaShape/KisToolBrush", i18n( "Paint" ))
-        {
-            setToolTip( i18n( "Paint freely" ) );
-            // Temporarily
-            setToolType( TOOL_TYPE_SHAPE );
-            setIcon( "tool_freehand" );
-            setShortcut( KShortcut( Qt::Key_B ) );
-            setPriority( 0 );
-            setActivationShapeId( KIS_LAYER_SHAPE_ID );
-            setInputDeviceAgnostic(false);
-        }
+            : KoToolFactory(parent, "KritaShape/KisToolBrush", i18n("Paint")) {
+        setToolTip(i18n("Paint with brushes"));
+        // Temporarily
+        setToolType(TOOL_TYPE_SHAPE);
+        setIcon("krita_tool_freehand");
+        setShortcut(KShortcut(Qt::Key_B));
+        setPriority(0);
+        //setActivationShapeId( KIS_LAYER_SHAPE_ID );
+        setInputDeviceAgnostic(false);
+    }
 
-    virtual ~KisToolBrushFactory(){}
+    virtual ~KisToolBrushFactory() {}
 
     virtual KoTool * createTool(KoCanvasBase *canvas) {
         return new KisToolBrush(canvas);
