@@ -21,22 +21,26 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
+#include "kis_tool_ellipse.h"
+
+
 #include <QPainter>
 
-#include <kdebug.h>
+#include <kis_debug.h>
 #include <klocale.h>
 
-#include "kis_painter.h"
-#include "kis_tool_ellipse.h"
 #include "KoPointerEvent.h"
-#include "kis_paintop_registry.h"
-#include "kis_cursor.h"
-#include "KoCanvasBase.h"
-#include "kis_layer.h"
+#include <KoCanvasBase.h>
+
+#include <kis_selection.h>
+#include <kis_painter.h>
+#include <kis_paintop_registry.h>
+#include <kis_cursor.h>
+#include <kis_layer.h>
 
 KisToolEllipse::KisToolEllipse(KoCanvasBase * canvas)
-    : KisToolShape(canvas, KisCursor::load("tool_ellipse_cursor.png", 6, 6)),
-      m_dragging (false)
+        : KisToolShape(canvas, KisCursor::load("tool_ellipse_cursor.png", 6, 6)),
+        m_dragging(false)
 {
     setObjectName("tool_ellipse");
 
@@ -52,10 +56,11 @@ KisToolEllipse::~KisToolEllipse()
 
 void KisToolEllipse::paint(QPainter& gc, const KoViewConverter &converter)
 {
-    double sx, sy;
+    Q_ASSERT(currentImage());
+    qreal sx, sy;
     converter.zoom(&sx, &sy);
 
-    gc.scale( sx/currentImage()->xRes(), sy/currentImage()->yRes() );
+    gc.scale(sx / currentImage()->xRes(), sy / currentImage()->yRes());
     if (m_dragging)
         paintEllipse(gc, QRect());
 }
@@ -63,12 +68,10 @@ void KisToolEllipse::paint(QPainter& gc, const KoViewConverter &converter)
 
 void KisToolEllipse::mousePressEvent(KoPointerEvent *event)
 {
-     if (!m_canvas || !currentImage()) return;
-
-    if (!currentBrush()) return;
+    if (!m_canvas || !currentImage()) return;
 
     if (event->button() == Qt::LeftButton) {
-	QPointF pos = convertToPixelCoord(event);
+        QPointF pos = convertToPixelCoord(event);
         m_dragging = true;
         m_dragStart = m_dragCenter = m_dragEnd = pos;
         //draw(m_dragStart, m_dragEnd);
@@ -84,7 +87,7 @@ void KisToolEllipse::mouseMoveEvent(KoPointerEvent *event)
         bound.setBottomRight(m_dragEnd);
         m_canvas->updateCanvas(convertToPt(bound.normalized()));
 
-	QPointF pos = convertToPixelCoord(event);
+        QPointF pos = convertToPixelCoord(event);
         // erase old lines on canvas
         //draw(m_dragStart, m_dragEnd);
         // move (alt) or resize ellipse
@@ -94,7 +97,7 @@ void KisToolEllipse::mouseMoveEvent(KoPointerEvent *event)
             m_dragEnd += trans;
         } else {
             QPointF diag = pos - (event->modifiers() & Qt::ControlModifier
-				  ? m_dragCenter : m_dragStart);
+                                  ? m_dragCenter : m_dragStart);
             // circle?
             if (event->modifiers() & Qt::ShiftModifier) {
                 double size = qMax(fabs(diag.x()), fabs(diag.y()));
@@ -113,13 +116,13 @@ void KisToolEllipse::mouseMoveEvent(KoPointerEvent *event)
         }
         // draw new lines on canvas
         //draw(m_dragStart, m_dragEnd);
-	bound.setTopLeft(m_dragStart);
+        bound.setTopLeft(m_dragStart);
         bound.setBottomRight(m_dragEnd);
-        /* FIXME Which rectangle to repaint */
+
         m_canvas->updateCanvas(convertToPt(bound.normalized()));
 
         m_dragCenter = QPointF((m_dragStart.x() + m_dragEnd.x()) / 2,
-                                (m_dragStart.y() + m_dragEnd.y()) / 2);
+                               (m_dragStart.y() + m_dragEnd.y()) / 2);
     }
 }
 
@@ -141,36 +144,30 @@ void KisToolEllipse::mouseReleaseEvent(KoPointerEvent *event)
         if (!currentImage())
             return;
 
-        if (!currentLayer()->paintDevice())
+        if (!currentNode()->paintDevice())
             return;
 
-        KisPaintDeviceSP device = currentLayer()->paintDevice();
-	delete m_painter;
-	m_painter = new KisPainter( device, currentLayer()->selection() );
-	Q_CHECK_PTR(m_painter);
+        KisPaintDeviceSP device = currentNode()->paintDevice();
+        delete m_painter;
+        m_painter = new KisPainter(device, currentSelection());
+        Q_CHECK_PTR(m_painter);
 
-        m_painter->beginTransaction (i18n ("Ellipse"));
-        m_painter->setBounds( currentImage()->bounds() );
-        m_painter->setPaintColor(currentFgColor());
-        m_painter->setBackgroundColor(currentBgColor());
-        m_painter->setFillStyle(fillStyle());
-        m_painter->setStrokeStyle(strokeStyle());
-        m_painter->setBrush(currentBrush());
-        m_painter->setPattern(currentPattern());
+        m_painter->beginTransaction(i18n("Ellipse"));
+        setupPainter(m_painter);
         m_painter->setOpacity(m_opacity);
         m_painter->setCompositeOp(m_compositeOp);
-        KisPaintOp * op = KisPaintOpRegistry::instance()->paintOp(currentPaintOp(), currentPaintOpSettings(), m_painter, currentImage());
-        m_painter->setPaintOp(op); // Painter takes ownership
 
-        m_painter->paintEllipse(QRectF(m_dragStart, m_dragEnd), PRESSURE_DEFAULT/*event->pressure()*/, event->xTilt(), event->yTilt());
+        m_painter->paintEllipse(QRectF(m_dragStart, m_dragEnd));
         QRegion bound = m_painter->dirtyRegion();
-        device->setDirty( bound );
+        device->setDirty(bound);
         notifyModified();
 
         m_canvas->addCommand(m_painter->endTransaction());
 
         delete m_painter;
         m_painter = 0;
+    } else {
+        KisToolPaint::mouseReleaseEvent(event);
     }
 }
 
@@ -185,9 +182,9 @@ void KisToolEllipse::paintEllipse(QPainter& gc, const QRect&)
 
         gc.setPen(pen);
 
-	start = QPoint(static_cast<int>(m_dragStart.x()), static_cast<int>(m_dragStart.y()));
-	end = QPoint(static_cast<int>(m_dragEnd.x()), static_cast<int>(m_dragEnd.y()));
-	gc.drawEllipse(QRect(start, end));
+        start = QPoint(static_cast<int>(m_dragStart.x()), static_cast<int>(m_dragStart.y()));
+        end = QPoint(static_cast<int>(m_dragEnd.x()), static_cast<int>(m_dragEnd.y()));
+        gc.drawEllipse(QRect(start, end));
         gc.setPen(old);
     }
 }
