@@ -1,7 +1,6 @@
 /*
  *  kis_tool_gradient.cc - part of Krita
  *
- *  Copyright (c) 2000 John Califf
  *  Copyright (c) 2002 Patrick Julien <freak@codepimps.org>
  *  Copyright (c) 2003 Boudewijn Rempt <boud@valdyas.org>
  *  Copyright (c) 2004-2007 Adrian Page <adrian@pagenet.plus.com>
@@ -35,36 +34,38 @@
 #include <QLayout>
 #include <QCheckBox>
 
-#include <kdebug.h>
+#include <kis_debug.h>
 #include <klocale.h>
 #include <knuminput.h>
+#include <kcombobox.h>
 
 #include "KoPointerEvent.h"
 #include "KoCanvasBase.h"
 #include "KoViewConverter.h"
 
-#include "kis_cmb_composite.h"
+#include "widgets/kis_cmb_composite.h"
 #include "kis_cursor.h"
-#include "kis_double_widget.h"
+#include "widgets/kis_double_widget.h"
 #include "kis_gradient_painter.h"
 #include "kis_painter.h"
 #include "KoProgressUpdater.h"
-#include "kis_resource_provider.h"
+#include "kis_canvas_resource_provider.h"
 #include "kis_config.h"
 #include "kis_layer.h"
+#include <kis_selection.h>
 
 #if defined(HAVE_OPENGL) && defined(HAVE_GLEW)
-#include "kis_opengl.h"
-#include "kis_opengl_gradient_program.h"
-#include "kis_opengl_canvas2.h"
-#include "kis_canvas2.h"
-#include "kis_int_spinbox.h"
+#include "opengl/kis_opengl.h"
+#include "opengl/kis_opengl_gradient_program.h"
+#include "opengl/kis_opengl_canvas2.h"
+#include "canvas/kis_canvas2.h"
+#include "KoSliderCombo.h"
 #include "kis_config_notifier.h"
 #endif
 
 KisToolGradient::KisToolGradient(KoCanvasBase * canvas)
-    : super(canvas, KisCursor::load("tool_gradient_cursor.png", 6, 6)),
-      m_dragging( false )
+        : KisToolPaint(canvas, KisCursor::load("tool_gradient_cursor.png", 6, 6)),
+        m_dragging(false)
 {
     setObjectName("tool_gradient");
 
@@ -86,13 +87,13 @@ KisToolGradient::~KisToolGradient()
 {
 }
 
-void KisToolGradient::paint( QPainter &painter, const KoViewConverter &converter )
+void KisToolGradient::paint(QPainter &painter, const KoViewConverter &converter)
 {
     if (m_dragging && m_startPos != m_endPos) {
 
 #if defined(HAVE_OPENGL) && defined(HAVE_GLEW)
         if (m_gradientProgram) {
-    
+
             QPointF gradientVector = m_endPos - m_startPos;
             double gradientVectorLength = sqrt((gradientVector.x() * gradientVector.x()) + (gradientVector.y() * gradientVector.y()));
 
@@ -122,12 +123,12 @@ void KisToolGradient::paint( QPainter &painter, const KoViewConverter &converter
                     glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
                     canvasWidget->makeCurrent();
-                    m_gradientProgram->activate(normalisedGradientVectorStart, 
+                    m_gradientProgram->activate(normalisedGradientVectorStart,
                                                 normalisedGradientVectorStart + normalisedGradientVector);
 
 //                     glValidateProgramARB(m_gradientProgram->handle());
-//                     kDebug(DBG_AREA_TOOLS) <<"Validate:";
-//                     kDebug(DBG_AREA_TOOLS) << m_gradientProgram->getInfoLog();
+//                     dbgTools <<"Validate:";
+//                     dbgTools << m_gradientProgram->getInfoLog();
 
 
                     glBegin(GL_QUADS);
@@ -158,9 +159,9 @@ void KisToolGradient::paint( QPainter &painter, const KoViewConverter &converter
         } else
 #endif
         {
-            double sx, sy;
+            qreal sx, sy;
             converter.zoom(&sx, &sy);
-            painter.scale( sx/currentImage()->xRes(), sy/currentImage()->yRes() );
+            painter.scale(sx / currentImage()->xRes(), sy / currentImage()->yRes());
             paintLine(painter);
         }
     }
@@ -194,13 +195,13 @@ void KisToolGradient::mousePressEvent(KoPointerEvent *e)
             }
 
             KisOpenGL::makeContextCurrent();
-            m_gradientProgram = new KisOpenGLGradientProgram(currentGradient(), 
-                                                             m_shape, 
-                                                             m_repeat, 
-                                                             m_reverse,
-                                                             currentImage()->colorSpace(),
-                                                             monitorProfile,
-                                                             m_previewOpacityPercent / 100.0);
+            m_gradientProgram = new KisOpenGLGradientProgram(currentGradient(),
+                    m_shape,
+                    m_repeat,
+                    m_reverse,
+                    currentImage()->colorSpace(),
+                    monitorProfile,
+                    m_previewOpacityPercent / 100.0);
         }
 #endif
     }
@@ -218,8 +219,7 @@ void KisToolGradient::mouseMoveEvent(KoPointerEvent *e)
 
         if ((e->modifiers() & Qt::ShiftModifier) == Qt::ShiftModifier) {
             m_endPos = straightLine(pos);
-        }
-        else {
+        } else {
             m_endPos = pos;
         }
 
@@ -251,16 +251,15 @@ void KisToolGradient::mouseReleaseEvent(KoPointerEvent *e)
 
         if ((e->modifiers() & Qt::ShiftModifier) == Qt::ShiftModifier) {
             m_endPos = straightLine(pos);
-        }
-        else {
+        } else {
             m_endPos = pos;
         }
 
         KisPaintDeviceSP device;
 
-        if (currentImage() && (device = currentLayer()->paintDevice())) {
+        if (currentImage() && (device = currentNode()->paintDevice())) {
 
-            KisGradientPainter painter(device, currentLayer()->selection());
+            KisGradientPainter painter(device, currentSelection());
 
             painter.beginTransaction(i18n("Gradient"));
 
@@ -299,12 +298,12 @@ QPointF KisToolGradient::straightLine(QPointF point)
     QPointF comparison = point - m_startPos;
     QPointF result;
 
-    if ( fabs(comparison.x()) > fabs(comparison.y())) {
+    if (fabs(comparison.x()) > fabs(comparison.y())) {
         result.setX(point.x());
         result.setY(m_startPos.y());
     } else {
-        result.setX( m_startPos.x() );
-        result.setY( point.y() );
+        result.setX(m_startPos.x());
+        result.setY(point.y());
     }
 
     return result;
@@ -330,32 +329,33 @@ void KisToolGradient::paintLine(QPainter& gc)
 
 QWidget* KisToolGradient::createOptionWidget()
 {
-    QWidget *widget = super::createOptionWidget();
+    QWidget *widget = KisToolPaint::createOptionWidget();
     Q_CHECK_PTR(widget);
-
+    widget->setObjectName(toolId() + " option widget");
+    
     m_lbShape = new QLabel(i18n("Shape:"), widget);
     m_lbRepeat = new QLabel(i18n("Repeat:"), widget);
 
-    m_ckReverse = new QCheckBox(i18n("Reverse"), widget);
+    m_ckReverse = new QCheckBox(i18nc("the gradient will be drawn with the color order reversed", "Reverse"), widget);
     m_ckReverse->setObjectName("reverse_check");
     connect(m_ckReverse, SIGNAL(toggled(bool)), this, SLOT(slotSetReverse(bool)));
 
-    m_cmbShape = new QComboBox(widget);
+    m_cmbShape = new KComboBox(widget);
     m_cmbShape->setObjectName("shape_combo");
     connect(m_cmbShape, SIGNAL(activated(int)), this, SLOT(slotSetShape(int)));
-    m_cmbShape->addItem(i18n("Linear"));
-    m_cmbShape->addItem(i18n("Bi-Linear"));
-    m_cmbShape->addItem(i18n("Radial"));
-    m_cmbShape->addItem(i18n("Square"));
-    m_cmbShape->addItem(i18n("Conical"));
-    m_cmbShape->addItem(i18n("Conical Symmetric"));
+    m_cmbShape->addItem(i18nc("the gradient will be drawn linearly", "Linear"));
+    m_cmbShape->addItem(i18nc("the gradient will be drawn bilinearly", "Bi-Linear"));
+    m_cmbShape->addItem(i18nc("the gradient will be drawn radially", "Radial"));
+    m_cmbShape->addItem(i18nc("the gradient will be drawn in a square around a centre", "Square"));
+    m_cmbShape->addItem(i18nc("the gradient will be drawn as an assymmetric cone", "Conical"));
+    m_cmbShape->addItem(i18nc("the gradient will be drawn as a symmetric cone", "Conical Symmetric"));
 
-    m_cmbRepeat = new QComboBox(widget);
+    m_cmbRepeat = new KComboBox(widget);
     m_cmbRepeat->setObjectName("repeat_combo");
     connect(m_cmbRepeat, SIGNAL(activated(int)), this, SLOT(slotSetRepeat(int)));
-    m_cmbRepeat->addItem(i18n("None"));
-    m_cmbRepeat->addItem(i18n("Forwards"));
-    m_cmbRepeat->addItem(i18n("Alternating"));
+    m_cmbRepeat->addItem(i18nc("The gradient will not repeat", "None"));
+    m_cmbRepeat->addItem(i18nc("The gradient will repeat forwards", "Forwards"));
+    m_cmbRepeat->addItem(i18nc("The gradient will repeat alternatingly", "Alternating"));
 
     addOptionWidgetOption(m_cmbShape, m_lbShape);
 
@@ -367,7 +367,8 @@ QWidget* KisToolGradient::createOptionWidget()
 
     m_slAntiAliasThreshold = new KDoubleNumInput(widget);
     m_slAntiAliasThreshold->setObjectName("threshold_slider");
-    m_slAntiAliasThreshold->setRange( 0, 1);
+    m_slAntiAliasThreshold->setMinimum(0);
+    m_slAntiAliasThreshold->setMaximum(1);
     m_slAntiAliasThreshold->setValue(m_antiAliasThreshold);
     connect(m_slAntiAliasThreshold, SIGNAL(valueChanged(double)), this, SLOT(slotSetAntiAliasThreshold(double)));
 
@@ -375,8 +376,8 @@ QWidget* KisToolGradient::createOptionWidget()
 
 #if defined(HAVE_OPENGL) && defined(HAVE_GLEW)
     m_lbPreviewOpacity = new QLabel(i18n("Preview opacity:"), widget);
-    m_slPreviewOpacity = new KisIntSpinbox( widget );
-    m_slPreviewOpacity->setRange(0, 100);
+    m_slPreviewOpacity = new KoSliderCombo(widget);
+    m_slPreviewOpacity->setDecimals(0);
     m_slPreviewOpacity->setValue(m_previewOpacityPercent);
     connect(m_slPreviewOpacity, SIGNAL(valueChanged(int)), this, SLOT(slotSetPreviewOpacity(int)));
 
