@@ -21,22 +21,26 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
+#include "kis_tool_rectangle.h"
+
 #include <QPainter>
 
-#include <kdebug.h>
+#include <kis_debug.h>
 #include <klocale.h>
 
 #include "KoPointerEvent.h"
 #include "kis_painter.h"
 #include "kis_paintop_registry.h"
-#include "kis_tool_rectangle.h"
+
 #include "kis_cursor.h"
 #include "kis_layer.h"
 #include "KoCanvasBase.h"
+#include <kis_selection.h>
+#include <kis_paint_device.h>
 
 KisToolRectangle::KisToolRectangle(KoCanvasBase * canvas)
-    : KisToolShape(canvas, KisCursor::load("tool_rectangle_cursor.png", 6, 6)),
-      m_dragging (false)
+        : KisToolShape(canvas, KisCursor::load("tool_rectangle_cursor.png", 6, 6)),
+        m_dragging(false)
 {
     setObjectName("tool_rectangle");
 
@@ -52,10 +56,13 @@ KisToolRectangle::~KisToolRectangle()
 
 void KisToolRectangle::paint(QPainter& gc, const KoViewConverter &converter)
 {
-    double sx, sy;
+    qreal sx, sy;
     converter.zoom(&sx, &sy);
-
-    gc.scale( sx/currentImage()->xRes(), sy/currentImage()->yRes() );
+    if (!currentImage()) {
+        warnKrita << "No currentImage!";
+        return;
+    }
+    gc.scale(sx / currentImage()->xRes(), sy / currentImage()->yRes());
     if (m_dragging)
         paintRectangle(gc, QRect());
 }
@@ -65,26 +72,23 @@ void KisToolRectangle::mousePressEvent(KoPointerEvent *e)
 {
     if (!m_canvas || !currentImage()) return;
 
-    if (!currentBrush()) return;
-
     if (e->button() == Qt::LeftButton) {
-	QPointF pos = convertToPixelCoord(e);
-	m_dragging = true;
-	m_dragStart = m_dragCenter = m_dragEnd = pos;
-	//draw(m_dragStart, m_dragEnd);
+        QPointF pos = convertToPixelCoord(e);
+        m_dragging = true;
+        m_dragStart = m_dragCenter = m_dragEnd = pos;
     }
 }
 
 void KisToolRectangle::mouseMoveEvent(KoPointerEvent *event)
 {
     if (m_dragging) {
-	QPointF pos = convertToPixelCoord(event);
+        QPointF pos = convertToPixelCoord(event);
         // erase old lines on canvas
-	// This is not enough, how to do?
-	QRectF bound;
+        // This is not enough, how to do?
+        QRectF bound;
         bound.setTopLeft(m_dragStart);
         bound.setBottomRight(m_dragEnd);
-	m_canvas->updateCanvas(convertToPt(bound.normalized()));
+        m_canvas->updateCanvas(convertToPt(bound.normalized()));
         //draw(m_dragStart, m_dragEnd);
         // move (alt) or resize rectangle
         if (event->modifiers() & Qt::AltModifier) {
@@ -112,14 +116,16 @@ void KisToolRectangle::mouseMoveEvent(KoPointerEvent *event)
         }
         // draw new lines on canvas
         //draw(m_dragStart, m_dragEnd);
-	//QRectF bound;
+        //QRectF bound;
         bound.setTopLeft(m_dragStart);
         bound.setBottomRight(m_dragEnd);
-        /* FIXME Which rectangle to repaint */
+
         m_canvas->updateCanvas(convertToPt(bound.normalized()));
 
         m_dragCenter = QPointF((m_dragStart.x() + m_dragEnd.x()) / 2,
                                (m_dragStart.y() + m_dragEnd.y()) / 2);
+    } else {
+        KisToolPaint::mouseReleaseEvent(event);
     }
 }
 
@@ -133,35 +139,27 @@ void KisToolRectangle::mouseReleaseEvent(KoPointerEvent *event)
     if (!currentImage())
         return;
 
-    KisPaintDeviceSP device = currentLayer()->paintDevice();
+    KisPaintDeviceSP device = currentNode()->paintDevice();
     if (!device) return;
 
-    if (m_dragging && currentBrush() && event->button() == Qt::LeftButton) {
+    if (m_dragging && event->button() == Qt::LeftButton) {
         m_dragging = false;
 
         if (m_dragStart == m_dragEnd)
             return;
 
         delete m_painter;
-        m_painter = new KisPainter( device, currentLayer()->selection() );
+        m_painter = new KisPainter(device, currentSelection());
         Q_CHECK_PTR(m_painter);
 
-        m_painter->beginTransaction (i18n ("Rectangle"));
-        m_painter->setBounds( currentImage()->bounds() );
-        m_painter->setPaintColor(currentFgColor());
-        m_painter->setBackgroundColor(currentBgColor());
-        m_painter->setFillStyle(fillStyle());
-        m_painter->setStrokeStyle(strokeStyle());
-        m_painter->setBrush(currentBrush());
-        m_painter->setPattern(currentPattern());
+        m_painter->beginTransaction(i18n("Rectangle"));
+        setupPainter(m_painter);
         m_painter->setOpacity(m_opacity);
         m_painter->setCompositeOp(m_compositeOp);
-        KisPaintOp * op = KisPaintOpRegistry::instance()->paintOp(currentPaintOp(), currentPaintOpSettings(), m_painter, currentImage());
-        m_painter->setPaintOp(op);
 
-        m_painter->paintRect(QRectF(m_dragStart, m_dragEnd), PRESSURE_DEFAULT/*event->pressure()*/, event->xTilt(), event->yTilt());
+        m_painter->paintRect(QRectF(m_dragStart, m_dragEnd));
         QRegion bound = m_painter->dirtyRegion();
-        device->setDirty( bound );
+        device->setDirty(bound);
         notifyModified();
 // Should not be necessary anymore
 #if 0
@@ -170,8 +168,8 @@ void KisToolRectangle::mouseReleaseEvent(KoPointerEvent *event)
 
         m_canvas->addCommand(m_painter->endTransaction());
 
-	delete m_painter;
-	m_painter = 0;
+        delete m_painter;
+        m_painter = 0;
     }
 }
 
@@ -195,9 +193,9 @@ void KisToolRectangle::paintRectangle(QPainter& gc, const QRect&)
 
         gc.setPen(pen);
 
-	start = QPoint(static_cast<int>(m_dragStart.x()), static_cast<int>(m_dragStart.y()));
-	end = QPoint(static_cast<int>(m_dragEnd.x()), static_cast<int>(m_dragEnd.y()));
-	gc.drawRect(QRect(start, end));
+        start = QPoint(static_cast<int>(m_dragStart.x()), static_cast<int>(m_dragStart.y()));
+        end = QPoint(static_cast<int>(m_dragEnd.x()), static_cast<int>(m_dragEnd.y()));
+        gc.drawRect(QRect(start, end));
         gc.setPen(old);
     }
 }
