@@ -19,7 +19,6 @@
  */
 
 #include "kis_tool_select_brush.h"
-
 #include <QEvent>
 #include <QLabel>
 #include <QLayout>
@@ -27,15 +26,14 @@
 #include <QRect>
 #include <QVBoxLayout>
 
-#include <kdebug.h>
+#include <kis_debug.h>
 #include <kactioncollection.h>
 #include <kaction.h>
 #include <klocale.h>
 
 #include "KoPointerEvent.h"
 
-#include "kis_brush.h"
-#include "kis_cmb_composite.h"
+#include "widgets/kis_cmb_composite.h"
 #include "kis_cursor.h"
 #include "kis_doc2.h"
 #include "kis_paintop.h"
@@ -49,11 +47,10 @@
 #include "kis_undo_adapter.h"
 
 KisToolSelectBrush::KisToolSelectBrush(KoCanvasBase *canvas)
-        : super(canvas, KisCursor::load("tool_brush_selection_cursor.png", 5, 5),
-                i18n( "Selection Brush" ))
+        : KisToolFreehand(canvas, KisCursor::load("tool_brush_selection_cursor.png", 5, 5),
+                          i18n("Selection Brush"))
 {
     setObjectName("tool_select_brush");
-    m_optWidget = 0;
     m_paintOnSelection = true;
 }
 
@@ -61,49 +58,38 @@ KisToolSelectBrush::~KisToolSelectBrush()
 {
 }
 
-void KisToolSelectBrush::activate()
+void KisToolSelectBrush::activate(bool tmp)
 {
-    super::activate();
-
-    if (!m_optWidget)
-        return;
-
-    m_optWidget->slotActivated();
+    KisToolFreehand::activate(tmp);
 }
 
 void KisToolSelectBrush::initPaint(KoPointerEvent* /*e*/)
 {
-    if (!currentImage() || !currentLayer()->paintDevice()) return;
+    if (!currentImage() || !currentNode() || !currentNode()->paintDevice()) return;
 
     m_mode = PAINT;
     m_dragDist = 0;
 
     // Create painter
-    KisPaintDeviceSP dev = currentLayer()->paintDevice();
+    KisPaintDeviceSP dev = currentNode()->paintDevice();
     if (m_painter)
         delete m_painter;
-    bool hasSelection = currentLayer()->selection();
 #if 0 // XXX_SELECTION
+    bool hasSelection = currentSelection();
+
     if (currentImage()->undo())
         m_transaction = new KisSelectedTransaction(i18n("Selection Brush"), dev);
 
-    if(! hasSelection)
-    {
-        dev->selection()->clear();
-        dev->emitSelectionChanged();
-    }
 #endif
-    KisSelectionSP selection = currentLayer()->selection();
+    KisSelectionSP selection = currentSelection();
 
     m_target = selection.data();
     m_painter = new KisPainter(selection);
     Q_CHECK_PTR(m_painter);
     m_painter->setPaintColor(KoColor(Qt::black, selection->colorSpace()));
-    m_painter->setBrush(currentBrush());
     m_painter->setOpacity(OPACITY_OPAQUE);//m_currentFgColor.colorSpace()->intensity8(m_currentFgColor));
     m_painter->setCompositeOp(selection->colorSpace()->compositeOp(COMPOSITE_OVER));
-    KisPaintOp * op = KisPaintOpRegistry::instance()->paintOp("paintbrush", 0, m_painter, currentImage());
-    m_painter->setPaintOp(op); // And now the painter owns the op and will destroy it.
+    m_painter->setPaintOpPreset(currentPaintOpPreset(), currentImage());
 
     // Set the cursor -- ideally. this should be a mask created from the brush,
     // now that X11 can handle colored cursors.
@@ -117,7 +103,7 @@ void KisToolSelectBrush::initPaint(KoPointerEvent* /*e*/)
 void KisToolSelectBrush::endPaint()
 {
     m_mode = HOVER;
-    if (currentImage() && currentLayer()) {
+    if (currentImage() && currentNode()) {
         if (currentImage()->undo() && m_painter) {
             // If painting in mouse release, make sure painter
             // is destructed or end()ed
@@ -125,38 +111,8 @@ void KisToolSelectBrush::endPaint()
         }
         delete m_painter;
         m_painter = 0;
-#if 0
-        if (currentLayer()->paintDevice())
-            currentLayer()->paintDevice()->emitSelectionChanged();
-#endif
         //notifyModified();
     }
 }
 
-QWidget* KisToolSelectBrush::createOptionWidget()
-{
-    // Commented out due to the fact that this doesn't actually work if you change the action
-#if 0
-    KisCanvas2* canvas = dynamic_cast<KisCanvas2*>(m_canvas);
-    Q_ASSERT(canvas);
-    m_optWidget = new KisSelectionOptions(canvas);
-    Q_CHECK_PTR(m_optWidget);
-    m_optWidget->setWindowTitle(i18n("Selection Brush"));
-
-    QVBoxLayout * l = dynamic_cast<QVBoxLayout*>(m_optWidget->layout());
-    Q_ASSERT(l);
-    if (l) {
-        l->addItem(new QSpacerItem(1, 1, QSizePolicy::Fixed, QSizePolicy::Expanding));
-    }
-
-    return m_optWidget;
-#endif
-    return 0;
-}
-
-QWidget* KisToolSelectBrush::optionWidget()
-{
-    return m_optWidget;
-}
-
 #include "kis_tool_select_brush.moc"
