@@ -23,7 +23,7 @@
 
 #include "KoToolFactory.h"
 
-#include "kis_layer_shape.h"
+#include "flake/kis_layer_shape.h"
 #include "kis_tool_freehand.h"
 
 class KisSelectionOptions;
@@ -33,19 +33,16 @@ class KisSelectionOptions;
  * current eraser shape. Not sure what kind of an icon could represent
  * this... Depends a bit on how we're going to visualize selections.
  */
-class KisToolSelectEraser : public KisToolFreehand {
+class KisToolSelectEraser : public KisToolFreehand
+{
     Q_OBJECT
-    typedef KisToolFreehand super;
 
 public:
     KisToolSelectEraser(KoCanvasBase *canvas);
     virtual ~KisToolSelectEraser();
 
-    virtual QWidget* createOptionWidget();
-    virtual QWidget* optionWidget();
-
 public slots:
-    virtual void activate();
+    virtual void activate(bool tmp);
 
 protected:
     virtual void initPaint(KoPointerEvent *e);
@@ -54,21 +51,21 @@ private:
     KisSelectionOptions * m_optWidget;
 };
 
-class KisToolSelectEraserFactory : public KoToolFactory {
-    typedef KoToolFactory super;
+class KisToolSelectEraserFactory : public KoToolFactory
+{
+
 public:
     KisToolSelectEraserFactory(QObject *parent, const QStringList&)
-        : KoToolFactory(parent,  "KisToolSelectEraser", i18n( "Selection Eraser" ))
-        {
-            setToolTip( i18n( "Erase parts of a selection with a brush" ) );
-            setToolType( TOOL_TYPE_SELECTED );
-            setIcon( "tool_eraser_selection" );
-            setShortcut(KShortcut(Qt::CTRL+Qt::SHIFT+Qt::Key_E));
-            setPriority( 51 );
-            setActivationShapeId( KIS_LAYER_SHAPE_ID );
-        }
+            : KoToolFactory(parent,  "KisToolSelectEraser", i18n("Selection Eraser")) {
+        setToolTip(i18n("Erase parts of a selection with a brush"));
+        setToolType(TOOL_TYPE_SELECTED);
+        setIcon("tool_eraser_selection");
+        setShortcut(KShortcut(Qt::CTRL + Qt::SHIFT + Qt::Key_E));
+        setPriority(51);
+        //setActivationShapeId( KIS_LAYER_SHAPE_ID );
+    }
 
-    virtual ~KisToolSelectEraserFactory(){}
+    virtual ~KisToolSelectEraserFactory() {}
 
     virtual KoTool * createTool(KoCanvasBase *canvas) {
         return  new KisToolSelectEraser(canvas);
