@@ -26,7 +26,7 @@
 
 #include <KoToolFactory.h>
 #include "kis_tool.h"
-#include "kis_layer_shape.h"
+#include "flake/kis_layer_shape.h"
 
 #include "ui_wdg_tool_crop.h"
 
@@ -36,8 +36,10 @@ class WdgToolCrop : public QWidget, public Ui::WdgToolCrop
 {
     Q_OBJECT
 
-    public:
-        WdgToolCrop(QWidget *parent) : QWidget(parent) { setupUi(this); }
+public:
+    WdgToolCrop(QWidget *parent) : QWidget(parent) {
+        setupUi(this);
+    }
 };
 
 /**
@@ -47,9 +49,9 @@ class WdgToolCrop : public QWidget, public Ui::WdgToolCrop
  *       - (when moving to Qt 4: replace rectangle with  darker, dimmer overlay layer
  *         like we have for selections right now)
  */
-class KisToolCrop : public KisTool {
+class KisToolCrop : public KisTool
+{
 
-    typedef KisTool super;
     Q_OBJECT
 
 public:
@@ -57,30 +59,28 @@ public:
     KisToolCrop(KoCanvasBase * canvas);
     virtual ~KisToolCrop();
 
-     virtual QWidget* createOptionWidget();
+    virtual QWidget* createOptionWidget();
     virtual QWidget* optionWidget();
 
-//     virtual enumToolType toolType() { return TOOL_TRANSFORM; }
-    virtual quint32 priority() { return 1; }
-
     virtual void mousePressEvent(KoPointerEvent *e);
     virtual void mouseMoveEvent(KoPointerEvent *e);
     virtual void mouseReleaseEvent(KoPointerEvent *e);
-    virtual void mouseDoubleClickEvent( KoPointerEvent *e );
+    virtual void mouseDoubleClickEvent(KoPointerEvent *e);
 
-    virtual void paint( QPainter &painter, const KoViewConverter &converter );
+    virtual void paint(QPainter &painter, const KoViewConverter &converter);
 
 public slots:
 
-    virtual void activate();
+    virtual void activate(bool);
     virtual void deactivate();
+
 private:
     QRectF boundingRect();
     QRectF borderLineRect();
     QPainterPath handlesPath();
     void paintOutlineWithHandles(QPainter& gc);
-    qint32 mouseOnHandle (const QPointF currentViewPoint);
-    void setMoveResizeCursor (qint32 handle);
+    qint32 mouseOnHandle(const QPointF currentViewPoint);
+    void setMoveResizeCursor(qint32 handle);
     void validateSelection(bool updateratio = true);
     void setOptionWidgetX(qint32 x);
     void setOptionWidgetY(qint32 y);
@@ -117,8 +117,7 @@ private:
     bool m_haveCropSelection;
     qint32 m_mouseOnHandleType;
 
-    enum handleType
-    {
+    enum handleType {
         None = 0,
         UpperLeft = 1,
         UpperRight = 2,
@@ -132,20 +131,20 @@ private:
     };
 };
 
-class KisToolCropFactory : public KoToolFactory {
+class KisToolCropFactory : public KoToolFactory
+{
 
 public:
     KisToolCropFactory(QObject *parent, const QStringList&)
-        : KoToolFactory(parent, "KisToolCrop", i18n( "Crop" ))
-        {
-            setToolTip(i18n("Crop the image to an area"));
-            setToolType(TOOL_TYPE_TRANSFORM);
-            setActivationShapeId( KIS_LAYER_SHAPE_ID );
-            setPriority( 10 );
-            setIcon("tool_crop");
-        }
+            : KoToolFactory(parent, "KisToolCrop", i18n("Crop")) {
+        setToolTip(i18n("Crop the image to an area"));
+        setToolType(TOOL_TYPE_TRANSFORM);
+        //setActivationShapeId( KIS_LAYER_SHAPE_ID );
+        setPriority(10);
+        setIcon("tool_crop");
+    }
 
-    virtual ~KisToolCropFactory(){}
+    virtual ~KisToolCropFactory() {}
 
     virtual KoTool * createTool(KoCanvasBase *canvas) {
         return new KisToolCrop(canvas);
