@@ -18,33 +18,46 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-
+#include "kis_tool_polygon.h"
 #include <math.h>
 
 #include <QPainter>
 #include <QSpinBox>
 
 #include <kaction.h>
-#include <kdebug.h>
 #include <kactioncollection.h>
 #include <klocale.h>
-#include <kdebug.h>
+#include <kis_debug.h>
 #include <knuminput.h>
 
 #include <KoPointerEvent.h>
 #include <KoCanvasBase.h>
 
+#include <kis_selection.h>
 #include "kis_painter.h"
+#include <kis_paint_device.h>
 #include "kis_paintop_registry.h"
 #include "kis_cursor.h"
 
-#include "kis_tool_polygon.h"
 
 KisToolPolygon::KisToolPolygon(KoCanvasBase *canvas)
-        : super(canvas, KisCursor::load("tool_polygon_cursor.png", 6, 6)),
-          m_dragging (false)
+        : KisToolShape(canvas, KisCursor::load("tool_polygon_cursor.png", 6, 6)),
+        m_dragging(false)
 {
     setObjectName("tool_polygon");
+
+    KAction *action = new KAction(i18n("&Finish Polygon"), this);
+    addAction("finish_polygon", action);
+    connect(action, SIGNAL(triggered()), this, SLOT(finish()));
+    action = new KAction(KIcon("dialog-cancel"), i18n("&Cancel"), this);
+    addAction("cancel_polygon", action);
+    connect(action, SIGNAL(triggered()), this, SLOT(cancel()));
+
+
+    QList<QAction*> list;
+    list.append(this->action("finish_polygon"));
+    list.append(this->action("cancel_polygon"));
+    setPopupActionList(list);
 }
 
 KisToolPolygon::~KisToolPolygon()
@@ -58,8 +71,7 @@ void KisToolPolygon::mousePressEvent(KoPointerEvent *event)
 
             m_dragging = true;
 
-            if (m_points.isEmpty())
-            {
+            if (m_points.isEmpty()) {
                 m_dragStart = convertToPixelCoord(event);
                 m_dragEnd = m_dragStart;
                 m_points.append(m_dragStart);
@@ -92,55 +104,48 @@ void KisToolPolygon::mouseReleaseEvent(KoPointerEvent *event)
         return;
 
     if (m_dragging && event->button() == Qt::LeftButton)  {
-            m_dragging = false;
-            m_points.append (m_dragEnd);
+        m_dragging = false;
+        m_points.append(m_dragEnd);
     }
 
     if (m_dragging && event->button() == Qt::RightButton) {
 
-        }
+    }
 }
 
-void KisToolPolygon::mouseDoubleClickEvent( KoPointerEvent * )
+void KisToolPolygon::mouseDoubleClickEvent(KoPointerEvent *)
 {
     finish();
 }
 
 void KisToolPolygon::keyPressEvent(QKeyEvent *e)
 {
-    if (e->key()==Qt::Key_Escape)
-    {
-        m_dragging = false;
-        m_points.clear();
-        // erase old lines on canvas
-        updateCanvasPixelRect(image()->bounds());
+    if (e->key() == Qt::Key_Escape) {
+        cancel();
     }
 }
 
+void KisToolPolygon::cancel()
+{
+    m_dragging = false;
+    m_points.clear();
+    // erase old lines on canvas
+    updateCanvasPixelRect(image()->bounds());
+}
+
 void KisToolPolygon::finish()
 {
     m_dragging = false;
 
-    KisPaintDeviceSP device = currentLayer()->paintDevice();
+    KisPaintDeviceSP device = currentNode()->paintDevice();
     if (device) {
-        KisPainter painter (device, currentLayer()->selection());
-        if (currentImage()->undo()) painter.beginTransaction (i18n ("Polygon"));
-        painter.setBounds( currentImage()->bounds() );
-        painter.setPaintColor(currentFgColor());
-        painter.setBackgroundColor(currentBgColor());
-        painter.setFillStyle(fillStyle());
-        painter.setStrokeStyle(strokeStyle());
-        painter.setBrush(currentBrush());
-        painter.setPattern(currentPattern());
+        KisPainter painter(device, currentSelection());
+        if (currentImage()->undo()) painter.beginTransaction(i18n("Polygon"));
+        setupPainter(&painter);
         painter.setOpacity(m_opacity);
         painter.setCompositeOp(m_compositeOp);
-        KisPaintOp * op = KisPaintOpRegistry::instance()->paintOp(currentPaintOp(),
-                                                                  currentPaintOpSettings(),
-                                                                  &painter, currentImage());
-        painter.setPaintOp(op); // Painter takes ownership
-
         painter.paintPolygon(m_points);
-        device->setDirty( painter.dirtyRegion() );
+        device->setDirty(painter.dirtyRegion());
         notifyModified();
 
         m_canvas->addCommand(painter.endTransaction());
@@ -178,8 +183,7 @@ void KisToolPolygon::paint(QPainter& gc, const KoViewConverter &converter)
     }
     for (vQPointF::iterator it = m_points.begin(); it != m_points.end(); ++it) {
 
-        if (it == m_points.begin())
-        {
+        if (it == m_points.begin()) {
             start = (*it);
         } else {
             end = (*it);
