@@ -18,6 +18,7 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
+#include "kis_tool_star.h"
 
 #include <math.h>
 
@@ -26,31 +27,28 @@
 #include <QLayout>
 #include <QGridLayout>
 
-#include <kdebug.h>
 #include <klocale.h>
-#include <kdebug.h>
 #include <knuminput.h>
 
 #include "KoCanvasBase.h"
-#include "kis_canvas2.h"
-#include "kis_view2.h"
-#include "kis_painter.h"
-#include "kis_int_spinbox.h"
 #include "KoPointerEvent.h"
-#include "kis_paintop_registry.h"
-#include "QPainter"
-#include "kis_cursor.h"
-#include "kis_int_spinbox.h"
 
-#include "kis_tool_star.h"
+#include <kis_debug.h>
+#include <canvas/kis_canvas2.h>
+#include <kis_painter.h>
+#include <kis_paintop_registry.h>
+#include <kis_cursor.h>
+#include <kis_paint_device.h>
+
+#include "kis_selection.h"
 
 KisToolStar::KisToolStar(KoCanvasBase * canvas)
-    : super(canvas, KisCursor::load("tool_star_cursor.png", 6, 6)),
-      m_dragging (false)
+        : KisToolShape(canvas, KisCursor::load("tool_star_cursor.png", 6, 6)),
+        m_dragging(false)
 {
     setObjectName("tool_star");
-    m_innerOuterRatio=40;
-    m_vertices=5;
+    m_innerOuterRatio = 40;
+    m_vertices = 5;
 }
 
 KisToolStar::~KisToolStar()
@@ -97,29 +95,21 @@ void KisToolStar::mouseReleaseEvent(KoPointerEvent *event)
         if (!currentImage())
             return;
 
-        if (!currentLayer()->paintDevice())
+        if (!currentNode()->paintDevice())
             return;
 
-        KisPaintDeviceSP device = currentLayer()->paintDevice();
-        KisPainter painter (device, currentLayer()->selection());
-        painter.beginTransaction (i18n("Star"));
-        painter.setBounds( currentImage()->bounds() );
-        painter.setPaintColor(currentFgColor());
-        painter.setBackgroundColor(currentBgColor());
-        painter.setFillStyle(fillStyle());
-        painter.setStrokeStyle(strokeStyle());
-        painter.setBrush(currentBrush());
-        painter.setPattern(currentPattern());
+        KisPaintDeviceSP device = currentNode()->paintDevice();
+        KisPainter painter(device, currentSelection());
+        painter.beginTransaction(i18n("Star"));
+        setupPainter(&painter);
         painter.setOpacity(m_opacity);
         painter.setCompositeOp(m_compositeOp);
-        KisPaintOp * op = KisPaintOpRegistry::instance()->paintOp(currentPaintOp(), currentPaintOpSettings(), &painter, currentImage());
-        painter.setPaintOp(op); // Painter takes ownership
 
         vQPointF coord = starCoordinates(m_vertices, m_dragStart.x(), m_dragStart.y(), m_dragEnd.x(), m_dragEnd.y());
 
         painter.paintPolygon(coord);
 
-        device->setDirty( painter.dirtyRegion() );
+        device->setDirty(painter.dirtyRegion());
         notifyModified();
         m_canvas->updateCanvas(convertToPt(boundingRect()));
 
@@ -132,10 +122,10 @@ void KisToolStar::paint(QPainter& gc, const KoViewConverter &converter)
     if (!m_dragging)
         return;
 
-    double sx, sy;
+    qreal sx, sy;
     converter.zoom(&sx, &sy);
 
-    gc.scale( sx/currentImage()->xRes(), sy/currentImage()->yRes() );
+    gc.scale(sx / currentImage()->xRes(), sy / currentImage()->yRes());
 
     if (!m_canvas)
         return;
@@ -153,29 +143,29 @@ void KisToolStar::paint(QPainter& gc, const KoViewConverter &converter)
 
 vQPointF KisToolStar::starCoordinates(int N, double mx, double my, double x, double y)
 {
-    double R=0, r=0;
-    qint32 n=0;
+    double R = 0, r = 0;
+    qint32 n = 0;
     double angle;
 
     vQPointF starCoordinatesArray(2*N);
 
     // the radius of the outer edges
-    R=sqrt((x-mx)*(x-mx)+(y-my)*(y-my));
+    R = sqrt((x - mx) * (x - mx) + (y - my) * (y - my));
 
     // the radius of the inner edges
-    r=R*m_innerOuterRatio/100.0;
+    r = R * m_innerOuterRatio / 100.0;
 
     // the angle
-    angle=-atan2((x-mx),(y-my));
+    angle = -atan2((x - mx), (y - my));
 
     //set outer edges
-    for(n=0;n<N;n++){
-        starCoordinatesArray[2*n] = QPointF(mx+R*cos(n * 2.0 * M_PI / N + angle),my+R*sin(n *2.0 * M_PI / N+angle));
+    for (n = 0;n < N;n++) {
+        starCoordinatesArray[2*n] = QPointF(mx + R * cos(n * 2.0 * M_PI / N + angle), my + R * sin(n * 2.0 * M_PI / N + angle));
     }
 
     //set inner edges
-    for(n=0;n<N;n++){
-        starCoordinatesArray[2*n+1] = QPointF(mx+r*cos((n + 0.5) * 2.0 * M_PI / N + angle),my+r*sin((n +0.5) * 2.0 * M_PI / N + angle));
+    for (n = 0;n < N;n++) {
+        starCoordinatesArray[2*n+1] = QPointF(mx + r * cos((n + 0.5) * 2.0 * M_PI / N + angle), my + r * sin((n + 0.5) * 2.0 * M_PI / N + angle));
     }
 
     return starCoordinatesArray;
@@ -184,21 +174,22 @@ vQPointF KisToolStar::starCoordinates(int N, double mx, double my, double x, dou
 QRectF KisToolStar::boundingRect()
 {
     //Calculating the radius
-    double radius = sqrt((m_dragEnd.x()-m_dragStart.x())*(m_dragEnd.x()-m_dragStart.x())+(m_dragEnd.y()-m_dragStart.y())*((m_dragEnd.y()-m_dragStart.y())));
-    return QRectF(m_dragStart.x()-radius, m_dragStart.y()-radius, 2*radius, 2*radius);
+    double radius = sqrt((m_dragEnd.x() - m_dragStart.x()) * (m_dragEnd.x() - m_dragStart.x()) + (m_dragEnd.y() - m_dragStart.y()) * ((m_dragEnd.y() - m_dragStart.y())));
+    return QRectF(m_dragStart.x() - radius, m_dragStart.y() - radius, 2*radius, 2*radius);
 }
 
 QWidget* KisToolStar::createOptionWidget()
 {
-    QWidget *widget = super::createOptionWidget();
+    QWidget *widget = KisToolShape::createOptionWidget();
 
     m_optWidget = new WdgToolStar(widget);
     Q_CHECK_PTR(m_optWidget);
-
+    m_optWidget->setObjectName(toolId() + " option widget");
+    
     m_optWidget->ratioSpinBox->setValue(m_innerOuterRatio);
 
     QGridLayout *optionLayout = new QGridLayout(widget);
-    super::addOptionWidgetLayout(optionLayout);
+    KisToolShape::addOptionWidgetLayout(optionLayout);
 
     optionLayout->addWidget(m_optWidget, 0, 0);
 
