@@ -19,6 +19,8 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
+#include "kis_tool_transform.h"
+
 
 #include <QPainter>
 #include <QPen>
@@ -28,7 +30,7 @@
 #include <QComboBox>
 #include <QApplication>
 
-#include <kdebug.h>
+#include <kis_debug.h>
 #include <kactioncollection.h>
 #include <kaction.h>
 #include <klocale.h>
@@ -38,6 +40,7 @@
 #include <KoID.h>
 #include <KoCanvasBase.h>
 #include <KoViewConverter.h>
+#include <KoSelection.h>
 #include <KoCompositeOp.h>
 #include <KoShapeManager.h>
 
@@ -49,42 +52,43 @@
 #include <kis_selected_transaction.h>
 #include <kis_selection.h>
 #include <kis_filter_strategy.h>
-#include <kis_cmb_idlist.h>
+#include <widgets/kis_cmb_idlist.h>
 #include <kis_transform_worker.h>
-#include "kis_tool_transform.h"
-#include "kis_layer_shape.h"
-#include "kis_mask_shape.h"
-#include "kis_layer_container_shape.h"
-#include "kis_shape_layer.h"
-#include "kis_resource_provider.h"
 
-namespace {
-    class TransformCmd : public KisSelectedTransaction {
-        typedef KisSelectedTransaction super;
+#include "flake/kis_layer_shape.h"
+#include "flake/kis_mask_shape.h"
+#include "flake/kis_layer_container_shape.h"
+#include "flake/kis_shape_layer.h"
+#include "kis_canvas_resource_provider.h"
 
-    public:
-        TransformCmd(KisToolTransform *tool, KisPaintDeviceSP device, double scaleX, double scaleY, QPointF translate, double a, KisSelectionSP origSel, QPoint startPos, QPoint endPos);
-        virtual ~TransformCmd();
+namespace
+{
+class TransformCmd : public KisSelectedTransaction
+{
 
-    public:
-        virtual void redo();
-        virtual void undo();
-        void transformArgs(double &sx, double &sy, QPointF &translate, double &a);
-        KisSelectionSP origSelection(QPoint &startPos, QPoint &endPos);
+public:
+    TransformCmd(KisToolTransform *tool, KisNodeSP node, double scaleX, double scaleY, QPointF translate, double a, KisSelectionSP origSel, QPoint startPos, QPoint endPos);
+    virtual ~TransformCmd();
 
-    private:
-        double m_scaleX;
-        double m_scaleY;
-        QPointF m_translate;
-        double m_a;
-        KisToolTransform *m_tool;
-        KisSelectionSP m_origSelection;
-        QPoint m_originalTopLeft;
-        QPoint m_originalBottomRight;
-    };
+public:
+    virtual void redo();
+    virtual void undo();
+    void transformArgs(double &sx, double &sy, QPointF &translate, double &a);
+    KisSelectionSP origSelection(QPoint &startPos, QPoint &endPos);
 
-    TransformCmd::TransformCmd(KisToolTransform *tool, KisPaintDeviceSP device, double scaleX, double scaleY, QPointF translate, double a, KisSelectionSP origSel, QPoint originalTopLeft, QPoint originalBottomRight) :
-        super(i18n("Transform"), device)
+private:
+    double m_scaleX;
+    double m_scaleY;
+    QPointF m_translate;
+    double m_a;
+    KisToolTransform *m_tool;
+    KisSelectionSP m_origSelection;
+    QPoint m_originalTopLeft;
+    QPoint m_originalBottomRight;
+};
+
+TransformCmd::TransformCmd(KisToolTransform *tool, KisNodeSP node, double scaleX, double scaleY, QPointF translate, double a, KisSelectionSP origSel, QPoint originalTopLeft, QPoint originalBottomRight)
+        : KisSelectedTransaction(i18n("Transform"), node)
         , m_scaleX(scaleX)
         , m_scaleY(scaleY)
         , m_translate(translate)
@@ -93,42 +97,42 @@ namespace {
         , m_origSelection(origSel)
         , m_originalTopLeft(originalTopLeft)
         , m_originalBottomRight(originalBottomRight)
-    {
-    }
+{
+}
 
-    TransformCmd::~TransformCmd()
-    {
-    }
+TransformCmd::~TransformCmd()
+{
+}
 
-    void TransformCmd::transformArgs(double &sx, double &sy, QPointF &translate, double &a)
-    {
-        sx = m_scaleX;
-        sy = m_scaleY;
-        translate = m_translate;
-        a = m_a;
-    }
+void TransformCmd::transformArgs(double &sx, double &sy, QPointF &translate, double &a)
+{
+    sx = m_scaleX;
+    sy = m_scaleY;
+    translate = m_translate;
+    a = m_a;
+}
 
-    KisSelectionSP TransformCmd::origSelection(QPoint &originalTopLeft, QPoint &originalBottomRight)
-    {
-        originalTopLeft = m_originalTopLeft;
-        originalBottomRight = m_originalBottomRight;
-        return m_origSelection;
-    }
+KisSelectionSP TransformCmd::origSelection(QPoint &originalTopLeft, QPoint &originalBottomRight)
+{
+    originalTopLeft = m_originalTopLeft;
+    originalBottomRight = m_originalBottomRight;
+    return m_origSelection;
+}
 
-    void TransformCmd::redo()
-    {
-        super::redo();
-    }
+void TransformCmd::redo()
+{
+    KisSelectedTransaction::redo();
+}
 
-    void TransformCmd::undo()
-    {
-        super::undo();
-    }
+void TransformCmd::undo()
+{
+    KisSelectedTransaction::undo();
+}
 }
 
 KisToolTransform::KisToolTransform(KoCanvasBase * canvas)
-    : super(canvas)
-    , m_canvas(canvas)
+        : KisTool(canvas, KisCursor::rotateCursor())
+        , m_canvas(canvas)
 {
     setObjectName("tool_transform");
     useCursor(KisCursor::selectCursor());
@@ -155,7 +159,7 @@ KisToolTransform::~KisToolTransform()
 void KisToolTransform::deactivate()
 {
     if (image()->undoAdapter())
-        image()->undoAdapter()->removeCommandHistoryListener( this );
+        image()->undoAdapter()->removeCommandHistoryListener(this);
 
     if (image()) return;
 
@@ -166,21 +170,18 @@ void KisToolTransform::activate(bool temporary)
 {
     Q_UNUSED(temporary);
 
-    if( m_currentLayer && m_currentLayer->paintDevice() )
-    {
+    if (currentNode() && currentNode()->paintDevice()) {
         //connect(m_subject, commandExecuted(K3Command *c), this, notifyCommandAdded( KCommand * c));
         //m_subject->undoAdapter()->setCommandHistoryListener( this );
 
-        TransformCmd * cmd=0;
+        TransformCmd * cmd = 0;
 
-        if(image()->undoAdapter()->presentCommand())
+        if (image()->undoAdapter()->presentCommand())
             cmd = dynamic_cast<TransformCmd*>(image()->undoAdapter()->presentCommand());
 
         if (cmd == 0) {
             initHandles();
-        }
-        else
-        {
+        } else {
             // One of our commands is on top
             // We should ask for tool args and orig selection
             cmd->transformArgs(m_scaleX, m_scaleY, m_translate, m_a);
@@ -188,33 +189,32 @@ void KisToolTransform::activate(bool temporary)
             m_canvas->updateCanvas(QRect(m_originalTopLeft, m_originalBottomRight));
         }
     }
-    m_currentLayer = m_canvas->resourceProvider()->resource( KisResourceProvider::CurrentKritaLayer ).value<KisLayerSP>();
+    currentNode() =
+        m_canvas->resourceProvider()->resource(KisCanvasResourceProvider::CurrentKritaNode).value<KisNodeSP>();
 }
 
 void KisToolTransform::initHandles()
 {
-    int x,y,w,h;
+    int x, y, w, h;
 
-    KisPaintDeviceSP dev = m_currentLayer->paintDevice();
+    KisPaintDeviceSP dev = currentNode()->paintDevice();
 
     // Create a lazy copy of the current state
     m_origDevice = new KisPaintDevice(*dev.data());
     Q_ASSERT(m_origDevice);
 
-    KisSelectionSP selection = m_currentLayer->selection();
-    if(selection)
-    {
+    KisSelectionSP selection = currentSelection();
+    if (selection) {
         QRect r = selection->selectedExactRect();
         m_origSelection = new KisSelection(*selection.data());
         r.getRect(&x, &y, &w, &h);
-    }
-    else {
-        dev->exactBounds(x,y,w,h);
+    } else {
+        dev->exactBounds(x, y, w, h);
         m_origSelection = 0;
     }
     m_originalTopLeft = QPoint(x, y);
-    m_originalBottomRight = QPoint(x+w-1, y+h-1);
-    m_originalCenter = QPointF(m_originalTopLeft + m_originalBottomRight + QPoint(1,1)) / 2.0;
+    m_originalBottomRight = QPoint(x + w - 1, y + h - 1);
+    m_originalCenter = QPointF(m_originalTopLeft + m_originalBottomRight + QPoint(1, 1)) / 2.0;
 
     m_a = 0.0;
     m_scaleX = 1.0;
@@ -228,101 +228,99 @@ void KisToolTransform::mousePressEvent(KoPointerEvent *e)
 {
     KisImageSP img = image();
 
-    if (img && m_currentLayer->paintDevice() && e->button() == Qt::LeftButton) {
+    if (img && currentNode()->paintDevice() && e->button() == Qt::LeftButton) {
         QPointF mousePos = QPointF(e->point.x() * img->xRes(), e->point.y() * img->yRes());
-        switch(m_function)
-        {
-            case ROTATE:
-                m_clickoffset = mousePos - m_translate;
-                m_clickangle = -m_a - atan2(m_clickoffset.x(),m_clickoffset.y());
-                m_clickoffset = QPoint(0, 0);
-                break;
-            case MOVE:
-                m_clickoffset = mousePos - m_translate;
-                break;
-            case TOPSCALE:
-                m_clickoffset = mousePos - (m_topleft + m_topright)/2.0;
-                break;
-            case TOPRIGHTSCALE:
-                m_clickoffset = mousePos - m_topright;
-                break;
-            case RIGHTSCALE:
-                m_clickoffset = mousePos - (m_topright + m_bottomright)/2.0;
-                break;
-            case BOTTOMRIGHTSCALE:
-                m_clickoffset = mousePos - m_bottomright;
-                break;
-            case BOTTOMSCALE:
-                m_clickoffset = mousePos - (m_bottomleft + m_bottomright)/2.0;
-                break;
-            case BOTTOMLEFTSCALE:
-                m_clickoffset = mousePos - m_bottomleft;
-                break;
-            case LEFTSCALE:
-                m_clickoffset = mousePos - (m_topleft + m_bottomleft)/2.0;
-                break;
-            case TOPLEFTSCALE:
-                m_clickoffset = mousePos - m_topleft;
-                break;
+        switch (m_function) {
+        case ROTATE:
+            m_clickoffset = mousePos - m_translate;
+            m_clickangle = -m_a - atan2(m_clickoffset.x(), m_clickoffset.y());
+            m_clickoffset = QPoint(0, 0);
+            break;
+        case MOVE:
+            m_clickoffset = mousePos - m_translate;
+            break;
+        case TOPSCALE:
+            m_clickoffset = mousePos - (m_topleft + m_topright) / 2.0;
+            break;
+        case TOPRIGHTSCALE:
+            m_clickoffset = mousePos - m_topright;
+            break;
+        case RIGHTSCALE:
+            m_clickoffset = mousePos - (m_topright + m_bottomright) / 2.0;
+            break;
+        case BOTTOMRIGHTSCALE:
+            m_clickoffset = mousePos - m_bottomright;
+            break;
+        case BOTTOMSCALE:
+            m_clickoffset = mousePos - (m_bottomleft + m_bottomright) / 2.0;
+            break;
+        case BOTTOMLEFTSCALE:
+            m_clickoffset = mousePos - m_bottomleft;
+            break;
+        case LEFTSCALE:
+            m_clickoffset = mousePos - (m_topleft + m_bottomleft) / 2.0;
+            break;
+        case TOPLEFTSCALE:
+            m_clickoffset = mousePos - m_topleft;
+            break;
         }
         m_selecting = true;
         m_actualyMoveWhileSelected = false;
     }
 }
 
-int KisToolTransform::det(QPointF v,QPointF w)
+int KisToolTransform::det(const QPointF & v, const QPointF & w)
 {
-    return int(v.x()*w.y()-v.y()*w.x());
+    return int(v.x()*w.y() - v.y()*w.x());
 }
 
-double KisToolTransform::distsq(QPointF v,QPointF w)
+double KisToolTransform::distsq(const QPointF & v, const QPointF & w)
 {
-    v -= w;
-    return v.x()*v.x() + v.y()*v.y();
+    QPointF v2 = v - w;
+    return v2.x()*v2.x() + v2.y()*v2.y();
 }
 
 void KisToolTransform::setFunctionalCursor()
 {
-    int rotOctant = 8 + int(8.5 + m_a* 4 / M_PI);
+    int rotOctant = 8 + int(8.5 + m_a * 4 / M_PI);
 
     int s;
-    if(m_scaleX*m_scaleY<0)
+    if ((m_scaleX * m_scaleY) < 0)
         s = -1;
     else
-        s=1;
+        s = 1;
 
-    switch(m_function)
-    {
-        case MOVE:
-            useCursor(KisCursor::moveCursor());
-            break;
-        case ROTATE:
-            useCursor(KisCursor::rotateCursor());
-            break;
-        case TOPSCALE:
-            useCursor(m_sizeCursors[(0*s +rotOctant)%8]);
-            break;
-        case TOPRIGHTSCALE:
-            useCursor(m_sizeCursors[(1*s +rotOctant)%8]);
-            break;
-        case RIGHTSCALE:
-            useCursor(m_sizeCursors[(2*s +rotOctant)%8]);
-            break;
-        case BOTTOMRIGHTSCALE:
-            useCursor(m_sizeCursors[(3*s +rotOctant)%8]);
-            break;
-        case BOTTOMSCALE:
-            useCursor(m_sizeCursors[(4*s +rotOctant)%8]);
-            break;
-        case BOTTOMLEFTSCALE:
-            useCursor(m_sizeCursors[(5*s +rotOctant)%8]);
-            break;
-        case LEFTSCALE:
-            useCursor(m_sizeCursors[(6*s +rotOctant)%8]);
-            break;
-        case TOPLEFTSCALE:
-            useCursor(m_sizeCursors[(7*s +rotOctant)%8]);
-            break;
+    switch (m_function) {
+    case MOVE:
+        useCursor(KisCursor::moveCursor());
+        break;
+    case ROTATE:
+        useCursor(KisCursor::rotateCursor());
+        break;
+    case TOPSCALE:
+        useCursor(m_sizeCursors[(0*s +rotOctant)%8]);
+        break;
+    case TOPRIGHTSCALE:
+        useCursor(m_sizeCursors[(1*s +rotOctant)%8]);
+        break;
+    case RIGHTSCALE:
+        useCursor(m_sizeCursors[(2*s +rotOctant)%8]);
+        break;
+    case BOTTOMRIGHTSCALE:
+        useCursor(m_sizeCursors[(3*s +rotOctant)%8]);
+        break;
+    case BOTTOMSCALE:
+        useCursor(m_sizeCursors[(4*s +rotOctant)%8]);
+        break;
+    case BOTTOMLEFTSCALE:
+        useCursor(m_sizeCursors[(5*s +rotOctant)%8]);
+        break;
+    case LEFTSCALE:
+        useCursor(m_sizeCursors[(6*s +rotOctant)%8]);
+        break;
+    case TOPLEFTSCALE:
+        useCursor(m_sizeCursors[(7*s +rotOctant)%8]);
+        break;
     }
 }
 
@@ -345,38 +343,33 @@ void KisToolTransform::mouseMoveEvent(KoPointerEvent *e)
 
         // transform mousePos coords, so it seems like it isn't rotated and centered at 0,0
         QPointF newpos = invrot(mousePos.x() - m_translate.x(), mousePos.y() - m_translate.y());
-        double dx=0, dy=0;
+        double dx = 0, dy = 0;
         double oldScaleX = m_scaleX;
         double oldScaleY = m_scaleY;
 
-        if(m_function == MOVE)
-        {
+        if (m_function == MOVE) {
             m_translate += mousePos - m_translate;
         }
 
-        if(m_function == ROTATE)
-        {
+        if (m_function == ROTATE) {
             m_a = -atan2(mousePos.x() - m_translate.x(), mousePos.y() - m_translate.y())
-                - m_clickangle;
+                  - m_clickangle;
         }
 
-        if(m_function == TOPSCALE)
-        {
+        if (m_function == TOPSCALE) {
             dy = (newpos.y() - m_scaleY * (m_originalTopLeft.y() - m_originalCenter.y())) / 2;
             m_scaleY = (newpos.y() - dy) / (m_originalTopLeft.y() - m_originalCenter.y());
 
             // enforce same acpect if shift button is pressed
-            if(e->modifiers() & Qt::ShiftModifier)
-            {
-                if(m_scaleX>0) // handle the mirrored cases
+            if (e->modifiers() & Qt::ShiftModifier) {
+                if (m_scaleX > 0) // handle the mirrored cases
                     m_scaleX = fabs(m_scaleY);
                 else
                     m_scaleX = -fabs(m_scaleY);
             }
         }
 
-        if(m_function == TOPRIGHTSCALE)
-        {
+        if (m_function == TOPRIGHTSCALE) {
             dx = (newpos.x() - m_scaleX * (m_originalBottomRight.x() - m_originalCenter.x())) / 2;
             m_scaleX = (newpos.x() - dx) / (m_originalBottomRight.x() - m_originalCenter.x());
 
@@ -384,19 +377,15 @@ void KisToolTransform::mouseMoveEvent(KoPointerEvent *e)
             m_scaleY = (newpos.y() - dy) / (m_originalTopLeft.y() - m_originalCenter.y());
 
             // enforce same aspect if shift button is pressed
-            if(e->modifiers() & Qt::ShiftModifier)
-            {
-                if(m_scaleX < m_scaleY)
-                {
-                    if(m_scaleX>0) // handle the mirrored cases
+            if (e->modifiers() & Qt::ShiftModifier) {
+                if (m_scaleX < m_scaleY) {
+                    if (m_scaleX > 0) // handle the mirrored cases
                         m_scaleX = fabs(m_scaleY);
                     else
                         m_scaleX = -fabs(m_scaleY);
                     dx = (m_scaleX - oldScaleX) * (m_originalBottomRight.x() - m_originalCenter.x());
-                }
-                else
-                {
-                    if(m_scaleY>0) // handle the mirrored cases
+                } else {
+                    if (m_scaleY > 0) // handle the mirrored cases
                         m_scaleY = fabs(m_scaleX);
                     else
                         m_scaleY = -fabs(m_scaleX);
@@ -405,23 +394,20 @@ void KisToolTransform::mouseMoveEvent(KoPointerEvent *e)
             }
         }
 
-        if(m_function == RIGHTSCALE)
-        {
+        if (m_function == RIGHTSCALE) {
             dx = (newpos.x() - m_scaleX * (m_originalBottomRight.x() - m_originalCenter.x())) / 2;
             m_scaleX = (newpos.x() - dx) / (m_originalBottomRight.x() - m_originalCenter.x());
 
             // enforce same acpect if shift button is pressed
-            if(e->modifiers() & Qt::ShiftModifier)
-            {
-                if(m_scaleY>0) // handle the mirrored cases
+            if (e->modifiers() & Qt::ShiftModifier) {
+                if (m_scaleY > 0) // handle the mirrored cases
                     m_scaleY = fabs(m_scaleX);
                 else
                     m_scaleY = -fabs(m_scaleX);
             }
         }
 
-        if(m_function == BOTTOMRIGHTSCALE)
-        {
+        if (m_function == BOTTOMRIGHTSCALE) {
             dx = (newpos.x() - m_scaleX * (m_originalBottomRight.x() - m_originalCenter.x())) / 2;
             m_scaleX = (newpos.x() - dx) / (m_originalBottomRight.x() - m_originalCenter.x());
 
@@ -429,19 +415,15 @@ void KisToolTransform::mouseMoveEvent(KoPointerEvent *e)
             m_scaleY = (newpos.y() - dy) / (m_originalBottomRight.y() - m_originalCenter.y());
 
             // enforce same acpect if shift button is pressed
-            if(e->modifiers() & Qt::ShiftModifier)
-            {
-                if(m_scaleX < m_scaleY)
-                {
-                    if(m_scaleX>0) // handle the mirrored cases
+            if (e->modifiers() & Qt::ShiftModifier) {
+                if (m_scaleX < m_scaleY) {
+                    if (m_scaleX > 0) // handle the mirrored cases
                         m_scaleX = fabs(m_scaleY);
                     else
                         m_scaleX = -fabs(m_scaleY);
                     dx = (m_scaleX - oldScaleX) * (m_originalBottomRight.x() - m_originalCenter.x());
-                }
-                else
-                {
-                    if(m_scaleY>0) // handle the mirrored cases
+                } else {
+                    if (m_scaleY > 0) // handle the mirrored cases
                         m_scaleY = fabs(m_scaleX);
                     else
                         m_scaleY = -fabs(m_scaleX);
@@ -450,23 +432,20 @@ void KisToolTransform::mouseMoveEvent(KoPointerEvent *e)
             }
         }
 
-        if(m_function == BOTTOMSCALE)
-        {
+        if (m_function == BOTTOMSCALE) {
             dy = (newpos.y() - m_scaleY * (m_originalBottomRight.y() - m_originalCenter.y())) / 2;
             m_scaleY = (newpos.y() - dy) / (m_originalBottomRight.y() - m_originalCenter.y());
 
             // enforce same acpect if shift button is pressed
-            if(e->modifiers() & Qt::ShiftModifier)
-            {
-                if(m_scaleX>0) // handle the mirrored cases
+            if (e->modifiers() & Qt::ShiftModifier) {
+                if (m_scaleX > 0) // handle the mirrored cases
                     m_scaleX = fabs(m_scaleY);
                 else
                     m_scaleX = -fabs(m_scaleY);
             }
         }
 
-        if(m_function == BOTTOMLEFTSCALE)
-        {
+        if (m_function == BOTTOMLEFTSCALE) {
             dx = (newpos.x() - m_scaleX * (m_originalTopLeft.x() - m_originalCenter.x())) / 2;
             m_scaleX = (newpos.x() - dx) / (m_originalTopLeft.x() - m_originalCenter.x());
 
@@ -474,19 +453,15 @@ void KisToolTransform::mouseMoveEvent(KoPointerEvent *e)
             m_scaleY = (newpos.y() - dy) / (m_originalBottomRight.y() - m_originalCenter.y());
 
             // enforce same acpect if shift button is pressed
-            if(e->modifiers() & Qt::ShiftModifier)
-            {
-                if(m_scaleX < m_scaleY)
-                {
-                    if(m_scaleX>0) // handle the mirrored cases
+            if (e->modifiers() & Qt::ShiftModifier) {
+                if (m_scaleX < m_scaleY) {
+                    if (m_scaleX > 0) // handle the mirrored cases
                         m_scaleX = fabs(m_scaleY);
                     else
                         m_scaleX = -fabs(m_scaleY);
                     dx = (m_scaleX - oldScaleX) * (m_originalTopLeft.x() - m_originalCenter.x());
-                }
-                else
-                {
-                    if(m_scaleY>0) // handle the mirrored cases
+                } else {
+                    if (m_scaleY > 0) // handle the mirrored cases
                         m_scaleY = fabs(m_scaleX);
                     else
                         m_scaleY = -fabs(m_scaleX);
@@ -495,23 +470,20 @@ void KisToolTransform::mouseMoveEvent(KoPointerEvent *e)
             }
         }
 
-        if(m_function == LEFTSCALE)
-        {
+        if (m_function == LEFTSCALE) {
             dx = (newpos.x() - m_scaleX * (m_originalTopLeft.x() - m_originalCenter.x())) / 2;
             m_scaleX = (newpos.x() - dx) / (m_originalTopLeft.x() - m_originalCenter.x());
 
             // enforce same acpect if shift button is pressed
-            if(e->modifiers() & Qt::ShiftModifier)
-            {
-                if(m_scaleY>0) // handle the mirrored cases
+            if (e->modifiers() & Qt::ShiftModifier) {
+                if (m_scaleY > 0) // handle the mirrored cases
                     m_scaleY = fabs(m_scaleX);
                 else
                     m_scaleY = -fabs(m_scaleX);
             }
         }
 
-        if(m_function == TOPLEFTSCALE)
-        {
+        if (m_function == TOPLEFTSCALE) {
             dx = (newpos.x() - m_scaleX * (m_originalTopLeft.x() - m_originalCenter.x())) / 2;
             m_scaleX = (newpos.x() - dx) / (m_originalTopLeft.x() - m_originalCenter.x());
 
@@ -519,19 +491,15 @@ void KisToolTransform::mouseMoveEvent(KoPointerEvent *e)
             m_scaleY = (newpos.y() - dy) / (m_originalTopLeft.y() - m_originalCenter.y());
 
             // enforce same acpect if shift button is pressed
-            if(e->modifiers() & Qt::ShiftModifier)
-            {
-                if(m_scaleX < m_scaleY)
-                {
-                    if(m_scaleX>0) // handle the mirrored cases
+            if (e->modifiers() & Qt::ShiftModifier) {
+                if (m_scaleX < m_scaleY) {
+                    if (m_scaleX > 0) // handle the mirrored cases
                         m_scaleX = fabs(m_scaleY);
                     else
                         m_scaleX = -fabs(m_scaleY);
                     dx = (m_scaleX - oldScaleX) * (m_originalTopLeft.x() - m_originalCenter.x());
-                }
-                else
-                {
-                    if(m_scaleY>0) // handle the mirrored cases
+                } else {
+                    if (m_scaleY > 0) // handle the mirrored cases
                         m_scaleY = fabs(m_scaleX);
                     else
                         m_scaleY = -fabs(m_scaleX);
@@ -543,16 +511,14 @@ void KisToolTransform::mouseMoveEvent(KoPointerEvent *e)
         m_translate += rot(dx, dy);
 
         m_canvas->updateCanvas(QRect(m_originalTopLeft, m_originalBottomRight));
-    }
-    else
-    {
-        if(det(mousePos - topleft, topright - topleft)>0)
+    } else {
+        if (det(mousePos - topleft, topright - topleft) > 0)
             m_function = ROTATE;
-        else if(det(mousePos - topright, bottomright - topright)>0)
+        else if (det(mousePos - topright, bottomright - topright) > 0)
             m_function = ROTATE;
-        else if(det(mousePos - bottomright, bottomleft - bottomright)>0)
+        else if (det(mousePos - bottomright, bottomleft - bottomright) > 0)
             m_function = ROTATE;
-        else if(det(mousePos - bottomleft, topleft - bottomleft)>0)
+        else if (det(mousePos - bottomleft, topleft - bottomleft) > 0)
             m_function = ROTATE;
         else
             m_function = MOVE;
@@ -560,21 +526,21 @@ void KisToolTransform::mouseMoveEvent(KoPointerEvent *e)
         double handleradius = m_canvas->viewConverter()->viewToDocumentX(5);
         handleradius *= handleradius; // square it so it fits with distsq
 
-        if(distsq(mousePos, (m_topleft + m_topright)/2.0)<=handleradius)
+        if (distsq(mousePos, (m_topleft + m_topright) / 2.0) <= handleradius)
             m_function = TOPSCALE;
-        if(distsq(mousePos, m_topright)<=handleradius)
+        if (distsq(mousePos, m_topright) <= handleradius)
             m_function = TOPRIGHTSCALE;
-        if(distsq(mousePos, (m_topright + m_bottomright)/2.0)<=handleradius)
+        if (distsq(mousePos, (m_topright + m_bottomright) / 2.0) <= handleradius)
             m_function = RIGHTSCALE;
-        if(distsq(mousePos, m_bottomright)<=handleradius)
+        if (distsq(mousePos, m_bottomright) <= handleradius)
             m_function = BOTTOMRIGHTSCALE;
-        if(distsq(mousePos, (m_bottomleft + m_bottomright)/2.0)<=handleradius)
+        if (distsq(mousePos, (m_bottomleft + m_bottomright) / 2.0) <= handleradius)
             m_function = BOTTOMSCALE;
-        if(distsq(mousePos, m_bottomleft)<=handleradius)
+        if (distsq(mousePos, m_bottomleft) <= handleradius)
             m_function = BOTTOMLEFTSCALE;
-        if(distsq(mousePos, (m_topleft + m_bottomleft)/2.0)<=handleradius)
+        if (distsq(mousePos, (m_topleft + m_bottomleft) / 2.0) <= handleradius)
             m_function = LEFTSCALE;
-        if(distsq(mousePos, m_topleft)<=handleradius)
+        if (distsq(mousePos, m_topleft) <= handleradius)
             m_function = TOPLEFTSCALE;
 
         setFunctionalCursor();
@@ -585,7 +551,7 @@ void KisToolTransform::mouseReleaseEvent(KoPointerEvent */*e*/)
 {
     m_selecting = false;
 
-    if(m_actualyMoveWhileSelected) {
+    if (m_actualyMoveWhileSelected) {
         QApplication::setOverrideCursor(KisCursor::waitCursor());
         m_canvas->updateCanvas(QRect(m_originalTopLeft, m_originalBottomRight));
         transform();
@@ -595,26 +561,26 @@ void KisToolTransform::mouseReleaseEvent(KoPointerEvent */*e*/)
 
 void KisToolTransform::recalcOutline()
 {
-    double x,y;
+    double x, y;
 
     m_sina = sin(m_a);
     m_cosa = cos(m_a);
 
     x = (m_originalTopLeft.x() - m_originalCenter.x()) * m_scaleX;
     y = (m_originalTopLeft.y() - m_originalCenter.y()) * m_scaleY;
-    m_topleft = rot(x,y) + m_translate;
+    m_topleft = rot(x, y) + m_translate;
 
     x = (m_originalBottomRight.x() - m_originalCenter.x()) * m_scaleX;
     y = (m_originalTopLeft.y() - m_originalCenter.y()) * m_scaleY;
-    m_topright = rot(x,y) + m_translate;
+    m_topright = rot(x, y) + m_translate;
 
     x = (m_originalTopLeft.x() - m_originalCenter.x()) * m_scaleX;
     y = (m_originalBottomRight.y() - m_originalCenter.y()) * m_scaleY;
-    m_bottomleft = rot(x,y) + m_translate;
+    m_bottomleft = rot(x, y) + m_translate;
 
     x = (m_originalBottomRight.x() - m_originalCenter.x()) * m_scaleX;
     y = (m_originalBottomRight.y() - m_originalCenter.y()) * m_scaleY;
-    m_bottomright = rot(x,y) + m_translate;
+    m_bottomright = rot(x, y) + m_translate;
 }
 
 void KisToolTransform::paint(QPainter& gc, const KoViewConverter &converter)
@@ -625,23 +591,23 @@ void KisToolTransform::paint(QPainter& gc, const KoViewConverter &converter)
 
     recalcOutline();
     KisImageSP img = image();
-    QPointF topleft = converter.documentToView(QPointF(m_topleft.x() / img->xRes(), m_topleft.y()/img->yRes()));
-    QPointF topright = converter.documentToView(QPointF(m_topright.x() / img->xRes(), m_topright.y()/img->yRes()));
-    QPointF bottomleft = converter.documentToView(QPointF(m_bottomleft.x() / img->xRes(), m_bottomleft.y()/img->yRes()));
-    QPointF bottomright = converter.documentToView(QPointF(m_bottomright.x() / img->xRes(), m_bottomright.y()/img->yRes()));
+    QPointF topleft = converter.documentToView(QPointF(m_topleft.x() / img->xRes(), m_topleft.y() / img->yRes()));
+    QPointF topright = converter.documentToView(QPointF(m_topright.x() / img->xRes(), m_topright.y() / img->yRes()));
+    QPointF bottomleft = converter.documentToView(QPointF(m_bottomleft.x() / img->xRes(), m_bottomleft.y() / img->yRes()));
+    QPointF bottomright = converter.documentToView(QPointF(m_bottomright.x() / img->xRes(), m_bottomright.y() / img->yRes()));
 
     QRectF handleRect(-4, -4, 8, 8);
 
     gc.setPen(pen);
 
     gc.drawRect(handleRect.translated(topleft));
-    gc.drawRect(handleRect.translated((topleft+topright)/2.0));
+    gc.drawRect(handleRect.translated((topleft + topright) / 2.0));
     gc.drawRect(handleRect.translated(topright));
-    gc.drawRect(handleRect.translated((topright+bottomright)/2.0));
+    gc.drawRect(handleRect.translated((topright + bottomright) / 2.0));
     gc.drawRect(handleRect.translated(bottomright));
-    gc.drawRect(handleRect.translated((bottomleft+bottomright)/2.0));
+    gc.drawRect(handleRect.translated((bottomleft + bottomright) / 2.0));
     gc.drawRect(handleRect.translated(bottomleft));
-    gc.drawRect(handleRect.translated((topleft+bottomleft)/2.0));
+    gc.drawRect(handleRect.translated((topleft + bottomleft) / 2.0));
 
     gc.drawLine(topleft, topright);
     gc.drawLine(topright, bottomright);
@@ -653,43 +619,41 @@ void KisToolTransform::paint(QPainter& gc, const KoViewConverter &converter)
 
 void KisToolTransform::transform()
 {
-    if (!image() || !m_currentLayer->paintDevice())
+    if (!image() || !currentNode()->paintDevice())
         return;
 
     QPointF t = m_translate - rot(m_originalCenter.x() * m_scaleX, m_originalCenter.y() * m_scaleY);
     //KoUpdater *progress = m_subject->progressDisplay();
 
     // This mementoes the current state of the active device.
-    TransformCmd * transaction = new TransformCmd(this, m_currentLayer->paintDevice(), m_scaleX,
-                                                  m_scaleY, m_translate, m_a, m_origSelection, m_originalTopLeft, m_originalBottomRight);
+    TransformCmd * transaction = new TransformCmd(this, currentNode(), m_scaleX,
+            m_scaleY, m_translate, m_a, m_origSelection, m_originalTopLeft, m_originalBottomRight);
 
     // Copy the original state back.
     QRect rc = m_origDevice->extent();
     rc = rc.normalized();
-    m_currentLayer->paintDevice()->clear();
-    KisPainter gc(m_currentLayer->paintDevice());
+    currentNode()->paintDevice()->clear();
+    KisPainter gc(currentNode()->paintDevice());
     gc.bitBlt(rc.x(), rc.y(), COMPOSITE_COPY, m_origDevice, rc.x(), rc.y(), rc.width(), rc.height());
     gc.end();
 
     // Also restore the original selection.
-    if(m_origSelection)
-    {
+    if (m_origSelection) {
         QRect rc = m_origSelection->selectedRect();
         rc = rc.normalized();
         // XXX_SELECTION This used to create a new selection!
-        if ( m_currentLayer->selection() ) m_currentLayer->selection()->clear();
-        KisPainter sgc(KisPaintDeviceSP(m_currentLayer->selection().data()));
+        if (currentSelection()) currentSelection()->clear();
+        KisPainter sgc(KisPaintDeviceSP(currentSelection().data()));
         sgc.bitBlt(rc.x(), rc.y(), COMPOSITE_COPY, KisPaintDeviceSP(m_origSelection.data()), rc.x(), rc.y(), rc.width(), rc.height());
         sgc.end();
-    }
-    else
-        if(m_currentLayer->selection())
-            m_currentLayer->selection()->clear();
+    } else
+        if (currentSelection())
+            currentSelection()->clear();
 
     // Perform the transform. Since we copied the original state back, this doesn't degrade
     // after many tweaks. Since we started the transaction before the copy back, the memento
     // has the previous state.
-    KisTransformWorker worker(m_currentLayer->paintDevice(), m_scaleX, m_scaleY, 0, 0, m_a, int(t.x()), int(t.y()), 0/*progress*/, m_filter);
+    KisTransformWorker worker(currentNode()->paintDevice(), m_scaleX, m_scaleY, 0, 0, m_a, int(t.x()), int(t.y()), 0/*progress*/, m_filter);
     worker.run();
 
 // XXX_PROGRESS, XXX_LAYERS
@@ -701,7 +665,7 @@ void KisToolTransform::transform()
 //         return;
 //     }
 
-    m_currentLayer->paintDevice()->setDirty(rc); // XXX: This is not enough - should union with new extent
+    currentNode()->paintDevice()->setDirty(rc); // XXX: This is not enough - should union with new extent
 
     // Else add the command -- this will have the memento from the previous state,
     // and the transformed state from the original device we cached in our activated()
@@ -714,7 +678,7 @@ void KisToolTransform::transform()
     }
 }
 
-void KisToolTransform::notifyCommandAdded( QUndoCommand * command)
+void KisToolTransform::notifyCommandAdded(QUndoCommand * command)
 {
     TransformCmd * cmd = dynamic_cast<TransformCmd*>(command);
     if (cmd == 0) {
@@ -725,21 +689,19 @@ void KisToolTransform::notifyCommandAdded( QUndoCommand * command)
     }
 }
 
-void KisToolTransform::notifyCommandExecuted( QUndoCommand * command)
+void KisToolTransform::notifyCommandExecuted(QUndoCommand * command)
 {
     Q_UNUSED(command);
-    TransformCmd * cmd=0;
+    TransformCmd * cmd = 0;
 
-    if(image()->undoAdapter()->presentCommand())
+    if (image()->undoAdapter()->presentCommand())
         cmd = dynamic_cast<TransformCmd*>(image()->undoAdapter()->presentCommand());
 
     if (cmd == 0) {
         // The command now on the top of the stack isn't one of ours
         // We should treat this as if the tool has been just activated
         initHandles();
-    }
-    else
-    {
+    } else {
         // One of our commands is now on top
         // We should ask for tool args and orig selection
         cmd->transformArgs(m_scaleX, m_scaleY, m_translate, m_a);
@@ -757,23 +719,24 @@ QWidget* KisToolTransform::createOptionWidget()
 {
     m_optWidget = new WdgToolTransform(0);
     Q_CHECK_PTR(m_optWidget);
-
+    m_optWidget->setObjectName(toolId() + " option widget");
+    
     m_optWidget->cmbFilter->clear();
     m_optWidget->cmbFilter->setIDList(KisFilterStrategyRegistry::instance()->listKeys());
 
     m_optWidget->cmbFilter->setCurrent("Mitchell");
     connect(m_optWidget->cmbFilter, SIGNAL(activated(const KoID &)),
-        this, SLOT(slotSetFilter(const KoID &)));
+            this, SLOT(slotSetFilter(const KoID &)));
 
     KoID filterID = m_optWidget->cmbFilter->currentItem();
     m_filter = KisFilterStrategyRegistry::instance()->value(filterID.id());
 
-/*
-    connect(m_optWidget->intStartX, SIGNAL(valueChanged(int)), this, SLOT(setStartX(int)));
-    connect(m_optWidget->intStartY, SIGNAL(valueChanged(int)), this, SLOT(setStartY(int)));
-    connect(m_optWidget->intEndX, SIGNAL(valueChanged(int)), this, SLOT(setEndX(int)));
-    connect(m_optWidget->intEndY, SIGNAL(valueChanged(int)), this, SLOT(setEndY(int)));
-*/
+    /*
+        connect(m_optWidget->intStartX, SIGNAL(valueChanged(int)), this, SLOT(setStartX(int)));
+        connect(m_optWidget->intStartY, SIGNAL(valueChanged(int)), this, SLOT(setStartY(int)));
+        connect(m_optWidget->intEndX, SIGNAL(valueChanged(int)), this, SLOT(setEndX(int)));
+        connect(m_optWidget->intEndY, SIGNAL(valueChanged(int)), this, SLOT(setEndY(int)));
+    */
     m_optWidget->intStartX->hide();
     m_optWidget->intStartY->hide();
     m_optWidget->intEndX->hide();
@@ -790,25 +753,4 @@ QWidget* KisToolTransform::optionWidget()
     return m_optWidget;
 }
 
-KisImageSP KisToolTransform::image() const
-{
-    KoShape * shape = m_canvas->shapeManager()->selection()->firstSelectedShape();
-
-    if ( !shape ) return 0;
-
-    if ( shape->shapeId() == KIS_LAYER_CONTAINER_ID ) {
-        return static_cast<KisLayerContainerShape*>( shape )->groupLayer()->image();
-    } else if ( shape->shapeId() ==  KIS_LAYER_SHAPE_ID) {
-        return static_cast<KisLayerShape*>( shape )->layer()->image();
-    } else if ( shape->shapeId() == KIS_MASK_SHAPE_ID ) {
-        // XXX
-        return 0;
-    } else if ( shape->shapeId() == KIS_SHAPE_LAYER_ID ) {
-        return static_cast<KisShapeLayer*>( shape )->image();
-    } else {
-        // First selected shape is not a krita layer type shape
-        return 0;
-    }
-}
-
 #include "kis_tool_transform.moc"
