@@ -24,7 +24,7 @@
 
 #include <klocale.h>
 #include <kstandarddirs.h>
-#include <kdebug.h>
+#include <kis_debug.h>
 
 #include <kis_meta_data_store.h>
 #include <kis_meta_data_entry.h>
@@ -33,10 +33,6 @@
 
 #include "kis_entry_editor.h"
 
-#ifdef Q_CC_MSVC
-#include <iso646.h>
-#endif
-
 struct KisMetaDataEditor::Private {
     KisMetaData::Store* originalStore;
     KisMetaData::Store* store;
@@ -49,113 +45,102 @@ KisMetaDataEditor::KisMetaDataEditor(QWidget* parent, KisMetaData::Store* origin
 {
     d->originalStore = originalStore;
     d->store = new KisMetaData::Store(*originalStore);
-    
+
     QStringList files = KGlobal::dirs()->findAllResources("data", "kritaplugins/metadataeditor/*.rc");
 
-    foreach(QString file, files)
-    {
-        
+    foreach(const QString & file, files) {
+
         QFile xmlFile(file);
         xmlFile.open(QFile::ReadOnly);
         QString errMsg;
         int errLine, errCol;
-        
+
         QDomDocument document;
-        if(not document.setContent(&xmlFile, false, &errMsg, &errLine, &errCol))
-        {
-            kDebug(41006) <<"Error reading XML at line" << errLine <<" column" << errCol <<" :" << errMsg;
+        if (!document.setContent(&xmlFile, false, &errMsg, &errLine, &errCol)) {
+            dbgPlugins << "Error reading XML at line" << errLine << " column" << errCol << " :" << errMsg;
         }
         QDomElement rootElement = document.documentElement();
-        if(rootElement.tagName() != "MetaDataEditor")
-        {
-            kDebug(41006) <<"Invalid XML file";
+        if (rootElement.tagName() != "MetaDataEditor") {
+            dbgPlugins << "Invalid XML file";
         }
-        
+
         const QString uiFileName = rootElement.attribute("uiFile");
         const QString pageName = rootElement.attribute("name");
         const QString iconName = rootElement.attribute("icon");
-        if(uiFileName == "") continue;
-        
+        if (uiFileName == "") continue;
+
         // Read the ui file
         QUiLoader loader;
-        QFile uiFile(KStandardDirs::locate("data","kritaplugins/metadataeditor/" + uiFileName));
+        QFile uiFile(KStandardDirs::locate("data", "kritaplugins/metadataeditor/" + uiFileName));
         uiFile.open(QFile::ReadOnly);
         QWidget *widget = dynamic_cast<QWidget*>(loader.load(&uiFile, this));
-        if(widget ==0)
-        {
-            kDebug(41006) <<"Failed to load ui file" << uiFileName;
+        if (widget == 0) {
+            dbgPlugins << "Failed to load ui file" << uiFileName;
             continue;
         }
         uiFile.close();
-        
+
         QDomNodeList list = rootElement.childNodes();
         const int size = list.size();
-        for(int i = 0; i < size; ++i) {
+        for (int i = 0; i < size; ++i) {
             QDomElement elem = list.item(i).toElement();
-            if( elem.isNull() or elem.tagName() != "EntryEditor" ) continue;
+            if (elem.isNull() || elem.tagName() != "EntryEditor") continue;
             const QString editorName = elem.attribute("editorName");
             const QString schemaUri = elem.attribute("schemaUri");
             const QString entryName = elem.attribute("entryName");
-            const QString editorSignal = "2"+elem.attribute("editorSignal");
+            const QString editorSignal = '2' + elem.attribute("editorSignal");
             const QString propertyName = elem.attribute("propertyName");
             const QString structureField = elem.attribute("structureField");
-            
+
             QWidget* obj = widget->findChild<QWidget*>(editorName);
-            if(obj)
-            {
+            if (obj) {
                 const KisMetaData::Schema* schema = KisMetaData::SchemaRegistry::instance()->schemaFromUri(schemaUri);
-                if(schema)
-                {
-                    if( not d->store->hasEntry( schema, entryName))
-                    {
-                        kDebug(41006) <<" Store doesn't have yet entry :" << entryName <<" in" << schemaUri  <<" ==" << schema->generateQualifiedName(entryName);
+                if (schema) {
+                    if (!d->store->containsEntry(schema, entryName)) {
+                        dbgPlugins << " Store does not have yet entry :" << entryName << " in" << schemaUri  << " ==" << schema->generateQualifiedName(entryName);
                     }
                     KisMetaData::Value& value = d->store->getEntry(schema, entryName).value();
                     KisEntryEditor* ee = 0;
-                    if( value.type() == KisMetaData::Value::Structure and not structureField.isEmpty())
-                    {
+                    if (value.type() == KisMetaData::Value::Structure && !structureField.isEmpty()) {
                         QMap<QString, KisMetaData::Value>* structure = value.asStructure();
-                        ee = new KisEntryEditor( obj, &(*structure)[ structureField ], propertyName);
+                        ee = new KisEntryEditor(obj, &(*structure)[ structureField ], propertyName);
                     } else {
-                        ee = new KisEntryEditor( obj, &value, propertyName);
+                        ee = new KisEntryEditor(obj, &value, propertyName);
                     }
-                    connect( obj, editorSignal.toAscii(), ee, SLOT(valueEdited()) );
+                    connect(obj, editorSignal.toAscii(), ee, SLOT(valueEdited()));
                     QList<KisEntryEditor*> otherEditors = d->entryEditors.values(&value);
-                    foreach(KisEntryEditor* oe, otherEditors)
-                    {
+                    foreach(KisEntryEditor* oe, otherEditors) {
                         connect(ee, SIGNAL(valueHasBeenEdited()), oe, SLOT(valueChanged()));
                         connect(oe, SIGNAL(valueHasBeenEdited()), ee, SLOT(valueChanged()));
                     }
                     d->entryEditors.insert(&value, ee);
                 } else {
-                    kDebug(41006) <<"Unknown schema :" << schemaUri;
+                    dbgPlugins << "Unknown schema :" << schemaUri;
                 }
             } else {
-                kDebug(41006) <<"Unknown object :" << editorName;
+                dbgPlugins << "Unknown object :" << editorName;
             }
         }
         xmlFile.close();
 
-        KPageWidgetItem *page = new KPageWidgetItem( widget, pageName );
-        if(iconName != "")
-        {
-            page->setIcon( KIcon( iconName ) );
+        KPageWidgetItem *page = new KPageWidgetItem(widget, pageName);
+        if (!iconName.isEmpty()) {
+            page->setIcon(KIcon(iconName));
         }
-        addPage( page );
+        addPage(page);
     }
 }
 
 KisMetaDataEditor::~KisMetaDataEditor()
 {
-    foreach(KisEntryEditor* e, d->entryEditors)
-    {
+    foreach(KisEntryEditor* e, d->entryEditors) {
         delete e;
     }
     delete d->store;
     delete d;
 }
 
-void KisMetaDataEditor::accept ()
+void KisMetaDataEditor::accept()
 {
     KPageDialog::accept();
     d->originalStore->copyFrom(d->store);
