@@ -16,6 +16,7 @@
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
 */
+
 #include "KritaShape.h"
 
 #include <QPainter>
@@ -24,7 +25,10 @@
 #include <QCoreApplication>
 
 #include <klocale.h>
-#include <kdebug.h>
+#include <kis_debug.h>
+
+#include <KoShapeSavingContext.h>
+#include <KoShapeLoadingContext.h>
 
 #include <KoColorProfile.h>
 #include <KoColorSpaceRegistry.h>
@@ -34,22 +38,24 @@
 #include "kis_image.h"
 #include "kis_doc2.h"
 #include "kis_group_layer.h"
+#include "kis_paint_layer.h"
+#include "kis_paint_device.h"
 
 class KritaShape::Private
 {
 public:
     KUrl url;
-    KoColorProfile * displayProfile;
+    const KoColorProfile * displayProfile;
     KisDoc2 * doc;
 };
 
 KritaShape::KritaShape(const KUrl& url, const QString & profileName)
-    : m_d( new Private())
+        : m_d(new Private())
 {
     m_d->url = url;
     m_d->doc = 0;
-    if ( !url.isEmpty() ) {
-        importImage( url );
+    if (!url.isEmpty()) {
+        importImage(url);
     }
     m_d->displayProfile = KoColorSpaceRegistry::instance()->profileByName(profileName);
     setKeepAspectRatio(true);
@@ -61,7 +67,7 @@ KritaShape::~KritaShape()
     delete m_d;
 }
 
-void KritaShape::importImage(const KUrl & url )
+void KritaShape::importImage(const KUrl & url)
 {
     delete m_d->doc;
     m_d->doc = new KisDoc2(0, 0, false);
@@ -72,7 +78,7 @@ void KritaShape::importImage(const KUrl & url )
 void KritaShape::slotLoadingFinished()
 {
     m_mutex.lock();
-    if ( m_d && m_d->doc && m_d->doc->image() ) {
+    if (m_d && m_d->doc && m_d->doc->image()) {
         // XXX: Resize image to correct aspect ratio
         m_waiter.wakeAll();
         update();
@@ -81,47 +87,53 @@ void KritaShape::slotLoadingFinished()
 
 }
 
-void KritaShape::paint( QPainter& painter, const KoViewConverter& converter )
+void KritaShape::paint(QPainter& painter, const KoViewConverter& converter)
 {
-    if ( m_d && m_d->doc && m_d->doc->image() ) {
+    if (m_d && m_d->doc && m_d->doc->image()) {
         // XXX: Only convert the bit the painter needs for painting?
         //      Or should we keep a converted qimage in readiness,
         //      just as with KisCanvas2?
-        KisImageSP kimage= m_d->doc->image();
+        KisImageSP kimage = m_d->doc->image();
 
         QImage qimg = kimage->convertToQImage(0, 0, kimage->width(), kimage->height(),
                                               m_d->displayProfile); // XXX: How about exposure?
 
-        const QRectF paintRect = QRectF( QPointF( 0.0, 0.0 ), size() );
-        applyConversion( painter, converter );
+        const QRectF paintRect = QRectF(QPointF(0.0, 0.0), size());
+        applyConversion(painter, converter);
         painter.drawImage(paintRect.toRect(), qimg);
 
-    }
-    else if(m_d->doc == 0)
-        tryLoadFromImageData(dynamic_cast<KoImageData*> (KoShape::userData()));
+    } else if (m_d->doc == 0)
+        tryLoadFromImageData(dynamic_cast<KoImageData*>(KoShape::userData()));
 }
 
-void KritaShape::setDisplayProfile( const QString & profileName ) {
-    m_d->displayProfile = KoColorSpaceRegistry::instance()->profileByName( profileName );
+void KritaShape::setDisplayProfile(const QString & profileName)
+{
+    m_d->displayProfile = KoColorSpaceRegistry::instance()->profileByName(profileName);
     update();
 }
 
-void KritaShape::saveOdf( KoShapeSavingContext & context ) const {
+void KritaShape::saveOdf(KoShapeSavingContext & context) const
+{
     // TODO
+    Q_UNUSED(context);
 }
-bool KritaShape::loadOdf( const KoXmlElement & element, KoShapeLoadingContext &context ) {
+bool KritaShape::loadOdf(const KoXmlElement & element, KoShapeLoadingContext &context)
+{
+    Q_UNUSED(element);
+    Q_UNUSED(context);
     return false; // TODO
 }
 
-void KritaShape::waitUntilReady() const {
-    if ( m_d && m_d->doc && m_d->doc->image() ) // all done
+void KritaShape::waitUntilReady() const
+{
+    if (m_d && m_d->doc && m_d->doc->image())   // all done
         return;
 
-    KoImageData *data = dynamic_cast<KoImageData*> (KoShape::userData());
-    if(data == 0 || !data->imageLocation().isValid())
+    KoImageData *data = dynamic_cast<KoImageData*>(KoShape::userData());
+    if (data == 0 || data->image().isNull())
         return; // no data available at all, so don't try to wait later on.
 
-    KritaShape *me = const_cast<KritaShape*> (this);
+    KritaShape *me = const_cast<KritaShape*>(this);
 
     m_mutex.lock();
     me->tryLoadFromImageData(data);
@@ -129,15 +141,16 @@ void KritaShape::waitUntilReady() const {
     m_mutex.unlock();
 }
 
-void KritaShape::tryLoadFromImageData(KoImageData *data) {
+void KritaShape::tryLoadFromImageData(KoImageData *data)
+{
 
-    if(data == 0)
+    if (data == 0)
         return;
 
-    KUrl url = data->imageLocation();
+    // TODO maybe we want to use the image rawData for that
     QImage image = data->image();
 
-    if(url.isEmpty() && image.isNull())
+    if (image.isNull())
         return;
 
     delete m_d->doc;
@@ -145,27 +158,23 @@ void KritaShape::tryLoadFromImageData(KoImageData *data) {
     m_d->doc = new KisDoc2(0, 0, false);
     connect(m_d->doc, SIGNAL(sigLoadingFinished()), this, SLOT(slotLoadingFinished()));
 
-    if(! url.isEmpty())
-        m_d->doc->openUrl(url);
-    else {
-        // Create an empty image
-        KisImageSP img = m_d->doc->newImage(i18n( "Converted from KoImageData" ), image.width(), image.height(), 0 );
+    // Create an empty image
+    KisImageSP img = m_d->doc->newImage(i18n("Converted from KoImageData"), image.width(), image.height(), 0);
 
-        // Convert the QImage to a paint device
-        KisPaintLayer * layer = dynamic_cast<KisPaintLayer*>( img->root()->firstChild().data() );
-        if ( layer )
-            layer->paintDevice()->convertFromQImage( image, "", 0, 0 );
+    // Convert the QImage to a paint device
+    KisPaintLayer * layer = dynamic_cast<KisPaintLayer*>(img->root()->firstChild().data());
+    if (layer)
+        layer->paintDevice()->convertFromQImage(image, "", 0, 0);
 
-        // emits sigLoadingFinished
-        m_d->doc->setCurrentImage( img );
-    }
+    // emits sigLoadingFinished
+    m_d->doc->setCurrentImage(img);
 }
 
 QImage KritaShape::convertToQImage()
 {
-    if ( m_d->doc && m_d->doc->image() ) {
+    if (m_d->doc && m_d->doc->image()) {
         KisImageSP img = m_d->doc->image();
-        return img->convertToQImage( 0, 0, img->width(), img->height(), m_d->displayProfile );
+        return img->convertToQImage(0, 0, img->width(), img->height(), m_d->displayProfile);
     }
     return QImage();
 }
