@@ -22,7 +22,7 @@
 #ifndef KIS_DOC_2_H_
 #define KIS_DOC_2_H_
 
-#include <kdebug.h>
+#include <kis_debug.h>
 
 #include <KoDocument.h>
 
@@ -41,7 +41,6 @@ class KisView2;
 class KisChildDoc;
 class KisUndoAdapter;
 class KisNodeModel;
-
 /**
  * The class that represents a Krita document containing content and
    settings.
@@ -62,20 +61,27 @@ class KRITAUI_EXPORT KisDoc2 : public KoDocument
     Q_OBJECT
 
 public:
-    KisDoc2(QWidget *parentWidget = 0, QObject* parent = 0, bool singleViewMode = false );
+    KisDoc2(QWidget *parentWidget = 0, QObject* parent = 0, bool singleViewMode = false);
     virtual ~KisDoc2();
 
 public:
     // Overide KoDocument
-    virtual bool wantExportConfirmation() const { return false; }
+    virtual bool wantExportConfirmation() const {
+        return false;
+    }
     virtual bool completeLoading(KoStore *store);
     virtual bool completeSaving(KoStore*);
-    virtual bool loadOdf( KoOdfReadStore & odfStore );
-    virtual bool saveOasis( KoStore*, KoXmlWriter* );
-    virtual bool loadChildren( KoStore* store);
-    virtual bool loadXML(QIODevice *, const KoXmlDocument& doc);
+    virtual bool loadOdf(KoOdfReadStore & odfStore);
+    virtual bool saveOdf(SavingContext &documentContext);
+    virtual bool loadChildren(KoStore* store);
+    virtual bool loadXML(const KoXmlDocument& doc, KoStore* store);
     virtual QByteArray mimeType() const;
-    virtual QWidget* createCustomDocumentWidget(QWidget *parent);
+    virtual QList<KoDocument::CustomDocumentWidgetItem> createCustomDocumentWidgets(QWidget *parent);
+
+    virtual KParts::Part * hitTest(QWidget * w, const QPoint& pt) {
+        return KoDocument::hitTest(w, pt);
+    }
+
     virtual KoDocument* hitTest(const QPoint &pos, KoView* view, const QMatrix& matrix = QMatrix());
 
     /**
@@ -94,7 +100,7 @@ public slots:
      * Initialize an empty document using default values
      * @since 1.5
      */
-     virtual void initEmpty();
+    virtual void initEmpty();
 
 public:
 
@@ -110,14 +116,14 @@ public:
      */
     KisImageSP newImage(const QString& name, qint32 width, qint32 height, const KoColorSpace * colorspace);
 
-    KisImageSP image();
+    KisImageSP image() const;
 
     /**
      * Adds the specified child document to this document; this
      * is not done with KoDocument::insertChild() because that
      * is protected and cannot be called from KisView2.
      */
-    KisChildDoc * createChildDoc( const QRect& rect, KoDocument* childDoc );
+    KisChildDoc * createChildDoc(const QRect& rect, KoDocument* childDoc);
 
     /**
      * Makes an otherwise empty document ready for import/export
@@ -129,26 +135,26 @@ public:
      */
     void setCurrentImage(KisImageSP image);
 
-    KisUndoAdapter * undoAdapter();
+    KisUndoAdapter * undoAdapter() const;
 
     /**
      * The shape controller matches internal krita image layers with
      * the flake shape hierarchy.
      */
-    KoShapeControllerBase * shapeController();
+    KoShapeControllerBase * shapeController() const;
 
-    KoShape * shapeForNode( KisNodeSP layer );
+    KoShape * shapeForNode(KisNodeSP layer) const;
 
     /**
      * Add a node to the shape controller
      */
-    KoShape * addShape( const KisNodeSP node);
+    KoShape * addShape(const KisNodeSP node);
 
     /**
      * The layer model provides a light-weight Qt model-view
      * compatible model on the internal Krita image layer hierarchy.
      */
-    KisNodeModel * nodeModel();
+    KisNodeModel * nodeModel() const;
 
 public slots:
     void slotImageUpdated();
@@ -182,14 +188,6 @@ private slots:
 
 private:
 
-    QDomElement saveImage(QDomDocument& doc, KisImageSP img);
-    KisImageSP loadImage(const KoXmlElement& elem);
-    void loadLayers(const KoXmlElement& element, KisImageSP img, KisGroupLayerSP parent);
-    KisLayerSP loadLayer(const KoXmlElement& elem, KisImageSP img);
-    KisLayerSP loadPaintLayer(const KoXmlElement& elem, KisImageSP img, const QString & name, qint32 x, qint32 y, qint32 opacity, bool visible, bool locked, const QString & compositeOp);
-    KisGroupLayerSP loadGroupLayer(const KoXmlElement& elem, KisImageSP img, const QString & name, qint32 x, qint32 y, qint32 opacity, bool visible, bool locked, const QString &compositeOp);
-    KisAdjustmentLayerSP loadAdjustmentLayer(const KoXmlElement& elem, KisImageSP img, const QString & name, qint32 x, qint32 y, qint32 opacity, bool visible, bool locked, const QString & compositeOp);
-    KisShapeLayerSP loadShapeLayer(const KoXmlElement& elem, KisImageSP img, const QString & name, qint32 x, qint32 y, qint32 opacity, bool visible, bool locked, const QString &compositeOp);
     bool init();
 
     void setIOSteps(qint32 nsteps);
@@ -204,4 +202,3 @@ private:
 };
 
 #endif // KIS_DOC_H_
-
