@@ -47,7 +47,8 @@ class KoChannelInfo;
  * some functionality will disappear, like listProducers(). Setting a histogram will discard
  * info on the layer, and setting a layer will discard info on the histogram.
  **/
-class KRITAUI_EXPORT KisHistogramView : public QLabel {
+class KRITAUI_EXPORT KisHistogramView : public QLabel
+{
     Q_OBJECT
 public:
     KisHistogramView(QWidget *parent = 0, const char *name = 0, Qt::WFlags f = 0);
@@ -61,9 +62,9 @@ public:
     /** Lists all producers currently available */
     QList<KoID> listProducers();
     /** Sets the currently displayed channels to channels of the producer with producerID as ID*/
-    void setCurrentChannels(const KoID& producerID, Q3ValueVector<KoChannelInfo *> channels);
+    void setCurrentChannels(const KoID& producerID, QList<KoChannelInfo *> channels);
     /** Be careful, producer will be modified */
-    void setCurrentChannels(KoHistogramProducerSP producer, Q3ValueVector<KoChannelInfo *> channels);
+    void setCurrentChannels(KoHistogramProducerSP producer, QList<KoChannelInfo *> channels);
     bool hasColor();
     void setColor(bool set);
 
