@@ -15,15 +15,16 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
+
 #include "kis_image_manager.h"
 
+
 #include <QString>
 
 #include <kstandardaction.h>
 #include <klocale.h>
 #include <kfiledialog.h>
 #include <kurl.h>
-#include <kstandardaction.h>
 #include <ktoggleaction.h>
 #include <kactioncollection.h>
 
@@ -34,28 +35,30 @@
 #include <kis_filter_strategy.h>
 #include <kis_shear_visitor.h>
 #include <kis_transform_worker.h>
+#include <kis_paint_device.h>
 
 #include "kis_layer_manager.h"
 #include "kis_statusbar.h"
 #include "kis_import_catcher.h"
 #include "kis_view2.h"
 #include "kis_doc2.h"
-#include "kis_dlg_image_properties.h"
-#include "kis_image_commands.h"
+#include "dialogs/kis_dlg_image_properties.h"
+#include "commands/kis_image_commands.h"
 
-KisImageManager::KisImageManager( KisView2 * view)
-    : m_view( view )
+
+KisImageManager::KisImageManager(KisView2 * view)
+        : m_view(view)
 {
 }
 
-void KisImageManager::setup( KActionCollection * actionCollection )
+void KisImageManager::setup(KActionCollection * actionCollection)
 {
     KAction *action  = new KAction(i18n("I&nsert Image as Layer..."), this);
-    actionCollection->addAction("insert_image_as_layer", action );
+    actionCollection->addAction("insert_image_as_layer", action);
     connect(action, SIGNAL(triggered()), this, SLOT(slotInsertImageAsLayer()));
 
     action  = new KAction(i18n("Properties..."), this);
-    actionCollection->addAction("img_properties", action );
+    actionCollection->addAction("img_properties", action);
     connect(action, SIGNAL(triggered()), this, SLOT(slotImageProperties()));
 }
 
@@ -86,7 +89,7 @@ qint32 KisImageManager::importImage(const KUrl& urlArg)
 
     if (urlArg.isEmpty()) {
         QString mimelist = KoFilterManager::mimeFilter("application/x-krita", KoFilterManager::Import).join(" ");
-        urls = KFileDialog::getOpenUrls(KUrl(QString::null), mimelist, 0, i18n("Import Image"));
+        urls = KFileDialog::getOpenUrls(KUrl(QString()), mimelist, 0, i18n("Import Image"));
     } else {
         urls.push_back(urlArg);
     }
@@ -95,7 +98,7 @@ qint32 KisImageManager::importImage(const KUrl& urlArg)
         return 0;
 
     for (KUrl::List::iterator it = urls.begin(); it != urls.end(); ++it) {
-        new KisImportCatcher( *it, m_view );
+        new KisImportCatcher(*it, m_view);
     }
 
     m_view->canvas()->update();
@@ -115,8 +118,12 @@ void KisImageManager::resizeCurrentImage(qint32 w, qint32 h, bool cropLayers)
 void KisImageManager::scaleCurrentImage(double sx, double sy, KisFilterStrategy *filterStrategy)
 {
     if (!m_view->image()) return;
-    // XXX_PROGRESS
-    m_view->image()->scale(sx, sy, 0, filterStrategy);
+
+    KoProgressUpdater updater(m_view->statusBar()->progress());
+    updater.start( 100, "Scale Image" );
+    KoUpdater up = updater.startSubtask();
+
+    m_view->image()->scale(sx, sy, &up, filterStrategy);
     m_view->image()->setModified();
     m_view->layerManager()->layersUpdated();
 }
@@ -124,8 +131,12 @@ void KisImageManager::scaleCurrentImage(double sx, double sy, KisFilterStrategy
 void KisImageManager::rotateCurrentImage(double radians)
 {
     if (!m_view->image()) return;
-    // XXX_PROGRESS
-    m_view->image()->rotate(radians, 0);
+
+    KoProgressUpdater updater(m_view->statusBar()->progress());
+    updater.start( 100, "Rotate Image" );
+    KoUpdater up = updater.startSubtask();
+
+    m_view->image()->rotate(radians, &up );
     m_view->image()->setModified();
     m_view->layerManager()->layersUpdated();
 }
@@ -133,8 +144,12 @@ void KisImageManager::rotateCurrentImage(double radians)
 void KisImageManager::shearCurrentImage(double angleX, double angleY)
 {
     if (!m_view->image()) return;
-    // XXX_PROGRESS
-    m_view->image()->shear(angleX, angleY, 0);
+
+    KoProgressUpdater updater(m_view->statusBar()->progress());
+    updater.start( 100, "Shear Image" );
+    KoUpdater up = updater.startSubtask();
+
+    m_view->image()->shear(angleX, angleY, &up );
     m_view->image()->setModified();
     m_view->layerManager()->layersUpdated();
 }
