@@ -44,7 +44,8 @@ class KisFilterConfiguration;
  * adding, removing, editing. It also keeps track of the active layer
  * for this view.
  */
-class KRITAUI_EXPORT KisLayerManager : public QObject {
+class KRITAUI_EXPORT KisLayerManager : public QObject
+{
 
     Q_OBJECT
 
@@ -65,8 +66,7 @@ signals:
 
     /// XXX: Move this to kisview or to kisresourceprovider? (BSAR)
     void currentColorSpaceChanged(const KoColorSpace * cs);
-    void sigActiveSelectionChanged(KisImageSP image);
-    void sigLayerActivated( KisLayerSP layer );
+    void sigLayerActivated(KisLayerSP layer);
 
 public slots:
 
@@ -92,8 +92,8 @@ public slots:
     void rotateLayerRight90();
     void mirrorLayerX();
     void mirrorLayerY();
-    void scaleLayer( double sx, double sy, KisFilterStrategy *filterStrategy );
-    void rotateLayer( double radians );
+    void scaleLayer(double sx, double sy, KisFilterStrategy *filterStrategy);
+    void rotateLayer(double radians);
     void shearLayer(double angleX, double angleY)        ;
     void flattenImage();
     void mergeLayer();
@@ -108,20 +108,24 @@ public slots:
     void addGroupLayer(KisNodeSP parent, KisNodeSP above);
 
     void addCloneLayer();
-    void addCloneLayer( KisNodeSP parent, KisNodeSP above );
+    void addCloneLayer(KisNodeSP parent, KisNodeSP above);
 
     void addShapeLayer();
-    void addShapeLayer( KisNodeSP parent, KisNodeSP above );
+    void addShapeLayer(KisNodeSP parent, KisNodeSP above);
 
     void addAdjustmentLayer();
     void addAdjustmentLayer(KisNodeSP parent, KisNodeSP above);
-    void addAdjustmentLayer(KisNodeSP parent, KisNodeSP above, const QString & name, KisFilterConfiguration * filter, KisSelectionSP selection);
+    KisAdjustmentLayerSP addAdjustmentLayer(KisNodeSP parent, KisNodeSP above, const QString & name, KisFilterConfiguration * filter, KisSelectionSP selection);
+
+    void addGeneratorLayer();
+    void addGeneratorLayer(KisNodeSP parent, KisNodeSP above);
+    void addGeneratorLayer(KisNodeSP parent, KisNodeSP above, const QString & name, KisFilterConfiguration * filter, KisSelectionSP selection);
 
 private:
 
     friend class KisNodeManager;
 
-    void activateLayer( KisLayerSP layer );
+    void activateLayer(KisLayerSP layer);
 
 
 private:
@@ -131,23 +135,24 @@ private:
 
     QList<QAction*> m_pluginActions;
 
-    QAction *m_imgFlatten;
-    QAction *m_imgMergeLayer;
-    QAction * m_actionAdjustmentLayer;
-    QAction *m_layerAdd;
-    QAction *m_layerAddCloneLayer;
-    QAction *m_layerAddShapeLayer;
-    QAction *m_layerBottom;
-    QAction *m_layerDup;
+    KAction *m_imgFlatten;
+    KAction *m_imgMergeLayer;
+    KAction *m_actionAdjustmentLayer;
+    KAction *m_actionGeneratorLayer;
+    KAction *m_layerAdd;
+    KAction *m_layerAddCloneLayer;
+    KAction *m_layerAddShapeLayer;
+    KAction *m_layerBottom;
+    KAction *m_layerDup;
     KToggleAction *m_layerHide;
-    QAction *m_layerLower;
-    QAction *m_layerProperties;
-    QAction *m_layerRaise;
-    QAction *m_layerRm;
-    QAction *m_layerSaveAs;
-    QAction *m_layerTop;
+    KAction *m_layerLower;
+    KAction *m_layerProperties;
+    KAction *m_layerRaise;
+    KAction *m_layerRm;
+    KAction *m_layerSaveAs;
+    KAction *m_layerTop;
     bool m_actLayerVis;
-    QAction *m_imgResizeToLayer;
+    KAction *m_imgResizeToLayer;
 
     KisLayerSP m_activeLayer;
 };
