@@ -32,14 +32,15 @@ class KToggleAction;
 /**
  * Handle the gui for manipulating masks.
  */
-class KisMaskManager : public QObject {
+class KisMaskManager : public QObject
+{
 
     Q_OBJECT
 
 public:
 
 
-    KisMaskManager(KisView2 * view );
+    KisMaskManager(KisView2 * view);
     ~KisMaskManager() {}
 
     void setup(KActionCollection * actionCollection);
@@ -58,7 +59,7 @@ public:
 
 signals:
 
-    void sigMaskActivated( KisMaskSP mask );
+    void sigMaskActivated(KisMaskSP mask);
 
 public slots:
 
@@ -66,33 +67,25 @@ public slots:
      * Create a new transparency mask.
      */
     void createTransparencyMask();
-    void createTransparencyMask( KisNodeSP parent, KisNodeSP above );
+    void createTransparencyMask(KisNodeSP parent, KisNodeSP above);
     /**
      * Create a new filter mask.
      */
     void createFilterMask();
-    void createFilterMask( KisNodeSP parent, KisNodeSP above );
+    void createFilterMask(KisNodeSP parent, KisNodeSP above);
     /**
      * create a new transformation mask. If the transform tool is
      * active, get the current transformation and selection and create
      * the mask from that.
      */
     void createTransformationMask();
-    void createTransformationMask( KisNodeSP parent, KisNodeSP above );
+    void createTransformationMask(KisNodeSP parent, KisNodeSP above);
 
     /**
      * Create a local, i.e., per-layer selection object.
      */
-    void createSelectionmask()
-        {
-            // XXX: implement when selection is no longer part of KisPaintDevice
-        }
-    void createSelectionMask( KisNodeSP parent, KisNodeSP above )
-        {
-            Q_UNUSED( parent );
-            Q_UNUSED( above );
-            // XXX: implement when selection is no longer part of KisPaintDevice
-        }
+    void createSelectionmask();
+    void createSelectionMask(KisNodeSP parent, KisNodeSP above);
 
     /**
      * Create a new local selection from the active mask.
@@ -111,21 +104,12 @@ public slots:
      */
     void duplicateMask();
 
-
     /**
      * Show the mask as an overlay. The mask properties determine the
      * color and opacity of the mask.
      */
     void showMask();
 
-
-    /**
-     * Enable the mask effect. Similar to hiding a layer. The mask
-     * effect can be enabled and disabled independently from whether
-     * the mask visualisation is shown.
-     */
-    void enableMask();
-
     /**
      * Delete the mask
      */
@@ -185,7 +169,7 @@ private:
      * Activate a new mask. There can be only one mask active per
      * view; and if the mask is active, it becomes the paint device.
      */
-    void activateMask( KisMaskSP mask );
+    void activateMask(KisMaskSP mask);
 
 
 private:
@@ -196,12 +180,12 @@ private:
     KAction *m_createTransparencyMask;
     KAction *m_createFilterMask;
     KAction *m_createTransformationMask;
+    KAction *m_createSelectionMask;
     KAction *m_maskToSelection;
     KAction *m_maskToLayer;
     KAction *m_duplicateMask;
 
     KToggleAction *m_showMask;
-    KToggleAction *m_enableMask;
     KAction *m_removeMask;
 
     KAction *m_raiseMask;
