@@ -21,12 +21,16 @@
 #include <KoDocumentSectionModel.h>
 #include "krita_export.h"
 #include <kis_types.h>
+
 /**
  * KisNodeModel offers a Qt model-view compatible view on the node
  * hierarchy.
  *
  * Note that there's a discrepancy between the krita node tree model
  * and the model Qt wants to see: we hide the root node from Qt.
+ *
+ * The node model shows an inverse view on the layer tree: we want the
+ * first layer to show up at the bottom.
  */
 class KRITAUI_EXPORT KisNodeModel : public KoDocumentSectionModel
 {
@@ -35,10 +39,10 @@ class KRITAUI_EXPORT KisNodeModel : public KoDocumentSectionModel
 
 public: // from QAbstractItemModel
 
-    KisNodeModel(QObject * parent );
+    KisNodeModel(QObject * parent);
     ~KisNodeModel();
 
-    void setImage( KisImageSP image );
+    void setImage(KisImageSP image);
 
     KisNodeSP nodeFromIndex(const QModelIndex &index);
     vKisNodeSP nodesFromIndexes(const QModelIndexList &list);
@@ -51,21 +55,28 @@ public: // from QAbstractItemModel
     virtual QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
     virtual Qt::ItemFlags flags(const QModelIndex &index) const;
     virtual bool setData(const QModelIndex &index, const QVariant &value, int role = Qt::EditRole);
-//     QMimeData * mimeData ( const QModelIndexList & indexes ) const;
-    virtual bool dropMimeData ( const QMimeData * data, Qt::DropAction action, int row, int column, const QModelIndex & parent );
-    virtual Qt::DropActions supportedDragActions () const;
+    virtual QStringList mimeTypes() const;
+    QMimeData * mimeData(const QModelIndexList & indexes) const;
+    virtual bool dropMimeData(const QMimeData * data, Qt::DropAction action, int row, int column, const QModelIndex & parent);
+    virtual Qt::DropActions supportedDragActions() const;
+    virtual Qt::DropActions supportedDropActions() const;
 
 signals:
 
-    void nodeActivated( KisNodeSP );
+    void nodeActivated(KisNodeSP);
+    void requestAddNode(KisNodeSP node, KisNodeSP activeNode);
+    void requestAddNode(KisNodeSP node, KisNodeSP parent, int index);
+    void requestMoveNode(KisNodeSP node, KisNodeSP activeNode);
+    void requestMoveNode(KisNodeSP node, KisNodeSP parent, int index);
 
-public slots:
-
-    void beginInsertNodes( KisNode * parent, int index );
-    void endInsertNodes( KisNode * parent, int index );
-    void beginRemoveNodes( KisNode * parent, int index );
-    void endRemoveNodes( KisNode * parent, int index );
+private slots:
 
+    void beginInsertNodes(KisNode * parent, int index);
+    void endInsertNodes(KisNode * parent, int index);
+    void beginRemoveNodes(KisNode * parent, int index);
+    void endRemoveNodes(KisNode * parent, int index);
+    void updateSettings();
+    void progressPercentageChanged(int, const KisNodeSP);
 private:
 
     class Private;
