@@ -20,8 +20,8 @@
 #define KIS_STATUSBAR_H
 
 #include <QObject>
-#include <QProgressBar>
 
+#include <KoProgressBar.h>
 #include <kis_types.h>
 
 class QLabel;
@@ -36,33 +36,33 @@ class KisStatusBar : public QObject
 
 public:
 
-    KisStatusBar(KStatusBar * statusBar, KisView2 * view );
+    KisStatusBar(KStatusBar * statusBar, KisView2 * view);
     ~KisStatusBar();
 
 public slots:
 
-    void setZoom( int percentage );
-    void documentMousePositionChanged( const QPointF &p );
-    void imageSizeChanged( qint32 w, qint32 h );
-    void setSelection( KisImageSP img );
-    void setProfile( KisImageSP img );
-    void setHelp( const QString &t );
+    void setZoom(int percentage);
+    void documentMousePositionChanged(const QPointF &p);
+    void imageSizeChanged(qint32 w, qint32 h);
+    void setSelection(KisImageSP img);
+    void setProfile(KisImageSP img);
+    void setHelp(const QString &t);
     void updateStatusBarProfileLabel();
-    QProgressBar * progress()
-        {
-            return m_progress;
-        }
+    KoProgressBar * progress() {
+        return m_progress;
+    }
 private:
 
     KisView2 * m_view;
-    QProgressBar * m_progress;
+    KoProgressBar * m_progress;
     KStatusBar * m_statusbar;
 
+    QLabel *m_selectionStatusLabel;
     QLabel *m_statusBarZoomLabel; // Make interactive line edit
     QLabel *m_statusBarPositionLabel;
     QLabel *m_sizeLabel;
 
-    KSqueezedTextLabel *m_statusBarSelectionLabel;
+    KSqueezedTextLabel *m_statusBarStatusLabel;
     KSqueezedTextLabel *m_statusBarProfileLabel;
     KSqueezedTextLabel *m_statusBarHelpLabel;
 
