@@ -22,7 +22,6 @@
 #include <QDockWidget>
 
 #include <KoView.h>
-#include <KoToolBox.h>
 #include <KoToolManager.h>
 #include <krita_export.h>
 #include <kis_types.h>
@@ -37,7 +36,7 @@ class KisDoc2;
 class KisFilterManager;
 class KisImage;
 class KisLayerManager;
-class KisResourceProvider;
+class KisCanvasResourceProvider;
 class KisSelectionManager;
 class KisStatusBar;
 class KisUndoAdapter;
@@ -45,6 +44,9 @@ class KisZoomManager;
 class KisImageManager;
 class KisNodeManager;
 class KisMaskManager;
+class KisPerspectiveGridManager;
+class KisGridManager;
+class KisLayerBox;
 
 /**
  * Krita view class
@@ -52,9 +54,10 @@ class KisMaskManager;
  * Following the broad model-view-controller idea this class shows you one view on the document.
  * There can be multiple views of the same document each in with independent settings for viewMode and zoom etc.
  */
-class KRITAUI_EXPORT KisView2 : public KoView {
+class KRITAUI_EXPORT KisView2 : public KoView
+{
 
-Q_OBJECT
+    Q_OBJECT
 
 public:
     /**
@@ -68,16 +71,13 @@ public:
 public:
 
     // QWidget overrides
-    virtual void dragEnterEvent ( QDragEnterEvent * event );
-    virtual void dropEvent ( QDropEvent * event );
+    virtual void dragEnterEvent(QDragEnterEvent * event);
+    virtual void dropEvent(QDropEvent * event);
 
     // KoView implementation
-    virtual void updateReadWrite( bool readwrite ) { Q_UNUSED(readwrite); }
-    void slotChildActivated(bool a);
-    void canvasAddChild( KoViewChild * child );
-
-    virtual void print(QPrinter &printer, QPrintDialog &printDialog);
-    virtual void setupPrinter(QPrinter &printer, QPrintDialog &printDialog);
+    virtual void updateReadWrite(bool readwrite) {
+        Q_UNUSED(readwrite);
+    }
 
 public:  // Krita specific interfaces
 
@@ -86,7 +86,7 @@ public:  // Krita specific interfaces
 
     /// The resource provider contains all per-view settings, such as
     /// current color, current paint op etc.
-    KisResourceProvider * resourceProvider();
+    KisCanvasResourceProvider * resourceProvider();
 
     /// Return the canvasbase class
     KisCanvas2 * canvasBase() const;
@@ -160,14 +160,21 @@ public:  // Krita specific interfaces
     /// another image) from the slots in the various managers
     void disconnectCurrentImage();
 
+    virtual KoPrintJob * createPrintJob();
+
+    KisGridManager * gridManager();
+    KisPerspectiveGridManager* perspectiveGridManager();
+
+    /// @return the layer box for this view
+    KisLayerBox* layerBox();
 private slots:
 
     void slotLoadingFinished();
     void slotUpdateFullScreen(bool);
     void slotPreferences();
     void slotEditPalette();
-    void slotSetImageSize( qint32 w, qint32 h );
-
+    void slotSetImageSize(qint32 w, qint32 h);
+    void slotTotalRefresh();
 private:
 
 
