@@ -46,25 +46,22 @@
 #include <float.h>
 #include <math.h>
 
-#include <QApplication>
-#include <QPainter>
-#include <QPolygon>
-#include <QRegExp>
-
+#include "CalculationSettings.h"
 #include "CellStorage.h"
 #include "Condition.h"
-#include "Doc.h"
+#include "Formula.h"
 #include "GenValidationStyle.h"
 #include "Global.h"
 #include "Localization.h"
 #include "LoadingInfo.h"
 #include "Map.h"
 #include "NamedAreaManager.h"
-#include "Object.h"
+#include "OdfLoadingContext.h"
+#include "OdfSavingContext.h"
 #include "RowColumnFormat.h"
 #include "Selection.h"
+#include "ShapeApplicationData.h"
 #include "Sheet.h"
-#include "SheetPrint.h"
 #include "Style.h"
 #include "StyleManager.h"
 #include "Util.h"
@@ -74,15 +71,19 @@
 #include "ValueFormatter.h"
 #include "ValueParser.h"
 
+#include <KoShape.h>
+#include <KoShapeLoadingContext.h>
+#include <KoShapeRegistry.h>
 #include <KoStyleStack.h>
 #include <KoXmlNS.h>
-#include <KoDom.h>
-#include <KoOasisStyles.h>
+#include <KoXmlReader.h>
+#include <KoOdfStylesReader.h>
 #include <KoXmlWriter.h>
 
-#include <kmessagebox.h>
-
 #include <kdebug.h>
+#include <KNotification>
+
+#include <QTimer>
 
 using namespace KSpread;
 
@@ -140,15 +141,9 @@ Sheet* Cell::sheet() const
     return d->sheet;
 }
 
-// Return the sheet that this cell belongs to.
-Doc* Cell::doc() const
-{
-    return sheet()->doc();
-}
-
 KLocale* Cell::locale() const
 {
-    return doc()->locale();
+    return sheet()->map()->calculationSettings()->locale();
 }
 
 // Return true if this is the default cell.
@@ -159,7 +154,7 @@ bool Cell::isDefault() const
         return false;
     if ( formula() != Formula() )
         return false;
-    if ( link() != QString() )
+    if (!link().isEmpty())
         return false;
     if ( doesMergeCells() == true )
         return false;
@@ -394,12 +389,10 @@ QString Cell::displayText() const
     const Style style = effectiveStyle();
     // Display a formula if warranted.  If not, display the value instead;
     // this is the most common case.
-    if ( isFormula() && sheet()->getShowFormula()
-           && !( sheet()->isProtected() && style.hideFormula() ) || isEmpty() )
+    if (isFormula() && sheet()->getShowFormula() && !( sheet()->isProtected() && style.hideFormula())) {
         string = userInput();
-    else
-    {
-        string = doc()->formatter()->formatText(value(), style.formatType(), style.precision(),
+    } else if (!isEmpty()) {
+        string = sheet()->map()->formatter()->formatText(value(), style.formatType(), style.precision(),
                                                 style.floatFormat(), style.prefix(),
                                                 style.postfix(), style.currency().symbol()).asString();
     }
@@ -528,12 +521,12 @@ QString Cell::encodeFormula(bool fixedReferences) const
             case Token::Cell:
             case Token::Range:
             {
-                if (doc()->namedAreaManager()->contains(token.text()))
+                if (sheet()->map()->namedAreaManager()->contains(token.text()))
                 {
                     result.append(token.text()); // simply keep the area name
                     break;
                 }
-                const Region region(token.text(), doc()->map());
+                const Region region(token.text(), sheet()->map());
                 // Actually, a contiguous region, but the fixation is needed
                 Region::ConstIterator end = region.constEnd();
                 for (Region::ConstIterator it = region.constBegin(); it != end; ++it)
@@ -707,10 +700,11 @@ bool Cell::makeFormula()
     // parse the formula and check for errors
     if ( !formula().isValid() )
     {
-        if (doc()->showMessageError())
-        {
-            KMessageBox::error( 0, i18n("Error in cell %1\n\n", fullName()) );
-        }
+        KNotification *notify = new KNotification("ParsingError");
+        notify->setText(i18n("Parsing of formula in cell <i>%1</i> failed.", fullName()));
+        notify->addContext("cell", fullName());
+        notify->addContext("expression", userInput());
+        QTimer::singleShot(0, notify, SLOT(sendEvent()));
         setValue( Value::errorPARSE() );
         return false;
     }
@@ -766,6 +760,7 @@ void Cell::parseUserInput( const QString& text )
     if ( text.isEmpty() )
     {
         setValue( Value::empty() );
+        setUserInput( text );
         setFormula( Formula() );
         return;
     }
@@ -780,10 +775,11 @@ void Cell::parseUserInput( const QString& text )
         // parse the formula and check for errors
         if ( !formula.isValid() )
         {
-            if (doc()->showMessageError())
-            {
-                KMessageBox::error( 0, i18n("Error in cell %1\n\n", fullName()) );
-            }
+            KNotification *notify = new KNotification("ParsingError");
+            notify->setText(i18n("Parsing of formula in cell <i>%1</i> failed.", fullName()));
+            notify->addContext("cell", fullName());
+            notify->addContext("expression", userInput());
+            QTimer::singleShot(0, notify, SLOT(sendEvent()));
             setValue( Value::errorPARSE() );
             return;
         }
@@ -804,13 +800,13 @@ void Cell::parseUserInput( const QString& text )
     else
     {
         // Parses the text and return the appropriate value.
-        value = doc()->parser()->parse( text );
+        value = sheet()->map()->parser()->parse( text );
 
 #if 0
         // Parsing as time acts like an autoformat: we even change the input text
         // [h]:mm:ss -> might get set by ValueParser
         if ( isTime() && ( formatType() != Format::Time7 ) )
-            setUserInput( locale()->formatTime( value().asDateTime( doc() ).time(), true ) );
+            setUserInput( locale()->formatTime( value().asDateTime( sheet()->map()->calculationSettings()).time(), true ) );
 #endif
 
         // convert first letter to uppercase ?
@@ -926,7 +922,7 @@ QDomElement Cell::save(QDomDocument& doc, int xOffset, int yOffset, bool era)
     // Save the formatting information
     //
     QDomElement formatElement( doc.createElement( "format" ) );
-    style().saveXML(doc, formatElement, sheet()->doc()->styleManager());
+    style().saveXML(doc, formatElement, sheet()->map()->styleManager());
     if ( formatElement.hasChildNodes() || formatElement.attributes().length() ) // don't save empty tags
         cell.appendChild( formatElement );
 
@@ -1016,7 +1012,7 @@ bool Cell::saveCellResult( QDomDocument& doc, QDomElement& result,
       if ( isDate() )
       {
           // serial number of date
-          QDate dd = value().asDateTime( this->doc() ).date();
+          QDate dd = value().asDateTime(sheet()->map()->calculationSettings()).date();
           dataType = "Date";
           str = "%1/%2/%3";
           str = str.arg(dd.year()).arg(dd.month()).arg(dd.day());
@@ -1025,7 +1021,7 @@ bool Cell::saveCellResult( QDomDocument& doc, QDomElement& result,
       {
           // serial number of time
           dataType = "Time";
-          str = value().asDateTime( this->doc() ).time().toString();
+          str = value().asDateTime(sheet()->map()->calculationSettings()).time().toString();
       }
       else
       {
@@ -1060,15 +1056,15 @@ bool Cell::saveCellResult( QDomDocument& doc, QDomElement& result,
   return true; /* really isn't much of a way for this function to fail */
 }
 
-void Cell::saveOasisAnnotation( KoXmlWriter &xmlwriter )
+void Cell::saveOdfAnnotation( KoXmlWriter &xmlwriter )
 {
     const QString comment = this->comment();
     if ( !comment.isEmpty() )
     {
         //<office:annotation draw:style-name="gr1" draw:text-style-name="P1" svg:width="2.899cm" svg:height="2.691cm" svg:x="2.858cm" svg:y="0.001cm" draw:caption-point-x="-2.858cm" draw:caption-point-y="-0.001cm">
         xmlwriter.startElement( "office:annotation" );
-        QStringList text = comment.split( "\n", QString::SkipEmptyParts );
-        for ( QStringList::Iterator it = text.begin(); it != text.end(); ++it ) {
+        const QStringList text = comment.split( "\n", QString::SkipEmptyParts );
+        for ( QStringList::ConstIterator it = text.begin(); it != text.end(); ++it ) {
             xmlwriter.startElement( "text:p" );
             xmlwriter.addTextNode( *it );
             xmlwriter.endElement();
@@ -1077,24 +1073,24 @@ void Cell::saveOasisAnnotation( KoXmlWriter &xmlwriter )
     }
 }
 
-QString Cell::saveOasisCellStyle( KoGenStyle &currentCellStyle, KoGenStyles &mainStyles )
+QString Cell::saveOdfCellStyle( KoGenStyle &currentCellStyle, KoGenStyles &mainStyles )
 {
     const Conditions conditions = this->conditions();
     if ( !conditions.isEmpty() )
     {
         // this has to be an automatic style
         currentCellStyle = KoGenStyle( KoGenStyle::StyleAutoTableCell, "table-cell" );
-        conditions.saveOasisConditions( currentCellStyle );
+        conditions.saveOdfConditions( currentCellStyle );
     }
-    return style().saveOasis( currentCellStyle, mainStyles, d->sheet->doc()->styleManager() );
+    return style().saveOdf( currentCellStyle, mainStyles, d->sheet->map()->styleManager() );
 }
 
 
-bool Cell::saveOasis( KoXmlWriter& xmlwriter, KoGenStyles &mainStyles,
+bool Cell::saveOdf( KoXmlWriter& xmlwriter, KoGenStyles &mainStyles,
                       int row, int column, int &repeated,
-                      GenValidationStyles &valStyle, const QMap<int, Style>& columnDefaultStyles,
-                      const QMap<int, Style>& rowDefaultStyles )
+                     OdfSavingContext& tableContext)
 {
+    // see: OpenDocument, 8.1.3 Table Cell
     if ( !isPartOfMerged() )
         xmlwriter.startElement( "table:table-cell" );
     else
@@ -1114,26 +1110,27 @@ bool Cell::saveOasis( KoXmlWriter& xmlwriter, KoGenStyles &mainStyles,
 #endif
     // NOTE save the value before the style as long as the Formatter does not work correctly
     if ( link().isEmpty() )
-      saveOasisValue (xmlwriter);
+      saveOdfValue (xmlwriter);
 
     // Either there's no column and row default and the style's not the default style,
     // or the style is different to one of them. The row default takes precedence.
-    if ((!rowDefaultStyles.contains(row) && !columnDefaultStyles.contains(column) &&
+    if ((!tableContext.rowDefaultStyles.contains(row) &&
+         !tableContext.columnDefaultStyles.contains(column) &&
          !(style().isDefault() && conditions().isEmpty())) ||
-        (rowDefaultStyles.contains(row) && rowDefaultStyles[row] != style()) ||
-        (columnDefaultStyles.contains(column) && columnDefaultStyles[column] != style()))
+        (tableContext.rowDefaultStyles.contains(row) && tableContext.rowDefaultStyles[row] != style()) ||
+        (tableContext.columnDefaultStyles.contains(column) && tableContext.columnDefaultStyles[column] != style()))
     {
-        KoGenStyle currentCellStyle; // the type determined in saveOasisCellStyle
-        saveOasisCellStyle( currentCellStyle, mainStyles );
+        KoGenStyle currentCellStyle; // the type determined in saveOdfCellStyle
+        saveOdfCellStyle( currentCellStyle, mainStyles );
         // skip 'table:style-name' attribute for the default style
         if ( !currentCellStyle.isDefaultStyle() )
-            xmlwriter.addAttribute( "table:style-name", mainStyles.styles()[currentCellStyle] );
+            xmlwriter.addAttribute( "table:style-name", mainStyles.styles().find(currentCellStyle).value() );
     }
 
     // group empty cells with the same style
     const QString comment = this->comment();
-    if ( isEmpty() && comment.isEmpty() && !isPartOfMerged() && !doesMergeCells() )
-    {
+    if (isEmpty() && comment.isEmpty() && !isPartOfMerged() && !doesMergeCells() &&
+        !tableContext.cellHasAnchoredShapes(*this)) {
       bool refCellIsDefault = isDefault();
       int j = column + 1;
       Cell nextCell = sheet()->cellStorage()->nextInRow( column, row );
@@ -1143,8 +1140,7 @@ bool Cell::saveOasis( KoXmlWriter& xmlwriter, KoGenStyles &mainStyles,
         //   the next cell is not the adjacent one
         // or
         //   the next cell is not empty
-        if ( nextCell.column() != j || !nextCell.isEmpty() )
-        {
+        if (nextCell.column() != j || (!nextCell.isEmpty() || tableContext.cellHasAnchoredShapes(*this))) {
           if ( refCellIsDefault )
           {
             // if the origin cell was a default cell,
@@ -1153,12 +1149,12 @@ bool Cell::saveOasis( KoXmlWriter& xmlwriter, KoGenStyles &mainStyles,
           }
           // otherwise we just stop here to process the adjacent
           // cell in the next iteration of the outer loop
-          // (in Sheet::saveOasisCells)
+          // (in Sheet::saveOdfCells)
           break;
         }
 
         if ( nextCell.isPartOfMerged() || nextCell.doesMergeCells() ||
-             !nextCell.comment().isEmpty() ||
+             !nextCell.comment().isEmpty() || tableContext.cellHasAnchoredShapes(*this) ||
              !(nextCell.style() == style() && nextCell.conditions() == conditions()) )
         {
           break;
@@ -1167,7 +1163,7 @@ bool Cell::saveOasis( KoXmlWriter& xmlwriter, KoGenStyles &mainStyles,
         // get the next cell and set the index to the adjacent cell
         nextCell = sheet()->cellStorage()->nextInRow( j++, row );
       }
-      kDebug(36003) << "Cell::saveOasis: empty cell in column" << column
+      kDebug(36003) << "Cell::saveOdf: empty cell in column" << column
                     << "repeated" << repeated << "time(s)" << endl;
 
       if ( repeated > 1 )
@@ -1178,12 +1174,12 @@ bool Cell::saveOasis( KoXmlWriter& xmlwriter, KoGenStyles &mainStyles,
     if ( !validity.isEmpty() )
     {
         GenValidationStyle styleVal(&validity);
-        xmlwriter.addAttribute( "table:validation-name", valStyle.lookup( styleVal ) );
+        xmlwriter.addAttribute( "table:validation-name", tableContext.valStyle.lookup( styleVal ) );
     }
     if ( isFormula() )
     {
       //kDebug(36003) <<"Formula found";
-      QString formula = Oasis::encodeFormula( userInput(), locale() );
+      QString formula = Odf::encodeFormula( userInput(), locale() );
       xmlwriter.addAttribute( "table:formula", formula );
     }
     else if ( !link().isEmpty() )
@@ -1191,11 +1187,12 @@ bool Cell::saveOasis( KoXmlWriter& xmlwriter, KoGenStyles &mainStyles,
         //kDebug(36003)<<"Link found";
         xmlwriter.startElement( "text:p" );
         xmlwriter.startElement( "text:a" );
+        const QString url = link();
         //Reference cell is started by '#'
-        if ( Util::localReferenceAnchor( link() ) )
-            xmlwriter.addAttribute( " xLinkDialog.href", ( '#'+link() ) );
+        if ( Util::localReferenceAnchor( url ) )
+            xmlwriter.addAttribute( "xlink:href", ( '#'+url ) );
         else
-            xmlwriter.addAttribute( " xLinkDialog.href", link() );
+            xmlwriter.addAttribute( "xlink:href", url );
         xmlwriter.addTextNode( userInput() );
         xmlwriter.endElement();
         xmlwriter.endElement();
@@ -1220,13 +1217,36 @@ bool Cell::saveOasis( KoXmlWriter& xmlwriter, KoGenStyles &mainStyles,
         xmlwriter.endElement();
     }
 
-    saveOasisAnnotation( xmlwriter );
+    // flake
+    // Save shapes that are anchored to this cell.
+    // see: OpenDocument, 2.3.1 Text Documents
+    // see: OpenDocument, 9.2 Drawing Shapes
+    if (tableContext.cellHasAnchoredShapes(*this)) {
+        const QList<KoShape*> shapes = tableContext.cellAnchoredShapes(*this);
+        for (int i = 0; i < shapes.count(); ++i) {
+            KoShape* const shape = shapes[i];
+            const QPointF bottomRight = shape->boundingRect().bottomRight();
+            double endX = 0.0;
+            double endY = 0.0;
+            const int col = sheet()->leftColumn(bottomRight.x(), endX);
+            const int row = sheet()->topRow(bottomRight.y(), endY);
+            shape->setAdditionalAttribute("table:end-cell-address", Cell(sheet(), col, row).name());
+            shape->setAdditionalAttribute("table:end-x", QString::number(bottomRight.x() - endX));
+            shape->setAdditionalAttribute("table:end-y", QString::number(bottomRight.y() - endY));
+            shapes[i]->saveOdf(tableContext.shapeContext);
+            shape->removeAdditionalAttribute("table:end-cell-address");
+            shape->removeAdditionalAttribute("table:end-x");
+            shape->removeAdditionalAttribute("table:end-y");
+        }
+    }
+
+    saveOdfAnnotation( xmlwriter );
 
     xmlwriter.endElement();
     return true;
 }
 
-void Cell::saveOasisValue (KoXmlWriter &xmlWriter)
+void Cell::saveOdfValue (KoXmlWriter &xmlWriter)
 {
   switch (value().format())
   {
@@ -1251,7 +1271,7 @@ void Cell::saveOasisValue (KoXmlWriter &xmlWriter)
     {
       xmlWriter.addAttribute( "office:value-type", "percentage" );
       xmlWriter.addAttribute( "office:value",
-          QString::number( numToDouble (value().asFloat() )) );
+          QString::number( (double) numToDouble (value().asFloat() )) );
       break;
     }
     case Value::fmt_Money:
@@ -1263,7 +1283,7 @@ void Cell::saveOasisValue (KoXmlWriter &xmlWriter)
         Currency currency = style.currency();
         xmlWriter.addAttribute( "office:currency", currency.code() );
       }
-      xmlWriter.addAttribute( "office:value", QString::number( numToDouble (value().asFloat()) ) );
+      xmlWriter.addAttribute( "office:value", QString::number( (double) numToDouble (value().asFloat()) ) );
       break;
     }
     case Value::fmt_DateTime: break;  //NOTHING HERE
@@ -1271,14 +1291,14 @@ void Cell::saveOasisValue (KoXmlWriter &xmlWriter)
     {
       xmlWriter.addAttribute( "office:value-type", "date" );
       xmlWriter.addAttribute( "office:date-value",
-          value().asDate( doc() ).toString( Qt::ISODate ) );
+          value().asDate(sheet()->map()->calculationSettings()).toString( Qt::ISODate ) );
       break;
     }
     case Value::fmt_Time:
     {
       xmlWriter.addAttribute( "office:value-type", "time" );
       xmlWriter.addAttribute( "office:time-value",
-          value().asTime( doc() ).toString( "PThhHmmMssS" ) );
+          value().asTime(sheet()->map()->calculationSettings()).toString( "PThhHmmMssS" ) );
       break;
     }
     case Value::fmt_String:
@@ -1290,12 +1310,12 @@ void Cell::saveOasisValue (KoXmlWriter &xmlWriter)
   };
 }
 
-bool Cell::loadOasis( const KoXmlElement& element, KoOasisLoadingContext& oasisContext )
+bool Cell::loadOdf(const KoXmlElement& element, OdfLoadingContext& tableContext)
 {
     kDebug(36003) <<"*** Loading cell properties ***** at" << name();
 
     //Search and load each paragraph of text. Each paragraph is separated by a line break.
-    loadOasisCellText( element );
+    loadOdfCellText( element );
 
     //
     // formula
@@ -1311,7 +1331,7 @@ bool Cell::loadOasis( const KoXmlElement& element, KoOasisLoadingContext& oasisC
             oasisFormula= oasisFormula.mid( 5 );
         else if (oasisFormula.startsWith( "kspr:" ) )
             oasisFormula= oasisFormula.mid( 5 );
-        oasisFormula = Oasis::decodeFormula( oasisFormula, locale() );
+        oasisFormula = Odf::decodeFormula( oasisFormula, locale() );
         setUserInput( oasisFormula );
     }
     else if ( !userInput().isEmpty() && userInput().at(0) == '=' ) //prepend ' to the text to avoid = to be painted
@@ -1322,9 +1342,10 @@ bool Cell::loadOasis( const KoXmlElement& element, KoOasisLoadingContext& oasisC
     //
     if ( element.hasAttributeNS( KoXmlNS::table, "validation-name" ) )
     {
-        kDebug(36003)<<" validation-name:"<<element.attributeNS( KoXmlNS::table,"validation-name", QString() );
+        const QString validationName = element.attributeNS(KoXmlNS::table,"validation-name", QString());
+        kDebug(36003) << " validation-name:" << validationName;
         Validity validity;
-        validity.loadOasisValidation( this, element.attributeNS( KoXmlNS::table, "validation-name", QString() ) );
+        validity.loadOdfValidation(this, validationName, tableContext);
         if ( !validity.isEmpty() )
             setValidity( validity );
     }
@@ -1351,7 +1372,7 @@ bool Cell::loadOasis( const KoXmlElement& element, KoOasisLoadingContext& oasisC
             if (ok)
                 setValue(value);
             if (!isFormula && userInput().isEmpty())
-                setUserInput(doc()->converter()->asString(value).asString());
+                setUserInput(sheet()->map()->converter()->asString(value).asString());
         }
 
         // currency value
@@ -1383,7 +1404,7 @@ bool Cell::loadOasis( const KoXmlElement& element, KoOasisLoadingContext& oasisC
                 value.setFormat(Value::fmt_Percent);
                 setValue(value);
                 if (!isFormula && userInput().isEmpty())
-                    setUserInput(doc()->converter()->asString(value).asString());
+                    setUserInput(sheet()->map()->converter()->asString(value).asString());
 // FIXME Stefan: Should be handled by Value::Format. Verify and remove!
 #if 0
                 Style style;
@@ -1421,7 +1442,7 @@ bool Cell::loadOasis( const KoXmlElement& element, KoOasisLoadingContext& oasisC
 
             if ( ok )
             {
-                setValue( Value( QDate( year, month, day ), doc() ) );
+                setValue( Value( QDate( year, month, day ), sheet()->map()->calculationSettings()) );
 // FIXME Stefan: Should be handled by Value::Format. Verify and remove!
 #if 0
                 Style style;
@@ -1469,7 +1490,7 @@ bool Cell::loadOasis( const KoXmlElement& element, KoOasisLoadingContext& oasisC
             {
                 // Value kval( timeToNum( hours, minutes, seconds ) );
                 // cell.setValue( kval );
-                setValue( Value( QTime( hours % 24, minutes, seconds ), doc() ) );
+                setValue( Value( QTime( hours % 24, minutes, seconds ), sheet()->map()->calculationSettings()) );
 // FIXME Stefan: Should be handled by Value::Format. Verify and remove!
 #if 0
                 Style style;
@@ -1534,7 +1555,7 @@ bool Cell::loadOasis( const KoXmlElement& element, KoOasisLoadingContext& oasisC
     //
     // cell comment/annotation
     //
-    KoXmlElement annotationElement = KoDom::namedItemNS( element, KoXmlNS::office, "annotation" );
+    KoXmlElement annotationElement = KoXml::namedItemNS( element, KoXmlNS::office, "annotation" );
     if ( !annotationElement.isNull() )
     {
         QString comment;
@@ -1555,14 +1576,12 @@ bool Cell::loadOasis( const KoXmlElement& element, KoOasisLoadingContext& oasisC
             setComment( comment );
     }
 
-    KoXmlElement frame = KoDom::namedItemNS( element, KoXmlNS::draw, "frame" );
-    if ( !frame.isNull() )
-      loadOasisObjects( frame, oasisContext );
+    loadOdfObjects(element, tableContext.odfContext);
 
     return true;
 }
 
-void Cell::loadOasisCellText( const KoXmlElement& parent )
+void Cell::loadOdfCellText( const KoXmlElement& parent )
 {
     //Search and load each paragraph of text. Each paragraph is separated by a line break
     KoXmlElement textParagraphElement;
@@ -1584,7 +1603,7 @@ void Cell::loadOasisCellText( const KoXmlElement& parent )
                 multipleTextParagraphsFound=true;
             }
 
-            KoXmlElement textA = KoDom::namedItemNS( textParagraphElement, KoXmlNS::text, "a" );
+            KoXmlElement textA = KoXml::namedItemNS( textParagraphElement, KoXmlNS::text, "a" );
             if( !textA.isNull() )
             {
                 if ( textA.hasAttributeNS( KoXmlNS::xlink, "href" ) )
@@ -1592,7 +1611,7 @@ void Cell::loadOasisCellText( const KoXmlElement& parent )
                     QString link = textA.attributeNS( KoXmlNS::xlink, "href", QString() );
                     cellText = textA.text();
                     setUserInput( cellText );
-                    // The value will be set later in loadOasis().
+                    // The value will be set later in loadOdf().
                     if ( (!link.isEmpty()) && (link[0]=='#') )
                         link=link.remove( 0, 1 );
                     setLink( link );
@@ -1604,7 +1623,7 @@ void Cell::loadOasisCellText( const KoXmlElement& parent )
     if (!cellText.isNull())
     {
         setUserInput( cellText );
-        // The value will be set later in loadOasis().
+        // The value will be set later in loadOdf().
     }
 
     //Enable word wrapping if multiple lines of text have been found.
@@ -1616,68 +1635,82 @@ void Cell::loadOasisCellText( const KoXmlElement& parent )
     }
 }
 
-void Cell::loadOasisObjects( const KoXmlElement &parent, KoOasisLoadingContext& oasisContext )
+void Cell::loadOdfObjects( const KoXmlElement &parent, KoOdfLoadingContext& odfContext )
 {
-#if 0 // KSPREAD_KOPART_EMBEDDING
-    for( KoXmlElement e = parent; !e.isNull(); e = e.nextSibling().toElement() )
+    // Register additional attributes, that identify shapes anchored in cells.
+    // Their dimensions need adjustment after all rows are loaded,
+    // because the position of the end cell is not always known yet.
+    KoShapeLoadingContext::addAdditionalAttributeData(KoShapeLoadingContext::AdditionalAttributeData(
+                                                      KoXmlNS::table, "end-cell-address",
+                                                      "table:end-cell-address"));
+    KoShapeLoadingContext::addAdditionalAttributeData(KoShapeLoadingContext::AdditionalAttributeData(
+                                                      KoXmlNS::table, "end-x",
+                                                      "table:end-x"));
+    KoShapeLoadingContext::addAdditionalAttributeData(KoShapeLoadingContext::AdditionalAttributeData(
+                                                      KoXmlNS::table, "end-y",
+                                                      "table:end-y"));
+
+    KoShapeLoadingContext shapeContext(odfContext, d->sheet->dataCenterMap());
+    KoXmlElement element;
+    forEachElement(element, parent)
     {
-        if ( e.localName() == "frame" && e.namespaceURI() == KoXmlNS::draw )
+        if (element.namespaceURI() != KoXmlNS::draw)
+            continue;
+
+        KoShape* shape = KoShapeRegistry::instance()->createShapeFromOdf(element, shapeContext);
+        if (!shape)
         {
-          EmbeddedObject *obj = 0;
-          KoXmlNode object = KoDom::namedItemNS( e, KoXmlNS::draw, "object" );
-          if ( !object.isNull() )
-          {
-            if ( !object.toElement().attributeNS( KoXmlNS::draw, "notify-on-update-of-ranges", QString()).isNull() )
-              obj = new EmbeddedChart( doc(), sheet() );
-            else
-              obj = new EmbeddedKOfficeObject( doc(), sheet() );
-          }
-          else
-          {
-            KoXmlNode image = KoDom::namedItemNS( e, KoXmlNS::draw, "image" );
-            if ( !image.isNull() )
-              obj = new EmbeddedPictureObject( sheet(), doc()->pictureCollection() );
-            else
-              kDebug(36003) <<"Object type wasn't loaded!";
-          }
-
-          if ( obj )
-          {
-            obj->loadOasis( e, oasisContext );
-            doc()->insertObject( obj );
-
-            QString ref = e.attributeNS( KoXmlNS::table, "end-cell-address", QString() );
-            if ( ref.isNull() )
-              continue;
-
-            ref = Oasis::decodeFormula( ref );
-            Point point( ref );
-            if ( !point.isValid() )
-              continue;
-
-            QRectF geometry = obj->geometry();
-            geometry.setLeft( geometry.left() + sheet()->columnPosition( d->column ) );
-            geometry.setTop( geometry.top() + sheet()->rowPosition( d->row ) );
-
-            QString str = e.attributeNS( KoXmlNS::table, "end-x", QString() );
-            if ( !str.isNull() )
-            {
-              uint end_x = (uint) KoUnit::parseValue( str );
-              geometry.setRight( sheet()->columnPosition( point.column() ) + end_x );
-            }
-
-            str = e.attributeNS( KoXmlNS::table, "end-y", QString() );
-            if ( !str.isNull() )
-            {
-              uint end_y = (uint) KoUnit::parseValue( str );
-              geometry.setBottom( sheet()->rowPosition( point.row() ) + end_y );
-            }
-
-            obj->setGeometry( geometry );
-          }
+            kDebug(36003) << "Unable to load shape.";
+            continue;
         }
+
+        d->sheet->addShape(shape);
+
+        // All three attributes are necessary for cell anchored shapes.
+        // Otherwise, they are anchored in the sheet.
+        if (!shape->hasAdditionalAttribute("table:end-cell-address") ||
+            !shape->hasAdditionalAttribute("table:end-x") ||
+            !shape->hasAdditionalAttribute("table:end-y"))
+        {
+            kDebug(36003) << "Not all attributes found, that are necessary for cell anchoring.";
+            continue;
+        }
+
+        Region endCell(Region::loadOdf(shape->additionalAttribute("table:end-cell-address")),
+                       d->sheet->map(), d->sheet);
+        if (!endCell.isValid() || !endCell.isSingular())
+            continue;
+
+        QString string = shape->additionalAttribute("table:end-x");
+        if (string.isNull())
+            continue;
+        double endX = KoUnit::parseValue(string);
+
+        string = shape->additionalAttribute("table:end-y");
+        if (string.isNull())
+            continue;
+        double endY = KoUnit::parseValue(string);
+
+        // The position is relative to the upper left sheet corner until now. Move it.
+        QPointF position = shape->position();
+        for (int col = 1; col < column(); ++col)
+            position += QPointF(d->sheet->columnFormat(col)->width(), 0.0);
+        for (int row = 1; row < this->row(); ++row)
+            position += QPointF(0.0, d->sheet->rowFormat(row)->height());
+        shape->setPosition(position);
+
+        // The column dimensions are already the final ones, but not the row dimensions.
+        // The default height is used for the not yet loaded rows.
+        // TODO Stefan: Honor non-default row heights later!
+        QSizeF size = QSizeF(endX, endY);
+        for (int col = column(); col < endCell.firstRange().left(); ++col)
+            size += QSizeF(d->sheet->columnFormat(col)->width(), 0.0);
+        for (int row = this->row(); row < endCell.firstRange().top(); ++row)
+            size += QSizeF(0.0, d->sheet->rowFormat(row)->height());
+        shape->setSize(size);
+
+        dynamic_cast<ShapeApplicationData*>(shape->applicationData())->setAnchoredToCell(true);
     }
-#endif // KSPREAD_KOPART_EMBEDDING
 }
 
 
@@ -1760,12 +1793,10 @@ bool Cell::load( const KoXmlElement & cell, int _xshift, int _yshift,
     if ( !conditionsElement.isNull())
     {
         Conditions conditions;
-        conditions.loadConditions( doc()->styleManager(), conditionsElement );
+        conditions.loadConditions(sheet()->map()->styleManager(), conditionsElement);
         if ( !conditions.isEmpty() )
             setConditions( conditions );
-    }
-    else if (paste && (mode == Paste::Normal) || (mode == Paste::NoBorder))
-    {
+    } else if (paste && (mode == Paste::Normal || mode == Paste::NoBorder)) {
       //clear the conditional formatting
       setConditions( Conditions() );
     }
@@ -1776,9 +1807,7 @@ bool Cell::load( const KoXmlElement & cell, int _xshift, int _yshift,
         Validity validity;
         if ( validity.loadXML( this, validityElement ) )
             setValidity( validity );
-    }
-    else if (paste && (mode == Paste::Normal) || (mode == Paste::NoBorder))
-    {
+    } else if (paste && (mode == Paste::Normal || mode == Paste::NoBorder)) {
       // clear the validity
       setValidity( Validity() );
     }
@@ -1876,7 +1905,7 @@ bool Cell::load( const KoXmlElement & cell, int _xshift, int _yshift,
             int day   = t.right( t.length() - pos1 - 1 ).toInt();
             QDate date( year, month, day );
             if ( date.isValid() )
-              setValue( Value( date, doc() ) );
+              setValue( Value( date, sheet()->map()->calculationSettings()) );
             else
               clear = false;
           }
@@ -1904,7 +1933,7 @@ bool Cell::load( const KoXmlElement & cell, int _xshift, int _yshift,
             second  = t.right( t.length() - pos1 - 1 ).toInt();
             QTime time( hours, minutes, second );
             if ( time.isValid() )
-              setValue( Value( time, doc() ) );
+              setValue( Value( time, sheet()->map()->calculationSettings()) );
             else
               clear = false;
           }
@@ -2023,7 +2052,7 @@ bool Cell::loadCellData(const KoXmlElement & text, Paste::Operation op )
           kWarning(36001) << "Couldn't parse '" << t << "' as number.";
         }
         /* We will need to localize the text version of the number */
-        KLocale* locale = doc()->locale();
+        KLocale* locale = sheet()->map()->calculationSettings()->locale();
 
         /* KLocale::formatNumber requires the precision we want to return.
         */
@@ -2056,9 +2085,9 @@ bool Cell::loadCellData(const KoXmlElement & text, Paste::Operation op )
         int pos1 = t.indexOf('/',pos+1);
         int month = t.mid(pos+1,((pos1-1)-pos)).toInt();
         int day = t.right(t.length()-pos1-1).toInt();
-        setValue( Value( QDate(year,month,day), doc() ) );
-        if ( value().asDate( doc() ).isValid() ) // Should always be the case for new docs
-          setUserInput( locale()->formatDate( value().asDate( doc() ), KLocale::ShortDate ) );
+        setValue( Value( QDate(year,month,day), sheet()->map()->calculationSettings()) );
+        if ( value().asDate(sheet()->map()->calculationSettings()).isValid() ) // Should always be the case for new docs
+          setUserInput( locale()->formatDate( value().asDate(sheet()->map()->calculationSettings()), KLocale::ShortDate ) );
         else // This happens with old docs, when format is set wrongly to date
         {
           parseUserInput( pasteOperation( t, userInput(), op ) );
@@ -2077,9 +2106,9 @@ bool Cell::loadCellData(const KoXmlElement & text, Paste::Operation op )
         pos1 = t.indexOf(':',pos+1);
         minutes = t.mid(pos+1,((pos1-1)-pos)).toInt();
         second = t.right(t.length()-pos1-1).toInt();
-        setValue( Value( QTime(hours,minutes,second), doc() ) );
-        if ( value().asTime( doc() ).isValid() ) // Should always be the case for new docs
-          setUserInput( locale()->formatTime( value().asTime( doc() ), true ) );
+        setValue( Value( QTime(hours,minutes,second), sheet()->map()->calculationSettings()) );
+        if ( value().asTime( sheet()->map()->calculationSettings()).isValid() ) // Should always be the case for new docs
+          setUserInput( locale()->formatTime( value().asTime( sheet()->map()->calculationSettings()), true ) );
         else  // This happens with old docs, when format is set wrongly to time
         {
           parseUserInput( pasteOperation( t, userInput(), op ) );
@@ -2115,8 +2144,8 @@ QTime Cell::toTime(const KoXmlElement &element)
     pos1 = t.indexOf(':',pos+1);
     minutes = t.mid(pos+1,((pos1-1)-pos)).toInt();
     second = t.right(t.length()-pos1-1).toInt();
-    setValue( Value( QTime(hours,minutes,second), doc() ) );
-    return value().asTime( doc() );
+    setValue( Value( QTime(hours,minutes,second), sheet()->map()->calculationSettings()) );
+    return value().asTime( sheet()->map()->calculationSettings());
 }
 
 QDate Cell::toDate(const KoXmlElement &element)
@@ -2132,8 +2161,8 @@ QDate Cell::toDate(const KoXmlElement &element)
     pos1 = t.indexOf('/',pos+1);
     month = t.mid(pos+1,((pos1-1)-pos)).toInt();
     day = t.right(t.length()-pos1-1).toInt();
-    setValue( Value( QDate(year,month,day), doc() ) );
-    return value().asDate( doc() );
+    setValue( Value( QDate(year,month,day), sheet()->map()->calculationSettings()) );
+    return value().asDate( sheet()->map()->calculationSettings());
 }
 
 QString Cell::pasteOperation( const QString &new_text, const QString &old_text, Paste::Operation op )
