@@ -35,12 +35,8 @@
 #define KSPREAD_CELL
 
 #include <QDate>
-#include <QLinkedList>
-#include <QList>
 #include <QSharedDataPointer>
 
-#include <KoXmlReader.h>
-
 #include "Condition.h"
 #include "Global.h"
 #include "Format.h"
@@ -57,13 +53,14 @@ class KLocale;
 class KoXmlWriter;
 class KoGenStyles;
 class KoGenStyle;
-class KoOasisLoadingContext;
+class KoOdfLoadingContext;
 
 namespace KSpread
 {
 class Doc;
 class Formula;
-class GenValidationStyles;
+class OdfLoadingContext;
+class OdfSavingContext;
 class Sheet;
 class Validity;
 class Value;
@@ -112,11 +109,6 @@ public:
      */
     Sheet* sheet() const;
 
-    /**
-     * \return the document this cell belongs to
-     */
-    Doc* doc() const;
-
     /**
      * Returns the locale setting of this cell.
      */
@@ -337,17 +329,16 @@ public:
      * \ingroup OpenDocument
      * Loads a cell from an OASIS XML element.
      * @param element An OASIS XML element
-     * @param oasisContext The loading context assoiated with the XML element
+     * @param tableContext The loading context assoiated with the XML element
      */
-    bool loadOasis( const KoXmlElement& element, KoOasisLoadingContext& oasisContext );
+    bool loadOdf(const KoXmlElement& element, OdfLoadingContext& tableContext);
 
     /**
      * \ingroup OpenDocument
      */
-    bool saveOasis( KoXmlWriter& xmlwriter, KoGenStyles& mainStyles,
+    bool saveOdf( KoXmlWriter& xmlwriter, KoGenStyles& mainStyles,
                     int row, int column, int &repeated,
-                    GenValidationStyles &valStyle, const QMap<int, Style>& columnDefaultStyles,
-                    const QMap<int, Style>& rowDefaultStyles );
+                   OdfSavingContext& savingContext);
 
     /**
      * Copies the format from \p cell .
@@ -536,7 +527,7 @@ public:
     Cell& operator=( const Cell& other );
 
     /**
-     * Tests wether this cell's location is less than the \p other 's.
+     * Tests whether this cell's location is less than the \p other 's.
      * (QMap support)
      * \note Does not compare the cell attributes/data.
      */
@@ -573,17 +564,17 @@ protected:
      * Load the text paragraphs from an OASIS XML cell description.
      * @param parent The DOM element representing the cell.
      */
-    void loadOasisCellText( const KoXmlElement& parent );
+    void loadOdfCellText( const KoXmlElement& parent );
 
     /**
      * \ingroup OpenDocument
      */
-    void loadOasisObjects( const KoXmlElement& e, KoOasisLoadingContext& oasisContext );
+    void loadOdfObjects( const KoXmlElement& e, KoOdfLoadingContext& odfContext );
 
     /**
      * \ingroup OpenDocument
      */
-    void saveOasisAnnotation( KoXmlWriter &xmlwriter );
+    void saveOdfAnnotation( KoXmlWriter &xmlwriter );
 
 private:
     class Private;
@@ -602,13 +593,13 @@ private:
     /**
      * \ingroup OpenDocument
      */
-    void saveOasisValue (KoXmlWriter &xmlWriter);
+    void saveOdfValue (KoXmlWriter &xmlWriter);
 
     /**
      * \ingroup OpenDocument
      * @return the OASIS style's name
      */
-    QString saveOasisCellStyle( KoGenStyle &currentCellStyle, KoGenStyles &mainStyles );
+    QString saveOdfCellStyle( KoGenStyle &currentCellStyle, KoGenStyles &mainStyles );
 };
 
 inline uint qHash( const Cell& cell )
@@ -620,4 +611,14 @@ inline uint qHash( const Cell& cell )
 
 Q_DECLARE_TYPEINFO( KSpread::Cell, Q_MOVABLE_TYPE );
 
+
+/***************************************************************************
+  kDebug support
+****************************************************************************/
+
+inline kdbgstream operator<<(kdbgstream str, const KSpread::Cell& cell)
+{
+    return str << qPrintable(KSpread::Cell::columnName(cell.column()) + QString::number(cell.row()));
+}
+
 #endif  // KSPREAD_CELL
