@@ -20,10 +20,11 @@
 
 #include "Formula.h"
 
+#include "CalculationSettings.h"
 #include "Cell.h"
 #include "CellStorage.h"
 #include "Sheet.h"
-#include "Doc.h"
+#include "Map.h"
 #include "NamedAreaManager.h"
 #include "Region.h"
 #include "Value.h"
@@ -171,6 +172,7 @@ Token::Op KSpread::matchOperator( const QString& text )
   if( text.length() == 2 )
   {
     if( text == "<>" ) result = Token::NotEqual;
+    if( text == "!=" ) result = Token::NotEqual;
     if( text == "<=" ) result = Token::LessEqual;
     if( text == ">=" ) result = Token::GreaterEqual;
     if( text == "==" ) result = Token::Equal;
@@ -200,7 +202,7 @@ static int opPrecedence( Token::Op op )
     case Token::LessEqual    : prec = 1; break;
     case Token::GreaterEqual : prec = 1; break;
 #ifdef KSPREAD_INLINE_ARRAYS
-    // FIXME Stefan: I don't know wether zero is right for this case. :-(
+    // FIXME Stefan: I don't know whether zero is right for this case. :-(
     case Token::CurlyBra     : prec = 0; break;
     case Token::CurlyKet     : prec = 0; break;
     case Token::Pipe         : prec = 0; break;
@@ -497,7 +499,7 @@ bool Formula::isValid() const
   {
     KLocale* locale = !d->cell.isNull() ? d->cell.locale() : 0;
     if ((!locale) && d->sheet)
-      locale = d->sheet->doc()->locale();
+      locale = d->sheet->map()->calculationSettings()->locale();
     Tokens tokens = scan( d->expression, locale );
     if( tokens.valid() )
       compile( tokens );
@@ -527,7 +529,7 @@ Tokens Formula::tokens() const
 {
   KLocale* locale = !d->cell.isNull() ? d->cell.locale() : 0;
   if ((!locale) && d->sheet)
-    locale = d->sheet->doc()->locale();
+    locale = d->sheet->map()->calculationSettings()->locale();
   return scan( d->expression, locale );
 }
 
@@ -654,8 +656,8 @@ Tokens Formula::scan( const QString& expr, const KLocale* locale ) const
        // consume as long as alpha, dollar sign, underscore, or digit
        if( isIdentifier( ch )  || ch.isDigit() ) tokenText.append( ex[i++] );
 
-       // a '!' ? then this must be sheet name, e.g "Sheet4!"
-       else if( ch == '!' )
+       // a '!' ? then this must be sheet name, e.g "Sheet4!", unless the next character is '='
+       else if( ch == '!' && ex[i+1] != '=' )
        {
           tokenText.append( ex[i++] );
           state = InCell;
@@ -1349,7 +1351,7 @@ void Formula::compile( const Tokens& tokens ) const
 
 bool Formula::isNamedArea( const QString& expr ) const
 {
-    return d->sheet ? d->sheet->doc()->namedAreaManager()->contains(expr) : false;
+    return d->sheet ? d->sheet->map()->namedAreaManager()->contains(expr) : false;
 }
 
 
@@ -1370,9 +1372,9 @@ Value Formula::eval() const
   QString c;
   QVector<Value> args;
 
-  const Doc *doc = d->sheet ? d->sheet->doc() : new Doc();
-  const ValueConverter* converter = doc->converter();
-  ValueCalc* calc = doc->calc();
+  const Map* map = d->sheet ? d->sheet->map() : new Map(0 /*document*/);
+  const ValueConverter* converter = map->converter();
+  ValueCalc* calc = map->calc();
 
   Function* function;
   FuncExtra fe;
@@ -1655,7 +1657,7 @@ Value Formula::eval() const
   }
 
   if ( !d->sheet )
-    delete doc;
+    delete map;
 
   // more than one value in stack ? unsuccessful execution...
   if( stack.count() != 1 )
@@ -1700,7 +1702,7 @@ QString Formula::dump() const
     QString vtext;
     Value val = d->constants[c];
     if( val.isString() ) vtext = QString("[%1]").arg( val.asString() );
-    else if( val.isNumber() ) vtext = QString("%1").arg( numToDouble (val.asFloat()) );
+    else if( val.isNumber() ) vtext = QString("%1").arg( (double) numToDouble (val.asFloat()) );
     else if( val.isBoolean() ) vtext = QString("%1").arg( val.asBoolean() ? "True":"False");
     else if( val.isError() ) vtext = "error";
     else vtext = "???";
