@@ -21,18 +21,14 @@
 #define KSPREAD_RECT_STORAGE
 
 #include <QCache>
+#include <QRegion>
 #include <QTimer>
 
 #include "kspread_export.h"
 
-#include "Binding.h"
-#include "Condition.h"
-#include "Doc.h"
+#include "Map.h"
 #include "Region.h"
 #include "RTree.h"
-#include "Validity.h"
-
-#include "database/Database.h"
 
 static const int g_garbageCollectionTimeOut = 100;
 
@@ -56,7 +52,7 @@ template<typename T>
 class KSPREAD_EXPORT RectStorage
 {
 public:
-    explicit RectStorage(Doc* doc);
+    explicit RectStorage(Map* map);
     RectStorage(const RectStorage& other);
     virtual ~RectStorage();
 
@@ -85,6 +81,11 @@ public:
      */
     void insert(const Region& region, const T& data);
 
+    /**
+     * Removes \p data from \p region .
+     */
+    void remove(const Region& region, const T& data);
+
     /**
      * Inserts \p number rows at the position \p position .
      * It extends or shifts rectangles, respectively.
@@ -152,7 +153,7 @@ protected:
     void invalidateCache( const QRect& rect );
 
 private:
-    Doc* m_doc;
+    Map* m_map;
     RTree<T> m_tree;
     QRegion m_usedArea;
     QMap<int, QPair<QRectF,T> > m_possibleGarbage;
@@ -162,14 +163,14 @@ private:
 };
 
 template<typename T>
-RectStorage<T>::RectStorage(Doc* doc)
-    : m_doc(doc)
+RectStorage<T>::RectStorage(Map* map)
+    : m_map(map)
 {
 }
 
 template<typename T>
 RectStorage<T>::RectStorage(const RectStorage& other)
-    : m_doc(other.m_doc)
+    : m_map(other.m_map)
     , m_usedArea(other.m_usedArea)
     , m_storedData(other.m_storedData)
 {
@@ -269,6 +270,22 @@ void RectStorage<T>::insert(const Region& region, const T& _data)
     }
 }
 
+template<typename T>
+void RectStorage<T>::remove(const Region& region, const T& data)
+{
+    if (!m_storedData.contains(data)) {
+        return;
+    }
+    const Region::ConstIterator end(region.constEnd());
+    for (Region::ConstIterator it(region.constBegin()); it != end; ++it) {
+        // keep track of the used area
+        m_usedArea -= (*it)->rect(); // FIXME Can still be used!
+        // insert data
+        m_tree.remove((*it)->rect(), data);
+        regionChanged((*it)->rect());
+    }
+}
+
 template<typename T>
 QList< QPair<QRectF,T> > RectStorage<T>::insertRows(int position, int number)
 {
@@ -499,7 +516,7 @@ void RectStorage<T>::garbageCollection()
     int zIndex = pairs.constBegin().key();
     DataPair pair = pairs[zIndex];
 
-    // check wether the default style is placed first
+    // check whether the default style is placed first
     if (zIndex == currentZIndex &&
         currentPair.second == T() &&
         pair.second == T() &&
@@ -548,7 +565,7 @@ void RectStorage<T>::garbageCollection()
 template<typename T>
 void RectStorage<T>::regionChanged( const QRect& rect )
 {
-    if (m_doc->isLoading())
+    if (m_map->isLoading())
          return;
     // mark the possible garbage
     // NOTE Stefan: The map may contain multiple indices. The already existing possible garbage has
@@ -576,25 +593,11 @@ void RectStorage<T>::invalidateCache( const QRect& invRect )
 
 
 
-class BindingStorage : public QObject, public RectStorage<Binding>
-{
-    Q_OBJECT
-public:
-    explicit BindingStorage(Doc* doc) : QObject(doc), RectStorage<Binding>(doc) {}
-    BindingStorage(const BindingStorage& other) : QObject(other.parent()), RectStorage<Binding>(other) {}
-
-protected Q_SLOTS:
-    virtual void triggerGarbageCollection() { QTimer::singleShot( g_garbageCollectionTimeOut, this, SLOT( garbageCollection() ) ); }
-    virtual void garbageCollection() { RectStorage<Binding>::garbageCollection(); }
-};
-
-
-
 class CommentStorage : public QObject, public RectStorage<QString>
 {
     Q_OBJECT
 public:
-    explicit CommentStorage(Doc* doc) : QObject(doc), RectStorage<QString>(doc) {}
+    explicit CommentStorage(Map* map) : QObject(map), RectStorage<QString>(map) {}
     CommentStorage(const CommentStorage& other) : QObject(other.parent()), RectStorage<QString>(other) {}
 
 protected Q_SLOTS:
@@ -604,39 +607,11 @@ protected Q_SLOTS:
 
 
 
-class ConditionsStorage : public QObject, public RectStorage<Conditions>
-{
-    Q_OBJECT
-public:
-    explicit ConditionsStorage(Doc* doc) : QObject(doc), RectStorage<Conditions>(doc) {}
-    ConditionsStorage(const ConditionsStorage& other) : QObject(other.parent()), RectStorage<Conditions>(other) {}
-
-protected Q_SLOTS:
-    virtual void triggerGarbageCollection() { QTimer::singleShot( g_garbageCollectionTimeOut, this, SLOT( garbageCollection() ) ); }
-    virtual void garbageCollection() { RectStorage<Conditions>::garbageCollection(); }
-};
-
-
-
-class DatabaseStorage : public QObject, public RectStorage<Database>
-{
-    Q_OBJECT
-public:
-    explicit DatabaseStorage(Doc* doc) : QObject(doc), RectStorage<Database>(doc) {}
-    DatabaseStorage(const DatabaseStorage& other) : QObject(other.parent()), RectStorage<Database>(other) {}
-
-protected Q_SLOTS:
-    virtual void triggerGarbageCollection() { QTimer::singleShot( g_garbageCollectionTimeOut, this, SLOT( garbageCollection() ) ); }
-    virtual void garbageCollection() { RectStorage<Database>::garbageCollection(); }
-};
-
-
-
 class FusionStorage : public QObject, public RectStorage<bool>
 {
     Q_OBJECT
 public:
-    explicit FusionStorage(Doc* doc) : QObject(doc), RectStorage<bool>(doc) {}
+    explicit FusionStorage(Map* map) : QObject(map), RectStorage<bool>(map) {}
     FusionStorage(const FusionStorage& other) : QObject(other.parent()), RectStorage<bool>(other) {}
 
 protected Q_SLOTS:
@@ -650,7 +625,7 @@ class MatrixStorage : public QObject, public RectStorage<bool>
 {
     Q_OBJECT
 public:
-    explicit MatrixStorage(Doc* doc) : QObject(doc), RectStorage<bool>(doc) {}
+    explicit MatrixStorage(Map* map) : QObject(map), RectStorage<bool>(map) {}
     MatrixStorage(const MatrixStorage& other) : QObject(other.parent()), RectStorage<bool>(other) {}
 
 protected Q_SLOTS:
@@ -658,20 +633,6 @@ protected Q_SLOTS:
     virtual void garbageCollection() { RectStorage<bool>::garbageCollection(); }
 };
 
-
-
-class ValidityStorage : public QObject, public RectStorage<Validity>
-{
-    Q_OBJECT
-public:
-    explicit ValidityStorage(Doc* doc) : QObject(doc), RectStorage<Validity>(doc) {}
-    ValidityStorage(const ValidityStorage& other) : QObject(other.parent()), RectStorage<Validity>(other) {}
-
-protected Q_SLOTS:
-    virtual void triggerGarbageCollection() { QTimer::singleShot( g_garbageCollectionTimeOut, this, SLOT( garbageCollection() ) ); }
-    virtual void garbageCollection() { RectStorage<Validity>::garbageCollection(); }
-};
-
 } // namespace KSpread
 
 #endif // KSPREAD_RECT_STORAGE
