@@ -23,15 +23,18 @@
 
 #include <kdebug.h>
 
-#include <KoZoomHandler.h>
+#include <KoCanvasBase.h>
+#include <KoCanvasController.h>
+#include <KoViewConverter.h>
 
 #include "Cell.h"
 #include "CellStorage.h"
-#include "Doc.h"
 #include "Editors.h"
 #include "RowColumnFormat.h"
 #include "Sheet.h"
 
+#include "commands/DataManipulators.h"
+
 using namespace KSpread;
 
 /***************************************************************************
@@ -64,6 +67,10 @@ public:
     activeElement = 0;
     activeSubRegionStart = 0;
     activeSubRegionLength = 1;
+
+    canvasBase = 0;
+    referenceMode = false;
+    lastEditorWithFocus = EmbeddedEditor;
   }
 
   Sheet* activeSheet;
@@ -79,24 +86,30 @@ public:
   int activeElement;
   int activeSubRegionStart;
   int activeSubRegionLength;
+
+  KoCanvasBase* canvasBase;
+  bool referenceMode : 1;
+  Region formerSelection; // for reference selection mode
+  Region oldSelection; // for select all
+  Editor lastEditorWithFocus;
 };
 
 /***************************************************************************
   class Selection
 ****************************************************************************/
-namespace KSpread {
 
-Selection::Selection( QObject* parent )
-    : QObject( parent )
+Selection::Selection(KoCanvasBase* canvasBase)
+    : KoToolSelection(canvasBase->canvasWidget())
     , Region(1,1)
     , d( new Private() )
 {
   d->activeSubRegionStart = 0;
   d->activeSubRegionLength = 1;
+  d->canvasBase = canvasBase;
 }
 
 Selection::Selection(const Selection& selection)
-    : QObject( selection.parent() )
+    : KoToolSelection(selection.parent())
     , Region()
     , d( new Private() )
 {
@@ -104,6 +117,7 @@ Selection::Selection(const Selection& selection)
     d->originSheet = selection.d->originSheet;
     d->activeSubRegionStart = 0;
     d->activeSubRegionLength = cells().count();
+    d->canvasBase = selection.d->canvasBase;
 }
 
 Selection::~Selection()
@@ -111,6 +125,11 @@ Selection::~Selection()
   delete d;
 }
 
+KoCanvasBase* Selection::canvas() const
+{
+    return d->canvasBase;
+}
+
 void Selection::initialize(const QPoint& point, Sheet* sheet)
 {
   if (!isValid(point))
@@ -674,8 +693,11 @@ bool Selection::isSingular() const
   return Region::isSingular();
 }
 
-QRectF Selection::selectionHandleArea( KoZoomHandler* zoomHandler ) const
+QRectF Selection::selectionHandleArea(const KoViewConverter* viewConverter) const
 {
+    if (d->referenceMode) {
+        return QRectF();
+    }
   int column, row;
 
   // complete rows/columns are selected, use the marker.
@@ -686,6 +708,9 @@ QRectF Selection::selectionHandleArea( KoZoomHandler* zoomHandler ) const
   }
   else
   {
+    if (!isValid()) {
+        return QRectF();
+    }
     column = lastRange().right();
     row = lastRange().bottom();
   }
@@ -696,8 +721,8 @@ QRectF Selection::selectionHandleArea( KoZoomHandler* zoomHandler ) const
   double width = cell.width();
   double height = cell.height();
 
-  const double unzoomedXPixel = zoomHandler->viewToDocumentX( 1.0 );
-  const double unzoomedYPixel = zoomHandler->viewToDocumentY( 1.0 );
+  const double unzoomedXPixel = viewConverter->viewToDocumentX( 1.0 );
+  const double unzoomedYPixel = viewConverter->viewToDocumentY( 1.0 );
 
   QRectF handle( xpos + width - 2 * unzoomedXPixel,
                 ypos + height - 2 * unzoomedYPixel,
@@ -713,7 +738,11 @@ QString Selection::name(Sheet* sheet) const
 
 void Selection::setActiveSheet( Sheet* sheet )
 {
+    if (d->activeSheet == sheet) {
+        return;
+    }
     d->activeSheet = sheet;
+    emit activeSheetChanged(sheet);
 }
 
 Sheet* Selection::activeSheet() const
@@ -766,7 +795,7 @@ void Selection::setActiveElement(int pos)
   d->activeElement = pos;
 }
 
-Region::Element* Selection::activeElement() const
+KSpread::Region::Element* Selection::activeElement() const
 {
   return (d->activeElement == cells().count()) ? 0 : cells()[d->activeElement];
 }
@@ -830,11 +859,6 @@ QString Selection::activeSubRegionName() const
   return names.isEmpty() ? "" : names.join(";");
 }
 
-void Selection::setMultipleOccurences(bool state)
-{
-  d->multipleOccurences = state;
-}
-
 void Selection::setSelectionMode(Mode mode)
 {
   d->selectionMode = mode;
@@ -845,6 +869,124 @@ const QList<QColor>& Selection::colors() const
   return d->colors;
 }
 
+void Selection::selectAll()
+{
+    if (!isAllSelected()) {
+        d->oldSelection = *this;
+        initialize(QRect(QPoint(KS_colMax, KS_rowMax), QPoint(1, 1)));
+    }
+    else {
+        initialize(d->oldSelection);
+        d->oldSelection.clear();
+    }
+}
+
+void Selection::setLastEditorWithFocus(Editor editor)
+{
+    d->lastEditorWithFocus = editor;
+}
+
+Selection::Editor Selection::lastEditorWithFocus() const
+{
+    return d->lastEditorWithFocus;
+}
+
+void Selection::startReferenceSelection(const Region& region)
+{
+    // (Tomas) do we really need this?
+    if (d->referenceMode) {
+        if (region.isValid()) {
+            initialize(region);
+        }
+        return;
+    }
+    // former selection exists - we are in ref mode already, even though it's suspended
+    if (!d->formerSelection.isEmpty())
+        return;
+
+    d->formerSelection = *this;
+    clear();
+    setOriginSheet(activeSheet());
+    if (region.isValid()) {
+        initialize(region);
+    }
+    // It is important to enable this AFTER we set the rect!
+    d->referenceMode = true;
+    d->multipleOccurences = true;
+    // Visual cue to indicate that the user can drag-select the selection selection
+    d->canvasBase->canvasWidget()->setCursor(Qt::CrossCursor);
+}
+
+void Selection::endReferenceSelection()
+{
+    // The reference selection may be temporarily disabled.
+    // The stored selection reliably indicates the reference selection mode.
+    if (d->formerSelection.isEmpty()) {
+        return;
+    }
+    d->referenceMode = false;
+    d->multipleOccurences = false;
+    if (originSheet() != activeSheet()) {
+        emit visibleSheetRequested(originSheet());
+    }
+    // While entering a formula the choose mode is turned on and off.
+    // Clear the choice. Otherwise, cell references will stay highlighted.
+    if (!isEmpty()) {
+        emit changed(*this);
+        clear();
+    }
+    initialize(d->formerSelection);
+    d->formerSelection.clear();
+    d->canvasBase->canvasWidget()->setCursor(Qt::ArrowCursor);
+}
+
+void Selection::setReferenceSelectionMode(bool enable)
+{
+    d->referenceMode = enable;
+    d->multipleOccurences = enable;
+    d->canvasBase->canvasWidget()->setCursor(enable ? Qt::CrossCursor : Qt::ArrowCursor);
+}
+
+bool Selection::referenceSelectionMode() const
+{
+    return d->referenceMode;
+}
+
+bool Selection::referenceSelection() const
+{
+  return (!d->formerSelection.isEmpty());
+}
+
+void Selection::emitAboutToModify()
+{
+    emit aboutToModify(*this);
+}
+
+void Selection::emitModified()
+{
+    emit modified(*this);
+}
+
+void Selection::emitRefreshSheetViews()
+{
+    emit refreshSheetViews();
+}
+
+void Selection::emitVisibleSheetRequested(Sheet* sheet)
+{
+    emit visibleSheetRequested(sheet);
+}
+
+void Selection::emitCloseEditor(bool saveChanges, bool expandMatrix)
+{
+    emit closeEditor(saveChanges, expandMatrix);
+}
+
+void Selection::emitRequestFocusEditor()
+{
+    emit requestFocusEditor();
+}
+
 QRect Selection::extendToMergedAreas(const QRect& _area) const
 {
   if (!d->activeSheet)
@@ -899,32 +1041,32 @@ QRect Selection::extendToMergedAreas(const QRect& _area) const
   return area;
 }
 
-Selection::Region::Point* Selection::createPoint(const QPoint& point) const
+KSpread::Region::Point* Selection::createPoint(const QPoint& point) const
 {
   return new Point(point);
 }
 
-Selection::Region::Point* Selection::createPoint(const QString& string) const
+KSpread::Region::Point* Selection::createPoint(const QString& string) const
 {
   return new Point(string);
 }
 
-Selection::Region::Point* Selection::createPoint(const Point& point) const
+KSpread::Region::Point* Selection::createPoint(const Region::Point& point) const
 {
   return new Point(point);
 }
 
-Selection::Region::Range* Selection::createRange(const QRect& rect) const
+KSpread::Region::Range* Selection::createRange(const QRect& rect) const
 {
   return new Range(rect);
 }
 
-Selection::Region::Range* Selection::createRange(const QString& string) const
+KSpread::Region::Range* Selection::createRange(const QString& string) const
 {
   return new Range(string);
 }
 
-Selection::Region::Range* Selection::createRange(const Range& range) const
+KSpread::Region::Range* Selection::createRange(const Region::Range& range) const
 {
   return new Range(range);
 }
@@ -1035,6 +1177,12 @@ Selection::Point::Point(const QString& string)
 {
 }
 
+Selection::Point::Point(const Region::Point& point)
+  : Region::Point(point),
+    m_color(Qt::black)
+{
+}
+
 /***************************************************************************
   class Range
 ****************************************************************************/
@@ -1051,6 +1199,10 @@ Selection::Range::Range(const QString& string)
 {
 }
 
-} // namespace KSpread
+Selection::Range::Range(const Region::Range& range)
+  : Region::Range(range),
+    m_color(Qt::black)
+{
+}
 
 #include "Selection.moc"
