@@ -23,21 +23,19 @@
 // Local
 #include "SheetPrint.h"
 
-#include <kprinter.h>
-
 #include <kdebug.h>
 #include <kmessagebox.h>
 
-#include "Doc.h"
+#include "ApplicationSettings.h"
+#include "part/Doc.h" // FIXME detach from part
 #include "Localization.h"
-#include "Object.h"
+#include "Map.h"
 #include "RowColumnFormat.h"
 #include "Sheet.h"
 #include "SheetView.h"
 #include "Selection.h"
-#include "Undo.h"
 
-#include "commands/DefinePrintRangeCommand.h"
+#include "part/commands/DefinePrintRangeCommand.h" // FIXME detach from part
 
 #include <KoDocumentInfo.h>
 #include <KoZoomHandler.h>
@@ -93,6 +91,7 @@ private:
 };
 
 
+#if 0 // KSPREAD_KOPART_EMBEDDING
 class PrintObject
 {
 public:
@@ -101,6 +100,7 @@ public:
     EmbeddedObject *obj;
     QPixmap *p;
 };
+#endif
 
 } // namespace KSpread
 
@@ -112,22 +112,8 @@ SheetPrint::SheetPrint( Sheet* sheet )
     m_pSheetView = new SheetView( sheet );
     m_pDoc = m_pSheet->doc();
 
-    m_bPrintGrid = false;
-    m_bPrintCommentIndicator = false;
-    m_bPrintFormulaIndicator = false;
-    m_bPrintObjects = true;
-    m_bPrintCharts = true;
-    m_bPrintGraphics = true;
+    m_settings = new PrintSettings();
 
-    m_leftBorder   = MM_TO_POINT(20.0);
-    m_rightBorder  = MM_TO_POINT(20.0);
-    m_topBorder    = MM_TO_POINT(20.0);
-    m_bottomBorder = MM_TO_POINT(20.0);
-
-    m_paperFormat = KoPageFormat::defaultFormat();
-    m_orientation = KoPageFormat::Portrait;
-    m_paperWidth = MM_TO_POINT( KoPageFormat::width( m_paperFormat, m_orientation ) );
-    m_paperHeight = MM_TO_POINT( KoPageFormat::height( m_paperFormat, m_orientation ) );
     m_printRange = QRect( QPoint( 1, 1 ), QPoint( KS_colMax, KS_rowMax ) );
     m_maxCheckedNewPageX = 1;
     m_maxCheckedNewPageY = 1;
@@ -138,29 +124,24 @@ SheetPrint::SheetPrint( Sheet* sheet )
     m_zoomHandler = new KoZoomHandler();
     m_iPageLimitX = 0;
     m_iPageLimitY = 0;
-
-    calcPaperSize();
 }
 
 SheetPrint::~SheetPrint()
 {
     delete m_zoomHandler;
+    delete m_settings;
     delete m_pSheetView;
 }
 
-QString SheetPrint::saveOasisSheetStyleLayout( KoGenStyles &mainStyles )
+QString SheetPrint::saveOdfSheetStyleLayout( KoGenStyles &mainStyles )
 {
-    KoGenStyle pageLayout( KoGenStyle::StylePageLayout );
+    KoGenStyle pageLayout = m_settings->pageLayout().saveOasis();
     //pageLayout.addAttribute( "style:page-usage", "all" ); FIXME
-    pageLayout.addPropertyPt( "fo:page-width", paperWidth() );
-    pageLayout.addPropertyPt( "fo:page-height", paperHeight() );
-    pageLayout.addProperty( "style:print-orientation", orientation() == KoPageFormat::Landscape ? "landscape" : "portrait" );
-    pageLayout.addPropertyPt( "fo:margin-left", leftBorder() );
-    pageLayout.addPropertyPt( "fo:margin-top", topBorder() );
-    pageLayout.addPropertyPt( "fo:margin-right", rightBorder() );
-    pageLayout.addPropertyPt( "fo:margin-bottom", bottomBorder() );
     //necessary for print setup
-    m_pSheet->saveOasisPrintStyleLayout( pageLayout );
+    m_pSheet->saveOdfPrintStyleLayout( pageLayout );
+    // this is called from Sheet::saveOdfSheetStyleName for writing the SytleMaster so 
+    // the style has to be in the styles.xml file and only there
+    pageLayout.setAutoStyleInStylesDotXml( true );
 
     return mainStyles.lookup( pageLayout, "pm" );
 }
@@ -170,28 +151,6 @@ QRect SheetPrint::cellsPrintRange()
 {
     // Find maximum right/bottom cell with content
     QRect cell_range( m_pSheet->usedArea() );
-
-#if 0 // KSPREAD_KOPART_EMBEDDING
-    // Now look at the children
-    Q3PtrListIterator<KoDocumentChild> cit( m_pDoc->children() );
-    double dummy;
-    int i;
-    for( ; cit.current(); ++cit )
-    {
-        //QRect, because KoChild doesn't use KoRect yet
-        QRect bound = cit.current()->boundingRect();
-
-        i = m_pSheet->leftColumn( bound.right(), dummy );
-        if ( i > cell_range.right() )
-            cell_range.setRight( i );
-
-        i = m_pSheet->topRow( bound.bottom(), dummy );
-        if ( i > cell_range.bottom() )
-            cell_range.setBottom( i );
-    }
-    cell_range = cell_range.intersect( m_printRange );
-#endif // KSPREAD_KOPART_EMBEDDING
-
     return cell_range;
 }
 
@@ -237,46 +196,24 @@ bool SheetPrint::pageNeedsPrinting( QRect& page_range )
 	    }
                // filled = true;
 
-#if 0 // KSPREAD_KOPART_EMBEDDING
-    //Page empty, but maybe children on it?
-
-        QRect intView = QRect( QPoint( m_zoomHandler->documentToViewX( m_pSheet->columnPosition( page_range.left() ) ),
-                                       m_zoomHandler->documentToViewY( m_pSheet->rowPosition( page_range.top() ) ) ),
-                               QPoint( m_zoomHandler->documentToViewX( m_pSheet->columnPosition( page_range.right() ) +
-                                                        m_pSheet->columnFormat( page_range.right() )->width() ),
-                                       m_zoomHandler->documentToViewY( m_pSheet->rowPosition( page_range.bottom() ) +
-                                                        m_pSheet->rowFormat( page_range.bottom() )->height() ) ) );
-
-        Q3PtrListIterator<KoDocumentChild> it( m_pDoc->children() );
-        for( ;it.current(); ++it )
-        {
-            QRect bound = it.current()->boundingRect();
-            if ( bound.intersects( intView ) )
-	    {
-                return true;
-	    }
-		//filled = true;
-        }
-#endif // KSPREAD_KOPART_EMBEDDING
-
     //Page has no visible content on it, so we don't need to paint it
     return false;
 }
 
-bool SheetPrint::print( QPainter &painter, KPrinter *_printer )
+bool SheetPrint::print( QPainter &painter, QPrinter *_printer )
 {
     kDebug(36001)<<"PRINTING ....";
 
     // Override the current grid pen setting, when set to disable
     QPen gridPen;
     bool oldShowGrid = m_pSheet->getShowGrid();
-    m_pSheet->setShowGrid( m_bPrintGrid );
-    if ( !m_bPrintGrid )
+    m_pSheet->setShowGrid(m_settings->printGrid());
+    if (!m_settings->printGrid())
     {
-        gridPen = QPen( m_pDoc->gridColor(), 1, Qt::SolidLine );
+        gridPen = QPen( m_pSheet->map()->settings()->gridColor(), 1, Qt::SolidLine );
         QPen nopen;
         nopen.setStyle( Qt::NoPen );
-        m_pDoc->setGridColor( Qt::white );
+        m_pSheet->map()->settings()->setGridColor( Qt::white );
     }
 
     //Update m_dPrintRepeatColumnsWidth for repeated columns
@@ -334,34 +271,6 @@ bool SheetPrint::print( QPainter &painter, KPrinter *_printer )
     m_uprintPages = page_list.count();
 
 
-#if 0 // KSPREAD_KOPART_EMBEDDING
-    //Cache all object so they only need to be repainted once.
-    foreach ( EmbeddedObject* object, m_pDoc->embeddedObjects() )
-    {
-      EmbeddedObject *obj = object;
-      if ( obj->sheet() != m_pSheet ||
-           !( (( obj->getType() == OBJECT_KOFFICE_PART || obj->getType() == OBJECT_PICTURE ) && m_bPrintObjects) ||
-           ( obj->getType() == OBJECT_CHART && m_bPrintCharts ) ) )
-        continue;
-
-      QRect zoomRectOld = m_pDoc->zoomRectOld( object->geometry() );
-      QPixmap *p = new QPixmap( zoomRectOld.size() );
-      QPainter painter(p);
-      painter.fillRect( p->rect(), QColor("white") );
-      painter.translate( -zoomRectOld.x(), -zoomRectOld.y() ); //we cant to paint at (0,0)
-      bool const isSelected = object->isSelected();
-      object->setSelected( false );
-      object->draw( &painter );
-      painter.end();
-      object->setSelected( isSelected );
-
-      PrintObject *po = new PrintObject();
-      m_printObjects.append( po );
-      po->obj = object;
-      po->p = p;
-    }
-#endif // KSPREAD_KOPART_EMBEDDING
-
     if ( page_list.count() == 0 )
     {
         // nothing to print
@@ -380,9 +289,9 @@ bool SheetPrint::print( QPainter &painter, KPrinter *_printer )
         QLinkedList<QRectF>::Iterator fit = page_frame_list.begin();
         QLinkedList<QPointF>::Iterator fito = page_frame_list_offset.begin();
 
-        painter.translate(leftBorder(), topBorder());
+        painter.translate(paperLayout().left, paperLayout().top);
         painter.scale( m_zoomHandler->zoomedResolutionX(), m_zoomHandler->zoomedResolutionY() );
-        painter.setClipRect(0, 0, (int)paperWidth(), (int)paperHeight());
+        painter.setClipRect(0, 0, (int)paperLayout().width, (int)paperLayout().height);
 
         for( ; it != page_list.end(); ++it, ++fit, ++fito, ++pageNo )
         {
@@ -396,26 +305,23 @@ bool SheetPrint::print( QPainter &painter, KPrinter *_printer )
         }
     }
 
-    if ( !m_bPrintGrid )
+    if (!m_settings->printGrid())
     {
         // Restore the grid pen
-        m_pDoc->setGridColor( gridPen.color() );
+        m_pSheet->map()->settings()->setGridColor( gridPen.color() );
     }
     m_pSheet->setShowGrid( oldShowGrid );
 
+#if 0 // KSPREAD_KOPART_EMBEDDING
     QList<PrintObject *>::iterator it;
     for ( it = m_printObjects.begin(); it != m_printObjects.end(); ++it )
       delete (*it)->p;
     m_printObjects.clear();
+#endif
 
     return ( page_list.count() > 0 );
 }
 
-float SheetPrint::prinsheetHeight() const
-{
-    return m_paperHeight - m_topBorder - m_bottomBorder;
-}
-
 void SheetPrint::printPage( QPainter &_painter, const QRect& page_range,
                                    const QRectF& view, const QPointF _childOffset )
 {
@@ -513,58 +419,9 @@ void SheetPrint::printRect( QPainter& painter, const QPointF& topLeft,
     m_pSheetView->setPaintCellRange( cellRange );
     m_pSheetView->paintCells( painter.device(), painter, paintRect, topLeft );
 
-#if 0 // KSPREAD_KOPART_EMBEDDING
-    //
-    // Draw the children
-    //
-    QRect zoomedView = m_zoomHandler->documentToView( view ).toRect();
-    //QPtrListIterator<KoDocumentChild> it( m_pDoc->children() );
-    //QPtrListIterator<EmbeddedObject> itObject( m_pDoc->embeddedObjects() );
-
-    QList<PrintObject *>::iterator itObject;
-    for ( itObject = m_printObjects.begin(); itObject != m_printObjects.end(); ++itObject ) {
-          EmbeddedObject *obj = (*itObject)->obj;
-//        QString tmp=QString("Testing child %1/%2 %3/%4 against view %5/%6 %7/%8")
-//        .arg(it.current()->contentRect().left())
-//        .arg(it.current()->contentRect().top())
-//        .arg(it.current()->contentRect().right())
-//        .arg(it.current()->contentRect().bottom())
-//        .arg(view.left()).arg(view.top()).arg(zoomedView.right()).arg(zoomedView.bottom());
-//        kDebug(36001)<<tmp<<" offset"<<_childOffset.x()<<"/"<<_childOffset.y();
-
-          QRectF const bound = obj->geometry();
-          QRect zoomedBound = m_pDoc->zoomRectOld( QRectF(bound.left(), bound.top(),
-              bound.width(),
-              bound.height() ) );
-#if 1
-//         kDebug(36001)  <<"cellRange(): Bounding rect of view:" << view
-//             << endl;
-//         kDebug(36001)  <<"cellRange(): Bounding rect of zoomed view:"
-//             << zoomedView << endl;
-//         kDebug(36001)  <<"cellRange(): Bounding rect of child:" << bound
-//             << endl;
-//         kDebug(36001)  <<"cellRange(): Bounding rect of zoomed child:"
-//             << zoomedBound << endl;
-#endif
-    if ( obj->sheet() == m_pSheet  && zoomedBound.intersects( zoomedView ) )
-    {
-            painter.save();
-
-            painter.translate( -zoomedView.left() + m_zoomHandler->documentToViewX( topLeft.x() ),
-                                -zoomedView.top() + m_zoomHandler->documentToViewY( topLeft.y() ) );
-
-            //obj->draw( &painter );
-            painter.drawPixmap( m_pDoc->zoomRectOld( obj->geometry() ).topLeft(), *(*itObject)->p ); //draw the cached object
-
-            //painter.fillRect(zoomedBound, QBrush("red")); //for debug purpose
-            painter.restore();
-        }
-    }
-#endif // KSPREAD_KOPART_EMBEDDING
-
     //Don't let obscuring cells and children overpaint this area
-    clipRegion -= QRegion((int)(leftBorder() + topLeft.x()),
-                          (int)(topBorder() + topLeft.y()),
+    clipRegion -= QRegion((int)(paperLayout().left + topLeft.x()),
+                          (int)(paperLayout().top + topLeft.y()),
                           (int)xpos,
                           (int)ypos);
 //     painter.setClipRegion( clipRegion );
@@ -583,38 +440,38 @@ void SheetPrint::printHeaderFooter( QPainter &painter, int pageNo )
     // print head line left
     w = fm.width( headLeft( pageNo, m_pSheet->sheetName() ) );
     if ( w > 0 )
-        painter.drawText((int)leftBorder(), (int)headFootDistance,
+        painter.drawText((int)paperLayout().left, (int)headFootDistance,
                          headLeft(pageNo, m_pSheet->sheetName()));
     // print head line middle
     w = fm.width( headMid( pageNo, m_pSheet->sheetName() ) );
     if ( w > 0 )
-        painter.drawText((int)(leftBorder() + ( prinsheetWidth() - w ) / 2.0),
+        painter.drawText((int)(paperLayout().left + ( m_settings->printWidth() - w ) / 2.0),
                          (int)headFootDistance,
                          headMid( pageNo, m_pSheet->sheetName() ) );
     // print head line right
     w = fm.width( headRight( pageNo, m_pSheet->sheetName() ) );
     if ( w > 0 )
-        painter.drawText((int)(leftBorder() + prinsheetWidth() - w),
+        painter.drawText((int)(paperLayout().left + m_settings->printWidth() - w),
                          (int)headFootDistance,
                          headRight( pageNo, m_pSheet->sheetName() ) );
 
     // print foot line left
     w = fm.width( footLeft( pageNo, m_pSheet->sheetName() ) );
     if ( w > 0 )
-        painter.drawText((int)leftBorder(),
-                         (int)(paperHeight() - headFootDistance),
+        painter.drawText((int)paperLayout().left,
+                         (int)(paperLayout().height - headFootDistance),
                          footLeft( pageNo, m_pSheet->sheetName() ) );
     // print foot line middle
     w = fm.width( footMid( pageNo, m_pSheet->sheetName() ) );
     if ( w > 0 )
-        painter.drawText((int)(leftBorder() + (prinsheetWidth() - w) / 2.0),
-                         (int)(paperHeight() - headFootDistance),
+        painter.drawText((int)(paperLayout().left + (m_settings->printWidth() - w) / 2.0),
+                         (int)(paperLayout().height - headFootDistance),
                          footMid( pageNo, m_pSheet->sheetName() ) );
     // print foot line right
     w = fm.width( footRight( pageNo, m_pSheet->sheetName() ) );
     if ( w > 0 )
-        painter.drawText((int)(leftBorder() + prinsheetWidth() - w),
-                         (int)(paperHeight() - headFootDistance),
+        painter.drawText((int)(paperLayout().left + m_settings->printWidth() - w),
+                         (int)(paperLayout().height - headFootDistance),
                          footRight( pageNo, m_pSheet->sheetName() ) );
 }
 
@@ -693,7 +550,7 @@ void SheetPrint::updateNewPageX( int _column )
 
         while ( ( col <= _column ) && ( col < m_printRange.right() ) )
         {
-            if ( x > prinsheetWidth() ) //end of page?
+            if ( x > m_settings->printWidth() ) //end of page?
             {
                 //We found a new page, so add it to the list
                 m_lnewPageListX.append( PrintNewPageEntry( col ) );
@@ -812,7 +669,7 @@ void SheetPrint::updateNewPageY( int _row )
 
         while ( ( row <= _row ) && ( row < m_printRange.bottom() ) )
         {
-            if ( y > prinsheetHeight() )
+            if ( y > m_settings->printHeight() )
             {
                 //We found a new page, so add it to the list
                 m_lnewPageListY.append( PrintNewPageEntry( row ) );
@@ -922,23 +779,6 @@ void SheetPrint::updateNewPageListY( int _row )
     m_maxCheckedNewPageY = _row;
 }
 
-void SheetPrint::definePrintRange( Selection* selection )
-{
-    if ( !selection->isSingular() )
-    {
-        QUndoCommand* command = new DefinePrintRangeCommand( m_pSheet );
-        m_pDoc->addCommand( command );
-        setPrintRange( selection->lastRange() );
-    }
-}
-
-void SheetPrint::resetPrintRange ()
-{
-    QUndoCommand* command = new DefinePrintRangeCommand( m_pSheet );
-    m_pDoc->addCommand( command );
-    setPrintRange( QRect( QPoint( 1, 1 ), QPoint( KS_colMax, KS_rowMax ) ) );
-}
-
 void SheetPrint::replaceHeadFootLineMacro ( QString &_text, const QString &_search, const QString &_replace )
 {
     if ( _search != _replace )
@@ -1028,8 +868,7 @@ void SheetPrint::setPaperOrientation( KoPageFormat::Orientation _orient )
     if ( m_pSheet->isProtected() )
         NO_MODIFICATION_POSSIBLE;
 
-    m_orientation = _orient;
-    calcPaperSize();
+    m_settings->setPageOrientation(_orient);
     updatePrintRepeatColumnsWidth();
     updatePrintRepeatRowsHeight();
     updateNewPageListX( m_printRange.left() ); //Reset the list
@@ -1040,106 +879,9 @@ void SheetPrint::setPaperOrientation( KoPageFormat::Orientation _orient )
 }
 
 
-KoPageLayout SheetPrint::paperLayout() const
+const KoPageLayout &SheetPrint::paperLayout() const
 {
-    KoPageLayout pl;
-    pl.format = m_paperFormat;
-    pl.orientation = m_orientation;
-    pl.width  =  m_paperWidth;
-    pl.height =  m_paperHeight;
-    pl.left   =  m_leftBorder;
-    pl.right  =  m_rightBorder;
-    pl.top    =  m_topBorder;
-    pl.bottom =  m_bottomBorder;
-    return pl;
-}
-
-
-void SheetPrint::setPaperLayout( float _leftBorder, float _topBorder,
-                                        float _rightBorder, float _bottomBorder,
-                                        KoPageFormat::Format _paper,
-                                        KoPageFormat::Orientation _orientation )
-{
-    if ( m_pSheet->isProtected() )
-        NO_MODIFICATION_POSSIBLE;
-
-    m_leftBorder   = _leftBorder;
-    m_rightBorder  = _rightBorder;
-    m_topBorder    = _topBorder;
-    m_bottomBorder = _bottomBorder;
-    m_paperFormat  = _paper;
-
-    setPaperOrientation( _orientation ); //calcPaperSize() is done here already
-
-//    QPtrListIterator<KoView> it( views() );
-//    for( ;it.current(); ++it )
-//    {
-//        View *v = static_cast<View *>( it.current() );
-          // We need to trigger the appropriate repaintings in the cells near the
-          // border of the page. The easiest way for this is to turn the borders
-          // off and on (or on and off if they were off).
-//        bool bBorderWasShown = v->activeSheet()->isShowPageBorders();
-//        v->activeSheet()->setShowPageBorders( !bBorderWasShown );
-//        v->activeSheet()->setShowPageBorders( bBorderWasShown );
-//    }
-
-    m_pDoc->setModified( true );
-}
-
-void SheetPrint::setPaperLayout( float _leftBorder, float _topBorder,
-                                        float _rightBorder, float _bottomBorder,
-                                        const QString& _paper,
-                                        const QString& _orientation )
-{
-    if ( m_pSheet->isProtected() )
-        NO_MODIFICATION_POSSIBLE;
-
-    KoPageFormat::Format f = paperFormat();
-    KoPageFormat::Orientation newOrientation = orientation();
-
-    if ( _orientation == "Portrait" )
-        newOrientation = KoPageFormat::Portrait;
-    else if ( _orientation == "Landscape" )
-        newOrientation = KoPageFormat::Landscape;
-
-
-    QString paper( _paper );
-    if ( paper[0].isDigit() ) // Custom format
-    {
-        const int i = paper.indexOf( 'x' );
-        if ( i < 0 )
-        {
-            // We have nothing useful, so assume ISO A4
-            f = KoPageFormat::IsoA4Size;
-        }
-        else
-        {
-            f = KoPageFormat::CustomSize;
-            m_paperWidth  = paper.left(i).toFloat();
-            m_paperHeight = paper.mid(i+1).toFloat();
-            if ( m_paperWidth < 10.0 )
-                m_paperWidth = KoPageFormat::width( KoPageFormat::IsoA4Size, newOrientation );
-            if ( m_paperHeight < 10.0 )
-                m_paperHeight = KoPageFormat::height( KoPageFormat::IsoA4Size, newOrientation );
-        }
-    }
-    else
-    {
-        f = KoPageFormat::formatFromString( paper );
-        if ( f == KoPageFormat::CustomSize )
-            // We have no idea about height or width, therefore assume ISO A4
-            f = KoPageFormat::IsoA4Size;
-    }
-    setPaperLayout( _leftBorder, _topBorder, _rightBorder, _bottomBorder, f, newOrientation );
-}
-
-void SheetPrint::calcPaperSize()
-{
-    if ( m_paperFormat != KoPageFormat::CustomSize )
-    {
-        m_paperWidth = KoPageFormat::width( m_paperFormat, m_orientation );
-        m_paperHeight = KoPageFormat::height( m_paperFormat, m_orientation );
-    }
+    return m_settings->pageLayout();
 }
 
 QList<PrintNewPageEntry>::iterator SheetPrint::findNewPageColumn( int col )
@@ -1167,25 +909,25 @@ QList<PrintNewPageEntry>::iterator SheetPrint::findNewPageRow( int row )
 }
 
 
-QString SheetPrint::paperFormatString()const
+QString SheetPrint::paperFormatString() const
 {
-    if ( m_paperFormat == KoPageFormat::CustomSize )
+    if ( paperLayout().format == KoPageFormat::CustomSize )
     {
         QString tmp;
-        tmp.sprintf( "%fx%f", m_paperWidth, m_paperHeight );
+        tmp.sprintf( "%fx%f", paperLayout().width, paperLayout().height );
         return tmp;
     }
 
-    return KoPageFormat::formatString( m_paperFormat );
+    return KoPageFormat::formatString( paperLayout().format );
 }
 
 const char* SheetPrint::orientationString() const
 {
-    switch( m_orientation )
+    switch( paperLayout().orientation )
     {
-    case KPrinter::Portrait:
+    case QPrinter::Portrait:
         return "Portrait";
-    case KPrinter::Landscape:
+    case QPrinter::Landscape:
         return "Landscape";
     }
 
@@ -1453,59 +1195,6 @@ void SheetPrint::calculateZoomForPageLimitY()
         m_zoomHandler->setZoom( origZoom );
 }
 
-void SheetPrint::setPrintGrid( bool _printGrid )
-{
-   if ( m_bPrintGrid == _printGrid )
-        return;
-
-    m_bPrintGrid = _printGrid;
-    m_pDoc->setModified( true );
-}
-
-void SheetPrint::setPrintObjects( bool _printObjects )
-{
-  if ( m_bPrintObjects == _printObjects )
-    return;
-
-  m_bPrintObjects = _printObjects;
-  m_pDoc->setModified( true );
-}
-
-void SheetPrint::setPrintCharts( bool _printCharts )
-{
-  if ( m_bPrintCharts == _printCharts )
-    return;
-
-  m_bPrintCharts = _printCharts;
-  m_pDoc->setModified( true );
-}
-
-void SheetPrint::setPrintGraphics( bool _printGraphics )
-{
-  if ( m_bPrintGraphics == _printGraphics )
-    return;
-
-  m_bPrintGraphics = _printGraphics;
-  m_pDoc->setModified( true );
-}
-
-void SheetPrint::setPrintCommentIndicator( bool _printCommentIndicator )
-{
-    if ( m_bPrintCommentIndicator == _printCommentIndicator )
-        return;
-
-    m_bPrintCommentIndicator = _printCommentIndicator;
-    m_pDoc->setModified( true );
-}
-
-void SheetPrint::setPrintFormulaIndicator( bool _printFormulaIndicator )
-{
-    if( m_bPrintFormulaIndicator == _printFormulaIndicator )
-        return;
-
-    m_bPrintFormulaIndicator = _printFormulaIndicator;
-    m_pDoc->setModified( true );
-}
 void SheetPrint::updatePrintRepeatColumnsWidth()
 {
     m_dPrintRepeatColumnsWidth = 0.0;
