@@ -22,11 +22,11 @@
 
 #include <QCache>
 #include <QRegion>
-#include <QSet>
 #include <QTimer>
 
-#include "Doc.h"
 #include "Global.h"
+#include "Map.h"
+#include "OdfSavingContext.h"
 #include "RTree.h"
 #include "Style.h"
 #include "StyleManager.h"
@@ -39,7 +39,7 @@ using namespace KSpread;
 class KDE_NO_EXPORT StyleStorage::Private
 {
 public:
-    Doc* doc;
+    Map* map;
     RTree<SharedSubStyle> tree;
     QMap<int, bool> usedColumns; // FIXME Stefan: Use QList and qUpperBound() for insertion.
     QMap<int, bool> usedRows;
@@ -50,19 +50,19 @@ public:
     QRegion cachedArea;
 };
 
-StyleStorage::StyleStorage(Doc* doc)
-    : QObject(doc)
+StyleStorage::StyleStorage(Map* map)
+    : QObject(map)
     , d(new Private)
 {
-    d->doc = doc;
+    d->map = map;
     d->cache.setMaxCost( g_maximumCachedStyles );
 }
 
 StyleStorage::StyleStorage(const StyleStorage& other)
-    : QObject(other.d->doc)
+    : QObject(other.d->map)
     , d(new Private)
 {
-    d->doc = other.d->doc;
+    d->map = other.d->map;
     d->tree = other.d->tree;
     d->usedColumns = other.d->usedColumns;
     d->usedRows = other.d->usedRows;
@@ -135,9 +135,7 @@ QRect StyleStorage::usedArea() const
     return QRect(QPoint(1, 1), d->usedArea.boundingRect().bottomRight());
 }
 
-void StyleStorage::saveOdfCreateDefaultStyles(int& maxCols, int& maxRows,
-                                              QMap<int, Style>& columnDefaultStyles,
-                                              QMap<int, Style>& rowDefaultStyles) const
+void StyleStorage::saveOdfCreateDefaultStyles(int& maxCols, int& maxRows, OdfSavingContext& tableContext) const
 {
 #if 0 // TODO
     // If we have both, column and row styles, we can take the short route.
@@ -146,12 +144,12 @@ void StyleStorage::saveOdfCreateDefaultStyles(int& maxCols, int& maxRows,
         for (int i = 0; i < d->usedColumns.count(); ++i)
         {
             const int col = d->usedColumns[i];
-            columnDefaultStyles[col].insertSubStyle(contains(QRect(col, 1, 1, KS_rowMax)));
+            tableContext.columnDefaultStyles[col].insertSubStyle(contains(QRect(col, 1, 1, KS_rowMax)));
         }
         for (int i = 0; i < d->usedColumns.count(); ++i)
         {
             const int row = d->usedRow[i];
-            rowDefaultStyles[row].insertSubStyle(contains(QRect(1, row, KS_colMax, 1)));
+            tableContext.rowDefaultStyles[row].insertSubStyle(contains(QRect(1, row, KS_colMax, 1)));
         }
         return;
     }
@@ -178,9 +176,9 @@ void StyleStorage::saveOdfCreateDefaultStyles(int& maxCols, int& maxRows,
             for (int col = rect.left(); col <= rect.right(); ++col)
             {
                 if (pairs[i].second.data()->type() == Style::DefaultStyleKey)
-                    columnDefaultStyles.remove(col);
+                    tableContext.columnDefaultStyles.remove(col);
                 else
-                    columnDefaultStyles[col].insertSubStyle(pairs[i].second);
+                    tableContext.columnDefaultStyles[col].insertSubStyle(pairs[i].second);
             }
         }
         // row default cell styles
@@ -189,9 +187,9 @@ void StyleStorage::saveOdfCreateDefaultStyles(int& maxCols, int& maxRows,
             for (int row = rect.top(); row <= rect.bottom(); ++row)
             {
                 if (pairs[i].second.data()->type() == Style::DefaultStyleKey)
-                    rowDefaultStyles.remove(row);
+                    tableContext.rowDefaultStyles.remove(row);
                 else
-                    rowDefaultStyles[row].insertSubStyle(pairs[i].second);
+                    tableContext.rowDefaultStyles[row].insertSubStyle(pairs[i].second);
             }
         }
     }
@@ -509,7 +507,7 @@ void StyleStorage::garbageCollection()
     const int currentZIndex = d->possibleGarbage.constBegin().key();
     const QPair<QRectF, SharedSubStyle> currentPair = d->possibleGarbage.take(currentZIndex);
 
-    // check wether the named style still exists
+    // check whether the named style still exists
     if ( currentPair.second->type() == Style::NamedStyleKey &&
          !styleManager()->style( static_cast<const NamedStyle*>(currentPair.second.data())->name ) )
     {
@@ -529,7 +527,7 @@ void StyleStorage::garbageCollection()
     int zIndex = pairs.constBegin().key();
     SharedSubStylePair pair = pairs[zIndex];
 
-    // check wether the default style is placed first
+    // check whether the default style is placed first
     if ( zIndex == currentZIndex &&
          currentPair.second->type() == Style::DefaultStyleKey &&
          pair.second->type() == Style::DefaultStyleKey &&
@@ -544,7 +542,7 @@ void StyleStorage::garbageCollection()
     }
 
     // special handling for indentation:
-    // check wether the default indentation is placed first
+    // check whether the default indentation is placed first
     if ( zIndex == currentZIndex &&
          currentPair.second->type() == Style::Indentation &&
          static_cast<const SubStyleOne<Style::Indentation, int>*>(currentPair.second.data())->value1 == 0 &&
@@ -559,7 +557,7 @@ void StyleStorage::garbageCollection()
     }
 
     // special handling for precision:
-    // check wether the storage default precision is placed first
+    // check whether the storage default precision is placed first
     if ( zIndex == currentZIndex &&
          currentPair.second->type() == Style::Precision &&
          static_cast<const SubStyleOne<Style::Precision, int>*>(currentPair.second.data())->value1 == 0 &&
@@ -645,7 +643,7 @@ void StyleStorage::garbageCollection()
 
 void StyleStorage::regionChanged( const QRect& rect )
 {
-    if ( d->doc->isLoading() )
+    if ( d->map->isLoading() )
          return;
     // mark the possible garbage
     // NOTE Stefan: The map may contain multiple indices. The already existing possible garbage has
@@ -762,7 +760,7 @@ Style StyleStorage::composeStyle( const QList<SharedSubStyle>& subStyles ) const
 
 StyleManager* StyleStorage::styleManager() const
 {
-    return d->doc->styleManager();
+    return d->map->styleManager();
 }
 
 #include "StyleStorage.moc"
