@@ -19,8 +19,9 @@
 */
 
 #include "Value.h"
-#include "Doc.h"
+#include "CalculationSettings.h"
 #include "CellStorage.h"
+#include "ValueStorage.h"
 
 #include <kdebug.h>
 #include <klocale.h>
@@ -256,6 +257,16 @@ Value::Value( double f )
   d->format = fmt_Number;
 }
 
+// create a floating-point value
+Value::Value( long double f )
+  : d( Private::null() )
+{
+  d->type = Float;
+  d->f = Number (f);
+  d->format = fmt_Number;
+}
+
+
 #ifdef KSPREAD_HIGH_PRECISION_SUPPORT
 // create a floating-point value
 Value::Value( Number f )
@@ -295,10 +306,10 @@ Value::Value (const char *s)
 }
 
 // create a floating-point value from date/time
-Value::Value( const QDateTime& dt, const Doc* doc )
+Value::Value(const QDateTime& dt, const CalculationSettings* settings)
     : d( Private::null() )
 {
-    const QDate refDate( doc->referenceDate() );
+    const QDate refDate(settings->referenceDate());
     const QTime refTime( 0, 0 );  // reference time is midnight
 
     d->type = Float;
@@ -308,10 +319,10 @@ Value::Value( const QDateTime& dt, const Doc* doc )
 }
 
 // create a floating-point value from time
-Value::Value( const QTime& time, const Doc* doc )
+Value::Value(const QTime& time, const CalculationSettings* settings)
     : d( Private::null() )
 {
-    Q_UNUSED( doc );
+    Q_UNUSED(settings);
     const QTime refTime( 0, 0 );  // reference time is midnight
 
     d->type = Float;
@@ -320,10 +331,10 @@ Value::Value( const QTime& time, const Doc* doc )
 }
 
 // create a floating-point value from date
-Value::Value( const QDate& date, const Doc* doc )
+Value::Value(const QDate& date, const CalculationSettings* settings)
     : d( Private::null() )
 {
-  const QDate refDate( doc->referenceDate() );
+    const QDate refDate(settings->referenceDate());
 
   d->type = Integer;
   d->i = refDate.daysTo( date );
@@ -376,7 +387,7 @@ Number Value::asFloat() const
     if ( type() == Float )
         result = d->f;
     else if ( type() == Integer )
-        result = static_cast<int>(d->i);
+        result = static_cast<Number>(d->i);
     else if ( type() == Complex )
         result = d->pc->real();
     return result;
@@ -391,7 +402,7 @@ complex<Number> Value::asComplex() const
     else if ( type() == Float )
         result = d->f;
     else if ( type() == Integer )
-        result = static_cast<int>(d->i);
+        result = static_cast<Number>(d->i);
     return result;
 }
 
@@ -407,6 +418,43 @@ QString Value::asString() const
   return result;
 }
 
+// get the value as QVariant
+QVariant Value::asVariant() const
+{
+  QVariant result;
+
+  switch ( d->type )
+    {
+        case Value::Empty:
+        default:
+            result = 0;
+            break;
+        case Value::Boolean:
+            result = d->b;
+            break;
+        case Value::Integer:
+            result = d->i;
+            break;
+        case Value::Float:
+            result = (double) numToDouble (d->f);
+            break;
+        case Value::Complex:
+            // FIXME: add support for complex numbers
+            // pc = new complex<Number>( *o.pc );
+            break;
+        case Value::String:
+        case Value::Error:
+            result = d->ps;
+            break;
+        case Value::Array:
+            // FIXME: not supported yet
+            //result = ValueArray( d->pa );
+            break;
+    }
+
+  return result;
+}
+
 // set error message
 void Value::setError( const QString& msg )
 {
@@ -428,9 +476,9 @@ QString Value::errorMessage() const
 }
 
 // get the value as date/time
-QDateTime Value::asDateTime( const Doc* doc ) const
+QDateTime Value::asDateTime(const CalculationSettings* settings) const
 {
-  QDateTime datetime( doc->referenceDate(), QTime(), Qt::UTC );
+    QDateTime datetime(settings->referenceDate(), QTime(), Qt::UTC);
 
   const int days = asInteger();
   const int msecs = qRound( ( numToDouble (asFloat() - days) ) * 86400000.0 ); // 24*60*60*1000
@@ -441,9 +489,9 @@ QDateTime Value::asDateTime( const Doc* doc ) const
 }
 
 // get the value as date
-QDate Value::asDate( const Doc* doc ) const
+QDate Value::asDate(const CalculationSettings* settings) const
 {
-  QDate dt( doc->referenceDate() );
+    QDate dt(settings->referenceDate());
 
   int i = asInteger();
   dt = dt.addDays( i );
@@ -452,9 +500,9 @@ QDate Value::asDate( const Doc* doc ) const
 }
 
 // get the value as time
-QTime Value::asTime( const Doc* doc ) const
+QTime Value::asTime(const CalculationSettings* settings) const
 {
-  Q_UNUSED(doc);
+    Q_UNUSED(settings);
   QTime dt;
 
   const int days = asInteger();
@@ -822,15 +870,15 @@ QTextStream& operator<<( QTextStream& ts, Value value )
       ts << ": " << value.asInteger(); break;
 
     case Value::Float:
-      ts << ": " << numToDouble (value.asFloat()); break;
+      ts << ": " << (double) numToDouble (value.asFloat()); break;
 
     case Value::Complex:
     {
       const complex<Number> complex( value.asComplex() );
-      ts << ": " << numToDouble (complex.real());
+      ts << ": " << (double) numToDouble (complex.real());
       if ( complex.imag() >= 0.0 )
           ts << '+';
-      ts << numToDouble (complex.imag()) << 'i';
+      ts << (double) numToDouble (complex.imag()) << 'i';
       break;
     }
 
