@@ -25,12 +25,12 @@
 #include "KoCanvasResourceProvider.h"
 #include "KoShape.h"
 
-#include "koChart.h"
+#include "KoChartInterface.h"
 
 #include "Binding.h"
-#include "Canvas.h"
+#include "CanvasResources.h"
 #include "CellStorage.h"
-#include "Doc.h"
+#include "part/Doc.h" // FIXME detach from part
 #include "Region.h"
 #include "Selection.h"
 #include "Sheet.h"
@@ -63,7 +63,13 @@ ChartDatabaseSelector::~ChartDatabaseSelector()
 
 void ChartDatabaseSelector::open(KoShape* shape)
 {
-    d->shape = dynamic_cast<KoChart::ChartInterface*>( shape );
+    QObject* const object = dynamic_cast<QObject*>(shape);
+    Q_ASSERT(object);
+    if (!object) {
+        return;
+    }
+    d->shape = qobject_cast<KoChart::ChartInterface*>(object);
+    Q_ASSERT(d->shape);
 }
 
 void ChartDatabaseSelector::save()
@@ -72,7 +78,10 @@ void ChartDatabaseSelector::save()
     if (!region.isValid() || !region.isContiguous())
         return;
     Binding binding(region);
-    d->shape->setModel(binding.model());
+    d->shape->setModel(binding.model(), region.rects());
+    d->shape->setFirstRowIsLabel( d->widget.m_firstRowAsLabel->isChecked() );
+    d->shape->setFirstColumnIsLabel( d->widget.m_firstColumnAsLabel->isChecked() );
+    d->shape->setDataDirection( d->widget.m_dataInRows->isChecked() ? Qt::Horizontal : Qt::Vertical );
     region.firstSheet()->cellStorage()->setBinding(region, binding);
 }
 
@@ -85,7 +94,7 @@ void ChartDatabaseSelector::showEvent( QShowEvent* event )
 {
     Q_UNUSED( event );
     Q_ASSERT( m_resourceProvider );
-    d->selection = static_cast<Selection*>( m_resourceProvider->resource( Canvas::Selection ).value<void*>() );
+    d->selection = static_cast<Selection*>(m_resourceProvider->resource(CanvasResource::Selection ).value<void*>());
     d->widget.m_cellRegion->setText( d->selection->Region::name() );
 }
 
