@@ -24,13 +24,14 @@
 
 #include <QFontMetricsF>
 #include <QWidget>
+#include <QPen>
 
 #include <klocale.h>
 
 #include "CellStorage.h"
 #include "CellStorage_p.h"
 #include "Damages.h"
-#include "Doc.h"
+#include "Map.h"
 #include "RowColumnFormat.h"
 #include "Sheet.h"
 
@@ -40,7 +41,8 @@ using namespace KSpread;
   class ResizeColumnManipulator
 ****************************************************************************/
 
-ResizeColumnManipulator::ResizeColumnManipulator()
+ResizeColumnManipulator::ResizeColumnManipulator(QUndoCommand* parent)
+    : AbstractRegionCommand(parent)
 {
     setText(i18n("Resize Column"));
 }
@@ -68,7 +70,8 @@ bool ResizeColumnManipulator::process(Element* element)
   class ResizeRowManipulator
 ****************************************************************************/
 
-ResizeRowManipulator::ResizeRowManipulator()
+ResizeRowManipulator::ResizeRowManipulator(QUndoCommand* parent)
+    : AbstractRegionCommand(parent)
 {
     setText(i18n("Resize Row"));
 }
@@ -251,8 +254,8 @@ QString HideShowManipulator::name() const
   class AdjustColumnRowManipulator
 ****************************************************************************/
 
-AdjustColumnRowManipulator::AdjustColumnRowManipulator()
-  : AbstractRegionCommand(),
+AdjustColumnRowManipulator::AdjustColumnRowManipulator(QUndoCommand* parent)
+  : AbstractRegionCommand(parent),
     m_adjustColumn(false),
     m_adjustRow(false)
 {
@@ -561,7 +564,7 @@ QSizeF AdjustColumnRowManipulator::textSize( const QString& text, const Style& s
     {
         // Vertical text.
 
-        double width = 0.0;
+        qreal width = 0.0;
         for ( int i = 0; i < text.length(); i++ )
             width = qMax( width, fontMetrics.width( text.at( i ) ) );
 
@@ -695,7 +698,7 @@ bool InsertDeleteColumnManipulator::process( Element* element )
 bool InsertDeleteColumnManipulator::postProcessing()
 {
     const QRect rect(QPoint(boundingRect().left(), 1), QPoint(KS_colMax, KS_rowMax));
-    m_sheet->doc()->addDamage(new CellDamage(m_sheet, Region(rect, m_sheet), CellDamage::Appearance));
+    m_sheet->map()->addDamage(new CellDamage(m_sheet, Region(rect, m_sheet), CellDamage::Appearance));
     return true;
 }
 
@@ -762,6 +765,6 @@ bool InsertDeleteRowManipulator::process( Element* element )
 bool InsertDeleteRowManipulator::postProcessing()
 {
     const QRect rect(QPoint(1, boundingRect().top()), QPoint(KS_colMax, KS_rowMax));
-    m_sheet->doc()->addDamage(new CellDamage(m_sheet, Region(rect, m_sheet), CellDamage::Appearance));
+    m_sheet->map()->addDamage(new CellDamage(m_sheet, Region(rect, m_sheet), CellDamage::Appearance));
     return true;
 }
