@@ -27,67 +27,61 @@
 // Local
 #include "FormulaDialog.h"
 
-#include <qtextbrowser.h>
 #include <KTabWidget>
 #include <QApplication>
 #include <QCloseEvent>
 #include <QGridLayout>
 #include <QVBoxLayout>
 
+#include "CalculationSettings.h"
 #include "Cell.h"
-#include "Canvas.h"
 #include "Util.h"
 #include "Editors.h"
-#include "Doc.h"
 #include "Localization.h"
 #include "Map.h"
 #include "Selection.h"
 #include "Sheet.h"
-#include "View.h"
 #include "Functions.h"
 
 #include <kcombobox.h>
 #include <kdebug.h>
+#include <ktextbrowser.h>
 
 #include <knumvalidator.h>
 #include <QEvent>
-#include <q3listbox.h>
 #include <QLabel>
 #include <QPushButton>
 #include <klineedit.h>
 #include <QLayout>
+#include <QStringListModel>
+#include <QSortFilterProxyModel>
+#include <QItemSelectionModel>
 
 using namespace KSpread;
 
-FormulaDialog::FormulaDialog( View* parent, const char* name,const QString& formulaName)
+FormulaDialog::FormulaDialog(QWidget* parent, Selection* selection, CellEditor* editor, const QString& formulaName)
   : KDialog( parent )
 {
     setCaption( i18n("Function") );
-    setObjectName( name );
     setModal( true );
     setButtons( Ok|Cancel );
-  //setWFlags( Qt::WDestructiveClose );
+    //setWFlags( Qt::WDestructiveClose );
 
-    m_pView = parent;
+    m_selection = selection;
+    m_editor = editor;
     m_focus = 0;
     m_desc = 0;
 
-    Cell cell( m_pView->activeSheet(), m_pView->selection()->marker() );
-     m_oldText=cell.userInput();
+    Cell cell( m_selection->activeSheet(), m_selection->marker() );
+    m_oldText = cell.userInput();
     // Make sure that there is a cell editor running.
-    if ( !m_pView->canvasWidget()->editor() )
-    {
-        m_pView->canvasWidget()->createEditor();
-        if(cell.userInput().isEmpty())
-          m_pView->canvasWidget()->editor()->setText( "=" );
+    if(cell.userInput().isEmpty())
+        m_editor->setText( "=" );
+    else
+        if(cell.userInput().at(0)!='=')
+            m_editor->setText( '='+cell.userInput() );
         else
-          if(cell.userInput().at(0)!='=')
-            m_pView->canvasWidget()->editor()->setText( '='+cell.userInput() );
-          else
-            m_pView->canvasWidget()->editor()->setText( cell.userInput() );
-    }
-
-    Q_ASSERT( m_pView->canvasWidget()->editor() );
+            m_editor->setText( cell.userInput() );
 
     QWidget *page = new QWidget(this);
     setMainWidget( page );
@@ -97,8 +91,8 @@ FormulaDialog::FormulaDialog( View* parent, const char* name,const QString& form
     grid1->setSpacing(KDialog::spacingHint());
 
     searchFunct = new KLineEdit(page);
-    QSizePolicy sp3( QSizePolicy::Preferred, QSizePolicy::Fixed );
-    searchFunct->setSizePolicy( sp3 );
+    searchFunct->setClearButtonShown(true);
+    searchFunct->setSizePolicy( QSizePolicy( QSizePolicy::Preferred, QSizePolicy::Fixed ) );
 
     grid1->addWidget( searchFunct, 0, 0 );
 
@@ -109,11 +103,31 @@ FormulaDialog::FormulaDialog( View* parent, const char* name,const QString& form
     typeFunction->insertItems( 0, cats  );
     grid1->addWidget( typeFunction, 1, 0 );
 
-    functions = new Q3ListBox(page);
-    QSizePolicy sp1( QSizePolicy::Preferred, QSizePolicy::Expanding );
-    functions->setSizePolicy( sp1 );
+    functions = new QListView(page);
+    functions->setSizePolicy( QSizePolicy( QSizePolicy::Preferred, QSizePolicy::Expanding ) );
+    functions->setSelectionMode( QAbstractItemView::SingleSelection );
+    functions->setEditTriggers( QAbstractItemView::NoEditTriggers );
     grid1->addWidget( functions, 2, 0 );
 
+    functionsModel = new QStringListModel(this);
+    proxyModel = new QSortFilterProxyModel(functions);
+    proxyModel->setSourceModel( functionsModel );
+    proxyModel->setFilterKeyColumn( 0 );
+    proxyModel->setFilterCaseSensitivity( Qt::CaseInsensitive );
+    functions->setModel(proxyModel);
+
+    QItemSelectionModel* selectionmodel = new QItemSelectionModel(proxyModel, this);
+    functions->setSelectionModel(selectionmodel);
+    connect(selectionmodel, SIGNAL(currentRowChanged(QModelIndex,QModelIndex)),
+            this, SLOT(slotSelected()));
+    // When items are activated on single click, also change the help page on mouse-over, otherwise there is no (easy) way to get
+    // the help without inserting the function
+    if (functions->style()->styleHint(QStyle::SH_ItemView_ActivateItemOnSingleClick, 0, functions)) {
+        connect(functions, SIGNAL(entered(QModelIndex)), this, SLOT(slotIndexSelected(QModelIndex)));
+        functions->setMouseTracking(true);
+    }
+    //connect(proxyModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)), this, SLOT(slotDataChanged(QModelIndex,QModelIndex)));
+
     selectFunction = new QPushButton( page );
     selectFunction->setToolTip( i18n("Insert function") );
     selectFunction->setIcon( BarIcon( "go-down", KIconLoader::SizeSmall ) );
@@ -123,11 +137,10 @@ FormulaDialog::FormulaDialog( View* parent, const char* name,const QString& form
     grid1->addWidget( result, 4, 0, 1, -1 );
 
     m_tabwidget = new KTabWidget( page );
-    QSizePolicy sp2( QSizePolicy::Expanding, QSizePolicy::Expanding );
-    m_tabwidget->setSizePolicy( sp2 );
+    m_tabwidget->setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding ) );
     grid1->addWidget( m_tabwidget, 0, 1, 4, 1 );
 
-    m_browser = new QTextBrowser( m_tabwidget );
+    m_browser = new KTextBrowser( m_tabwidget, true );
     m_browser->document()->setDefaultStyleSheet("h1 { font-size:x-large; } h2 { font-size:large; } h3 { font-size:medium; }");
     m_browser->setMinimumWidth( 300 );
 
@@ -184,67 +197,83 @@ FormulaDialog::FormulaDialog( View* parent, const char* name,const QString& form
     connect( this, SIGNAL( okClicked() ), this, SLOT( slotOk() ) );
     connect( typeFunction, SIGNAL( activated(const QString &) ),
              this, SLOT( slotActivated(const QString &) ) );
+/*
     connect( functions, SIGNAL( highlighted(const QString &) ),
              this, SLOT( slotSelected(const QString &) ) );
     connect( functions, SIGNAL( selected(const QString &) ),
              this, SLOT( slotSelected(const QString &) ) );
-    connect( functions, SIGNAL( doubleClicked(Q3ListBoxItem * ) ),
-             this ,SLOT( slotDoubleClicked(Q3ListBoxItem *) ) );
+*/
+    connect( functions, SIGNAL( activated(QModelIndex) ),
+             this ,SLOT( slotDoubleClicked(QModelIndex) ) );
 
     slotActivated(i18n("All"));
 
     connect( selectFunction, SIGNAL(clicked()),
-	     this,SLOT(slotSelectButton()));
+             this,SLOT(slotSelectButton()));
 
     connect( firstElement,SIGNAL(textChanged ( const QString & )),
-	     this,SLOT(slotChangeText(const QString &)));
+             this,SLOT(slotChangeText(const QString &)));
     connect( secondElement,SIGNAL(textChanged ( const QString & )),
-	     this,SLOT(slotChangeText(const QString &)));
+             this,SLOT(slotChangeText(const QString &)));
     connect( thirdElement,SIGNAL(textChanged ( const QString & )),
-	     this,SLOT(slotChangeText(const QString &)));
+             this,SLOT(slotChangeText(const QString &)));
     connect( fourElement,SIGNAL(textChanged ( const QString & )),
-	     this,SLOT(slotChangeText(const QString &)));
+             this,SLOT(slotChangeText(const QString &)));
     connect( fiveElement,SIGNAL(textChanged ( const QString & )),
-	     this,SLOT(slotChangeText(const QString &)));
+             this,SLOT(slotChangeText(const QString &)));
 
-    connect( m_pView->choice(), SIGNAL(changed(const Region&)),
+    connect( m_selection, SIGNAL(changed(const Region&)),
              this, SLOT(slotSelectionChanged()));
 
-    connect( m_browser, SIGNAL( linkClicked( const QString& ) ),
+    connect( m_browser, SIGNAL( urlClick( const QString& ) ),
              this, SLOT( slotShowFunction( const QString& ) ) );
 
     // Save the name of the active sheet.
-    m_sheetName = m_pView->activeSheet()->sheetName();
+    m_sheetName = m_selection->activeSheet()->sheetName();
     // Save the cells current text.
-    QString tmp_oldText = m_pView->canvasWidget()->editor()->text();
+    QString tmp_oldText = m_editor->text();
     // Position of the cell.
-    m_column = m_pView->selection()->marker().x();
-    m_row = m_pView->selection()->marker().y();
+    m_column = m_selection->marker().x();
+    m_row = m_selection->marker().y();
 
     if( tmp_oldText.isEmpty() )
         result->setText("=");
     else
     {
         if( tmp_oldText.at(0)!='=')
-	    result->setText( '=' + tmp_oldText );
+            result->setText( '=' + tmp_oldText );
         else
-	    result->setText( tmp_oldText );
+            result->setText( tmp_oldText );
     }
 
     // Allow the user to select cells on the spreadsheet.
-    m_pView->canvasWidget()->startChoose();
+    m_selection->startReferenceSelection();
 
     qApp->installEventFilter( this );
 
     // Was a function name passed along with the constructor ? Then activate it.
     if( !formulaName.isEmpty() )
     {
-        functions->setCurrentItem( functions->index( functions->findItem( formulaName ) ) );
-        slotDoubleClicked( functions->findItem( formulaName ) );
+        kDebug()<<"formulaName="<<formulaName;
+#if 0
+        QList<QListWidgetItem *> items = functions->findItems( formulaName, Qt::MatchFixedString );
+        if( items.count() > 0 ) {
+            functions->setCurrentItem( items[0] );
+            slotDoubleClicked( items[0] );
+        }
+#else
+        int row = functionsModel->stringList().indexOf(formulaName);
+        const QModelIndex sourcemodelindex = functionsModel->index(row, 0);
+        const QModelIndex proxymodelindex = proxyModel->mapFromSource(sourcemodelindex);
+        if( proxymodelindex.isValid() ) {
+            functions->setCurrentIndex(proxymodelindex);
+            slotDoubleClicked(proxymodelindex);
+        }
+#endif
     }
     else
     {
-	// Set keyboard focus to allow selection of a formula.
+        // Set keyboard focus to allow selection of a formula.
         searchFunct->setFocus();
     }
 
@@ -252,13 +281,13 @@ FormulaDialog::FormulaDialog( View* parent, const char* name,const QString& form
     searchFunct->setCompletionMode( KGlobalSettings::CompletionAuto );
     searchFunct->setCompletionObject( &listFunct, true );
 
-    if( functions->currentItem() == -1 )
+    if( functions->currentIndex().isValid() )
         selectFunction->setEnabled( false );
 
     connect( searchFunct, SIGNAL( textChanged( const QString & ) ),
-	     this, SLOT( slotSearchText(const QString &) ) );
+             this, SLOT( slotSearchText(const QString &) ) );
     connect( searchFunct, SIGNAL( returnPressed() ),
-	     this, SLOT( slotPressReturn() ) );
+             this, SLOT( slotPressReturn() ) );
 
     resize( QSize(660, 520).expandedTo(minimumSizeHint()) );
 }
@@ -281,9 +310,9 @@ void FormulaDialog::slotPressReturn()
 
 void FormulaDialog::slotSearchText(const QString &_text)
 {
-    QString result = listFunct.makeCompletion( _text.toUpper() );
-    if( !result.isNull() )
-        functions->setCurrentItem( functions->index( functions->findItem( result ) ) );
+    proxyModel->setFilterFixedString(_text);
+    if( functions->currentIndex().isValid() )
+        functions->scrollTo( functions->currentIndex() );
 }
 
 bool FormulaDialog::eventFilter( QObject* obj, QEvent* ev )
@@ -302,86 +331,65 @@ bool FormulaDialog::eventFilter( QObject* obj, QEvent* ev )
         return false;
 
     if ( m_focus )
-        m_pView->canvasWidget()->startChoose();
+        m_selection->startReferenceSelection();
 
     return false;
 }
 
 void FormulaDialog::slotOk()
 {
-    m_pView->doc()->emitBeginOperation( false );
-
-    m_pView->canvasWidget()->endChoose();
-    // Switch back to the old sheet
-    if( m_pView->activeSheet()->sheetName() !=  m_sheetName )
-    {
-        Sheet *sheet=m_pView->doc()->map()->findSheet(m_sheetName);
-        if( sheet)
-	    m_pView->setActiveSheet(sheet);
-    }
+    m_selection->endReferenceSelection();
 
     // Revert the marker to its original position
-    m_pView->selection()->initialize( QPoint( m_column, m_row ) );
+    m_selection->initialize( QPoint( m_column, m_row ) );
 
     // If there is still an editor then set the text.
     // Usually the editor is always in place.
-    if( m_pView->canvasWidget()->editor() != 0 )
+    if( m_editor != 0 )
     {
-        Q_ASSERT( m_pView->canvasWidget()->editor() );
+        Q_ASSERT( m_editor );
         QString tmp = result->text();
         if( tmp.at(0) != '=')
 	    tmp = '=' + tmp;
-        int pos = m_pView->canvasWidget()->editor()->cursorPosition()+ tmp.length();
-        m_pView->canvasWidget()->editor()->setText( tmp );
-        m_pView->canvasWidget()->editor()->setFocus();
-        m_pView->canvasWidget()->editor()->setCursorPosition( pos );
+        int pos = m_editor->cursorPosition()+ tmp.length();
+        m_editor->setText( tmp );
+        m_editor->setFocus();
+        m_editor->setCursorPosition( pos );
     }
 
-    m_pView->slotUpdateView( m_pView->activeSheet() );
+    m_selection->emitModified();
     accept();
-    //  delete this;
+    deleteLater();
 }
 
 void FormulaDialog::slotClose()
 {
-    m_pView->doc()->emitBeginOperation( false );
-
-    m_pView->canvasWidget()->endChoose();
-
-    // Switch back to the old sheet
-    if(m_pView->activeSheet()->sheetName() !=  m_sheetName )
-    {
-        Sheet *sheet=m_pView->doc()->map()->findSheet(m_sheetName);
-        if( !sheet )
-	    return;
-	m_pView->setActiveSheet(sheet);
-    }
+    deleteLater();
 
+    m_selection->endReferenceSelection();
 
     // Revert the marker to its original position
-    m_pView->selection()->initialize( QPoint( m_column, m_row ) );
+    m_selection->initialize( QPoint( m_column, m_row ) );
 
     // If there is still an editor then reset the text.
     // Usually the editor is always in place.
-    if( m_pView->canvasWidget()->editor() != 0 )
+    if( m_editor != 0 )
     {
-        Q_ASSERT( m_pView->canvasWidget()->editor() );
-        m_pView->canvasWidget()->editor()->setText( m_oldText );
-        m_pView->canvasWidget()->editor()->setFocus();
+        Q_ASSERT( m_editor );
+        m_editor->setText( m_oldText );
+        m_editor->setFocus();
     }
 
-    m_pView->slotUpdateView( m_pView->activeSheet() );
+    m_selection->emitModified();
     reject();
-    //laurent 2002-01-03 comment this line otherwise kspread crash
-    //but dialog box is not deleted => not good
-    //delete this;
 }
 
 void FormulaDialog::slotSelectButton()
 {
-    if( functions->currentItem() != -1 )
+    if( functions->currentIndex().isValid() )
     {
-        slotDoubleClicked(functions->findItem(functions->text(functions->currentItem())));
+//slotDoubleClicked(functions->findItem(functions->text(functions->currentItem())));
+        slotDoubleClicked( functions->currentIndex() );
     }
 }
 
@@ -406,7 +414,7 @@ QString FormulaDialog::createFormula()
     QString tmp( "" );
 
     if ( !m_desc )
-	return QString();
+        return QString();
 
     bool first = true;
 
@@ -415,40 +423,40 @@ QString FormulaDialog::createFormula()
     if(!firstElement->text().isEmpty() && count >= 1 )
     {
         tmp=tmp+createParameter(firstElement->text(), 0 );
-	first = false;
+        first = false;
     }
 
     if(!secondElement->text().isEmpty() && count >= 2 )
     {
-	first = false;
-	if ( !first )
-	    tmp=tmp+';'+createParameter(secondElement->text(), 1 );
-	else
-	    tmp=tmp+createParameter(secondElement->text(), 1 );
+        first = false;
+        if ( !first )
+            tmp=tmp+';'+createParameter(secondElement->text(), 1 );
+        else
+            tmp=tmp+createParameter(secondElement->text(), 1 );
     }
     if(!thirdElement->text().isEmpty() && count >= 3 )
     {
-	first = false;
-	if ( !first )
-	    tmp=tmp+';'+createParameter(thirdElement->text(), 2 );
-        else
-	    tmp=tmp+createParameter(thirdElement->text(), 2 );
+        first = false;
+        if ( !first )
+            tmp=tmp+';'+createParameter(thirdElement->text(), 2 );
+            else
+            tmp=tmp+createParameter(thirdElement->text(), 2 );
     }
     if(!fourElement->text().isEmpty() && count >= 4 )
     {
-	first = false;
-	if ( !first )
-	    tmp=tmp+';'+createParameter(fourElement->text(), 3 );
-        else
-	    tmp=tmp+createParameter(fourElement->text(), 3 );
+        first = false;
+        if ( !first )
+            tmp=tmp+';'+createParameter(fourElement->text(), 3 );
+            else
+            tmp=tmp+createParameter(fourElement->text(), 3 );
     }
     if(!fiveElement->text().isEmpty() && count >= 5 )
     {
-	first = false;
-	if ( !first )
-	    tmp=tmp+';'+createParameter(fiveElement->text(), 4 );
-        else
-	    tmp=tmp+createParameter(fiveElement->text(), 4 );
+        first = false;
+        if ( !first )
+            tmp=tmp+';'+createParameter(fiveElement->text(), 4 );
+            else
+            tmp=tmp+createParameter(fiveElement->text(), 4 );
     }
 
     return(tmp);
@@ -457,10 +465,10 @@ QString FormulaDialog::createFormula()
 QString FormulaDialog::createParameter( const QString& _text, int param )
 {
     if ( _text.isEmpty() )
-	return QString( "" );
+        return QString( "" );
 
     if ( !m_desc )
-	return QString( "" );
+        return QString( "" );
 
     QString text;
 
@@ -469,71 +477,70 @@ QString FormulaDialog::createParameter( const QString& _text, int param )
     switch( elementType )
     {
     case KSpread_Any:
-	{
-		bool isNumber;
-		double tmp = m_pView->doc()->locale()->readNumber( _text, &isNumber );
-		Q_UNUSED( tmp );
+    {
+        bool isNumber;
+        double tmp = m_selection->activeSheet()->map()->calculationSettings()->locale()->readNumber( _text, &isNumber );
+        Q_UNUSED( tmp );
 
-		//In case of number or boolean return _text, else return value as KSpread_String
-		if ( isNumber || _text.toUpper() =="FALSE" || _text.toUpper() == "TRUE" )
-			return _text;
-	}
+        //In case of number or boolean return _text, else return value as KSpread_String
+        if ( isNumber || _text.toUpper() =="FALSE" || _text.toUpper() == "TRUE" )
+            return _text;
+    }
         // fall through
     case KSpread_String:
-	{
-	    // Does the text start with quotes?
-	    if ( _text[0] == '"' )
-	    {
-  	        text = '\\'; // changed: was '"'
+    {
+        // Does the text start with quotes?
+        if ( _text[0] == '"' )
+        {
+            text = '\\'; // changed: was '"'
 
-		// Escape quotes
-		QString tmp = _text;
-		int pos;
-		int start = 1;
-		while( ( pos = tmp.indexOf( '"', start ) ) != -1 )
-		{
-		  if (tmp[pos - 1] != '\\')
-		    tmp.replace( pos, 1, "\\\"" );
-		  else
-		    start = pos + 1;
-		}
-
-		text += tmp;
-		text += '"';
-	    }
-	    else
-	    {
-                const Region region(_text, m_pView->doc()->map());
-		if (!region.isValid())
-		{
-		    text = '"';
-
-		    // Escape quotes
-		    QString tmp = _text;
-		    int pos;
-		    int start = 1;
-		    while( ( pos = tmp.indexOf( '"', start ) ) != -1 )
-		    {
-  		        if (tmp[pos - 1] != '\\')
-			  tmp.replace( pos, 1, "\\\"" );
-			else
-			  start = pos + 1;
-		    }
-
-		    text += tmp;
-		    text += '"';
-		}
-		else
-		    text = _text;
+            // Escape quotes
+            QString tmp = _text;
+            int pos;
+            int start = 1;
+            while( ( pos = tmp.indexOf( '"', start ) ) != -1 )
+            {
+                if (tmp[pos - 1] != '\\')
+                tmp.replace( pos, 1, "\\\"" );
+                else
+                start = pos + 1;
             }
+
+            text += tmp;
+            text += '"';
         }
-	return text;
+        else
+        {
+            const Region region(_text, m_selection->activeSheet()->map());
+            if (!region.isValid())
+            {
+                text = '"';
+
+                // Escape quotes
+                QString tmp = _text;
+                int pos;
+                int start = 1;
+                while( ( pos = tmp.indexOf( '"', start ) ) != -1 )
+                {
+                    if (tmp[pos - 1] != '\\')
+                tmp.replace( pos, 1, "\\\"" );
+                else
+                start = pos + 1;
+                }
+
+                text += tmp;
+                text += '"';
+            }
+            else
+                text = _text;
+        }
+    } return text;
     case KSpread_Float:
-	return _text;
+        return _text;
     case KSpread_Boolean:
-	return _text;
+        return _text;
     case KSpread_Int:
-	return _text;
+        return _text;
     }
 
     // Never reached
@@ -550,40 +557,43 @@ static void showEntry( KLineEdit* edit, QLabel* label,
     KFloatValidator *validate=0;
     switch( elementType )
     {
-    case KSpread_String:
-    case KSpread_Boolean:
-    case KSpread_Any:
-      edit->setValidator(0);
-      break;
-    case KSpread_Float:
-        validate=new KFloatValidator (edit);
-        validate->setAcceptLocalizedNumbers(true);
-        edit->setValidator(validate);
-        edit->setText( "0" );
-      break;
-    case KSpread_Int:
-      edit->setValidator(new QIntValidator (edit));
-      edit->setText( "0" );
-      break;
-      }
+        case KSpread_String:
+        case KSpread_Boolean:
+        case KSpread_Any:
+            edit->setValidator(0);
+            break;
+        case KSpread_Float:
+            validate=new KFloatValidator (edit);
+            validate->setAcceptLocalizedNumbers(true);
+            edit->setValidator(validate);
+            edit->setText( "0" );
+            break;
+        case KSpread_Int:
+            edit->setValidator(new QIntValidator (edit));
+            edit->setText( "0" );
+            break;
+    }
 
 }
 
-void FormulaDialog::slotDoubleClicked( Q3ListBoxItem* item )
+void FormulaDialog::slotDoubleClicked( QModelIndex item )
 {
-    if ( !item )
-	return;
+    if ( !item.isValid() ) {
+        item = functions->currentIndex();
+        if ( !item.isValid() )
+            return;
+    }
     refresh_result = false;
     if ( !m_desc )
     {
-	m_browser->setText( "" );
-	return;
+        m_browser->setText( "" );
+        return;
     }
 
     m_focus = 0;
     int old_length = result->text().length();
 
-    // Dont change order of these function calls due to a bug in Qt 2.2
+    // Do not change order of these function calls due to a bug in Qt 2.2
     m_browser->setText( m_desc->toQML() );
     m_tabwidget->setTabEnabled( m_tabwidget->indexOf(m_input), true );
     m_tabwidget->setCurrentIndex( 1 );
@@ -596,59 +606,58 @@ void FormulaDialog::slotDoubleClicked( Q3ListBoxItem* item )
         m_focus = firstElement;
         firstElement->setFocus();
 
-	showEntry( firstElement, label1, m_desc, 0 );
+        showEntry( firstElement, label1, m_desc, 0 );
     }
     else
     {
-	label1->hide();
-	firstElement->hide();
+        label1->hide();
+        firstElement->hide();
     }
 
     if( m_desc->params() > 1 )
     {
-	showEntry( secondElement, label2, m_desc, 1 );
+        showEntry( secondElement, label2, m_desc, 1 );
     }
     else
     {
-	label2->hide();
-	secondElement->hide();
+        label2->hide();
+        secondElement->hide();
     }
 
     if( m_desc->params() > 2 )
     {
-	showEntry( thirdElement, label3, m_desc, 2 );
+        showEntry( thirdElement, label3, m_desc, 2 );
     }
     else
     {
-	label3->hide();
-	thirdElement->hide();
+        label3->hide();
+        thirdElement->hide();
     }
 
     if( m_desc->params() > 3 )
     {
-	showEntry( fourElement, label4, m_desc, 3 );
+        showEntry( fourElement, label4, m_desc, 3 );
     }
     else
     {
-	label4->hide();
-	fourElement->hide();
+        label4->hide();
+        fourElement->hide();
     }
 
     if( m_desc->params() > 4 )
     {
-	showEntry( fiveElement, label5, m_desc, 4 );
+        showEntry( fiveElement, label5, m_desc, 4 );
     }
     else
     {
-	label5->hide();
-	fiveElement->hide();
+        label5->hide();
+        fiveElement->hide();
     }
 
     if( m_desc->params() > 5 )
-    {
         kDebug(36001) <<"Error in param->nb_param";
-    }
     refresh_result= true;
+
     //
     // Put the new function call in the result.
     //
@@ -662,37 +671,54 @@ void FormulaDialog::slotDoubleClicked( Q3ListBoxItem* item )
         m_rightText="";
         m_leftText=result->text();
     }
-    int pos = result->cursorPosition();
-    result->setText( m_leftText+functions->text( functions->currentItem() ) + "()" + m_rightText);
 
-    if (result->text()[0] != '=')
-      result->setText('=' + result->text());
+    int pos = result->cursorPosition();
+
+    {
+        const QString text = proxyModel->data( functions->currentIndex() ).toString();
+        result->setText( m_leftText+text + "()" + m_rightText);
+        if (result->text()[0] != '=')
+            result->setText('=' + result->text());
+    }
 
     //
     // Put focus somewhere is there are no KLineEdits visible
     //
     if( m_desc->params() == 0 )
     {
-	label1->show();
-	label1->setText( i18n("This function has no parameters.") );
-
+        label1->show();
+        label1->setText( i18n("This function has no parameters.") );
         result->setFocus();
-        result->setCursorPosition(pos+functions->text(functions->currentItem()).length()+2);
+        const QString text = proxyModel->data( functions->currentIndex() ).toString();
+        result->setCursorPosition(pos+text.length()+2);
     }
     slotChangeText( "" );
 }
 
-void FormulaDialog::slotSelected( const QString& function )
+void FormulaDialog::slotIndexSelected( const QModelIndex& index )
 {
-    FunctionDescription* desc =
-        FunctionRepository::self()->functionInfo (function);
+    // This slot is only called when single-click to activate is uesd in the listbox, when the mouse moves over a item; to prevent
+    // the active selection to change after the user activated one, slotSelected is only called when the current tab is the Help tab,
+    // and not when the parameters tab is active
+    if ( m_tabwidget->currentIndex() != 0 ) return;
+    QString function = proxyModel->data( index ).toString();
+    slotSelected(function);
+}
+
+void FormulaDialog::slotSelected( const QString& afunction )
+{
+    QString function = afunction;
+    if( function.isNull() )
+        function = proxyModel->data( functions->currentIndex() ).toString();
+
+    FunctionDescription* desc = FunctionRepository::self()->functionInfo (function);
     if ( !desc )
     {
-      m_browser->setText (i18n ("Description is not available."));
-      return;
+        m_browser->setText (i18n ("Description is not available."));
+        return;
     }
 
-    if( functions->currentItem() !=- 1 )
+    if( functions->currentIndex().isValid() )
         selectFunction->setEnabled( true );
 
     // Lock
@@ -705,7 +731,7 @@ void FormulaDialog::slotSelected( const QString& function )
     m_browser->setText( m_desc->toQML() );
     //m_browser->setContentsPos( 0, 0 );
 
-    m_focus=0;
+    m_focus = 0;
 
     m_tabwidget->setCurrentIndex( 0 );
     m_tabwidget->setTabEnabled( m_tabwidget->indexOf(m_input), false );
@@ -727,9 +753,13 @@ void FormulaDialog::slotShowFunction( const QString& function )
     slotActivated( category );
 
     // select the function
-    Q3ListBoxItem* item = functions->findItem( function,
-        QKeySequence::ExactMatch | Qt::CaseSensitive );
-    if( item ) functions->setCurrentItem( item );
+    //Q3ListBoxItem* item = functions->findItem( function, QKeySequence::ExactMatch | Qt::CaseSensitive );
+    //if( item ) functions->setCurrentItem( item );
+    int row = functionsModel->stringList().indexOf(function);
+    const QModelIndex sourcemodelindex = functionsModel->index(row, 0);
+    const QModelIndex proxymodelindex = proxyModel->mapFromSource(sourcemodelindex);
+    if( proxymodelindex.isValid() )
+        functions->setCurrentIndex( proxymodelindex );
 
     slotSelected( function );
 }
@@ -739,10 +769,10 @@ void FormulaDialog::slotSelectionChanged()
     if ( !m_focus )
         return;
 
-    if (m_pView->choice()->isValid())
+    if (m_selection->isValid())
     {
-      QString area = m_pView->choice()->name();
-      m_focus->setText( area );
+        QString area = m_selection->name();
+        m_focus->setText( area );
     }
 }
 
@@ -756,23 +786,22 @@ void FormulaDialog::slotActivated( const QString& category )
 
     kDebug(36001)<<"category:"<<category<<" ("<<lst.count()<<"functions)";
 
-    functions->clear();
-    functions->insertStringList( lst );
+    functionsModel->setStringList( lst );
 
     QStringList upperList;
     for ( QStringList::Iterator it = lst.begin(); it != lst.end();++it )
       upperList.append((*it).toUpper());
-
     listFunct.setItems( upperList );
 
     // Go to the first function in the list.
-    functions->setCurrentItem(0);
-    slotSelected( functions->text(0) );
+    const QString text = proxyModel->data( proxyModel->index(0,0) ).toString();
+    slotSelected(text);
 }
 
 void FormulaDialog::closeEvent ( QCloseEvent * e )
 {
-  e->accept();
+    deleteLater();
+    e->accept();
 }
 
 #include "FormulaDialog.moc"
