@@ -24,7 +24,12 @@
 
 #include "LinkDialog.h"
 
-#include <QFrame>
+#include "Cell.h"
+#include "Map.h"
+#include "NamedAreaManager.h"
+#include "Selection.h"
+#include "Sheet.h"
+
 #include <QLabel>
 #include <QVBoxLayout>
 
@@ -38,6 +43,7 @@
 #include <klocale.h>
 #include <krecentdocument.h>
 #include <kurlrequester.h>
+#include <kurlcompletion.h>
 
 using namespace KSpread;
 
@@ -51,17 +57,18 @@ public:
     QWidget* mailPage;
     KLineEdit* mailText;
     KLineEdit* mailLink;
+    KLineEdit* mailSubject;
     QWidget* filePage;
     KLineEdit* fileText;
     KUrlRequester* fileLink;
     QWidget* cellPage;
     KLineEdit* cellText;
-    KLineEdit* cellLink;
+    KComboBox* cellLink;
     KPageWidgetItem* p1, *p2, *p3, *p4;
 };
 
-LinkDialog::LinkDialog( QWidget*, const char* )
-    : KPageDialog()
+LinkDialog::LinkDialog(QWidget* parent, Selection* selection)
+    : KPageDialog(parent)
     , d( new Private )
 {
     setCaption( i18n("Insert Link") );
@@ -71,10 +78,9 @@ LinkDialog::LinkDialog( QWidget*, const char* )
     // link for web or ftp
     d->internetPage = new QWidget();
     d->p1 = addPage( d->internetPage, i18n( "Internet" ) );
-    d->p1->setIcon( KIcon( BarIcon( "html",KIconLoader::SizeMedium ) ) );
+    d->p1->setHeader( i18n("Link to Internet Address") );
+    d->p1->setIcon( KIcon("internet-web-browser") );
     QVBoxLayout* iLayout = new QVBoxLayout( d->internetPage );
-    iLayout->setMargin( marginHint() );
-    iLayout->setSpacing( spacingHint() );
     iLayout->addWidget( new QLabel( i18n("Text to display:" ), d->internetPage ) );
     d->internetText = new KLineEdit( d->internetPage );
     iLayout->addWidget( d->internetText );
@@ -88,32 +94,35 @@ LinkDialog::LinkDialog( QWidget*, const char* )
     // link for e-mail
     d->mailPage = new QWidget();
     d->p2 = addPage( d->mailPage, i18n( "Mail" ) );
-    d->p2->setIcon( KIcon( BarIcon( "mail",KIconLoader::SizeMedium ) ) );
+    d->p2->setHeader( i18n("Link to Mail Address") );
+    d->p2->setIcon( KIcon("mail-message") );
     QVBoxLayout* mLayout = new QVBoxLayout( d->mailPage );
-    mLayout->setMargin( marginHint() );
-    mLayout->setSpacing( spacingHint() );
     mLayout->addWidget( new QLabel( i18n("Text to display:" ), d->mailPage ) );
     d->mailText = new KLineEdit( d->mailPage );
     mLayout->addWidget( d->mailText );
     mLayout->addWidget( new QLabel( i18n("Email:" ), d->mailPage ) );
     d->mailLink = new KLineEdit( d->mailPage );
     mLayout->addWidget( d->mailLink );
-    mLayout->addItem( new QSpacerItem( 0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding ) );
     connect( d->mailText, SIGNAL( textChanged( const QString& ) ), this,
         SLOT( setText( const QString& ) ) );
+    mLayout->addWidget( new QLabel( i18n("Subject:" ), d->mailPage ) );
+    d->mailSubject = new KLineEdit( d->mailPage );
+    mLayout->addWidget( d->mailSubject );
+    mLayout->addItem( new QSpacerItem( 0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding ) );
 
     // link for external file
     d->filePage = new QWidget();
     d->p3 = addPage( d->filePage, i18n( "File" ) );
-    d->p3->setIcon( KIcon( BarIcon( "document-open",KIconLoader::SizeMedium ) ) );
+    d->p3->setHeader( i18n("Link to File") );
+    d->p3->setIcon( KIcon("system-file-manager") );
     QVBoxLayout* fLayout = new QVBoxLayout( d->filePage );
-    fLayout->setMargin( marginHint() );
-    fLayout->setSpacing( spacingHint() );
     fLayout->addWidget( new QLabel( i18n("Text to display:" ), d->filePage ) );
     d->fileText = new KLineEdit( d->filePage );
     fLayout->addWidget( d->fileText );
     fLayout->addWidget( new QLabel( i18n("File location:" ), d->filePage ) );
     d->fileLink = new KUrlRequester( d->filePage );
+    d->fileLink->completionObject()->setReplaceHome(true);
+    d->fileLink->completionObject()->setReplaceEnv(true);
     fLayout->addWidget( d->fileLink );
     fLayout->addWidget( new QLabel( i18n("Recent file:" ), d->filePage ) );
     KComboBox* recentFile = new KComboBox( d->filePage );
@@ -125,7 +134,6 @@ LinkDialog::LinkDialog( QWidget*, const char* )
     QObject::connect( recentFile, SIGNAL( highlighted ( const QString &) ),
         d->fileLink->lineEdit(), SLOT( setText( const QString & ) ) );
 
-
     // populate recent files
     int index = 0;
     QStringList fileList = KRecentDocument::recentDocuments();
@@ -144,14 +152,26 @@ LinkDialog::LinkDialog( QWidget*, const char* )
     // link to another cell
     d->cellPage = new QWidget();
     d->p4 = addPage( d->cellPage, i18n( "Cell" ) );
-    d->p4->setIcon( KIcon( BarIcon( "table",KIconLoader::SizeMedium ) ) );
+    d->p4->setHeader( i18n("Link to Cell") );
+    d->p4->setIcon( KIcon("table") );
     QVBoxLayout* cLayout = new QVBoxLayout( d->cellPage );
-    cLayout->setSpacing( spacingHint() );
     cLayout->addWidget( new QLabel( i18n("Text to display:" ), d->cellPage ) );
     d->cellText = new KLineEdit( d->cellPage );
     cLayout->addWidget( d->cellText );
-    cLayout->addWidget( new QLabel( i18n("Cell:" ), d->cellPage ) );
-    d->cellLink = new KLineEdit( d->cellPage );
+    cLayout->addWidget( new QLabel( i18n("Cell or Named Area:" ), d->cellPage ) );
+    d->cellLink = new KComboBox( d->cellPage );
+    d->cellLink->setEditable(true);
+
+    const Sheet *sheet = selection->activeSheet();
+    if( sheet && selection ) {
+        Cell cell(sheet, selection->cursor());
+        d->cellLink->addItem( cell.fullName() );
+    }
+
+    const NamedAreaManager *manager = selection->activeSheet()->map()->namedAreaManager();
+    d->cellLink->addItems( manager->areaNames() );
+
+    d->cellLink->setItemText(d->cellLink->currentIndex(), "");
     cLayout->addWidget( d->cellLink );
     cLayout->addItem( new QSpacerItem( 0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding ) );
     connect( d->cellText, SIGNAL( textChanged( const QString& ) ), this,
@@ -191,21 +211,26 @@ QString LinkDialog::link() const
       if( !str.isEmpty() )
         if( str.indexOf( "mailto:" )==-1 )
           str.prepend( "mailto:" );
-      str = d->fileLink->lineEdit()->text();
-      if( !str.isEmpty() )
-        if( str.indexOf( "file:/" )==-1 )
-          str.prepend( "file://" );
+      const QString subject = d->mailSubject->text().trimmed();
+      if( ! subject.isEmpty() )
+        str.append( QString("?subject=%1").arg( QString(QUrl::toPercentEncoding(subject)) ) );
     }
     else if ( currentPage() == d->p3 )
     {
-      str = d->fileLink->lineEdit()->text();
-      if( !str.isEmpty() )
-        if( str.indexOf( "file:/" )==-1 )
-          str.prepend( "file://" );
+      KUrl url = d->fileLink->url();
+      if( url.isValid() ) {
+        str = url.url();
+      }
+      else {
+        str = d->fileText->text();
+        if( !str.isEmpty() )
+          if( ! str.contains( QRegExp("^(file|mailto|http|https|ftp):") ) )
+            str.prepend( "file://" );
+      }
     }
     else if ( currentPage() == d->p4 )
     {
-      str = d->cellLink->text();
+      str = d->cellLink->currentText();
     }
 
     return str;
@@ -259,8 +284,15 @@ void LinkDialog::setLink( const QString& link )
 
     if( link.startsWith( "mailto:" ) )
     {
-      d->mailLink->setText( link.mid( QString("mailto:").length() ) );
-      setCurrentPage( d->p1 );
+      QUrl url(link);
+      if ( url.isValid() ) {
+        d->mailLink->setText( url.toString(QUrl::RemoveScheme|QUrl::RemoveQuery) );
+        d->mailSubject->setText( url.queryItemValue("subject") );
+      }
+      else {
+        d->mailLink->setText( link.mid( QString("mailto:").length() ) );
+      }
+      setCurrentPage( d->p2 );
       return;
     }
 
@@ -269,12 +301,12 @@ void LinkDialog::setLink( const QString& link )
       QString s = link.mid( QString("file:/").length() );
       while(s.startsWith("//")) s.remove(0,1);
       d->fileLink->lineEdit()->setText(s);
-      setCurrentPage( d->p2 );
+      setCurrentPage( d->p3 );
       return;
     }
 
     // assume cell reference
-    d->cellLink->setText( link );
+    d->cellLink->setItemText(d->cellLink->currentIndex(), link);
     setCurrentPage( d->p4 );
 }
 
