@@ -62,7 +62,8 @@
 // KOffice
 #include <KoCanvasBase.h>
 #include <KoCanvasController.h>
-#include <KoResourceManager.h>
+#include <KoCanvasResourceProvider.h>
+#include <KoColor.h>
 #include <KoViewConverter.h>
 
 // KDE
@@ -71,95 +72,81 @@
 #include <KFontSizeAction>
 
 // Qt
-#include <QApplication>
-#include <QGridLayout>
 #include <QPainter>
-#include <QToolButton>
 
 using namespace KSpread;
 
 void CellToolBase::Private::updateEditor(const Cell& cell)
 {
-    const Cell& theCell = cell.isPartOfMerged() ? cell.masterCell() : cell;
-    const Style style = theCell.style();
+    const Style style = cell.style();
     if (q->selection()->activeSheet()->isProtected() && style.hideFormula()) {
-        userInput->setPlainText(theCell.displayText());
+        optionWidget.userInput->setPlainText(cell.displayText());
     } else if (q->selection()->activeSheet()->isProtected() && style.hideAll()) {
-        userInput->clear();
+        optionWidget.userInput->clear();
     } else {
-        userInput->setPlainText(theCell.userInput());
+        optionWidget.userInput->setPlainText(cell.userInput());
     }
 }
 
 void CellToolBase::Private::updateLocationComboBox()
 {
     QString address;
-    const QList< QPair<QRectF, QString> > names = q->selection()->activeSheet()->cellStorage()->namedAreas(*q->selection());
-    {
-        QRect range;
-        if (q->selection()->isSingular()) range = QRect(q->selection()->marker(), QSize(1, 1));
-        else range = q->selection()->lastRange();
-        for (int i = 0; i < names.size(); i++) {
-            if (names[i].first.toRect() == range) {
-                address = names[i].second;
-            }
-        }
-    }
-    if (address.isEmpty()) {
-        if (q->selection()->activeSheet()->getLcMode()) {
-            if (q->selection()->isSingular()) {
-                address = 'L' + QString::number(q->selection()->marker().y()) +
-                        'C' + QString::number(q->selection()->marker().x());
-            } else {
-                const QRect lastRange = q->selection()->lastRange();
-                address = QString::number(lastRange.bottom() - lastRange.top() + 1) + "Lx";
-                if (Region::Range(lastRange).isRow())
-                    address += QString::number(q->maxCol() - lastRange.left() + 1) + 'C';
-                else
-                    address += QString::number(lastRange.right() - lastRange.left() + 1) + 'C';
-            }
+    if (q->selection()->activeSheet()->getLcMode()) {
+        if (q->selection()->isSingular()) {
+            address = 'L' + QString::number(q->selection()->marker().y()) +
+                      'C' + QString::number(q->selection()->marker().x());
         } else {
-            address = q->selection()->name();
+            const QRect lastRange = q->selection()->lastRange();
+            address = QString::number(lastRange.bottom() - lastRange.top() + 1) + "Lx";
+            if (Region::Range(lastRange).isRow())
+                address += QString::number(q->maxCol() - lastRange.left() + 1) + 'C';
+            else
+                address += QString::number(lastRange.right() - lastRange.left() + 1) + 'C';
         }
+    } else {
+        address = q->selection()->name();
     }
     if (address != locationComboBox->lineEdit()->text())
         locationComboBox->lineEdit()->setText(address);
 }
 
-#define ACTION_EXEC( name, command ) { \
-        QAction *a = q->action(name); \
-        const bool blocked = a->blockSignals(true); \
-        a->command; \
-        a->blockSignals(blocked); \
-    }
-
 void CellToolBase::Private::updateActions(const Cell& cell)
 {
     const Style style = cell.style();
 
     // -- font actions --
-    ACTION_EXEC("bold", setChecked(style.bold()));
-    ACTION_EXEC("italic", setChecked(style.italic()));
-    ACTION_EXEC("underline", setChecked(style.underline()));
-    ACTION_EXEC("strikeOut", setChecked(style.strikeOut()));
-
-    static_cast<KFontAction*>(q->action("font"))->setFont(style.fontFamily());
+    q->action("bold")->setChecked(style.bold());
+    q->action("italic")->setChecked(style.italic());
+    q->action("underline")->setChecked(style.underline());
+    q->action("strikeOut")->setChecked(style.strikeOut());
+    // workaround for bug #59291 (crash upon starting from template)
+    // certain Qt and Fontconfig combination fail miserably if can not
+    // find the font name (e.g. not installed in the system)
+    QStringList fontList;
+    KFontChooser::getFontList(fontList, 0);
+    QString fontFamily = style.fontFamily();
+    for (QStringList::Iterator it = fontList.begin(); it != fontList.end(); ++it) {
+        if ((*it).toLower() == fontFamily.toLower()) {
+            static_cast<KFontAction*>(q->action("font"))->setFont(fontFamily);
+            break;
+        }
+    }
     static_cast<KFontSizeAction*>(q->action("fontSize"))->setFontSize(style.fontSize());
     // -- horizontal alignment actions --
-    ACTION_EXEC("alignLeft", setChecked(style.halign() == Style::Left));
-    ACTION_EXEC("alignCenter", setChecked(style.halign() == Style::Center));
-    ACTION_EXEC("alignRight", setChecked(style.halign() == Style::Right));
+    q->action("alignLeft")->setChecked(style.halign() == Style::Left);
+    q->action("alignCenter")->setChecked(style.halign() == Style::Center);
+    q->action("alignRight")->setChecked(style.halign() == Style::Right);
     // -- vertical alignment actions --
-    ACTION_EXEC("alignTop", setChecked(style.valign() == Style::Top));
-    ACTION_EXEC("alignMiddle", setChecked(style.valign() == Style::Middle));
-    ACTION_EXEC("alignBottom", setChecked(style.valign() == Style::Bottom));
+    q->action("alignTop")->setChecked(style.valign() == Style::Top);
+    q->action("alignMiddle")->setChecked(style.valign() == Style::Middle);
+    q->action("alignBottom")->setChecked(style.valign() == Style::Bottom);
 
-    ACTION_EXEC("verticalText", setChecked(style.verticalText()));
-    ACTION_EXEC("wrapText", setChecked(style.wrapText()));
+    q->action("verticalText")->setChecked(style.verticalText());
+    q->action("wrapText")->setChecked(style.wrapText());
 
     Format::Type ft = style.formatType();
-    ACTION_EXEC("percent", setChecked(ft == Format::Percentage));
-    ACTION_EXEC("currency", setChecked(ft == Format::Money));
+    q->action("percent")->setChecked(ft == Format::Percentage);
+    q->action("currency")->setChecked(ft == Format::Money);
 
     if (!q->selection()->activeSheet()->isProtected() || style.notProtected()) {
         q->action("clearComment")->setEnabled(!cell.comment().isEmpty());
@@ -209,8 +196,8 @@ void CellToolBase::Private::setProtectedActionsEnabled(bool enable)
     const QList<KAction*> actions = q->actions().values();
     for (int i = 0; i < actions.count(); ++i)
         actions[i]->setEnabled(enable);
-    formulaButton->setEnabled(enable);
-    userInput->setEnabled(enable);
+    optionWidget.formulaButton->setEnabled(enable);
+    optionWidget.userInput->setEnabled(enable);
 
     // These actions are always enabled.
     q->action("copy")->setEnabled(true);
@@ -252,9 +239,6 @@ void CellToolBase::Private::processEnterKey(QKeyEvent* event)
         case BottomFirst:
             direction = BottomFirst;
             break;
-        case NoMovement:
-            direction = NoMovement;
-            break;
         }
     }
 
@@ -489,11 +473,11 @@ void CellToolBase::Private::processDeleteKey(QKeyEvent* event)
 
 void CellToolBase::Private::processF2Key(QKeyEvent*  event)
 {
-    userInput->setFocus();
+    optionWidget.userInput->setFocus();
     if (q->editor()) {
-        QTextCursor textCursor = userInput->textCursor();
+        QTextCursor textCursor = optionWidget.userInput->textCursor();
         textCursor.setPosition(q->editor()->cursorPosition());
-        userInput->setTextCursor(textCursor);
+        optionWidget.userInput->setTextCursor(textCursor);
     }
     event->accept(); // QKeyEvent
 }
@@ -504,9 +488,9 @@ void CellToolBase::Private::processF4Key(QKeyEvent* event)
     */
     if (q->editor()) {
         q->editor()->handleKeyPressEvent(event);
-        QTextCursor textCursor = userInput->textCursor();
+        QTextCursor textCursor = optionWidget.userInput->textCursor();
         textCursor.setPosition(q->editor()->cursorPosition());
-        userInput->setTextCursor(textCursor);
+        optionWidget.userInput->setTextCursor(textCursor);
     }
 }
 
@@ -849,10 +833,10 @@ bool CellToolBase::Private::formatKeyPress(QKeyEvent * _ev)
 
     case Qt::Key_Ampersand:
         command->setText(i18n("Change Border"));
-        command->setTopBorderPen(QPen(q->canvas()->resourceManager()->foregroundColor().toQColor(), 1, Qt::SolidLine));
-        command->setBottomBorderPen(QPen(q->canvas()->resourceManager()->foregroundColor().toQColor(), 1, Qt::SolidLine));
-        command->setLeftBorderPen(QPen(q->canvas()->resourceManager()->foregroundColor().toQColor(), 1, Qt::SolidLine));
-        command->setRightBorderPen(QPen(q->canvas()->resourceManager()->foregroundColor().toQColor(), 1, Qt::SolidLine));
+        command->setTopBorderPen(QPen(q->canvas()->resourceProvider()->foregroundColor().toQColor(), 1, Qt::SolidLine));
+        command->setBottomBorderPen(QPen(q->canvas()->resourceProvider()->foregroundColor().toQColor(), 1, Qt::SolidLine));
+        command->setLeftBorderPen(QPen(q->canvas()->resourceProvider()->foregroundColor().toQColor(), 1, Qt::SolidLine));
+        command->setRightBorderPen(QPen(q->canvas()->resourceProvider()->foregroundColor().toQColor(), 1, Qt::SolidLine));
         break;
 
     default:
@@ -947,9 +931,6 @@ QRect CellToolBase::Private::moveDirection(KSpread::MoveTo direction, bool exten
 
         destination = QPoint(1, qMin(cursor.y() + offset, q->maxRow()));
         break;
-    case NoMovement:
-        destination = cursor;
-        break;
     }
 
     if (extendSelection) {
@@ -1192,7 +1173,7 @@ QList<QAction*> CellToolBase::Private::popupActionList() const
             actions.append(popupMenuActions["separator3"]);
             actions.append(popupMenuActions["insertCell"]);
             actions.append(popupMenuActions["deleteCell"]);
-        } else if (q->selection()->isColumnSelected()) {
+        } else if (q->selection()->isColumnSelected() && q->selection()->isSingular()) {
             actions.append(q->action("resizeCol"));
             actions.append(popupMenuActions["adjustColumn"]);
             actions.append(popupMenuActions["separator4"]);
@@ -1229,7 +1210,7 @@ QList<QAction*> CellToolBase::Private::popupActionList() const
                     break;
                 }
             }
-        } else if (q->selection()->isRowSelected()) {
+        } else if (q->selection()->isRowSelected() && q->selection()->isSingular()) {
             actions.append(q->action("resizeRow"));
             actions.append(popupMenuActions["adjustRow"]);
             actions.append(popupMenuActions["separator5"]);
@@ -1327,16 +1308,3 @@ void CellToolBase::Private::createPopupMenuActions()
     connect(action, SIGNAL(triggered(bool)), q, SLOT(listChoosePopupMenu()));
     popupMenuActions.insert("listChoose", action);
 }
-
-void CellToolBase::Private::relayoutDocker(bool wide)
-{
-    // user input is moved accordingly to the "wide" param, the rest stays in one place
-    if (userInput->hasFocus()) return;  // do nothing while the user input has focus
-    widgetLayout->removeWidget(userInput);
-    if (wide)
-        widgetLayout->addWidget(userInput, 0, 3);
-    else
-        widgetLayout->addWidget(userInput, 1, 0, 1, 5);
-    hasWideLayout = wide;
-}
-
