@@ -50,13 +50,15 @@
 #include <QTextLayout>
 
 // KOffice
+#include <KoGlobal.h>
+#include <KoPostscriptPaintDevice.h>
 #include <KoZoomHandler.h>
 
 // KSpread
-#include "Canvas.h"
+#include "ApplicationSettings.h"
 #include "CellStorage.h"
 #include "Condition.h"
-#include "Doc.h"
+#include "Map.h"
 #include "RowColumnFormat.h"
 #include "Selection.h"
 #include "Sheet.h"
@@ -65,7 +67,6 @@
 #include "StyleManager.h"
 #include "Value.h"
 #include "ValueFormatter.h"
-#include "View.h"
 
 using namespace KSpread;
 
@@ -74,7 +75,7 @@ const int s_borderSpace = 1;
 class CellView::Private : public QSharedData
 {
 public:
-    Private( Style* defaultStyle, double defaultWidth, double defaultHeight )
+    Private( Style* defaultStyle, qreal defaultWidth, qreal defaultHeight )
         : style( *defaultStyle )
         , width( defaultWidth )
         , height( defaultHeight )
@@ -95,15 +96,15 @@ public:
     }
 
     Style style;
-    double  width;
-    double  height;
+    qreal  width;
+    qreal  height;
 
     // Position and dimension of displayed text.
     // Doc coordinate system; points; no zoom
-    double  textX;
-    double  textY;
-    double  textWidth;
-    double  textHeight;
+    qreal  textX;
+    qreal  textY;
+    qreal  textWidth;
+    qreal  textHeight;
 
     bool hidden         : 1;
     bool merged         : 1;
@@ -143,9 +144,9 @@ public:
 
 
 CellView::CellView( SheetView* sheetView )
-    : d( new Private( sheetView->sheet()->doc()->styleManager()->defaultStyle(),
-                      sheetView->sheet()->doc()->defaultColumnFormat()->width(),
-                      sheetView->sheet()->doc()->defaultRowFormat()->height() ) )
+    : d( new Private( sheetView->sheet()->map()->styleManager()->defaultStyle(),
+                      sheetView->sheet()->map()->defaultColumnFormat()->width(),
+                      sheetView->sheet()->map()->defaultRowFormat()->height() ) )
 {
 }
 
@@ -178,9 +179,9 @@ CellView::CellView( SheetView* sheetView, int col, int row )
             d->style.merge( *style );
     }
 
-    if ( cell.width() != sheetView->sheet()->doc()->defaultColumnFormat()->width() )
+    if ( cell.width() != sheetView->sheet()->map()->defaultColumnFormat()->width() )
         d->width = cell.width();
-    if ( cell.height() != sheetView->sheet()->doc()->defaultRowFormat()->height() )
+    if ( cell.height() != sheetView->sheet()->map()->defaultRowFormat()->height() )
         d->height = cell.height();
 
     if (sheet->columnFormat(col)->isHiddenOrFiltered() ||
@@ -211,7 +212,7 @@ CellView::CellView( SheetView* sheetView, int col, int row )
     {
         // Format the value appropriately and set the display text.
         // The format of the resulting value is used below to determine the alignment.
-        value = cell.doc()->formatter()->formatText(cell.value(), d->style.formatType(),
+        value = sheet->map()->formatter()->formatText(cell.value(), d->style.formatType(),
                                                     d->style.precision(), d->style.floatFormat(),
                                                     d->style.prefix(), d->style.postfix(),
                                                     d->style.currency().symbol());
@@ -260,17 +261,22 @@ Style CellView::style() const
     return d->style;
 }
 
-double CellView::textWidth() const
+qreal CellView::textWidth() const
 {
   return d->textWidth;
 }
 
-double CellView::textHeight() const
+qreal CellView::textHeight() const
 {
   return d->textHeight;
 }
 
-QString CellView::testAnchor( const Cell& cell, double x, double y ) const
+QRectF CellView::textRect() const
+{
+    return QRectF(d->textX, d->textY, d->textWidth, d->textWidth);
+}
+
+QString CellView::testAnchor( const Cell& cell, qreal x, qreal y ) const
 {
   if( cell.link().isEmpty() )
     return QString();
@@ -339,12 +345,12 @@ void CellView::paintCellContents( const QRectF& paintRect, QPainter& painter,
 
     // 1. Paint possible comment indicator.
     if ( !dynamic_cast<QPrinter*>(painter.device())
-            || cell.sheet()->print()->printCommentIndicator() )
+            || cell.sheet()->printSettings()->printCommentIndicator() )
         paintCommentIndicator( painter, coordinate, cell );
 
     // 2. Paint possible formula indicator.
     if ( !dynamic_cast<QPrinter*>(painter.device())
-            || cell.sheet()->print()->printFormulaIndicator() )
+            || cell.sheet()->printSettings()->printFormulaIndicator() )
     {
         paintFormulaIndicator( painter, coordinate, cell );
         paintMatrixElementIndicator( painter, coordinate, cell );
@@ -442,7 +448,7 @@ void CellView::paintCellBorders( const QRectF& paintRegion, QPainter& painter,
     // 1. Paint the default borders if we are on screen or if we are printing
     //    and the checkbox to do this is checked.
     if ( painter.device()->devType() != QInternal::Printer ||
-         sheet->print()->printGrid())
+         sheet->printSettings()->printGrid())
         paintDefaultBorders( painter, paintRegion, cellRect, paintBorder, cell );
 #endif
 
@@ -730,7 +736,7 @@ void CellView::paintDefaultBorders( QPainter& painter, const QRectF& paintRect,
 
     // Set the single-pixel width pen for drawing the borders with.
     // NOTE Stefan: Use a cosmetic pen (width = 0), because we want the grid always one pixel wide
-    painter.setPen( QPen( cell.doc()->gridColor(), 0, Qt::SolidLine ) );
+    painter.setPen( QPen( cell.sheet()->map()->settings()->gridColor(), 0, Qt::SolidLine ) );
 
     QLineF line;
 
@@ -830,7 +836,7 @@ void CellView::paintDefaultBorders( QPainter& painter, const QRectF& paintRect,
         }
 #endif
 
-        //painter.setPen( QPen( cell.doc()->gridColor(), 1, Qt::SolidLine ) );
+        //painter.setPen( QPen( cell.sheet()->map()->settings()->gridColor(), 1, Qt::SolidLine ) );
 
         // If we are on paper printout, we limit the length of the lines.
         // On paper, we always have full cells, on screen not.
@@ -903,7 +909,7 @@ void CellView::paintCommentIndicator( QPainter& painter,
     if ( ( !cell.comment().isEmpty() )
             && d->width > 10.0
             && d->height > 10.0
-            && ( cell.sheet()->print()->printCommentIndicator()
+            && ( cell.sheet()->printSettings()->printCommentIndicator()
             || ( !dynamic_cast<QPrinter*>(painter.device()) && cell.sheet()->getShowCommentIndicator() ) ) )
     {
         QColor penColor = Qt::red;
@@ -1081,9 +1087,7 @@ void CellView::paintText( QPainter& painter,
   }
 
   QPen tmpPen( textColorPrint );
-
-    // Set the font according to the current zoom.
-    QFont font = effectiveFont( paintDevice );
+    QFont font = d->style.font();
 
     // Check for red font color for negative values.
     if ( cell.value().isNumber()
@@ -1131,8 +1135,8 @@ void CellView::paintText( QPainter& painter,
 #endif
   painter.setPen( tmpPen );
 
-  double indent = 0.0;
-  double offsetCellTooShort = 0.0;
+  qreal indent = 0.0;
+  qreal offsetCellTooShort = 0.0;
   const Style::HAlign hAlign = d->style.halign();
 
     // Apply indent if text is align to left not when text is at right or middle.
@@ -1145,8 +1149,9 @@ void CellView::paintText( QPainter& painter,
   if ( hAlign == Style::Right && !cell.isEmpty() && !d->fittingWidth )
     offsetCellTooShort = 4;
 
-  const QFontMetricsF fontMetrics( font, paintDevice );
-  double offsetFont = 0.0;
+  KoPostscriptPaintDevice device;
+  const QFontMetricsF fontMetrics(font, &device);
+  qreal offsetFont = 0.0;
 
   if ( style().valign() == Style::Bottom && style().underline() )
     offsetFont = fontMetrics.underlinePos() + 1;
@@ -1175,13 +1180,13 @@ void CellView::paintText( QPainter& painter,
     const int angle = tmpAngle;
 
     painter.rotate( angle );
-    double x;
+    qreal x;
     if ( angle > 0 )
       x = indent + coordinate.x();
     else
       x = indent + coordinate.x()
           - ( fontMetrics.descent() + fontMetrics.ascent() ) * ::sin( angle * M_PI / 180 );
-    double y;
+    qreal y;
     if ( angle > 0 )
       y = coordinate.y() + d->textY;
     else
@@ -1199,7 +1204,7 @@ void CellView::paintText( QPainter& painter,
   else if ( tmpVerticalText && !d->displayText.isEmpty() ) {
       // Case 4: Vertical text.
       QStringList textLines = d->displayText.split('\n');
-      double dx = 0.0;
+      qreal dx = 0.0;
       for (int i = 0; i < textLines.count(); ++i)
       {
           QStringList textColumn;
@@ -1238,7 +1243,7 @@ void CellView::paintPageBorders(QPainter& painter, const QPointF& coordinate,
     if ( print->isColumnOnNewPage( cell.column() )
          && cell.row() <= print->printRange().bottom() )
     {
-      painter.setPen( cell.doc()->pageBorderColor() );
+      painter.setPen( cell.sheet()->map()->settings()->pageBorderColor() );
 
       if ( cell.sheet()->layoutDirection() == Qt::RightToLeft )
         line = QLineF( coordinate.x() + d->width, coordinate.y(),
@@ -1252,7 +1257,7 @@ void CellView::paintPageBorders(QPainter& painter, const QPointF& coordinate,
     if ( print->isRowOnNewPage( cell.row() ) &&
          ( cell.column() <= print->printRange().right() ) )
     {
-      painter.setPen( cell.doc()->pageBorderColor() );
+      painter.setPen( cell.sheet()->map()->settings()->pageBorderColor() );
       line = QLineF( coordinate.x(),  coordinate.y(),
                         coordinate.x() + d->width, coordinate.y() );
       painter.drawLine( line );
@@ -1261,7 +1266,7 @@ void CellView::paintPageBorders(QPainter& painter, const QPointF& coordinate,
     if ( paintBorder & RightBorder ) {
       if ( print->isColumnOnNewPage( cell.column() + 1 )
            && cell.row() <= print->printRange().bottom() ) {
-        painter.setPen( cell.doc()->pageBorderColor() );
+        painter.setPen( cell.sheet()->map()->settings()->pageBorderColor() );
 
         if ( cell.sheet()->layoutDirection() == Qt::RightToLeft )
           line = QLineF( coordinate.x(), coordinate.y(),
@@ -1276,7 +1281,7 @@ void CellView::paintPageBorders(QPainter& painter, const QPointF& coordinate,
     if ( paintBorder & BottomBorder ) {
       if ( print->isRowOnNewPage( cell.row() + 1 )
            && cell.column() <= print->printRange().right() ) {
-        painter.setPen( cell.doc()->pageBorderColor() );
+        painter.setPen( cell.sheet()->map()->settings()->pageBorderColor() );
         line = QLineF( coordinate.x(),  coordinate.y() + d->height,
                           coordinate.x() + d->width, coordinate.y() + d->height );
         painter.drawLine( line );
@@ -1696,7 +1701,7 @@ QString CellView::textDisplaying( const QFontMetricsF& fm, const Cell& cell )
     // Non-vertical text: the ordinary case.
 
     // Not enough space but align to left
-    double  len = 0.0;
+    qreal  len = 0.0;
 
     len = d->width;
 #if 0
@@ -1708,7 +1713,7 @@ QString CellView::textDisplaying( const QFontMetricsF& fm, const Cell& cell )
 #endif
 
     QString  tmp;
-    double   tmpIndent = 0.0;
+    qreal   tmpIndent = 0.0;
     if ( !cell.isEmpty() )
       tmpIndent = style().indentation();
 
@@ -1773,10 +1778,10 @@ QString CellView::textDisplaying( const QFontMetricsF& fm, const Cell& cell )
     // Vertical text.
 
     const RowFormat  *rl = cell.sheet()->rowFormat( cell.row() );
-    double      tmpIndent = 0.0;
+    qreal      tmpIndent = 0.0;
 
     // Not enough space but align to left.
-    double  len = 0.0;
+    qreal  len = 0.0;
 
     len = d->width;
 #if 0
@@ -1818,22 +1823,6 @@ QString CellView::textDisplaying( const QFontMetricsF& fm, const Cell& cell )
 //                        End of Painting
 // ================================================================
 
-
-
-// Get the effective font.
-//
-// Used in makeLayout().
-//
-QFont CellView::effectiveFont( QPaintDevice* paintDevice ) const
-{
-    QFont tmpFont = paintDevice ? QFont( d->style.font(), paintDevice ) : d->style.font();
-    // Scale the font size according to the current zoom.
-//     tmpFont.setPointSizeF( tmpFont.pointSizeF() / cell.doc()->resolutionY() );
-    return tmpFont;
-}
-
-
-
 // ================================================================
 //                              Layout
 
@@ -1855,19 +1844,20 @@ void CellView::makeLayout( SheetView* sheetView, const Cell& cell )
     // painting of it.  Now it is time to see if the contents fits into
     // the cell and, if not, maybe rearrange the outtext a bit.
 
-    // First, Determine the correct font with zoom taken into account.
-    const QFont font(effectiveFont(sheetView->paintDevice()));
-    const QFontMetricsF fontMetrics(font, sheetView->paintDevice());
+    // First, create a device independent font and its metrics.
+    KoPostscriptPaintDevice device;
+    const QFont font(d->style.font(), &device);
+    const QFontMetricsF fontMetrics(font, &device);
 
     // Then calculate text dimensions, i.e. d->textWidth and d->textHeight,
-    // and check wether the text fits into the cell dimension by the way.
+    // and check whether the text fits into the cell dimension by the way.
     d->calculateTextSize(font, fontMetrics);
 
     // Obscure horizontal cells, if necessary.
     if (!d->fittingWidth)
     {
         obscureHorizontalCells(sheetView, cell);
-        // Recalculate the text dimensions and check wether the text fits.
+        // Recalculate the text dimensions and check whether the text fits.
         d->calculateTextSize(font, fontMetrics);
     }
 
@@ -1875,7 +1865,7 @@ void CellView::makeLayout( SheetView* sheetView, const Cell& cell )
     if (!d->fittingHeight)
     {
         obscureVerticalCells(sheetView, cell);
-        // Recalculate the text dimensions and check wether the text fits.
+        // Recalculate the text dimensions and check whether the text fits.
         d->calculateTextSize(font, fontMetrics);
     }
 
@@ -1885,7 +1875,7 @@ void CellView::makeLayout( SheetView* sheetView, const Cell& cell )
         // Truncate the output text.
         d->displayText = textDisplaying(fontMetrics, cell);
 //         d->truncateText(font, fontMetrics);
-//         // Recalculate the text dimensions and check wether the text fits.
+//         // Recalculate the text dimensions and check whether the text fits.
 //         d->calculateTextSize(font, fontMetrics);
     }
 
@@ -1898,12 +1888,12 @@ void CellView::calculateCellDimension( const Cell& cell )
 {
     Q_UNUSED( cell );
 #if 0
-  double width  = cell.sheet()->columnFormat( cell.column() )->width();
-  double height = cell.sheet()->rowFormat( cell.row() )->height();
+  qreal width  = cell.sheet()->columnFormat( cell.column() )->width();
+  qreal height = cell.sheet()->rowFormat( cell.row() )->height();
 
   // Calculate extraWidth and extraHeight if we have a merged cell.
   if ( cell.testFlag( Cell::Flag_Merged ) ) {
-    // FIXME: Introduce double extraWidth/Height here and use them
+    // FIXME: Introduce qreal extraWidth/Height here and use them
     //        instead (see FIXME about this in paintCell()).
 
     for ( int x = cell.column() + 1; x <= cell.column() + d->obscuredCellsX; x++ )
@@ -1931,19 +1921,19 @@ void CellView::calculateCellDimension( const Cell& cell )
 void CellView::textOffset( const QFontMetricsF& fontMetrics, const Cell& cell )
 {
     Q_UNUSED(cell)
-    const double ascent = fontMetrics.ascent();
+    const qreal ascent = fontMetrics.ascent();
     const Style::HAlign hAlign = d->style.halign();
     const Style::VAlign vAlign = d->style.valign();
     const int tmpAngle = d->style.angle();
     const bool tmpVerticalText = d->style.verticalText();
     const bool tmpMultiRow = d->style.wrapText() || d->displayText.contains('\n');
 
-  double  w = d->width;
-  double  h = d->height;
+  qreal  w = d->width;
+  qreal  h = d->height;
 
   // doc coordinate system; no zoom applied
-  const double effTop = s_borderSpace + 0.5 * d->style.topBorderPen().width();
-  const double effBottom = h - s_borderSpace - 0.5 * d->style.bottomBorderPen().width();
+  const qreal effTop = s_borderSpace + 0.5 * d->style.topBorderPen().width();
+  const qreal effBottom = h - s_borderSpace - 0.5 * d->style.bottomBorderPen().width();
 
   // Calculate d->textY based on the vertical alignment and a few
   // other inputs.
@@ -2100,148 +2090,16 @@ void CellView::textOffset( const QFontMetricsF& fontMetrics, const Cell& cell )
   }
 }
 
-
-// Recalculate the current text dimensions, i.e. d->textWidth and
-// d->textHeight.
-//
-// Used in makeLayout().
-//
-void CellView::textSize( const QFontMetricsF& fm )
-{
-    const Style::VAlign vAlign = d->style.valign();
-    const int tmpAngle = d->style.angle();
-    const bool tmpVerticalText = d->style.verticalText();
-    const bool fontUnderlined = d->style.underline();
-
-    // Set d->textWidth and d->textHeight to correct values according to
-    // if the text is horizontal, vertical or rotated.
-    if ( !tmpVerticalText && !tmpAngle ) {
-        // Horizontal text.
-
-        d->textWidth = ( fm.size( 0, d->displayText ).width() );
-        double offsetFont = 0.0;
-        if ( ( vAlign == Style::Bottom ) && fontUnderlined ) {
-            offsetFont = fm.underlinePos() + 1;
-        }
-
-        d->textHeight = ( fm.ascent() + fm.descent() + offsetFont )
-                * ( d->displayText.count( '\n' ) + 1 );
-    }
-    else if ( tmpAngle!= 0 ) {
-        // Rotated text.
-
-        const double height = fm.ascent() + fm.descent();
-        const double width  = fm.width( d->displayText );
-        d->textHeight = height * ::cos( tmpAngle * M_PI / 180 ) + qAbs( width * ::sin( tmpAngle * M_PI / 180 ) );
-
-        d->textWidth = qAbs( height * ::sin( tmpAngle * M_PI / 180 ) ) + width * ::cos( tmpAngle * M_PI / 180 );
-    }
-    else {
-        // Vertical text.
-
-        double width = 0.0;
-        for ( int i = 0; i < d->displayText.length(); i++ )
-            width = qMax( width, fm.width( d->displayText.at( i ) ) );
-
-        d->textWidth  = width;
-        d->textHeight = ( fm.ascent() + fm.descent() ) * d->displayText.length();
-    }
-}
-
-void CellView::breakLines( const QFontMetricsF& fontMetrics )
-{
-  if ( style().wrapText() &&
-       d->textWidth > ( d->width - 2 * s_borderSpace
-           - 0.5 * style().leftBorderPen().width() - 0.5 * style().rightBorderPen().width() &&
-       // FIXME Stefan: consider this later in the wrap algo.
-       d->textHeight + fontMetrics.ascent() + fontMetrics.descent() < d->height ) )
-  {
-    // don't remove the existing LF, these are intended line wraps (whishlist #9881)
-    QString outText = d->displayText;
-
-    // Break the line at appropriate places, i.e. spaces, if
-    // necessary.  This means to change the spaces where breaks occur
-    // into newlines.
-    if ( !outText.contains(' ') )
-    {
-      // no spaces -> impossible to wrap
-      return;
-    }
-    else
-    {
-      d->displayText.clear();
-
-      // Make sure that we have a space at the end.
-      outText += ' ';
-
-      int start = 0;    // Start of the line we are handling now
-      int breakpos = 0;   // The next candidate pos to break the string
-      int pos1 = 0;
-      const int availableWidth = (int) ( d->width - 2 * s_borderSpace
-          - 0.5 * style().leftBorderPen().width() - 0.5 * style().rightBorderPen().width() );
-
-      do {
-
-        breakpos = outText.indexOf( ' ', breakpos );
-        int linefeed = outText.indexOf( '\n', pos1 );
-
-//         kDebug(36004) <<"start:" << start <<"; breakpos:" << breakpos <<"; pos1:" << pos1 <<"; linefeed:" << linefeed;
-
-        //don't miss LF as a position to calculate current lineWidth
-        int work_breakpos = breakpos;
-        if (pos1 < linefeed && linefeed < breakpos)
-          work_breakpos = linefeed;
-
-        const double lineWidth = fontMetrics.width( d->displayText.mid( start, (pos1 - start) )
-                                                    + outText.mid( pos1, work_breakpos - pos1 ) );
-
-        //linefeed could be -1 when no linefeed is found!
-        if (breakpos > linefeed && linefeed > 0)
-        {
-//           kDebug(36004) <<"applying linefeed to start;";
-          start = linefeed;
-        }
-
-        if ( lineWidth <= availableWidth ) {
-            // We have room for the rest of the line.  End it here.
-          d->displayText += outText.mid( pos1, breakpos - pos1 );
-          pos1 = breakpos;
-        }
-        else {
-          // Still not enough room.  Try to split further.
-          if ( outText.at( pos1 ) == ' ' )
-            pos1++;
-
-          if ( pos1 != 0 && breakpos != -1 ) {
-            d->displayText += '\n' + outText.mid( pos1, breakpos - pos1 );
-          }
-          else
-            d->displayText += outText.mid( pos1, breakpos - pos1 );
-
-          start = pos1;
-          pos1 = breakpos;
-        }
-
-        breakpos++;
-      } while( outText.indexOf( ' ', breakpos ) != -1 );
-
-      // cut off the last line feed
-      if (!outText.isEmpty() && outText[outText.length()-1] == '\n')
-          outText = outText.left(outText.length()-1);
-    }
-  }
-}
-
 void CellView::obscureHorizontalCells( SheetView* sheetView, const Cell& masterCell )
 {
     if ( d->hidden )
         return;
 
-    double extraWidth = 0.0;
+    qreal extraWidth = 0.0;
     const Style::HAlign align = d->style.halign();
 
     // Get indentation.  This is only used for left aligned text.
-    double indent = 0.0;
+    qreal indent = 0.0;
     if ( align == Style::Left && !masterCell.isEmpty() )
         indent = style().indentation();
 
@@ -2324,7 +2182,7 @@ void CellView::obscureVerticalCells( SheetView* sheetView, const Cell& masterCel
     if ( d->hidden )
         return;
 
-    double extraHeight = 0.0;
+    qreal extraHeight = 0.0;
 
     // Do we have to occupy additional cells at the bottom ?
     //
@@ -2381,36 +2239,48 @@ void CellView::drawText( QPainter& painter, const QPointF& location, const QStri
 {
     Q_UNUSED( cell )
 
-    const QFontMetricsF fontMetrics(effectiveFont(painter.device()), painter.device());
-    const double leading = fontMetrics.leading();
+    const qreal scaleX = POINT_TO_INCH(qreal(KoGlobal::dpiX()));
+    const qreal scaleY = POINT_TO_INCH(qreal(KoGlobal::dpiY()));
+
+    // Qt scales the font already with the logical resolution. Do not do it twice!
+    painter.save();
+    painter.scale(1.0 / scaleX, 1.0 / scaleY);
+
+    const QFontMetricsF fontMetrics(d->style.font());
+    const qreal leading = fontMetrics.leading() / scaleY;
 
     const QTextOption options = d->textOptions();
 
-    double offset = 1.0 - fontMetrics.ascent();
+    qreal offset = 1.0 - fontMetrics.ascent() / scaleY;
     for (int i = 0; i < textLines.count(); ++i)
     {
-        QTextLayout textLayout(textLines[i], effectiveFont(painter.device()));
+        QTextLayout textLayout(textLines[i], d->style.font());
         textLayout.setCacheEnabled(true);
         textLayout.setTextOption(options);
         textLayout.beginLayout();
-        double height = 0.0;
+        qreal height = 0.0;
         forever
         {
-            if (offset + height + leading + fontMetrics.height() > d->height)
+            if (offset + height + leading + fontMetrics.height() / scaleY > d->height)
                 break;
             QTextLine line = textLayout.createLine();
             if (!line.isValid())
                 break;
-            line.setLineWidth(d->width);
+            line.setLineWidth(scaleX * (d->width - 2 * s_borderSpace
+                              - 0.5 * d->style.leftBorderPen().width()
+                              - 0.5 * d->style.rightBorderPen().width()));
             height += leading;
-            line.setPosition(QPoint(0, qRound(height)));
+            line.setPosition(QPoint(scaleX * (s_borderSpace + 0.5 * d->style.leftBorderPen().width()),
+                                    qRound(scaleX * height)));
             height += line.height();
         }
         textLayout.endLayout();
 
-        textLayout.draw(&painter, QPointF(location.x(), location.y() + offset));
+        textLayout.draw(&painter, QPointF(scaleX * location.x(), scaleY * (location.y() + offset)));
         offset += height;
     }
+
+    painter.restore();
 }
 
 void CellView::obscure( int col, int row )
@@ -2440,12 +2310,12 @@ bool CellView::obscuresCells() const
     return !d->obscured && ( d->obscuredCellsX != 0 || d->obscuredCellsY != 0 );
 }
 
-double CellView::cellHeight() const
+qreal CellView::cellHeight() const
 {
     return d->height;
 }
 
-double CellView::cellWidth() const
+qreal CellView::cellWidth() const
 {
     return d->width;
 }
@@ -2489,7 +2359,7 @@ void CellView::Private::calculateTextSize(const QFont& font, const QFontMetricsF
 void CellView::Private::calculateHorizontalTextSize(const QFont& font, const QFontMetricsF& fontMetrics)
 {
     const QStringList textLines = displayText.split('\n');
-    const double leading = fontMetrics.leading();
+    const qreal leading = fontMetrics.leading();
     const QTextOption options = textOptions();
 
     textHeight = 0.0;
@@ -2509,9 +2379,9 @@ void CellView::Private::calculateHorizontalTextSize(const QFont& font, const QFo
                 break; // forever
             line.setLineWidth(width);
             textHeight += leading + line.height();
-            if (textHeight > (height - 2 * s_borderSpace
-                              - 0.5 * style.topBorderPen().width()
-                              - 0.5 * style.bottomBorderPen().width()))
+            if ((textHeight - fontMetrics.descent()) > (height - 2 * s_borderSpace
+                                                        - 0.5 * style.topBorderPen().width()
+                                                        - 0.5 * style.bottomBorderPen().width()))
             {
                 fittingHeight = false;
                 break; // forever
@@ -2542,9 +2412,9 @@ void CellView::Private::calculateVerticalTextSize(const QFont& font, const QFont
 void CellView::Private::calculateAngledTextSize(const QFont& font, const QFontMetricsF& fontMetrics)
 {
     Q_UNUSED(font)
-    const double angle = style.angle();
-    const double height = fontMetrics.ascent() + fontMetrics.descent();
-    const double width  = fontMetrics.width(displayText);
+    const qreal angle = style.angle();
+    const qreal height = fontMetrics.ascent() + fontMetrics.descent();
+    const qreal width  = fontMetrics.width(displayText);
     textHeight = height * ::cos(angle * M_PI / 180) + qAbs(width * ::sin(angle * M_PI / 180));
     textWidth = qAbs(height * ::sin(angle * M_PI / 180)) + width * ::cos(angle * M_PI / 180);
     fittingHeight = textHeight <= this->width;
@@ -2563,12 +2433,11 @@ void CellView::Private::truncateText(const QFont& font, const QFontMetricsF& fon
 
 void CellView::Private::truncateHorizontalText(const QFont& font, const QFontMetricsF& fontMetrics)
 {
-    Q_UNUSED(font)
     if (!style.wrapText())
     {
         const QStringList textLines = displayText.split('\n');
         displayText.clear();
-        double height = font.pointSizeF();
+        qreal height = font.pointSizeF();
         for (int i = 0; i < textLines.count(); ++i)
         {
             if (height > this->height)
