@@ -20,19 +20,20 @@
 #include "Editors.h"
 
 // KSpread
-#include "Canvas.h"
+#include "ApplicationSettings.h"
 #include "Cell.h"
-#include "Doc.h"
+#include "CellToolBase.h"
 #include "Formula.h"
 #include "Functions.h"
+#include "Map.h"
 #include "NamedAreaManager.h"
 #include "Selection.h"
 #include "Sheet.h"
 #include "Style.h"
-#include "View.h"
 
 // KOffice
-#include <KoZoomHandler.h>
+#include <KoGlobal.h>
+#include <KoViewConverter.h>
 
 // KDE
 #include <kdebug.h>
@@ -56,11 +57,11 @@
 #include <QKeyEvent>
 #include <QLabel>
 #include <QResizeEvent>
-#include <QScrollBar>
 #include <QTimer>
 #include <QToolButton>
 #include <QToolTip>
 #include <QVBoxLayout>
+#include <QScrollBar>
 
 #include "commands/NamedAreaCommand.h"
 
@@ -80,25 +81,25 @@ class FormulaEditorHighlighter::Private
 public:
   Private()
   {
-    canvas = 0;
+    selection = 0;
     tokens = Tokens();
     rangeCount = 0;
     rangeChanged = false;
   }
 
   // source for cell reference checking
-  Canvas* canvas;
+  Selection* selection;
   Tokens tokens;
   uint rangeCount;
   bool rangeChanged;
 };
 
 
-FormulaEditorHighlighter::FormulaEditorHighlighter(QTextEdit* textEdit, Canvas* canvas)
+FormulaEditorHighlighter::FormulaEditorHighlighter(QTextEdit* textEdit, Selection* selection)
     : QSyntaxHighlighter(textEdit)
     , d( new Private )
 {
-  d->canvas = canvas;
+    d->selection = selection;
 }
 
 FormulaEditorHighlighter::~FormulaEditorHighlighter()
@@ -114,7 +115,7 @@ const Tokens& FormulaEditorHighlighter::formulaTokens() const
 void FormulaEditorHighlighter::highlightBlock( const QString& text )
 {
   // reset syntax highlighting
-  setFormat(0, text.length(), Qt::black);
+  setFormat(0, text.length(), QApplication::palette().text().color());
 
   // save the old ones to identify range changes
   Tokens oldTokens = d->tokens;
@@ -130,7 +131,7 @@ void FormulaEditorHighlighter::highlightBlock( const QString& text )
   uint oldRangeCount = d->rangeCount;
 
   d->rangeCount = 0;
-  QList<QColor> colors = d->canvas->choice()->colors();
+  QList<QColor> colors = d->selection->colors();
   QList<QString> alreadyFoundRanges;
 
   for (int i = 0; i < d->tokens.count(); ++i)
@@ -174,6 +175,8 @@ void FormulaEditorHighlighter::highlightBlock( const QString& text )
       case Token::Integer:     // 14, 3, 1977
       case Token::Float:       // 3.141592, 1e10, 5.9e-7
       case Token::String:      // "KOffice", "The quick brown fox..."
+      case Token::Error:
+          break;
       case Token::Operator:    // +, *, /, -
         {
             switch (token.asOperator())
@@ -340,16 +343,22 @@ FunctionCompletion::FunctionCompletion( CellEditor* editor )
   d->completionListBox->setFrameStyle( QFrame::NoFrame );
 //   d->completionListBox->setVariableWidth( true );
   d->completionListBox->installEventFilter( this );
-  connect( d->completionListBox, SIGNAL(selected(const QString&)), this,
-    SLOT(itemSelected(const QString&)) );
-  connect( d->completionListBox, SIGNAL(highlighted(const QString&)), this,
-    SLOT(itemSelected(const QString&)) );
+  connect( d->completionListBox, SIGNAL(currentRowChanged(int)), SLOT(itemSelected()) );
+  // When items are activated on single click, also change the help page on mouse-over, otherwise there is no (easy) way to get
+  // the help (with the mouse) without inserting the function
+  if ( d->completionListBox->style()->styleHint(QStyle::SH_ItemView_ActivateItemOnSingleClick, 0, d->completionListBox) ) {
+    connect( d->completionListBox, SIGNAL(itemEntered(QListWidgetItem*)), this, SLOT(itemSelected(QListWidgetItem*)) );
+    d->completionListBox->setMouseTracking(true);
+  }
+
+  connect( d->completionListBox, SIGNAL(activated(QModelIndex)), SLOT(doneCompletion()) );
   layout->addWidget( d->completionListBox );
 
   d->hintLabel = new QLabel( 0, (Qt::WFlags)
     Qt::WStyle_Customize | Qt::WStyle_NoBorder | Qt::WStyle_Tool |  Qt::WX11BypassWM );
   d->hintLabel->setFrameStyle( QFrame::Plain | QFrame::Box );
   d->hintLabel->setPalette( QToolTip::palette() );
+  d->hintLabel->setWordWrap( true );
   d->hintLabel->hide();
 }
 
@@ -359,8 +368,15 @@ FunctionCompletion::~FunctionCompletion()
       delete d;
 }
 
-void FunctionCompletion::itemSelected( const QString& item )
+void FunctionCompletion::itemSelected( QListWidgetItem* listItem )
 {
+    QString item;
+    if ( listItem ) {
+        item = listItem->text();
+    } else {
+        item = d->completionListBox->currentItem()->text();
+    }
+
     KSpread::FunctionDescription* desc;
     desc = KSpread::FunctionRepository::self()->functionInfo(item);
     if(!desc)
@@ -483,7 +499,7 @@ class CellEditor::Private
 {
 public:
   Cell                      cell;
-  Canvas*                   canvas;
+  Selection*                selection;
   KTextEdit*                textEdit;
   FormulaEditorHighlighter* highlighter;
   FunctionCompletion*       functionCompletion;
@@ -505,12 +521,12 @@ public:
 };
 
 
-CellEditor::CellEditor( const Cell& _cell, Canvas* _parent, bool captureAllKeyEvents, const char* /*_name*/ )
-    : QWidget( _parent )
+CellEditor::CellEditor(QWidget* parent, Selection* selection, bool captureAllKeyEvents)
+    : QWidget(parent)
     , d( new Private )
 {
-  d->cell = _cell;
-  d->canvas = _parent;
+  d->cell = Cell(selection->activeSheet(), selection->marker());
+  d->selection = selection;
   d->textEdit = new KTextEdit(this);
   d->globalCursorPos = QPoint();
   d->captureAllKeyEvents = captureAllKeyEvents;
@@ -524,7 +540,10 @@ CellEditor::CellEditor( const Cell& _cell, Canvas* _parent, bool captureAllKeyEv
   d->currentToken = 0;
   d->rangeCount = 0;
 
-//TODO - Get rid of QTextEdit margins, this doesn't seem easily possible in Qt 3.3, so a job for Qt 4 porting.
+  QBoxLayout* layout = new QBoxLayout(QBoxLayout::LeftToRight, this);
+  layout->setMargin(0);
+  layout->setSpacing(0);
+  layout->addWidget(d->textEdit);
 
   d->textEdit->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
   d->textEdit->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
@@ -532,7 +551,7 @@ CellEditor::CellEditor( const Cell& _cell, Canvas* _parent, bool captureAllKeyEv
   d->textEdit->setLineWidth(0);
   d->textEdit->installEventFilter( this );
 
-  d->highlighter = new FormulaEditorHighlighter( d->textEdit, _parent );
+  d->highlighter = new FormulaEditorHighlighter( d->textEdit, selection );
 
   d->functionCompletion = new FunctionCompletion( this );
   d->functionCompletionTimer = new QTimer( this );
@@ -542,42 +561,35 @@ CellEditor::CellEditor( const Cell& _cell, Canvas* _parent, bool captureAllKeyEv
   connect( d->functionCompletionTimer, SIGNAL( timeout() ),
     SLOT( triggerFunctionAutoComplete() ) );
 
-  if ( !_cell.style().wrapText() )
+  if ( !d->cell.style().wrapText() )
     d->textEdit->setWordWrapMode( QTextOption::NoWrap );
   else
     d->textEdit->setWordWrapMode( QTextOption::WordWrap );
 
 #if 0 // FIXME Implement a completion aware KTextEdit.
-  d->textEdit->setCompletionMode(canvas()->view()->doc()->completionMode());
-  d->textEdit->setCompletionObject(&canvas()->view()->doc()->completion(), true);
+  d->textEdit->setCompletionMode(selection()->view()->doc()->completionMode());
+  d->textEdit->setCompletionObject(&selection()->view()->doc()->completion(), true);
 #endif
 
   setFocusProxy( d->textEdit );
 
   connect( d->textEdit, SIGNAL( cursorPositionChanged() ), this, SLOT (slotCursorPositionChanged()));
   connect( d->textEdit, SIGNAL( textChanged() ), this, SLOT( slotTextChanged() ) );
-
+    connect(d->textEdit->document(), SIGNAL(modificationChanged(bool)),
+            this, SIGNAL(modificationChanged(bool)));
 // connect( d->textEdit, SIGNAL(completionModeChanged( KGlobalSettings::Completion )),this,SLOT (slotCompletionModeChanged(KGlobalSettings::Completion)));
 
   // A choose should always start at the edited cell
-//  canvas()->setChooseMarkerRow( canvas()->selection()->marker().y() );
-//  canvas()->setChooseMarkerColumn( canvas()->selection()->marker().x() );
+//  selection()->setChooseMarkerRow( selection()->d->selection->marker().y() );
+//  selection()->setChooseMarkerColumn( selection()->d->selection->marker().x() );
 
-  // set font size according to zoom factor
-  QFont font( _cell.style().font() );
-  font.setPointSizeF( 0.01 * _parent->view()->zoomHandler()->zoomInPercent() * font.pointSizeF() );
-  d->textEdit->setFont( font );
-
-  if (d->fontLength == 0)
-  {
-    QFontMetrics fm( d->textEdit->font() );
-    d->fontLength = fm.width('x');
-  }
+  connect( d->selection, SIGNAL( destroyed() ), SLOT( slotSelectionDestroyed() ) );
 }
 
 CellEditor::~CellEditor()
 {
-  canvas()->endChoose();
+  if (selection())
+    selection()->endReferenceSelection();
 
   delete d->highlighter;
   delete d->functionCompletion;
@@ -590,9 +602,9 @@ const Cell& CellEditor::cell() const
   return d->cell;
 }
 
-Canvas* CellEditor::canvas() const
+Selection* CellEditor::selection() const
 {
-  return d->canvas;
+  return d->selection;
 }
 
 QPoint CellEditor::globalCursorPosition() const
@@ -668,11 +680,13 @@ void CellEditor::functionAutoComplete( const QString& item )
   d->textEdit->setTextCursor( textCursor );
   d->textEdit->insertPlainText( item );
   d->textEdit->blockSignals( false );
+  // call slotTextChanged to sync the text between the inline editor and the editor in the docker
+  slotTextChanged();
 }
 
 void CellEditor::slotCursorPositionChanged()
 {
-//   kDebug() << endl <<"position:" << cursorPosition();
+  // kDebug() <<"position:" << cursorPosition()<<endl;;
 
   // TODO Stefan: optimize this function!
 
@@ -758,8 +772,8 @@ void CellEditor::slotCursorPositionChanged()
   // only change the active sub region, if we have found one.
   if (regionStart != -1)
   {
-    d->canvas->choice()->setActiveElement(currentRange);
-    d->canvas->choice()->setActiveSubRegion(regionStart, regionEnd-regionStart+1);
+    selection()->setActiveElement(currentRange);
+    selection()->setActiveSubRegion(regionStart, regionEnd-regionStart+1);
   }
 
   // triggered by keyboard action?
@@ -769,14 +783,12 @@ void CellEditor::slotCursorPositionChanged()
     {
       d->highlighter->resetRangeChanged();
 
-      disconnect( d->canvas->choice(), SIGNAL(changed(const Region&)),
-                  d->canvas->view(), SLOT(slotScrollChoice(const Region&)) );
-      d->canvas->doc()->emitBeginOperation();
+      //selection()->blockSignals(true);
       setUpdateChoice(false);
 
       Tokens tokens = d->highlighter->formulaTokens();
-      d->canvas->choice()->update(); // set the old one dirty
-      d->canvas->choice()->clear();
+      selection()->update(); // set the old one dirty
+      selection()->clear();
       Region tmpRegion;
       Region::ConstIterator it;
 
@@ -791,26 +803,24 @@ void CellEditor::slotCursorPositionChanged()
         Token::Type type = token.type();
         if (type == Token::Cell || type == Token::Range)
         {
-          Region region(token.text(), d->canvas->view()->doc()->map(), d->canvas->activeSheet());
+          Region region(token.text(), selection()->activeSheet()->map(), selection()->activeSheet());
           it = region.constBegin();
 
-          if (!alreadyUsedRegions.contains(region))
+          if (it != region.constEnd() && !alreadyUsedRegions.contains(region))
           {
             QRect r=(*it)->rect();
 
-            if (d->canvas->choice()->isEmpty())
-                d->canvas->choice()->initialize((*it)->rect(), (*it)->sheet());
+            if (selection()->isEmpty())
+                selection()->initialize((*it)->rect(), (*it)->sheet());
             else
-                d->canvas->choice()->extend((*it)->rect(), (*it)->sheet());
+                selection()->extend((*it)->rect(), (*it)->sheet());
 
             alreadyUsedRegions.append(region);
           }
         }
       }
       setUpdateChoice(true);
-      d->canvas->doc()->emitEndOperation();
-      connect( d->canvas->choice(), SIGNAL(changed(const Region&)),
-               d->canvas->view(), SLOT(slotScrollChoice(const Region&)) );
+      //selection()->blockSignals(false);
     }
   }
 }
@@ -830,38 +840,34 @@ void CellEditor::copy()
   d->textEdit->copy();
 }
 
-void CellEditor::setEditorFont(QFont const & font, bool updateSize, KoZoomHandler* zoomHandler)
+void CellEditor::setEditorFont(QFont const & font, bool updateSize, const KoViewConverter *viewConverter)
 {
-  QFont tmpFont( font );
-  tmpFont.setPointSizeF( 0.01 * zoomHandler->zoomInPercent() * tmpFont.pointSizeF() );
-  d->textEdit->setFont( tmpFont );
+    const double scaleY = POINT_TO_INCH(double(KoGlobal::dpiY()));
+    d->textEdit->setFont(QFont(font.family(), viewConverter->documentToViewY(font.pointSizeF()) / scaleY));
 
-  if (updateSize)
-  {
-    QFontMetrics fm( d->textEdit->font() );
-    d->fontLength = fm.width('x');
-
-    int mw = fm.width( d->textEdit->toPlainText() ) + d->fontLength;
-    // don't make it smaller: then we would have to repaint the obscured cells
-    if (mw < width())
-      mw = width();
-
-    int mh = fm.height();
-    if (mh < height())
-      mh = height();
-
-    setGeometry(x(), y(), mw, mh);
-  }
+    if (updateSize) {
+        QFontMetrics fontMetrics(d->textEdit->font());
+        d->fontLength = 0.0; // fontMetrics.width('x');
+        int width = fontMetrics.width(d->textEdit->toPlainText()) + d->fontLength;
+        // don't make it smaller: then we would have to repaint the obscured cells
+        if (width < this->width())
+            width = this->width();
+        int height = fontMetrics.height();
+        if (height < this->height())
+            height = this->height();
+        setGeometry(x(), y(), width, height);
+    }
 }
 
 void CellEditor::slotCompletionModeChanged(KGlobalSettings::Completion _completion)
 {
-  canvas()->view()->doc()->setCompletionMode( _completion );
+  selection()->activeSheet()->map()->settings()->setCompletionMode( _completion );
 }
 
 void CellEditor::slotTextChanged()
 {
-//   kDebug() ;
+    // Fix the position.
+    d->textEdit->verticalScrollBar()->setValue(4);
 
   //FIXME - text() may return richtext?
   QString t = text();
@@ -931,9 +937,10 @@ void CellEditor::slotTextChanged()
       return;
     }
   }
-
-  canvas()->view()->editWidget()->setText( t );
-  // canvas()->view()->editWidget()->textCursor().setPosition( d->textEdit->cursorPosition() );
+  
+  if (hasFocus())  // update the external editor, but only if we have focus
+    emit textChanged(d->textEdit->toPlainText());
+  // selection()->view()->editWidget()->textCursor().setPosition( d->textEdit->cursorPosition() );
 }
 
 void CellEditor::setCheckChoice(bool state)
@@ -955,46 +962,51 @@ bool CellEditor::checkChoice()
   QString text = d->textEdit->toPlainText();
   if ( text[0] != '=' )
   {
-    canvas()->setChooseMode(false);
+    selection()->setReferenceSelectionMode(false);
+//    d->checkChoice = true;
+    return true;
   }
-  else
+
+  // switch to reference selection mode if we haven't yet
+  selection()->startReferenceSelection();
+
+  // if we don't have focus, we show highlighting, but don't do the rest
+  if (!hasFocus()) {
+    selection()->setReferenceSelectionMode(false);
+//    d->checkChoice = true;
+    return true;
+  }
+
+  int cur = d->textEdit->textCursor().position();
+
+  Tokens tokens = d->highlighter->formulaTokens();
+
+  if (tokens.count())  // formula not empty?
   {
-    int cur = d->textEdit->textCursor().position();
-
-    Tokens tokens = d->highlighter->formulaTokens();
-
-    // empty formula?
-    if (tokens.count() < 1)
+    Token token;
+    for (int i = 0; i < tokens.count(); ++i)
     {
-      canvas()->startChoose();
+      if (tokens[i].pos() >= cur - 1) // without '='
+      {
+        break;
+      }
+      token = tokens[i];
+      d->currentToken = i;
+    }
+
+    Token::Type type = token.type();
+    if (type == Token::Operator && token.asOperator() != Token::RightPar)
+    {
+      selection()->setReferenceSelectionMode(true);
+    }
+    else if (type == Token::Cell || type == Token::Range)
+    {
+      d->length_namecell = token.text().length();
+      selection()->setReferenceSelectionMode(true);
     }
     else
     {
-      Token token;
-      for (int i = 0; i < tokens.count(); ++i)
-      {
-        if (tokens[i].pos() >= cur - 1) // without '='
-        {
-          break;
-        }
-        token = tokens[i];
-        d->currentToken = i;
-      }
-
-      Token::Type type = token.type();
-      if (type == Token::Operator && token.asOperator() != Token::RightPar)
-      {
-        canvas()->setChooseMode(true);
-      }
-      else if (type == Token::Cell || type == Token::Range)
-      {
-        d->length_namecell = token.text().length();
-        canvas()->setChooseMode(true);
-      }
-      else
-      {
-        canvas()->setChooseMode(false);
-      }
+      selection()->setReferenceSelectionMode(false);
     }
   }
 
@@ -1016,7 +1028,7 @@ void CellEditor::updateChoice()
     // prevent recursion
     d->updatingChoice = true;
 
-    Selection* choice = d->canvas->choice();
+    Selection* choice = selection();
 
     if (choice->isEmpty())
         return;
@@ -1063,7 +1075,7 @@ void CellEditor::updateChoice()
     setCheckChoice( true );
     setCursorPosition( start + d->length_namecell );
 
-    d->canvas->view()->editWidget()->setText( newText );
+    emit textChanged(d->textEdit->toPlainText());
 
     d->updatingChoice = false;
 }
@@ -1121,7 +1133,7 @@ void CellEditor::handleKeyPressEvent( QKeyEvent * _ev )
     d->textEdit->setPlainText(newString);
     setCursorPosition( cur );
 
-    _ev->accept();
+    _ev->accept(); // QKeyEvent
 
     return;
   }
@@ -1141,15 +1153,17 @@ QString CellEditor::text() const
     return d->textEdit->toPlainText();
 }
 
-void CellEditor::setText(const QString& text)
+void CellEditor::setText(const QString& text, int cursorPos)
 {
+    if (text == d->textEdit->toPlainText()) return;
+
     d->textEdit->setPlainText(text);
     //Usability : It is usually more convenient if the cursor is positioned at the end of the text so it can
     //be quickly deleted using the backspace key
 
     // This also ensures that the caret is sized correctly for the text
-    setCursorPosition( text.length() );
-    kDebug() <<"text cursor positioned at the end";
+    if ((cursorPos < 0) || cursorPos > text.length()) cursorPos = text.length();
+    setCursorPosition (cursorPos);
 
     if (d->fontLength == 0)
     {
@@ -1168,9 +1182,10 @@ void CellEditor::setCursorPosition( int pos )
     QTextCursor textCursor( d->textEdit->textCursor() );
     textCursor.setPosition( pos );
     d->textEdit->setTextCursor( textCursor );
+//     kDebug() << "pos" << pos << "textCursor" << d->textEdit->textCursor().position();
     // FIXME Stefan: The purpose of this connection is?
     //               Disabled to avoid cursor jumps to the end of the line on every key event.
-//     canvas()->view()->editWidget()->setCursorPosition( pos );
+//     selection()->view()->editWidget()->setCursorPosition( pos );
 }
 
 bool CellEditor::eventFilter( QObject* obj, QEvent* ev )
@@ -1181,25 +1196,19 @@ bool CellEditor::eventFilter( QObject* obj, QEvent* ev )
 
     if ( ev->type() == QEvent::FocusOut )
     {
-        canvas()->setLastEditorWithFocus( Canvas::CellEditor );
+        selection()->setLastEditorWithFocus(Selection::EmbeddedEditor);
         return false;
     }
 
     if ( ev->type() == QEvent::KeyPress || ev->type() == QEvent::KeyRelease )
     {
         QKeyEvent* ke = static_cast<QKeyEvent*>( ev );
-        if ( !( ke->modifiers() & Qt::ShiftModifier ) || canvas()->chooseMode() )
+        if ( !( ke->modifiers() & Qt::ShiftModifier ) || selection()->referenceSelectionMode() )
         {
             switch ( ke->key() )
             {
-                // If the user presses the return key to finish editing this cell,
-                // choose mode must be turned off first. Otherwise it will merely
-                // select a different cell.
                 case Qt::Key_Return:
                 case Qt::Key_Enter:
-                    kDebug() <<"CellEditor::eventFilter: quitting choose mode";
-                    canvas()->endChoose();
-                    // fall through
                 case Qt::Key_Up:
                 case Qt::Key_Down:
                 case Qt::Key_PageDown:
@@ -1215,7 +1224,7 @@ bool CellEditor::eventFilter( QObject* obj, QEvent* ev )
         // QKeyEvent::text() is empty, if a modifier was pressed/released.
         if ( ev->type() == QEvent::KeyPress && !ke->text().isEmpty() )
         {
-            canvas()->setChooseMode(false);
+            selection()->setReferenceSelectionMode(false);
         }
         // Forward left/right arrows to parent, so that pressing left/right
         // in this editor leaves editing mode, unless this editor has been set
@@ -1252,36 +1261,39 @@ void CellEditor::setCursorToRange(uint pos)
   d->updatingChoice = false;
 }
 
-
+void CellEditor::slotSelectionDestroyed()
+{
+kDebug() << "editor destroyed";
+  d->selection = 0;
+}
 
 /*****************************************************************************
  *
- * ComboboxLocationEditWidget
+ * LocationComboBox
  *
  ****************************************************************************/
 
-ComboboxLocationEditWidget::ComboboxLocationEditWidget( QWidget * _parent,
-                                                      View * _view )
+LocationComboBox::LocationComboBox(QWidget* _parent, Selection* selection)
     : KComboBox( _parent)
 {
-    m_locationWidget = new LocationEditWidget( _parent, _view );
+    m_locationWidget = new LocationEditWidget( _parent, selection);
     setLineEdit( m_locationWidget );
     insertItem( 0,"" );
 
-    const QList<QString> areaNames = _view->doc()->namedAreaManager()->areaNames();
+    const QList<QString> areaNames = selection->activeSheet()->map()->namedAreaManager()->areaNames();
     for (int i = 0; i < areaNames.count(); ++i)
         slotAddAreaName(areaNames[i]);
     connect( this, SIGNAL( activated ( const QString & ) ), m_locationWidget, SLOT( slotActivateItem() ) );
 }
 
 
-void ComboboxLocationEditWidget::slotAddAreaName( const QString &_name)
+void LocationComboBox::slotAddAreaName( const QString &_name)
 {
     insertItem( count(), _name );
     m_locationWidget->addCompletionItem( _name );
 }
 
-void ComboboxLocationEditWidget::slotRemoveAreaName( const QString &_name )
+void LocationComboBox::slotRemoveAreaName( const QString &_name )
 {
     for ( int i = 0; i<count(); i++ )
     {
@@ -1302,10 +1314,9 @@ void ComboboxLocationEditWidget::slotRemoveAreaName( const QString &_name )
  *
  ****************************************************************************/
 
-LocationEditWidget::LocationEditWidget( QWidget * _parent,
-                                                      View * _view )
+LocationEditWidget::LocationEditWidget(QWidget* _parent, Selection* selection)
     : KLineEdit( _parent),
-      m_pView(_view)
+      m_selection(selection)
 {
     setCompletionObject( &completionList,true );
     setCompletionMode(KGlobalSettings::CompletionAuto  );
@@ -1333,22 +1344,22 @@ void LocationEditWidget::slotActivateItem()
 bool LocationEditWidget::activateItem()
 {
     // Set the focus back on the canvas.
-    m_pView->canvasWidget()->setFocus();
+    parentWidget()->setFocus();
 
     const QString text = this->text();
-    // check wether an already existing named area was entered
-    Region region = m_pView->doc()->namedAreaManager()->namedArea(text);
+    // check whether an already existing named area was entered
+    Region region = m_selection->activeSheet()->map()->namedAreaManager()->namedArea(text);
     if (region.isValid())
     {
-        m_pView->selection()->initialize(Region(text, m_pView->doc()->map(), m_pView->activeSheet()));
+        m_selection->initialize(Region(text, m_selection->activeSheet()->map(), m_selection->activeSheet()));
         return true;
     }
 
-    // check wether a valid cell region was entered
-    region = Region(text, m_pView->doc()->map(), m_pView->activeSheet());
+    // check whether a valid cell region was entered
+    region = Region(text, m_selection->activeSheet()->map(), m_selection->activeSheet());
     if (region.isValid())
     {
-        m_pView->selection()->initialize(region);
+        m_selection->initialize(region);
         return true;
     }
 
@@ -1366,9 +1377,9 @@ bool LocationEditWidget::activateItem()
     if (validName)
     {
         NamedAreaCommand* command = new NamedAreaCommand();
-        command->setSheet(m_pView->activeSheet());
+        command->setSheet(m_selection->activeSheet());
         command->setAreaName(text);
-        command->add(Region(m_pView->selection()->lastRange(), m_pView->activeSheet()));
+        command->add(Region(m_selection->lastRange(), m_selection->activeSheet()));
         if (command->execute())
             return true;
         else
@@ -1386,7 +1397,7 @@ void LocationEditWidget::keyPressEvent( QKeyEvent * _ev )
     {
         KLineEdit::keyPressEvent( _ev );
         // Never allow that keys are passed on to the parent.
-        _ev->accept();
+        _ev->accept(); // QKeyEvent
 
         return;
     }
@@ -1399,33 +1410,157 @@ void LocationEditWidget::keyPressEvent( QKeyEvent * _ev )
     {
         if ( activateItem() )
             return;
-        _ev->accept();
+        _ev->accept(); // QKeyEvent
     }
     break;
     // Escape pressed, restore original value
     case Qt::Key_Escape:
-        if ( m_pView->selection()->isSingular() ) {
-            setText( Cell::columnName( m_pView->selection()->marker().x() )
-                     + QString::number( m_pView->selection()->marker().y() ) );
+        if ( m_selection->isSingular() ) {
+            setText( Cell::columnName( m_selection->marker().x() )
+                     + QString::number( m_selection->marker().y() ) );
         } else {
-            setText( Cell::columnName( m_pView->selection()->lastRange().left() )
-                     + QString::number( m_pView->selection()->lastRange().top() )
+            setText( Cell::columnName( m_selection->lastRange().left() )
+                     + QString::number( m_selection->lastRange().top() )
                      + ':'
-                     + Cell::columnName( m_pView->selection()->lastRange().right() )
-                     + QString::number( m_pView->selection()->lastRange().bottom() ) );
+                     + Cell::columnName( m_selection->lastRange().right() )
+                     + QString::number( m_selection->lastRange().bottom() ) );
         }
-        m_pView->canvasWidget()->setFocus();
-        _ev->accept();
+        parentWidget()->setFocus();
+        _ev->accept(); // QKeyEvent
         break;
     default:
         KLineEdit::keyPressEvent( _ev );
         // Never allow that keys are passed on to the parent.
-        _ev->accept();
+        _ev->accept(); // QKeyEvent
     }
 }
 
 
+/****************************************************************
+ *
+ * ExternalEditor
+ * The editor that appears in the tool option widget and allows
+ * to edit the cell's content.
+ *
+ ****************************************************************/
 
+class ExternalEditor::Private
+{
+public:
+    CellToolBase* cellTool;
+    FormulaEditorHighlighter* highlighter;
+    bool isArray;
+};
+
+ExternalEditor::ExternalEditor(QWidget *parent)
+    : KTextEdit(parent)
+    , d(new Private)
+{
+    d->cellTool = 0;
+    d->highlighter = 0;
+    d->isArray = false;
+    setMinimumSize(100, 30);
+    connect (this, SIGNAL (textChanged()), this, SLOT (slotTextChanged ()));
+}
+
+ExternalEditor::~ExternalEditor()
+{
+    delete d->highlighter;
+    delete d;
+}
+
+QSize ExternalEditor::sizeHint() const
+{
+    return document()->size().toSize();
+}
+
+void ExternalEditor::setCellTool(CellToolBase* cellTool)
+{
+    d->cellTool = cellTool;
+    d->highlighter = new FormulaEditorHighlighter(this, cellTool->selection());
+}
+
+void ExternalEditor::applyChanges()
+{
+    Q_ASSERT(d->cellTool);
+    d->cellTool->deleteEditor(true, d->isArray); // save changes
+    d->isArray = false;
+    d->cellTool->canvas()->canvasWidget()->setFocus();
+}
+
+void ExternalEditor::discardChanges()
+{
+    Q_ASSERT(d->cellTool);
+    clear();
+    d->cellTool->deleteEditor(false); // discard changes
+    d->cellTool->canvas()->canvasWidget()->setFocus();
+    d->cellTool->selection()->update();
+}
+
+void ExternalEditor::setText(const QString &text)
+{
+    Q_ASSERT(d->cellTool);
+    if (toPlainText() == text) {
+        return;
+    }
+    KTextEdit::setPlainText(text);
+    QTextCursor textCursor = this->textCursor();
+    textCursor.setPosition(d->cellTool->editor()->cursorPosition());
+    setTextCursor(textCursor);
+}
+
+void ExternalEditor::keyPressEvent(QKeyEvent *event)
+{
+    Q_ASSERT(d->cellTool);
+    if (!d->cellTool->selection()->activeSheet()->map()->isReadWrite()) {
+        return;
+    }
+
+    // Create the embedded editor, if necessary.
+    if (!d->cellTool->editor()) {
+        d->cellTool->createEditor(false /* keep content */, false /* no focus */);
+    }
+
+    // the Enter and Esc key are handled by the embedded editor
+    if ((event->key() == Qt::Key_Return) || (event->key() == Qt::Key_Enter) ||
+        (event->key() == Qt::Key_Escape)) {
+        d->cellTool->editor()->setFocus();
+        QApplication::sendEvent (d->cellTool->editor(), event);
+        event->accept();
+        return;
+    }
+    if (event->key() == Qt::Key_F2) {
+        // Switch the focus back to the embedded editor.
+        d->cellTool->editor()->setFocus();
+    }
+    // call inherited handler
+    KTextEdit::keyPressEvent (event);
+}
+
+void ExternalEditor::focusInEvent(QFocusEvent* event)
+{
+    Q_ASSERT(d->cellTool);
+    // when the external editor gets focus, create also the internal editor
+    // this in turn means that ranges will be instantly highlighted right
+    if (!d->cellTool->editor())
+        d->cellTool->createEditor(false /* keep content */, false /* no focus */);
+    KTextEdit::focusInEvent(event);
+}
+
+void ExternalEditor::focusOutEvent(QFocusEvent* event)
+{
+    Q_ASSERT(d->cellTool);
+    d->cellTool->selection()->setLastEditorWithFocus(Selection::ExternalEditor);
+    KTextEdit::focusOutEvent(event);
+}
+
+void ExternalEditor::slotTextChanged ()
+{
+  if (!hasFocus()) return;  // only report change if we have focus
+  emit textChanged (toPlainText());
+}
+
+#if 0 // KSPREAD_DISCARD_FORMULA_BAR
 /****************************************************************
  *
  * EditWidget
@@ -1493,7 +1628,7 @@ void EditWidget::slotDoneEdit()
 
 void EditWidget::keyPressEvent ( QKeyEvent* _ev )
 {
-    // Dont handle special keys and accelerators, except Enter ones
+    // Don't handle special keys and accelerators, except Enter ones
     if (( ( _ev->modifiers() & ( Qt::AltModifier | Qt::ControlModifier ) )
          || ( _ev->modifiers() & Qt::ShiftModifier )
          || ( _ev->key() == Qt::Key_Shift )
@@ -1530,7 +1665,6 @@ void EditWidget::keyPressEvent ( QKeyEvent* _ev )
       m_isArray = (_ev->modifiers() & Qt::AltModifier) &&
           (_ev->modifiers() & Qt::ControlModifier);
       slotDoneEdit();
-      m_pCanvas->view()->updateEditWidget();
       _ev->accept();
       break;
     case Qt::Key_F2:
@@ -1572,6 +1706,7 @@ void EditWidget::setText( const QString& t )
 
   KLineEdit::setText( t );
 }
+#endif
 
 
 
@@ -1584,7 +1719,7 @@ void EditWidget::setText( const QString& t )
 class RegionSelector::Private
 {
   public:
-    View* view;
+    Selection* selection;
     QDialog* parentDialog;
     KDialog* dialog;
     KTextEdit* textEdit;
@@ -1597,7 +1732,7 @@ class RegionSelector::Private
 
 RegionSelector* RegionSelector::Private::s_focussedSelector = 0;
 
-RegionSelector::RegionSelector( QWidget* parent )
+RegionSelector::RegionSelector(QWidget* parent)
   : QWidget( parent ),
     d( new Private )
 {
@@ -1605,12 +1740,12 @@ RegionSelector::RegionSelector( QWidget* parent )
 
   d->displayMode = Widget;
   d->parentDialog = 0;
-  d->view = 0;
+  d->selection = 0;
   d->dialog = 0;
-  d->highlighter = 0;
   d->button = new QToolButton( this );
   d->button->setCheckable( true );
   d->button->setIcon( KIcon( "selection" ) );
+  d->highlighter = 0;
   d->textEdit = new KTextEdit( this );
   d->textEdit->setLineWrapMode( QTextEdit::NoWrap );
   d->textEdit->setWordWrapMode( QTextOption::NoWrap );
@@ -1634,7 +1769,7 @@ RegionSelector::RegionSelector( QWidget* parent )
 
 RegionSelector::~RegionSelector()
 {
-  d->view->canvasWidget()->endChoose();
+  d->selection->endReferenceSelection();
   delete d;
 }
 
@@ -1644,12 +1779,11 @@ void RegionSelector::setSelectionMode( SelectionMode mode )
   // TODO adjust selection
 }
 
-void RegionSelector::setView( View* view )
+void RegionSelector::setSelection(Selection* selection)
 {
-  d->view = view;
-  d->highlighter = new FormulaEditorHighlighter( d->textEdit, view->canvasWidget() );
-  connect( d->view->choice(), SIGNAL( changed(const Region&) ),
-           this, SLOT( choiceChanged() ) );
+    d->selection = selection;
+    d->highlighter = new FormulaEditorHighlighter(d->textEdit, d->selection);
+    connect(d->selection, SIGNAL(changed(const Region&)), this, SLOT(choiceChanged()));
 }
 
 void RegionSelector::setDialog( QDialog* dialog )
@@ -1677,14 +1811,14 @@ bool RegionSelector::eventFilter( QObject* object, QEvent* event )
   else if ( event->type() == QEvent::FocusIn )
   {
     Private::s_focussedSelector = this;
-    d->view->canvasWidget()->startChoose();
+    d->selection->startReferenceSelection();
     if (d->selectionMode == SingleCell)
     {
-      d->view->choice()->setSelectionMode( Selection::SingleCell );
+      d->selection->setSelectionMode( Selection::SingleCell );
     }
     else
     {
-      d->view->choice()->setSelectionMode( Selection::MultipleCells );
+      d->selection->setSelectionMode( Selection::MultipleCells );
     }
     // TODO Stefan: initialize choice
   }
@@ -1749,9 +1883,9 @@ void RegionSelector::choiceChanged()
   if ( Private::s_focussedSelector != this )
     return;
 
-  if ( d->view->choice()->isValid() )
+  if ( d->selection->isValid() )
   {
-    QString area = d->view->choice()->name();
+    QString area = d->selection->name();
     d->textEdit->setPlainText( area );
   }
 }
