@@ -24,7 +24,6 @@
 #include "KWOdfLoader.h"
 #include "KWOdfSharedLoadingData.h"
 #include "KWDocument.h"
-#include "KWPage.h"
 #include "KWPageManager.h"
 #include "frames/KWTextFrameSet.h"
 #include "frames/KWTextFrame.h"
@@ -36,8 +35,9 @@
 #include <KoXmlReader.h>
 #include <KoXmlNS.h>
 #include <KoShapeRegistry.h>
-#include <KoShapeFactoryBase.h>
+#include <KoShapeFactory.h>
 #include <KoTextShapeData.h>
+#include <KoDataCenter.h>
 #include <KoShapeLoadingContext.h>
 #include <KoStyleManager.h>
 #include <KoOdfLoadingContext.h>
@@ -46,29 +46,44 @@
 #include <QTextCursor>
 #include <KDebug>
 
-#include <rdf/KoDocumentRdfBase.h>
-
 KWOdfLoader::KWOdfLoader(KWDocument *document)
-        : QObject(document),
-        m_document(document)
+        : QObject(),
+        m_document(document),
+        m_currentFrame(0)
 {
-    connect(this, SIGNAL(progressUpdate(int)), m_document, SIGNAL(sigProgress(int)));
+    connect(this, SIGNAL(sigProgress(int)), m_document, SIGNAL(sigProgress(int)));
 }
 
 KWOdfLoader::~KWOdfLoader()
 {
 }
 
-KWDocument *KWOdfLoader::document() const
+KWDocument* KWOdfLoader::document() const
 {
     return m_document;
 }
 
-//1.6: KWDocument::loadOasis
-bool KWOdfLoader::load(KoOdfReadStore &odfStore)
+KWPageManager* KWOdfLoader::pageManager()
 {
-    emit progressUpdate(0);
-    //kDebug(32001) << "========================> KWOdfLoader::load START";
+    return & m_document->m_pageManager;
+}
+
+//QString KWOdfLoader::currentMasterPage() const { return m_currentMasterPage; }
+QString KWOdfLoader::currentFramesetName() const
+{
+    return m_currentFrame ? m_currentFrame->frameSet()->name() : QString();
+}
+
+KWTextFrame* KWOdfLoader::currentFrame() const
+{
+    return m_currentFrame;
+}
+
+//1.6: KWDocument::loadOasis
+bool KWOdfLoader::load(KoOdfReadStore & odfStore)
+{
+    emit sigProgress(0);
+    kDebug(32001) << "========================> KWOdfLoader::load START";
 
     KoXmlElement content = odfStore.contentDoc().documentElement();
     KoXmlElement realBody(KoXml::namedItemNS(content, KoXmlNS::office, "body"));
@@ -84,7 +99,7 @@ bool KWOdfLoader::load(KoOdfReadStore &odfStore)
         KoXmlElement childElem;
         QString localName;
         forEachElement(childElem, realBody)
-            localName = childElem.localName();
+        localName = childElem.localName();
         if (localName.isEmpty())
             m_document->setErrorMessage(i18n("Invalid OASIS OpenDocument file. No tag found inside office:body."));
         else
@@ -94,35 +109,27 @@ bool KWOdfLoader::load(KoOdfReadStore &odfStore)
 
     // TODO check versions and mimetypes etc.
 
-    bool hasMainText = false;
-    KoXmlElement childElem;
-    forEachElement(childElem, body) {
-        if (childElem.namespaceURI() == KoXmlNS::text
-                && childElem.localName() != "page-sequence"
-                && childElem.localName() != "tracked-changes") {
-            hasMainText = true;
-            break;
-        }
-        if (childElem.namespaceURI() == KoXmlNS::table
-                && childElem.localName() == "table") {
-            hasMainText = true;
-            break;
-        }
-    }
-
-    KoOdfLoadingContext odfContext(odfStore.styles(), odfStore.store(), m_document->componentData());
-    KoShapeLoadingContext sc(odfContext, m_document->resourceManager());
+    KoOdfLoadingContext odfContext(odfStore.styles(), odfStore.store());
+    KoShapeLoadingContext sc(odfContext, m_document->dataCenterMap());
 
     // Load all styles before the corresponding paragraphs try to use them!
-    KWOdfSharedLoadingData *sharedData = new KWOdfSharedLoadingData(this);
-    KoStyleManager *styleManager = m_document->resourceManager()->resource(KoText::StyleManager).value<KoStyleManager*>();
+    //KoTextSharedLoadingData * sharedData = new KoTextSharedLoadingData();
+    KWOdfSharedLoadingData * sharedData = new KWOdfSharedLoadingData(this);
+    KoStyleManager *styleManager = dynamic_cast<KoStyleManager *>(m_document->dataCenterMap()["StyleManager"]);
     Q_ASSERT(styleManager);
-    sharedData->loadOdfStyles(sc, styleManager);
+    sharedData->loadOdfStyles(odfContext, styleManager);
     sc.addSharedData(KOTEXT_SHARED_LOADING_ID, sharedData);
 
-    KoOdfLoadingContext context(odfStore.styles(), odfStore.store(), m_document->componentData());
+    KoTextLoader * loader = new KoTextLoader(sc);
+    Q_UNUSED(loader);
+    KoOdfLoadingContext context(odfStore.styles(), odfStore.store());
 
-    loadMasterPageStyles(context, hasMainText);
+
+    KoColumns columns;
+    columns.columns = 1;
+    columns.columnSpacing = m_document->config().defaultColumnSpacing();
+
+    loadMasterPageStyles(context);
 
 #if 0 //1.6:
     KWOasisLoader oasisLoader(this);
@@ -130,7 +137,7 @@ bool KWOdfLoader::load(KoOdfReadStore &odfStore)
     m_processingType = (!KoXml::namedItemNS(body, KoXmlNS::text, "page-sequence").isNull()) ? DTP : WP;
     m_hasTOC = false;
     m_tabStop = MM_TO_POINT(15);
-    const KoXmlElement *defaultParagStyle = styles.defaultStyle("paragraph");
+    const KoXmlElement* defaultParagStyle = styles.defaultStyle("paragraph");
     if (defaultParagStyle) {
         KoStyleStack stack;
         stack.push(*defaultParagStyle);
@@ -147,6 +154,7 @@ bool KWOdfLoader::load(KoOdfReadStore &odfStore)
 #endif
 
     // Load all styles before the corresponding paragraphs try to use them!
+
 #if 0 //1.6:
     if (m_frameStyleColl->loadOasisStyles(context) == 0) {
         // no styles loaded -> load default styles
@@ -158,61 +166,72 @@ bool KWOdfLoader::load(KoOdfReadStore &odfStore)
     }
     static_cast<KWVariableSettings *>(m_varColl->variableSetting())->loadNoteConfiguration(styles.officeStyle());
     loadDefaultTableTemplates();
-//#else
+#else
     /*
     // We always needs at least one valid default paragraph style
     KoParagraphStyle *defaultParagraphStyle = m_document->styleManager()->defaultParagraphStyle();
-    //const KoXmlElement *defaultParagraphStyle = context.stylesReader().defaultStyle("paragraph");
-    //if(! defaultParagraphStyle) {
+    //const KoXmlElement* defaultParagraphStyle = context.stylesReader().defaultStyle("paragraph");
+    //if( ! defaultParagraphStyle ) {
     KoParagraphStyle *parastyle = new KoParagraphStyle();
     parastyle->setName("Standard");
     m_document->styleManager()->add(parastyle);
-    context.styleStack().setTypeProperties("paragraph"); // load all style attributes from "style:paragraph-properties"
+    context.styleStack().setTypeProperties( "paragraph" ); // load all style attributes from "style:paragraph-properties"
     parastyle->loadOasis(context.styleStack()); // load the KoParagraphStyle from the stylestack
     KoCharacterStyle *charstyle = parastyle->characterStyle();
-    context.styleStack().setTypeProperties("text"); // load all style attributes from "style:text-properties"
+    context.styleStack().setTypeProperties( "text" ); // load all style attributes from "style:text-properties"
     charstyle->loadOasis(context.styleStack()); // load the KoCharacterStyle from the stylestack
     //}
     */
 #endif
 
-    // load text:page-sequence
-    KoXmlElement pageSequence = KoXml::namedItemNS(body, KoXmlNS::text, "page-sequence");
-    if (! pageSequence.isNull()) {
-        KWPageManager *pageManager = m_document->pageManager();
-        KoXmlElement page;
-        forEachElement(page, pageSequence) {
-            if (page.namespaceURI() == KoXmlNS::text && page.localName() == "page") {
-                QString master = page.attributeNS(KoXmlNS::text, "master-page-name", QString());
-                pageManager->appendPage(pageManager->pageStyle(master));
-            }
-        }
-    }
+    KWord::TextFrameSetType type = KWord::MainTextFrameSet;
+    KWTextFrameSet *fs = new KWTextFrameSet(m_document, type);
+    fs->setAllowLayout(false);
+    fs->setName(i18n("Main Text Frameset"));
+    //fs->loadOasisContent( body, context );
 
-    KoTextShapeData textShapeData;
-    if (hasMainText) {
-        KWTextFrameSet *mainFs = new KWTextFrameSet(m_document, KWord::MainTextFrameSet);
-        mainFs->setAllowLayout(false);
-        mainFs->setPageStyle(m_document->pageManager()->pageStyle("Standard"));
-        m_document->addFrameSet(mainFs);
-        textShapeData.setDocument(mainFs->document(), false);
-    }
+    // Get the factory for the TextShape
+    KoShapeFactory *factory = KoShapeRegistry::instance()->value(TextShape_SHAPEID);
+    Q_ASSERT(factory);
+    // Create a TextShape
+    KoShape *shape = factory->createDefaultShapeAndInit(m_document->dataCenterMap());
+    Q_ASSERT(shape);
+    // The TextShape will be displayed within a KWTextFrame
+    KWTextFrame *frame = new KWTextFrame(shape, fs);
+    Q_UNUSED(frame);
+    m_currentFrame = frame;
+    m_document->addFrameSet(fs);
+
+    // The KoTextShapeData does contain the data for our TextShape
+    KoTextShapeData *textShapeData = dynamic_cast<KoTextShapeData*>(shape->userData());
+    Q_ASSERT(textShapeData);
+    textShapeData->setDocument(fs->document(), false /*transferOwnership*/);
     // Let the TextShape handle loading the body element.
-    textShapeData.loadOdf(body, sc, m_document->documentRdfBase());
+    textShapeData->loadOdf(body, sc);
 
-    // Grab weak references to all the Rdf stuff that was loaded
-    if (KoDocumentRdfBase *rdf = m_document->documentRdfBase()) {
-        rdf->updateInlineRdfStatements(textShapeData.document());
-    }
+    /*
+    QTextCursor cursor( fs->document() );
+    Q_ASSERT(! d->frameLoader);
+    d->frameLoader = new KWOpenDocumentFrameLoader(this);
+    loadBody(context, body, cursor);
+    */
 
     loadSettings(odfStore.settingsDoc());
 
-    //kDebug(32001) << "========================> KWOdfLoader::load END";
-    emit progressUpdate(100);
+#if 0 //1.6:
+    // This sets the columns and header/footer flags, and calls recalcFrames, so it must be done last.
+    setPageLayout(m_pageLayout, m_loadingInfo->columns, m_loadingInfo->hf, false);
+#endif
+
+    //delete d->frameLoader;
+    //d->frameLoader = 0;
+
+    kDebug(32001) << "========================> KWOdfLoader::load END";
+    emit sigProgress(100);
     return true;
 }
 
-void KWOdfLoader::loadSettings(const KoXmlDocument &settingsDoc)
+void KWOdfLoader::loadSettings(const KoXmlDocument& settingsDoc)
 {
     if (settingsDoc.isNull())
         return;
@@ -228,19 +247,19 @@ void KWOdfLoader::loadSettings(const KoXmlDocument &settingsDoc)
     if (!configurationSettings.isNull()) {
         const QString ignorelist = configurationSettings.parseConfigItemString("SpellCheckerIgnoreList");
         kDebug(32001) << "Ignorelist:" << ignorelist;
-        //1.6: m_document->setSpellCheckIgnoreList(QStringList::split(',', ignorelist));
+        //1.6: m_document->setSpellCheckIgnoreList( QStringList::split( ',', ignorelist ) );
     }
-    //1.6: m_document->variableCollection()->variableSetting()->loadOasis(settings);
+    //1.6: m_document->variableCollection()->variableSetting()->loadOasis( settings );
 }
 
-void KWOdfLoader::loadMasterPageStyles(KoOdfLoadingContext &context, bool hasMainText)
+void KWOdfLoader::loadMasterPageStyles(KoOdfLoadingContext& context)
 {
     kDebug(32001) << " !!!!!!!!!!!!!! loadMasterPageStyles called !!!!!!!!!!!!!!";
     kDebug(32001) << "Number of items :" << context.stylesReader().masterPages().size();
 
     //TODO probably we should introduce more logic to handle the "standard" even
     //in faulty documents. See also bugreport #129585 as example.
-    const KoOdfStylesReader &styles = context.stylesReader();
+    const KoOdfStylesReader& styles = context.stylesReader();
     QHashIterator<QString, KoXmlElement *> it(styles.masterPages());
     while (it.hasNext()) {
         it.next();
@@ -252,58 +271,77 @@ void KWOdfLoader::loadMasterPageStyles(KoOdfLoadingContext &context, bool hasMai
         const KoXmlElement *masterNode = it.value();
         const KoXmlElement *masterPageStyle = masterNode ? styles.findStyle(masterNode->attributeNS(KoXmlNS::style, "page-layout-name", QString())) : 0;
         if (masterPageStyle) {
-            masterPage.loadOdf(context, *masterPageStyle, m_document->resourceManager());
-            loadHeaderFooter(context, masterPage, *masterNode, LoadHeader);
-            loadHeaderFooter(context, masterPage, *masterNode, LoadFooter);
+            KoPageLayout pageLayout = KoPageLayout::standardLayout();
+            pageLayout.loadOasis(*masterPageStyle);
+            masterPage.setPageLayout(pageLayout);
+            loadHeaderFooter(context, masterPage, *masterNode, *masterPageStyle, true); // Load headers
+            loadHeaderFooter(context, masterPage, *masterNode, *masterPageStyle, false); // Load footers
         }
-        masterPage.setHasMainTextFrame(hasMainText);
         if (!alreadyExists)
             m_document->pageManager()->addPageStyle(masterPage);
     }
 }
 
-// helper function to create a KWTextFrameSet for a header/footer.
-void KWOdfLoader::loadHeaderFooterFrame(KoOdfLoadingContext &context, const KWPageStyle &pageStyle, const KoXmlElement &elem, KWord::TextFrameSetType fsType)
+// helper function to create a KWTextFrameSet+KWTextFrame for a header/footer.
+void KWOdfLoader::loadHeaderFooterFrame(KoOdfLoadingContext& context, const KWPageStyle &pageStyle, const KoXmlElement& elem, KWord::HeaderFooterType hfType, KWord::TextFrameSetType fsType)
 {
     KWTextFrameSet *fs = new KWTextFrameSet(m_document, fsType);
-    fs->setPageStyle(pageStyle);
     fs->setAllowLayout(false);
-    m_document->addFrameSet(fs);
+    switch (fsType) {
+    case KWord::OddPagesHeaderTextFrameSet: fs->setName(i18n("Odd Pages Header %1", pageStyle.name())); break;
+    case KWord::EvenPagesHeaderTextFrameSet: fs->setName(i18n("Even Pages Header %1", pageStyle.name())); break;
+    case KWord::OddPagesFooterTextFrameSet: fs->setName(i18n("Odd Pages Footer %1", pageStyle.name())); break;
+    case KWord::EvenPagesFooterTextFrameSet: fs->setName(i18n("Even Pages Footer %1", pageStyle.name())); break;
+    default: break;
+    }
 
     kDebug(32001) << "KWOdfLoader::loadHeaderFooterFrame localName=" << elem.localName() << " type=" << fs->name();
 
+    // Add the frameset and the shape for the header/footer to the document.
+    KoShapeFactory *sf = KoShapeRegistry::instance()->value(TextShape_SHAPEID);
+    Q_ASSERT(sf);
+    KoShape *s = sf->createDefaultShapeAndInit(m_document->dataCenterMap());
+    Q_ASSERT(s);
+    KWTextFrame *f = new KWTextFrame(s, fs);
+    m_document->addFrameSet(fs);
+
+    KWTextFrame *prevFrame = m_currentFrame;
+    m_currentFrame = f;
+
     // use auto-styles from styles.xml, not those from content.xml
     context.setUseStylesAutoStyles(true);
 
-    KoShapeLoadingContext ctxt(context, m_document->resourceManager());
+    KoShapeLoadingContext ctxt(context, m_document->dataCenterMap());
     KoTextLoader loader(ctxt);
     QTextCursor cursor(fs->document());
     loader.loadBody(elem, cursor);
 
     // restore use of auto-styles from content.xml, not those from styles.xml
     context.setUseStylesAutoStyles(false);
+
+    m_currentFrame = prevFrame; // restore the previous current frame
 }
 
 //1.6: KWOasisLoader::loadOasisHeaderFooter
-void KWOdfLoader::loadHeaderFooter(KoOdfLoadingContext &context, KWPageStyle &pageStyle, const KoXmlElement &masterPage, HFLoadType headerFooter)
+void KWOdfLoader::loadHeaderFooter(KoOdfLoadingContext& context, KWPageStyle &pageStyle, const KoXmlElement& masterPage, const KoXmlElement& masterPageStyle, bool isHeader)
 {
     // The actual content of the header/footer.
-    KoXmlElement elem = KoXml::namedItemNS(masterPage, KoXmlNS::style, headerFooter == LoadHeader ? "header" : "footer");
+    KoXmlElement elem = KoXml::namedItemNS(masterPage, KoXmlNS::style, isHeader ? "header" : "footer");
     // The two additional elements <style:header-left> and <style:footer-left> specifies if defined that even and odd pages
     // should be displayed different. If they are missing, the conent of odd and even (aka left and right) pages are the same.
-    KoXmlElement leftElem = KoXml::namedItemNS(masterPage, KoXmlNS::style, headerFooter == LoadHeader ? "header-left" : "footer-left");
+    KoXmlElement leftElem = KoXml::namedItemNS(masterPage, KoXmlNS::style, isHeader ? "header-left" : "footer-left");
     // Used in KWPageStyle to determine if, and what kind of header/footer to use.
     KWord::HeaderFooterType hfType = elem.isNull() ? KWord::HFTypeNone : leftElem.isNull() ? KWord::HFTypeUniform : KWord::HFTypeEvenOdd;
 
     if (! leftElem.isNull()) {   // header-left and footer-left
-        loadHeaderFooterFrame(context, pageStyle, leftElem, headerFooter == LoadHeader ? KWord::EvenPagesHeaderTextFrameSet : KWord::EvenPagesFooterTextFrameSet);
+        loadHeaderFooterFrame(context, pageStyle, leftElem, hfType, isHeader ? KWord::EvenPagesHeaderTextFrameSet : KWord::EvenPagesFooterTextFrameSet);
     }
 
     if (! elem.isNull()) {   // header and footer
-        loadHeaderFooterFrame(context, pageStyle, elem, headerFooter == LoadHeader ? KWord::OddPagesHeaderTextFrameSet : KWord::OddPagesFooterTextFrameSet);
+        loadHeaderFooterFrame(context, pageStyle, elem, hfType, isHeader ? KWord::OddPagesHeaderTextFrameSet : KWord::OddPagesFooterTextFrameSet);
     }
 
-    if (headerFooter == LoadHeader) {
+    if (isHeader) {
         pageStyle.setHeaderPolicy(hfType);
     } else {
         pageStyle.setFooterPolicy(hfType);
