@@ -1,5 +1,5 @@
 /* This file is part of the KOffice project
- * Copyright (C) 2005, 2007 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2005, 2007-2009 Thomas Zander <zander@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -18,14 +18,17 @@
  */
 
 #include "KWStartupWidget.h"
-#include <KWDocument.h>
-#include "KWPageLayout.h"
+
 #include "KWDocumentColumns.h"
-#include "KWPagePreview.h"
+#include "../KWPage.h"
+#include "../KWDocument.h"
+
+#include <KoPageLayoutWidget.h>
+#include <KoPagePreviewWidget.h>
 
 KWStartupWidget::KWStartupWidget(QWidget *parent, KWDocument *doc, const KoColumns &columns)
-    : QWidget(parent),
-    m_unit(KoUnit::Millimeter)
+        : QWidget(parent),
+        m_unit(KoUnit::Millimeter)
 {
     widget.setupUi(this);
     // TODO get unit from config and set it on m_unit
@@ -41,64 +44,67 @@ KWStartupWidget::KWStartupWidget(QWidget *parent, KWDocument *doc, const KoColum
     setFocusProxy(widget.createButton);
 
     QVBoxLayout *lay = new QVBoxLayout(widget.sizeTab);
-    m_sizeWidget = new KWPageLayout(widget.sizeTab, m_layout);
+    m_sizeWidget = new KoPageLayoutWidget(widget.sizeTab, m_layout);
     m_sizeWidget->setUnit(m_unit);
+    // I'm very sad that I have to add the next line; but it takes too much time to get the pagespread working again
+    m_sizeWidget->showPageSpread(false);
     lay->addWidget(m_sizeWidget);
     lay->setMargin(0);
 
     lay = new QVBoxLayout(widget.columnsTab);
     m_columnsWidget = new KWDocumentColumns(widget.columnsTab, m_columns);
     m_columnsWidget->setUnit(m_unit);
+    m_columnsWidget->setShowPreview(false);
     lay->addWidget(m_columnsWidget);
     lay->setMargin(0);
 
     lay = new QVBoxLayout(widget.previewPane);
     widget.previewPane->setLayout(lay);
     lay->setMargin(0);
-    KWPagePreview *prev = new KWPagePreview(widget.previewPane);
+    KoPagePreviewWidget *prev = new KoPagePreviewWidget(widget.previewPane);
     lay->addWidget(prev);
     prev->setColumns(columns);
     prev->setPageLayout(m_layout);
 
-    connect (m_sizeWidget, SIGNAL( layoutChanged(const KoPageLayout&)), this, SLOT(sizeUpdated(const KoPageLayout&)));
-    connect (widget.createButton, SIGNAL( clicked() ), this, SLOT (buttonClicked()) );
-    connect (widget.mainText, SIGNAL(toggled(bool)), m_sizeWidget, SLOT(setTextAreaAvailable(bool)));
-    connect (widget.mainText, SIGNAL(toggled(bool)), m_columnsWidget, SLOT(setTextAreaAvailable(bool)));
-    connect (m_sizeWidget, SIGNAL(unitChanged(const KoUnit&)), m_columnsWidget, SLOT(setUnit(const KoUnit&)));
-    connect (m_columnsWidget, SIGNAL(columnsChanged(const KoColumns&)), prev, SLOT(setColumns(const KoColumns&)));
-    connect (m_columnsWidget, SIGNAL(columnsChanged(const KoColumns&)), this, SLOT(columnsUpdated(const KoColumns&)));
-    connect (m_sizeWidget, SIGNAL(layoutChanged(const KoPageLayout&)), prev, SLOT(setPageLayout(const KoPageLayout&)));
+    connect(m_sizeWidget, SIGNAL(layoutChanged(const KoPageLayout&)), this, SLOT(sizeUpdated(const KoPageLayout&)));
+    connect(widget.createButton, SIGNAL(clicked()), this, SLOT(buttonClicked()));
+    connect(widget.mainText, SIGNAL(toggled(bool)), m_sizeWidget, SLOT(setTextAreaAvailable(bool)));
+    connect(widget.mainText, SIGNAL(toggled(bool)), m_columnsWidget, SLOT(setTextAreaAvailable(bool)));
+    connect(m_sizeWidget, SIGNAL(unitChanged(const KoUnit&)), m_columnsWidget, SLOT(setUnit(const KoUnit&)));
+    connect(m_columnsWidget, SIGNAL(columnsChanged(const KoColumns&)), prev, SLOT(setColumns(const KoColumns&)));
+    connect(m_columnsWidget, SIGNAL(columnsChanged(const KoColumns&)), this, SLOT(columnsUpdated(const KoColumns&)));
+    connect(m_sizeWidget, SIGNAL(layoutChanged(const KoPageLayout&)), prev, SLOT(setPageLayout(const KoPageLayout&)));
 }
 
-void KWStartupWidget::unitChanged(const KoUnit &unit) {
+void KWStartupWidget::unitChanged(const KoUnit &unit)
+{
     m_unit = unit;
 }
 
-void KWStartupWidget::sizeUpdated(const KoPageLayout &layout) {
+void KWStartupWidget::sizeUpdated(const KoPageLayout &layout)
+{
     m_layout = layout;
 }
 
-void KWStartupWidget::columnsUpdated(const KoColumns &columns) {
+void KWStartupWidget::columnsUpdated(const KoColumns &columns)
+{
     m_columns = columns;
 }
 
-void KWStartupWidget::buttonClicked() {
-    m_doc->clear();
+void KWStartupWidget::buttonClicked()
+{
+    m_doc->initEmpty();
 
-    if(m_layout.left < 0) {
+    if (m_layout.left < 0) {
         m_layout.width /= 2.0;
-        m_doc->m_pageManager.setPreferPageSpread(true);
+        m_doc->pageManager()->setPreferPageSpread(true);
     }
-    m_doc->setDefaultPageLayout(m_layout);
-    KWPageSettings settings;
-    settings.setColumns(m_columns);
-    settings.setMainTextFrame(widget.mainText->isChecked());
-    m_doc->setPageSettings(settings);
+    KWPageStyle style = m_doc->pageManager()->defaultPageStyle();
+    Q_ASSERT(style.isValid());
+    style.setColumns(m_columns);
+    style.setMainTextFrame(widget.mainText->isChecked());
+    style.setPageLayout(m_layout);
     m_doc->setUnit(m_unit);
-    m_doc->setStartPage(m_sizeWidget->startPageNumber());
 
-    m_doc->appendPage();
     emit documentSelected();
 }
-
-#include <KWStartupWidget.moc>
