@@ -1,5 +1,5 @@
 /* This file is part of the KOffice project
- * Copyright (C) 2005,2008, 2010 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2005,2008 Thomas Zander <zander@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -115,7 +115,7 @@ void TestPageCommands::testInsertPageCommand() // move of frames
 
     KWFrameSet *fs = new KWFrameSet();
     MockShape *shape = new MockShape();
-    KWFrame *frame = new KWFrame(shape, fs);
+    KWFrame * frame = new KWFrame(shape, fs);
     Q_UNUSED(frame);
     document.addFrameSet(fs);
     QPointF startPos = shape->position();
@@ -210,15 +210,15 @@ void TestPageCommands::testInsertPageCommand3() // restore all properties
 
     KWPage page = command1.page();
     KWPageStyle style = page.pageStyle();
-    style.setHasMainTextFrame(false);
+    style.setMainTextFrame(false);
     style.setFootnoteDistance(10);
     KoPageLayout layout;
     layout.width = 400;
     layout.height = 300;
-    layout.leftMargin = 4;
-    layout.rightMargin = 6;
-    layout.topMargin = 7;
-    layout.bottomMargin = 5;
+    layout.left = 4;
+    layout.right = 6;
+    layout.top = 7;
+    layout.bottom = 5;
     style.setPageLayout(layout);
     page.setPageStyle(style);
 
@@ -356,15 +356,15 @@ void TestPageCommands::testRemovePageCommand3() // test restore all properties
 
     KWPage page = insertCommand.page();
     KWPageStyle style = page.pageStyle();
-    style.setHasMainTextFrame(false);
+    style.setMainTextFrame(false);
     style.setFootnoteDistance(10);
     KoPageLayout layout;
     layout.width = 400;
     layout.height = 300;
-    layout.leftMargin = 4;
-    layout.rightMargin = 6;
-    layout.topMargin = 7;
-    layout.bottomMargin = 5;
+    layout.left = 4;
+    layout.right = 6;
+    layout.top = 7;
+    layout.bottom = 5;
     style.setPageLayout(layout);
     page.setPageStyle(style);
 
@@ -437,12 +437,12 @@ void TestPageCommands::testPagePropertiesCommand() // basic properties change
     KWPageManager *manager = document.pageManager();
 
     KWPageStyle style("pagestyle1");
-    KoPageLayout oldLayout;
+    KoPageLayout oldLayout = KoPageLayout::standardLayout();
     oldLayout.format = KoPageFormat::IsoA4Size;
     oldLayout.width = 101;
     oldLayout.height = 102;
-    oldLayout.leftMargin = -1;
-    oldLayout.rightMargin = -1;
+    oldLayout.left = -1;
+    oldLayout.right = -1;
     oldLayout.pageEdge = 7;
     oldLayout.bindingSide = 13;
     style.setPageLayout(oldLayout);
@@ -464,11 +464,11 @@ void TestPageCommands::testPagePropertiesCommand() // basic properties change
     QCOMPARE(page1.pageSide(), KWPage::Right);
 
     // new ;)
-    KoPageLayout newLayout;
+    KoPageLayout newLayout = KoPageLayout::standardLayout();
     newLayout.width = 401;
     newLayout.height = 405;
-    newLayout.leftMargin = 11;
-    newLayout.rightMargin = 18;
+    newLayout.left = 11;
+    newLayout.right = 18;
     newLayout.pageEdge = -1;
     newLayout.bindingSide = -1;
     KoColumns newColumns;
@@ -508,69 +508,5 @@ void TestPageCommands::testPagePropertiesCommand() // basic properties change
     QCOMPARE(page2.pageSide(), KWPage::Left);
 }
 
-void TestPageCommands::testMakePageSpread()
-{
-    KWDocument document;
-    KWPageManager *manager = document.pageManager();
-
-    KWPageStyle style("pagestyle1");
-    KoPageLayout layout = style.pageLayout();
-
-    manager->addPageStyle(style);
-    KWPage page1 = manager->appendPage(style);
-    QCOMPARE(page1.width(), layout.width);
-    QCOMPARE(page1.pageNumber(), 1);
-    QCOMPARE(page1.pageSide(), KWPage::Right);
-    QCOMPARE(manager->pageCount(), 1);
-    // make it a pagespread
-    layout.leftMargin = -1;
-    layout.rightMargin = -1;
-    layout.pageEdge = 7;
-    layout.bindingSide = 13;
-    KWPagePropertiesCommand cmd1(&document, page1, layout,
-        style.direction(), style.columns());
-    cmd1.redo();
-    QCOMPARE(page1.pageNumber(), 1);
-    QCOMPARE(page1.pageSide(), KWPage::Right);
-    QCOMPARE(page1.width(), style.pageLayout().width);
-    QCOMPARE(manager->pageCount(), 1);
-    KoPageLayout newLayout = style.pageLayout();
-    QCOMPARE(newLayout.width, layout.width);
-    QCOMPARE(newLayout.leftMargin, layout.leftMargin);
-    QCOMPARE(newLayout.rightMargin, layout.rightMargin);
-    QCOMPARE(newLayout.pageEdge, layout.pageEdge);
-    QCOMPARE(newLayout.bindingSide, layout.bindingSide);
-
-    cmd1.undo();
-    QCOMPARE(page1.width(), style.pageLayout().width);
-    QCOMPARE(page1.pageNumber(), 1);
-    QCOMPARE(page1.pageSide(), KWPage::Right);
-    QCOMPARE(manager->pageCount(), 1);
-
-    // create another page. So we have 2 single sided pages. (Right/Left)
-    KWPage page2 = manager->appendPage(style);
-    QCOMPARE(page2.width(), style.pageLayout().width);
-    QCOMPARE(page2.pageNumber(), 2);
-    QCOMPARE(page2.pageSide(), KWPage::Left);
-    QCOMPARE(manager->pageCount(), 2);
-
-    // avoid reusing cmd1 as that assumes the constructor doens't do anything. Which is
-    // not a restriction we put on the command. (i.e. that doesn't *have* to work)
-    KWPagePropertiesCommand cmd2(&document, page1, layout,
-        style.direction(), style.columns());
-    cmd2.redo();
-
-    QCOMPARE(page1.width(), style.pageLayout().width);
-    QCOMPARE(page1.pageNumber(), 1);
-    QCOMPARE(page1.pageSide(), KWPage::Right);
-    QCOMPARE(page2.pageNumber(), 2);
-    QCOMPARE(page2.pageSide(), KWPage::PageSpread);
-    QCOMPARE(page2.width(), style.pageLayout().width * 2);
-    QCOMPARE(manager->pageCount(), 3);
-
-    cmd2.undo();
-    // test for page side etc.
-}
-
 QTEST_KDEMAIN(TestPageCommands, GUI)
-#include <TestPageCommands.moc>
+#include "TestPageCommands.moc"
