@@ -42,13 +42,16 @@ class KoShapeController;
 class KoShapeControllerBase;
 class KoCanvasController;
 class KoShape;
+class KoSnapGuide;
+class KoGuidesData;
 
 /**
  * KoCanvasBase is the interface actual application canvas classes
  * should implement. Flake tools know about the canvas, so they can
  * do things like scroll, redraw, set a cursor etc.
  */
-class FLAKE_EXPORT KoCanvasBase {
+class FLAKE_EXPORT KoCanvasBase
+{
 
 public:
 
@@ -57,7 +60,7 @@ public:
      * @param shapeControllerBase the implementation of the shapeController that the
      *   application provides to allow shapes to be added in multiple views.
      */
-    explicit KoCanvasBase( KoShapeControllerBase * shapeControllerBase );
+    explicit KoCanvasBase(KoShapeControllerBase * shapeControllerBase);
     virtual ~KoCanvasBase();
 
 public:
@@ -65,10 +68,10 @@ public:
     /**
      * retrieve the grid size setting.
      * The grid spacing will be provided in pt.
-     * @param horizontal a pointer to a double that will be filled with the horizontal grid-spacing
-     * @param vertical a pointer to a double that will be filled with the vertical grid-spacing
+     * @param horizontal a pointer to a qreal that will be filled with the horizontal grid-spacing
+     * @param vertical a pointer to a qreal that will be filled with the vertical grid-spacing
      */
-    virtual void gridSize(double *horizontal, double *vertical) const = 0;
+    virtual void gridSize(qreal *horizontal, qreal *vertical) const = 0;
 
     /**
      * return if snap to grid is enabled.
@@ -142,7 +145,9 @@ public:
      * By default the origin of the canvas widget and the position of the
      * document origin are coincident, thus an empty point is returned.
      */
-    virtual QPoint documentOrigin() const { return QPoint(0,0); }
+    virtual QPoint documentOrigin() const {
+        return QPoint(0, 0);
+    }
 
     /**
      * This method should somehow call QWidget::updateMicroFocus() on the canvas widget.
@@ -176,6 +181,22 @@ public:
      */
     virtual void ensureVisible(const QRectF &rect);
 
+    /**
+     * Returns the snap guide of the canvas
+     */
+    KoSnapGuide * snapGuide() const;
+
+    /**
+     * Returns the guides data.
+     *
+     * Applications that want to have guides should reimplement this
+     * function and return the KOGuideData object.
+     * The default implementation returns 0.
+     *
+     * @return pointer to the guide data or zero if there is none
+     */
+    virtual KoGuidesData * guidesData();
+
 protected:
     friend class KoCanvasController;
     /// called by KoCanvasController to set the controller that handles this canvas.
