@@ -43,9 +43,4 @@ QString KoDevice::id() const
     return d->id;
 }
 
-QString KoDevice::name() const
-{
-    return objectName();
-}
-
-#include <KoDevice.moc>
+#include "KoDevice.moc"
