@@ -1,6 +1,6 @@
 /* This file is part of the KDE project
-   Copyright (C) 2006-2007 Thorsten Zachmann <zachmann@kde.org>
-   Copyright (C) 2006 Casper Boemann Rasmussen <cbr@boemann.dk>
+   Copyright (C) 2006-2008 Thorsten Zachmann <zachmann@kde.org>
+   Copyright (C) 2006, 2008 Casper Boemann <cbr@boemann.dk>
    Copyright (C) 2006 Thomas Zander <zander@kde.org>
    Copyright (C) 2007 Jan Hambrecht <jaham@gmx.net>
 
@@ -29,9 +29,12 @@
 #include <QMatrix>
 #include <QVector>
 #include <QSet>
+#include <QMap>
 #include <QBrush>
+#include <QMetaType>
 
 #include <KoXmlReaderForward.h>
+#include <KoSnapData.h>
 
 #include "flake_export.h"
 
@@ -40,8 +43,8 @@ class QRectF;
 class QPainterPath;
 
 class KoShapeContainer;
-class KoShapeConnection;
 class KoShapeBorderModel;
+class KoShapeBackground;
 class KoShapeManager;
 class KoShapeUserData;
 class KoViewConverter;
@@ -50,6 +53,10 @@ class KoShapeSavingContext;
 class KoCanvasBase;
 class KoShapeLoadingContext;
 class KoGenStyle;
+class KoShapeControllerBase;
+class KoDataCenter;
+class KoShapeShadow;
+class KoEventAction;
 
 /**
  *
@@ -79,6 +86,20 @@ class KoGenStyle;
  *   <li>through an associated tool,
  *   <li>through a factory
  * </ul>
+ *
+ * <h1>Shape interaction notifications</h1>
+ * We had several notification methods that allow your shape to be notified of changes in other
+ * shapes positions or rotation etc.
+ * <ol><li>The most general is KoShape::shapeChanged().<br>
+ * a virtual method that you can use to check various changed to your shape made by tools or otherwise.</li>
+ * <li>for shape hierarchies the parent may receive a notification when a child was modified.
+ *  This is done though KoShapeContainerModel::childChanged()</li>
+ * <li>any shape that is at a similar position as another shape there is collision detection.
+ * You can register your shape to be sensitive to any changes like moving or whatever to
+ * <b>other</b> shapes that intersect yours.
+ * Such changes will then be notified to your shape using the method from (1) You should call
+ * KoShape::setCollisionDetection(bool) to enable this.
+ * </ol>
  */
 class FLAKE_EXPORT KoShape
 {
@@ -92,7 +113,11 @@ public:
         SizeChanged,    ///< used after a setSize()
         GenericMatrixChange,    ///< used after the matrix was changed without knowing which property explicitly changed
         ParentChanged,   ///< used after a setParent()
-        CollisionDetected ///< used when another shape moved in our boundingrect
+        CollisionDetected, ///< used when another shape moved in our boundingrect
+        Deleted, ///< the shape was deleted
+        BorderChanged, ///< the shapes border has changed
+        BackgroundChanged, ///< the shapes background has changed
+        ShadowChanged ///< the shapes shadow has changed
     };
 
     /**
@@ -141,7 +166,7 @@ public:
      *
      * @return false if loading failed
      */
-    virtual bool loadOdf( const KoXmlElement & element, KoShapeLoadingContext &context ) = 0;
+    virtual bool loadOdf(const KoXmlElement & element, KoShapeLoadingContext &context) = 0;
 
     /**
      * @brief store the shape data as ODF XML.
@@ -149,14 +174,31 @@ public:
      * OpenDocument 9.2 Drawing Shapes.
      * @see saveOdfAttributes
      */
-    virtual void saveOdf( KoShapeSavingContext & context ) const = 0;
+    virtual void saveOdf(KoShapeSavingContext & context) const = 0;
 
     /**
-     * When saving this shape to ODF, you may nest it in a 'draw:frame' and shape properties should be
-     * saved as attributes on that element instead of on the normal shape.  After starting to write such
-     * an element you can call this method to write all relevant properties.
+     * This method can be used while saving the shape as ODF to add the data
+     * stored on this shape to the current element.
+     *
+     * @param context the context for the current save.
+     * @param attributes a number of OdfAttribute items to state which attributes to save.
+     * @see saveOdf
      */
-    void saveOdfFrameAttributes( KoShapeSavingContext & context ) const;
+    void saveOdfAttributes(KoShapeSavingContext &context, int attributes) const;
+
+    /**
+     * This method can be used while saving the shape as Odf to add common child elements
+     *
+     * The office:event-listeners and draw:glue-point are saved.
+     */
+    void saveOdfCommonChildElements(KoShapeSavingContext &context) const;
+
+    /**
+     * After the shape has been created this method is called so it can get access to any DataCenter it
+     * might want.
+     * The default implementation does nothing.
+     */
+    virtual void init(const QMap<QString, KoDataCenter *> & dataCenterMap);
 
     /**
      * @brief Scale the shape using the zero-point which is the top-left corner.
@@ -165,17 +207,7 @@ public:
      * @param sx scale in x direction
      * @param sy scale in y direction
      */
-    void setScale( double sx, double sy );
-
-    /**
-     * Return the current scaling adjustment over the X axis.
-     */
-    KDE_DEPRECATED double scaleX() const;
-
-    /**
-     * Return the current scaling adjustment over the Y axis.
-     */
-    KDE_DEPRECATED double scaleY() const;
+    void setScale(qreal sx, qreal sy);
 
     /**
      * @brief Rotate the shape (relative)
@@ -184,13 +216,13 @@ public:
      *
      * @param angle change the angle of rotation increasing it with 'angle' degrees
      */
-    void rotate( double angle );
+    void rotate(qreal angle);
 
     /**
      * Return the current rotation in degrees.
      * It returns NaN if the shape has a shearing or scaling transformation applied.
      */
-    double rotation() const;
+    qreal rotation() const;
 
     /**
      * @brief Shear the shape
@@ -200,19 +232,7 @@ public:
      * @param sx shear in x direction
      * @param sy shear in y direction
      */
-    void setShear( double sx, double sy );
-
-    /**
-     * Return the current horizontal shearing angle for this shape.
-     * @return the current horizontal shearing angle for this shape.
-     */
-    KDE_DEPRECATED double shearX() const;
-
-    /**
-     * Return the current vertical shearing angle for this shape.
-     * @return the current vertical shearing angle for this shape.
-     */
-    KDE_DEPRECATED double shearY() const;
+    void setShear(qreal sx, qreal sy);
 
     /**
      * @brief Resize the shape
@@ -223,7 +243,7 @@ public:
      * Easiest example of this difference is that using this method will not distort the
      * size of pattern-fills and borders.
      */
-    virtual void setSize( const QSizeF &size );
+    virtual void setSize(const QSizeF &size);
 
     /**
      * @brief Get the size of the shape in pt.
@@ -239,7 +259,7 @@ public:
      *
      * @param position the new position of the shape
      */
-    virtual void setPosition( const QPointF &position );
+    virtual void setPosition(const QPointF &position);
 
     /**
      * @brief Get the position of the shape in pt
@@ -253,7 +273,7 @@ public:
      * @param position the position where the user clicked.
      * @return true when it hits.
      */
-    virtual bool hitTest( const QPointF &position ) const;
+    virtual bool hitTest(const QPointF &position) const;
 
     /**
      * @brief Get the bounding box of the shape
@@ -274,24 +294,38 @@ public:
      *   The point does not have to be inside the boundings rectangle.  The point is in pt,
      *   just like the rest of the KoShape class uses.
      */
-    void addConnectionPoint( const QPointF &point );
+    void addConnectionPoint(const QPointF &point);
 
     /**
      * Return a list of the connection points that have been added to this shape.
      * @return a list of the connectors that have been added to this shape.
      */
-    KDE_DEPRECATED QList<QPointF> connectors() const;
     QList<QPointF> connectionPoints() const;
 
     /**
-     * Set the background of the shape.
-     * A QBrush can have a plain color, be fully transparent or have a complex fill.
-     * setting such a brush will allow the shape to fill itself using that brush and
-     * will be able to tell if its transparent or not.
-     * @param brush the brush for the background.
+     * Add a event action
      */
-    void setBackground ( const QBrush & brush );
+    void addEventAction(KoEventAction * action);
 
+    /**
+     * Remove a event action
+     */
+    void removeEventAction(KoEventAction * action);
+
+    /**
+     * Get all event actions
+     */
+    QList<KoEventAction *> eventActions() const;
+
+    /**
+     * Set the background of the shape.
+     * A shape background can be a plain color, a gradient, a pattern, be fully transparent
+     * or have a complex fill.
+     * Setting such a background will allow the shape to be filled and will be able to tell
+     * if it is transparent or not.
+     * @param background the new shape background.
+     */
+    void setBackground(KoShapeBackground * background);
     /**
      * return the brush used to paint te background of this shape with.
      * A QBrush can have a plain color, be fully transparent or have a complex fill.
@@ -299,7 +333,7 @@ public:
      * will be able to tell if its transparent or not.
      * @return the background-brush
      */
-    QBrush background() const;
+    KoShapeBackground * background() const;
 
     /**
      * Returns true if there is some transparency, false if the shape is fully opaque.
@@ -333,18 +367,38 @@ public:
 
     /**
      * Changes the Shape to be visible or invisible.
-     * Being visible means being painted and printed, as well as being used for
+     * Being visible means being painted, as well as being used for
      *   things like guidelines or searches.
      * @param on when true; set the shape to be visible.
      */
     void setVisible(bool on);
     /**
      * Returns current visibility state of this shape.
-     * Being visible means being painted and printed, as well as being used for
+     * Being visible means being painted, as well as being used for
      *   things like guidelines or searches.
+     * @param recursive when true, checks visibility recursively
      * @return current visibility state of this shape.
      */
-    bool isVisible() const;
+    bool isVisible(bool recursive = false) const;
+
+    /**
+     * Changes the shape to be printable or not. The default is true.
+     *
+     * If a Shape's print flag is true, the shape will be printed. If
+     * false, the shape will not be printed. If a shape is not visible (@see isVisible),
+     * it isPrinted will return false, too.
+     */
+    void setPrintable(bool on);
+
+    /**
+     * Returns the current printable state of this shape.
+     *
+     * A shape can be visible but not printable, not printable and not visible
+     * or visible and printable, but not invisible and still printable.
+     *
+     * @return current printable state of this shape.
+     */
+    bool isPrintable() const;
 
     /**
      * Makes it possible for the user to select this shape.
@@ -437,6 +491,12 @@ public:
      */
     KoInsets borderInsets() const;
 
+    /// Sets the new shadow, removing the old one
+    void setShadow(KoShapeShadow * shadow);
+
+    /// Returns the currently set shadow or 0 if there is now shadow set
+    KoShapeShadow * shadow() const;
+
     /**
      * Setting the shape to keep its aspect-ratio has the effect that user-scaling will
      * keep the width/hight ratio intact so as not to distort shapes that rely on that
@@ -459,7 +519,7 @@ public:
      * @param anchor The place on the (unaltered) shape that you want the position of.
      * @return the point that is the absolute, centered position of this shape.
      */
-    QPointF absolutePosition(KoFlake::Position anchor = KoFlake::CenteredPositon) const;
+    QPointF absolutePosition(KoFlake::Position anchor = KoFlake::CenteredPosition) const;
 
     /**
      * Move this shape to an absolute position where the end location will be the same
@@ -475,7 +535,7 @@ public:
      * @param newPosition the new absolute center of the shape.
      * @param anchor The place on the (unaltered) shape that you set the position of.
      */
-    void setAbsolutePosition(QPointF newPosition, KoFlake::Position anchor = KoFlake::CenteredPositon);
+    void setAbsolutePosition(QPointF newPosition, KoFlake::Position anchor = KoFlake::CenteredPosition);
 
     /**
      * Set a data object on the shape to be used by an application.
@@ -535,13 +595,13 @@ public:
      *
      * @param matrix the transformation matrix to apply
      */
-    void applyAbsoluteTransformation(const QMatrix &matrix );
+    void applyAbsoluteTransformation(const QMatrix &matrix);
 
     /**
      * Sets a new transformation matrix describing the local transformations on this shape.
      * @param matrix the new transformation matrix
      */
-    void setTransformation( const QMatrix &matrix );
+    void setTransformation(const QMatrix &matrix);
 
     /// Returns the shapes local transformation matrix
     QMatrix transformation() const;
@@ -553,7 +613,7 @@ public:
      *
      * @param matrix the transformation matrix to apply
      */
-    void applyTransformation( const QMatrix &matrix );
+    void applyTransformation(const QMatrix &matrix);
 
     /**
      * Copy all the settings from the parameter shape and apply them to this shape.
@@ -573,9 +633,32 @@ public:
     static void applyConversion(QPainter &painter, const KoViewConverter &converter);
 
     /**
-     * Return all the connections made to or from this shape.
+     * @brief Transforms point from shape coordinates to document coordinates
+     * @param point in shape coordinates
+     * @return point in document coordinates
      */
-    QList<KoShapeConnection*> connections() const;
+    QPointF shapeToDocument(const QPointF &point) const;
+
+    /**
+     * @brief Transforms rect from shape coordinates to document coordinates
+     * @param rect in shape coordinates
+     * @return rect in document coordinates
+     */
+    QRectF shapeToDocument(const QRectF &rect) const;
+
+    /**
+     * @brief Transforms point from document coordinates to shape coordinates
+     * @param point in document coordinates
+     * @return point in shape coordinates
+     */
+    QPointF documentToShape(const QPointF &point) const;
+
+    /**
+     * @brief Transform rect from document coordinates to shape coordinates
+     * @param rect in document coordinates
+     * @return rect in shape coordinates
+     */
+    QRectF documentToShape(const QRectF &rect) const;
 
     /**
      * Returns the name of the shape.
@@ -587,7 +670,7 @@ public:
      * Sets the name of the shape.
      * @param name the new shape name
      */
-    void setName( const QString & name );
+    void setName(const QString & name);
 
     /**
      * Update the position of the shape in the tree of the KoShapeManager.
@@ -614,18 +697,80 @@ public:
      */
     void deleteLater();
 
+    /// checks recursively if the shape or one of its parents is not visible or locked
+    bool isEditable() const;
+
+    /// Removes connection point with given index
+    void removeConnectionPoint(int index);
+
+    /// Adds a shape which depends on this shape
+    void addDependee(KoShape * shape);
+
+    /// Removes as shape depending on this shape
+    void removeDependee(KoShape * shape);
+
+    /// Returns additional snap data the shape wants to have snapping to
+    virtual KoSnapData snapData() const;
+
+    /**
+     * Set additional attribute
+     *
+     * This can be used to attach additional attributes to a shape for attributes
+     * that are application specific like presentation:placeholder
+     *
+     * @param name The name of the attribute in the following form prefix:tag e.g. presentation:placeholder
+     * @param value The value of the attribute
+     */
+    void setAdditionalAttribute(const char * name, const QString & value);
+
+    /**
+     * Remove additional attribute
+     *
+     * @param name The name of the attribute in the following form prefix:tag e.g. presentation:placeholder
+     */
+    void removeAdditionalAttribute(const char * name);
+
+    /**
+     * Check if additional attribute is set
+     *
+     * @param name The name of the attribute in the following form prefix:tag e.g. presentation:placeholder
+     *
+     * @return true if there is a attribute with prefix:tag set, false otherwise
+     */
+    bool hasAdditionalAttribute(const char * name);
+
+    /**
+     * Get additional attribute
+     *
+     * @param name The name of the attribute in the following form prefix:tag e.g. presentation:placeholder
+     *
+     * @return The value of the attribute if it exists or a null string if not found.
+     */
+    QString additionalAttribute(const char * name);
+
 protected:
 
-/* ** loading saving helper methods */
+    /* ** loading saving helper methods */
     /// attributes from ODF 1.1 chapter 9.2.15 Common Drawing Shape Attributes
     enum OdfAttribute {
-        OdfTransformation = 1,  ///< Store transformation information
-        OdfSize = 2,            ///< Store size information
-        OdfPosition = 4,        ///< Store position of shape
-        OdfMandatories = 8,     ///< Id, z-index, layer and style
+        OdfTransformation = 1,       ///< Store transformation information
+        OdfSize = 2,                 ///< Store size information
+        OdfPosition = 8,             ///< Store position
+        OdfAdditionalAttributes = 4, ///< Store additional attributes of the shape
+        OdfCommonChildElements = 16, ///< Event actions and connection points
+        OdfLayer = 64,               ///< Store layer name
+        OdfStyle = 128,              ///< Store the style
+        OdfId = 256,                 ///< Store the unique ID
+        OdfName = 512,               ///< Store the name of the shape
+        OdfZIndex = 1024,            ///< This only loads the z-index; when saving, it is reflected by the order of the shapes.
+        OdfViewbox = 2048,           ///< Store the viewbox
 
-        /// A mask for all the attributes a 'draw:frame' requires
-        FrameAttributes = OdfMandatories | OdfSize | OdfPosition | OdfTransformation
+        /// A mask for all mandatory attributes
+        OdfMandatories = OdfLayer | OdfStyle | OdfId | OdfName | OdfZIndex,
+        /// A mask for geometry attributes
+        OdfGeometry = OdfPosition | OdfSize,
+        /// A mask for all the attributes
+        OdfAllAttributes = OdfTransformation | OdfGeometry | OdfAdditionalAttributes | OdfMandatories | OdfCommonChildElements
     };
 
     /**
@@ -635,29 +780,14 @@ protected:
      * @param element element which represents the shape in odf
      * @param attributes a number of OdfAttribute items to state which attributes to load.
      */
-    bool loadOdfAttributes( const KoXmlElement & element, KoShapeLoadingContext &context, int attributes );
+    bool loadOdfAttributes(const KoXmlElement & element, KoShapeLoadingContext &context, int attributes);
 
     /**
      * Parses the transformation attribute from the given string
      * @param transform the transform attribute string
      * @return the resulting transformation matrix
      */
-    QMatrix parseOdfTransform( const QString &transform );
-
-    /**
-     * This method can be used while saving the shape as ODF to add the data
-     * stored on this shape to the current element.
-     *
-     * @param context the context for the current save.
-     * @param attributes a number of OdfAttribute items to state which attributes to save.
-     * @see saveOdf
-     */
-    void saveOdfAttributes( KoShapeSavingContext &context, int attributes ) const;
-
-    /**
-     * Add a new draw-glue-point element for each connections() present on this shape.
-     */
-    void saveOdfConnections( KoShapeSavingContext &context ) const;
+    QMatrix parseOdfTransform(const QString &transform);
 
     /**
      * @brief Saves the style used for the shape
@@ -670,7 +800,7 @@ protected:
      * @return the name of the style
      * @see saveOdf
      */
-    virtual QString saveStyle( KoGenStyle &style, KoShapeSavingContext &context ) const;
+    virtual QString saveStyle(KoGenStyle &style, KoShapeSavingContext &context) const;
 
     /**
      * Loads the stroke and fill style from the given element.
@@ -678,20 +808,24 @@ protected:
      * @param element the xml element to  load the style from
      * @param context the loading context used for loading
      */
-    virtual void loadStyle( const KoXmlElement & element, KoShapeLoadingContext &context );
+    virtual void loadStyle(const KoXmlElement & element, KoShapeLoadingContext &context);
 
     /// Loads the fill style
-    QBrush loadOdfFill( const KoXmlElement & element, KoShapeLoadingContext & context );
+    //QBrush loadOdfFill( const KoXmlElement & element, KoShapeLoadingContext & context );
+    KoShapeBackground * loadOdfFill(const KoXmlElement & element, KoShapeLoadingContext & context);
 
     /// Loads the stroke style
-    KoShapeBorderModel * loadOdfStroke( const KoXmlElement & element, KoShapeLoadingContext & context );
+    KoShapeBorderModel * loadOdfStroke(const KoXmlElement & element, KoShapeLoadingContext & context);
+
+    /// Loads the shadow style
+    KoShapeShadow * loadOdfShadow(const KoXmlElement & element, KoShapeLoadingContext & context);
 
     /**
      * Fills the style stack and returns the value of the given style property (e.g fill, stroke).
      */
-    QString getStyleProperty( const char *property, const KoXmlElement & element, KoShapeLoadingContext & context );
+    QString getStyleProperty(const char *property, const KoXmlElement & element, KoShapeLoadingContext & context);
 
-/* ** end loading saving */
+    /* ** end loading saving */
 
 
     /**
@@ -699,7 +833,12 @@ protected:
      * This is called whenever the shape, position rotation or scale properties were altered.
      * @param type an indicator which type was changed.
      */
-    virtual void shapeChanged(ChangeType type) { Q_UNUSED(type); }
+    virtual void shapeChanged(ChangeType type) {
+        Q_UNUSED(type);
+    }
+
+    /// This is called whenever a shape this shape depends on has changed
+    virtual void notifyShapeChanged(KoShape * shape, ChangeType type);
 
     /**
      * Set the property collision detection.
@@ -717,29 +856,16 @@ protected:
     /// return the current matrix that contains the rotation/scale/position of this shape
     const QMatrix& matrix() const;
 
-    friend class KoShapeConnection;
-    /**
-     * Add a connection to the list of connections of this shape.
-     * This is typically called only from the constructor of the KoShapeConnection class.
-     */
-    void addConnection(KoShapeConnection *connection);
-    /**
-     * Remove a connection to the list of connections of this shape.
-     * This is typically called only from the destructor of the KoShapeConnection class.
-     */
-    void removeConnection(KoShapeConnection *connection);
-
-    /// Removes connection point with given index
-    void removeConnectionPoint( int index );
-
 private:
     friend class KoShapeManager;
-    void addShapeManager( KoShapeManager * manager );
-    void removeShapeManager( KoShapeManager * manager );
+    void addShapeManager(KoShapeManager * manager);
+    void removeShapeManager(KoShapeManager * manager);
 
     class Private;
     friend class Private;
     Private * const d;
 };
 
+Q_DECLARE_METATYPE(KoShape*)
+
 #endif
