@@ -18,25 +18,24 @@
  */
 
 #include "KoShapeBackground.h"
-#include "KoShapeBackground_p.h"
 
-KoShapeBackgroundPrivate::KoShapeBackgroundPrivate()
-    : refCount(0)
+#include <QtCore/QAtomicInt>
+
+class KoShapeBackground::Private
 {
-}
-KoShapeBackground::KoShapeBackground(KoShapeBackgroundPrivate &dd)
-    :d_ptr(&dd)
-{
-}
+public:
+    Private() : refCount(0) { }
+    QAtomicInt refCount;
+};
 
 KoShapeBackground::KoShapeBackground()
-    : d_ptr(new KoShapeBackgroundPrivate())
+        : d(new Private())
 {
 }
 
 KoShapeBackground::~KoShapeBackground()
 {
-    delete d_ptr;
+    delete d;
 }
 
 bool KoShapeBackground::hasTransparency()
@@ -46,18 +45,15 @@ bool KoShapeBackground::hasTransparency()
 
 void KoShapeBackground::addUser()
 {
-    Q_D(KoShapeBackground);
     d->refCount.ref();
 }
 
 bool KoShapeBackground::removeUser()
 {
-    Q_D(KoShapeBackground);
     return d->refCount.deref();
 }
 
 int KoShapeBackground::useCount() const
 {
-    Q_D(const KoShapeBackground);
     return d->refCount;
 }
