@@ -25,7 +25,8 @@
 #include <QPainter>
 #include <QPainterPath>
 
-class ChildrenData : public KoShapeContainerModel {
+class ChildrenData : public KoShapeContainerModel
+{
 public:
     ChildrenData() {}
     ~ChildrenData() {
@@ -39,26 +40,26 @@ public:
 
     void setClipping(const KoShape *child, bool clipping) {
         Relation *relation = findRelation(child);
-        if(relation == 0) // throw exception?
+        if (relation == 0) // throw exception?
             return;
-        if(relation->m_inside == clipping)
+        if (relation->m_inside == clipping)
             return;
         relation->m_inside = clipping;
-        relation->child()->repaint();
+        relation->child()->update();
         relation->child()->notifyChanged();
-        relation->child()->repaint();
+        relation->child()->update();
     }
 
     bool childClipped(const KoShape *child) const {
         Relation *relation = findRelation(child);
-        if(relation == 0) // throw exception?
+        if (relation == 0) // throw exception?
             return false;
         return relation->m_inside;
     }
 
     void remove(KoShape *child) {
         Relation *relation = findRelation(child);
-        if(relation == 0)
+        if (relation == 0)
             return;
         m_relations.removeAll(relation);
     }
@@ -69,8 +70,8 @@ public:
 
     QList<KoShape*> iterator() const {
         QList<KoShape*> answer;
-        foreach (Relation *relation, m_relations)
-            answer.append(relation->child());
+        foreach(Relation *relation, m_relations)
+        answer.append(relation->child());
         return answer;
     }
 
@@ -79,24 +80,27 @@ public:
     }
 
     void containerChanged(KoShapeContainer *) { }
-    void childChanged(KoShape *, KoShape::ChangeType ) { }
+    void childChanged(KoShape *, KoShape::ChangeType) { }
 
 private:
     /**
      * This class is a simple data-storage class for Relation objects.
      */
-    class Relation {
-        public:
-            explicit Relation(KoShape *child) :m_inside(false) , m_child(child) { }
-            KoShape* child() { return m_child; }
-            bool m_inside; ///< if true, the child will be clipped by the parent.
-        private:
-            KoShape *m_child;
+    class Relation
+    {
+    public:
+        explicit Relation(KoShape *child) : m_inside(false) , m_child(child) { }
+        KoShape* child() {
+            return m_child;
+        }
+        bool m_inside; ///< if true, the child will be clipped by the parent.
+    private:
+        KoShape *m_child;
     };
 
     Relation* findRelation(const KoShape *child) const {
         foreach(Relation *relation, m_relations) {
-            if(relation->child() == child)
+            if (relation->child() == child)
                 return relation;
         }
         return 0;
@@ -106,79 +110,87 @@ private: // members
     QList <Relation *> m_relations;
 };
 
-class KoShapeContainer::Private {
+class KoShapeContainer::Private
+{
 public:
     Private() : children(0) {}
     ~Private() {
-        if(children)
-        {
-            foreach (KoShape *shape, children->iterator())
-                shape->setParent(0);
+        if (children) {
+            foreach(KoShape *shape, children->iterator())
+            shape->setParent(0);
             delete children;
         }
     }
     KoShapeContainerModel *children;
 };
 
-KoShapeContainer::KoShapeContainer() : KoShape(), d(new Private()) {
+KoShapeContainer::KoShapeContainer() : KoShape(), d(new Private())
+{
 }
 
 KoShapeContainer::KoShapeContainer(KoShapeContainerModel *model)
-: KoShape(),
- d(new Private())
+        : KoShape(),
+        d(new Private())
 {
     d->children = model;
 }
 
-KoShapeContainer::~KoShapeContainer() {
+KoShapeContainer::~KoShapeContainer()
+{
     delete d;
 }
 
-void KoShapeContainer::addChild(KoShape *shape) {
+void KoShapeContainer::addChild(KoShape *shape)
+{
     Q_ASSERT(shape);
-    if(shape->parent() == this && iterator().contains( shape ) )
+    if (shape->parent() == this && iterator().contains(shape))
         return;
-    if(d->children == 0)
+    if (d->children == 0)
         d->children = new ChildrenData();
-    if( shape->parent() )
-        shape->parent()->removeChild( shape );
+    if (shape->parent() && shape->parent() != this)
+        shape->parent()->removeChild(shape);
     d->children->add(shape);
     shape->setParent(this);
     childCountChanged();
 }
 
-void KoShapeContainer::removeChild(KoShape *shape) {
+void KoShapeContainer::removeChild(KoShape *shape)
+{
     Q_ASSERT(shape);
-    if(d->children == 0)
+    if (d->children == 0)
         return;
     d->children->remove(shape);
     shape->setParent(0);
     childCountChanged();
 }
 
-int  KoShapeContainer::childCount() const {
-    if(d->children == 0)
+int  KoShapeContainer::childCount() const
+{
+    if (d->children == 0)
         return 0;
     return d->children->count();
 }
 
-bool KoShapeContainer::isChildLocked(const KoShape *child) const {
-    if(d->children == 0)
+bool KoShapeContainer::isChildLocked(const KoShape *child) const
+{
+    if (d->children == 0)
         return false;
     return d->children->isChildLocked(child);
 }
 
-void KoShapeContainer::setClipping(const KoShape *child, bool clipping) {
-    if(d->children == 0)
+void KoShapeContainer::setClipping(const KoShape *child, bool clipping)
+{
+    if (d->children == 0)
         return;
     d->children->setClipping(child, clipping);
 }
 
-void KoShapeContainer::paint(QPainter &painter, const KoViewConverter &converter) {
+void KoShapeContainer::paint(QPainter &painter, const KoViewConverter &converter)
+{
     painter.save();
     paintComponent(painter, converter);
     painter.restore();
-    if(d->children == 0 || d->children->count() == 0)
+    if (d->children == 0 || d->children->count() == 0)
         return;
 
     QList<KoShape*> sortedObjects = d->children->iterator();
@@ -188,70 +200,75 @@ void KoShapeContainer::paint(QPainter &painter, const KoViewConverter &converter
 
     // clip the children to the parent outline.
     QMatrix m;
-    double zoomX, zoomY;
+    qreal zoomX, zoomY;
     converter.zoom(&zoomX, &zoomY);
     m.scale(zoomX, zoomY);
     painter.setClipPath(m.map(outline()));
 
-    foreach (KoShape *shape, sortedObjects) {
+    foreach(KoShape *shape, sortedObjects) {
         //kDebug(30006) <<"KoShapeContainer::painting shape:" << shape->shapeId() <<"," << shape->boundingRect();
-        if(! shape->isVisible())
+        if (! shape->isVisible())
             continue;
-        if(! childClipped(shape) ) // the shapeManager will have to draw those, or else we can't do clipRects
-            return;
-        if(painter.hasClipping()) {
-            QRectF rect = converter.viewToDocument( painter.clipRegion().boundingRect() );
+        if (! childClipped(shape))  // the shapeManager will have to draw those, or else we can't do clipRects
+            continue;
+        if (painter.hasClipping()) {
+            QRectF rect = converter.viewToDocument(painter.clipRegion().boundingRect());
             rect = matrix().mapRect(rect);
             // don't try to draw a child shape that is not in the clipping rect of the painter.
-            if(! rect.intersects(shape->boundingRect()))
+            if (! rect.intersects(shape->boundingRect()))
                 continue;
         }
 
         painter.save();
-        painter.setMatrix( shape->absoluteTransformation(&converter) * baseMatrix );
+        painter.setMatrix(shape->absoluteTransformation(&converter) * baseMatrix);
         shape->paint(painter, converter);
         painter.restore();
-        if(shape->border()) {
+        if (shape->border()) {
             painter.save();
-            painter.setMatrix( shape->absoluteTransformation(&converter) * baseMatrix );
+            painter.setMatrix(shape->absoluteTransformation(&converter) * baseMatrix);
             shape->border()->paintBorder(shape, painter, converter);
             painter.restore();
         }
     }
 }
 
-void KoShapeContainer::shapeChanged(ChangeType type) {
-    if(d->children == 0)
+void KoShapeContainer::shapeChanged(ChangeType type)
+{
+    if (d->children == 0)
         return;
-    if(! (type == RotationChanged || type == ScaleChanged || type == ShearChanged
-                || type == SizeChanged || type == PositionChanged))
+    if (!(type == RotationChanged || type == ScaleChanged || type == ShearChanged
+            || type == SizeChanged || type == PositionChanged))
         return;
     d->children->containerChanged(this);
-    foreach (KoShape *shape, d->children->iterator())
-        shape->notifyChanged();
+    foreach(KoShape *shape, d->children->iterator())
+    shape->notifyChanged();
 }
 
-bool KoShapeContainer::childClipped(const KoShape *child) const {
-    if(d->children == 0) // throw exception??
+bool KoShapeContainer::childClipped(const KoShape *child) const
+{
+    if (d->children == 0) // throw exception??
         return false;
     return d->children->childClipped(child);
 }
 
-void KoShapeContainer::repaint() const {
-    KoShape::repaint();
-    if(d->children)
-        foreach ( KoShape *shape, d->children->iterator())
-            shape->repaint();
+void KoShapeContainer::update() const
+{
+    KoShape::update();
+    if (d->children)
+        foreach(KoShape *shape, d->children->iterator())
+        shape->update();
 }
 
-QList<KoShape*> KoShapeContainer::iterator() const {
-    if(d->children == 0)
+QList<KoShape*> KoShapeContainer::iterator() const
+{
+    if (d->children == 0)
         return QList<KoShape*>();
 
     return d->children->iterator();
 }
 
-KoShapeContainerModel *KoShapeContainer::model() const {
+KoShapeContainerModel *KoShapeContainer::model() const
+{
     return d->children;
 }
 
