@@ -1,6 +1,7 @@
 /* This file is part of the KDE project
  * Copyright (c) 2006 Boudewijn Rempt (boud@valdyas.org)
  * Copyright (C) 2006-2007 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2008 Thorsten Zachmann <zachmann@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -25,8 +26,9 @@
 #include <QString>
 #include <QWidget>
 #include <QList>
+#include <QMap>
 
-#include <flake_export.h>
+#include "flake_export.h"
 
 #include <KoXmlReader.h>
 
@@ -34,6 +36,8 @@ class KoShape;
 class KoProperties;
 class KoShapeConfigFactory;
 class KoShapeConfigWidgetBase;
+class KoShapeControllerBase;
+class KoDataCenter;
 
 #define SHAPETEMPLATE_MIMETYPE "application/x-flake-shapetemplate"
 #define SHAPEID_MIMETYPE "application/x-flake-shapeId"
@@ -45,13 +49,15 @@ class KoShapeConfigWidgetBase;
  */
 struct FLAKE_EXPORT KoShapeTemplate {
     KoShapeTemplate() {
-        id="";
-        name="";
-        toolTip="";
-        icon="";
+        id = "";
+        name = "";
+        toolTip = "";
+        icon = "";
+        properties = 0;
     }
     QString id;         ///< The id of the shape
     QString name;       ///< The name to be shown for this template
+    QString family;       ///< The family of the shape (possible values are: "funny","arrow")
     QString toolTip;    ///< The tooltip text for the template
     QString icon;       ///< Icon name
     /**
@@ -83,7 +89,8 @@ public:
  * After you created the factory you should create a plugin that can announce the factory to the
  * KoShapeRegistry.  See the KoPluginLoader as well.
  */
-class FLAKE_EXPORT KoShapeFactory : public QObject {
+class FLAKE_EXPORT KoShapeFactory : public QObject
+{
     Q_OBJECT
 public:
 
@@ -92,28 +99,53 @@ public:
      * @param parent the parent QObject for memory management usage.
      * @param id a string that will be used internally for referencing the shape, for
      *   example for use by the KoTool::activateTemporary.
-     * @param name the user visible name of the tool this factory creates.
+     * @param name the user visible name of the shape this factory creates.
      */
     KoShapeFactory(QObject *parent, const QString &id, const QString &name);
     virtual ~KoShapeFactory();
 
     /**
-     * This method should be implemented by factories to create a shape that the user
-     * gets when doing a base insert. For example from a script.  The created shape
-     * should have its values set to good defaults that the user can then adjust further if
-     * needed.  Including the KoShape:setShapeId(), with the Id from this factory
-     * The default shape position is not relevant, it will be moved by the caller.
+     * This method should be called to create a shape that the user gets when doing a base insert.
+     * For example from a script or during loading.
+     *
+     * This function will call init on the shape with the dataCenterMap that was given. You
+     * should make sure to pass the dataCenterMap so that all works as expected.
+     * However in some cases it is ok to not pass the shapeController e.g. when it is only
+     * a temporary shape that will not be inserted in the document. The shape can then set
+     * things that e.g. need to be available during loading of the shape.
+     * This method internally calls createShape( params );
+     *
+     * @param dataCenterMap The dataCenterMap of the document.
+     *
      * @return a new shape
+     *
+     * @see KoShapeFactory::createDefaultShape
+     * @see KoShape::init
      */
-    virtual KoShape * createDefaultShape() const = 0;
+    KoShape * createDefaultShapeAndInit(const QMap<QString, KoDataCenter *> & dataCenterMap) const;
+
     /**
-     * This method should be implemented by factories to create a shape based on a set of
-     * properties that are specifically made for this shape-type.
-     * This method should also set this factories shapeId on the shape using KoShape::setShapeId()
+     * This method should be called to create a shape based on a set of properties that are
+     * specifically made for this shape-type.
+     *
+     * This function will call init on the shape with the dataCenterMap that was given. You
+     * should make sure to pass the dataCenterMap so that all works as expected.
+     * However in some cases it is ok to not pass the shapeController e.g. when it is only
+     * a temporary shape that will not be inserted in the document. The shape can then set
+     * things that e.g. need to be available during loading of the shape.
+     * This method internally calls createShape( params );
+     *
+     * @param params The properties to used to create the shape
+     * @param dataCenterMap The dataCenterMap of the document.
+     *
      * @return a new shape
+     *
+     * @see KoShapeFactory::createShape
      * @see KoShapeTemplate::properties
+     * @see KoShape::init
      */
-    virtual KoShape * createShape(const KoProperties * params) const = 0;
+    KoShape * createShapeAndInit(const KoProperties * params, const QMap<QString, KoDataCenter *> & dataCenterMap) const;
+
     /**
      * Create a list of option panels to show on creating a new shape.
      * The shape type this factory creates may have general or specific setting panels
@@ -149,6 +181,14 @@ public:
      */
     const QList<KoShapeConfigFactory*> &panelFactories();
 
+    /**
+     * pouplates the dataCenterMap with any datacenters the shapes might need.
+     * The default implementation does nothing
+     * @param dataCenterMap The map which the shapeControllerBase owns.
+     */
+    virtual void populateDataCenterMap(QMap<QString, KoDataCenter *>   & dataCenterMap) {
+        Q_UNUSED(dataCenterMap)
+    }
     /**
      * return the id for the shape this factory creates.
      * @return the id for the shape this factory creates.
@@ -174,7 +214,11 @@ public:
      * @return the user visible (and translated) name to be seen by the user.
      */
     QString name() const;
-
+    /**
+     * return the non-visible name of the family the default shape belongs to.
+     * @return the family name.
+     */
+    QString family() const;
     /// lower prio means the shape is more generic and will be checked later
     quint32 loadingPriority() const;
 
@@ -190,8 +234,34 @@ public:
     /// started at argument element. ('draw:line' / 'draw:frame' / etc)
     virtual bool supports(const KoXmlElement & e) const;
 
+    /**
+     * This functions decides if the shape should be shown in the
+     * shape selector or not.
+     * The default implementation returns false.
+     */
+    virtual bool hidden() const;
+
 protected:
 
+    /**
+     * This method should be implemented by factories to create a shape that the user
+     * gets when doing a base insert. For example from a script.  The created shape
+     * should have its values set to good defaults that the user can then adjust further if
+     * needed.  Including the KoShape:setShapeId(), with the Id from this factory
+     * The default shape position is not relevant, it will be moved by the caller.
+     * @return a new shape
+     */
+    virtual KoShape * createDefaultShape() const = 0;
+
+    /**
+     * This method should be implemented by factories to create a shape based on a set of
+     * properties that are specifically made for this shape-type.
+     * This method should also set this factories shapeId on the shape using KoShape::setShapeId()
+     * @return a new shape
+     * @see KoShapeTemplate::properties
+     */
+    virtual KoShape * createShape(const KoProperties * params) const = 0;
+
     /**
      * Add a template with the properties of a speficic type of shape this factory can generate
      * using the createShape() method.
@@ -212,12 +282,19 @@ protected:
      */
     void setIcon(const QString & iconName);
 
+    /**
+     * Set the family name of the default shape
+     * @param family the family name of the default shape this factory creates.
+     *   for example "funny", "arrows", "geometrics". Use "" for default
+     */
+    void setFamily(const QString &family);
+
     /**
      * Set the loading priority for this icon; higher priority means
      * the shape is more specific which means it will be earlier in
      * the queue to try loading a particular odf element.
      */
-    void setLoadingPriority( quint32 priority );
+    void setLoadingPriority(quint32 priority);
 
     /**
      * Set the name used for quick checking whether this shapefactory
@@ -230,7 +307,7 @@ protected:
      * @param elementNames the name of the element itself, like "draw"
      *
      */
-    void setOdfElementNames( const QString & nameSpace, const QStringList & elementNames );
+    void setOdfElementNames(const QString & nameSpace, const QStringList & elementNames);
 
 private:
     class Private;
