@@ -24,7 +24,7 @@
 
 #include <QList>
 
-#include <flake_export.h>
+#include "flake_export.h"
 
 class KoShapeSavingContext;
 class KoShapeLoadingContext;
@@ -33,14 +33,16 @@ class KoShapeLoadingContext;
  * Provide grouping for shapes.
  * The group shape allows you to add children which will then be grouped in selections
  * and actions.
- * <p>If you have a set of shapes, that together make up a bigger shape it is often
+ * <p>If you have a set of shapes that together make up a bigger shape it is often
  * useful to group them together so the user will perceive the different shapes as
- * actually being one.  This means that if the user clicks on one object, all objects
- * in the group will be selected at ones, making the tools that works on
+ * actually being one.  This means that if the user clicks on one shape, all shapes
+ * in the group will be selected at once, making the tools that works on
  * selections alter all of them at the same time.
- * <p>Note that while this object is also a shape, it is not actually visible.
+ * <p>Note that while this object is also a shape, it is not actually visible and the user
+ * can't interact with it.
  */
-class FLAKE_EXPORT KoShapeGroup : public KoShapeContainer {
+class FLAKE_EXPORT KoShapeGroup : public KoShapeContainer
+{
 public:
     /// Constructor
     KoShapeGroup();
@@ -49,14 +51,13 @@ public:
     /// This implementation is empty since a group is itself not visible.
     void paintComponent(QPainter &painter, const KoViewConverter &converter);
     /// always returns false since the group itself can't be selected or hit
-    bool hitTest( const QPointF &position ) const;
+    bool hitTest(const QPointF &position) const;
     /// reimplemented from KoShape
-    virtual void saveOdf( KoShapeSavingContext & context ) const;
+    virtual void saveOdf(KoShapeSavingContext & context) const;
     // reimplemented
-    virtual bool loadOdf( const KoXmlElement & element, KoShapeLoadingContext &context );
+    virtual bool loadOdf(const KoXmlElement & element, KoShapeLoadingContext &context);
 
 private:
-
     void childCountChanged();
 };
 
