@@ -29,7 +29,7 @@
 #include <KoPluginLoader.h>
 #include <KoXmlReader.h>
 #include <KoXmlNS.h>
-#include <KoOasisLoadingContext.h>
+#include <KoOdfLoadingContext.h>
 #include <KoStyleStack.h>
 
 #include <QString>
@@ -37,40 +37,8 @@
 #include <QMultiMap>
 #include <QPainter>
 
-#include <kdebug.h>
-#include <k3staticdeleter.h>
-
-// just for loading the frame attributes
-class FrameShape : public KoShape
-{
-public:
-    virtual void paint(QPainter &painter, const KoViewConverter &converter) 
-    {
-        Q_UNUSED( painter );
-        Q_UNUSED( converter );
-    };
-
-    virtual bool loadOdf( const KoXmlElement & element, KoShapeLoadingContext &context )
-    {
-        loadOdfAttributes( element, context, FrameAttributes );
-        return true;
-    };
-    virtual void saveOdf( KoShapeSavingContext & context ) const 
-    {
-        Q_UNUSED( context );
-    };
-
-    void setAttributes( KoShape * shape )
-    {
-        if( ! shape )
-            return;
-        shape->setSize( size() );
-        shape->setTransformation( transformation() );
-        shape->setZIndex( zIndex() );
-        shape->setName( name() );
-    }
-};
-
+#include <KDebug>
+#include <KGlobal>
 
 class KoShapeRegistry::Private
 {
@@ -81,7 +49,7 @@ public:
 };
 
 KoShapeRegistry::KoShapeRegistry()
-    : d(new Private())
+        : d(new Private())
 {
 }
 
@@ -90,14 +58,15 @@ KoShapeRegistry::~KoShapeRegistry()
     delete d;
 }
 
-void KoShapeRegistry::init() {
+void KoShapeRegistry::init()
+{
     KoPluginLoader::PluginsConfig config;
     config.whiteList = "FlakePlugins";
     config.blacklist = "FlakePluginsDisabled";
     config.group = "koffice";
-    KoPluginLoader::instance()->load( QString::fromLatin1("KOffice/Flake"),
-                                      QString::fromLatin1("[X-Flake-MinVersion] <= 0"),
-                                      config);
+    KoPluginLoader::instance()->load(QString::fromLatin1("KOffice/Flake"),
+                                     QString::fromLatin1("[X-Flake-MinVersion] <= 0"),
+                                     config);
     config.whiteList = "ShapePlugins";
     config.blacklist = "ShapePluginsDisabled";
     KoPluginLoader::instance()->load(QString::fromLatin1("KOffice/Shape"),
@@ -105,123 +74,120 @@ void KoShapeRegistry::init() {
                                      config);
 
     // Also add our hard-coded basic shape
-    add( new KoPathShapeFactory(this, QStringList()) );
-    add( new KoConnectionShapeFactory( this ) );
+    add(new KoPathShapeFactory(this, QStringList()));
+    add(new KoConnectionShapeFactory(this));
 
     // Now all shape factories are registered with us, determine their
     // assocated odf tagname & priority and prepare ourselves for
     // loading ODF.
 
     QList<KoShapeFactory*> factories = values();
-    for ( int i = 0; i < factories.size(); ++i ) {
-        KoShapeFactory * factory = factories[i];
-        if ( factory->odfNameSpace().isEmpty() || factory->odfElementNames().isEmpty() )
-        {
-            kDebug(30006) <<"Shape factory" << factory->id() <<" does not have OdfNamespace defined, ignoring";
-        }
-        else {
-            foreach( QString elementName, factory->odfElementNames() ) {
-
-                QPair<QString, QString> p ( factory->odfNameSpace(), elementName );
-
-                QMultiMap<int, KoShapeFactory*> priorityMap = d->factoryMap[p];
-
-                d->factoryMap[p].insert( factory->loadingPriority(), factory );
-
-                kDebug(30006) <<"Inserting factory" << factory->id() <<" for"
-                         << p << " with priority "
-                         << factory->loadingPriority() << " into factoryMap making "
-                         << d->factoryMap[p].size() << " entries. " << endl;
-            }
-        }
+    for (int i = 0; i < factories.size(); ++i) {
+        insertFactory(factories[i]);
     }
 }
 
-KoShapeRegistry *KoShapeRegistry::s_singleton = 0;
-static K3StaticDeleter<KoShapeRegistry> staticShapeRegistryDeleter;
-
 KoShapeRegistry* KoShapeRegistry::instance()
 {
-    if(KoShapeRegistry::s_singleton == 0)
-    {
-        staticShapeRegistryDeleter.setObject(s_singleton, new KoShapeRegistry());
-        KoShapeRegistry::s_singleton->init();
+    K_GLOBAL_STATIC(KoShapeRegistry, s_instance)
+    if (!s_instance.exists()) {
+        s_instance->init();
+    }
+    return s_instance;
+}
+
+void KoShapeRegistry::addFactory(KoShapeFactory * factory)
+{
+    add(factory);
+    insertFactory(factory);
+}
+
+void KoShapeRegistry::insertFactory(KoShapeFactory * factory)
+{
+    if (factory->odfNameSpace().isEmpty() || factory->odfElementNames().isEmpty()) {
+        kDebug(30006) << "Shape factory" << factory->id() << " does not have OdfNamespace defined, ignoring";
+    } else {
+        foreach(const QString & elementName, factory->odfElementNames()) {
+
+            QPair<QString, QString> p(factory->odfNameSpace(), elementName);
+
+            QMultiMap<int, KoShapeFactory*> priorityMap = d->factoryMap[p];
+
+            d->factoryMap[p].insert(factory->loadingPriority(), factory);
+
+            kDebug(30006) << "Inserting factory" << factory->id() << " for"
+            << p << " with priority "
+            << factory->loadingPriority() << " into factoryMap making "
+            << d->factoryMap[p].size() << " entries. " << endl;
+        }
     }
-    return KoShapeRegistry::s_singleton;
 }
 
 KoShape * KoShapeRegistry::createShapeFromOdf(const KoXmlElement & e, KoShapeLoadingContext & context) const
 {
-    kDebug(30006) <<"Going to check for" << e.namespaceURI() <<":" << e.tagName();
+    kDebug(30006) << "Going to check for" << e.namespaceURI() << ":" << e.tagName();
 
     // If the element is in a frame, the frame is already added by the
     // application and we only want to create a shape from the
     // embedded element. The very first shape we create is accepted.
     // XXX: we might want to have some code to determine which is the
     // "best" of the creatable shapes.
-    if ( e.tagName() == "frame" && e.namespaceURI() == KoXmlNS::draw ) {
+    if (e.tagName() == "frame" && e.namespaceURI() == KoXmlNS::draw) {
 
         KoXmlElement element;
-        forEachElement( element, e ) {
-            KoShape * shape = createShapeInternal( element, context );
-            if ( shape )
-            {
-                FrameShape frame;
-                if( frame.loadOdf( e, context ) )
-                    frame.setAttributes( shape );
-
+        forEachElement(element, e) {
+            KoShape * shape = createShapeInternal(e, context, element);
+            if (shape) {
                 return shape;
             }
         }
     }
     // Hardwire the group shape into the loading as it should not appear
     // in the shape selector
-    else if( e.localName() == "g" && e.namespaceURI() == KoXmlNS::draw )
-    {
+    else if (e.localName() == "g" && e.namespaceURI() == KoXmlNS::draw) {
         KoShapeGroup * group = new KoShapeGroup();
 
-        context.koLoadingContext().styleStack().save();
-        bool loaded = group->loadOdf( e, context );
-        context.koLoadingContext().styleStack().restore();
+        context.odfLoadingContext().styleStack().save();
+        bool loaded = group->loadOdf(e, context);
+        context.odfLoadingContext().styleStack().restore();
 
-        if( loaded )
+        if (loaded)
             return group;
 
         delete group;
-    }
-    else {
-        return createShapeInternal( e, context );
+    } else {
+        return createShapeInternal(e, context, e);
     }
 
     return 0;
 }
 
-KoShape * KoShapeRegistry::createShapeInternal( const KoXmlElement & element, KoShapeLoadingContext & context ) const
+KoShape * KoShapeRegistry::createShapeInternal(const KoXmlElement & fullElement, KoShapeLoadingContext & context, const KoXmlElement & element) const
 {
     QPair<QString, QString> p = QPair<QString, QString>(element.namespaceURI(), element.tagName());
     kDebug(30006) << p;
 
-    if ( !d->factoryMap.contains( p ) ) return 0;
+    if (!d->factoryMap.contains(p)) return 0;
 
-    QMultiMap<int,KoShapeFactory*> priorityMap = d->factoryMap[p];
+    QMultiMap<int, KoShapeFactory*> priorityMap = d->factoryMap[p];
     QList<KoShapeFactory*> factories = priorityMap.values();
 
     // Higher numbers are more specific, map is sorted by keys
-    for ( int i = factories.size() - 1; i >= 0; --i ) {
+    for (int i = factories.size() - 1; i >= 0; --i) {
 
         KoShapeFactory * factory = factories[i];
-        if ( factory->supports( element ) ) {
+        if (factory->supports(element)) {
 
-            KoShape * shape = factory->createDefaultShape();
+            KoShape * shape = factory->createDefaultShapeAndInit(context.dataCenterMap());
 
-            if( shape->shapeId().isEmpty() )
+            if (shape->shapeId().isEmpty())
                 shape->setShapeId(factory->id());
 
-            context.koLoadingContext().styleStack().save();
-            bool loaded = shape->loadOdf( element, context );
-            context.koLoadingContext().styleStack().restore();
+            context.odfLoadingContext().styleStack().save();
+            bool loaded = shape->loadOdf(fullElement, context);
+            context.odfLoadingContext().styleStack().restore();
 
-            if( loaded )
+            if (loaded)
                 return shape;
 
             // Maybe a shape with a lower priority can load our
