@@ -1,8 +1,9 @@
 /* This file is part of the KDE project
  *
  * Copyright (c) 2005-2006 Boudewijn Rempt <boud@valdyas.org>
- * Copyright (C) 2006-2007 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2006-2008 Thomas Zander <zander@kde.org>
  * Copyright (C) 2006 Thorsten Zachmann <zachmann@kde.org>
+ * Copyright (C) 2008 Jan Hambrecht <jaham@gmx.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -24,19 +25,25 @@
 #include "KoToolManager_p.h"
 #include "KoToolRegistry.h"
 #include "KoToolProxy.h"
-#include "KoCreatePathToolFactory.h"
-#include "KoCreateShapesToolFactory.h"
-#include "KoCreateShapesTool.h"
-#include "KoInteractionToolFactory.h"
-#include "KoPathToolFactory.h"
+#include "KoSelection.h"
+#include "tools/KoCreatePathToolFactory.h"
+#include "tools/KoCreateShapesToolFactory.h"
+#include "tools/KoCreateShapesTool.h"
+#include "tools/KoPathToolFactory.h"
 #include "KoCanvasController.h"
+#include "KoShape.h"
+#include "KoShapeLayer.h"
 #include "KoShapeRegistry.h"
 #include "KoShapeManager.h"
 #include "KoCanvasBase.h"
-#include "KoZoomTool.h"
-#include "KoZoomToolFactory.h"
-#include "KoPanTool.h"
-#include "KoPanToolFactory.h"
+#include "KoDeviceRegistry.h"
+#include "KoDeviceEvent.h"
+#include "KoPointerEvent.h"
+#include "tools/KoZoomTool.h"
+#include "tools/KoZoomToolFactory.h"
+#include "tools/KoPanTool.h"
+#include "tools/KoPanToolFactory.h"
+#include "tools/KoGuidesTool.h"
 
 // Qt + kde
 #include <QWidget>
@@ -54,20 +61,20 @@
 #include <QTimer>
 #include <kactioncollection.h>
 #include <kdebug.h>
-#include <k3staticdeleter.h>
+#include <kglobal.h>
 #include <kaction.h>
 #include <QStack>
 #include <QLabel>
 
-class CanvasData {
+class CanvasData
+{
 public:
     CanvasData(KoCanvasController *cc, const KoInputDevice &id)
-        : activeTool(0),
-        canvas(cc),
-        inputDevice(id),
-        dummyToolWidget(0),
-        dummyToolLabel(0)
-    {
+            : activeTool(0),
+            canvas(cc),
+            inputDevice(id),
+            dummyToolWidget(0),
+            dummyToolLabel(0) {
     }
 
     KoTool *activeTool;     // active Tool
@@ -81,15 +88,17 @@ public:
     QLabel *dummyToolLabel;
 };
 
-class KoToolManager::Private {
+class KoToolManager::Private
+{
 public:
-    Private() : defaultTool(0), canvasData(0)
-    {
+    Private() : canvasData(0) {
         tabletEventTimer.setSingleShot(true);
     }
+    ~Private() {
+        qDeleteAll(tools);
+    }
 
     QList<ToolHelper*> tools; // list of all available tools via their factories.
-    ToolHelper *defaultTool; // the pointer thingy
 
     QHash<KoTool*, int> uniqueToolIds; // for the changedTool signal
     QHash<KoCanvasController*, QList<CanvasData*> > canvasses;
@@ -99,33 +108,38 @@ public:
 
     KoInputDevice inputDevice;
     QTimer tabletEventTimer; // Runs for a short while after any tablet event is
-                             // received to help correct input device detection.
+    // received to help correct input device detection.
 
     // helper method.
     CanvasData *createCanvasData(KoCanvasController *controller, KoInputDevice device) {
         QHash<QString, KoTool*> origHash;
-        if(canvasses.contains(controller))
+        if (canvasses.contains(controller))
             origHash = canvasses.value(controller).first()->allTools;
 
         QHash<QString, KoTool*> toolsHash;
         foreach(ToolHelper *tool, tools) {
-            if(tool->inputDeviceAgnostic() && origHash.contains(tool->id())) {
+            if (tool->inputDeviceAgnostic() && origHash.contains(tool->id())) {
                 // reuse ones that are marked as inputDeviceAgnostic();
                 toolsHash.insert(tool->id(), origHash.value(tool->id()));
                 continue;
             }
-            kDebug(30006) <<"Creating tool" << tool->id() <<". Activated on:" << tool->activationShapeId() <<", prio:" << tool->priority();
+            if (! tool->canCreateTool(controller->canvas())) {
+                kDebug(30006) << "Skipping the creation of tool" << tool->id();
+                continue;
+            }
+            kDebug(30006) << "Creating tool" << tool->id() << ". Activated on:" << tool->activationShapeId() << ", prio:" << tool->priority();
             KoTool *tl = tool->createTool(controller->canvas());
+            Q_ASSERT(tl);
             uniqueToolIds.insert(tl, tool->uniqueId());
             toolsHash.insert(tool->id(), tl);
             tl->setObjectName(tool->id());
-            foreach(QAction *action, tl->actions().values())
+            foreach(KAction *action, tl->actions())
                 action->setEnabled(false);
-            KoZoomTool *zoomTool = dynamic_cast<KoZoomTool*> (tl);
-            if(zoomTool)
+            KoZoomTool *zoomTool = dynamic_cast<KoZoomTool*>(tl);
+            if (zoomTool)
                 zoomTool->setCanvasController(controller);
-            KoPanTool *panTool = dynamic_cast<KoPanTool*> (tl);
-            if(panTool)
+            KoPanTool *panTool = dynamic_cast<KoPanTool*>(tl);
+            if (panTool)
                 panTool->setCanvasController(controller);
         }
         KoCreateShapesTool *createTool = dynamic_cast<KoCreateShapesTool*>(toolsHash.value(KoCreateShapesTool_ID));
@@ -141,47 +155,55 @@ public:
 
 // ******** KoToolManager **********
 KoToolManager::KoToolManager()
-    : QObject(),
-    d(new Private())
+        : QObject(),
+        d(new Private())
 {
-    connect(QApplication::instance(), SIGNAL(focusChanged(QWidget*,QWidget*)),
-            this, SLOT(movedFocus(QWidget*,QWidget*)));
+    connect(QApplication::instance(), SIGNAL(focusChanged(QWidget*, QWidget*)),
+            this, SLOT(movedFocus(QWidget*, QWidget*)));
     QApplication::instance()->installEventFilter(this);
 }
 
-KoToolManager::~KoToolManager() {
+KoToolManager::~KoToolManager()
+{
     delete d;
 }
 
-void KoToolManager::setup() {
+void KoToolManager::setup()
+{
     if (d->tools.size() > 0)
         return;
 
-    d->tools.append( new ToolHelper(new KoCreatePathToolFactory(this)) );
-    d->tools.append( new ToolHelper(new KoCreateShapesToolFactory(this)) );
-    d->tools.append( new ToolHelper(new KoPathToolFactory(this)) );
-    d->tools.append( new ToolHelper(new KoZoomToolFactory(this)) );
-    d->tools.append( new ToolHelper(new KoPanToolFactory(this)) );
-    d->defaultTool = new ToolHelper(new KoInteractionToolFactory(this));
-    d->tools.append(d->defaultTool);
+    d->tools.append(new ToolHelper(new KoCreatePathToolFactory(this)));
+    d->tools.append(new ToolHelper(new KoCreateShapesToolFactory(this)));
+    d->tools.append(new ToolHelper(new KoPathToolFactory(this)));
+    d->tools.append(new ToolHelper(new KoZoomToolFactory(this)));
+    d->tools.append(new ToolHelper(new KoPanToolFactory(this)));
 
     KoShapeRegistry::instance();
     KoToolRegistry *registry = KoToolRegistry::instance();
-    foreach(QString id, registry->keys()) {
+    foreach(const QString & id, registry->keys()) {
         ToolHelper *t = new ToolHelper(registry->value(id));
         d->tools.append(t);
     }
 
     // connect to all tools so we can hear their button-clicks
     foreach(ToolHelper *tool, d->tools)
-        connect(tool, SIGNAL(toolActivated(ToolHelper*)), this, SLOT(toolActivated(ToolHelper*)));
+    connect(tool, SIGNAL(toolActivated(ToolHelper*)), this, SLOT(toolActivated(ToolHelper*)));
+
+    // load pluggable input devices
+    KoDeviceRegistry::instance();
 }
 
-QList<KoToolManager::Button> KoToolManager::createToolList() const {
+QList<KoToolManager::Button> KoToolManager::createToolList(KoCanvasBase *canvas) const
+{
     QList<KoToolManager::Button> answer;
     foreach(ToolHelper *tool, d->tools) {
-        if(tool->id() == KoCreateShapesTool_ID)
+        if (tool->id() == KoCreateShapesTool_ID)
             continue; // don't show this one.
+        if (tool->id() == KoGuidesTool_ID)
+            continue; // don't show this one.
+        if (! tool->canCreateTool(canvas))
+            continue;
         Button button;
         button.button = tool->createButton();
         button.section = tool->toolType();
@@ -193,46 +215,43 @@ QList<KoToolManager::Button> KoToolManager::createToolList() const {
     return answer;
 }
 
-KoInputDevice KoToolManager::currentInputDevice() const {
+void KoToolManager::requestToolActivation(KoCanvasController * controller)
+{
+    if (d->canvasses.contains(controller)) {
+        QString activeToolId = d->canvasses.value(controller).first()->activeToolId;
+        foreach(ToolHelper * th, d->tools) {
+            if (th->id() == activeToolId) {
+                toolActivated(th);
+                break;
+            }
+        }
+    }
+}
+
+KoInputDevice KoToolManager::currentInputDevice() const
+{
     return d->inputDevice;
 }
 
-void KoToolManager::registerTools(KActionCollection *ac, KoCanvasController *controller) {
+void KoToolManager::registerTools(KActionCollection *ac, KoCanvasController *controller)
+{
     setup();
 
-    class ToolSwitchAction : public KAction {
-      public:
-        ToolSwitchAction(KActionCollection *parent, const QString &toolId) : KAction(parent) {
-            m_toolId = toolId;
-        }
-      private:
-        void slotTriggered() {
-            KoToolManager::instance()->switchToolRequested(m_toolId);
-        }
-        QString m_toolId;
-    };
-
-    foreach(ToolHelper *th, d->tools) {
-        ToolSwitchAction *tsa = new ToolSwitchAction(ac, th->id());
-        ac->addAction("tool_"+ th->name(), tsa);
-        tsa->setShortcut(th->shortcut());
-        tsa->setText(i18n("Activate %1", th->name()));
-    }
-
-    if(! d->canvasses.contains(controller)) {
+    if (! d->canvasses.contains(controller)) {
         kWarning(30006) << "registerTools called on a canvasController that has not been registered (yet)!\n";
         return;
     }
     CanvasData *cd = d->canvasses.value(controller).first();
     foreach(KoTool *tool, cd->allTools) {
-        QHash<QString, QAction*> actions = tool->actions();
-        foreach(QString name, actions.keys())
+        QHash<QString, KAction*> actions = tool->actions();
+        foreach(const QString & name, actions.keys())
             ac->addAction(name, actions[name]);
     }
 }
 
-void KoToolManager::addController(KoCanvasController *controller ) {
-    if(d->canvasses.keys().contains(controller))
+void KoToolManager::addController(KoCanvasController *controller)
+{
+    if (d->canvasses.keys().contains(controller))
         return;
     setup();
     attachCanvas(controller);
@@ -240,13 +259,15 @@ void KoToolManager::addController(KoCanvasController *controller ) {
     connect(controller, SIGNAL(canvasSet(KoCanvasController*)), this, SLOT(attachCanvas(KoCanvasController*)));
 }
 
-void KoToolManager::removeCanvasController(KoCanvasController *controller) {
+void KoToolManager::removeCanvasController(KoCanvasController *controller)
+{
     detachCanvas(controller);
     disconnect(controller, SIGNAL(canvasRemoved(KoCanvasController*)), this, SLOT(detachCanvas(KoCanvasController*)));
     disconnect(controller, SIGNAL(canvasSet(KoCanvasController*)), this, SLOT(attachCanvas(KoCanvasController*)));
 }
 
-void KoToolManager::toolActivated(ToolHelper *tool) {
+void KoToolManager::toolActivated(ToolHelper *tool)
+{
     Q_ASSERT(d->canvasData);
     KoTool *t = d->canvasData->allTools.value(tool->id());
     Q_ASSERT(t);
@@ -257,18 +278,19 @@ void KoToolManager::toolActivated(ToolHelper *tool) {
     switchTool(t, false);
 }
 
-void KoToolManager::switchTool(const QString &id, bool temporary) {
+void KoToolManager::switchTool(const QString &id, bool temporary)
+{
     if (d->canvasData->activeTool && temporary)
         d->canvasData->stack.push(d->canvasData->activeToolId);
     d->canvasData->activeToolId = id;
     KoTool *tool = d->canvasData->allTools.value(id);
     if (! tool) {
-        kWarning(30006) << "KoToolManager::switchTool() " << (temporary?"temporary":"") << " got request to unknown tool: '" << id << "'\n";
+        kWarning(30006) << "KoToolManager::switchTool() " << (temporary ? "temporary" : "") << " got request to unknown tool: '" << id << "'\n";
         return;
     }
 
     foreach(ToolHelper *th, d->tools) {
-        if(th->id() == id) {
+        if (th->id() == id) {
             d->canvasData->activationShapeId = th->activationShapeId();
             break;
         }
@@ -277,10 +299,13 @@ void KoToolManager::switchTool(const QString &id, bool temporary) {
     switchTool(tool, temporary);
 }
 
-void KoToolManager::switchTool(KoTool *tool, bool temporary) {
+void KoToolManager::switchTool(KoTool *tool, bool temporary)
+{
     Q_ASSERT(tool);
     if (d->canvasData == 0)
         return;
+    if (d->canvasData->activeTool == tool && tool->toolId() != KoInteractionTool_ID)
+        return;
 
     bool newActiveTool = d->canvasData->activeTool != 0;
 
@@ -289,27 +314,29 @@ void KoToolManager::switchTool(KoTool *tool, bool temporary) {
         // check if this tool is inputDeviceAgnostic and used by other devices, in which case we should not deactivate.
         QList<CanvasData*> items = d->canvasses[d->canvasData->canvas];
         foreach(CanvasData *cd, items) {
-            if(cd == d->canvasData) continue;
-            if(cd->activeTool == d->canvasData->activeTool) {
+            if (cd == d->canvasData) continue;
+            if (cd->activeTool == d->canvasData->activeTool) {
                 newActiveTool = false;
                 break;
             }
         }
     }
 
-    if(newActiveTool) {
-        foreach(QAction *action, d->canvasData->activeTool->actions().values())
+    if (newActiveTool) {
+        foreach(KAction *action, d->canvasData->activeTool->actions())
             action->setEnabled(false);
         // repaint the decorations before we deactivate the tool as it might deleted
         // data needed for the repaint
         d->canvasData->activeTool->deactivate();
         disconnect(d->canvasData->activeTool, SIGNAL(cursorChanged(QCursor)),
-                this, SLOT(updateCursor(QCursor)));
+                   this, SLOT(updateCursor(QCursor)));
         disconnect(d->canvasData->activeTool, SIGNAL(activateTool(const QString &)),
-                this, SLOT(switchToolRequested(const QString &)));
+                   this, SLOT(switchToolRequested(const QString &)));
         disconnect(d->canvasData->activeTool, SIGNAL(activateTemporary(const QString &)),
-                this, SLOT(switchToolTemporaryRequested(const QString &)));
+                   this, SLOT(switchToolTemporaryRequested(const QString &)));
         disconnect(d->canvasData->activeTool, SIGNAL(done()), this, SLOT(switchBackRequested()));
+        disconnect(d->canvasData->activeTool, SIGNAL(statusTextChanged(const QString &)),
+                   this, SIGNAL(changedStatusText(const QString &)));
     }
 
     d->canvasData->activeTool = tool;
@@ -320,10 +347,15 @@ void KoToolManager::switchTool(KoTool *tool, bool temporary) {
     connect(d->canvasData->activeTool, SIGNAL(activateTemporary(const QString &)),
             this, SLOT(switchToolTemporaryRequested(const QString &)));
     connect(d->canvasData->activeTool, SIGNAL(done()), this, SLOT(switchBackRequested()));
+    connect(d->canvasData->activeTool, SIGNAL(statusTextChanged(const QString &)),
+            this, SIGNAL(changedStatusText(const QString &)));
+
+    // emit a empty status text to clear status text from last active tool
+    emit changedStatusText(QString());
 
     // we expect the tool to emit a cursor on activation.  This is for quick-fail :)
     d->canvasData->canvas->canvas()->canvasWidget()->setCursor(Qt::ForbiddenCursor);
-    foreach(QAction *action, d->canvasData->activeTool->actions().values()) {
+    foreach(KAction *action, d->canvasData->activeTool->actions()) {
         action->setEnabled(true);
         d->canvasData->canvas->addAction(action);
     }
@@ -332,39 +364,42 @@ void KoToolManager::switchTool(KoTool *tool, bool temporary) {
     postSwitchTool();
 }
 
-void KoToolManager::postSwitchTool() {
+void KoToolManager::postSwitchTool()
+{
 #ifndef NDEBUG
     int canvasCount = 1;
     foreach(QList<CanvasData*> list, d->canvasses) {
         bool first = true;
         foreach(CanvasData *data, list) {
-            if(first)
-                kDebug(30006) <<"Canvas" << canvasCount++;
-            kDebug(30006) <<"  +- Tool:" << data->activeToolId  << (data == d->canvasData?" *":"");
+            if (first)
+                kDebug(30006) << "Canvas" << canvasCount++;
+            kDebug(30006) << "  +- Tool:" << data->activeToolId  << (data == d->canvasData ? " *" : "");
             first = false;
         }
     }
 #endif
-    if(d->canvasData->canvas->canvas()) {
+    if (d->canvasData->canvas->canvas()) {
         KoCanvasBase *canvas = d->canvasData->canvas->canvas();
         KoToolProxy *tp = d->proxies.value(canvas);
-        if(tp)
+        if (tp)
             tp->setActiveTool(d->canvasData->activeTool);
         canvas->updateInputMethodInfo();
     }
 
-    QWidget *toolWidget = d->canvasData->activeTool->optionWidget();
-    if(toolWidget == 0) { // no option widget.
+    QMap<QString, QWidget *> optionWidgetMap = d->canvasData->activeTool->optionWidgets();
+    if (optionWidgetMap.empty()) { // no option widget.
+        QWidget *toolWidget;
         QString name;
-        foreach( ToolHelper * tool, d->tools ) {
-            if ( tool->id() == d->canvasData->activeTool->toolId() ) {
+        foreach(ToolHelper * tool, d->tools) {
+            if (tool->id() == d->canvasData->activeTool->toolId()) {
                 name = tool->name();
                 break;
             }
         }
         toolWidget = d->canvasData->dummyToolWidget;
-        if(toolWidget == 0) {
+        if (toolWidget == 0) {
             toolWidget = new QWidget();
+            toolWidget->setObjectName( "DummyToolWidget" );
             QVBoxLayout *layout = new QVBoxLayout(toolWidget);
             layout->setMargin(3);
             d->canvasData->dummyToolLabel = new QLabel(toolWidget);
@@ -374,13 +409,15 @@ void KoToolManager::postSwitchTool() {
             d->canvasData->dummyToolWidget = toolWidget;
         }
         d->canvasData->dummyToolLabel->setText(i18n("Active tool: %1", name));
+        optionWidgetMap.insert(i18n("Tool Options"), toolWidget);
     }
-    d->canvasData->canvas->setToolOptionWidget(toolWidget);
+    d->canvasData->canvas->setToolOptionWidgets(optionWidgetMap);
     emit changedTool(d->canvasData->canvas, d->uniqueToolIds.value(d->canvasData->activeTool));
 }
 
 
-void KoToolManager::attachCanvas(KoCanvasController *controller) {
+void KoToolManager::attachCanvas(KoCanvasController *controller)
+{
     CanvasData *cd = d->createCanvasData(controller, KoInputDevice::mouse());
     // switch to new canvas as the active one.
     d->canvasData = cd;
@@ -390,22 +427,40 @@ void KoToolManager::attachCanvas(KoCanvasController *controller) {
     d->canvasses[controller] = canvasses;
 
     KoToolProxy *tp = d->proxies[controller->canvas()];
-    if(tp)
+    if (tp)
         tp->setCanvasController(controller);
 
-    if (cd->activeTool == 0)
-        toolActivated(d->defaultTool);
+    if (cd->activeTool == 0) {
+        // no active tool, so we activate the highest priority main tool
+        int highestPriority = INT_MAX;
+        ToolHelper * helper = 0;
+        foreach(ToolHelper * th, d->tools) {
+            if (th->toolType() == KoToolFactory::mainToolType()) {
+                if (th->priority() < highestPriority) {
+                    highestPriority = qMin(highestPriority, th->priority());
+                    helper = th;
+                }
+            }
+        }
+        if (helper)
+            toolActivated(helper);
+    }
 
     Connector *connector = new Connector(controller->canvas()->shapeManager());
     connect(connector, SIGNAL(selectionChanged(QList<KoShape*>)), this,
             SLOT(selectionChanged(QList<KoShape*>)));
+    connect(controller->canvas()->shapeManager()->selection(),
+            SIGNAL(currentLayerChanged(const KoShapeLayer*)),
+            this, SLOT(currentLayerChanged(const KoShapeLayer*)));
 
     d->canvasData->canvas->activate();
+    emit changedCanvas(d->canvasData ? d->canvasData->canvas->canvas() : 0);
 }
 
-void KoToolManager::movedFocus(QWidget *from, QWidget *to) {
+void KoToolManager::movedFocus(QWidget *from, QWidget *to)
+{
     Q_UNUSED(from);
-    if (to == 0 || d->canvasData && to == d->canvasData->canvas)
+    if (to == 0 || (d->canvasData && to == d->canvasData->canvas))
         return;
 
     KoCanvasController *newCanvas = 0;
@@ -422,14 +477,15 @@ void KoToolManager::movedFocus(QWidget *from, QWidget *to) {
     if (d->canvasData && newCanvas == d->canvasData->canvas)
         return;
 
-    if(! d->canvasses.contains(newCanvas))
+    if (! d->canvasses.contains(newCanvas))
         return;
     foreach(CanvasData *data, d->canvasses.value(newCanvas)) {
-        if(data->inputDevice == d->inputDevice) {
+        if (data->inputDevice == d->inputDevice) {
             d->canvasData = data;
-            postSwitchTool();
             d->canvasData->canvas->canvas()->canvasWidget()->setCursor(d->canvasData->activeTool->cursor());
             d->canvasData->canvas->activate();
+            postSwitchTool();
+            emit changedCanvas(d->canvasData ? d->canvasData->canvas->canvas() : 0);
             return;
         }
     }
@@ -437,18 +493,38 @@ void KoToolManager::movedFocus(QWidget *from, QWidget *to) {
     d->canvasData = d->canvasses.value(newCanvas).first();
     d->inputDevice = d->canvasData->inputDevice;
     d->canvasData->canvas->activate();
+    emit changedCanvas(d->canvasData ? d->canvasData->canvas->canvas() : 0);
 }
 
-void KoToolManager::detachCanvas(KoCanvasController *controller) {
+void KoToolManager::detachCanvas(KoCanvasController *controller)
+{
+    // check if we are removing the active canvas controller
     if (d->canvasData && d->canvasData->canvas == controller) {
-        d->canvasData = 0; // replace with a blank one
+        KoCanvasController *newCanvas = 0;
+        // try to find another canvas controller beside the one we are removing
+        foreach(KoCanvasController* canvas, d->canvasses.keys()) {
+            if (canvas != controller) {
+                // yay found one
+                newCanvas = canvas;
+                break;
+            }
+        }
+        if (newCanvas) {
+            // activate the found canvas controller
+            d->canvasData = d->canvasses.value(newCanvas).first();
+            d->inputDevice = d->canvasData->inputDevice;
+            d->canvasData->canvas->activate();
+        } else {
+            // as a last resort just set a blank one
+            d->canvasData = 0;
+        }
     }
 
     QList<KoTool *> tools;
     foreach(CanvasData *cd, d->canvasses.value(controller)) {
-        foreach(KoTool *tool, cd->allTools.values())
-        if(! tools.contains(tool))
-            tools.append(tool);
+        foreach(KoTool *tool, cd->allTools)
+            if (! tools.contains(tool))
+                tools.append(tool);
         delete cd;
     }
     foreach(KoTool *tool, tools) {
@@ -456,26 +532,31 @@ void KoToolManager::detachCanvas(KoCanvasController *controller) {
         delete tool;
     }
     d->canvasses.remove(controller);
+    emit changedCanvas(d->canvasData ? d->canvasData->canvas->canvas() : 0);
 }
 
-void KoToolManager::updateCursor(QCursor cursor) {
+void KoToolManager::updateCursor(QCursor cursor)
+{
     Q_ASSERT(d->canvasData);
     Q_ASSERT(d->canvasData->canvas);
     Q_ASSERT(d->canvasData->canvas->canvas());
     d->canvasData->canvas->canvas()->canvasWidget()->setCursor(cursor);
 }
 
-void KoToolManager::switchToolRequested(const QString & id) {
+void KoToolManager::switchToolRequested(const QString & id)
+{
     while (!d->canvasData->stack.isEmpty()) // switching means to flush the stack
         d->canvasData->stack.pop();
     switchTool(id, false);
 }
 
-void KoToolManager::switchToolTemporaryRequested(const QString &id) {
+void KoToolManager::switchToolTemporaryRequested(const QString &id)
+{
     switchTool(id, true);
 }
 
-void KoToolManager::switchBackRequested() {
+void KoToolManager::switchBackRequested()
+{
     if (d->canvasData->stack.isEmpty()) {
         // default to changing to the interactionTool
         switchTool(KoInteractionTool_ID, false);
@@ -489,16 +570,31 @@ KoCreateShapesTool * KoToolManager::shapeCreatorTool(KoCanvasBase *canvas) const
     foreach(KoCanvasController *controller, d->canvasses.keys()) {
         if (controller->canvas() == canvas) {
             KoCreateShapesTool *createTool = dynamic_cast<KoCreateShapesTool*>
-                (d->canvasData->allTools.value(KoCreateShapesTool_ID));
+                                             (d->canvasData->allTools.value(KoCreateShapesTool_ID));
             Q_ASSERT(createTool /* ID changed? */);
             return createTool;
         }
     }
-    Q_ASSERT( 0 ); // this should not happen
+    Q_ASSERT(0);   // this should not happen
     return 0;
 }
 
-void KoToolManager::selectionChanged(QList<KoShape*> shapes) {
+KoGuidesTool * KoToolManager::guidesTool(KoCanvasBase * canvas) const
+{
+    foreach(KoCanvasController *controller, d->canvasses.keys()) {
+        if (controller->canvas() == canvas) {
+            KoGuidesTool * guidesTool = dynamic_cast<KoGuidesTool*>
+                                        (d->canvasData->allTools.value(KoGuidesTool_ID));
+            Q_ASSERT(guidesTool /* ID changed? */);
+            return guidesTool;
+        }
+    }
+    Q_ASSERT(0);   // this should not happen
+    return 0;
+}
+
+void KoToolManager::selectionChanged(QList<KoShape*> shapes)
+{
     QList<QString> types;
     foreach(KoShape *shape, shapes) {
         if (! types.contains(shape->shapeId())) {
@@ -510,27 +606,46 @@ void KoToolManager::selectionChanged(QList<KoShape*> shapes) {
     // there needs to be at least one shape for a tool without an activationShapeId
     // to work
     // if not change the current tool to the default tool
-    if ( ! ( d->canvasData->activationShapeId.isNull() && shapes.size() > 0 ) && d->canvasData->activationShapeId != "flake/always" &&
-            ! types.contains( d->canvasData->activationShapeId ) ) {
+    if (!(d->canvasData->activationShapeId.isNull() && shapes.size() > 0) 
+    && d->canvasData->activationShapeId != "flake/always" 
+    && d->canvasData->activationShapeId != "flake/edit" 
+    && ! types.contains(d->canvasData->activationShapeId)) {
         switchTool(KoInteractionTool_ID, false);
     }
 
     emit toolCodesSelected(d->canvasData->canvas, types);
 }
 
-KoCanvasController *KoToolManager::activeCanvasController() const {
-    if( ! d->canvasData )
+void KoToolManager::currentLayerChanged(const KoShapeLayer* layer)
+{
+    emit currentLayerChanged(d->canvasData->canvas, layer);
+    bool enabled = layer == 0 || (layer->isEditable() && layer->isVisible());
+    KoToolProxy *proxy = d->proxies.value(d->canvasData->canvas->canvas());
+    if (proxy)
+        proxy->setActiveTool(enabled ? d->canvasData->activeTool : 0);
+}
+
+KoCanvasController *KoToolManager::activeCanvasController() const
+{
+    if (! d->canvasData)
         return 0;
     return d->canvasData->canvas;
 }
 
-KoToolProxy *KoToolManager::createToolProxy(KoCanvasBase *parentCanvas) {
-    KoToolProxy *tp = new KoToolProxy(parentCanvas);
-    d->proxies.insert(parentCanvas, tp);
-    return tp;
+void KoToolManager::switchToolByShortcut(QKeyEvent *event)
+{
+    QKeySequence item(event->key() | ((Qt::ControlModifier | Qt::AltModifier) & event->modifiers()));
+
+    foreach(ToolHelper *th, d->tools) {
+        if (th->shortcut().contains(item)) {
+            event->accept();
+            switchTool(th->id(), false);
+        }
+    }
 }
 
-QString KoToolManager::preferredToolForSelection(const QList<KoShape*> &shapes) {
+QString KoToolManager::preferredToolForSelection(const QList<KoShape*> &shapes)
+{
     QList<QString> types;
     foreach(KoShape *shape, shapes)
         if (! types.contains(shape->shapeId()))
@@ -539,11 +654,11 @@ QString KoToolManager::preferredToolForSelection(const QList<KoShape*> &shapes)
     QString toolType = KoInteractionTool_ID;
     int prio = INT_MAX;
     foreach(ToolHelper *helper, d->tools) {
-        if(helper->priority() >= prio)
+        if (helper->priority() >= prio)
             continue;
-        if(helper->toolType() == KoToolFactory::mainToolType())
+        if (helper->toolType() == KoToolFactory::mainToolType())
             continue;
-        if(types.contains(helper->activationShapeId())) {
+        if (types.contains(helper->activationShapeId())) {
             toolType = helper->id();
             prio = helper->priority();
         }
@@ -553,11 +668,12 @@ QString KoToolManager::preferredToolForSelection(const QList<KoShape*> &shapes)
 
 #define MSECS_TO_IGNORE_SWITCH_TO_MOUSE_AFTER_TABLET_EVENT_RECEIVED 100
 
-void KoToolManager::switchInputDevice(const KoInputDevice &device) {
+void KoToolManager::switchInputDevice(const KoInputDevice &device)
+{
     if (!device.isMouse()) {
         d->tabletEventTimer.start(MSECS_TO_IGNORE_SWITCH_TO_MOUSE_AFTER_TABLET_EVENT_RECEIVED);
     }
-    if(d->inputDevice == device) return;
+    if (d->inputDevice == device) return;
     if (device.isMouse() && d->tabletEventTimer.isActive()) {
         // Ignore switch to mouse for a short time after a tablet event
         // is received, as this is likely to be either the mouse event sent
@@ -568,15 +684,17 @@ void KoToolManager::switchInputDevice(const KoInputDevice &device) {
     d->inputDevice = device;
     QList<CanvasData*> items = d->canvasses[d->canvasData->canvas];
     foreach(CanvasData *cd, items) {
-        if(cd->inputDevice == device) {
+        if (cd->inputDevice == device) {
             d->canvasData = cd;
-            if(cd->activeTool == 0)
+            if (cd->activeTool == 0)
                 switchTool(KoInteractionTool_ID, false);
             else {
                 postSwitchTool();
                 d->canvasData->canvas->canvas()->canvasWidget()->setCursor(d->canvasData->activeTool->cursor());
             }
             d->canvasData->canvas->activate();
+            emit inputDeviceChanged(device);
+            emit changedCanvas(d->canvasData ? d->canvasData->canvas->canvas() : 0);
             return;
         }
     }
@@ -593,6 +711,7 @@ void KoToolManager::switchInputDevice(const KoInputDevice &device) {
     switchToolRequested(oldTool);
     emit inputDeviceChanged(device);
     d->canvasData->canvas->activate();
+    emit changedCanvas(d->canvasData ? d->canvasData->canvas->canvas() : 0);
 }
 
 bool KoToolManager::eventFilter(QObject *object, QEvent *event)
@@ -602,8 +721,7 @@ bool KoToolManager::eventFilter(QObject *object, QEvent *event)
     case QEvent::TabletPress:
     case QEvent::TabletRelease:
     case QEvent::TabletEnterProximity:
-    case QEvent::TabletLeaveProximity:
-    {
+    case QEvent::TabletLeaveProximity: {
         QTabletEvent *tabletEvent = static_cast<QTabletEvent *>(event);
 
         KoInputDevice id(tabletEvent->device(), tabletEvent->pointerType(), tabletEvent->uniqueId());
@@ -622,24 +740,32 @@ bool KoToolManager::eventFilter(QObject *object, QEvent *event)
     return QObject::eventFilter(object, event);
 }
 
-void KoToolManager::registerToolProxy(KoToolProxy *proxy, KoCanvasBase *canvas) {
+void KoToolManager::registerToolProxy(KoToolProxy *proxy, KoCanvasBase *canvas)
+{
     d->proxies.insert(canvas, proxy);
     foreach(KoCanvasController *controller, d->canvasses.keys()) {
-        if(controller->canvas() == canvas) {
+        if (controller->canvas() == canvas) {
             proxy->setCanvasController(controller);
             break;
         }
     }
 }
 
+void KoToolManager::injectDeviceEvent(KoDeviceEvent * event)
+{
+    if (d->canvasData->canvas->canvas()) {
+        if (static_cast<KoDeviceEvent::Type>(event->type()) == KoDeviceEvent::ButtonPressed)
+            d->canvasData->activeTool->customPressEvent(event->pointerEvent());
+        else if (static_cast<KoDeviceEvent::Type>(event->type()) == KoDeviceEvent::ButtonReleased)
+            d->canvasData->activeTool->customReleaseEvent(event->pointerEvent());
+        else if (static_cast<KoDeviceEvent::Type>(event->type()) ==  KoDeviceEvent::PositionChanged)
+            d->canvasData->activeTool->customMoveEvent(event->pointerEvent());
+    }
+}
 
-//static
-KoToolManager* KoToolManager::s_instance = 0;
-static K3StaticDeleter<KoToolManager> staticToolManagerDeleter;
-
-KoToolManager* KoToolManager::instance() {
-    if (s_instance == 0)
-        staticToolManagerDeleter.setObject(s_instance, new KoToolManager());
+KoToolManager* KoToolManager::instance()
+{
+    K_GLOBAL_STATIC(KoToolManager, s_instance)
     return s_instance;
 }
 
