@@ -4,12 +4,19 @@
 #include <kdebug.h>
 #include <KoShapeManager.h>
 #include <KoSelection.h>
+#include <KoLineBorder.h>
+#include <KoShapeShadow.h>
 
-void TestShapeAt::test() {
+#include <kcomponentdata.h>
+
+void TestShapeAt::test()
+{
     MockShape shape1;
     MockShape shape2;
     MockShape shape3;
 
+    KComponentData componentData("TestShapeAt");    // we need an instance for that canvas
+
     MockCanvas canvas;
     KoShapeManager manager(&canvas);
     shape1.setPosition(QPointF(100, 100));
@@ -21,8 +28,8 @@ void TestShapeAt::test() {
     QVERIFY(manager.shapeAt(QPointF(110, 140)) != 0);
     QVERIFY(manager.shapeAt(QPointF(100, 100)) != 0);
     QVERIFY(manager.shapeAt(QPointF(100, 100), KoFlake::Selected) == 0);
-    QVERIFY(manager.shapeAt(QPointF(100, 100), KoFlake::Unselected) != 0 );
-    QVERIFY(manager.shapeAt(QPointF(100, 100), KoFlake::NextUnselected) != 0 );
+    QVERIFY(manager.shapeAt(QPointF(100, 100), KoFlake::Unselected) != 0);
+    QVERIFY(manager.shapeAt(QPointF(100, 100), KoFlake::NextUnselected) != 0);
 
     shape2.setPosition(QPointF(80, 80));
     shape2.setSize(QSizeF(50, 50));
@@ -79,5 +86,23 @@ void TestShapeAt::test() {
     QCOMPARE(manager.shapeAt(QPointF(125, 105), KoFlake::NextUnselected, true), &shape1);
 }
 
+void TestShapeAt::testShadow()
+{
+    MockShape shape;
+    shape.setPosition(QPointF(20, 30));
+    shape.setSize(QSizeF(50, 70));
+    QCOMPARE(shape.boundingRect(), QRectF(20, 30, 50, 70));
+
+    KoLineBorder *border = new KoLineBorder();
+    border->setLineWidth(20); // which means the shape grows 10 in all directions.
+    shape.setBorder(border);
+    QCOMPARE(shape.boundingRect(), QRectF(10, 20, 70, 90));
+
+    KoShapeShadow *shadow = new KoShapeShadow();
+    shadow->setOffset(QPointF(5, 9));
+    shape.setShadow(shadow);
+    QCOMPARE(shape.boundingRect(), QRectF(10, 20, 75, 99));
+}
+
 QTEST_MAIN(TestShapeAt)
 #include "TestShapeAt.moc"
