@@ -1,7 +1,8 @@
 #include "TestShapeContainer.h"
 #include "MockShapes.h"
 
-void TestShapeContainer::testModel() {
+void TestShapeContainer::testModel()
+{
     MockContainerModel *model = new MockContainerModel();
     MockContainer container(model);
     MockShape *shape1 = new MockShape();
@@ -21,7 +22,7 @@ void TestShapeContainer::testModel() {
     QCOMPARE(model->childChangedCalled(), 3);
     QCOMPARE(model->proposeMoveCalled(), 0);
 
-    shape1->setAbsolutePosition( shape1->absolutePosition() + QPointF(10.,40.) );
+    shape1->setAbsolutePosition(shape1->absolutePosition() + QPointF(10., 40.));
     QCOMPARE(model->containerChangedCalled(), 0);
     QCOMPARE(model->childChangedCalled(), 5); // we get a generic Matrix as well as a position change...
     QCOMPARE(model->proposeMoveCalled(), 0);
@@ -38,5 +39,29 @@ void TestShapeContainer::testModel() {
     QCOMPARE(model->proposeMoveCalled(), 0);
 }
 
+void TestShapeContainer::testSetParent()
+{
+    MockContainerModel *model1 = new MockContainerModel();
+    MockContainer container1(model1);
+    MockContainerModel *model2 = new MockContainerModel();
+    MockContainer container2(model2);
+    MockShape shape;
+    // init test
+    QCOMPARE(model1->iterator().count(), 0);
+    QCOMPARE(model2->iterator().count(), 0);
+
+    shape.setParent(&container1);
+    QCOMPARE(model1->iterator().count(), 1);
+    QCOMPARE(model2->iterator().count(), 0);
+    QCOMPARE(shape.parent(), &container1);
+
+    shape.setParent(&container2);
+    QCOMPARE(model1->iterator().count(), 0);
+    QCOMPARE(container1.iterator().count(), 0);
+    QCOMPARE(model2->iterator().count(), 1);
+    QCOMPARE(container2.iterator().count(), 1);
+    QCOMPARE(shape.parent(), &container2);
+}
+
 QTEST_MAIN(TestShapeContainer)
 #include "TestShapeContainer.moc"
