@@ -24,6 +24,7 @@
 
 #include <QWidgetAction>
 #include <QMenu>
+#include <QHBoxLayout>
 
 #include <KColorDialog>
 #include <klocale.h>
@@ -40,19 +41,21 @@ public:
 };
 
 KoColorSetAction::KoColorSetAction(QObject *parent)
-    : QAction(parent),
+    : KAction(parent),
     d(new KoColorSetActionPrivate())
 {
     d->menu = new QMenu();
     QWidgetAction *wdgAction = new QWidgetAction(d->menu);
     d->colorSetWidget = new KoColorSetWidget(d->menu);
     connect(d->colorSetWidget, SIGNAL(colorChanged(const KoColor &, bool)), this, SLOT(handleColorChange(const KoColor &, bool)));
-    connect(d->colorSetWidget, SIGNAL(widgetSizeChanged(const QSize &)), this, SLOT(resizeMenu(const QSize &)));
+//    connect(d->colorSetWidget, SIGNAL(widgetSizeChanged(const QSize &)), this, SLOT(resizeMenu(const QSize &)));
     wdgAction->setDefaultWidget(d->colorSetWidget);
     d->menu->addAction(wdgAction);
     setMenu(d->menu);
-    setIcon(KIcon("textcolor"));
-    setToolTip(i18n("Text Color..."));
+    new QHBoxLayout(d->menu);
+    d->menu->layout()->addWidget(d->colorSetWidget);
+    d->menu->layout()->setMargin(4);
+
     connect(this, SIGNAL(triggered()), this, SLOT(showCustomColorDialog()));
 }
 
@@ -70,11 +73,6 @@ void KoColorSetAction::handleColorChange(const KoColor &color, bool final)
     }
 }
 
-void KoColorSetAction::resizeMenu(const QSize &size)
-{
-    d->menu->resize(size);
-}
-
 void KoColorSetAction::showCustomColorDialog()
 {
     QColor color;
