@@ -24,7 +24,7 @@
 #include "KoToolDocker.h"
 #include <KoDockFactory.h>
 
-#include <koguiutils_export.h>
+#include "koguiutils_export.h"
 
 /**
  * The factory class for the KoToolDocker.
@@ -32,13 +32,23 @@
 class KOGUIUTILS_EXPORT KoToolDockerFactory : public KoDockFactory
 {
 public:
-    KoToolDockerFactory() {}
-    ~KoToolDockerFactory() {}
+    KoToolDockerFactory( QString dockWidgetId = 0 );
+    ~KoToolDockerFactory();
 
     /// reimplemented from parent
     virtual QString id() const;
+
     /// reimplemented from parent
     QDockWidget* createDockWidget();
+    
+    
+    DockPosition defaultDockPosition() const
+    {
+        return DockRight;
+    }
+private:
+    class Private;
+    Private * const d;
 };
 
 #endif
