@@ -26,6 +26,8 @@
 #include <KoStore.h>
 #include <KoOdfWriteStore.h>
 #include <KoDocumentAdaptor.h>
+#include <KoDocument.h>
+#include <KoEmbeddedDocumentSaver.h>
 
 /************************************************************************************************
  * KoScriptingOdfReader
@@ -134,7 +136,7 @@ void dumpElem(KoXmlElement elem, int level=0) {
     QString prefix="";
     for(int i = 0; i < level; ++i) prefix+="  ";
     qDebug()<<QString("%1  %2").arg(prefix).arg(elem.tagName());
-    foreach(QString s,elem.attributeNames()) qDebug()<<QString("%1    %2 = %3").arg(prefix).arg(s).arg(elem.attribute(s));
+    foreach(const QString &s,elem.attributeNames()) qDebug()<<QString("%1    %2 = %3").arg(prefix).arg(s).arg(elem.attribute(s));
     level++;
     KoXmlElement e;
     forEachElement(e, elem) dumpElem(e,level);
@@ -210,15 +212,17 @@ class KoScriptingOdfStore::Private
             //qDebug()<<"KoScriptingOdfStore::getByteArray() Reading ByteArray.";
             QBuffer buffer(&byteArray);
             KoStore* store = KoStore::createStore(&buffer, KoStore::Write, "KrossScript", KoStore::Tar);
-            KoOdfWriteStore oasisStore(store);
+            KoOdfWriteStore odfStore(store);
+            odfStore.manifestWriter("");
+            KoEmbeddedDocumentSaver embeddedSaver;
+            KoDocument::SavingContext documentContext( odfStore, embeddedSaver );
             QByteArray mime = getMimeType();
-            KoXmlWriter* manifestWriter = mime.isNull() ? 0 : oasisStore.manifestWriter(mime);
-            if( ! document->saveOasis(store, manifestWriter) ) {
+            if( ! document->saveOdf( documentContext ) ) {
                 qWarning()<<"KoScriptingOdfStore::open() Failed to save Oasis to ByteArray";
                 byteArray = QByteArray();
             }
             //oasisStore.closeContentWriter();
-            oasisStore.closeManifestWriter();
+            odfStore.closeManifestWriter();
             delete store;
             return byteArray;
         }
