@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-   Copyright (C) 2006-2007 Thorsten Zachmann <zachmann@kde.org>
+   Copyright (C) 2006-2008 Thorsten Zachmann <zachmann@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -34,6 +34,7 @@ class KoPAPageBase;
 class KoPAMasterPage;
 class KoPALoadingContext;
 class KoPASavingContext;
+class KoXmlWriter;
 
 class KoInlineTextObjectManager;
 
@@ -48,10 +49,15 @@ public:
 
     void paintContent( QPainter &painter, const QRect &rect);
 
-    bool loadXML( QIODevice *, const KoXmlDocument & doc );
+    bool loadXML( const KoXmlDocument & doc, KoStore *store );
     bool loadOdf( KoOdfReadStore & odfStore );
 
-    bool saveOasis( KoStore* store, KoXmlWriter* manifestWriter );
+    bool saveOdf( SavingContext & documentContext );
+
+    /**
+     * The tag the body is saved in
+     */
+    virtual const char *odfTagName( bool withNamespace ) = 0;
 
     /**
      * Load master pages
@@ -69,14 +75,31 @@ public:
      */
     QList<KoPAPageBase *> loadOdfPages( const KoXmlElement & body, KoPALoadingContext & context );
 
+    /// reimplemented
+    virtual bool completeLoading( KoStore* store );
+
+    /// reimplemented
+    virtual bool completeSaving( KoStore* store );
+
+
     /**
      * Save pages
      *
-     * This is used by saveOasis and for copy and paste of pages.
+     * This is used by saveOdf and for copy and paste of pages.
      *
      * For all pages that are specified also the master slide has to be specified.
      */
-    bool saveOasisPages( KoPASavingContext & paContext, QList<KoPAPageBase *> &pages, QList<KoPAPageBase *> &masterPages );
+    bool saveOdfPages( KoPASavingContext & paContext, QList<KoPAPageBase *> &pages, QList<KoPAPageBase *> &masterPages );
+
+    /**
+     * Save document styles
+     */
+    virtual void saveOdfDocumentStyles( KoPASavingContext & context );
+
+    /**
+     * Load document styles
+     */
+    virtual bool loadOdfDocumentStyles( KoPALoadingContext & context );
 
     /**
      * Get page by index.
@@ -86,6 +109,18 @@ public:
      */
     KoPAPageBase* pageByIndex( int index, bool masterPage ) const;
 
+    /// reimplemnted
+    virtual int pageCount() const;
+
+    /**
+     * Get the index of the page
+     *
+     * @param page The page you want to get the index for
+     *
+     * @return The index of the page or -1 if the page is not found
+     */
+    int pageIndex( KoPAPageBase * page ) const;
+
     /**
      * Get page by navigation
      *
@@ -126,6 +161,15 @@ public:
      */
     int takePage( KoPAPageBase *page );
 
+    /**
+     * Remove the page from the document
+     *
+     * This generates the command and adds the command that deletes the page
+     *
+     * @param page The page that gets removed
+     */
+    virtual void removePage( KoPAPageBase * page );
+
     void addShape( KoShape *shape );
     void removeShape( KoShape* shape );
 
@@ -158,9 +202,57 @@ public:
     /// return the inlineTextObjectManager for this document.
     KoInlineTextObjectManager *inlineTextObjectManager() const;
 
+    void setRulersVisible(bool visible);
+    bool rulersVisible() const;
+
+    virtual QMap<QString, KoDataCenter *>  dataCenterMap() const;
+
+    /**
+     * Get the page on which the shape is located
+     *
+     * @param shape The shape for which the page should be found
+     * @return The page on which the shape is located
+     */
+    KoPAPageBase * pageByShape( KoShape * shape ) const;
+
+    /**
+     * Update all views this document is displayed on
+     *
+     * @param page specify a page to be updated, all views with this page as active page will be updated.
+     */
+    void updateViews(KoPAPageBase *page);
+
+    /**
+     * Get the page type used in the document
+     *
+     * The default page type KoPageApp::Page is returned
+     */
+    virtual KoPageApp::PageType pageType() const;
+
+signals:
+    void shapeAdded(KoShape* shape);
+    void shapeRemoved(KoShape* shape);
+    void pageAdded(KoPAPageBase* page);
+    void pageRemoved(KoPAPageBase* page);
+
 protected:
+    void insertIntoDataCenterMap(QString key, KoDataCenter *dc);
+
     virtual KoView *createViewInstance( QWidget *parent ) = 0;
-    virtual const char *odfTagName( bool withNamespace ) = 0;
+
+    /**
+     * Load the epilouge
+     *
+     * The default implementation is empty
+     */
+    virtual bool loadOdfEpilogue( const KoXmlElement & body, KoPALoadingContext & context );
+
+    /**
+     * Save the epilouge
+     *
+     * The default implementation is empty
+     */
+    virtual bool saveOdfEpilogue( KoPASavingContext & paContext );
 
     /**
      * This function is called by at the end of addShape. This is used 
@@ -179,12 +271,14 @@ protected:
     virtual void postRemoveShape( KoPAPageBase * page, KoShape * shape );
 
     /**
-     * Get the page on which the shape is located
+     * This function is called with the command that will remove the page
+     * given.
+     * The default implementation is empty.
      *
-     * @param shape The shape for which the page should be found
-     * @return The page on which the shape is located
+     * @param page The page that will be removed
+     * @param parent The command that will be used to delete the page
      */
-    KoPAPageBase * pageByShape( KoShape * shape ) const;
+    virtual void pageRemoved( KoPAPageBase * page, QUndoCommand * parent );
 
     /**
      * @brief Enables/Disables the given actions in all views
@@ -196,6 +290,11 @@ protected:
      */
     void setActionEnabled( int actions, bool enable );
 
+    /// Load the configuration
+    void loadConfig();
+    /// Save the configuration
+    void saveConfig();
+
 private:
     class Private;
     Private * const d;
