@@ -1,5 +1,6 @@
 /* This file is part of the KDE project
  * Copyright (C) 2007 Jan Hambrecht <jaham@gmx.net>
+ * Copyright (C) 2008 Fredy Yanardi <fyanardi@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -30,11 +31,12 @@
 #include <KoCanvasController.h>
 #include <KoShapeControllerBase.h>
 #include <KoSelection.h>
-#include <KoZoomHandler.h>
 #include <KoShapeLayer.h>
 #include <KoShapeGroup.h>
 #include <KoShapeGroupCommand.h>
 #include <KoShapeUngroupCommand.h>
+#include <KoShapeRenameCommand.h>
+#include <KoZoomHandler.h>
 
 #include <klocale.h>
 #include <kicon.h>
@@ -44,17 +46,18 @@
 #include <QtCore/QAbstractItemModel>
 #include <QtCore/QMimeData>
 
-
-
-KoPADocumentModel::KoPADocumentModel( KoPADocument *document )
-: m_document( document )
+KoPADocumentModel::KoPADocumentModel( QObject* parent, KoPADocument *document )
+: KoDocumentSectionModel( parent )
+, m_master(false)
 , m_lastContainer( 0 )
 {
+    setDocument( document );
     setSupportedDragActions( Qt::MoveAction );
 }
 
 void KoPADocumentModel::update()
 {
+    emit layoutAboutToBeChanged();
     emit layoutChanged();
 }
 
@@ -65,7 +68,7 @@ int KoPADocumentModel::rowCount( const QModelIndex &parent ) const
 
     // check if parent is root node
     if( ! parent.isValid() )
-        return m_document->pages().count();
+        return m_document->pages(m_master).count();
 
     Q_ASSERT(parent.model() == this);
     Q_ASSERT(parent.internalPointer());
@@ -90,8 +93,8 @@ QModelIndex KoPADocumentModel::index( int row, int column, const QModelIndex &pa
     // check if parent is root node
     if( ! parent.isValid() )
     {
-        if( row >= 0 && row < m_document->pages().count() )
-            return createIndex( row, column, m_document->pages().at(row) );
+        if( row >= 0 && row < m_document->pages(m_master).count() )
+            return createIndex( row, column, m_document->pages(m_master).at(row) );
         else
             return QModelIndex();
     }
@@ -132,7 +135,7 @@ QModelIndex KoPADocumentModel::parent( const QModelIndex &child ) const
     if( ! grandParentShape )
     {
         KoPAPageBase* page = dynamic_cast<KoPAPageBase*>( parentShape);
-        return createIndex( m_document->pages().indexOf( page ), 0, parentShape );
+        return createIndex( m_document->pages(m_master).indexOf( page ), 0, parentShape );
     }
 
     return createIndex( indexFromChild( grandParentShape, parentShape ), 0, parentShape );
@@ -156,23 +159,36 @@ QVariant KoPADocumentModel::data( const QModelIndex &index, int role ) const
             if( name.isEmpty() )
             {
                 if ( dynamic_cast<KoPAPageBase *>( shape ) ) {
-                    name = i18n("Page");
+                    if(m_document->pageType() == KoPageApp::Slide ) {
+                        name = i18n("Slide %1",  m_document->pageIndex(dynamic_cast<KoPAPageBase *>(shape)) + 1);
+                    } else {
+                        name = i18n("Page  %1", m_document->pageIndex(dynamic_cast<KoPAPageBase *>(shape)) + 1);
+                    }
                 }
                 else if( dynamic_cast<KoShapeLayer*>( shape ) ) {
-                    name = i18n("Layer");
+                    name = i18n("Layer") + QString(" (%1)").arg(shape->zIndex());
                 }
                 else if( dynamic_cast<KoShapeGroup*>( shape ) ) {
-                    name = i18n("Group");
+                    name = i18n("Group") + QString(" (%1)").arg(shape->zIndex());
                 }
                 else {
-                    name = i18n("Shape");
+                    name = i18n("Shape") + QString(" (%1)").arg(shape->zIndex());
                 }
             }
-            return name + QString(" (%1)").arg( shape->zIndex() );
+            return name;
         }
         case Qt::DecorationRole: return QVariant();//return shape->icon();
         case Qt::EditRole: return shape->name();
-        case Qt::SizeHintRole: return shape->size();
+        case Qt::SizeHintRole:
+        {
+            KoPAPageBase *page = dynamic_cast<KoPAPageBase*>(shape);
+            if (page) { // return actual page size for page
+                KoPageLayout layout = page->pageLayout();
+                return QSize(layout.width, layout.height);
+            }
+            else
+                return shape->size();
+        }
         case ActiveRole:
         {
             KoCanvasController * canvasController = KoToolManager::instance()->activeCanvasController();
@@ -198,7 +214,7 @@ QVariant KoPADocumentModel::data( const QModelIndex &index, int role ) const
                 foreach( KoShape* shape, container->iterator() )
                     bbox = bbox.united( shape->outline().boundingRect() );
             }
-            return double(bbox.width()) / bbox.height();
+            return qreal(bbox.width()) / bbox.height();
         }
         default:
             if (role >= int(BeginThumbnailRole))
@@ -238,8 +254,11 @@ bool KoPADocumentModel::setData(const QModelIndex &index, const QVariant &value,
     {
         case Qt::DisplayRole:
         case Qt::EditRole:
-            shape->setName( value.toString() );
-            break;
+        {
+            QUndoCommand * cmd = new KoShapeRenameCommand( shape, value.toString() );
+            // TODO 2.1 use different text for the command if e.g. it is a page/slide or layer
+            m_document->addCommand( cmd );
+        }   break;
         case PropertiesRole:
             setProperties( shape, value.value<PropertyList>());
             break;
@@ -266,7 +285,7 @@ KoDocumentSectionModel::PropertyList KoPADocumentModel::properties( KoShape* sha
 {
     PropertyList l;
     l << Property(i18n("Visible"), SmallIcon("14_layer_visible"), SmallIcon("14_layer_novisible"), shape->isVisible());
-    l << Property(i18n("Locked"), SmallIcon("security-high"), SmallIcon("security-low"), shape->isLocked());
+    l << Property(i18n("Locked"), SmallIcon("object-locked"), SmallIcon("object-unlocked"), shape->isLocked());
     return l;
 }
 
@@ -284,23 +303,38 @@ void KoPADocumentModel::setProperties( KoShape* shape, const PropertyList &prope
 
 QImage KoPADocumentModel::createThumbnail( KoShape* shape, const QSize &thumbSize ) const
 {
-    KoShapePainter painter;
+    QSize size(thumbSize.width(), thumbSize.height());
+    KoShapePainter shapePainter;
 
     QList<KoShape*> shapes;
 
+    KoPAPageBase *page = dynamic_cast<KoPAPageBase*>(shape);
+    if (page) { // We create a thumbnail with actual width / height ratio for page
+        KoZoomHandler zoomHandler;
+        KoPageLayout layout = page->pageLayout();
+        qreal ratio = (zoomHandler.resolutionX() * layout.width) / (zoomHandler.resolutionY() * layout.height);
+        if ( ratio > 1 ) {
+            size.setHeight( size.width() / ratio );
+        }
+        else {
+            size.setWidth( size.height() * ratio );
+        }
+        QPixmap pixmap = page->thumbnail( size );
+        return pixmap.toImage();
+    }
+
     KoShapeContainer * container = dynamic_cast<KoShapeContainer*>( shape );
     if( container )
         shapes = container->iterator();
     else
         shapes.append( shape );
 
-    painter.setShapes( shapes );
+    shapePainter.setShapes( shapes );
 
-    QImage thumb( thumbSize, QImage::Format_RGB32 );
+    QImage thumb( size, QImage::Format_RGB32 );
     // draw the background of the thumbnail
     thumb.fill( QColor( Qt::white ).rgb() );
-
-    painter.paintShapes( thumb );
+    shapePainter.paintShapes( thumb );
 
     return thumb;
 }
@@ -428,7 +462,7 @@ bool KoPADocumentModel::dropMimeData( const QMimeData * data, Qt::DropAction act
 
     if( ! parent.isValid() )
     {
-        kDebug(38000) <<"KoPADocumentModel::dropMimeData parent = root";
+        kDebug(30010) <<"KoPADocumentModel::dropMimeData parent = root";
         return false;
     }
     KoShape *shape = static_cast<KoShape*>( parent.internalPointer() );
@@ -438,10 +472,11 @@ bool KoPADocumentModel::dropMimeData( const QMimeData * data, Qt::DropAction act
         KoShapeGroup * group = dynamic_cast<KoShapeGroup*>( container );
         if( group )
         {
-            kDebug(38000) <<"KoPADocumentModel::dropMimeData parent = group";
+            kDebug(30010) <<"KoPADocumentModel::dropMimeData parent = group";
             if( ! toplevelShapes.count() )
                 return false;
 
+            emit layoutAboutToBeChanged();
             beginInsertRows( parent, group->childCount(), group->childCount()+toplevelShapes.count() );
 
             QUndoCommand * cmd = new QUndoCommand();
@@ -455,12 +490,14 @@ bool KoPADocumentModel::dropMimeData( const QMimeData * data, Qt::DropAction act
             canvasController->canvas()->addCommand( cmd );
 
             endInsertRows();
+            emit layoutChanged();
         }
         else
         {
-            kDebug(38000) <<"KoPADocumentModel::dropMimeData parent = container";
+            kDebug(30010) <<"KoPADocumentModel::dropMimeData parent = container";
             if( toplevelShapes.count() )
             {
+                emit layoutAboutToBeChanged();
                 beginInsertRows( parent, container->childCount(), container->childCount()+toplevelShapes.count() );
 
                 QUndoCommand * cmd = new QUndoCommand();
@@ -484,6 +521,7 @@ bool KoPADocumentModel::dropMimeData( const QMimeData * data, Qt::DropAction act
                 canvasController->canvas()->addCommand( cmd );
 
                 endInsertRows();
+                emit layoutChanged();
             }
             else if( layers.count() )
             {
@@ -498,7 +536,7 @@ bool KoPADocumentModel::dropMimeData( const QMimeData * data, Qt::DropAction act
     }
     else
     {
-        kDebug(38000) <<"KoPADocumentModel::dropMimeData parent = shape";
+        kDebug(30010) <<"KoPADocumentModel::dropMimeData parent = shape";
         if( ! toplevelShapes.count() )
             return false;
 
@@ -531,7 +569,7 @@ QModelIndex KoPADocumentModel::parentIndexFromShape( const KoShape * child )
     if( parentLayer ) {
         KoPAPageBase * page = dynamic_cast<KoPAPageBase*>( parentLayer->parent() );
         if ( page )
-            return createIndex( m_document->pages().count() - 1 - m_document->pages().indexOf( page ), 0, parentLayer );
+            return createIndex( m_document->pages(m_master).count() - 1 - m_document->pages(m_master).indexOf( page ), 0, parentLayer );
     }
     // get the grandparent to determine the row of the parent shape
     KoShapeContainer *grandParentShape = parentShape->parent();
@@ -544,7 +582,22 @@ QModelIndex KoPADocumentModel::parentIndexFromShape( const KoShape * child )
 void KoPADocumentModel::setDocument( KoPADocument* document )
 {
     m_document = document;
-    update();
+
+    if ( m_document )
+    {
+        connect( m_document, SIGNAL(pageAdded( KoPAPageBase* ) ), this, SLOT( update() ) );
+        connect( m_document, SIGNAL(pageRemoved( KoPAPageBase* ) ), this, SLOT( update() ) );
+        connect( m_document, SIGNAL(shapeAdded( KoShape* ) ), this, SLOT( update() ) );
+        connect( m_document, SIGNAL(shapeRemoved( KoShape* ) ), this, SLOT( update() ) );
+    }
+
+    reset();
+}
+
+void KoPADocumentModel::setMasterMode(bool master)
+{
+    m_master = master;
+    update(); // Rebuild the model
 }
 
 #include "KoPADocumentModel.moc"
