@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-   Copyright (C) 2006-2007 Thorsten Zachmann <zachmann@kde.org>
+   Copyright (C) 2006-2008 Thorsten Zachmann <zachmann@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -42,6 +42,21 @@ public:
     /// Set the page layout to @p layout
     void setPageLayout(const KoPageLayout& layout) { m_pageLayout = layout; }
 
+    /// reimplemented
+    virtual bool displayMasterShapes();
+
+    /// reimplemented
+    virtual void setDisplayMasterShapes( bool display );
+
+    /// reimplemented
+    virtual bool displayMasterBackground();
+
+    /// reimplemented
+    virtual void setDisplayMasterBackground( bool display );
+
+    /// reimplemented
+    virtual QPixmap thumbnail( const QSize& size = QSize( 512, 512 ) );
+
 protected:
     /// Reimplemented from KoPageBase
     virtual void loadOdfPageTag( const KoXmlElement &element, KoPALoadingContext &loadingContext );
