@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-   Copyright (C) 2006-2007 Thorsten Zachmann <zachmann@kde.org>
+   Copyright (C) 2006-2008 Thorsten Zachmann <zachmann@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -23,15 +23,17 @@
 
 #include <QList>
 #include <QString>
+#include <QPixmap>
 
 #include <KoShapeContainer.h>
 #include <KoXmlReader.h>
-#include <KoPASavingContext.h>
 
+#include "KoPageApp.h"
+#include "KoPASavingContext.h"
 #include "kopageapp_export.h"
 
 struct KoPageLayout;
-class KoOasisLoadingContext;
+class KoOdfLoadingContext;
 class KoGenStyle;
 class KoShape;
 class KoPALoadingContext;
@@ -58,20 +60,84 @@ public:
      */
     virtual void saveOdf( KoShapeSavingContext & context ) const = 0;
 
-    // reimplemented
+    /// reimplemented
     virtual bool loadOdf( const KoXmlElement & element, KoShapeLoadingContext & context );
 
 
     /// @return the layout of the page
     virtual KoPageLayout & pageLayout() = 0;
+    virtual const KoPageLayout & pageLayout() const = 0;
 
     virtual void paintComponent(QPainter& painter, const KoViewConverter& converter);
 
+    /**
+     * @brief Paint background
+     *
+     * @param painter used to paint the background
+     * @param converter to convert between internal and view coordinates
+     */
+    virtual void paintBackground( QPainter & painter, const KoViewConverter & converter );
+
+    /**
+     * Get if master shapes should be displayed
+     *
+     * For master pages this always returns false
+     *
+     * @return true if master shapes should be displayed
+     */
+    virtual bool displayMasterShapes() = 0;
+
+    /**
+     * Set if the master shapes should be displayed
+     *
+     * For master pages this does nothing
+     */
+    virtual void setDisplayMasterShapes( bool display ) = 0;
+
+    /**
+     * Get if master page background should be used
+     *
+     * For master pages this always returns false
+     *
+     * @return true if master page background should be used
+     */
+    virtual bool displayMasterBackground() = 0;
+
+    virtual void setDisplayMasterBackground( bool display ) = 0;
+
+    virtual QPixmap thumbnail( const QSize& size = QSize( 512, 512 ) ) = 0;
+
+    /// reimplemented
+    virtual QSizeF size() const;
+
+    /**
+     * This function is called after a shape is added to the document on this page
+     * The default implementation is empty.
+     *
+     * @param shape The shape that was added
+     */
+    virtual void shapeAdded( KoShape * shape );
+
+    /**
+     * This function is called after a shape is removed from the document off this page
+     * The default implementation is empty.
+     *
+     * @param shape The shape that was removed
+     */
+    virtual void shapeRemoved( KoShape * shape );
+
+    /**
+     * Get the page type used in the document
+     *
+     * The default page type KoPageApp::Page is returned
+     */
+    virtual KoPageApp::PageType pageType() const;
+
 protected:
     /**
      * @param paContext the pageapp saving context
      */
-    void saveOdfPageContent( KoPASavingContext & paContext ) const;
+    virtual void saveOdfPageContent( KoPASavingContext & paContext ) const;
 
     /**
      * @brief Save the shapes of a page
@@ -98,9 +164,10 @@ protected:
      *
      * Here is a empty implementation as not all page apps presentations notes.
      *
+     * @param paContext the pageapp saving context
      * @return true on success, false otherwise
      */
-    virtual bool saveOdfPresentationNotes() const { return true; }
+    virtual bool saveOdfPresentationNotes(KoPASavingContext &paContext) const { Q_UNUSED( paContext ); return true; }
 
     /**
      * @brief Save the style of the page
@@ -128,6 +195,17 @@ protected:
      * @param paContext the pageapp loading context
      */
     virtual void loadOdfPageTag( const KoXmlElement &element, KoPALoadingContext &loadingContext );
+
+    /**
+     * @brief Load extra page data
+     *
+     * This method gets called after all shapes of the page are loaded. 
+     * The default implentation is empty
+     *
+     * @param element the page element
+     * @param paContext the pageapp loading context
+     */
+    virtual void loadOdfPageExtra( const KoXmlElement &element, KoPALoadingContext & loadingContext );
 };
 
 #endif /* KOPAPAGEBASE_H */
