@@ -20,11 +20,16 @@
 #include "KoPASavingContext.h"
 
 #include <QPixmap>
+#include <QRegExp>
 
-KoPASavingContext::KoPASavingContext( KoXmlWriter &xmlWriter, KoGenStyles& mainStyles, int page, SavingMode savingMode )
-: KoShapeSavingContext( xmlWriter, mainStyles, savingMode )
+#include "KoPAPage.h"
+
+KoPASavingContext::KoPASavingContext( KoXmlWriter &xmlWriter, KoGenStyles& mainStyles, KoEmbeddedDocumentSaver &embeddedSaver,
+                                      int page, SavingMode savingMode )
+: KoShapeSavingContext( xmlWriter, mainStyles, embeddedSaver, savingMode )
 , m_page( page )
 , m_masterPageIndex( 0 )
+, m_clearDrawIds( false )
 {
 }
 
@@ -37,7 +42,7 @@ void KoPASavingContext::addMasterPage( const KoPAMasterPage * masterPage, const
     m_masterPageNames.insert( masterPage, name );
 }
 
-QString KoPASavingContext::masterPageName( const KoPAMasterPage * masterPage )
+QString KoPASavingContext::masterPageName( const KoPAMasterPage * masterPage ) const
 {
     QMap<const KoPAMasterPage *, QString>::const_iterator it( m_masterPageNames.find( masterPage ) );
     if (  it != m_masterPageNames.constEnd() ) {
@@ -66,3 +71,33 @@ int KoPASavingContext::page()
 {
     return m_page;
 }
+
+void KoPASavingContext::setClearDrawIds( bool clear )
+{
+    m_clearDrawIds = clear;
+}
+
+bool KoPASavingContext::isSetClearDrawIds()
+{
+    return m_clearDrawIds;
+}
+
+QString KoPASavingContext::pageName( const KoPAPage * page )
+{
+    QString name;
+    QMap<const KoPAPage *, QString>::iterator it( m_pageToNames.find( page ) );
+    if ( it != m_pageToNames.end() ) {
+        name = it.value();
+    }
+    else {
+        name = page->name();
+        QRegExp rx( "^page[0-9]+$" );
+        if ( name.isEmpty() || m_pageNames.contains( name ) || rx.search( name ) != -1 ) {
+            name = "page" + QString::number( m_page );
+        }
+        Q_ASSERT( !m_pageNames.contains( name ) );
+        m_pageNames.insert( name );
+        m_pageToNames.insert( page, name );
+    }
+    return name;
+}
