@@ -26,18 +26,20 @@
 #include <KoZoomHandler.h>
 #include <KoRuler.h>
 #include "KoPageApp.h"
-#include "KoPADocumentStructureDocker.h"
 #include "kopageapp_export.h"
 
 class KoCanvasController;
+class KoFind;
 class KoPACanvas;
 class KoPADocument;
 class KToggleAction;
 class KoPAPageBase;
 class KoPAViewMode;
+class KoPADocumentStructureDocker;
 class KoShapeManager;
 class KoZoomAction;
 class KoZoomController;
+class QTextDocument;
 
 /// Creates a view with a KoPACanvas and rulers
 class KOPAGEAPP_EXPORT KoPAView : public KoView
@@ -46,7 +48,12 @@ class KOPAGEAPP_EXPORT KoPAView : public KoView
 public:
     enum KoPAAction
     {
-        ActionDeletePage = 1
+        ActionInsertPage = 1,
+        ActionCopyPage   = 2,
+        ActionDeletePage = 4,
+        ActionViewShowMasterPages = 8,
+        ActionFormatMasterPage = 16,
+        AllActions       = 0xFF
     };
 
     /**
@@ -59,10 +66,15 @@ public:
 
     void updateReadWrite( bool readwrite );
 
-    virtual KoViewConverter * viewConverter() { return &m_zoomHandler; }
+    virtual KoViewConverter * viewConverter( KoPACanvas * canvas ) { Q_UNUSED( canvas ); return &m_zoomHandler; }
 
     KoZoomHandler* zoomHandler() { return &m_zoomHandler; }
 
+    KoZoomController *zoomController() { return m_zoomController; }
+
+    KoRuler *horizontalRuler() { return m_horizontalRuler; }
+    KoRuler *verticalRuler() { return m_verticalRuler; }
+
     KoPACanvas * kopaCanvas() { return m_canvas; }
     KoPACanvas * kopaCanvas() const { return m_canvas; }
 
@@ -70,9 +82,6 @@ public:
     /// @return Page that is shown in the canvas
     KoPAPageBase* activePage() const;
 
-    /// Set the active page and updates the UI
-    void updateActivePage( KoPAPageBase * page );
-
     /// Set page shown in the canvas to @p page
     void setActivePage( KoPAPageBase * page );
 
@@ -105,9 +114,19 @@ public:
     void setActionEnabled( int actions, bool enable );
 
 public slots:
+    /// Set the active page and updates the UI
+    void updateActivePage( KoPAPageBase * page );
+
     /// Shows/hides the rulers
     void setShowRulers(bool show);
 
+    /// Insert a new page after the current one
+    void insertPage();
+
+signals:
+    /// Emitted every time the active page is changed
+    void activePageChanged();
+
 protected:
     /// creates the widgets (called from the constructor)
     void initGUI();
@@ -117,14 +136,26 @@ protected:
     /// Returns the document structure docker
     KoPADocumentStructureDocker* documentStructureDocker() const;
 
+    /// Called when receiving a PartActivateEvent
+    virtual void partActivateEvent(KParts::PartActivateEvent* event);
+
+    /// Update page navigation actions
+    void updatePageNavigationActions();
+
+    bool isMasterUsed( KoPAPageBase * page );
+
 protected slots:
-    void viewSnapToGrid();
-    void viewGrid();
-    void slotZoomChanged( KoZoomMode::Mode mode, double zoom );
+    void viewSnapToGrid(bool snap);
+    void viewGrid(bool show);
+    void viewGuides(bool show);
+    void slotZoomChanged( KoZoomMode::Mode mode, qreal zoom );
 
-    void editCut();
-    void editCopy();
     void editPaste();
+    void editDeleteSelection();
+    void editSelectAll();
+    void editDeselectAll();
+
+    void formatMasterPage();
 
     /// Change the current view mode to work on master pages
     void setMasterMode( bool master );
@@ -138,12 +169,38 @@ protected slots:
     /// Called when the selection changed
     virtual void selectionChanged();
 
-    /// Insert a new page after the current one
-    void insertPage();
+    /// Copy Page
+    void copyPage();
 
     /// Delete the current page
     void deletePage();
 
+    /// Called when the clipboard changed
+    virtual void clipboardDataChanged();
+
+    /// Go to the previous page
+    void goToPreviousPage();
+    /// Go to the next page
+    void goToNextPage();
+    /// Go to the first page
+    void goToFirstPage();
+    /// Go to the last page
+    void goToLastPage();
+
+    /**
+     * Set the next document that should be used in find
+     *
+     * @param document The current document
+     */
+    void findDocumentSetNext( QTextDocument * document );
+
+    /**
+     * Set the previous document that should be used in find
+     *
+     * @param document The current document
+     */
+    void findDocumentSetPrevious( QTextDocument * document );
+
 protected:
     KoPADocument *m_doc;
     KoPACanvas *m_canvas;
@@ -158,18 +215,29 @@ private:
     KoZoomController * m_zoomController;
     KoZoomHandler m_zoomHandler;
 
+    KAction *m_editPaste;
+    KAction *m_deleteSelectionAction;
+
     KToggleAction *m_actionViewSnapToGrid;
     KToggleAction *m_actionViewShowGrid;
+    KToggleAction *m_actionViewShowGuides;
     KToggleAction *m_actionViewShowMasterPages;
 
     KAction * m_actionInsertPage;
+    KAction * m_actionCopyPage;
     KAction * m_actionDeletePage;
 
+    KAction *m_actionMasterPage;
+
     KoRuler *m_horizontalRuler;
     KoRuler *m_verticalRuler;
     KToggleAction* m_viewRulers;
 
     KoZoomAction *m_zoomAction;
+
+    KoFind * m_find;
+
+    KoPAViewMode *m_viewModeNormal;
 };
 
 #endif /* KOPAVIEW_H */
