@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
- * Copyright ( C ) 2007 Thorsten Zachmann <zachmann@kde.org>
+ * Copyright (C) 2007-2008 Thorsten Zachmann <zachmann@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -42,16 +42,30 @@ KoPAViewModeNormal::~KoPAViewModeNormal()
 {
 }
 
-void KoPAViewModeNormal::paintEvent( QPaintEvent* event )
+void KoPAViewModeNormal::paintEvent( KoPACanvas *canvas, QPaintEvent* event )
 {
+    Q_ASSERT( m_canvas == canvas );
+
     QPainter painter( m_canvas );
     painter.translate( -m_canvas->documentOffset() );
     painter.setRenderHint( QPainter::Antialiasing );
-    painter.setClipRect( event->rect().translated( m_canvas->documentOffset() ) );
+    QRectF clipRect = event->rect().translated( m_canvas->documentOffset() );
+    painter.setClipRect( clipRect );
 
-    m_canvas->masterShapeManager()->paint( painter, *( m_view->viewConverter() ), false );
-    m_canvas->shapeManager()->paint( painter, *( m_view->viewConverter() ), false );
-    m_toolProxy->paint( painter, *( m_view->viewConverter() ) );
+    KoViewConverter * converter = m_view->viewConverter( m_canvas );
+    m_view->activePage()->paintBackground( painter, *converter );
+    if ( m_view->activePage()->displayMasterShapes() ) {
+        m_canvas->masterShapeManager()->paint( painter, *converter, false );
+    }
+    m_canvas->shapeManager()->paint( painter, *converter, false );
+    painter.setRenderHint( QPainter::Antialiasing, false );
+
+    QRectF updateRect = converter->viewToDocument( clipRect );
+    m_canvas->document()->gridData().paintGrid( painter, *converter, updateRect );
+    m_canvas->document()->guidesData().paintGuides( painter, *converter, updateRect );
+
+    painter.setRenderHint( QPainter::Antialiasing );
+    m_toolProxy->paint( painter, *converter );
 }
 
 void KoPAViewModeNormal::tabletEvent( QTabletEvent *event, const QPointF &point )
@@ -123,12 +137,12 @@ void KoPAViewModeNormal::setMasterMode( bool master )
     KoPAPage * page = dynamic_cast<KoPAPage *>( m_view->activePage() );
     if ( m_masterMode ) {
         if ( page ) {
-            m_view->setActivePage( page->masterPage() );
+            m_view->updateActivePage( page->masterPage() );
             m_savedPage = page;
         }
     }
     else if ( m_savedPage ) {
-        m_view->setActivePage( m_savedPage );
+        m_view->updateActivePage( m_savedPage );
         m_savedPage = 0;
     }
 }
@@ -137,3 +151,39 @@ bool KoPAViewModeNormal::masterMode()
 {
     return m_masterMode;
 }
+
+void KoPAViewModeNormal::addShape( KoShape *shape )
+{
+    // the KoShapeController sets the active layer as parent
+    KoPAPageBase * page( m_view->kopaDocument()->pageByShape( shape ) );
+
+    bool isMaster = dynamic_cast<KoPAMasterPage*>( page ) != 0;
+
+    KoPAPage * p;
+    if ( page == m_view->activePage() ) {
+        m_view->kopaCanvas()->shapeManager()->add( shape );
+    }
+    else if ( isMaster && ( p = dynamic_cast<KoPAPage*>( m_view->activePage() ) ) != 0 ) {
+        if ( p->masterPage() == page ) {
+            m_view->kopaCanvas()->masterShapeManager()->add( shape );
+        }
+    }
+}
+
+void KoPAViewModeNormal::removeShape( KoShape *shape )
+{
+    KoPAPageBase * page( m_view->kopaDocument()->pageByShape( shape ) );
+
+    bool isMaster = dynamic_cast<KoPAMasterPage*>( page ) != 0;
+
+    KoPAPage * p;
+    if ( page == m_view->activePage() ) {
+        m_view->kopaCanvas()->shapeManager()->remove( shape );
+    }
+    else if ( isMaster && ( p = dynamic_cast<KoPAPage*>( m_view->activePage() ) ) != 0 ) {
+        if ( p->masterPage() == page ) {
+            m_view->kopaCanvas()->masterShapeManager()->remove( shape );
+        }
+    }
+}
+
