@@ -20,11 +20,10 @@
 #define KOINLINETEXTOBJECTMANAGER_H
 
 #include "KoInlineObject.h"
-#include "KoInlineObjectFactory.h"
 #include "KoBookmarkManager.h"
 #include "KoVariableManager.h"
 
-#include <kotext_export.h>
+#include "kotext_export.h"
 
 // Qt + kde
 #include <QHash>
@@ -41,16 +40,17 @@ class QAction;
  * KoTextDocumentLayout for that specific textDocument, your inline text object will get painted
  * properly.
  */
-class KOTEXT_EXPORT KoInlineTextObjectManager : public QObject {
+class KOTEXT_EXPORT KoInlineTextObjectManager : public QObject
+{
     Q_OBJECT
 // TODO, when to delete the inlineObject s
 public:
-     /// Constructor
+    /// Constructor
     explicit KoInlineTextObjectManager(QObject *parent = 0);
 
     /**
      * Retrieve a formerly added inline object based on the format.
-     * @param format the textCharFormat 
+     * @param format the textCharFormat
      */
     KoInlineObject *inlineTextObject(const QTextCharFormat &format) const;
     /**
@@ -133,8 +133,8 @@ public:
     QList<KoTextLocator*> textLocators() const;
 
 public slots:
-    void documentInformationUpdated (const QString &info, const QString &data);
-    
+    void documentInformationUpdated(const QString &info, const QString &data);
+
 signals:
     /**
      * Emitted whenever a propery is set and it turns out to be changed.
@@ -143,7 +143,7 @@ signals:
 
 private:
     enum Properties {
-        InlineInstanceId = 577297549
+        InlineInstanceId = 577297549 // If you change this, don't forget to change KoCharacterStyle.h
     };
 
     QHash<int, KoInlineObject*> m_objects;
