@@ -1,5 +1,6 @@
 /* This file is part of the KDE project
  * Copyright (C) 2006-2007 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2008 Thorsten Zachmann <zachmann@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,7 +22,7 @@
 
 // KOffice libs
 #include "KoInlineObject.h"
-#include <kotext_export.h>
+#include "kotext_export.h"
 
 // Qt + kde
 #include <QHash>
@@ -29,7 +30,9 @@
 
 class QTextInlineObject;
 class QTextDocument;
+class KoXmlElement;
 class KoShape;
+class KoShapeLoadingContext;
 class KoProperties;
 class QWidget;
 class KoVariableManager;
@@ -41,7 +44,8 @@ class VariablePrivate;
  * is used as text.  This class is pretty boring in that it has just a setValue() to alter the
  * text shown; we depend on plugin writers to create more exciting ways to update variables.
  */
-class KOTEXT_EXPORT KoVariable : public KoInlineObject {
+class KOTEXT_EXPORT KoVariable : public KoInlineObject
+{
 public:
     /**
      * Constructor.
@@ -66,14 +70,29 @@ public:
      * attributes from the ODF file (if applicable), so it would be useful to synchronize
      * the property names based on that.
      */
-    virtual void setProperties(const KoProperties *props) { Q_UNUSED(props); }
+    virtual void setProperties(const KoProperties *props) {
+        Q_UNUSED(props);
+    }
 
     /**
      * If this variable has user-editable options it should provide a widget that is capable
      * of manipulating these options so the text-tool can use it to show that to the user.
      * Note that all manipulations should have a direct effect on the variable itself.
      */
-    virtual QWidget *createOptionsWidget() { return 0; }
+    virtual QWidget *createOptionsWidget() {
+        return 0;
+    }
+
+    /**
+     * Load a variable from odf.
+     * TODO make abstract when it is implemented in all variables
+     *
+     * @param element element which represents the shape in odf
+     * @param context the KoShapeLoadingContext used for loading
+     *
+     * @return false if loading failed
+     */
+    virtual bool loadOdf(const KoXmlElement &element, KoShapeLoadingContext &context);
 
 protected:
     /**
@@ -92,11 +111,11 @@ protected:
 
 private:
     void updatePosition(const QTextDocument *document, QTextInlineObject object,
-            int posInDocument, const QTextCharFormat &format);
+                        int posInDocument, const QTextCharFormat &format);
     void resize(const QTextDocument *document, QTextInlineObject object,
-            int posInDocument, const QTextCharFormat &format, QPaintDevice *pd);
-    void paint (QPainter &painter, QPaintDevice *pd, const QTextDocument *document,
-            const QRectF &rect, QTextInlineObject object, int posInDocument, const QTextCharFormat &format);
+                int posInDocument, const QTextCharFormat &format, QPaintDevice *pd);
+    void paint(QPainter &painter, QPaintDevice *pd, const QTextDocument *document,
+               const QRectF &rect, QTextInlineObject object, int posInDocument, const QTextCharFormat &format);
 
 private:
     VariablePrivate * const d;
