@@ -21,11 +21,14 @@
 #include "TestLoading.h"
 
 #include <QtGui>
-#include <KDebug>
+#include <QDebug>
 #include <QtScript>
 #include <QtTest>
 
+#include <kdeversion.h>
+
 #include <KoStyleManager.h>
+#include <KoDocument.h>
 #include <KoOdfStylesReader.h>
 #include <KoStore.h>
 #include <KoOdfStylesReader.h>
@@ -45,8 +48,6 @@
 #include <kcomponentdata.h>
 #include <KoTextDebug.h>
 #include <KoListStyle.h>
-#include <KoTableStyle.h>
-#include <KoTableCellStyle.h>
 #include <KoTextDocumentLayout.h>
 #include <KoStyleManager.h>
 #include <KoCharacterStyle.h>
@@ -55,12 +56,7 @@
 #include <KoEmbeddedDocumentSaver.h>
 #include <KoInlineTextObjectManager.h>
 #include <KoTextSharedLoadingData.h>
-#include <KoTextSharedSavingData.h>
 #include <KoTextDocument.h>
-#include <kstandarddirs.h>
-
-#include <KoGenChanges.h>
-#include <KoChangeTracker.h>
 
 typedef KoText::Tab KoTextTab;
 // because in a QtScript, I don't seem to be able to use a namespaced type
@@ -75,7 +71,7 @@ static void showDocument(QTextDocument *document)
 }
 
 // Functions that help compare two QTextDocuments
-bool TestLoading::compareFragments(const QTextFragment &actualFragment, const QTextFragment &expectedFragment)
+static bool compareFragments(const QTextFragment &actualFragment, const QTextFragment &expectedFragment)
 {
     if (actualFragment.text() != expectedFragment.text()) {
         qDebug() << "compareFragments: text not equal " << actualFragment.text() << expectedFragment.text();
@@ -152,7 +148,7 @@ bool TestLoading::compareFragments(const QTextFragment &actualFragment, const QT
     return equal;
 }
 
-bool TestLoading::compareTabProperties(QVariant actualTabs, QVariant expectedTabs)
+static bool compareTabProperties(QVariant actualTabs, QVariant expectedTabs)
 {
     QList<QVariant> actualTabList = qvariant_cast<QList<QVariant> >(actualTabs);
     QList<QVariant> expectedTabList = qvariant_cast<QList<QVariant> >(expectedTabs);
@@ -177,7 +173,7 @@ bool TestLoading::compareTabProperties(QVariant actualTabs, QVariant expectedTab
     return true;
 }
 
-bool TestLoading::compareBlockFormats(const QTextBlockFormat &actualFormat, const QTextBlockFormat &expectedFormat)
+static bool compareBlockFormats(const QTextBlockFormat &actualFormat, const QTextBlockFormat &expectedFormat)
 {
     if (actualFormat.background() != expectedFormat.background()
             || actualFormat.alignment() != expectedFormat.alignment()
@@ -197,7 +193,7 @@ bool TestLoading::compareBlockFormats(const QTextBlockFormat &actualFormat, cons
         switch (id) {
         case KoParagraphStyle::UnnumberedListItem:
         case KoParagraphStyle::IsListHeader:
-            if (actualProperty[id].toInt() != static_cast<int>(expectedProperty[id].toBool()))
+            if (actualProperty[id].toInt() != expectedProperty[id].toBool())
                 match = false;
             break;
         case KoParagraphStyle::AutoTextIndent:
@@ -245,7 +241,7 @@ bool TestLoading::compareBlockFormats(const QTextBlockFormat &actualFormat, cons
     return match;
 }
 
-bool TestLoading::compareListFormats(const QTextListFormat &actualFormat, const QTextListFormat &expectedFormat)
+static bool compareListFormats(const QTextListFormat &actualFormat, const QTextListFormat &expectedFormat)
 {
     QMap<int, QVariant> actualProperties = actualFormat.properties();
     actualProperties.remove(KoListStyle::StyleId);
@@ -261,7 +257,7 @@ bool TestLoading::compareListFormats(const QTextListFormat &actualFormat, const
     return actualProperties == expectedProperties;
 }
 
-bool TestLoading::compareBlocks(const QTextBlock &actualBlock, const QTextBlock &expectedBlock)
+static bool compareBlocks(const QTextBlock &actualBlock, const QTextBlock &expectedBlock)
 {
     QTextList *actualList = actualBlock.textList();
     QTextList *expectedList = expectedBlock.textList();
@@ -321,203 +317,14 @@ bool TestLoading::compareBlocks(const QTextBlock &actualBlock, const QTextBlock
     return equal;
 }
 
-bool TestLoading::compareTableCellFormats(QTextTableCellFormat &actualFormat, QTextTableCellFormat &expectedFormat)
+static bool compareTables(QTextTable * /*actualTable*/, QTextTable * /*expectedTable*/)
 {
-    if (actualFormat.background() != expectedFormat.background()
-            || actualFormat.leftPadding() != expectedFormat.leftPadding()
-            || actualFormat.rightPadding() != expectedFormat.rightPadding()
-            || actualFormat.topPadding() != expectedFormat.topPadding()
-            || actualFormat.bottomPadding() != expectedFormat.bottomPadding()) {
-        return false;
-    }
-
-    // check custom properties
-    const QMap<int, QVariant> actualProperty = actualFormat.properties();
-    const QMap<int, QVariant> expectedProperty = expectedFormat.properties();
-    QList<int> allPropertyIds = actualProperty.keys();
-    allPropertyIds << expectedProperty.keys();
-    bool match = true;
-    foreach(int id, allPropertyIds) {
-        QString key, value;
-        switch (id) {
-        // double properties
-        case KoTableCellStyle::LeftBorderSpacing:
-        case KoTableCellStyle::RightBorderSpacing:
-        case KoTableCellStyle::TopBorderSpacing:
-        case KoTableCellStyle::BottomBorderSpacing:
-            if (abs(actualProperty[id].toDouble() - expectedProperty[id].toDouble()) > 1e-10)
-                match = false;
-            break;
-        // string properties
-        case KoTableCellStyle::MasterPageName:
-            if (actualProperty[id].toString() != expectedProperty[id].toString())
-                match = false;
-            break;
-        // pen properties
-        case KoTableCellStyle::LeftBorderOuterPen:
-        case KoTableCellStyle::LeftBorderInnerPen:
-        case KoTableCellStyle::RightBorderOuterPen:
-        case KoTableCellStyle::RightBorderInnerPen:
-        case KoTableCellStyle::TopBorderOuterPen:
-        case KoTableCellStyle::TopBorderInnerPen:
-        case KoTableCellStyle::BottomBorderOuterPen:
-        case KoTableCellStyle::BottomBorderInnerPen: {
-            QPen actualPen = qvariant_cast<QPen>(actualProperty[id]);
-            QPen expectedPen = qvariant_cast<QPen>(expectedProperty[id]);
-            if (actualPen != expectedPen)
-                match = false;
-            break;
-        }
-        // brush properties
-        case KoTableCellStyle::CellBackgroundBrush: {
-            QBrush actualBrush = qvariant_cast<QBrush>(actualProperty[id]);
-            QBrush expectedBrush = qvariant_cast<QBrush>(expectedProperty[id]);
-            if (actualBrush != expectedBrush)
-                match = false;
-            break;
-        }
-        }
-        if (!match) {
-            qDebug() << "Actual property:   " << KoTextDebug::tableCellAttributes(actualFormat);
-            qDebug() << "Expected property: " << KoTextDebug::tableCellAttributes(expectedFormat);
-            qDebug() << "At index: QTextTableFormat::UserProperty + " << id + 7001 - QTextFormat::UserProperty;
-            return false;
-        }
-    }
-    return match;
+    // FIXME: Cells of Tables are QTextTableCell's which contain QTextFrames
+    // KWord does not implement tables, yet.
+    return false;
 }
 
-bool TestLoading::compareTableCells(QTextTableCell &actualCell, QTextTableCell &expectedCell)
-{
-    // compare cell formats
-    QTextTableCellFormat actualFormat = actualCell.format().toTableCellFormat();
-    QTextTableCellFormat expectedFormat = expectedCell.format().toTableCellFormat();
-
-    if (!compareTableCellFormats(actualFormat, expectedFormat)) {
-        qDebug() << "compareTableCells: table cell properties mismatch at " << actualCell.firstCursorPosition().block().text();
-        return false;
-    }
-
-    // compare cell content
-    QTextFrame::iterator actualIterator = actualCell.begin();
-    QTextFrame::iterator expectedIterator = expectedCell.begin();
-
-    for (; !actualIterator.atEnd() && !expectedIterator.atEnd(); ++actualIterator, ++expectedIterator) {
-        QTextFrame *actualChildFrame = actualIterator.currentFrame();
-        QTextBlock actualTextBlock = actualIterator.currentBlock();
-
-        if (actualChildFrame) {
-            QTextFrame *expectedChildFrame = expectedIterator.currentFrame();
-            if (!expectedChildFrame) {
-                qDebug() << "compareTableCells: Unexpected frame at " << actualTextBlock.text();
-                return false;
-            }
-            QTextTable *actualTable = qobject_cast<QTextTable *>(actualChildFrame);
-            QTextTable *expectedTable = qobject_cast<QTextTable *>(expectedChildFrame);
-            if (actualTable) {
-                if (!expectedTable) {
-                    qDebug() << "compareTableCells: Unexpected table at " << actualTextBlock.text();
-                    return false;
-                }
-                if (!compareTables(actualTable, expectedTable))
-                    return false;
-            } else {
-                if (expectedTable) {
-                    qDebug() << "compareTableCells: Expecting table at " << actualTextBlock.text();
-                    return false;
-                }
-                if (!compareFrames(actualChildFrame, expectedChildFrame))
-                    return false;
-            }
-        } else if (actualTextBlock.isValid()) {
-            QTextBlock expectedTextBlock = expectedIterator.currentBlock();
-            if (!expectedTextBlock.isValid()) {
-                qDebug() << "compareTableCells: Unexpected text block at " << actualTextBlock.text();
-                return false;
-            }
-            if (!compareBlocks(actualTextBlock, expectedTextBlock))
-                return false;
-        } else {
-            qDebug() << "compareTableCells: neither frame nor block! - internal error!";
-            return false;
-        }
-    }
-    return actualIterator.atEnd() == expectedIterator.atEnd();
-}
-
-bool TestLoading::compareTableFormats(QTextTableFormat &actualFormat, QTextTableFormat &expectedFormat)
-{
-    if (actualFormat.background() != expectedFormat.background()
-            || actualFormat.alignment() != expectedFormat.alignment()
-            || actualFormat.width() != expectedFormat.width()) {
-        return false;
-    }
-
-    // check custom properties
-    const QMap<int, QVariant> actualProperty = actualFormat.properties();
-    const QMap<int, QVariant> expectedProperty = expectedFormat.properties();
-    QList<int> allPropertyIds = actualProperty.keys();
-    allPropertyIds << expectedProperty.keys();
-    bool match = true;
-    foreach(int id, allPropertyIds) {
-        QString key, value;
-        switch (id) {
-        // bool properties
-        case KoTableStyle::BreakBefore:
-        case KoTableStyle::BreakAfter:
-        case KoTableStyle::KeepWithNext:
-        case KoTableStyle::MayBreakBetweenRows:
-        case KoTableStyle::CollapsingBorders:
-            if (actualProperty[id].toBool() != expectedProperty[id].toBool())
-                match = false;
-            break;
-        // double properties
-        case QTextFormat::FrameLeftMargin:
-        case QTextFormat::FrameRightMargin:
-        case QTextFormat::FrameTopMargin:
-        case QTextFormat::FrameBottomMargin:
-            if (abs(actualProperty[id].toDouble() - expectedProperty[id].toDouble()) > 1e-10)
-                match = false;
-            break;
-        // string properties
-        case KoTableStyle::MasterPageName:
-            if (actualProperty[id].toString() != expectedProperty[id].toString())
-                match = false;
-            break;
-        }
-        if (!match) {
-            qDebug() << "Actual property:   " << KoTextDebug::tableAttributes(actualFormat);
-            qDebug() << "Expected property: " << KoTextDebug::tableAttributes(expectedFormat);
-            qDebug() << "At index: QTextTableFormat::UserProperty + " << id - QTextTableFormat::UserProperty;
-            return false;
-        }
-    }
-    return match;
-}
-
-bool TestLoading::compareTables(QTextTable *actualTable, QTextTable *expectedTable)
-{
-    QTextTableFormat actualFormat = actualTable->format();
-    QTextTableFormat expectedFormat = expectedTable->format();
-
-    if (!compareTableFormats(actualFormat, expectedFormat)) {
-        qDebug() << "compareTables: table properties mismatch at " << actualTable->cellAt(0, 0).firstCursorPosition().block().text();
-        return false;
-    }
-
-    for (int row = 0; row < expectedTable->rows(); ++row) {
-        for (int col = 0; col < expectedTable->columns(); ++col) {
-            QTextTableCell actualCell = actualTable->cellAt(row, col);
-            QTextTableCell expectedCell = expectedTable->cellAt(row, col);
-            if (!compareTableCells(actualCell, expectedCell)) {
-                return false;
-            }
-        }
-    }
-    return true;
-}
-
-bool TestLoading::compareFrames(QTextFrame *actualFrame, QTextFrame *expectedFrame)
+static bool compareFrames(QTextFrame *actualFrame, QTextFrame *expectedFrame)
 {
     QTextFrame::iterator actualIterator = actualFrame->begin();
     QTextFrame::iterator expectedIterator = expectedFrame->begin();
@@ -565,7 +372,7 @@ bool TestLoading::compareFrames(QTextFrame *actualFrame, QTextFrame *expectedFra
     return actualIterator.atEnd() == expectedIterator.atEnd();
 }
 
-bool TestLoading::compareDocuments(QTextDocument *actualDocument, QTextDocument *expectedDocument)
+static bool compareDocuments(QTextDocument *actualDocument, QTextDocument *expectedDocument)
 {
     QTextFrame *actualFrame = actualDocument->rootFrame();
     QTextFrame *expectedFrame = expectedDocument->rootFrame();
@@ -681,7 +488,6 @@ static QScriptValue copyFormatProperties(QScriptContext *context, QScriptEngine
 TestLoading::TestLoading()
 {
     componentData = new KComponentData("TestLoading");
-    componentData->dirs()->addResourceType("styles", "data", "kword/styles/");
 }
 
 TestLoading::~TestLoading()
@@ -745,16 +551,8 @@ void TestLoading::initTestCase()
 
     engine = new QScriptEngine();
 
-    QScriptValue rc = engine->importExtension("qt.core");
-    if (rc.isError()) {
-        kWarning() << "Failed to find Qt bindings, aborting";
-        QSKIP("Failed to find Qt bindings (qtscript generator), aborting", SkipAll);
-    }
-    rc = engine->importExtension("qt.gui");
-    if (rc.isError()) {
-        kWarning() << "Failed to load QtGui bindings, aborting";
-        abort();
-    }
+    engine->importExtension("qt.core");
+    engine->importExtension("qt.gui");
 
     QScriptValue globalObject = engine->globalObject();
     globalObject.setProperty("qApp", engine->newQObject(qApp));
@@ -788,7 +586,7 @@ void TestLoading::cleanup()
 {
 }
 
-void TestLoading::addData(LoadSave loadSave)
+void TestLoading::addData()
 {
     QTest::newRow("bulletedList") << "TextContents/Lists/bulletedList";
     QTest::newRow("continueNumbering") << "TextContents/Lists/continueNumbering";
@@ -804,37 +602,19 @@ void TestLoading::addData(LoadSave loadSave)
 
     QTest::newRow("attributedText") << "TextContents/Paragraph/attributedText";
     QTest::newRow("basicContents") << "TextContents/Paragraph/basicContents";
-    QTest::newRow("nestedSpan") << "TextContents/Paragraph/nestedSpan";
     QTest::newRow("whitespace") << "TextContents/Paragraph/whitespace";
 
     QTest::newRow("fontSize1") << "TextContents/TextFormatting/fontSize";
 
     QTest::newRow("fontColors") << "TextContents/TextFormatting/fontColors";
 
-    if (loadSave == TestLoadingData) {
-        QTest::newRow("tableWidth") << "FormattingProperties/TableFormattingProperties/tableWidth";
-        QTest::newRow("tableTopAndBottomMargin") << "FormattingProperties/TableFormattingProperties/tableTopAndBottomMargin";
-        QTest::newRow("tableLeftAndRightMargin") << "FormattingProperties/TableFormattingProperties/tableLeftAndRightMargin";
-    }
-
-    // TODO: Write tests for these.
-    //QTest::newRow("borderModelProperty") << "FormattingProperties/TableFormattingProperties/borderModelProperty";
-    //QTest::newRow("breakBeforeAndBreakAfter") << "FormattingProperties/TableFormattingProperties/breakBeforeAndBreakAfter";
-    //QTest::newRow("display") << "FormattingProperties/TableFormattingProperties/display";
-    //QTest::newRow("mayBreakBetweenRows") << "FormattingProperties/TableFormattingProperties/mayBreakBetweenRows";
-    //QTest::newRow("pageNumber") << "FormattingProperties/TableFormattingProperties/pageNumber";
-    //QTest::newRow("tableAlignment") << "FormattingProperties/TableFormattingProperties/tableAlignment";
-    //QTest::newRow("tableBackgroundAndBackgroundImage") << "FormattingProperties/TableFormattingProperties/tableBackgroundAndBackgroundImage";
-    //QTest::newRow("tableMargins") << "FormattingProperties/TableFormattingProperties/tableMargins";
-    //QTest::newRow("tableShadow") << "FormattingProperties/TableFormattingProperties/tableShadow";
-    //QTest::newRow("writingMode") << "FormattingProperties/TableFormattingProperties/writingMode";
-
     QTest::newRow("color") << "FormattingProperties/TextFormattingProperties/color";
-
     QTest::newRow("country") << "FormattingProperties/TextFormattingProperties/country";
     QTest::newRow("fontCharset") << "FormattingProperties/TextFormattingProperties/fontCharacterSet";
     QTest::newRow("fontFamily") << "FormattingProperties/TextFormattingProperties/fontFamily";
+#if QT_VERSION >= KDE_MAKE_VERSION(4,5,0)
     QTest::newRow("fontFamilyGeneric") << "FormattingProperties/TextFormattingProperties/fontFamilyGeneric";
+#endif
     QTest::newRow("fontName") << "FormattingProperties/TextFormattingProperties/fontName";
     QTest::newRow("fontPitch") << "FormattingProperties/TextFormattingProperties/fontPitch";
     QTest::newRow("fontSize2") << "FormattingProperties/TextFormattingProperties/fontSize";
@@ -842,7 +622,9 @@ void TestLoading::addData(LoadSave loadSave)
     QTest::newRow("fontWeight") << "FormattingProperties/TextFormattingProperties/fontWeight";
     QTest::newRow("fontVariant") << "FormattingProperties/TextFormattingProperties/fontVariant";
     QTest::newRow("language") << "FormattingProperties/TextFormattingProperties/language";
+#if QT_VERSION >= KDE_MAKE_VERSION(4,5,0)
     QTest::newRow("letterKerning") << "FormattingProperties/TextFormattingProperties/letterKerning";
+#endif
     QTest::newRow("lineThroughColor") << "FormattingProperties/TextFormattingProperties/lineThroughColor";
     QTest::newRow("lineThroughMode") << "FormattingProperties/TextFormattingProperties/lineThroughMode";
     QTest::newRow("lineThroughStyle") << "FormattingProperties/TextFormattingProperties/lineThroughStyle";
@@ -865,7 +647,6 @@ void TestLoading::addData(LoadSave loadSave)
     QTest::newRow("dropCapsDistance") << "FormattingProperties/ParagraphFormattingProperties/dropCapsDistance";
     QTest::newRow("dropCapsLength") << "FormattingProperties/ParagraphFormattingProperties/dropCapsLength";
     QTest::newRow("dropCapsLines") << "FormattingProperties/ParagraphFormattingProperties/dropCapsLines";
-    QTest::newRow("lineSpacing") << "FormattingProperties/ParagraphFormattingProperties/lineSpacing";
     QTest::newRow("margin") << "FormattingProperties/ParagraphFormattingProperties/margin";
     QTest::newRow("marginLeftRight") << "FormattingProperties/ParagraphFormattingProperties/marginLeftRight";
     QTest::newRow("marginTopBottom") << "FormattingProperties/ParagraphFormattingProperties/marginTopBottom";
@@ -910,22 +691,21 @@ QTextDocument *TestLoading::documentFromOdt(const QString &odt)
     KoXmlElement body = KoXml::namedItemNS(realBody, KoXmlNS::office, "text");
 
     KoStyleManager *styleManager = new KoStyleManager;
-    KoChangeTracker *changeTracker = new KoChangeTracker;
 
-    KoOdfLoadingContext odfLoadingContext(odfReadStore.styles(), odfReadStore.store(), *componentData);
-    KoShapeLoadingContext shapeLoadingContext(odfLoadingContext, 0);
+    KoOdfLoadingContext odfLoadingContext(odfReadStore.styles(), odfReadStore.store());
+    QMap<QString, KoDataCenter *> dataCenterMap;
+    KoShapeLoadingContext shapeLoadingContext(odfLoadingContext, dataCenterMap);
     KoTextSharedLoadingData *textSharedLoadingData = new KoTextSharedLoadingData;
-    textSharedLoadingData->loadOdfStyles(shapeLoadingContext, styleManager);
+    textSharedLoadingData->loadOdfStyles(odfLoadingContext, styleManager);
     shapeLoadingContext.addSharedData(KOTEXT_SHARED_LOADING_ID, textSharedLoadingData);
 
     KoTextShapeData *textShapeData = new KoTextShapeData;
     QTextDocument *document = new QTextDocument;
     textShapeData->setDocument(document, false /* ownership */);
     KoTextDocumentLayout *layout = new KoTextDocumentLayout(textShapeData->document());
-    layout->setInlineTextObjectManager(new KoInlineTextObjectManager(layout)); // required while saving
+    layout->setInlineObjectTextManager(new KoInlineTextObjectManager(layout)); // required while saving
     KoTextDocument(document).setStyleManager(styleManager);
     textShapeData->document()->setDocumentLayout(layout);
-    KoTextDocument(document).setChangeTracker(changeTracker);
 
     if (!textShapeData->loadOdf(body, shapeLoadingContext)) {
         qDebug() << "KoTextShapeData failed to load ODT";
@@ -962,74 +742,41 @@ QString TestLoading::documentToOdt(QTextDocument *document)
     KoXmlWriter xmlWriter(&contentTmpFile, 1);
 
     KoGenStyles mainStyles;
-    KoStyleManager *styleMan = KoTextDocument(document).styleManager();
-    Q_UNUSED(styleMan);
     KoEmbeddedDocumentSaver embeddedSaver;
-
-    KoGenChanges changes;
     KoShapeSavingContext context(xmlWriter, mainStyles, embeddedSaver);
 
-    KoSharedSavingData *sharedData = context.sharedData(KOTEXT_SHARED_SAVING_ID);
-    KoTextSharedSavingData *textSharedData = 0;
-    if (sharedData) {
-        textSharedData = dynamic_cast<KoTextSharedSavingData *>(sharedData);
-    }
-
-    kDebug(32500) << "sharedData" << sharedData << "textSharedData" << textSharedData;
-
-    if (!textSharedData) {
-        textSharedData = new KoTextSharedSavingData();
-        textSharedData->setGenChanges(changes);
-        if (!sharedData) {
-            context.addSharedData(KOTEXT_SHARED_SAVING_ID, textSharedData);
-        } else {
-            kWarning(32500) << "A different type of sharedData was found under the" << KOTEXT_SHARED_SAVING_ID;
-            Q_ASSERT(false);
-        }
-    }
+    xmlWriter.startElement("office:body");
+    xmlWriter.startElement("office:text");
 
     KoTextShapeData *textShapeData = new KoTextShapeData;
     textShapeData->setDocument(document, false /* ownership */);
-    if (qobject_cast<KoTextDocumentLayout *>(document->documentLayout()) == 0) {
+    if (dynamic_cast<KoTextDocumentLayout *>(document->documentLayout()) == 0) {
         // Setup layout and managers just like kotext
         KoTextDocumentLayout *layout = new KoTextDocumentLayout(textShapeData->document());
         textShapeData->document()->setDocumentLayout(layout);
-        layout->setInlineTextObjectManager(new KoInlineTextObjectManager(layout)); // required while saving
+        layout->setInlineObjectTextManager(new KoInlineTextObjectManager(layout)); // required while saving
         KoStyleManager *styleManager = new KoStyleManager;
         KoTextDocument(textShapeData->document()).setStyleManager(styleManager);
     }
-    KoChangeTracker* changeTracker = new KoChangeTracker;
-    KoTextDocument(textShapeData->document()).setChangeTracker(changeTracker);
-
     textShapeData->saveOdf(context);
 
+    xmlWriter.endElement(); // office:text
+    xmlWriter.endElement(); // office:body
+
     contentTmpFile.close();
 
-    mainStyles.saveOdfStyles(KoGenStyles::DocumentAutomaticStyles, contentWriter);
-
-    contentWriter->startElement("office:body");
-    contentWriter->startElement("office:text");
-
-//    changes.saveOdfChanges(contentWriter, false);
-
+    mainStyles.saveOdfAutomaticStyles(contentWriter, false);
     contentWriter->addCompleteElement(&contentTmpFile);
 
-    contentWriter->endElement(); //office text
-    contentWriter->endElement(); //office body
-
     contentWriter->endElement(); // root element
     contentWriter->endDocument();
     delete contentWriter;
 
-
     if (!store->close())
         qWarning() << "Failed to close the store";
 
-    mainStyles.saveOdfStylesDotXml(store, manifestWriter);
-
     odfWriteStore.closeManifestWriter();
 
-
     delete store;
     delete textShapeData;
 
@@ -1040,7 +787,7 @@ void TestLoading::testLoading_data()
 {
     QTest::addColumn<QString>("testcase");
 
-    addData(TestLoadingData);
+    addData();
 }
 
 void TestLoading::testLoading()
@@ -1071,7 +818,7 @@ void TestLoading::testSaving_data()
 {
     QTest::addColumn<QString>("testcase");
 
-    addData(TestSavingData);
+    addData();
 }
 
 void TestLoading::testSaving()
@@ -1106,4 +853,4 @@ int main(int argc, char *argv[])
     return QTest::qExec(new TestLoading, argc, argv);
 }
 
-#include <TestLoading.moc>
+#include "TestLoading.moc"
