@@ -1,5 +1,7 @@
 /* This file is part of the KDE project
- * Copyright (C) 2006-2007 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2006-2008 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2008 Thorsten Zachmann <zachmann@kde.org>
+ * Copyright (C) 2008 Girish Ramakrishnan <girish@forwardbias.in>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -24,12 +26,13 @@
 #include <QVariant>
 #include <QString>
 #include <QTextCharFormat>
-#include <kotext_export.h>
+#include "kotext_export.h"
+#include <kdeversion.h>
 
 class StylePrivate;
 class QTextBlock;
 class KoStyleStack;
-class KoTextLoadingContext;
+class KoOdfLoadingContext;
 class KoGenStyle;
 
 /**
@@ -43,22 +46,34 @@ class KoGenStyle;
  * equals styleId() of that style.
  * @see KoStyleManager
  */
-class KOTEXT_EXPORT KoCharacterStyle : public QObject {
+class KOTEXT_EXPORT KoCharacterStyle : public QObject
+{
     Q_OBJECT
 public:
     /// list of character style properties we can store in a QCharFormat
     enum Property {
-        StyleId = QTextFormat::UserProperty+1, ///< The id stored in the charFormat to link the text to this style.
+        StyleId = QTextFormat::UserProperty + 1, ///< The id stored in the charFormat to link the text to this style.
         HasHyphenation,
         FontId,
         StrikeOutStyle,
         StrikeOutType,
         StrikeOutColor,
+        StrikeOutWidth,
+        StrikeOutWeight,
+        StrikeOutMode,
+        StrikeOutText,
         UnderlineStyle,
         UnderlineType,
-        UnderlineColor,
-        TransformText,
-        Spelling
+        UnderlineWidth,
+        UnderlineWeight,
+        UnderlineMode,
+        Spelling,
+        Language,
+        Country,
+        FontCharset,
+        DropCapsAdditionalFormatting, // to identify the purpose of the additionalFormatting
+        // set on the layout. is not copied during applyStyle().
+        InlineInstanceId = 577297549 // Internal: Reserved for KoInlineTextObjectManager
     };
 
     /// list of possible line type : no line, single line, double line
@@ -68,7 +83,7 @@ public:
         DoubleLine
     };
 
-    /// list of possible line style. 
+    /// list of possible line style.
     enum LineStyle {
         NoLineStyle = Qt::NoPen,
         SolidLine = Qt::SolidLine,
@@ -80,13 +95,23 @@ public:
         WaveLine
     };
 
-    /// Text transformation
-    enum Transform {
-        MixedCase = 0, ///< No text-transformation. This is the default value.
-        SmallCaps, ///< Small capitalized letters.
-        AllUppercase, ///< Uppercase characters. E.g. "My teXT"=>"MY TEXT"
-        AllLowercase, ///< Lowercase characters. E.g. "My teXT"=>"my text"
-        Capitalize ///< Capitalize characters. E.g. "my text"=>"My Text"
+    enum LineWeight {
+        AutoLineWeight,
+        NormalLineWeight,
+        BoldLineWeight,
+        ThinLineWeight,
+        DashLineWeight, // ## ??what the heck does this mean??
+        MediumLineWeight,
+        ThickLineWeight,
+        PercentLineWeight,
+        LengthLineWeight
+    };
+
+    /// list of possible line modes.
+    enum LineMode {
+        NoLineMode,
+        ContinuousLineMode,
+        SkipWhiteSpaceLineMode
     };
 
     /**
@@ -95,9 +120,7 @@ public:
      */
     explicit KoCharacterStyle(QObject *parent = 0);
     /// Copy constructor
-    KoCharacterStyle(const KoCharacterStyle &other);
-    /// Copy constructor
-    KoCharacterStyle(const QTextCharFormat &format);
+    KoCharacterStyle(const QTextCharFormat &format, QObject *parent = 0);
     /// Destructor
     ~KoCharacterStyle();
 
@@ -105,37 +128,37 @@ public:
     QFont font() const;
 
     /// See similar named method on QTextCharFormat
-    void setFontFamily (const QString &family);
+    void setFontFamily(const QString &family);
     /// See similar named method on QTextCharFormat
-    QString fontFamily () const;
+    QString fontFamily() const;
     /// See similar named method on QTextCharFormat
-    void setFontPointSize (qreal size);
+    void setFontPointSize(qreal size);
     /// See similar named method on QTextCharFormat
-    double fontPointSize () const;
+    qreal fontPointSize() const;
     /// See similar named method on QTextCharFormat
-    void setFontWeight (int weight);
+    void setFontWeight(int weight);
     /// See similar named method on QTextCharFormat
-    int fontWeight () const;
+    int fontWeight() const;
     /// See similar named method on QTextCharFormat
-    void setFontItalic (bool italic);
+    void setFontItalic(bool italic);
     /// See similar named method on QTextCharFormat
-    bool fontItalic () const;
+    bool fontItalic() const;
     /// See similar named method on QTextCharFormat
-    void setFontOverline (bool overline);
+    void setFontOverline(bool overline);
     /// See similar named method on QTextCharFormat
-    bool fontOverline () const;
+    bool fontOverline() const;
     /// See similar named method on QTextCharFormat
-    void setFontFixedPitch (bool fixedPitch);
+    void setFontFixedPitch(bool fixedPitch);
     /// See similar named method on QTextCharFormat
-    bool fontFixedPitch () const;
+    bool fontFixedPitch() const;
     /// See similar named method on QTextCharFormat
-    void setVerticalAlignment (QTextCharFormat::VerticalAlignment alignment);
+    void setVerticalAlignment(QTextCharFormat::VerticalAlignment alignment);
     /// See similar named method on QTextCharFormat
-    QTextCharFormat::VerticalAlignment verticalAlignment () const;
+    QTextCharFormat::VerticalAlignment verticalAlignment() const;
     /// See similar named method on QTextCharFormat
-    void setTextOutline (const QPen &pen);
+    void setTextOutline(const QPen &pen);
     /// See similar named method on QTextCharFormat
-    QPen textOutline () const;
+    QPen textOutline() const;
     /// See similar named method on QTextCharFormat
     void setFontLetterSpacing(qreal spacing);
     /// See similar named method on QTextCharFormat
@@ -144,58 +167,97 @@ public:
     void setFontWordSpacing(qreal spacing);
     /// See similar named method on QTextCharFormat
     qreal fontWordSpacing() const;
+    /// Set the text capitalization
+    void setFontCapitalization(QFont::Capitalization capitalization);
+    /// Return how the text should be capitalized
+    QFont::Capitalization fontCapitalization() const;
+
+#if QT_VERSION >= KDE_MAKE_VERSION(4,5,0)
+    /// See similar named method on QTextCharFormat (Qt 4.5 and above)
+    void setFontStyleHint(QFont::StyleHint styleHint);
+    /// See similar named method on QTextCharFormat (Qt 4.5 and above)
+    QFont::StyleHint fontStyleHint() const;
+    /// See similar named method on QTextCharFormat (Qt 4.5 and above)
+    void setFontKerning(bool enable);
+    /// See similar named method on QTextCharFormat (Qt 4.5 and above)
+    bool fontKerning() const;
+#endif
+    /// See similar named method on QTextCharFormat
+    void setBackground(const QBrush &brush);
+    /// See similar named method on QTextCharFormat
+    QBrush background() const;
+    /// See similar named method on QTextCharFormat
+    void clearBackground();
 
     /// See similar named method on QTextCharFormat
-    void setBackground (const QBrush &brush);
+    void setForeground(const QBrush &brush);
     /// See similar named method on QTextCharFormat
-    QBrush background () const;
+    QBrush foreground() const;
     /// See similar named method on QTextCharFormat
-    void clearBackground ();
-
-    /// See similar named method on QTextCharFormat
-    void setForeground (const QBrush &brush);
-    /// See similar named method on QTextCharFormat
-    QBrush foreground () const;
-    /// See similar named method on QTextCharFormat
-    void clearForeground ();
+    void clearForeground();
 
     /// Apply a font strike out style to this KoCharacterStyle
-    void setStrikeOutStyle (LineStyle style);
+    void setStrikeOutStyle(LineStyle style);
     /// Get the current font strike out style of this KoCharacterStyle
-    LineStyle strikeOutStyle () const;
+    LineStyle strikeOutStyle() const;
+    /// Apply a font strike out width to this KoCharacterStyle
+    void setStrikeOutWidth(LineWeight weight, qreal width);
+    /// Get the current font strike out width of this KoCharacterStyle
+    void strikeOutWidth(LineWeight& weight, qreal& width) const;
     /// Apply a font strike out color to this KoCharacterStyle
-    void setStrikeOutColor (const QColor &color);
+    void setStrikeOutColor(const QColor &color);
     /// Get the current font strike out color of this KoCharacterStyle
-    QColor strikeOutColor () const;
+    QColor strikeOutColor() const;
     /// Apply a font strike out color to this KoCharacterStyle
-    void setStrikeOutType (LineType lineType);
+    void setStrikeOutType(LineType lineType);
     /// Get the current font strike out color of this KoCharacterStyle
-    LineType strikeOutType () const;
+    LineType strikeOutType() const;
+    /// Apply a strike out mode of this KoCharacterStyle
+    void setStrikeOutMode(LineMode lineMode);
+    /// Get the current strike out mode of this KoCharacterStyle
+    LineMode strikeOutMode() const;
+    /// Apply a strike out text of this KoCharacterStyle
+    void setStrikeOutText(const QString& text);
+    /// Get the current strike out text of this KoCharacterStyle
+    QString strikeOutText() const;
 
     /// Apply a font underline style to this KoCharacterStyle
-    void setUnderlineStyle (LineStyle style);
+    void setUnderlineStyle(LineStyle style);
     /// Get the current font underline style of this KoCharacterStyle
-    LineStyle underlineStyle () const;
+    LineStyle underlineStyle() const;
+    /// Apply a font underline width to this KoCharacterStyle
+    void setUnderlineWidth(LineWeight weight, qreal width);
+    /// Get the current font underline width of this KoCharacterStyle
+    void underlineWidth(LineWeight& weight, qreal& width) const;
     /// Apply a font underline color to this KoCharacterStyle
-    void setUnderlineColor (const QColor &color);
+    void setUnderlineColor(const QColor &color);
     /// Get the current font underline color of this KoCharacterStyle
-    QColor underlineColor () const;
+    QColor underlineColor() const;
     /// Apply a font underline color to this KoCharacterStyle
-    void setUnderlineType (LineType lineType);
+    void setUnderlineType(LineType lineType);
     /// Get the current font underline color of this KoCharacterStyle
-    LineType underlineType () const;
+    LineType underlineType() const;
+    /// Apply a underline mode to this KoCharacterStyle
+    void setUnderlineMode(LineMode mode);
+    /// Get the current underline mode of this KoCharacterStyle
+    LineMode underlineMode() const;
 
-
-    /// Set the text tranformation.
-    void setTransform(Transform transformtext);
-    /// Return how the text should be transformed.
-    Transform transform() const;
+    /// Set the country
+    void setCountry(const QString &country);
+    /// Set the language
+    void setLanguage(const QString &language);
+    /// Get the country
+    QString country() const;
+    /// Get the language
+    QString language() const;
 
     void setHasHyphenation(bool on);
     bool hasHyphenation() const;
 
     void copyProperties(const KoCharacterStyle *style);
+    void copyProperties(const QTextCharFormat &format);
 
+    KoCharacterStyle *clone(QObject *parent = 0);
 
     /// return the name of the style.
     QString name() const;
@@ -228,16 +290,41 @@ public:
      * Load the style from the \a KoStyleStack style stack using the
      * OpenDocument format.
      */
-    void loadOasis(KoTextLoadingContext& context);
+    void loadOdf(KoOdfLoadingContext& context);
 
     /// return true if this style has a non-default value set for the Property
     bool hasProperty(int key) const;
 
-    bool operator==( const KoCharacterStyle &other ) const;
-    
-    void removeDuplicates ( const KoCharacterStyle &other );
-    
-    void saveOdf ( KoGenStyle *target );
+    bool operator==(const KoCharacterStyle &other) const;
+
+    /*
+     * Removes properties from this style that have the same value in other style.
+     */
+    void removeDuplicates(const KoCharacterStyle &other);
+
+    /*
+     * Removes properties from this style that have the same value in other format.
+     */
+    void removeDuplicates(const QTextCharFormat &other_format);
+
+    void saveOdf(KoGenStyle &style);
+
+    /**
+     * Returns true if this style has no properties set. Else, returns false.
+     */
+    bool isEmpty() const;
+
+    /**
+     * Return the value of key as represented on this style.
+     * You should consider using the direct accessors for individual properties instead.
+     * @param key the Property to request.
+     * @returns a QVariant which holds the property value.
+     */
+    QVariant value(int key) const;
+
+signals:
+    void nameChanged(const QString &newName);
+
 private:
     class Private;
     Private * const d;
