@@ -1,5 +1,7 @@
 /* This file is part of the KDE project
- * Copyright (C) 2006, 2007 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2006-2008 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2008 Thorsten Zachmann <zachmann@kde.org>
+ * Copyright (C) 2008 Girish Ramakrishnan <girish@forwardbias.in>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -33,6 +35,8 @@ class KoListStyle;
 class QTextBlock;
 class KoStyleStack;
 class KoGenStyle;
+class KoXmlElement;
+class KoOdfLoadingContext;
 
 /**
  * A container for all properties for the paragraph wide style.
@@ -42,11 +46,12 @@ class KoGenStyle;
  * a specific KoParagraphStyle.
  * @see KoStyleManager
  */
-class KOTEXT_EXPORT KoParagraphStyle : public QObject {
+class KOTEXT_EXPORT KoParagraphStyle : public QObject
+{
     Q_OBJECT
 public:
     enum Property {
-        StyleId = QTextFormat::UserProperty+1,
+        StyleId = QTextFormat::UserProperty + 1,
         // Linespacing properties
         PercentLineHeight,  ///< this propery is used for a percentage of the highest character on that line
         FixedLineHeight,    ///< this propery is used to use a non-default line height
@@ -60,6 +65,7 @@ public:
         DropCapsLength, ///< Number of glyphs to show as drop-caps
         DropCapsLines,  ///< Number of lines that the drop-caps span
         DropCapsDistance,   ///< Distance between drop caps and text
+        DropCapsTextStyle,  ///< Text style of dropped chars. Not supported yet.
         FollowDocBaseline,  ///< If true the baselines will be aligned with the doc-wide grid
         BreakBefore,    ///< If true, insert a frame break before this paragraph
         BreakAfter,     ///< If true, insert a frame break after this paragraph
@@ -98,13 +104,23 @@ public:
         BottomBorderColor,      ///< The border Color
 
         // lists
-        ExplicitListValue,      ///< Int with the list-value that that parag will have. Ignored if this is not a list.
+        ListStyleId,            ///< Style Id of associated list style
+        ListStartValue,         ///< Int with the list-value that that parag will have. Ignored if this is not a list.
         RestartListNumbering,   ///< boolean to indicate that this paragraph will have numbering restart at the list-start. Ignored if this is not a list.
-        ListLevel,               ///< int with the list-level that the paragraph will get when this is a list
+        ListLevel,               ///< int with the list-level that the paragraph will get when this is a list (numbered paragraphs)
         IsListHeader,           ///< bool, if true the paragraph shows up as a list item, but w/o a list label.
+        UnnumberedListItem,     ///< bool. if true this paragraph is part of a list but is not numbered
 
-        TabPositions,            ///< A list of tab positions
-        TextProgressionDirection
+        AutoTextIndent,         ///< bool, says whether the paragraph is auto-indented or not
+
+        TabStopDistance,        ///< Double, Length. specifies that there's a tab stop every n inches
+        ///< (after the last of the TabPositions, if any)
+        TabPositions,           ///< A list of tab positions
+        TextProgressionDirection,
+
+        MasterPageName,         ///< Optional name of the master-page
+        
+        OutlineLevel            ///< Outline level for headings
 // do 15.5.24
 // continue at 15.5.28
     };
@@ -127,14 +143,18 @@ public:
     };
 
     /// Constructor
-    KoParagraphStyle();
-    /// Copy constructor
-    KoParagraphStyle(const KoParagraphStyle &orig);
-    /// Copy constructor
-    KoParagraphStyle(const QTextFormat &textFormat);
+    KoParagraphStyle(QObject *parent = 0);
+    /// Creates a KoParagraphStyle with the given block format, the block character format and \a parent
+    KoParagraphStyle(const QTextBlockFormat &blockFormat, const QTextCharFormat &blockCharFormat, QObject *parent = 0);
     /// Destructor
     ~KoParagraphStyle();
 
+    /// Creates a KoParagraphStyle that represents the formatting of \a block.
+    static KoParagraphStyle *fromBlock(const QTextBlock &block, QObject *parent = 0);
+
+    /// creates a clone of this style with the specified parent
+    KoParagraphStyle *clone(QObject *parent = 0);
+
     //  ***** Linespacing
     /**
      * Sets the line height as a percentage of the highest character on that line.
@@ -152,18 +172,18 @@ public:
      * Sets the line height to a specific pt-based height, ignoring the font size.
      * Setting this will ignore the lineHeightPercent() and lineSpacing() values.
      */
-    void setLineHeightAbsolute(double height);
+    void setLineHeightAbsolute(qreal height);
     /// @see setLineHeightAbsolute
-    double lineHeightAbsolute() const;
+    qreal lineHeightAbsolute() const;
 
     /**
      * Sets the line height to have a minimum height in pt.
      * You should consider doing a remove(KoParagraphStyle::FixedLineHeight); because if set, it will
      *  be used instead of this value.
      */
-    void setMinimumLineHeight(double height);
+    void setMinimumLineHeight(qreal height);
     /// @see setMinimumLineHeight
-    double minimumLineHeight() const;
+    qreal minimumLineHeight() const;
 
     /**
      * Sets the space between two lines to be a specific height. The total linespacing will become
@@ -171,9 +191,9 @@ public:
      * You should consider doing a remove(KoParagraphStyle::FixedLineHeight) and a
      * remove(KoParagraphStyle::PercentLineHeight); because if set, they will be used instead of this value.
      */
-    void setLineSpacing(double spacing);
+    void setLineSpacing(qreal spacing);
     /// @see setLineSpacing
-    double lineSpacing() const;
+    qreal lineSpacing() const;
 
     /**
      * If set to true the font-encoded height will be used instead of the font-size propery
@@ -194,7 +214,7 @@ public:
      * There are only 3 valid options, Left, Center and Justified. (where Left will
      * be right aligned for RTL text).
      */
-    void setAlignLastLine(Qt::Alignment alignment );
+    void setAlignLastLine(Qt::Alignment alignment);
     /**
      * @see setAlignLastLine
      */
@@ -266,12 +286,12 @@ public:
      * @see setDropCaps
      * @see setDropCapsLines
      */
-    void setDropCapsDistance(double distance);
+    void setDropCapsDistance(qreal distance);
     /**
      * Set dropCaps distance
      * @see setDropCapsDistance
      */
-    double dropCapsDistance() const;
+    qreal dropCapsDistance() const;
     /**
      * If true the baselines will be aligned with the doc-wide grid
      */
@@ -283,62 +303,62 @@ public:
     bool followDocBaseline() const;
 
     /// See similar named method on QTextBlockFormat
-    void setBackground (const QBrush &brush);
+    void setBackground(const QBrush &brush);
     /// See similar named method on QTextBlockFormat
-    QBrush background () const;
+    QBrush background() const;
     /// See similar named method on QTextBlockFormat
-    void clearBackground ();
-    
+    void clearBackground();
+
     void setBreakBefore(bool on);
     bool breakBefore();
     void setBreakAfter(bool on);
     bool breakAfter();
-    void setLeftPadding(double padding);
-    double leftPadding();
-    void setTopPadding(double padding);
-    double topPadding();
-    void setRightPadding(double padding);
-    double rightPadding();
-    void setBottomPadding(double padding);
-    double bottomPadding();
-    void setPadding(double padding);
+    void setLeftPadding(qreal padding);
+    qreal leftPadding();
+    void setTopPadding(qreal padding);
+    qreal topPadding();
+    void setRightPadding(qreal padding);
+    qreal rightPadding();
+    void setBottomPadding(qreal padding);
+    qreal bottomPadding();
+    void setPadding(qreal padding);
 
-    void setLeftBorderWidth(double width);
-    double leftBorderWidth();
-    void setLeftInnerBorderWidth(double width);
-    double leftInnerBorderWidth();
-    void setLeftBorderSpacing(double width);
-    double leftBorderSpacing();
+    void setLeftBorderWidth(qreal width);
+    qreal leftBorderWidth();
+    void setLeftInnerBorderWidth(qreal width);
+    qreal leftInnerBorderWidth();
+    void setLeftBorderSpacing(qreal width);
+    qreal leftBorderSpacing();
     void setLeftBorderStyle(BorderStyle style);
     BorderStyle leftBorderStyle();
     void setLeftBorderColor(const QColor& color);
     QColor leftBorderColor();
-    void setTopBorderWidth(double width);
-    double topBorderWidth();
-    void setTopInnerBorderWidth(double width);
-    double topInnerBorderWidth();
-    void setTopBorderSpacing(double width);
-    double topBorderSpacing();
+    void setTopBorderWidth(qreal width);
+    qreal topBorderWidth();
+    void setTopInnerBorderWidth(qreal width);
+    qreal topInnerBorderWidth();
+    void setTopBorderSpacing(qreal width);
+    qreal topBorderSpacing();
     void setTopBorderStyle(BorderStyle style);
     BorderStyle topBorderStyle();
     void setTopBorderColor(const QColor& color);
     QColor topBorderColor();
-    void setRightBorderWidth(double width);
-    double rightBorderWidth();
-    void setRightInnerBorderWidth(double width);
-    double rightInnerBorderWidth();
-    void setRightBorderSpacing(double width);
-    double rightBorderSpacing();
+    void setRightBorderWidth(qreal width);
+    qreal rightBorderWidth();
+    void setRightInnerBorderWidth(qreal width);
+    qreal rightInnerBorderWidth();
+    void setRightBorderSpacing(qreal width);
+    qreal rightBorderSpacing();
     void setRightBorderStyle(BorderStyle style);
     BorderStyle rightBorderStyle();
     void setRightBorderColor(const QColor& color);
     QColor rightBorderColor();
-    void setBottomBorderWidth(double width);
-    double bottomBorderWidth();
-    void setBottomInnerBorderWidth(double width);
-    double bottomInnerBorderWidth();
-    void setBottomBorderSpacing(double width);
-    double bottomBorderSpacing();
+    void setBottomBorderWidth(qreal width);
+    qreal bottomBorderWidth();
+    void setBottomInnerBorderWidth(qreal width);
+    qreal bottomInnerBorderWidth();
+    void setBottomBorderSpacing(qreal width);
+    qreal bottomBorderSpacing();
     void setBottomBorderStyle(BorderStyle style);
     BorderStyle bottomBorderStyle();
     void setBottomBorderColor(const QColor& color);
@@ -349,41 +369,45 @@ public:
 
     // ************ properties from QTextBlockFormat
     /// duplicated property from QTextBlockFormat
-    void setTopMargin(double topMargin);
+    void setTopMargin(qreal topMargin);
     /// duplicated property from QTextBlockFormat
-    double topMargin() const;
+    qreal topMargin() const;
     /// duplicated property from QTextBlockFormat
-    void setBottomMargin (double margin);
+    void setBottomMargin(qreal margin);
     /// duplicated property from QTextBlockFormat
-    double bottomMargin () const;
+    qreal bottomMargin() const;
     /// duplicated property from QTextBlockFormat
-    void setLeftMargin (double margin);
+    void setLeftMargin(qreal margin);
     /// duplicated property from QTextBlockFormat
-    double leftMargin () const;
+    qreal leftMargin() const;
     /// duplicated property from QTextBlockFormat
-    void setRightMargin (double margin);
+    void setRightMargin(qreal margin);
     /// duplicated property from QTextBlockFormat
-    double rightMargin () const;
+    qreal rightMargin() const;
     /// set the margin around the paragraph, making the margin on all sides equal.
-    void setMargin (double margin);
+    void setMargin(qreal margin);
 
     void setIsListHeader(bool on);
     bool isListHeader() const;
 
     /// duplicated property from QTextBlockFormat
-    void setAlignment (Qt::Alignment alignment);
+    void setAlignment(Qt::Alignment alignment);
     /// duplicated property from QTextBlockFormat
-    Qt::Alignment alignment () const;
+    Qt::Alignment alignment() const;
     /// duplicated property from QTextBlockFormat
-    void setTextIndent (double margin);
+    void setTextIndent(qreal margin);
     /// duplicated property from QTextBlockFormat
-    double textIndent () const;
+    qreal textIndent() const;
+    /// Custom KoParagraphStyle property for auto-text-indent
+    void setAutoTextIndent(bool on);
+    bool autoTextIndent() const;
+
 #if 0
-as this is a duplicate of leftMargin, lets make it very clear we are using that one.
+    as this is a duplicate of leftMargin, lets make it very clear we are using that one.
     /// duplicated property from QTextBlockFormat
-    void setIndent (int indent);
+    void setIndent(int indent);
     /// duplicated property from QTextBlockFormat
-    int indent () const;
+    int indent() const;
 #endif
     /// duplicated property from QTextBlockFormat
     void setNonBreakableLines(bool on);
@@ -391,10 +415,10 @@ as this is a duplicate of leftMargin, lets make it very clear we are using that
     bool nonBreakableLines() const;
 
     /// set the parent style this one inherits its unset properties from.
-    void setParent(KoParagraphStyle *parent);
+    void setParentStyle(KoParagraphStyle *parent);
 
     /// return the parent style
-    KoParagraphStyle *parent() const;
+    KoParagraphStyle *parentStyle() const;
 
     /// the 'next' style is the one used when the user creates a new paragrap after this one.
     void setNextStyle(int next);
@@ -403,7 +427,7 @@ as this is a duplicate of leftMargin, lets make it very clear we are using that
     int nextStyle() const;
 
     /// return the name of the style.
-    const QString& name() const;
+    QString name() const;
 
     /// set a user-visible name on the style.
     void setName(const QString &name);
@@ -414,11 +438,26 @@ as this is a duplicate of leftMargin, lets make it very clear we are using that
     /// each style has a unique ID (non persistent) given out by the styleManager
     void setStyleId(int id);
 
+    /// return the optional name of the master-page or a QString() if this paragraph isn't attached to a master-page.
+    QString masterPageName() const;
+    /// Set the name of the master-page.
+    void setMasterPageName(const QString& name);
+
+
+    /// Set the list start value
+    void setListStartValue(int value);
+    /// Returns the list start value
+    int listStartValue() const;
+
     /// Set to true if this paragraph is marked to start the list numbering from the first entry.
     void setRestartListNumbering(bool on);
     /// return if this paragraph is marked to start the list numbering from the first entry.
     bool restartListNumbering();
 
+    /// Set the tab stop distance for this paragraph style.
+    void setTabStopDistance(qreal value);
+    /// return the tab stop distance for this paragraph style
+    qreal tabStopDistance() const;
     /// Set the tab data for this paragraph style.
     void setTabPositions(const QList<KoText::Tab> &tabs);
     /// return the tabs data for this paragraph style
@@ -429,6 +468,18 @@ as this is a duplicate of leftMargin, lets make it very clear we are using that
     /// return the list level.
     int listLevel() const;
 
+    /**
+     * Return the outline level of this block, or 0 if it's not a heading.
+     * This information is here and not in the styles because the OpenDocument specification says so.
+     * See ODF Spec 1.1, §14.1, Outline Numbering Level, but also other parts of the specification.
+     */
+    int outlineLevel() const;
+
+    /**
+     * Change this block outline level
+     */
+    void setOutlineLevel(int outline);
+
     /// copy all the properties from the other style to this style, effectively duplicating it.
     void copyProperties(const KoParagraphStyle *style);
 
@@ -443,7 +494,7 @@ as this is a duplicate of leftMargin, lets make it very clear we are using that
      * Apply this style to the textBlock by copying all properties from this, parent and
      * the character style (where relevant) to the target block formats.
      */
-    void applyStyle(QTextBlock &block) const;
+    void applyStyle(QTextBlock &block, bool applyListStyle = true) const;
 
     /// return the character style for this paragraph style
     KoCharacterStyle *characterStyle();
@@ -458,42 +509,72 @@ as this is a duplicate of leftMargin, lets make it very clear we are using that
      * @see setListStyle()
      * @see removeListStyle()
      */
-    KoListStyle listStyle() const;
+    KoListStyle *listStyle() const;
     /**
      * Set a new liststyle on this paragraph style, making all paragraphs that use this style
      *  automatically be part of the list.
      * @see setListStyle()
      * @see removeListStyle()
      */
-    void setListStyle(const KoListStyle &style);
-    /**
-     * Remove any set list style on this paragraphs style.
-     * Stops all paragraphs that follow this style from being a list item.
-     */
-    void removeListStyle();
+    void setListStyle(KoListStyle *style);
 
+    void remove(int key);
+
+    /// Compare the paragraph, character and list properties of this style with the other
+    bool operator==(const KoParagraphStyle &other) const;
+    /// Compare the paragraph properties of this style with other
+    bool compareParagraphProperties(const KoParagraphStyle &other) const;
+    /// Compare the character properties of this style with other
+    bool compareCharacterProperties(const KoParagraphStyle &other) const;
+
+    void removeDuplicates(const KoParagraphStyle &other);
+
+    /**
+     * Load the style form the element
+     *
+     * @param context the odf loading context
+     * @param element the element containing the
+     */
+    void loadOdf(const KoXmlElement* element, KoOdfLoadingContext & context);
+
+    void saveOdf(KoGenStyle &style);
+
+    /**
+     * Returns true if this paragraph style has the property set.
+     * Note that this method does not delegate to the parent style.
+     * @param key the key as found in the Property enum
+     */
+    bool hasProperty(int key) const;
+
+    /**
+     * Set a property with key to a certain value, overriding the value from the parent style.
+     * If the value set is equal to the value of the parent style, the key will be removed instead.
+     * @param key the Property to set.
+     * @param value the new value to set on this style.
+     * @see hasProperty(), value()
+     */
+    void setProperty(int key, const QVariant &value);
+    /**
+     * Return the value of key as represented on this style, taking into account parent styles.
+     * You should consider using the direct accessors for individual properties instead.
+     * @param key the Property to request.
+     * @returns a QVariant which holds the property value.
+     */
+    QVariant value(int key) const;
+
+signals:
+    void nameChanged(const QString &newName);
+
+private:
     /**
      * Load the style from the \a KoStyleStack style stack using the
      * OpenDocument format.
      */
-    void loadOasis(KoStyleStack& styleStack);
-
-    static KoParagraphStyle *fromBlock(const QTextBlock &block);
-
-    void remove(int key);
-    
-    bool operator==( const KoParagraphStyle &other ) const;
-    
-    void removeDuplicates ( const KoParagraphStyle &other );
-    
-    void saveOdf ( KoGenStyle *target );
-private:
-    void setProperty(int key, const QVariant &value);
-    double propertyDouble(int key) const;
+    void loadOdfProperties(KoStyleStack& styleStack);
+    qreal propertyDouble(int key) const;
     int propertyInt(int key) const;
     bool propertyBoolean(int key) const;
     QColor propertyColor(int key) const;
-    QVariant value(int key) const;
 
     class Private;
     Private * const d;
