@@ -22,11 +22,11 @@
 #ifndef koGlobal_h
 #define koGlobal_h
 
-#include <QStringList>
-#include <QFont>
-#include <QMap>
+#include <QtCore/QStringList>
+#include <QtGui/QFont>
+#include <QtCore/QMap>
 
-#include <komain_export.h>
+#include "komain_export.h"
 
 class KConfig;
 
@@ -64,7 +64,7 @@ public:
         return self()->m_dpiY;
     }
     /// @internal, for KoApplication
-    static void setDPI( int x, int y );
+    static void setDPI(int x, int y);
 
     /// Return the list of available languages, in their displayable form
     /// (translated names)
@@ -77,9 +77,9 @@ public:
         return self()->_listOfLanguageTags();
     }
     /// For a given language display name, return its tag
-    static QString tagOfLanguage( const QString & _lang );
+    static QString tagOfLanguage(const QString & _lang);
     /// For a given language tag, return its display name
-    static QString languageFromTag( const QString &_lang );
+    static QString languageFromTag(const QString &_lang);
 
     ~KoGlobal();
 
@@ -100,9 +100,6 @@ private:
     int m_dpiY;
     // No BC problem here, constructor is private, feel free to add members
 
-    // Singleton pattern. Maybe this should even be refcounted, so
-    // that it gets cleaned up when closing all koffice parts in e.g. konqueror?
-    static KoGlobal* s_global;
     friend class this_is_a_singleton; // work around gcc warning
 };
 
