@@ -1,12 +1,10 @@
 /* This file is part of the KDE project
    Copyright (C) 2004-2006 David Faure <faure@kde.org>
    Copyright (C) 2007-2008 Thorsten Zachmann <zachmann@kde.org>
-   Copyright (C) 2009 Inge Wallin <inge@lysator.liu.se>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   License version 2 as published by the Free Software Foundation.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,7 +23,6 @@
 #include <QtCore/QList>
 #include <QtCore/QMap>
 #include <QtCore/QString>
-#include <iostream>
 #include "koodf_export.h"
 
 class KoGenStyles;
@@ -43,20 +40,18 @@ class KOODF_EXPORT KoGenStyle
 public:
     /**
      * Possible values for the "type" argument of the KoGenStyle constructor.
-     * @note If there is still something missing, add it here so that it is possible to use the same
-     *       saving code in all applications.
+     * If there is a still missing add it here so that it is possible to use the same
+     * saving code in all applications.
      */
     enum Type {
         StylePageLayout,             ///< style:page-layout as in odf 14.3 Page Layout
         StyleUser,                   ///< style:style with style:paragraph-properties as in odf 14.1 Style Element (office:styles)
-        StyleAuto,                   ///< style:style with style:paragraph-properties as in odf 14.1 Style Element (office:automatic-styles)
+        StyleAuto,                   ///< style:style with style:paragraph-properties as in odf 14.1 Style Element
         StyleText,                   ///< style:style with style:text-properties as in odf 14.8.1 Text Styles (office:styles)
-        StyleTextAuto,               ///< style:style with style:text-properties as in odf 14.8.1 Text Styles (office:automatic-styles)
-        StyleChart,                  ///< style:style with style:chart-properties
-        StyleChartAuto,              ///< style:style with style:chart-properties
+        StyleTextAuto,               ///< style:style with style:text-properties as in odf 14.8.1 Text Styles
         StyleList,                   ///< text:list-style as in odf 14.10 List Style (office:styles)
-        StyleListAuto,               ///< text:list-style as in odf 14.10 List Style (office:automatic-styles)
-        StyleSectionAuto,            ///< style:style with style:section-properties as in odf 14.8.3 Section Styles (office:automatic-styles)
+        StyleListAuto,               ///< text:list-style as in odf 14.10 List Style
+        StyleSectionAuto,            ///< style:style with style:section-properties as in odf 14.8.3 Section Styles
         StyleNumericNumber,          ///< number:number-style as in odf 14.7.1 Number Style
         StyleNumericDate,            ///< number:date-style as in odf 14.7.4 Date Style
         StyleNumericTime,            ///< number:time-style as in odf 14.7.5 Time Style
@@ -66,28 +61,25 @@ public:
         StyleNumericCurrency,        ///< number:currency-style as in odf 14.7.2 Currency Style
         StyleNumericText,            ///< number:text-style 14.7.7 Text Style not used
         StyleHatch,                  ///< draw:hatch as in odf 14.14.3 Hatch (office:styles)
-        StyleGraphicAuto,            ///< style:style with style:graphic-properties as in 14.13.1 Graphic and Presentation Styles (office:automatic-styles)
-        StylePresentationAuto,       ///< style:style with style:graphic-properties as in 14.13.1 Graphic and Presentation Styles (office:automatic-styles)
+        StyleGraphicAuto,            ///< style:style with style:graphic-properties as in 14.13.1 Graphic and Presentation Styles
+        StylePresentationAuto,       ///< style:style with style:graphic-properties as in 14.13.1 Graphic and Presentation Styles
         StyleStrokeDash,             ///< draw:stroke-dash as in odf 14.14.7 Stroke Dash (office:styles)
         StyleGradient,               ///< draw:gradient as in odf 14.14.1 Gradient (office:styles)
         StyleGradientLinear,         ///< svg:linearGradient as in odf 14.14.2 SVG Gradients (office:styles)
-        StyleGradientRadial,         ///< svg:radialGradient as in odf 14.14.2 SVG Gradients (office:styles)
-        StyleGradientConical,        ///< koffice:conicalGradient koffice extension for conical gradients 
+        StyleGradientRadial,         ///< svg-radialGradient as in odf 14.14.2 SVG Gradients (office:styles)
         StyleFillImage,              ///< draw:fill-image as in odf 14.14.4 Fill Image (office:styles)
         StyleDrawingPage,            ///< style:drawing-page-properties as in odf 14.13.2 Drawing Page Style
         StyleNumericBoolean,         ///< number:boolean 14.7.6 Boolean Style not used
         StyleOpacity,                ///< draw:opacity as in odf 14.14.5 Opacity Gradient not used
         StyleMarker,                 ///< draw:marker as in odf 14.14.6 Marker
         StylePresentationPageLayout, ///< style:presentation-page-layout as in odf 14.15 Presentation Page Layouts
-        StyleAutoTable,              ///< style:table-properties as in odf 15.8 Table Formatting Properties (office:automatic-styles)
-// office:style needed?
+        StyleAutoTable,              ///< style:table-properties as in odf 15.8 Table Formatting Properties // office:style needed?
         StyleTableColumn,            ///< style:table-column-properties as in odf 15.9 Column Formatting Properties (office:style)
-        StyleAutoTableColumn,        ///< style:table-column-properties as in odf 15.9 Column Formatting Properties (office:automatic-styles)
+        StyleAutoTableColumn,        ///< style:table-column-properties as in odf 15.9 Column Formatting Properties
         StyleTableRow,               ///< style:table-row-properties as in odf 15.10 Table Row Formatting Properties (office:style)
-        StyleAutoTableRow,           ///< style:table-row-properties as in odf 15.10 Table Row Formatting Properties (office:automatic-styles)
+        StyleAutoTableRow,           ///< style:table-row-properties as in odf 15.10 Table Row Formatting Properties
         StyleTableCell,              ///< style:table-cell-properties as in odf 15.11 Table Cell Formatting Properties (office:style)
-        StyleAutoTableCell,          ///< style:table-cell-properties as in odf 15.11 Table Cell Formatting Properties (office:automatic-styles)
-        StyleTable,                  ///< style:table-properties as in odf1.2 16.9 Table Formatting Properties (office:style)
+        StyleAutoTableCell,          ///< style:table-cell-properties as in odf 15.11 Table Cell Formatting Properties
         //   TODO differently
         StyleMaster                  ///< 14.4 Master Pages
         /// style:default-style as in odf 14.2 Default Styles
@@ -150,11 +142,6 @@ public:
         return m_familyName.data();
     }
 
-    /// Sets the name of style's parent.
-    void setParentName(const QString& name) {
-        m_parentName = name;
-    }
-
     /// Return the name of style's parent, if set
     QString parentName() const {
         return m_parentName;
@@ -273,7 +260,7 @@ public:
      * The value of @p elementName isn't used, except that it must be unique.
      */
     void addChildElement(const QString& elementName, const QString& elementContents) {
-        m_properties[ChildElement].insertMulti(elementName, elementContents);
+        m_properties[ChildElement].insert(elementName, elementContents);
     }
 
     /**
@@ -349,7 +336,6 @@ private:
 private:
     // Note that the copy constructor and assignment operator are allowed.
     // Better not use pointers below!
-    // TODO turn this into a QSharedData class
     Type m_type;
     QByteArray m_familyName;
     QString m_parentName;
