@@ -20,41 +20,38 @@
 #include <QByteArray>
 #include <QBitArray>
 
+
 #include "KoColorSpace.h"
 #include "KoChannelInfo.h"
-#include <kdebug.h>
-
+#include "DebugPigment.h"
 #include "KoCompositeOp.h"
 #include "KoColorTransformation.h"
 #include "KoColorTransformationFactory.h"
 #include "KoColorTransformationFactoryRegistry.h"
+#include "KoColorConversionCache.h"
 #include "KoColorConversionSystem.h"
 #include "KoColorSpaceRegistry.h"
+#include "KoColorProfile.h"
 #include "KoCopyColorConversionTransformation.h"
 #include "KoFallBackColorTransformation.h"
+#include "KoUniqueNumberForIdServer.h"
 
 struct KoColorSpace::Private {
     QString id;
+    quint32 idNumber;
     QString name;
     QHash<QString, KoCompositeOp *> compositeOps;
-    KoColorSpaceRegistry * parent;
     QList<KoChannelInfo *> channels;
     KoMixColorsOp* mixColorsOp;
     KoConvolutionOp* convolutionOp;
     QThreadStorage< QVector<quint8>* > conversionCache;
 
-    mutable const KoColorSpace *lastUsedDstColorSpace;
-    mutable KoColorConversionTransformation* lastUsedTransform;
-    
+
     mutable KoColorConversionTransformation* transfoToRGBA16;
     mutable KoColorConversionTransformation* transfoFromRGBA16;
     mutable KoColorConversionTransformation* transfoToLABA16;
     mutable KoColorConversionTransformation* transfoFromLABA16;
 
-// cmsHTRANSFORM is a void *, so this should work.
-    typedef QMap<const KoColorSpace *, KoColorConversionTransformation*>  TransformMap;
-    mutable TransformMap transforms; // Cache for existing transforms
-
 };
 
 KoColorSpace::KoColorSpace()
@@ -62,16 +59,14 @@ KoColorSpace::KoColorSpace()
 {
 }
 
-KoColorSpace::KoColorSpace(const QString &id, const QString &name, KoColorSpaceRegistry * parent, KoMixColorsOp* mixColorsOp, KoConvolutionOp* convolutionOp )
+KoColorSpace::KoColorSpace(const QString &id, const QString &name, KoMixColorsOp* mixColorsOp, KoConvolutionOp* convolutionOp )
     : d (new Private())
 {
     d->id = id;
+    d->idNumber = KoUniqueNumberForIdServer::instance()->numberForId( d->id );
     d->name = name;
-    d->parent = parent;
     d->mixColorsOp = mixColorsOp;
     d->convolutionOp = convolutionOp;
-    d->lastUsedDstColorSpace = 0;
-    d->lastUsedTransform = 0;
     d->transfoToRGBA16 = 0;
     d->transfoFromRGBA16 = 0;
     d->transfoToLABA16 = 0;
@@ -80,11 +75,26 @@ KoColorSpace::KoColorSpace(const QString &id, const QString &name, KoColorSpaceR
 
 KoColorSpace::~KoColorSpace()
 {
+    qDeleteAll(d->compositeOps);
+    foreach(KoChannelInfo * channel, d->channels)
+    {
+        delete channel;
+    }
+    KoColorConversionCache* cache = KoColorSpaceRegistry::instance()->colorConversionCache();
+    if (cache) {
+        cache->colorSpaceIsDestroyed(this);
+    }
     delete d->mixColorsOp;
     delete d->convolutionOp;
     delete d;
 }
 
+bool KoColorSpace::operator==(const KoColorSpace& rhs) const {
+    const KoColorProfile* p1 = rhs.profile();
+    const KoColorProfile* p2 = profile();
+    return d->idNumber == rhs.d->idNumber && ((p1 == p2) || (*p1 == *p2 ));
+}
+
 QString KoColorSpace::id() const {return d->id;}
 
 QString KoColorSpace::name() const {return d->name;}
@@ -146,7 +156,7 @@ quint8 *KoColorSpace::allocPixelBuffer(quint32 numPixels) const
     return new quint8[pixelSize()*numPixels];
 }
 
-QList<KoCompositeOp*> KoColorSpace::userVisiblecompositeOps() const
+QList<KoCompositeOp*> KoColorSpace::compositeOps() const
 {
     return d->compositeOps.values();
 }
@@ -165,8 +175,10 @@ const KoCompositeOp * KoColorSpace::compositeOp(const QString & id) const
 {
     if ( d->compositeOps.contains( id ) )
         return d->compositeOps.value( id );
-    else
+    else {
+        warnPigment << "Asking for non-existent composite operation " << id << ", returning " << COMPOSITE_OVER;
         return d->compositeOps.value( COMPOSITE_OVER );
+    }
 }
 
 void KoColorSpace::addCompositeOp(const KoCompositeOp * op)
@@ -178,33 +190,34 @@ void KoColorSpace::addCompositeOp(const KoCompositeOp * op)
 
 const KoColorConversionTransformation* KoColorSpace::toLabA16Converter() const
 {
-    if(not d->transfoToLABA16)
+    if(!d->transfoToLABA16)
     {
-        d->transfoToLABA16 = d->parent->colorConversionSystem()->createColorConverter(this, KoColorSpaceRegistry::instance()->lab16("") ) ;
+        d->transfoToLABA16 = KoColorSpaceRegistry::instance()->colorConversionSystem()->createColorConverter(this, KoColorSpaceRegistry::instance()->lab16("") ) ;
     }
     return d->transfoToLABA16;
 }
+
 const KoColorConversionTransformation* KoColorSpace::fromLabA16Converter() const
 {
-    if(not d->transfoFromLABA16)
+    if(!d->transfoFromLABA16)
     {
-        d->transfoFromLABA16 = d->parent->colorConversionSystem()->createColorConverter( KoColorSpaceRegistry::instance()->lab16("") , this ) ;
+        d->transfoFromLABA16 = KoColorSpaceRegistry::instance()->colorConversionSystem()->createColorConverter( KoColorSpaceRegistry::instance()->lab16("") , this ) ;
     }
     return d->transfoFromLABA16;
 }
 const KoColorConversionTransformation* KoColorSpace::toRgbA16Converter() const
 {
-    if(not d->transfoToRGBA16)
+    if(!d->transfoToRGBA16)
     {
-        d->transfoToRGBA16 = d->parent->colorConversionSystem()->createColorConverter( this, KoColorSpaceRegistry::instance()->rgb16("") ) ;
+        d->transfoToRGBA16 = KoColorSpaceRegistry::instance()->colorConversionSystem()->createColorConverter( this, KoColorSpaceRegistry::instance()->rgb16("") ) ;
     }
     return d->transfoToRGBA16;
 }
 const KoColorConversionTransformation* KoColorSpace::fromRgbA16Converter() const
 {
-    if(not d->transfoFromRGBA16)
+    if(!d->transfoFromRGBA16)
     {
-        d->transfoFromRGBA16 = d->parent->colorConversionSystem()->createColorConverter( KoColorSpaceRegistry::instance()->rgb16("") , this ) ;
+        d->transfoFromRGBA16 = KoColorSpaceRegistry::instance()->colorConversionSystem()->createColorConverter( KoColorSpaceRegistry::instance()->rgb16("") , this ) ;
     }
     return d->transfoFromRGBA16;
 }
@@ -231,11 +244,11 @@ void KoColorSpace::fromRgbA16(const quint8 * src, quint8 * dst, quint32 nPixels)
 
 KoColorConversionTransformation* KoColorSpace::createColorConverter(const KoColorSpace * dstColorSpace, KoColorConversionTransformation::Intent renderingIntent) const
 {
-    if( this == dstColorSpace)
+    if( *this == *dstColorSpace)
     {
         return new KoCopyColorConversionTransformation(this);
     } else {
-        return d->parent->colorConversionSystem()->createColorConverter( this, dstColorSpace, renderingIntent);
+        return KoColorSpaceRegistry::instance()->colorConversionSystem()->createColorConverter( this, dstColorSpace, renderingIntent);
     }
 }
 
@@ -245,40 +258,13 @@ bool KoColorSpace::convertPixelsTo(const quint8 * src,
         quint32 numPixels,
         KoColorConversionTransformation::Intent renderingIntent) const
 {
-    if (dstColorSpace->id() == this->id()
-        && dstColorSpace->profile() == profile())
+    if( *this == *dstColorSpace)
     {
-        if (src!= dst)
-            memcpy (dst, src, numPixels * this->pixelSize());
-
-        return true;
+        memcpy( dst, src, numPixels * sizeof(quint8) * pixelSize());
+    } else {
+        KoCachedColorConversionTransformation cct = KoColorSpaceRegistry::instance()->colorConversionCache()->cachedConverter(this, dstColorSpace, renderingIntent);
+        cct.transformation()->transform(src, dst, numPixels);
     }
-
-    KoColorConversionTransformation* tf = 0;
-
-    if (d->lastUsedTransform != 0 && d->lastUsedDstColorSpace != 0) {
-        if (dstColorSpace->id() == d->lastUsedDstColorSpace->id() &&
-            dstColorSpace->profile() == d->lastUsedDstColorSpace->profile()) {
-            tf = d->lastUsedTransform;
-            }
-    }
-
-    if (not tf) {
-
-        if (!d->transforms.contains(dstColorSpace)) {
-    // XXX: Should we clear the transform cache if it gets too big?
-            tf = this->createColorConverter(dstColorSpace, renderingIntent);
-            d->transforms[dstColorSpace] = tf;
-        }
-        else {
-            tf = d->transforms[dstColorSpace];
-        }
-
-        d->lastUsedTransform = tf;
-        d->lastUsedDstColorSpace = dstColorSpace;
-    }
-    tf->transform(src, dst, numPixels);
-
     return true;
 }
 
@@ -338,12 +324,13 @@ void KoColorSpace::bitBlt(quint8 *dst,
                           const KoCompositeOp * op,
                           const QBitArray & channelFlags) const
 {
-    Q_ASSERT(op->colorSpace() == this);
+    Q_ASSERT_X(*op->colorSpace() == *this, "KoColorSpace::bitBlt", QString("Composite op is for color space %1 while this is %2" ).arg( op->colorSpace()->id() ).arg(id()).toLatin1() );
 
     if (rows <= 0 || cols <= 0)
         return;
 
-    if (this != srcSpace) {
+    if (!(*this == *srcSpace)) {
+
         quint32 len = pixelSize() * rows * cols;
 
         QVector<quint8> * conversionCache = threadLocalConversionCache(len);
@@ -389,7 +376,7 @@ void KoColorSpace::bitBlt(quint8 *dst,
                           qint32 cols,
                           const KoCompositeOp * op) const
 {
-    Q_ASSERT(op->colorSpace() == this);
+    Q_ASSERT(*op->colorSpace() == *this);
 
     if (rows <= 0 || cols <= 0)
         return;
@@ -440,10 +427,10 @@ QVector<quint8> * KoColorSpace::threadLocalConversionCache(quint32 size) const
     return ba;
 }
 
-KoColorTransformation* KoColorSpace::createColorTransformation( QString id, QHash<QString, QVariant> parameters) const
+KoColorTransformation* KoColorSpace::createColorTransformation( const QString & id, const QHash<QString, QVariant> & parameters) const
 {
     KoColorTransformationFactory* factory = KoColorTransformationFactoryRegistry::instance()->get( id );
-    if(not factory) return 0;
+    if(!factory) return 0;
     QPair<KoID, KoID> model( colorModelId(), colorDepthId() );
     QList< QPair<KoID, KoID> > models = factory->supportedModels();
     if(models.contains(model))
@@ -451,6 +438,7 @@ KoColorTransformation* KoColorSpace::createColorTransformation( QString id, QHas
         return factory->createTransformation( this, parameters);
     } else {
         // Find the best solution
+        // TODO use the color conversion cache
         KoColorConversionTransformation* csToFallBack = 0;
         KoColorConversionTransformation* fallBackToCs = 0;
         KoColorSpaceRegistry::instance()->colorConversionSystem()->createColorConverters(this, models, csToFallBack, fallBackToCs);
@@ -460,3 +448,19 @@ KoColorTransformation* KoColorSpace::createColorTransformation( QString id, QHas
         return new KoFallBackColorTransformation( csToFallBack, fallBackToCs, transfo);
     }
 }
+
+QImage KoColorSpace::convertToQImage(const quint8 *data, qint32 width, qint32 height,
+        const KoColorProfile *dstProfile,
+        KoColorConversionTransformation::Intent renderingIntent) const
+
+{
+    QImage img = QImage(width, height, QImage::Format_ARGB32);
+
+    const KoColorSpace * dstCS = KoColorSpaceRegistry::instance()->rgb8(dstProfile);
+
+    if (data)
+        this->convertPixelsTo(const_cast<quint8 *>(data), img.bits(), dstCS, width * height, renderingIntent);
+
+    return img;
+}
+
