@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2006 Cyrille Berger <cberger@cberger.bet
+ *  Copyright (c) 2006-2007 Cyrille Berger <cberger@cberger.bet
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,7 +20,7 @@
 #ifndef KOCOLORSPACEMATHS_H_
 #define KOCOLORSPACEMATHS_H_
 
-#include <pigment_export.h>
+#include "pigment_export.h"
 #include <KoIntegerMaths.h>
 #include "KoChannelInfo.h"
 
@@ -29,7 +29,7 @@
 /**
  * This is an empty shell that needs to be "specialized" for each possible
  * numerical type (quint8, quint16...).
- * 
+ *
  * It needs to defines some static constant fields :
  * - zeroValue : the zero for this numerical type
  * - unitValue : the maximum value of the normal dynamic range
@@ -37,7 +37,7 @@
  * - min : the minimum value
  * - epsilon : a value close to zero but different of zero
  * - bits : the bit depth
- * 
+ *
  * And some types :
  * - compositetype the type used for composite operations (usually one with
  *   a higher bit depth)
@@ -102,7 +102,7 @@ class PIGMENTCMS_EXPORT KoColorSpaceMathsTraits<quint32> {
 #include <config-openexr.h>
 #ifdef HAVE_OPENEXR
 #include <half.h>
-                 
+
 template<>
 class PIGMENTCMS_EXPORT KoColorSpaceMathsTraits<half> {
     public:
@@ -130,11 +130,24 @@ class PIGMENTCMS_EXPORT KoColorSpaceMathsTraits<float> {
         static const KoChannelInfo::enumChannelValueType channelValueType;
 };
 
+template<>
+class PIGMENTCMS_EXPORT KoColorSpaceMathsTraits<double> {
+    public:
+        typedef double compositetype;
+        static const double zeroValue;
+        static const double unitValue;
+        static const double max;
+        static const double min;
+        static const double epsilon;
+        static const qint8 bits = 64;
+        static const KoChannelInfo::enumChannelValueType channelValueType;
+};
+
 /**
  * This class defines some elementary operations used by various color
  * space. It's intended to be generic, but some specialization exists
- * either for optimization or just for beeing buildable.
- * 
+ * either for optimization or just for being buildable.
+ *
  * @param _T some numerical type with an existing trait
  * @param _Tdst some other numerical type with an existing trait, it is
  *              only needed if different of _T
@@ -144,7 +157,7 @@ class KoColorSpaceMaths {
     typedef KoColorSpaceMathsTraits<_T> traits;
     typedef typename traits::compositetype traits_compositetype;
     public:
-        inline static traits_compositetype multiply(traits_compositetype a, 
+        inline static traits_compositetype multiply(traits_compositetype a,
             typename  KoColorSpaceMathsTraits<_Tdst>::compositetype b)
         {
             return ((traits_compositetype)a * b ) /  KoColorSpaceMathsTraits<_Tdst>::unitValue;
@@ -179,20 +192,46 @@ class KoColorSpaceMaths {
 };
 
 //------------------------------ double specialization ------------------------------//
-
 template<>
-inline quint8 KoColorSpaceMaths<float,quint8>::scaleToA(float a)
+inline quint8 KoColorSpaceMaths<double,quint8>::scaleToA(double a)
 {
-    float v = a * 255;
+    double v = a * 255;
     return (quint8)(CLAMP(v, 0, 255));
 }
 
 template<>
-inline float KoColorSpaceMaths<quint8,float>::scaleToA(quint8 a)
+inline double KoColorSpaceMaths<quint8,double>::scaleToA(quint8 a)
 {
     return a * ( 1.0 / 255.0 );
 }
 
+template<>
+inline quint16 KoColorSpaceMaths<double, quint16>::scaleToA(double a)
+{
+    double v = a * 0xFFFF;
+    return (quint16)(CLAMP(v, 0, 0xFFFF));
+}
+
+template<>
+inline double KoColorSpaceMaths<quint16, double>::scaleToA(quint16 a)
+{
+    return a * ( 1.0 / 0xFFFF );
+}
+
+//------------------------------ half specialization ------------------------------//
+
+template<>
+inline float KoColorSpaceMaths<double, float>::scaleToA(double a)
+{
+    return (float)a;
+}
+
+template<>
+inline double KoColorSpaceMaths<float, double>::scaleToA(float a)
+{
+    return a;
+}
+
 template<>
 inline quint16 KoColorSpaceMaths<float,quint16>::scaleToA(float a)
 {
@@ -206,6 +245,19 @@ inline float KoColorSpaceMaths<quint16,float>::scaleToA(quint16 a)
     return a * ( 1.0 / 0xFFFF );
 }
 
+template<>
+inline quint8 KoColorSpaceMaths<float,quint8>::scaleToA(float a)
+{
+    float v = a * 255;
+    return (quint8)(CLAMP(v, 0, 255));
+}
+
+template<>
+inline float KoColorSpaceMaths<quint8,float>::scaleToA(quint8 a)
+{
+    return a * ( 1.0 / 255.0 );
+}
+
 template<>
 inline float KoColorSpaceMaths<float>::blend(float a, float b, float alpha)
 {
@@ -216,6 +268,18 @@ inline float KoColorSpaceMaths<float>::blend(float a, float b, float alpha)
 
 #ifdef HAVE_OPENEXR
 
+template<>
+inline half KoColorSpaceMaths<double, half>::scaleToA(double a)
+{
+    return (half)a;
+}
+
+template<>
+inline double KoColorSpaceMaths<half, double>::scaleToA(half a)
+{
+    return a;
+}
+
 template<>
 inline float KoColorSpaceMaths<half,float>::scaleToA(half a)
 {
@@ -240,7 +304,6 @@ inline half KoColorSpaceMaths<quint8,half>::scaleToA(quint8 a)
 {
     return a * ( 1.0 / 255.0 );
 }
-#include <kdebug.h>
 template<>
 inline quint16 KoColorSpaceMaths<half,quint16>::scaleToA(half a)
 {
@@ -254,6 +317,12 @@ inline half KoColorSpaceMaths<quint16,half>::scaleToA(quint16 a)
     return a * ( 1.0 / 0xFFFF );
 }
 
+template<>
+inline half KoColorSpaceMaths<half,half>::scaleToA(half a)
+{
+    return a;
+}
+
 template<>
 inline half KoColorSpaceMaths<half>::blend(half a, half b, half alpha)
 {
