@@ -20,23 +20,29 @@
 
 #include "KoColorSpaceRegistry.h"
 
+#include <QHash>
+
 #include <QStringList>
 #include <QDir>
 
 #include <kcomponentdata.h>
 #include <kstandarddirs.h>
-#include <kdebug.h>
 #include <kmessagebox.h>
 #include <klocale.h>
 #include <kservice.h>
 #include <kservicetypetrader.h>
+#include <k3staticdeleter.h>
 
 #include <lcms.h>
 
 #include "KoPluginLoader.h"
-#include "KoColorSpace.h"
-#include "KoColorConversionSystem.h"
+
+#include "DebugPigment.h"
 #include "KoBasicHistogramProducers.h"
+#include "KoColorSpace.h"
+#include "KoColorConversionCache.h"
+#include "KoColorConversionSystem.h"
+#include "KoIccColorSpaceEngine.h"
 
 #include "colorprofiles/KoLcmsColorProfileContainer.h"
 #include "colorspaces/KoAlphaColorSpace.h"
@@ -44,22 +50,34 @@
 #include "colorspaces/KoRgbU16ColorSpace.h"
 #include "colorspaces/KoRgbU8ColorSpace.h"
 
+#include <config-openctl.h>
+#ifdef HAVE_OPENCTL
+#include <OpenCTL/ModulesManager.h>
+#include "colorprofiles/KoCtlColorProfile.h"
+#endif
+
 struct KoColorSpaceRegistry::Private {
-    QMap<QString, KoColorProfile * > profileMap;
-    QMap<QString, const KoColorSpace * > csMap;
+    QHash<QString, KoColorProfile * > profileMap;
+    QHash<QString, const KoColorSpace * > csMap;
     typedef QList<KisPaintDeviceAction *> PaintActionList;
-    QMap<QString, PaintActionList> paintDevActionMap;
+    QHash<QString, PaintActionList> paintDevActionMap;
     const KoColorSpace *alphaCs;
     KoColorConversionSystem *colorConversionSystem;
+    KoColorConversionCache* colorConversionCache;
     static KoColorSpaceRegistry *singleton;
+    const KoColorSpace *rgbU8sRGB;
+    const KoColorSpace *lab16sLAB;
 };
 
 KoColorSpaceRegistry *KoColorSpaceRegistry::Private::singleton = 0;
+// deleting the registry still gives errors
+//static K3StaticDeleter<KoColorSpaceRegistry> staticDeleter;
 
 KoColorSpaceRegistry* KoColorSpaceRegistry::instance()
 {
     if(KoColorSpaceRegistry::Private::singleton == 0)
     {
+        //staticDeleter.setObject(KoColorSpaceRegistry::Private::singleton, new KoColorSpaceRegistry());
         KoColorSpaceRegistry::Private::singleton = new KoColorSpaceRegistry();
         KoColorSpaceRegistry::Private::singleton->init();
     }
@@ -69,8 +87,13 @@ KoColorSpaceRegistry* KoColorSpaceRegistry::instance()
 
 void KoColorSpaceRegistry::init()
 {
+    d->rgbU8sRGB = 0;
+    d->lab16sLAB = 0;
     d->colorConversionSystem = new KoColorConversionSystem;
-    // prepare a list of the profiles
+    d->colorConversionCache = new KoColorConversionCache;
+    // Initialise color engine
+    KoColorSpaceEngineRegistry::instance()->add( new KoIccColorSpaceEngine );
+    // prepare a list of the ICC profiles
     KGlobal::mainComponent().dirs()->addResourceType("icc_profiles", 0, "share/color/icc/");
 
     QStringList profileFilenames;
@@ -90,14 +113,46 @@ void KoColorSpaceRegistry::init()
 
             profile->load();
             if (profile->valid()) {
-                kDebug(31000) << "Valid profile : " << profile->name();
+                dbgPigmentCSRegistry << "Valid profile : " << profile->name();
                 d->profileMap[profile->name()] = profile;
             } else {
-                kDebug(31000) << "Invalid profile : " << profile->name();
+                dbgPigmentCSRegistry << "Invalid profile : " << profile->name();
+                delete profile;
             }
         }
     }
+#ifdef HAVE_OPENCTL
+    // Set PigmentCMS's ctl module directory
+    QStringList ctlModulesDirs = KGlobal::mainComponent().dirs()->findDirs( "data", "pigmentcms/ctlmodules/");
+    dbgPigmentCSRegistry << ctlModulesDirs;
+    foreach(const QString & dir, ctlModulesDirs)
+    {
+        dbgPigmentCSRegistry << "Append : " << dir << " to the list of CTL modules";
+        OpenCTL::ModulesManager::instance()->addDirectory( dir.toAscii().data());
+    }
+    // Load CTL Profiles
+    KGlobal::mainComponent().dirs()->addResourceType("ctl_profiles", "data", "pigmentcms/ctlprofiles/");
+    QStringList ctlprofileFilenames;
+    ctlprofileFilenames += KGlobal::mainComponent().dirs()->findAllResources("ctl_profiles", "*.ctlp",  KStandardDirs::Recursive);
+    dbgPigmentCSRegistry << "There are " << ctlprofileFilenames.size() << " CTL profiles";
+    if (!ctlprofileFilenames.empty()) {
+        KoColorProfile* profile = 0;
+        for( QStringList::Iterator it = ctlprofileFilenames.begin(); it != ctlprofileFilenames.end(); ++it ) {
+            dbgPigmentCSRegistry << "Load profile : " << *it;
+            profile = new KoCtlColorProfile(*it);
+            profile->load();
+            if (profile->valid()) {
+                dbgPigment << "Valid profile : " << profile->name();
+                d->profileMap[profile->name()] = profile;
+            } else {
+                dbgPigment << "Invalid profile : " << profile->name();
+                delete profile;
+            }
+        }
+    }
+#endif
 
+    // Initialise LAB
     KoColorProfile *labProfile = KoLcmsColorProfileContainer::createFromLcmsProfile(cmsCreateLabProfile(NULL));
     addProfile(labProfile);
     add(new KoLabColorSpaceFactory());
@@ -121,11 +176,11 @@ void KoColorSpaceRegistry::init()
     cmsHPROFILE hProfile = cmsCreateGrayProfile(cmsD50_xyY(), Gamma);
     cmsFreeGamma(Gamma);
     KoColorProfile *defProfile = KoLcmsColorProfileContainer::createFromLcmsProfile(hProfile);
-    kDebug(31000) << "Gray " << defProfile->name();
+    dbgPigmentCSRegistry << "Gray " << defProfile->name();
     addProfile(defProfile);
 
     // Create the built-in colorspaces
-    d->alphaCs = new KoAlphaColorSpace(this);
+    d->alphaCs = new KoAlphaColorSpace();
 
     KoPluginLoader::PluginsConfig config;
     config.whiteList = "ColorSpacePlugins";
@@ -138,17 +193,31 @@ void KoColorSpaceRegistry::init()
     configExtensions.blacklist = "ColorSpaceExtensionsPluginsDisabled";
     configExtensions.group = "koffice";
     KoPluginLoader::instance()->load("KOffice/ColorSpaceExtension","[X-Pigment-MinVersion] <= 0", configExtensions);
-
 }
 
 KoColorSpaceRegistry::KoColorSpaceRegistry() : d(new Private())
 {
     d->colorConversionSystem = 0;
+    d->colorConversionCache = 0;
 }
 
 KoColorSpaceRegistry::~KoColorSpaceRegistry()
 {
     delete d->colorConversionSystem;
+    foreach( KoColorProfile* profile, d->profileMap) {
+        delete profile;
+    }
+    d->profileMap.clear();
+    foreach( const KoColorSpace * cs, d->csMap) {
+        delete cs;
+    }
+    d->csMap.clear();
+    // deleting colorspaces calls a function in the cache
+    delete d->colorConversionCache;
+    d->colorConversionCache = 0;
+    delete d->alphaCs;
+    delete d->rgbU8sRGB;
+    delete d->lab16sLAB;
     delete d;
 }
 
@@ -164,7 +233,7 @@ void KoColorSpaceRegistry::add(const QString &id, KoColorSpaceFactory* item)
     d->colorConversionSystem->insertColorSpace(item);
 }
 
-KoColorProfile *  KoColorSpaceRegistry::profileByName(const QString & name) const
+const KoColorProfile *  KoColorSpaceRegistry::profileByName(const QString & name) const
 {
     if (d->profileMap.find(name) == d->profileMap.end()) {
         return 0;
@@ -173,16 +242,24 @@ KoColorProfile *  KoColorSpaceRegistry::profileByName(const QString & name) cons
     return d->profileMap[name];
 }
 
-QList<KoColorProfile *>  KoColorSpaceRegistry::profilesFor(const QString &id)
+QList<const KoColorProfile *>  KoColorSpaceRegistry::profilesFor(const QString &id)
 {
     return profilesFor(value(id));
 }
 
-QList<KoColorProfile *>  KoColorSpaceRegistry::profilesFor(KoColorSpaceFactory * csf)
+const KoColorSpace *  KoColorSpaceRegistry::colorSpace(const KoID &csID, const QString & profileName)
 {
-    QList<KoColorProfile *>  profiles;
+    return colorSpace(csID.id(), profileName);
+}
 
-    QMap<QString, KoColorProfile * >::Iterator it;
+
+QList<const KoColorProfile *>  KoColorSpaceRegistry::profilesFor(const KoColorSpaceFactory * csf)
+{
+    QList<const KoColorProfile *>  profiles;
+    if (csf == 0)
+        return profiles;
+
+    QHash<QString, KoColorProfile * >::Iterator it;
     for (it = d->profileMap.begin(); it != d->profileMap.end(); ++it) {
         KoColorProfile *  profile = it.value();
         if(csf->profileIsCompatible(profile))
@@ -194,23 +271,59 @@ QList<KoColorProfile *>  KoColorSpaceRegistry::profilesFor(KoColorSpaceFactory *
     return profiles;
 }
 
+QList<const KoColorSpaceFactory*> KoColorSpaceRegistry::colorSpacesFor( const KoColorProfile* _profile)
+{
+    QList<const KoColorSpaceFactory*> csfs;
+    foreach( KoColorSpaceFactory* csf, values() )
+    {
+        if( csf->profileIsCompatible( _profile ) )
+        {
+            csfs.push_back( csf );
+        }
+    }
+    return csfs;
+}
+
+QList<const KoColorProfile *>  KoColorSpaceRegistry::profilesFor(const KoID& id)
+{
+    return profilesFor(id.id());
+}
+
 void KoColorSpaceRegistry::addProfile(KoColorProfile *p)
 {
       if (p->valid()) {
           d->profileMap[p->name()] = p;
+          d->colorConversionSystem->insertColorProfile( p );
       }
 }
 
+void KoColorSpaceRegistry::addProfile(const KoColorProfile* profile)
+{
+  addProfile( profile->clone() );
+}
+
 void KoColorSpaceRegistry::addPaintDeviceAction(const KoColorSpace* cs,
-        KisPaintDeviceAction* action) {
+        KisPaintDeviceAction* action)
+{
     d->paintDevActionMap[cs->id()].append(action);
 }
 
 QList<KisPaintDeviceAction *>
-KoColorSpaceRegistry::paintDeviceActionsFor(const KoColorSpace* cs) {
+KoColorSpaceRegistry::paintDeviceActionsFor(const KoColorSpace* cs)
+{
     return d->paintDevActionMap[cs->id()];
 }
 
+bool KoColorSpaceRegistry::isCached(const QString & csId, const QString & profileName) const
+{
+    return !(d->csMap.find(idsToCacheName(csId, profileName)) == d->csMap.end());
+}
+
+QString KoColorSpaceRegistry::idsToCacheName(const QString & csId, const QString & profileName) const
+{
+  return csId + "<comb>" + profileName;
+}
+
 const KoColorSpace * KoColorSpaceRegistry::colorSpace(const QString &csID, const QString &pName)
 {
     QString profileName = pName;
@@ -225,27 +338,28 @@ const KoColorSpace * KoColorSpaceRegistry::colorSpace(const QString &csID, const
         profileName = csf->defaultProfile();
     }
 
-    QString name = csID + "<comb>" + profileName;
+    QString name = idsToCacheName(csID, profileName);
 
-    if (d->csMap.find(name) == d->csMap.end()) {
+    if (!isCached(csID, profileName)) {
         KoColorSpaceFactory *csf = value(csID);
         if(!csf)
         {
-            kDebug(31000) <<"Unknown color space type :" << csf;
+            dbgPigmentCSRegistry <<"Unknown color space type :" << csf;
             return 0;
         }
 
-        KoColorProfile *p = profileByName(profileName);
+        const KoColorProfile *p = profileByName(profileName);
         if(!p && !profileName.isEmpty())
         {
-            kDebug(31000) <<"Profile not found :" << profileName;
+            dbgPigmentCSRegistry <<"Profile not found :" << profileName;
             return 0;
         }
-        const KoColorSpace *cs = csf->createColorSpace(this, p);
+        const KoColorSpace *cs = csf->createColorSpace( p);
         if(!cs)
             return 0;
 
         d->csMap[name] = cs;
+        dbgPigmentCSRegistry << "colorspace count: " << d->csMap.count() << ", adding name: " << name;
     }
 
     if(d->csMap.contains(name))
@@ -259,7 +373,16 @@ const KoColorSpace * KoColorSpaceRegistry::colorSpace(const QString &csID, const
 {
     if( profile )
     {
-        const KoColorSpace *cs = colorSpace( csID, profile->name());
+        const KoColorSpace *cs = 0;
+        if(isCached( csID, profile->name() ) )
+        {
+            cs = colorSpace( csID, profile->name());
+        }
+        
+        if( not d->profileMap.contains( profile->name() ) )
+        {
+          addProfile( profile );
+        }
 
         if(!cs)
         {
@@ -267,16 +390,17 @@ const KoColorSpace * KoColorSpaceRegistry::colorSpace(const QString &csID, const
             KoColorSpaceFactory *csf = value(csID);
             if(!csf)
             {
-                kDebug(31000) <<"Unknown color space type :" << csf;
+                dbgPigmentCSRegistry <<"Unknown color space type :" << csf;
                 return 0;
             }
 
-            cs = csf->createColorSpace(this, const_cast<KoColorProfile *>(profile));
+            cs = csf->createColorSpace( profile);
             if(!cs )
                 return 0;
 
             QString name = csID + "<comb>" + profile->name();
             d->csMap[name] = cs;
+            dbgPigmentCSRegistry << "colorspace count: " << d->csMap.count() << ", adding name: " << name;
         }
 
         return cs;
@@ -292,11 +416,29 @@ const KoColorSpace * KoColorSpaceRegistry::alpha8()
 
 const KoColorSpace * KoColorSpaceRegistry::rgb8(const QString &profileName)
 {
+    if( profileName.isEmpty() )
+    {
+        if(!d->rgbU8sRGB)
+        {
+            d->rgbU8sRGB = colorSpace(KoRgbU8ColorSpace::colorSpaceId(), profileName);
+        }
+        Q_ASSERT( d->rgbU8sRGB);
+        return d->rgbU8sRGB;
+    }
     return colorSpace(KoRgbU8ColorSpace::colorSpaceId(), profileName);
 }
 
 const KoColorSpace * KoColorSpaceRegistry::rgb8(const KoColorProfile * profile)
 {
+    if(profile == 0 )
+    {
+        if(!d->rgbU8sRGB)
+        {
+            d->rgbU8sRGB = colorSpace(KoRgbU8ColorSpace::colorSpaceId(), profile);
+        }
+        Q_ASSERT( d->rgbU8sRGB);
+        return d->rgbU8sRGB;
+    }
     return colorSpace(KoRgbU8ColorSpace::colorSpaceId(), profile);
 }
 
@@ -312,11 +454,28 @@ const KoColorSpace * KoColorSpaceRegistry::rgb16(const KoColorProfile * profile)
 
 const KoColorSpace * KoColorSpaceRegistry::lab16(const QString &profileName)
 {
+    if( profileName.isEmpty() )
+    {
+        if(!d->lab16sLAB)
+        {
+            d->lab16sLAB = colorSpace(KoLabColorSpace::colorSpaceId(), profileName);
+        }
+        return d->lab16sLAB;
+    }
     return colorSpace(KoLabColorSpace::colorSpaceId(), profileName);
 }
 
 const KoColorSpace * KoColorSpaceRegistry::lab16(const KoColorProfile * profile)
 {
+    if( profile == 0 )
+    {
+        if(!d->lab16sLAB)
+        {
+            d->lab16sLAB = colorSpace(KoLabColorSpace::colorSpaceId(), profile);
+        }
+        Q_ASSERT( d->lab16sLAB );
+        return d->lab16sLAB;
+    }
     return colorSpace(KoLabColorSpace::colorSpaceId(), profile);
 }
 
@@ -326,24 +485,29 @@ QList<KoID> KoColorSpaceRegistry::colorModelsList(ColorSpaceListVisibility optio
     QList<KoColorSpaceFactory*> factories = values();
     foreach(KoColorSpaceFactory* factory, factories)
     {
-        if(not ids.contains(factory->colorModelId())
-           and ( option == AllColorSpaces or factory->userVisible() ) )
+        if(!ids.contains(factory->colorModelId())
+           && ( option == AllColorSpaces || factory->userVisible() ) )
         {
             ids << factory->colorModelId();
         }
     }
     return ids;
 }
-
 QList<KoID> KoColorSpaceRegistry::colorDepthList(const KoID& colorModelId, ColorSpaceListVisibility option ) const
+{
+    return colorDepthList(colorModelId.id(), option);
+}
+
+
+QList<KoID> KoColorSpaceRegistry::colorDepthList(const QString & colorModelId, ColorSpaceListVisibility option ) const
 {
     QList<KoID> ids;
     QList<KoColorSpaceFactory*> factories = values();
     foreach(KoColorSpaceFactory* factory, factories)
     {
-        if(not ids.contains(factory->colorDepthId())
-           and factory->colorModelId() == colorModelId
-           and ( option == AllColorSpaces or factory->userVisible() ))
+        if(!ids.contains(KoID(factory->colorDepthId()))
+           && factory->colorModelId().id() == colorModelId
+           && ( option == AllColorSpaces || factory->userVisible() ))
         {
             ids << factory->colorDepthId();
         }
@@ -351,12 +515,12 @@ QList<KoID> KoColorSpaceRegistry::colorDepthList(const KoID& colorModelId, Color
     return ids;
 }
 
-QString KoColorSpaceRegistry::colorSpaceId(QString colorModelId, QString colorDepthId)
+QString KoColorSpaceRegistry::colorSpaceId(const QString & colorModelId, const QString & colorDepthId)
 {
     QList<KoColorSpaceFactory*> factories = values();
     foreach(KoColorSpaceFactory* factory, factories)
     {
-        if(factory->colorModelId().id() == colorModelId and factory->colorDepthId().id() == colorDepthId )
+        if(factory->colorModelId().id() == colorModelId && factory->colorDepthId().id() == colorDepthId )
         {
             return factory->id();
         }
@@ -375,4 +539,7 @@ const KoColorConversionSystem* KoColorSpaceRegistry::colorConversionSystem() con
     return d->colorConversionSystem;
 }
 
-#include "KoColorSpaceRegistry.moc"
+KoColorConversionCache* KoColorSpaceRegistry::colorConversionCache() const
+{
+    return d->colorConversionCache;
+}
