@@ -23,7 +23,6 @@
 #include <QImage>
 #include <QBitArray>
 
-#include <kdebug.h>
 #include <klocale.h>
 
 #include <lcms.h>
@@ -36,7 +35,6 @@
 namespace {
     const quint8 PIXEL_MASK = 0;
 
-
     class CompositeOver : public KoCompositeOp {
 
     public:
@@ -48,6 +46,8 @@ namespace {
 
     public:
 
+        using KoCompositeOp::composite;
+
         void composite(quint8 *dst,
                        qint32 dststride,
                        const quint8 *src,
@@ -145,6 +145,8 @@ namespace {
 
     public:
 
+        using KoCompositeOp::composite;
+
         void composite(quint8 *dst,
                        qint32 dststride,
                        const quint8 *src,
@@ -207,8 +209,6 @@ namespace {
 
     };
 
-
-
     class CompositeErase : public KoCompositeOp {
 
     public:
@@ -220,6 +220,8 @@ namespace {
 
     public:
 
+        using KoCompositeOp::composite;
+
         void composite(quint8 *dst,
                        qint32 dststride,
                        const quint8 *src,
@@ -272,7 +274,6 @@ namespace {
             }
     };
 
-
     class CompositeSubtract : public KoCompositeOp {
 
     public:
@@ -284,6 +285,7 @@ namespace {
 
     public:
 
+        using KoCompositeOp::composite;
 
         void composite(quint8 *dst,
                        qint32 dststride,
@@ -344,8 +346,8 @@ namespace {
 
 }
 
-KoAlphaColorSpace::KoAlphaColorSpace(KoColorSpaceRegistry * parent) :
-    KoColorSpaceAbstract<AlphaU8Traits>("ALPHA", i18n("Alpha mask"),  parent )
+KoAlphaColorSpace::KoAlphaColorSpace() :
+    KoColorSpaceAbstract<AlphaU8Traits>("ALPHA", i18n("Alpha mask") )
 {
     addChannel(new KoChannelInfo(i18n("Alpha"), 0, KoChannelInfo::ALPHA, KoChannelInfo::UINT8));
     addCompositeOp( new CompositeOver( this ) );
@@ -358,36 +360,20 @@ KoAlphaColorSpace::~KoAlphaColorSpace()
 {
 }
 
-void KoAlphaColorSpace::fromQColor(const QColor& /*c*/, quint8 *dst, const KoColorProfile * /*profile*/) const
+void KoAlphaColorSpace::fromQColor(const QColor& c, quint8 *dst, const KoColorProfile * /*profile*/) const
 {
-    dst[PIXEL_MASK] = OPACITY_OPAQUE;
+    dst[PIXEL_MASK] = c.alpha();
 }
 
-void KoAlphaColorSpace::fromQColor(const QColor& /*c*/, quint8 opacity, quint8 *dst, const KoColorProfile * /*profile*/) const
+void KoAlphaColorSpace::toQColor(const quint8 * src, QColor *c, const KoColorProfile * /*profile*/) const
 {
-    dst[PIXEL_MASK] = opacity;
-}
-
-void KoAlphaColorSpace::alpha(const quint8 *pixel, quint8 *alpha) const
-{
-    *alpha = *pixel;
-}
-
-void KoAlphaColorSpace::toQColor(const quint8 * /*src*/, QColor *c, const KoColorProfile * /*profile*/) const
-{
-    c->setRgb(255, 255, 255);
-}
-
-void KoAlphaColorSpace::toQColor(const quint8 *src, QColor *c, quint8 *opacity, const KoColorProfile * /*profile*/) const
-{
-    c->setRgb(255, 255, 255);
-    *opacity = src[PIXEL_MASK];
+    c->setRgba(qRgba(255, 255, 255, src[PIXEL_MASK]));
 }
 
 quint8 KoAlphaColorSpace::difference(const quint8 *src1, const quint8 *src2) const
 {
     // Arithmetic operands smaller than int are converted to int automatically
-    return QABS(src2[PIXEL_MASK] - src1[PIXEL_MASK]);
+    return qAbs(src2[PIXEL_MASK] - src1[PIXEL_MASK]);
 }
 
 void KoAlphaColorSpace::mixColors(const quint8 **colors, const quint8 *weights, quint32 nColors, quint8 *dst) const
@@ -412,17 +398,9 @@ bool KoAlphaColorSpace::convertPixelsTo(const quint8 *src,
     // No lcms trickery here, we are only a opacity channel
     qint32 size = dstColorSpace->pixelSize();
 
-    quint32 j = 0;
-    quint32 i = 0;
+    memset(dst, 0, numPixels * size);
+    dstColorSpace->applyInverseAlphaU8Mask(dst, src, numPixels);
 
-    while ( i < numPixels ) {
-
-        dstColorSpace->fromQColor(Qt::red, OPACITY_OPAQUE - *(src + i), (dst + j));
-
-        i += 1;
-        j += size;
-
-    }
     return true;
 
 }
@@ -466,8 +444,7 @@ void KoAlphaColorSpace::convolveColors(quint8** colors, qint32 * kernelValues, q
 }
 
 QImage KoAlphaColorSpace::convertToQImage(const quint8 *data, qint32 width, qint32 height,
-                                   const KoColorProfile *  /*dstProfile*/, KoColorConversionTransformation::Intent /*renderingIntent*/,
-                                   float /*exposure*/) const
+                                   const KoColorProfile *  /*dstProfile*/, KoColorConversionTransformation::Intent /*renderingIntent*/) const
 {
     QImage img(width, height, QImage::Format_RGB32);
 
@@ -480,3 +457,8 @@ QImage KoAlphaColorSpace::convertToQImage(const quint8 *data, qint32 width, qint
     return img;
 }
 
+KoColorSpace* KoAlphaColorSpace::clone() const
+{
+    return new KoAlphaColorSpace();
+}
+
