@@ -17,10 +17,10 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-#include <QImage>
 #include <QColor>
+#include <QDomElement>
+#include <QImage>
 
-#include <kdebug.h>
 #include <klocale.h>
 
 #include <KoIntegerMaths.h>
@@ -65,12 +65,12 @@ private:
     quint32 m_psize;
 };
 
-KoRgbU8ColorSpace::KoRgbU8ColorSpace(KoColorSpaceRegistry * parent, KoColorProfile *p) :
-    KoLcmsColorSpace<RgbU8Traits>(colorSpaceId(), i18n("RGB 8-bit integer/channel)"), parent, TYPE_BGRA_8, icSigRgbData, p)
+KoRgbU8ColorSpace::KoRgbU8ColorSpace( KoColorProfile *p) :
+    KoLcmsColorSpace<RgbU8Traits>(colorSpaceId(), i18n("RGB 8-bit integer/channel)"),  TYPE_BGRA_8, icSigRgbData, p)
 {
-    addChannel(new KoChannelInfo(i18n("Red"), 2, KoChannelInfo::COLOR, KoChannelInfo::UINT8, 1, QColor(255,0,0)));
+    addChannel(new KoChannelInfo(i18n("Red"),   2, KoChannelInfo::COLOR, KoChannelInfo::UINT8, 1, QColor(255,0,0)));
     addChannel(new KoChannelInfo(i18n("Green"), 1, KoChannelInfo::COLOR, KoChannelInfo::UINT8, 1, QColor(0,255,0)));
-    addChannel(new KoChannelInfo(i18n("Blue"), 0, KoChannelInfo::COLOR, KoChannelInfo::UINT8, 1, QColor(0,0,255)));
+    addChannel(new KoChannelInfo(i18n("Blue"),  0, KoChannelInfo::COLOR, KoChannelInfo::UINT8, 1, QColor(0,0,255)));
     addChannel(new KoChannelInfo(i18n("Alpha"), 3, KoChannelInfo::ALPHA, KoChannelInfo::UINT8));
 
     init();
@@ -82,6 +82,8 @@ KoRgbU8ColorSpace::KoRgbU8ColorSpace(KoColorSpaceRegistry * parent, KoColorProfi
     addCompositeOp( new KoRgbU8CompositeOp(this, COMPOSITE_SATURATION,  i18n( "Saturation" )));
     addCompositeOp( new KoRgbU8CompositeOp(this, COMPOSITE_VALUE,  i18n( "Value" )));
     addCompositeOp( new KoRgbU8CompositeOp(this, COMPOSITE_COLOR,  i18n( "Color" )));
+    addCompositeOp( new KoRgbU8CompositeOp(this, COMPOSITE_ADD,  i18n( "Add" )));
+    addCompositeOp( new KoRgbU8CompositeOp(this, COMPOSITE_SUBTRACT,  i18n( "Substract" )));
 }
 
 
@@ -95,3 +97,33 @@ QString KoRgbU8ColorSpace::colorSpaceId()
     return QString("RGBA");
 }
 
+
+KoColorSpace* KoRgbU8ColorSpace::clone() const
+{
+    return new KoRgbU8ColorSpace( profile()->clone());
+}
+
+void KoRgbU8ColorSpace::colorToXML( const quint8* pixel, QDomDocument& doc, QDomElement& colorElt) const
+{
+    const RgbU8Traits::Pixel* p = reinterpret_cast<const RgbU8Traits::Pixel*>( pixel );
+    QDomElement labElt = doc.createElement( "RGB" );
+    labElt.setAttribute("r", KoColorSpaceMaths< RgbU8Traits::channels_type, qreal>::scaleToA( p->red) );
+    labElt.setAttribute("g", KoColorSpaceMaths< RgbU8Traits::channels_type, qreal>::scaleToA( p->green) );
+    labElt.setAttribute("b", KoColorSpaceMaths< RgbU8Traits::channels_type, qreal>::scaleToA( p->blue) );
+    labElt.setAttribute("space", profile()->name() );
+    colorElt.appendChild( labElt );
+}
+
+void KoRgbU8ColorSpace::colorFromXML( quint8* pixel, const QDomElement& elt) const
+{
+    RgbU8Traits::Pixel* p = reinterpret_cast<RgbU8Traits::Pixel*>( pixel );
+    p->red = KoColorSpaceMaths< qreal, RgbU8Traits::channels_type >::scaleToA(elt.attribute("r").toDouble());
+    p->green = KoColorSpaceMaths< qreal, RgbU8Traits::channels_type >::scaleToA(elt.attribute("g").toDouble());
+    p->blue = KoColorSpaceMaths< qreal, RgbU8Traits::channels_type >::scaleToA(elt.attribute("b").toDouble());
+}
+
+quint8 KoRgbU8ColorSpace::intensity8(const quint8 * src) const
+{
+    const RgbU8Traits::Pixel* p = reinterpret_cast<const RgbU8Traits::Pixel*>( src );
+    return (quint8)(p->red * 0.30 + p->green * 0.59 + p->blue * 0.11);
+}
