@@ -19,9 +19,9 @@
 #ifndef KORESOURCE_H
 #define KORESOURCE_H
 
-#include <QImage>
-#include <QObject>
-#include <QString>
+#include <QtGui/QImage>
+#include <QtCore/QObject>
+#include <QtCore/QString>
 #include <koresource_export.h>
 
 class QDomDocument;
@@ -64,8 +64,18 @@ public:
      */
     virtual QImage img() const {return QImage(); }
 
+private:
+    /// XXX: doc?
     virtual void toXML(QDomDocument& , QDomElement&) const;
-    
+
+public:
+
+    /// Returns if resource can be removed by the user
+    bool removable() const;
+
+    /// Returns the default file extension which should be when saving the resource
+    virtual QString defaultFileExtension() const;
+
 public:
     QString filename() const;
     void setFilename(const QString& filename);
