@@ -35,10 +35,6 @@
 #include <kdebug.h>
 #include <klocale.h>
 
-
-#define PREVIEW_WIDTH 64
-#define PREVIEW_HEIGHT 64
-
 KoGradientSegment::RGBColorInterpolationStrategy *KoGradientSegment::RGBColorInterpolationStrategy::m_instance = 0;
 KoGradientSegment::HSVCWColorInterpolationStrategy *KoGradientSegment::HSVCWColorInterpolationStrategy::m_instance = 0;
 KoGradientSegment::HSVCCWColorInterpolationStrategy *KoGradientSegment::HSVCCWColorInterpolationStrategy::m_instance = 0;
@@ -72,11 +68,6 @@ bool KoSegmentGradient::save()
     return false;
 }
 
-QImage KoSegmentGradient::img() const
-{
-    return m_img;
-}
-
 bool KoSegmentGradient::init()
 {
     QFile file(filename());
@@ -101,7 +92,7 @@ bool KoSegmentGradient::init()
 
     if (nameDefinition.startsWith("Name: ")) {
         QString nameText = nameDefinition.right(nameDefinition.length() - 6);
-        setName(i18n(nameText.toAscii()));
+        setName(nameText);
 
         numSegmentsText = fileContent.readLine();
     }
@@ -130,21 +121,21 @@ bool KoSegmentGradient::init()
 
         QString segmentText = fileContent.readLine();
         QTextStream segmentFields(&segmentText);
-        QStringList values = segmentText.split(" ");
+        QStringList values = segmentText.split(' ');
 
-        double leftOffset = values[0].toDouble();
-        double middleOffset = values[1].toDouble();
-        double rightOffset = values[2].toDouble();
+        qreal leftOffset = values[0].toDouble();
+        qreal middleOffset = values[1].toDouble();
+        qreal rightOffset = values[2].toDouble();
 
-        double leftRed = values[3].toDouble();
-        double leftGreen = values[4].toDouble();
-        double leftBlue = values[5].toDouble();
-        double leftAlpha = values[6].toDouble();
+        qreal leftRed = values[3].toDouble();
+        qreal leftGreen = values[4].toDouble();
+        qreal leftBlue = values[5].toDouble();
+        qreal leftAlpha = values[6].toDouble();
 
-        double rightRed = values[7].toDouble();
-        double rightGreen = values[8].toDouble();
-        double rightBlue = values[9].toDouble();
-        double rightAlpha = values[10].toDouble();
+        qreal rightRed = values[7].toDouble();
+        qreal rightGreen = values[8].toDouble();
+        qreal rightBlue = values[9].toDouble();
+        qreal rightAlpha = values[10].toDouble();
 
         int interpolationType = values[11].toInt();
         int colorInterpolationType = values[12].toInt();
@@ -176,7 +167,7 @@ bool KoSegmentGradient::init()
     }
 
     if (!m_segments.isEmpty()) {
-        m_img = generatePreview(PREVIEW_WIDTH, PREVIEW_HEIGHT);
+        updatePreview();
         setValid(true);
         return true;
     }
@@ -185,15 +176,7 @@ bool KoSegmentGradient::init()
     }
 }
 
-void KoSegmentGradient::setImage(const QImage& img)
-{
-    m_img = img;
-    m_img.detach();
-
-    setValid(true);
-}
-
-KoGradientSegment *KoSegmentGradient::segmentAt(double t) const
+KoGradientSegment *KoSegmentGradient::segmentAt(qreal t) const
 {
     Q_ASSERT(t >= 0 || t <= 1);
     Q_ASSERT(!m_segments.empty());
@@ -208,7 +191,7 @@ KoGradientSegment *KoSegmentGradient::segmentAt(double t) const
     return 0;
 }
 
-void KoSegmentGradient::colorAt(KoColor& dst, double t) const
+void KoSegmentGradient::colorAt(KoColor& dst, qreal t) const
 {
     const KoGradientSegment *segment = segmentAt(t);
     Q_ASSERT(segment != 0);
@@ -218,56 +201,26 @@ void KoSegmentGradient::colorAt(KoColor& dst, double t) const
     }
 }
 
-QImage KoSegmentGradient::generatePreview(int width, int height) const
-{
-    QImage img(width, height, QImage::Format_RGB32);
-
-    KoColor c;
-    QColor color;
-    for (int y = 0; y < img.height(); y++) {
-        for (int x = 0; x < img.width(); x++) {
-
-            int backgroundRed = 128 + 63 * ((x / 4 + y / 4) % 2);
-            int backgroundGreen = backgroundRed;
-            int backgroundBlue = backgroundRed;
-
-            quint8 opacity;
-            double t = static_cast<double>(x) / (img.width() - 1);
-
-            colorAt(c, t);
-            c.toQColor( &color, &opacity );
-
-            double alpha = static_cast<double>(opacity) / OPACITY_OPAQUE;
-
-            int red = static_cast<int>((1 - alpha) * backgroundRed + alpha * color.red() + 0.5);
-            int green = static_cast<int>((1 - alpha) * backgroundGreen + alpha * color.green() + 0.5);
-            int blue = static_cast<int>((1 - alpha) * backgroundBlue + alpha * color.blue() + 0.5);
-
-            img.setPixel(x, y, qRgb(red, green, blue));
-        }
-    }
-
-    return img;
-}
-
 QGradient* KoSegmentGradient::toQGradient() const
 {
     QGradient* gradient = new QLinearGradient();;
 
     QColor color;
-    quint8 opacity;
     foreach(KoGradientSegment* segment, m_segments) {
-        segment->startColor().toQColor(&color, &opacity);
-        color.setAlpha(opacity);
+        segment->startColor().toQColor(&color);
         gradient->setColorAt( segment->startOffset() , color );
-        segment->endColor().toQColor(&color, &opacity);
-        color.setAlpha(opacity);
+        segment->endColor().toQColor(&color);
         gradient->setColorAt( segment->endOffset() , color );
     }
     return gradient;
 }
 
-KoGradientSegment::KoGradientSegment(int interpolationType, int colorInterpolationType, double startOffset, double middleOffset, double endOffset, const KoColor& startColor, const KoColor& endColor)
+QString KoSegmentGradient::defaultFileExtension() const
+{
+    return QString(".ggr");
+}
+
+KoGradientSegment::KoGradientSegment(int interpolationType, int colorInterpolationType, qreal startOffset, qreal middleOffset, qreal endOffset, const KoColor& startColor, const KoColor& endColor)
 {
     m_interpolator = 0;
 
@@ -359,22 +312,22 @@ const KoColor& KoGradientSegment::endColor() const
     return m_endColor;
 }
 
-double KoGradientSegment::startOffset() const
+qreal KoGradientSegment::startOffset() const
 {
     return m_startOffset;
 }
 
-double KoGradientSegment::middleOffset() const
+qreal KoGradientSegment::middleOffset() const
 {
     return m_middleOffset;
 }
 
-double KoGradientSegment::endOffset() const
+qreal KoGradientSegment::endOffset() const
 {
     return m_endOffset;
 }
 
-void KoGradientSegment::setStartOffset(double t)
+void KoGradientSegment::setStartOffset(qreal t)
 {
     m_startOffset = t;
     m_length = m_endOffset - m_startOffset;
@@ -386,7 +339,7 @@ void KoGradientSegment::setStartOffset(double t)
         m_middleT = (m_middleOffset - m_startOffset) / m_length;
     }
 }
-void KoGradientSegment::setMiddleOffset(double t)
+void KoGradientSegment::setMiddleOffset(qreal t)
 {
     m_middleOffset = t;
 
@@ -398,7 +351,7 @@ void KoGradientSegment::setMiddleOffset(double t)
     }
 }
 
-void KoGradientSegment::setEndOffset(double t)
+void KoGradientSegment::setEndOffset(qreal t)
 {
     m_endOffset = t;
     m_length = m_endOffset - m_startOffset;
@@ -457,11 +410,11 @@ void KoGradientSegment::setColorInterpolation(int colorInterpolationType)
     }
 }
 
-void KoGradientSegment::colorAt(KoColor& dst, double t) const
+void KoGradientSegment::colorAt(KoColor& dst, qreal t) const
 {
     Q_ASSERT(t > m_startOffset - DBL_EPSILON && t < m_endOffset + DBL_EPSILON);
 
-    double segmentT;
+    qreal segmentT;
 
     if (m_length < DBL_EPSILON) {
         segmentT = 0.5;
@@ -470,7 +423,7 @@ void KoGradientSegment::colorAt(KoColor& dst, double t) const
         segmentT = (t - m_startOffset) / m_length;
     }
 
-    double colorT = m_interpolator->valueAt(segmentT, m_middleT);
+    qreal colorT = m_interpolator->valueAt(segmentT, m_middleT);
 
     m_colorInterpolator->colorAt(dst, colorT, m_startColor, m_endColor);
 
@@ -498,7 +451,7 @@ KoGradientSegment::RGBColorInterpolationStrategy *KoGradientSegment::RGBColorInt
     return m_instance;
 }
 
-void KoGradientSegment::RGBColorInterpolationStrategy::colorAt(KoColor& dst, double t, const KoColor& start, const KoColor& end) const
+void KoGradientSegment::RGBColorInterpolationStrategy::colorAt(KoColor& dst, qreal t, const KoColor& start, const KoColor& end) const
 {
     m_start.fromKoColor(start);
     m_end.fromKoColor(end);
@@ -507,7 +460,7 @@ void KoGradientSegment::RGBColorInterpolationStrategy::colorAt(KoColor& dst, dou
     colors[0] = start.data();
     colors[1] = end.data();
 
-    quint8 colorWeights[2];
+    qint16 colorWeights[2];
     colorWeights[0] = static_cast<quint8>((1.0 - t) * 255 + 0.5);
     colorWeights[1] = 255 - colorWeights[0];
 
@@ -531,15 +484,13 @@ KoGradientSegment::HSVCWColorInterpolationStrategy *KoGradientSegment::HSVCWColo
     return m_instance;
 }
 
-void KoGradientSegment::HSVCWColorInterpolationStrategy::colorAt(KoColor& dst, double t, const KoColor& start, const KoColor& end) const
+void KoGradientSegment::HSVCWColorInterpolationStrategy::colorAt(KoColor& dst, qreal t, const KoColor& start, const KoColor& end) const
 {
     QColor sc;
     QColor ec;
-    quint8 startOpacity;
-    quint8 endOpacity;
 
-    start.toQColor( &sc, &startOpacity);
-    end.toQColor( &ec, &endOpacity);
+    start.toQColor( &sc);
+    end.toQColor( &ec);
 
     int s = static_cast<int>(sc.saturation() + t * (ec.saturation() - sc.saturation()) + 0.5);
     int v = static_cast<int>(sc.value() + t * (ec.value() - sc.value()) + 0.5);
@@ -556,11 +507,12 @@ void KoGradientSegment::HSVCWColorInterpolationStrategy::colorAt(KoColor& dst, d
         }
     }
     // XXX: added an explicit cast. Is this correct?
-    quint8 opacity = static_cast<quint8>(startOpacity + t * (endOpacity - startOpacity));
+    quint8 opacity = static_cast<quint8>(sc.alpha() + t * (ec.alpha() - sc.alpha()));
 
     QColor result;
     result.setHsv( h, s, v);
-    dst.fromQColor(result, opacity);
+    result.setAlpha( opacity );
+    dst.fromQColor(result);
 }
 
 KoGradientSegment::HSVCCWColorInterpolationStrategy::HSVCCWColorInterpolationStrategy() :
@@ -579,15 +531,13 @@ KoGradientSegment::HSVCCWColorInterpolationStrategy *KoGradientSegment::HSVCCWCo
     return m_instance;
 }
 
-void KoGradientSegment::HSVCCWColorInterpolationStrategy::colorAt(KoColor& dst, double t, const KoColor& start, const KoColor& end) const
+void KoGradientSegment::HSVCCWColorInterpolationStrategy::colorAt(KoColor& dst, qreal t, const KoColor& start, const KoColor& end) const
 {
     QColor sc;
     QColor se;
-    quint8 startOpacity;
-    quint8 endOpacity;
 
-    start.toQColor( &sc, &startOpacity);
-    end.toQColor( &se, &endOpacity);
+    start.toQColor( &sc);
+    end.toQColor( &se);
 
     int s = static_cast<int>(sc.saturation() + t * (se.saturation() - sc.saturation()) + 0.5);
     int v = static_cast<int>(sc.value() + t * (se.value() - sc.value()) + 0.5);
@@ -604,11 +554,12 @@ void KoGradientSegment::HSVCCWColorInterpolationStrategy::colorAt(KoColor& dst,
         }
     }
     // XXX: Added an explicit static cast
-    quint8 opacity = static_cast<quint8>(startOpacity + t * (endOpacity - startOpacity));
+    quint8 opacity = static_cast<quint8>(sc.alpha() + t * (se.alpha() - sc.alpha()));
 
     QColor result;
     result.setHsv( h, s, v);
-    dst.fromQColor( result, opacity);
+    result.setAlpha( opacity );
+    dst.fromQColor( result );
 }
 
 KoGradientSegment::LinearInterpolationStrategy *KoGradientSegment::LinearInterpolationStrategy::instance()
@@ -621,12 +572,12 @@ KoGradientSegment::LinearInterpolationStrategy *KoGradientSegment::LinearInterpo
     return m_instance;
 }
 
-double KoGradientSegment::LinearInterpolationStrategy::calcValueAt(double t, double middle)
+qreal KoGradientSegment::LinearInterpolationStrategy::calcValueAt(qreal t, qreal middle)
 {
     Q_ASSERT(t > -DBL_EPSILON && t < 1 + DBL_EPSILON);
     Q_ASSERT(middle > -DBL_EPSILON && middle < 1 + DBL_EPSILON);
 
-    double value = 0;
+    qreal value = 0;
 
     if (t <= middle) {
         if (middle < DBL_EPSILON) {
@@ -648,7 +599,7 @@ double KoGradientSegment::LinearInterpolationStrategy::calcValueAt(double t, dou
     return value;
 }
 
-double KoGradientSegment::LinearInterpolationStrategy::valueAt(double t, double middle) const
+qreal KoGradientSegment::LinearInterpolationStrategy::valueAt(qreal t, qreal middle) const
 {
     return calcValueAt(t, middle);
 }
@@ -668,12 +619,12 @@ KoGradientSegment::CurvedInterpolationStrategy *KoGradientSegment::CurvedInterpo
     return m_instance;
 }
 
-double KoGradientSegment::CurvedInterpolationStrategy::valueAt(double t, double middle) const
+qreal KoGradientSegment::CurvedInterpolationStrategy::valueAt(qreal t, qreal middle) const
 {
     Q_ASSERT(t > -DBL_EPSILON && t < 1 + DBL_EPSILON);
     Q_ASSERT(middle > -DBL_EPSILON && middle < 1 + DBL_EPSILON);
 
-    double value = 0;
+    qreal value = 0;
 
     if (middle < DBL_EPSILON) {
         middle = DBL_EPSILON;
@@ -694,10 +645,10 @@ KoGradientSegment::SineInterpolationStrategy *KoGradientSegment::SineInterpolati
     return m_instance;
 }
 
-double KoGradientSegment::SineInterpolationStrategy::valueAt(double t, double middle) const
+qreal KoGradientSegment::SineInterpolationStrategy::valueAt(qreal t, qreal middle) const
 {
-    double lt = LinearInterpolationStrategy::calcValueAt(t, middle);
-    double value = (sin(-M_PI_2 + M_PI * lt) + 1.0) / 2.0;
+    qreal lt = LinearInterpolationStrategy::calcValueAt(t, middle);
+    qreal value = (sin(-M_PI_2 + M_PI * lt) + 1.0) / 2.0;
 
     return value;
 }
@@ -712,10 +663,10 @@ KoGradientSegment::SphereIncreasingInterpolationStrategy *KoGradientSegment::Sph
     return m_instance;
 }
 
-double KoGradientSegment::SphereIncreasingInterpolationStrategy::valueAt(double t, double middle) const
+qreal KoGradientSegment::SphereIncreasingInterpolationStrategy::valueAt(qreal t, qreal middle) const
 {
-    double lt = LinearInterpolationStrategy::calcValueAt(t, middle) - 1;
-    double value = sqrt(1 - lt * lt);
+    qreal lt = LinearInterpolationStrategy::calcValueAt(t, middle) - 1;
+    qreal value = sqrt(1 - lt * lt);
 
     return value;
 }
@@ -730,10 +681,10 @@ KoGradientSegment::SphereDecreasingInterpolationStrategy *KoGradientSegment::Sph
     return m_instance;
 }
 
-double KoGradientSegment::SphereDecreasingInterpolationStrategy::valueAt(double t, double middle) const
+qreal KoGradientSegment::SphereDecreasingInterpolationStrategy::valueAt(qreal t, qreal middle) const
 {
-    double lt = LinearInterpolationStrategy::calcValueAt(t, middle);
-    double value = 1 - sqrt(1 - lt * lt);
+    qreal lt = LinearInterpolationStrategy::calcValueAt(t, middle);
+    qreal value = 1 - sqrt(1 - lt * lt);
 
     return value;
 }
