@@ -17,6 +17,10 @@
  * Boston, MA 02110-1301, USA.
  */
 #include "Plugin.h"
+#include "strokedocker/StrokeDockerFactory.h"
+#include "shapeproperties/ShapePropertiesDockerFactory.h"
+#include "shapecollection/KoShapeCollectionDocker.h"
+#include "shadowdocker/ShadowDockerFactory.h"
 #include "shapeselector/ShapeSelectorFactory.h"
 
 #include <KoDockRegistry.h>
@@ -29,6 +33,10 @@ Plugin::Plugin( QObject *parent, const QStringList& )
     : QObject(parent)
 {
     Q_UNUSED(parent);
+    KoDockRegistry::instance()->add( new StrokeDockerFactory() );
+    KoDockRegistry::instance()->add( new ShapePropertiesDockerFactory() );
+    KoDockRegistry::instance()->add( new KoShapeCollectionDockerFactory() );
+    KoDockRegistry::instance()->add( new ShadowDockerFactory() );
     KoDockRegistry::instance()->add( new ShapeSelectorFactory() );
 }
 
