@@ -378,7 +378,7 @@ void Thesaurus::slotFindTerm()
     slotFindTerm(m_edit->currentText());
 }
 
-// Triggered when a list item is double-clicked.
+// Triggered when a list item is qreal-clicked.
 void Thesaurus::slotFindTermFromList(QListWidgetItem *item)
 {
     slotFindTerm(item->text());
@@ -440,8 +440,8 @@ void Thesaurus::findTermThesaurus(const QString &searchTerm)
         return;
     }
 
-    if (!m_thesProc->waitForReadyRead())
-        ;
+    if (!m_thesProc->waitForReadyRead()) {
+    }
     QByteArray byteArray = m_thesProc->readAllStandardOutput();
     QString stdoutString(byteArray);
     QStringList lines = stdoutString.split(QChar('\n'));
