@@ -1,7 +1,6 @@
 /* This file is part of the KDE project
- * Copyright (C) 2006-2009 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2006-2007 Thomas Zander <zander@kde.org>
  * Copyright (C) 2008 Thorsten Zachmann <zachmann@kde.org>
- * Copyright (C) 2009 KO GmbH <cbo@kogmbh.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -24,39 +23,35 @@
 
 #include "TextShape.h"
 
-#include <KoToolBase.h>
+#include <KoTextSelectionHandler.h>
+#include <KoTool.h>
 
-#include <QClipboard>
+#include <QTextCursor>
 #include <QHash>
 #include <QTextBlock>
-#include <QTextCursor>
 #include <QTimer>
-#include <QPointer>
-
-class TextEditingPluginContainer;
-class InsertCharacter;
-class KoChangeTracker;
-class KoCharacterStyle;
-class KoColor;
-class KoColorPopupAction;
-class KoParagraphStyle;
-class KoStyleManager;
-class KoTextEditingPlugin;
-class KoTextEditor;
-class UndoTextCommand;
+#include <QClipboard>
 
 class KAction;
-class KFontSizeAction;
-class KFontAction;
+class KoAction;
+class KoStyleManager;
+class KoTextEditingPlugin;
+class KoColor;
+class KoColorSetAction;
+class UndoTextCommand;
+class InsertCharacter;
+class ChangeTracker;
 
 class QUndoCommand;
+class KFontSizeAction;
+class KFontAction;
 
 class MockCanvas;
 
 /**
  * This is the tool for the text-shape (which is a flake-based plugin).
  */
-class TextTool : public KoToolBase
+class TextTool : public KoTool
 {
     Q_OBJECT
 public:
@@ -80,7 +75,7 @@ public:
     virtual void keyReleaseEvent(QKeyEvent *event);
 
     /// reimplemented from superclass
-    virtual void activate(ToolActivation toolActivation, const QSet<KoShape*> &shapes);
+    virtual void activate(bool temporary = false);
     /// reimplemented from superclass
     virtual void deactivate();
     /// reimplemented from superclass
@@ -88,8 +83,6 @@ public:
     ///reimplemented
     virtual void deleteSelection();
     /// reimplemented from superclass
-    virtual void cut();
-    /// reimplemented from superclass
     virtual bool paste();
     /// reimplemented from superclass
     virtual QStringList supportedPasteMimeTypes() const;
@@ -111,9 +104,9 @@ public:
     /// The following two methods allow an undo/redo command to tell the tool, it will modify the QTextDocument and wants to be parent of the undo/redo commands resulting from these changes.
 
     void startEditing(QUndoCommand* command);
-
+    
     void stopEditing();
-
+    
     const QTextCursor cursor();
 
 
@@ -122,12 +115,9 @@ public slots:
     void startTextEditingPlugin(const QString &pluginId);
     /// add a command to the undo stack, executing it as well.
     void addCommand(QUndoCommand *command);
-    /// reimplemented from KoToolBase
+    /// reimplemented from KoTool
     virtual void resourceChanged(int key, const QVariant &res);
 
-    /// call this when the 'is-bidi' boolean has been changed.
-    void isBidiUpdated();
-
 signals:
     /// emitted every time a different styleManager is set.
     void styleManagerChanged(KoStyleManager *manager);
@@ -139,14 +129,6 @@ signals:
     void blockChanged(const QTextBlock &block);
 
 private slots:
-    /// make the selected text bold or not
-    void bold(bool);
-    /// make the selected text italic or not
-    void italic(bool);
-    /// underline of the selected text
-    void underline(bool underline);
-    /// strikethrough of the selected text
-    void strikeOut(bool strikeOut);
     /// insert a non breaking space at the caret position
     void nonbreakingSpace();
     /// insert a non breaking hyphen at the caret position
@@ -171,28 +153,14 @@ private slots:
     void decreaseIndent();
     /// move the paragraph indent of the selected text to be more (right on LtR text)
     void increaseIndent();
-    /// Increase the font size. This will preserve eventual difference in font size within the selection.
-    void increaseFontSize();
-    /// Decrease font size. See above.
-    void decreaseFontSize();
-    /// Set font family
-    void setFontFamily(const QString &);
-    /// Set Font size
-    void setFontSize(int size);
     /// Default Format
     void setDefaultFormat();
     /// see KoTextSelectionHandler::insertIndexMarker
     void insertIndexMarker();
-    /// shows a dialog to insert a table
-    void insertTable();
     /// shows a dialog to alter the paragraph properties
     void formatParagraph();
-    //When enabled, display changes
-    void toggleShowChanges(bool);
     /// When enabled, make the change tracker record changes made while typing
-    void toggleRecordChanges(bool);
-    /// Configure Change Tracking
-    void configureChangeTracking();
+    void toggleTrackChanges(bool);
     /// select all text in the current document.
     void selectAll();
     /// show the style manager
@@ -201,10 +169,6 @@ private slots:
     void setTextColor(const KoColor &color);
     /// change background color of a selected text
     void setBackgroundColor(const KoColor &color);
-    /// set Paragraph style of current selection. Exisiting style will be completely overridden.
-    void setStyle(KoParagraphStyle *syle);
-    /// set the characterStyle of the current selection. see above.
-    void setStyle(KoCharacterStyle *style);
 
     /// add a KoDocument wide undo command which will call undo on the qtextdocument.
     void addUndoCommand();
@@ -216,9 +180,12 @@ private slots:
 
     /// show the insert special character docker.
     void insertSpecialCharacter();
-    /// insert string
-    void insertString(const QString &string);
 
+    /// method that will be called in an alternative thread for updating the paragraph direction at a character pos
+    void updateParagraphDirection(const QVariant &variant);
+    /// method that will be called in the UI thread directly after the one above
+    void updateParagraphDirectionUi();
+    
     /// returns the focus to canvas when styles are selected in the optionDocker
     void returnFocusToCanvas();
 
@@ -227,62 +194,43 @@ private slots:
 
     void blinkCaret();
 
-    // called when the m_textShapeData has been deleted.
-    void shapeDataRemoved();
-
-    //Show tooltip with change info
-    void showChangeTip();
-
-    /// print debug about the details of the text document
-    void debugTextDocument();
-    /// print debug about the details of the styles on the current text document
-    void debugTextStyles();
-    /// the document we are editing has received an extra shape
-    void shapeAddedToDoc(KoShape *shape);
-    void ensureCursorVisible();
-
-    void testSlot(bool);
-
 #ifndef NDEBUG
 protected:
     explicit TextTool(MockCanvas *canvas);
 #endif
-    friend class TestChangeTrackedDelete;
 
 private:
+    bool pasteHelper(QClipboard::Mode mode);
     void repaintCaret();
     void repaintSelection();
     void repaintSelection(int from, int to);
+    void ensureCursorVisible();
     QRectF textRect(int startPosition, int endPosition) const;
     int pointToPosition(const QPointF & point) const;
+    void updateSelectionHandler();
     void updateActions();
     void updateStyleManager();
     void setShapeData(KoTextShapeData *data);
     void updateSelectedShape(const QPointF &point);
-    void updateSelectionHandler();
+
     void editingPluginEvents();
     void finishedWord();
     void finishedParagraph();
-    void readConfig();
-    void writeConfig();
 
 private:
     friend class UndoTextCommand;
     friend class TextCommandBase;
     friend class ChangeTracker;
-    friend class TextPasteCommand;
-    friend class TextCutCommand;
-    friend class ShowChangesCommand;
-    friend class ChangeTrackedDeleteCommand;
-    friend class DeleteCommand;
     TextShape *m_textShape;
     KoTextShapeData *m_textShapeData;
-    QPointer<KoTextEditor> m_textEditor;
-    KoChangeTracker *m_changeTracker;
+    QTextCursor m_caret;
+    ChangeTracker *m_changeTracker;
+    KoTextSelectionHandler m_selectionHandler;
     bool m_allowActions;
     bool m_allowAddUndoCommand;
     bool m_trackChanges;
-    bool m_allowResourceManagerUpdates;
+    bool m_allowResourceProviderUpdates;
+    bool m_needSpellChecking;
     int m_prevCursorPosition; /// used by editingPluginEvents
 
     QTimer m_caretTimer;
@@ -299,18 +247,26 @@ private:
     KAction *m_actionFormatSub;
     KAction *m_actionFormatIncreaseIndent;
     KAction *m_actionFormatDecreaseIndent;
-    KAction *m_actionShowChanges;
-    KAction *m_actionRecordChanges;
-    KAction *m_configureChangeTracking;
     KFontSizeAction *m_actionFormatFontSize;
     KFontAction *m_actionFormatFontFamily;
-    KoColorPopupAction *m_actionFormatTextColor;
-    KoColorPopupAction *m_actionFormatBackgroundColor;
+    KoColorSetAction *m_actionFormatTextColor;
+    KoColorSetAction *m_actionFormatBackgroundColor;
+
+    QHash<QString, KoTextEditingPlugin*> m_textEditingPlugins;
+    KoTextEditingPlugin *m_spellcheckPlugin;
 
     QUndoCommand *m_currentCommand; //this command will be the direct parent of undoCommands generated as the result of QTextDocument changes
 
     bool m_currentCommandHasChildren;
 
+    // update Parag direction will be multi-threaded.
+    struct UpdatePageDirection {
+        KoAction *action;
+        QTextBlock block;
+        KoText::Direction direction;
+    };
+    UpdatePageDirection m_updateParagDirection;
+
     /// structure that allows us to remember the text position and selection of previously edited documents.
     struct TextSelection {
         QTextDocument *document; // be warned that this may end up being a dangling pointer, so don't use.
@@ -320,15 +276,6 @@ private:
     QList<TextSelection> m_previousSelections;
 
     InsertCharacter *m_specialCharacterDocker;
-
-    TextEditingPluginContainer *m_textEditingPlugins;
-
-    bool m_textTyping;
-    bool m_textDeleting;
-
-    QTimer m_changeTipTimer;
-    int m_changeTipCursorPos;
-    QPoint m_changeTipPos;
 };
 
 #endif
